<?php

/**
 * HCE project node management request API.
 * Node object for node management.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013-2014 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
namespace HCE\admin {
	require_once __DIR__ . '/../transport/Constants.inc.php';
	/**
	 * HCE Node class
	 * Implements basic properties of node
	 */
	class Node {
		/**
		 * Node host name
		 */
		protected $host =\HCE\transport\PROTOCOL_HOST_DEFAULT;

		/**
		 * Node port
		 */
		protected $port = \HCE\transport\PROTOCOL_PORT_ADMIN_DEFAULT;

		/**
		 * Timeout
		 */
		protected $timeout = NODE_TIMEOUT;

		/**
		 * Node error code of last operation result
		 */
		protected $error = 0;

		/**
		 * Set host name
		 *
		 * @param $hostName -
		 *        	host name
		 *
		 */
		public function setHost($hostName) {
			$this->host = $hostName;
		}

		/**
		 * Returns host name
		 *
		 * @return host name
		 */
		public function getHost() {
			return $this->host;
		}

		/**
		 * Set port
		 *
		 * @param $port -
		 *        	port
		 *
		 */
		public function setPort($port) {
			$this->port = $port;
		}

		/**
		 * Returns port
		 *
		 * @return port
		 */
		public function getPort() {
			return $this->port;
		}

		/**
		 * Set timeout
		 *
		 * @param $timeout -
		 *        	request timeout
		 *
		 */
		public function setTimeout($timeout) {
			$this->timeout = $timeout;
		}

		/**
		 * Returns request timeout
		 *
		 * @return timeout
		 */
		public function getTimeout() {
			return $this->timeout;
		}

		/**
		 * Set error code
		 *
		 * @param $errorCode -
		 *        	error code
		 *
		 */
		public function setErrorCode($errorCode) {
			$this->error = $errorCode;
		}

		/**
		 * Returns erro code of last executed request
		 *
		 * @return error code
		 */
		public function getErrorCode() {
			return $this->error;
		}

		/**
		 * Create instance of node object
		 *
		 * @param $host -
		 *        	host name
		 *
		 * @param $port -
		 *        	port
		 *
		 * @param $timeout -
		 *        	timeout
		 *
		 */
		public function __construct($host = null, $port = null, $timeout = null) {
			// Init host
			if ($host !== null) {
				$this->setHost ( $host );
			}
			// Init port
			if ($port !== null) {
				$this->setPort ( $port );
			}
			// Init timeout
			if ($timeout !== null) {
				$this->setTimeout ( $timeout );
			}
		}

		/**
		 * delete node object
		 */
		public function __destruct() {
		}
	}
}

?>