<?php

/**
 * HCE project node management request API.
 * Node command object for node management.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013-2014 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
namespace HCE\admin {

	require_once __DIR__ . '/../transport/Constants.inc.php';
	/**
	 * HCE Node command class
	 * Implements basic comands for low level node management
	 */
	class NodeCommand {
		/**
		 * Admin node management handler name
		 */
		protected $handlerName =\HCE\transport\HANDLER_ADMIN;

		/**
		 * Admin node management command name
		 */
		protected $commandName = CMD_ECHO;

		/**
		 * Admin node management command parameters
		 */
		protected $commandParameters = array ();

		/**
		 * Request error code of last operation result
		 */
		protected $error = 0;

		/**
		 * Set handler name
		 *
		 * @param $handlerName -
		 *        	handler name
		 *
		 */
		public function setHandlerName($handlerName) {
			$this->handlerName = $handlerName;
		}

		/**
		 * Returns handler name
		 *
		 * @return handler name
		 */
		public function getHandlerName() {
			return $this->handlerName;
		}

		/**
		 * Set command name
		 *
		 * @param $commandName -
		 *        	command name
		 *
		 */
		public function setCommandName($commandName) {
			$this->commandName = $commandName;
		}

		/**
		 * Returns command name
		 *
		 * @return command name
		 */
		public function getCommandName() {
			return $this->commandName;
		}

		/**
		 * Set command parameters array
		 *
		 * @param $commandParameters -
		 *        	command parameters array
		 *
		 * @return true if set or false if not.
		 */
		public function setCommandParameters($commandParameters) {
			if (is_array ( $commandParameters )) {
				$this->commandParameters = $commandParameters;
				return true;
			} else {
				return false;
			}
		}

		/**
		 * Returns command parameters array
		 *
		 * @return command parameters
		 */
		public function getCommandParameters() {
			return $this->commandParameters;
		}

		/**
		 * Get command parameters string buffer
		 *
		 * @return admin command parameters string buffer
		 */
		public function getParametersData($delimiter = CMD_PARAMETERS_DELIMITER) {
			return implode ( $delimiter, $this->getCommandParameters () );
		}

		/**
		 * Get command message body string buffer
		 *
		 * @return admin command body ready to sent by transport
		 */
		public function getMessageBody($delimiter = CMD_PARAMETERS_DELIMITER) {
			return $this->getHandlerName () . CMD_DELIMITER . $this->getCommandName () . CMD_DELIMITER . $this->getParametersData() . CMD_DELIMITER;
		}

		/**
		 * Set error code
		 *
		 * @param $errorCode -
		 *        	error code
		 *
		 */
		public function setErrorCode($errorCode) {
			$this->error = $errorCode;
		}

		/**
		 * Returns erro code of last executed request
		 *
		 * @return error code
		 */
		public function getErrorCode() {
			return $this->error;
		}

		/**
		 * Create instance of node command object
		 *
		 * @param $commandName -
		 *        	command name
		 *
		 * @param $commandParameters -
		 *        	command parameters
		 *
		 */
		public function __construct($commandName = null, $commandParameters = null, $handlerName = null) {
			// Set command name
			if ($commandName !== null) {
				$this->setCommandName ( $commandName );
			}
			// Set command parameters
			if ($commandParameters !== null) {
				$this->setCommandParameters ( $commandParameters );
			}
			if ($handlerName !== null) {
				$this->setHandlerName ( $handlerName );
			}
		}

		/**
		 * delete node command object
		 */
		public function __destruct() {
			$this->commandParameters = null;
		}
	}
}

?>