<?php

/**
 * HCE project node management request API.
 * Basic request object for node management.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013-2014 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
namespace HCE\admin {

	require_once __DIR__ . '/../transport/Constants.inc.php';
	require_once __DIR__ . '/../transport/Connection.inc.php';
	require_once __DIR__ . '/../transport/Request.inc.php';

	require_once 'Constants.inc.php';
	require_once 'Node.inc.php';
	require_once 'NodeCommand.inc.php';
	require_once 'NodeManageResponse.inc.php';

	/**
	 * HCE Node management request class
	 */
	class NodeManageRequest {
		/**
		 * Transport request object
		 */
		protected $transportRequest = NULL;

		/**
		 * Transport connection
		 */
		protected $transportConnection = NULL;

		/**
		 * Request error code of last operation result
		 */
		protected $error = 0;

		/**
		 * Set error code
		 *
		 * @param $errorCode -
		 *        	error code
		 *
		 */
		public function setErrorCode($errorCode) {
			$this->error = $errorCode;
		}

		/**
		 * Returns erro code of last executed request
		 *
		 * @return error code
		 */
		public function getErrorCode() {
			return $this->error;
		}

		/**
		 * create node manager request instance
		 *
		 * @param $transportConnection -
		 *        	transport connection
		 *
		 * @param $transportRequest -
		 *        	transport request
		 */
		public function __construct($transportConnection = null, $transportRequest = null) {
			// Init transport connection
			$this->transportConnection = $transportConnection;

			// Init transport request
			$this->transportRequest = $transportRequest;
		}

		/**
		 * delete node manage request object
		 */
		public function __destruct() {
			$this->transportRequest = NULL;
			$this->transportConnection = NULL;
		}

		/**
		 * Execute request command and returns $NodeManageResponse object instance
		 *
		 * @param $command -
		 *        	command object
		 *
		 * @param $node -
		 *        	node object
		 *
		 * @return Response object
		 */
		public function execute($command, $node) {
			// Make $NodeManageResponse object instance to return
			$nodeManageResponseObject = new \HCE\admin\NodeManageResponse ();

			// Create connection with default initialization
			$connection = new \HCE\transport\Connection ();
			if (! $connection->error) {
				// Set connection properties
				$connection->host = $node->getHost ();
				$connection->port = $node->getPort ();
				// Create request with default initialization
				$request = new \HCE\transport\Request ( $connection );
				if (! $request->getErrorCode ()) {
					// Try to open connection
					if ($connection->open ()) {
						// Execute request
						$response = $request->execute ( $command->getMessageBody (), $node->getTimeout () );
						// Check transport request state
						if ($response->getErrorCode ()) {
							// Error of transport request execution
							$nodeManageResponseObject->setErrorCode ( ERROR_TRANSPORT_REQUEST );
						}
						// Set transport response
						$nodeManageResponseObject->setResponse ( $response );
						// Close connection
						$connection->close ();
					} else {
						// Error open connection
						$nodeManageResponseObject->setErrorCode ( ERROR_CONNECTION_OPEN );
					}
				} else {
					// Error create request object
					$nodeManageResponseObject->setErrorCode ( ERROR_REQUEST_CREATE );
				}
			} else {
				// Error create connection object
				$nodeManageResponseObject->setErrorCode ( ERROR_CONNECTION_CREATE );
			}

			return $nodeManageResponseObject;
		}
	}
}

?>