<?php

/**
 * HCE project node manage response API.
 * Basic response object for node management.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013-2014 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
namespace HCE\admin {

	require_once 'Constants.inc.php';

	/**
	 * HCE Request class implements basic functionality of client-side of HCE Cluster Node application interaction
	 */
	class NodeManageResponse {
		/**
		 * Response data
		 */
		protected $transportResponse = null;

		/**
		 * Response error code of last operation result
		 */
		protected $error = 0;

		/**
		 * Returns response
		 *
		 * @return transport response object
		 */
		public function getResponse() {
			return $this->transportResponse;
		}

		/**
		 * Set response
		 *
		 * @param $transportResponse -
		 *        	transport response object
		 *
		 */
		public function setResponse($transportResponse) {
			$this->transportResponse = $transportResponse;
		}

		/**
		 * Set error code
		 *
		 * @param $errorCode -
		 *        	error code
		 *
		 */
		public function setErrorCode($errorCode) {
			$this->error = $errorCode;
		}

		/**
		 * Returns erro code of last executed request
		 *
		 * @return error code
		 */
		public function getErrorCode() {
			return $this->error;
		}

		/**
		 * Returns transport response message raw body field
		 *
		 * @return response body
		 */
		public function getResponseMessageBody($messageItem = 0) {
			return $this->transportResponse->getResponseItemField ( $messageItem, 'body' );
		}

		/**
		 * Returns transport response message id field
		 *
		 * @return response id
		 */
		public function getResponseMessageId($messageItem = 0) {
			return $this->transportResponse->getResponseItemField ( $messageItem, 'id' );
		}

		/**
		 * Returns transport response message error field
		 *
		 * @return response error code
		 */
		public function getResponseMessageErrorCode($messageItem = 0) {
			return $this->transportResponse->getResponseItemField ( $messageItem, 'error' );
		}

		/**
		 * Returns array of fields parsed from transport response message raw body
		 *
		 * @return fields array(FIELD_NAME=>FIELD_VALUE), common FIELD_NAME is 'status', another fields names depends on command; all command specific fields can be NULL in case of
		 *         wrong response or ERROR response status
		 */
		public function getResponseCommandFields($messageItem = 0) {
			$ret = array (
					NODE_MANAGE_RESPONSE_FIELD_STATUS => ERROR_OK
			);

			if ($this->transportResponse->getResponseItemField ( $messageItem, 'error' )) {
				$ret [NODE_MANAGE_RESPONSE_FIELD_STATUS] = ERROR_ERROR;
			} else {
				// Get raw body
				$body = $this->transportResponse->getResponseItemField ( $messageItem, 'body' );
				// Parse body
				$parts = explode ( CMD_RESPONSE_DELIMITER, $body );
				if ($parts [0] !== ERROR_OK) {
					$ret [NODE_MANAGE_RESPONSE_FIELD_STATUS] = ERROR_ERROR;
				} else {
					$ret [NODE_MANAGE_RESPONSE_FIELD_STATUS] = ERROR_OK;
					for($i = 1; $i < count ( $parts ); $i ++) {
						$part = explode ( CMD_RESPONSE_ITEMS_DELIMITER, $parts [$i] );
						$ret [$part [0]] = $part [1];
					}
				}
			}

			return $ret;
		}

		/**
		 * create instance
		 */
		public function __construct($transportResponse = null) {
			// Init transport respose
			if ($transportResponse === null) {
				$this->transportResponse = new \HCE\transport\Response ();
			}
		}

		/**
		 * Delete node manage response objext
		 */
		public function __destruct() {
			$this->transportResponse = NULL;
		}
	}
}

?>