<?php

/**
 * HCE project node manager API.
 * Manager object for node management.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013-2014 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
namespace HCE\admin {

	require_once 'Constants.inc.php';
	require_once 'Node.inc.php';
	require_once 'NodeCommand.inc.php';
	require_once 'NodeManageRequest.inc.php';
	require_once 'NodeManageResponse.inc.php';

	/**
	 * HCE Node management request class
	 */
	class NodeManager {
		/**
		 * Nodes array
		 */
		protected $nodes = array ();

		/**
		 * Transport responses objects items array
		 */
		protected $responses = array ();

		/**
		 * Request error code of last operation result
		 */
		protected $error = 0;

		/**
		 * Set nodes array
		 *
		 * @param $nodes -
		 *        	nodes array
		 *
		 */
		public function setNodes($nodes) {
			$this->nodes = $nodes;
		}

		/**
		 * Returns nodes array
		 *
		 * @return nodes
		 */
		public function getNodes() {
			return $this->nodes;
		}

		/**
		 * Set error code
		 *
		 * @param $errorCode -
		 *        	error code
		 *
		 */
		public function setErrorCode($errorCode) {
			$this->error = $errorCode;
		}

		/**
		 * Returns erro code of last executed request
		 *
		 * @return error code
		 */
		public function getErrorCode() {
			return $this->error;
		}
		/**
		 * Set response data item
		 *
		 * @param $responseItem -
		 *        	is array with fields zmq message, id, body and error code - returned from request
		 */
		public function addResponseItem($responseItem) {
			if ($responseItem !== NULL) {
				$this->responses [] = $responseItem;
			}
		}

		/**
		 * UnSet response data item
		 *
		 * @param $index -
		 *        	item index to delete
		 */
		public function deleteResponseItem($index) {
			if (isset ( $this->responses [$index] )) {
				unset ( $this->responses [$index] );
			}
		}

		/**
		 * Set responses items array
		 *
		 * @param $responses -
		 *        	array of response data items
		 */
		public function setResponses($responses) {
			$this->responses = $responses;
		}

		/**
		 * Get response item
		 *
		 * @param $index -
		 *        	index of item
		 *
		 */
		public function getResponseItem($index = NULL) {
			if ($index === NULL) {
				return count ( $this->responses );
			} else {
				return $this->responses [$index];
			}
		}

		/**
		 * Get responses array
		 */
		public function getResponses() {
			return $this->responses;
		}

		/**
		 * create instance
		 *
		 * @param $nodes -
		 *        	nodes array
		 *
		 */
		public function __construct($nodes = null) {
			// Set nodes
			if ($nodes !== null) {
				$this->setNodes ( $nodes );
			}
		}

		/**
		 * delete node manage request object
		 */
		public function __destruct() {
			$this->nodes = null;
			$this->responses = null;
		}

		/**
		 * Execute command request for all nodes and fill $responses array
		 *
		 * @param $commandObject -
		 *        	node command object
		 *
		 * @return bool true if no errors and false if at least one request for one node failed
		 */
		public function execute($commandObject) {
			$ret = true;

			// Make $NodeManageResponse object instance to return
			$request = new NodeManageRequest ();

			// For each node
			foreach ( $this->nodes as $node ) {
				// Execute command for node
				$response = $request->execute ( $commandObject, $node );
				// accumulate response item
				$this->addResponseItem ( $response );
				if ($response->getErrorCode ()) {
					$ret = false;
				}
			}

			return $ret;
		}

		/**
		 * Get dump of all responses data in format of php var_dump printout
		 *
		 * @return dump string
		 */
		public function getResponsesDump() {
			$ret = '';

			// Process all responses
			foreach ( $this->responses as $response ) {
				if ($response->getErrorCode ()) {
					$ret .= MSG_NODE_MANAGE_RESPONSE_ITEM_ERROR . $response->getErrorCode () . PHP_EOL;
				} else {
					$transportResponse = $response->getResponse ();
					if ($transportResponse->getErrorCode ()) {
						$ret .= MSG_TRANSPORT_RESPONSE_ITEM_ERROR . $transportResponse->getErrorCode () . PHP_EOL;
					} else {
						// Get transport response items from transport response object
						$transportResponsesItems = $transportResponse->getResponses ();
						// Dump all items data
						foreach ( $transportResponsesItems as $transportResponseItem ) {
							$ret .= var_export ( $transportResponseItem, true );
						}
					}
				}
			}

			return $ret;
		}

		/**
		 * Get data array of transport responses fields
		 *
		 * @return array of transport responses data fields as array('error', 'id', 'body')
		 */
		public function getResponsesData() {
			$ret = array ();

			// Process all responses
			foreach ( $this->responses as $response ) {
				$ret [] = array (
						$response->getResponseMessageErrorCode (),
						$response->getResponseMessageId (),
						$response->getResponseMessageBody ()
				);
			}

			return $ret;
		}

		/**
		 * Get deserialized commadn's response fields array
		 *
		 * @return array of deserialized command response arrays of fields as array(FIELD_NAME=>FIELD_VALUE), common FIELD_NAME is 'status', another fields names depends on command,
		 *         or not present in case of wrong response or ERROR response status received
		 */
		public function getResponsesFields() {
			$ret = array ();

			// Process all responses
			foreach ( $this->responses as $response ) {
				$ret [] = $response->getResponseCommandFields ();
			}

			return $ret;
		}
	}
}

?>