<?php

/**
 * HCE project node connection API.
 * Basic connnection API for network transport.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013-2014 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
namespace HCE\transport {

	require_once 'zmsg.php';
	require_once 'Constants.inc.php';
	require_once 'IdGenerator.inc.php';

	/**
	 * HCE Connection class, implements basic functionality of client-side API for HCE node application interaction
	 */
	class Connection {
		/**
		 * Connection array fields container
		 */
		protected $connectionArray = array ();

		/**
		 * Set connection array
		 *
		 * @param $connectionArray -
		 *        	connection array
		 */
		public function setConnectionArray($connectionArray = NULL) {
			$this->connectionArray = $connectionArray;
		}

		/**
		 * returns connection array fields by name as class members fields values
		 *
		 * @return field value
		 */
		public function getConnectionArray() {
			return $this->connectionArray;
		}

		/**
		 * returns connection array fields by name as class members fields values
		 *
		 * @param $name -
		 *        	field name
		 * @return field value
		 */
		public function __get($name) {
			return $this->connectionArray [$name];
		}

		/**
		 * create instance and set the HCE connection array property (new zmq context, zmq socket and connect using specified or default options)
		 *
		 * @param $connectionArray -
		 *        	array of connection configuration options: host, port, type {HCE\transport\CONNECTION_TYPE_ADMIN, HCE\transport\CONNECTION_TYPE_ROUTER}, identity - string of unique client Id
		 *        	in cluster
		 *
		 */
		public function __construct($connectionArray = NULL) {
			// Init connection array
			$ret = array (
					'error' => PROTOCOL_ERROR_OK,
					'context' => NULL,
					'socket' => NULL,
					'type' => NULL,
					'host' => NULL,
					'port' => NULL,
					'identity' => NULL
			);

			// Check connection parameters
			if ($connectionArray === NULL) {
				$con_ar = array (
						'host' => PROTOCOL_HOST_DEFAULT,
						'port' => PROTOCOL_PORT_ADMIN_DEFAULT,
						'type' => PROTOCOL_CONNECTION_TYPE_ADMIN,
						'identity' => IdGenerator::getClientId ()
				);
			} else {
				if (! isset ( $connectionArray ['host'] ) || ! isset ( $connectionArray ['port'] ) || ! isset ( $connectionArray ['type'] ) || ! isset ( $connectionArray ['identity'] )) {
					$ret ['error'] = PROTOCOL_ERROR_CONNECTION_PARAMS;
				} else {
					$con_ar = $connectionArray;
				}
			}

			// Create socket
			if ($ret ['error'] == PROTOCOL_ERROR_OK) {
				// Create zmq context
				$ret ['context'] = new \ZMQContext ();
				if ($ret ['context'] == NULL) {
					$ret ['error'] = PROTOCOL_ERROR_CONTEXT_CREATE;
				} else {
					// Create zmq socket
					$ret ['socket'] = new \ZMQSocket ( $ret ['context'], \ZMQ::SOCKET_DEALER );
					if ($ret ['socket'] === NULL) {
						$ret ['error'] = PROTOCOL_ERROR_SOCKET_CREATE;
					} else {
						// Set request identity
						$ret ['socket']->setSockOpt ( \ZMQ::SOCKOPT_IDENTITY, $con_ar ['identity'] );
						if ($con_ar ['type'] == PROTOCOL_CONNECTION_TYPE_ADMIN) {
							// Set linger 0, do not wait on close
							$ret ['socket']->setSockOpt ( \ZMQ::SOCKOPT_LINGER, 0 );
						}

						// Set connection array properties
						$ret ['host'] = $con_ar ['host'];
						$ret ['port'] = $con_ar ['port'];
						$ret ['type'] = $con_ar ['type'];
						$ret ['identity'] = $con_ar ['identity'];
					}
				}
			}

			// Set connection array
			$this->setConnectionArray ( $ret );
		}

		/**
		 * Open connection, connect ZMQ socket
		 *
		 * @return ZMQ socket connect
		 */
		public function open() {
			//TODO try and catch zmq exeptions
			return $this->connectionArray ['socket']->connect ( $this->getDSN () );
		}

		/**
		 * Close connection, disconnect ZMQ socket
		 *
		 * @return ZMQ socket disconnect
		 */
		public function close() {
			return $this->connectionArray ['socket']->disconnect ( $this->getDSN () );
		}

		/**
		 * Reset connection, disconnect ZMQ socket
		 *
		 * @return ZMQ socket connect
		 */
		public function reset() {
			$this->connectionArray ['socket']->disconnect ( $this->getDSN () );
			return $this->connectionArray ['socket']->connect ();
		}

		/**
		 * Get ZMQ socket't dsn string
		 *
		 * @return ZMQ socket dsn string
		 */
		public function getDSN($protocol = PROTOCOL_ADMIN_DEFAULT) {
			return $protocol . '://' . $this->connectionArray ['host'] . ':' . $this->connectionArray ['port'];
		}

		/**
		 * Delete HCE Connection object (disconnect zmq socket, free zmq socket and context)
		 *
		 */
		public function __destruct() {
			$this->connectionArray ['context'] = NULL;
			$this->connectionArray ['socket'] = NULL;
			$this->connectionArray ['host'] = NULL;
			$this->connectionArray ['port'] = NULL;
			$this->connectionArray ['type'] = NULL;
			$this->connectionArray ['identity'] = NULL;
		}
	}
}

?>