<?php

/**
 * HCE project node transport API.
 * Unique Id generator object.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013-2014 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
namespace HCE\transport {

	/**
	 * HCE Connection class implements basic functionality of client-side of HCE Cluster Node application interaction
	 */
	class IdGenerator {
		/**
		 * create instance of IdGenerator
		 */
		public function __construct() {
		}

		/**
		 * delete the IdGenerator object instance
		 */
		public function __destruct() {
		}

		/**
		 * generate unique client identifier
		 *
		 * @param $type -
		 *        	type of algorithm of Id generation algorithm, supported values from 0 to 5
		 * @param $prefix -
		 *        	$prefix for Id string
		 *
		 * @return unique client identifier string
		 */
		public static function getClientId($type = 1, $prefix = null) {
			if ($prefix === null) {
				$prefix = date ( 'Y-m-d H:i:s' ) . '-' . microtime ( true ) . '-';
			}

			$ret = PROTOCOL_CLIENT_IDENTITY_PREFIX . $prefix;

			if ($type == 0) {
				// Simple fast
				$ret .= self::getUniqueId ( 1 );
			} else {
				// More accurate complexity
				$ret .= self::getUniqueId ( 3 ) . '-' . self::getUniqueId ( 5 );
			}

			return $ret;
		}

		/**
		 * Generate unique message identifier
		 *
		 * @param $type -
		 *        	type of algorithm of Id generation algorithm, supported values from 0 to 5
		 * @param $prefix -
		 *        	$prefix for Id string
		 *
		 * @return unique message identifier string
		 */
		public static function getMessageId($type = 1, $prefix = '') {
			$ret = PROTOCOL_MSG_ID_PREFIX . $prefix;

			if ($type == 0) {
				// Simple fast
				$ret .= self::getUniqueId ( 0 );
			} else {
				// More accurate complexity
				$ret .= self::getUniqueId ( 4 );
			}

			return $ret;
		}

		/**
		 * Generate unique identifier by one of several methods
		 *
		 * @param $type -
		 *        	type of algorithm of Id generation algorithm, supported values from 0 to 5
		 *
		 * @return unique identifier string
		 */
		public static function getUniqueId($type = 0) {
			$ret = null;

			if ($type == 0) {
				// Complex string
				$ret = uniqid ( md5 ( rand () ), true );
			} elseif ($type == 1) {
				// Complex string hexadec
				$ret = dechex ( time () ) . dechex ( mt_rand ( 1, 65535 ) );
			} elseif ($type == 2) {
				// Random for better crypto cypher hexadec
				$r = unpack ( 'v*', fread ( fopen ( '/dev/random', 'r' ), 16 ) );
				$ret = sprintf ( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x', $r [1], $r [2], $r [3], $r [4] & 0x0fff | 0x4000, $r [5] & 0x3fff | 0x8000, $r [6], $r [7], $r [8] );
			} elseif ($type == 3) {
				// Host name and microseconds
				$ret = uniqid ( php_uname ( 'n' ) . '-', true );
			} elseif ($type == 4) {
				// Another fast useful for file name
				$ret = time () . substr ( md5 ( microtime ( true ) ), 0, rand ( 5, 12 ) );
			} elseif ($type == 5) {
				// The field names refer to RFC 4122 section 4.1.2
				$ret = sprintf ( '%04x%04x-%04x-%03x4-%04x-%04x%04x%04x', mt_rand ( 0, 65535 ), mt_rand ( 0, 65535 ), 				// 32 bits for "time_low"
				mt_rand ( 0, 65535 ), 				// 16 bits for "time_mid"
				mt_rand ( 0, 4095 ), 				// 12 bits before the 0100 of (version) 4 for "time_hi_and_version"
				bindec ( substr_replace ( sprintf ( '%016b', mt_rand ( 0, 65535 ) ), '01', 6, 2 ) ),
						// 8 bits, the last two of which (positions 6 and 7) are 01, for "clk_seq_hi_res"
						// (hence, the 2nd hex digit after the 3rd hyphen can only be 1, 5, 9 or d)
						// 8 bits for "clk_seq_low"
						mt_rand ( 0, 65535 ), mt_rand ( 0, 65535 ), mt_rand ( 0, 65535 ) ); // 48 bits for "node"
			}

			return $ret;
		}
	}
}

?>