<?php

/**
 * HCE project node response API.
 * Basic response API for network transport.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013-2014 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
namespace HCE\transport {

	/**
	 * HCE Response class implements basic functionality of client-side of HCE Cluster Node application interaction
	 */
	class Response {
		/**
		 * HCE Response class implements basic functionality of client-side of HCE Cluster Node application interaction
		 */
		protected $responseData = array ();

		/**
		 * Response error code of last operation result
		 */
		protected $error = 0;

		/**
		 * Set response data item
		 *
		 * @param $responseItem -
		 *        	is array with fields zmq message, id, body and error code - returned from request
		 */
		public function addResponseItem($responseItem) {
			if ($responseItem !== NULL) {
				$this->responseData [] = $responseItem;
			}
		}

		/**
		 * UnSet response data item
		 *
		 * @param $index -
		 *        	item index to delete
		 */
		public function deleteResponseItem($index) {
			if (isset ( $this->{$responseData} [$index] )) {
				unset ( $this->responseData [$index] );
			}
		}

		/**
		 * Set response data items array
		 *
		 * @param $responses -
		 *        	array of response data items
		 */
		public function setResponses($responses) {
			$this->responseData = $responses;
		}

		/**
		 * Get response item
		 *
		 * @param $index -
		 *        	item index number from zero
		 *
		 */
		public function getResponseItem($index = NULL) {
			if ($index === NULL) {
				return count ( $this->$responseData );
			} else {
				return $this->responseData [$index];
			}
		}

		/**
		 * Get response items array
		 */
		public function getResponses() {
			return $this->responseData;
		}

		/**
		 * Set error code
		 *
		 * @param $errorCode -
		 *        	error code
		 *
		 */
		public function setErrorCode($errorCode) {
			$this->error = $errorCode;
		}

		/**
		 * Returns erro code of last executed request
		 *
		 * @return error code
		 */
		public function getErrorCode() {
			return $this->error;
		}

		/**
		 * Get response item field
		 *
		 * @param $index -
		 *        	item number
		 * @param $fieldName -
		 *        	item field name, supported "error", "ZMQMsg", "id", "body"
		 *
		 */
		public function getResponseItemField($index = NULL, $fieldName = NULL) {
			if ($index === NULL) {
				return count ( $this->responseData );
			} else {
				if (isset ( $this->responseData [$index] [$fieldName] )) {
					return $this->responseData [$index] [$fieldName];
				} else {
					return NULL;
				}
			}
		}

		/**
		 * create instance and set response message data item
		 *
		 * @param $responseItem -
		 *        	response data item array
		 */
		public function __construct($responseItem = null) {
			$this->addResponseItem ( $responseItem );
		}

		/**
		 * delete HCE response instance
		 */
		public function __destruct() {
			$this->responseData = NULL;
		}
	}
}

?>