#!/usr/bin/python


"""
HCE project, Python bindings, Distributed Tasks Manager application.
Event objects definitions.

@package: dc
@file URLFetch_json_to_db-task_convertor.py
@author Oleksii <developers.hce@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""


import ppath
from ppath import sys

import json
from subprocess import Popen
from subprocess import PIPE
import logging
try:
  import cPickle as pickle
except ImportError:
  import pickle

import dc.Constants as DC_CONSTS
from dcc.DCCObjectsSerializator import DCCObjectsSerializator
import app.Utils as Utils  # pylint: disable=F0401

MSG_ERROR_EXIT_STATUS = "Exit failure. "

MSG_DEBUG_INPUT_PICKLE = "Input pickle: "
MSG_DEBUG_INPUT_UNPICKLE = "Input unpickle: "
MSG_DEBUG_APPENDED_URL = "Append url: "
MSG_DEBUG_APPENDED_UNIQ_URL = "Append uniq url: "
MSG_DEBUG_OUTPUT_BATCH_ITEM = "Output batch item: "
MSG_DEBUG_OUTPUT_PICKLE = "Output pickle: "
MSG_DEBUG_SEND_PICKLE = "Send pickle. Done."

MSG_INFO_PROCESSOR_EXIT_CODE = "Scraper exit_code: "
MSG_INFO_PROCESSOR_OUTPUT = "Scraper output: "
MSG_INFO_PROCESSOR_ERROR = "Scraper err: "


STATUS_SUCCESS = 0
STATUS_FAILURE = 1


logging.basicConfig(filename="../log/URLFetch_json_to_db-task_convertor.log", filemode="w")
logger = logging.getLogger("")
logger.setLevel("DEBUG")


def getURLFetchjson():
  input_url_fetch_json = None
  input_url_fetch_json = sys.stdin.read()
  logger.debug(MSG_DEBUG_INPUT_PICKLE + '\n' + str(input_url_fetch_json))
  return json.loads(input_url_fetch_json)


def createOutputPickle(input_url_fetch_json):
  eventType = DC_CONSTS.EVENT_TYPES.URL_FETCH
  convertor = DCCObjectsSerializator()
  eventObj = convertor.URLFetchDeserialize(input_url_fetch_json)
  logger.debug(MSG_DEBUG_OUTPUT_PICKLE + '\n' + str(eventObj))
  drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(eventType, eventObj)
  output_pickle = pickle.dumps(drceSyncTasksCoverObj)
  logger.debug(MSG_DEBUG_OUTPUT_PICKLE + '\n' + str(output_pickle))
  return output_pickle


def sendToDbTask(output_pickle):
  cmd = "cd ~/hce-node-bundle/api/python/bin && ./db-task.py --c=../ini/db-task.ini"
  process = Popen(cmd, stdout=PIPE, stdin=PIPE, shell=True, close_fds=True)
  (output, err) = process.communicate(input=output_pickle)
  exit_code = process.wait()
  logger.info(MSG_INFO_PROCESSOR_EXIT_CODE + str(exit_code))
  logger.info(MSG_INFO_PROCESSOR_OUTPUT + str(output))
  logger.info(MSG_INFO_PROCESSOR_ERROR + str(err))
  logger.info(MSG_INFO_PROCESSOR_ERROR + str(pickle.loads(output)))
  return output


def sendPickle(output_pickle):
  sys.stdout.write(output_pickle)
  logger.debug(MSG_DEBUG_SEND_PICKLE)


if __name__ == "__main__":
  try:
    exit_status = STATUS_SUCCESS

    # read URLFetch json
    input_url_fetch_json = getURLFetchjson()

    # pickle it
    output_pickle = createOutputPickle(input_url_fetch_json)

    # send it to db-task
    output = sendToDbTask(output_pickle)

    # receive pickled response and send it to convertor
    sendPickle(output)
  except Exception, err:
    tbi = Utils.getTracebackInfo()
    logger.error(MSG_ERROR_EXIT_STATUS + err.message + '\n' + tbi)
    exit_status = STATUS_FAILURE
  exit(exit_status)
