#!/usr/bin/python


"""
HCE project, Python bindings, Distributed Tasks Manager application.
Event objects definitions.

@package: dc
@file db-task.py
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

#import sys
import ppath  # pylint: disable=W0611
from ppath import sys

import dc.EventObjects as dc_event  # pylint: disable=F0401, W0611
import ConfigParser
import logging
import logging.config
import MySQLdb
# import pickle
try:
  import cPickle as pickle
except ImportError:
  import pickle
import os
import traceback  # pylint: disable=W0611
import dc_db.Constants as Constants
from optparse import OptionParser
from dc_db.TasksManager import TasksManager  # pylint: disable=F0401
from dc_db.Constants import LOGGER_NAME  # pylint: disable=F0401
from dc.Constants import DRCESyncTasksCover  # pylint: disable=F0401, W0611
import app.Utils as Utils  # pylint: disable=F0401

logger = logging.getLogger(LOGGER_NAME)

parser = OptionParser()
parser.add_option("-c", "--cfg", type="string", help="config file", dest="cfg")


def exceptionMySQLLogging(abortStr, errCode):
  tbi = Utils.getTracebackInfo()
  logger.error(str(vars(err)) + "\n" + tbi)
  sys.stderr.write(str(vars(err)) + "\n" + tbi)
  logger.error(abortStr)
  sys.stderr.write(abortStr)
  os._exit(errCode)


if __name__ == "__main__":
  options, arguments = parser.parse_args()
  if options.__dict__["cfg"]:
    try:
      cfgParser = ConfigParser.ConfigParser()
      cfgParser.read(options.__dict__["cfg"])

      # init loggers
      logging.config.fileConfig(cfgParser.get("TasksManager", "log_cfg"))
      logger = logging.getLogger(LOGGER_NAME)
      # Replace logger name for processes instances
      lfn = Utils.LoggerFileName()
      fname = lfn.findReplace()
      if fname != "":
        pin = lfn.getFreeProcInstanceNumber(os.path.basename(fname))
        if pin != "" and pin != "0":
          pin = "." + pin + ".log"
          fname = lfn.findReplace(fname + pin)
          logger = logging.getLogger(LOGGER_NAME)


      dbTask = TasksManager(cfgParser)

      drceSyncTasksCoverPickled = sys.stdin.read()
      if drceSyncTasksCoverPickled[-1] != '\x2E':
        drceSyncTasksCoverPickled += '\x2E'

      logger.debug("Receved object:\n" + str(drceSyncTasksCoverPickled) + "\n")
      drceSyncTasksCover = pickle.loads(drceSyncTasksCoverPickled)
      responseDRCESyncTasksCover = dbTask.process(drceSyncTasksCover)
      sys.stdout.write(pickle.dumps(responseDRCESyncTasksCover))
      sys.stdout.flush()

      if dbTask.SQLErrorCode != Constants.EXIT_CODE_OK:
        logger.error("Exited with dbTask.SQLErrorCode=" + str(dbTask.SQLErrorCode))
        sys.stderr.write("Exited with dbTask.SQLErrorCode=" + str(dbTask.SQLErrorCode) + "\n" + 
                         "dbTask.SQLErrorMessage=\"" + dbTask.SQLErrorString + "\"\n")
        os._exit(dbTask.SQLErrorCode)
      else:
        logger.info("Finished normally!")
    except MySQLdb.OperationalError as err:
      exceptionMySQLLogging("Aborted cause exception! MySQLdb.OperationalError {%s}\n" % str(err.message),
                            Constants.EXIT_CODE_GLOBAL_ERROR)
    except Exception as err:
      exceptionMySQLLogging("Aborted cause exception! {%s}\n" % str(err.message), Constants.EXIT_CODE_GLOBAL_ERROR)
  else:
    logger.error("Invalid command line arguments. To help - python ./db_task.py -h")
    sys.stderr.write("Invalid command line arguments. To help - python ./db_task.py -h")
    os._exit(Constants.EXIT_CODE_CONFIG_ERROR)
