#!/usr/bin/python


"""
  HCE project, Python bindings, Distributed Tasks Manager application.
  Event objects definitions.
  
  @package: dc
  @file urls-to-batch-task.py
  @author Oleksii <developers.hce@gmail.com>
  @link: http://hierarchical-cluster-engine.com/
  @copyright: Copyright &copy; 2013-2014 IOIX Ukraine
  @license: http://hierarchical-cluster-engine.com/license/
  @since: 0.1
  """

import ppath
from ppath import sys

import json
import logging


import dc_crawler.CrawlerTask
import dc_crawler.Constants as CONSTS
import dc_processor.ProcessorTask
import app.Utils as Utils


logging.basicConfig(filename="../log/dc-urls-mask.log", filemode="w")
logger = logging.getLogger("")
logger.setLevel("DEBUG")


MSG_ERROR_EXIT_STATUS = "Exit failure. "

MSG_DEBUG_INPUT_ERROR_MASK = "Input error mask: "
MSG_DEBUG_CONVERTED_MASKS = "Converted masks: "


STATUS_SUCCESS = 0
STATUS_FAILURE = 1

defined_masks = {
# CrawlerTask section
"CRAWLER-TASK-ERROR-BAD-URL" : CONSTS.ERROR_BAD_URL,
"CRAWLER-TASK-ERROR-REQUEST-TIMEOUT" : CONSTS.ERROR_REQUEST_TIMEOUT,
"CRAWLER-TASK-ERROR-HTTP-ERROR" : CONSTS.ERROR_HTTP_ERROR,
"CRAWLER-TASK-ERROR-EMPTY-RESPONSE" : CONSTS.ERROR_EMPTY_RESPONSE,
"CRAWLER-TASK-ERROR-WRONG-MIME" : CONSTS.ERROR_WRONG_MIME,
"CRAWLER-TASK-ERROR-CONNECTION-ERROR" : CONSTS.ERROR_CONNECTION_ERROR,
"CRAWLER-TASK-ERROR-PAGE-CONVERT-ERROR" : CONSTS.ERROR_PAGE_CONVERT_ERROR,
"CRAWLER-TASK-ERROR-BAD-REDIRECTION" : CONSTS.ERROR_BAD_REDIRECTION,
"CRAWLER-TASK-ERROR-RESPONSE-SIZE-ERROR" : CONSTS.ERROR_RESPONSE_SIZE_ERROR,
"CRAWLER-TASK-ERROR-AUTH-ERROR" : CONSTS.ERROR_AUTH_ERROR,
"CRAWLER-TASK-ERROR-WRITE-FILE-ERROR" : CONSTS.ERROR_WRITE_FILE_ERROR,
"CRAWLER-TASK-ERROR-ROBOTS-NOT-ALLOW" : CONSTS.ERROR_ROBOTS_NOT_ALLOW,
"CRAWLER-TASK-ERROR-HTML-PARSE-ERROR" : CONSTS.ERROR_HTML_PARSE_ERROR,
"CRAWLER-TASK-ERROR-BAD-ENCODING" : CONSTS.ERROR_BAD_ENCODING,
"CRAWLER-TASK-ERROR-SITE-MAX-ERRORS" : CONSTS.ERROR_SITE_MAX_ERRORS,
"CRAWLER-TASK-ERROR-SITE-MAX-RESOURCES" : CONSTS.ERROR_SITE_MAX_RESOURCES,
"CRAWLER-TASK-ERROR-MIME-NOT-WRITE" : CONSTS.ERROR_MIME_NOT_WRITE,
"CRAWLER-TASK-ERROR-MAX-ALLOW-HTTP-REDIRECTS" : CONSTS.ERROR_MAX_ALLOW_HTTP_REDIRECTS,
"CRAWLER-TASK-ERROR-MAX-ALLOW-HTML-REDIRECTS" : CONSTS.ERROR_MAX_ALLOW_HTML_REDIRECTS,
"CRAWLER-TASK-ERROR-HTML-STRUCTURE" : CONSTS.ERROR_HTML_STRUCTURE,
"CRAWLER-TASK-ERROR-DTD-INVALID" : CONSTS.ERROR_DTD_INVALID,
"CRAWLER-TASK-ERROR-CONTENT-TYPE" : CONSTS.ERROR_CONTENT_TYPE,
"CRAWLER-TASK-ERROR-FETCH-AMBIGUOUS-REQUEST" : CONSTS.ERROR_FETCH_AMBIGUOUS_REQUEST,
"CRAWLER-TASK-ERROR-FETCH-CONNECTION-ERROR" : CONSTS.ERROR_FETCH_CONNECTION_ERROR,
"CRAWLER-TASK-ERROR-FETCH-HTTP-ERROR" : CONSTS.ERROR_FETCH_HTTP_ERROR,
"CRAWLER-TASK-ERROR-FETCH-INVALID-URL" : CONSTS.ERROR_FETCH_INVALID_URL,
"CRAWLER-TASK-ERROR-FETCH-TOO-MANY-REDIRECTS" : CONSTS.ERROR_FETCH_TOO_MANY_REDIRECTS,
"CRAWLER-TASK-ERROR-FETCH-CONNECTION-TIMEOUT" : CONSTS.ERROR_FETCH_CONNECTION_TIMEOUT,
"CRAWLER-TASK-ERROR-FETCH-READ-TIMEOUT" : CONSTS.ERROR_FETCH_READ_TIMEOUT,
"CRAWLER-TASK-ERROR-FETCH-TIMEOUT" : CONSTS.ERROR_FETCH_TIMEOUT,
"CRAWLER-TASK-ERROR-FETCH-UNDEFINED" : CONSTS.ERROR_FETCH_UNDEFINED,

# ProcessorTask section
"PROCESSOR-TASK-ERROR-SITE-MAX-ERRORS" : dc_processor.ProcessorTask.ERROR_MASK_SITE_MAX_ERRORS,
"PROCESSOR-TASK-ERROR-SITE-MAX-RESOURCES" : dc_processor.ProcessorTask.ERROR_MASK_SITE_MAX_RESOURCES,
"PROCESSOR-TASK-ERROR-SITE-UNSUPPORTED-CONTENT-TYPE" : dc_processor.ProcessorTask.ERROR_MASK_SITE_UNSUPPORTED_CONTENT_TYPE,
"PROCESSOR-TASK-ERROR-URL-ENCODING-ERROR" : dc_processor.ProcessorTask.ERROR_MASK_URL_ENCODING_ERROR,
"PROCESSOR-TASK-ERROR-SCRAPER-ERROR" : dc_processor.ProcessorTask.ERROR_MASK_SCRAPER_ERROR,
"PROCESSOR-TASK-ERROR-MISSED-RAW-CONTENT-ON-DISK" : dc_processor.ProcessorTask.ERROR_MASK_MISSED_RAW_CONTENT_ON_DISK
}


def getErrorMask():
  error_mask = None
  if len(sys.argv)==2:
    try:
      error_mask = int(sys.argv[1])
    except:
      print "error mask must be integer"
      sys.exit(2)
  else:
    print "Usage: ./dc-urls-mask.py <error mask>"
    sys.exit(2)
  logger.debug(MSG_DEBUG_INPUT_ERROR_MASK+ '\n' + str(error_mask))
  return error_mask


def convertMask(error_mask):
  founded = False
  masks = {"errors":[]}
  logger.debug("defined_masks: %s", str(defined_masks))
  logger.debug("error_mask: %s", str(type(error_mask)))
  for key in defined_masks:
    if defined_masks[key] & error_mask:
      masks["errors"].append(key)
      founded = True
  if founded:
    logger.debug(MSG_DEBUG_CONVERTED_MASKS + '\n' + str(masks))
  else:
    print "Mask " + str(error_mask) + " not defined both CrawlerTask or ProcessorTask"
    masks = {}
  return json.dumps(masks)




if __name__ == "__main__":
  try:
    exit_status = STATUS_SUCCESS
    error_mask = getErrorMask()
    masks = convertMask(error_mask)
    print masks
  except Exception, err:
    tbi = Utils.getTracebackInfo()
    logger.error(MSG_ERROR_EXIT_STATUS + err.message + '\n' + tbi)
    exit_status = STATUS_FAILURE
  exit(exit_status)
