#!/usr/bin/python


"""
  HCE project,  Python bindings, Distributed Tasks Manager application.
  Event objects definitions.

  @package: dc
  @file RTCFinalizer.py
  @author Oleksii <developers.hce@gmail.com>
  @link: http://hierarchical-cluster-engine.com/
  @copyright: Copyright &copy; 2013-2014 IOIX Ukraine
  @license: http://hierarchical-cluster-engine.com/license/
  @since: 0.1
  """


import ppath
import sys
import os
import pickle
import logging
import logging.config
import ConfigParser

from app.Utils import varDump
import app.Utils as Utils  # pylint: disable=F0401
import dc.EventObjects as dc_event
import dc.Constants as DC_CONSTS
from dc_db.TasksManager import TasksManager as DBTasksManager
import app.Consts as APP_CONSTS


EXIT_SUCCESS = 0
EXIT_FAILURE = 1

LOGGER_NAME = "RTCFinalizer"

MSG_ERROR_READ_BATCH = "ERROR READ BATCH FROM STDIN"
MSG_ERROR_DELETE_URL = "Delete url has failed"

DB_TASK_INI = "../ini/db-task.ini"


log_conf_file = "../ini/rtc-finalizer_log.ini"
logging.config.fileConfig(log_conf_file)
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
logger.setLevel("DEBUG")


class RTCFinalizer(object):

  def __init__(self):
    self.batch = None
    self.items = None
    self.error = EXIT_SUCCESS
    cfgParser = ConfigParser.ConfigParser()
    cfgParser.read(DB_TASK_INI)
    self.dbTask = DBTasksManager(cfgParser)
    self.urlContentResponse = None


  def getBatchFromInput(self):
    # read pickled batch object from stdin and unpickle it
    input_pickled_object = sys.stdin.read()
    #print input_pickled_object
    input_data = (pickle.loads(input_pickled_object))
    #print("Batch item: siteId: %s, urlId: %s" %(input_data.siteId, input_data.urlId))
    self.batch = input_data
    self.items = self.batch.items
    logger.debug("Batch: %s", varDump(self.batch))


  def getURLContent(self):
    urlContentRequest = []
    num_of_items = len(self.items)
    logger.debug("Num of items in batch: <<%s>>" % (num_of_items))
    item_no = 1
    for item in self.items:
      if not item:
        urlContentRequest.append(None)
        logger.debug("Item is None.")
      else:
        siteId = item.siteId
        url = item.urlObj.url
        urlId = item.urlId
        logger.debug("Item #%s: siteId: <<%s>>, urlId: <<%s>>, url: <<%s>>" % (item_no, siteId, urlId, url))
        _urlContentRequest = dc_event.URLContentRequest(siteId, url)
        _urlContentRequest.dbFieldsList = ["Status", "Crawled", "Processed", "ContentType", "Charset", "ErrorMask", "CrawlingTime", "ProcessingTime", "HTTPCode", "Size", "LinksI", "LinksE", "RawContentMd5", "LastModified", "CDate", "UDate", "TagsMask", "TagsCount", "PDate", "ContentURLMd5"]
        urlContentRequest.append(_urlContentRequest)
      item_no = item_no + 1
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_CONTENT, urlContentRequest)
    responseDRCESyncTasksCover = self.dbTask.process(drceSyncTasksCoverObj)
    self.urlContentResponse = responseDRCESyncTasksCover.eventObject
    logger.debug("urlContentResponse: %s", varDump(self.urlContentResponse))


  def sendURLContent(self):
    print pickle.dumps(self.urlContentResponse)
    sys.stdout.flush()



  def getURLContentFromBatch(self):
    self.urlContentResponse = []
    for item in self.items:
      url = item.urlObj.url
      logger.debug("item.urlPutObj.putDict.data: %s", varDump(item.urlPutObj.putDict["data"]))
      content = dc_event.Content(item.urlPutObj.putDict["data"], fieldId=dc_event.Content.CONTENT_PROCESSOR_CONTENT)
      urlContentResponse = dc_event.URLContentResponse(url, rawContents=None, processedContents=[content])
      urlContentResponse.status = 7
      self.urlContentResponse.append(urlContentResponse)
    logger.debug("urlContentResponse: %s", varDump(self.urlContentResponse))


  def deleteURLContent(self):
    items = self.batch.items
    urlDeleteRequest = []
    num_of_items = len(items)
    logger.debug("Num of items to delete in batch: <<%s>>" % (num_of_items))
    item_no = 1
    for item in self.items:
      if not item: continue
      siteId = item.siteId
      url = item.urlObj.url
      urlId = item.urlId
      logger.debug("Delete item #%s: siteId: <<%s>>, urlId: <<%s>>, url: <<%s>>" % (item_no, siteId, urlId, url))
      urlDeleteRequest.append(dc_event.URLDelete(siteId, url, reason=dc_event.URLDelete.REASON_RT_FINALIZER))
      item_no = item_no + 1
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_DELETE, urlDeleteRequest)
    responseDRCESyncTasksCover = self.dbTask.process(drceSyncTasksCoverObj)
    self.urlDeleteResponse = responseDRCESyncTasksCover.eventObject
    logger.debug("urlDeleteResponse: %s", varDump(self.urlDeleteResponse))
    for status in self.urlDeleteResponse.statuses:
      if status == False:
        self.error = EXIT_FAILURE
        logger.debug(MSG_ERROR_DELETE_URL)
      else:
        pass


  def process(self):
    self.getBatchFromInput()
    # Check if Real-Time crawling
    if self.batch.crawlerType == dc_event.Batch.TYPE_REAL_TIME_CRAWLER and \
      len(self.items) > 0 and \
      self.items[0] is not None and \
      "DB_TASK_MODE" in self.items[0].properties and \
      self.items[0].urlPutObj is not None and \
      self.items[0].properties["DB_TASK_MODE"] == "RO":
      self.getURLContentFromBatch()
      self.deleteURLContent()
    else:
      self.getURLContent()
    self.sendURLContent()


  def getError(self):
    return self.error


if __name__ == "__main__":
  try:
    rtcFinalizer = RTCFinalizer()
    rtcFinalizer.process()
    error = rtcFinalizer.getError()
  except Exception, err:
    tbi = Utils.getTracebackInfo()
    logger.error(str(vars(err)) + "\n" + tbi)
    sys.stderr.write("Traceback info:" + str(tbi))
    sys.stderr.write("Error:" + str(err))
    sys.stdout.flush()
    sys.stderr.flush()
    os._exit(EXIT_FAILURE)
  sys.stderr.write("Error:" + str(error))
  sys.stdout.flush()
  sys.stderr.flush()
  os._exit(error)

