#!/usr/bin/python


"""
  HCE project,  Python bindings, Distributed Tasks Manager application.
  Event objects definitions.
  
  @package: dc
  @file RTCPreprocessor.py
  @author Oleksii <developers.hce@gmail.com>
  @link: http://hierarchical-cluster-engine.com/
  @copyright: Copyright &copy; 2013-2014 IOIX Ukraine
  @license: http://hierarchical-cluster-engine.com/license/
  @since: 0.1
  """


import ppath
import sys
import os
import pickle
import logging
import logging.config

from app.Utils import varDump
import app.Utils as Utils  # pylint: disable=F0401
import dc.EventObjects as dc_event
import dc.Constants as DC_CONSTS
from dc_db.TasksManager import TasksManager as DBTasksManager
import app.Consts as APP_CONSTS


EXIT_SUCCESS = 0
EXIT_FAILURE = 1

ERROR_EMPTY_ENV_VARS = 2

DRCE_NODE_NUMBER = "DRCE_NODE_NUMBER"
DRCE_NODES_TOTAL = "DRCE_NODES_TOTAL"

MSG_ERROR_READ_BATCH = "ERROR READ BATCH FROM STDIN"
MSG_ERROR_DELETE_URL = "Delete url has failed"


log_conf_file = "../ini/rtc-preprocessor_log.ini"
logging.config.fileConfig(log_conf_file)
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
logger.setLevel("DEBUG")


class RTCPreprocessor(object):
  
  def __init__(self):
    self.batch = None
    self.error = EXIT_SUCCESS
    self.envVars = {
      DRCE_NODES_TOTAL: 1,
      DRCE_NODE_NUMBER: 1
    }
  
  
  def getBatchFromInput(self):
    self.pickled_object = sys.stdin.read()
  
  
  def cutBatch(self):
    self.batch = (pickle.loads(self.pickled_object))
    logger.debug("Input batch: %s", varDump(self.batch))
    items = self.batch.items
    num_of_items = len(items)
    splitted_items = self.split(self.batch.items, int(self.envVars[DRCE_NODES_TOTAL]))
    logger.debug("Input items: %s", str(self.batch.items))
    logger.debug("Splitted items: %s", str(splitted_items))
    self.batch.items = splitted_items[int(self.envVars[DRCE_NODE_NUMBER])-1]
    logger.debug("Output items: %s", str(self.batch.items))
    logger.debug("Output batch: %s", varDump(self.batch))
    self.pickled_object = pickle.dumps(self.batch)
  
  
  def split(self, arr, count):
    return [arr[i::count] for i in range(count)]


  def sendBatch(self):
    print self.pickled_object
    sys.stdout.flush()


  def getEnvVars(self):
    for key,value in self.envVars.items():
      if key in os.environ and os.environ[key] != "":
        self.envVars[key] = os.environ[key]
        logger.debug("os.environ[%s]: set to <<%s>>" % (key, self.envVars[key]))
      else:
        logger.debug("os.environ[%s]: not set. Use default value: <<%s>>" % (key, self.envVars[key]))
        self.error = ERROR_EMPTY_ENV_VARS


  def process(self):
    try:
      self.getBatchFromInput()
      self.getEnvVars()
      if self.error != ERROR_EMPTY_ENV_VARS:
        self.cutBatch()
      self.sendBatch()
    except Exception:
      self.error = EXIT_FAILURE
        
        
  def getError(self):
    return self.error


if __name__ == "__main__":
  try:
    rtcPreprocessor = RTCPreprocessor()
    rtcPreprocessor.process()
    error = rtcPreprocessor.getError()
  except Exception, err:
    tbi = Utils.getTracebackInfo()
    logger.error(str(vars(err)) + "\n" + tbi)
    sys.stderr.write(tbi)
    sys.stdout.flush()
    sys.stderr.flush()
    os._exit(EXIT_FAILURE)
  except:
    tbi = Utils.getTracebackInfo()
    logger.error(str(vars(err)) + "\n" + tbi)
    sys.stderr.write(tbi)
    sys.stdout.flush()
    sys.stderr.flush()
  os._exit(error)
