#!/usr/bin/python


"""
HCE project, Python bindings, Distributed Tasks Manager application.
Event objects definitions.

@package: dc
@file urls-to-batch-task.py
@author Oleksii <developers.hce@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import ppath
from ppath import sys

import ctypes
import zlib
import logging
import logging.config
import json
import copy
from dcc.DCCObjectsSerializator import DCCObjectsSerializator
try:
  import cPickle as pickle
except ImportError:
  import pickle

from transport.IDGenerator import IDGenerator
import app.Utils as Utils  # pylint: disable=F0401
from dc.EventObjects import Batch
from dc.EventObjects import BatchItem
import dc.EventObjects as dc_event
import app.Consts as APP_CONSTS


log_conf_file = "../ini/urls-to-batch-task_log.ini"
logging.config.fileConfig(log_conf_file)
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
logger.setLevel("DEBUG")

"""
logging.basicConfig(filename="../log/urls-to-batch-task.log", filemode="w")
logger = logging.getLogger("")
logger.setLevel("DEBUG")
"""

MSG_ERROR_EXIT_STATUS = "Exit failure. "

MSG_DEBUG_INPUT_PICKLE = "Input pickle: "
MSG_DEBUG_INPUT_UNPICKLE = "Input unpickle: "
MSG_DEBUG_LEN_URL_LIST = "Input url list count: "
MSG_DEBUG_INPUT_URL_LIST = "Append url: "
MSG_DEBUG_UNIQ_URL_LIST = "Append uniq url: "
MSG_DEBUG_OUTPUT_BATCH_ITEM = "Output batch item: "
MSG_DEBUG_OUTPUT_BATCH = "Output batch: "
MSG_DEBUG_OUTPUT_PICKLE = "Output pickle: "
MSG_DEBUG_SEND_PICKLE = "Send pickle. Done."


STATUS_SUCCESS = 0
STATUS_FAILURE = 1

def getInputPickle():
  input_pickle = None
  input_pickle = sys.stdin.read()
  logger.debug(MSG_DEBUG_INPUT_PICKLE + '\n' + str(input_pickle))
  return input_pickle


def unpickleInput(input_pickle):
  # input_unpickled_obj = input_pickle
  input_unpickled_obj = None
  input_unpickled_obj = pickle.loads(input_pickle).eventObject
  logger.debug(MSG_DEBUG_INPUT_UNPICKLE + '\n' + str(input_unpickled_obj))
  return input_unpickled_obj


def loadListOfURLs(input_unpickled_obj):
  list_of_url_obj = input_unpickled_obj
  # list_of_url_obj = []
  # items_list = json.loads(input_unpickled_obj)["itemsList"]
  # logger.debug(MSG_DEBUG_INPUT_URL_LIST + str(items_list))
  # logger.debug(MSG_DEBUG_INPUT_URL_LIST + str(type(items_list)))
  # list_of_urls = items_list[0]["itemObject"]
  # logger.debug(MSG_DEBUG_INPUT_URL_LIST + str(list_of_urls))
  # convertor = DCCObjectsSerializator()
  # list_of_url_obj = convertor.URLNewDeserialize(list_of_urls)
  logger.debug(MSG_DEBUG_LEN_URL_LIST + str(len(list_of_url_obj)))
  logger.debug(MSG_DEBUG_INPUT_URL_LIST + str(list_of_url_obj))
  return list_of_url_obj


def getListOfUniqueURLs(list_of_url_obj):
  list_of_uniq_urls = []
  seen = set()
  list_of_uniq_urls = [url_obj for url_obj in list_of_url_obj if url_obj.urlMd5 not in seen and not seen.add(url_obj.urlMd5)]
  logger.debug(MSG_DEBUG_UNIQ_URL_LIST + str(list_of_uniq_urls))
  return list_of_uniq_urls


def createBatchId():
  batch_id = None
  idGenerator = IDGenerator()
  batch_id = ctypes.c_uint32(zlib.crc32(idGenerator.get_connection_uid())).value
  return batch_id


def createBatchItems(list_of_uniq_urls):
  list_of_batch_items = []
  for url_obj in list_of_uniq_urls:
    url_obj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
    site_id = url_obj.siteId
    url_id = url_obj.urlMd5
    logger.debug(MSG_DEBUG_OUTPUT_BATCH_ITEM + str(url_obj))
    batch_item = BatchItem(site_id, url_id, url_obj)
    list_of_batch_items.append(batch_item)
  return list_of_batch_items


def createOutputBatch(batch_id, list_of_batch_items):
  output_batch = None
  output_batch = Batch(batch_id, list_of_batch_items)
  logger.debug(MSG_DEBUG_OUTPUT_BATCH + str(list_of_batch_items))
  logger.debug(MSG_DEBUG_OUTPUT_BATCH + str(output_batch))
  return output_batch


def createOutputPickle(output_batch):
  output_pickle = None
  output_pickle = pickle.dumps(output_batch)
  logger.debug(MSG_DEBUG_OUTPUT_PICKLE + str(output_pickle))
  return output_pickle


def sendPickle(output_pickle):
  sys.stdout.write(output_pickle)
  logger.debug(MSG_DEBUG_SEND_PICKLE)


if __name__ == "__main__":
  try:
    exit_status = STATUS_SUCCESS
    input_pickle = getInputPickle()
    input_unpickled_obj = unpickleInput(input_pickle)
    list_of_url_obj = loadListOfURLs(input_unpickled_obj)
    list_of_uniq_urls = getListOfUniqueURLs(list_of_url_obj)
    batch_id = createBatchId()
    list_of_batch_items = createBatchItems(list_of_uniq_urls)
    output_batch = createOutputBatch(batch_id, list_of_batch_items)
    output_pickle = createOutputPickle(output_batch)
    sendPickle(output_pickle)
  except Exception, err:
    tbi = Utils.getTracebackInfo()
    logger.error(MSG_ERROR_EXIT_STATUS + err.message + '\n' + tbi)
    exit_status = STATUS_FAILURE
  exit(exit_status)
