#!/bin/bash

CRAWL_DELAY=120
DELETE_DELAY=90

TEMPFILE=`mktemp /tmp/dc_test_rnd_site.XXXXXX`
trap "rm -f $TEMPFILE" 0 1 2 5 15

function checkErrors(){
    #$1 - file with temp log
    #$2 - command
    while read LINE
    do
        if [[ $LINE =~ "errorCode" ]]
        then
            CHECK=$(echo $LINE | awk -F\: '{print $2}' | sed 's/ \|,//g')
            if [[ $CHECK =~ ';' ]]
            then
                CHECKARR=($(echo $CHECK | sed 's/"//g' | tr ";" "\n"))
                for ERR in ${CHECKARR[@]}
                do
                    if [[ $ERR != 0 ]]
                    then
                        read errMsg
                        echo $errMsg
                        echo $2 "error number "$ERR
                        echo -e "\n#-#FATAL:"$2" error="$ERR"-#" >> ../log/$0.log
                        exit 1
                    fi
                done
            else
                if [[ $CHECK != 0 ]]
                then
                    read errMsg
                    echo $2 "error number "$CHECK $errMsg
                    echo -e "\n#-#FATAL:"$2" error="$CHECK"-#" >> ../log/$0.log
                    exit 1
                fi
            fi
        fi
    done < $1
}


#Remove log files
if [[ -f ../log/* ]]
then
    rm ../log/*
fi
echo "" > ../log/$0.log 2>&1

# ADD and Check Permissions #
cd ../../../usr/bin/
./hce-node-permissions.sh
CHECK_PERM=$(./check_permissions.sh)
cd ../../api/python/ftests/
if [[ $CHECK_PERM =~ 'Error' ]]
then
    echo 'Error: Permissions error'
    echo $CHECK_PERM >> ../log/$0.log
    echo "#-#FATAL:PERMISSIONS_ERROR-#"  >> ../log/$0.log
    exit 1
fi

#Before actions
source ./dc_tests_before.sh

sleep 2

echo -e "\n#-#\nSite find" >> ../log/$0.log 2>&1
../bin/dc-client.py --config=../ini/dc-client.ini --command=SITE_FIND --file=../data/ftests/dcc_site_find_rnd.json | tee -a ../log/$0.log 2>&1 > $TEMPFILE
checkErrors $TEMPFILE SITE_FIND

echo -e "\n#-#\nDelete site" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=SITE_DELETE --file=../data/ftests/dcc_site_delete_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE SITE_DELETE

echo -e "\n#-#\nCreate new site" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=SITE_NEW --file=../data/ftests/dcc_site_new_ok_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE SITE_NEW

echo -e "\n#-#\nSite find" >> ../log/$0.log 2>&1
../bin/dc-client.py --config=../ini/dc-client.ini --command=SITE_FIND --file=../data/ftests/dcc_site_find_rnd.json | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE SITE_FIND

echo -e "\n#-#\nCheck site status before crawling" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=SITE_STATUS --file=../data/ftests/dcc_site_status_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE SITE_STATUS

echo -e "\n#-#\nAdd new URL" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=URL_NEW --file=../data/ftests/dcc_url_new_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE URL_NEW

echo -e "\n#-#\nCheck URLs status before crawling" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=URL_STATUS --file=../data/ftests/dcc_url_status_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE URL_STATUS

echo -e "\n#-#\nCrawling started at:" >> ../log/$0.log 2>&1
date >> ../log/$0.log 2>&1

echo -e "\n#-#\nSleep $CRAWL_DELAY sec" >> ../log/$0.log 2>&1
sleep $CRAWL_DELAY

echo -e "\n#-#\nCrawling finished 1 at:" >> ../log/$0.log 2>&1
date >> ../log/$0.log 2>&1

echo -e "\n#-#\nCheck sites status after crawling" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=SITE_STATUS --file=../data/ftests/dcc_site_status_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE SITE_STATUS

echo -e "\n#-#\nCheck URLs status after crawling" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=URL_STATUS --file=../data/ftests/dcc_url_status_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE URL_STATUS

echo -e "\n#-#\nGet data of URL" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=URL_CONTENT --file=../data/ftests/dcc_url_content_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE URL_CONTENT

#echo "Cleanup sites data" >> ../log/$0.log 2>&1
#../manage/dc-client_start.sh "--command=SITE_CLEANUP --file=../data/ftests/dcc_site_cleanup_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
#checkErrors $TEMPFILE SITE_CLEANUP

#echo "Update site" >> ../log/$0.log 2>&1
#../manage/dc-client_start.sh "--command=SITE_UPDATE --file=../data/ftests/dcc_site_update_rnd.json" >> ../log/$0.log 2>&1

#echo "Check sites status after update" >> ../log/$0.log 2>&1
#../manage/dc-client_start.sh "--command=SITE_STATUS --file=../data/ftests/dcc_site_status_rnd.json" >> ../log/$0.log 2>&1

#echo "Update URL" >> ../log/$0.log 2>&1
#../manage/dc-client_start.sh "--command=URL_UPDATE --file=../data/ftests/dcc_url_update_rnd.json" >> ../log/$0.log 2>&1

#echo "Crawling started at:" >> ../log/$0.log 2>&1
#date >> ../log/$0.log 2>&1

#echo "Sleep 60 sec" >> ../log/$0.log 2>&1
#sleep 60

#echo "Crawling finished 2 at:" >> ../log/$0.log 2>&1
#date >> ../log/$0.log 2>&1

#echo "Check sites status after crawling" >> ../log/$0.log 2>&1
#../manage/dc-client_start.sh "--command=SITE_STATUS --file=../data/ftests/dcc_site_status_rnd.json" >> ../log/$0.log 2>&1

echo "Get URL content before delete" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=URL_CONTENT --file=../data/ftests/dcc_url_content_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE URL_CONTENT

echo "Delete URLs and resources crawled" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=URL_DELETE --file=../data/ftests/dcc_url_delete_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE URL_DELETE

echo -e "\n#-#\nSleep $DELETE_DELAY sec" >> ../log/$0.log 2>&1
sleep $DELETE_DELAY

echo "Get URL content after delete" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=URL_CONTENT --file=../data/ftests/dcc_url_content_rnd.json" | tee -a ../log/$0.log > $TEMPFILE 2>&1
checkErrors $TEMPFILE URL_CONTENT

echo "URLs content store for viewer" >> ../log/$0.log 2>&1
../manage/dc-client_start.sh "--command=URL_CONTENT --file=../data/ftests/dcc_url_content_rnd.json" | tee -a ../log/dcc_url_content_rnd.result.json > $TEMPFILE 2>&1
checkErrors $TEMPFILE URL_CONTENT
echo "Get content to view tags" >> ../log/$0.log 2>&1
cd ../bin && ./json-field.py -f "itemsList:0:itemObject:0:processedContents:0:buffer" -b 1 < ../log/dcc_url_content_rnd.result.json > ../log/dcc_url_content_rnd.result.json.value

echo "Test fetch content automation script" >> ../log/$0.log 2>&1
#cd ../ftests && ./dc-kvdb-resource.sh 699fcf4591fc23e79b839d8819904293 699fcf4591fc23e79b839d8819904293 >> ../log/$0.log 2>&1
cd ../ftests && ./dc-kvdb-resource.sh b85ab149a528bd0a024fa0f43e80b5fc b85ab149a528bd0a024fa0f43e80b5fc >> ../log/$0.log 2>&1

#echo "Delete site" >> ../log/$0.log 2>&1
#../manage/dc-client_start.sh "--command=SITE_DELETE --file=../data/ftests/dcc_site_delete_rnd.json" >> ../log/$0.log 2>&1

#echo "Check sites status after delete" >> ../log/$0.log 2>&1
#../manage/dc-client_start.sh "--command=SITE_STATUS --file=../data/ftests/dcc_site_status_rnd.json" >> ../log/$0.log 2>&1


#echo "After tests cleaning..." >> ../log/$0.log 2>&1

#After actions
#./dc_tests_after.sh

echo "Finished..." >> ../log/$0.log 2>&1