#!/bin/bash

if [[ $USER == 'root' ]]
then
  echo "You run script from root or sudo. Run from www-data. Exit"
  exit 1
fi


BUNDLE_DIR=$HOME'/hce-node-bundle'
JSONDIR=$BUNDLE_DIR'/api/python/data/ftests/template_scraping'
COMPARATOR=$BUNDLE_DIR'/api/php/bin/formats-standards-comparator.php'
JSON_FIELD=$BUNDLE_DIR'/api/python/bin/json-field.py'
LOG_DIR='/tmp/TBS'
FIELD_PATH='itemsList:0:itemObject:0:processedContents:0:buffer'
FORMAT_PATH='items:0:properties:template:templates:0:output_format:name'
IGNORED_FIELDS=('0:pubdate')
FAIL=()
SUCCESS=()

RED='\e[1;31m'
BLUE='\e[1;34m'
TUR='\e[1;36m'
YEL='\e[1;33m'
DEF='\e[0m'

if [[ ! -d $JSONDIR ]]
then
    echo -e $RED"Direcroty "$JSONDIR" not found. Exit"$DEF
    exit 1
fi
if [[ ! -f $COMPARATOR ]]
then
    echo -e $RED"File "$COMPARATOR" not found. Exit"$DEF
    exit 1
fi

if [[ ! -d $LOG_DIR ]]
then
    mkdir $LOG_DIR
fi

TEMPFILE=$(mktemp /tmp/$(basename $0).XXXXX)
trap "rm -f $TEMPFILE" 0 1 2 5 15


function checkIgnored() {
    # $1 - input to check
    echo -n > $TEMPFILE
    echo "$1" | while read LINE
    do
        for I in "${IGNORED_FIELDS[@]}"
        do
            if [[ ! "$LINE" =~ "$I" ]]
            then
                echo -e $JSONFILE" -$RED fail, $LINE"$DEF >> $TEMPFILE
            fi
        done
    done
}


cd $BUNDLE_DIR/api/python/bin

for OUT in $(ls $JSONDIR | grep '.json.out')
do
    JSONFILE=${OUT%.out}
    FORMAT=$($JSON_FIELD  --field=$FORMAT_PATH < $JSONDIR/$JSONFILE)

    ./dc-client.py --config=../ini/dc-client_TBS.ini --command=BATCH --file=$JSONDIR/$JSONFILE > $TEMPFILE

    CHECK=$($COMPARATOR --original=$JSONDIR/$OUT --compare=$TEMPFILE --path=$FIELD_PATH --format=$FORMAT)
    if [[ -z $CHECK ]]
    then
        SUCCESS+=("$JSONFILE")
        echo -e $JSONFILE" -$BLUE success $DEF"
    else
        checkIgnored "$CHECK"
        CHECK_LINE=$(wc -l $TEMPFILE | awk '{print $1}')
        if [[ $CHECK_LINE -gt 0 ]]
        then
            FAIL+=("$JSONFILE")
            echo -e $JSONFILE" -$RED fail $DEF, $CHECK"
        else
            SUCCESS+=("$JSONFILE")
            echo -e $JSONFILE" -$BLUE success $DEF"
        fi
    fi
done

if [[ ${SUCCESS[@]} ]]
then
    echo -e $BLUE"Number of success: "${#SUCCESS[@]} $DEF
fi

if [[ ${FAIL[@]} ]]
then
    echo -e $RED"Number of fails: "${#FAIL[@]} $DEF
    exit 1
fi

exit 0
