'''
Created on Mar 17, 2015

@package: app
@author: scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''
import re
import datetime
import logging
import copy
import dc.Constants as DC_CONSTS

logger = logging.getLogger(DC_CONSTS.LOGGER_NAME)

class Filters(object):

  BD_NAME = "dc_sites"
  REPLACE_MARKER = '%'
  ACTION_MULTIPLE = -1

  FILTER_NAME_PATTERN = "Pattern"
  FILTER_NAME_SUBJECT = "Subject"
  FILTER_NAME_OP_CODE = "OperationCode"
  FILTER_NAME_STAGE = "Stage"
  FILTER_NAME_ACTION = "Action"
  FILTER_NAME_GROUP = "Group_Id"

  STAGE_COLLECT_URLS = 0
  STAGE_BEFORE_DOM_PRE = 1
  STAGE_AFTER_DOM_PRE = 2
  STAGE_AFTER_DOM = 3
  STAGE_AFTER_PROCESSOR = 4
  STAGE_ALL = 5
  
  LOGIC_OR = 0
  LOGIC_AND = 1
  
  # Constants for operation codes, implements in Filters.filterAll method body
  OC_RE = 0
  OC_EQ = 1
  OC_NOTEQ = 2
  OC_EQLESS = 3
  OC_EQMORE = 4
  OC_LESS = 5
  OC_MORE = 6


  ##constructor
  #
  #@param filters - incoming filters, makes deepcopy inside class constructor
  #@param dbTaskWrapper - db-task wrapper (not used now)
  #@param siteId - used with db-task wrapper (not used now)
  def __init__(self, filters, dbTaskWrapper=None, siteId=None, readMode=0):
    if filters is not None:
      # if filters is not None istantiate self.filters directly
      self.filters = copy.deepcopy(filters)
    else:
      # else fill self.filters from database, using dbTask and siteId params (make SiteStatusTask request)
      # Look CrawlerTask.readSiteFromDB method, return Site.filters field and parse it
      self.filters = []
      if dbTaskWrapper is not None and siteId is not None:
        self.readFiltersFromDB(dbTaskWrapper, siteId, readMode)


  ##readFiltersFromDB read filters from DB (using SQLCustom request to db-task) for specific site
  #
  #@param dbTaskWrapper - db-task wrapper
  #@param siteId - site Id
  def readFiltersFromDB(self, dbTaskWrapper, siteId, readMode):
    SQL_SELECT_TEMPLATE = "SELECT `Pattern`, `Subject`, `OperationCode`, `Stage`, `Action`, `Group_Id` " + \
    "FROM `sites_filters` WHERE `Mode`=%s AND `State`=1 AND `Site_Id`='%s'"
    query = SQL_SELECT_TEMPLATE % (str(readMode), str(siteId))
    logger.info(">>> Filter start SQL Req")
    customResponse = dbTaskWrapper.customRequest(query, self.BD_NAME)
    logger.info(">>> Filter end SQL Req")
    if customResponse is not None:
      for elem in customResponse:
        if elem is not None:
          localDict = {}
          localDict[self.FILTER_NAME_PATTERN] = elem[0]
          localDict[self.FILTER_NAME_SUBJECT] = elem[1]
          localDict[self.FILTER_NAME_OP_CODE] = elem[2]
          localDict[self.FILTER_NAME_STAGE] = elem[3]
          localDict[self.FILTER_NAME_ACTION] = elem[4]
          localDict[self.FILTER_NAME_GROUP] = elem[5]
          self.filters.append(localDict)


  ##macroReplace makes macroreplacement in incoming pattern string by string from values dict
  #
  #@param pattern - incoming pattern string
  #@param values - dict with incoming old substrings (values.keys) and new substrings (values.values)
  #@param marker - additional prefix+suffix for values.keys
  #@return replacemented string
  def macroReplace(self, pattern, values, marker):
    ret = copy.copy(pattern)
    for key in values:
      if values[key] is not None:
        findKey = marker + key + marker
        ret = ret.replace(findKey, str(values[key]))
    return ret


  ##comparing values comparing method
  #
  #@param value1 - comparing value1
  #@param value2 - comparing value2
  #@param OCType - operation type
  #@return bool result of values comparing
  def comparing(self, value1, value2, OCType):
    ret = False
    try:
      if OCType == self.OC_RE:
        pattern = re.compile(value2)
        if pattern.match(value1) is not None:
          ret = True
      elif OCType == self.OC_EQ:
        ret = (int(value1) == int(value2))
      elif OCType == self.OC_NOTEQ:
        ret = (int(value1) != int(value2))
      elif OCType == self.OC_EQLESS:
        ret = (int(value1) <= int(value2))
      elif OCType == self.OC_EQMORE:
        ret = (int(value1) >= int(value2))
      elif OCType == self.OC_LESS:
        ret = (int(value1) < int(value2))
      elif OCType == self.OC_MORE:
        ret = (int(value1) > int(value2))
    except ValueError as exp:
      logger.debug(">>> Value error = " + str(exp))
    except Exception as exp:
      logger.debug(">>> Common exception, OCType = " + str(OCType) + " val1 = " + str(value1) + " val2 = " \
                    + str(value2))
    return ret


  ##filterAll method applyes all filters and return result
  #
  #@param stage - current stage, available stages looks above
  #@param value - replacement values (dict)
  #@param logic - 
  #@return bool result of values comparing
  def filterAll(self, stage, value, logic=LOGIC_AND, subject=None):
    ret = []
    localGroupDict = {}
    for localFilter in self.filters:
      if stage == self.STAGE_ALL or localFilter[self.FILTER_NAME_STAGE] == self.STAGE_ALL or \
      localFilter[self.FILTER_NAME_STAGE] == stage:
        if subject is not None and localFilter[self.FILTER_NAME_SUBJECT] == "":
          localStage = subject
        else:
          localStage = localFilter[self.FILTER_NAME_SUBJECT]

        #   make macroreplacement localFilter["Subject"] by correspond value from value param dict
        #   apply localFilter["Patter"] for previously getted result (result of macroreplacement)
        if localFilter[self.FILTER_NAME_PATTERN] is not None:
          localPattern = self.macroReplace(localFilter[self.FILTER_NAME_PATTERN], value, self.REPLACE_MARKER)
          if logic == self.LOGIC_OR:
          # Return ret = [x1, x2, ... xN] where (xN = +/- localFilter["Action"] value) of applying pattern
          # and +/- depend on corresponds filter for own "Pattern" or not.
          # for example: 
          # we have 3 applying filters with localFilter["Action"] = {3, 4, 5} correspondingly,
          # 1th filter doesn't correspond own localFilter["Pattern"] , 2th and 3th - correspond
          # that we return ret = [-3, 4, 5]
            '''
            if self.comparing(localStage, localPattern, localFilter[self.FILTER_NAME_OP_CODE]):
              ret.append(localFilter[self.FILTER_NAME_ACTION])
            else:
              ret.append(localFilter[self.FILTER_NAME_ACTION] * self.ACTION_MULTIPLE)
            '''
            if self.comparing(localStage, localPattern, localFilter[self.FILTER_NAME_OP_CODE]):
              localRes = localFilter[self.FILTER_NAME_ACTION]
            else:
              localRes = localFilter[self.FILTER_NAME_ACTION] * self.ACTION_MULTIPLE
            
            if localFilter[self.FILTER_NAME_GROUP] in localGroupDict:
              if localGroupDict[localFilter[self.FILTER_NAME_GROUP]] > 0:
                localGroupDict[localFilter[self.FILTER_NAME_GROUP]] = localRes
            else:
              localGroupDict[localFilter[self.FILTER_NAME_GROUP]] = localRes
            
          elif logic == self.LOGIC_AND:
          # If all applying filters correspond own "Pattern"s that ret = [True], else ret = [False]
            if self.comparing(localStage, localPattern, localFilter[self.FILTER_NAME_OP_CODE]):
              ret = [True]
            else:
              ret = [False]
              break
    if logic == self.LOGIC_OR:
      ret = localGroupDict.values()
    return ret
  

  #Method found value in existing filters ([self.FILTER_NAME_ACTION] field)
  #return bool result of finding
  def isExistInActions(self, value):
    ret = False
    for localFilter in self.filters:
      if localFilter is not None and localFilter[self.FILTER_NAME_ACTION] == value:
        ret = True
        break
    return ret
