'''
Created on Mar 28, 2014

@package: app
@author: scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

import json
import re
import sys
import os
import traceback
import urlparse
import Exceptions
import types
import time
from scrapy.utils.url import canonicalize_url
from datetime import datetime
from decimal import Decimal
from bs4 import BeautifulSoup, Comment
import logging
import app.Consts as APP_CONSTS
import Url

try:
  import cPickle as pickle
except ImportError:
  import pickle


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)



# #PropertiesValidator contains only 1 method - isValueIn, that find value in class(classType param)
# attributes that began with "prefix"
#
class PropertiesValidator(object):


  def __init__(self):
    pass


  @staticmethod
  def isValueIn(classType, prefix, value):
    retVal = False
    for localValue in classType.__dict__:
      if str(localValue).find(prefix) == 0:
        if value == getattr(classType, localValue, None):
          retVal = True
          break
    return retVal


# #getPath global function, finds and return value by path in json string or dict document
# dictionary - incoming dictionary (optional)
# jsonString - incoming json string (optional)
# path - incoming path to find
# method return valid value or raises exceptions -
# [ValueError] - bad jsodnString format
# [TypeError, KeyError, IndexError] - excpetions raised if path not found
# Warning!!! path don't checks by syntaxis
def getPath(dictionary, jsonString, path):
  if jsonString != None:
    dictionary = json.loads(jsonString)
  for i, p in re.findall(r'(\d+)|(\w+)', path):
    dictionary = dictionary[p or int(i)]
  return dictionary



# #Json serialization
#
class JsonSerializable(object):

  # #constructor
  # initialize task's fields
  #
  def __init__(self):
    pass

  @staticmethod
  def json_serial(obj):
    if isinstance(obj, datetime):
      return obj.isoformat()
    else:
      if isinstance(obj, Decimal):
        return str(obj)
      else:
        #if isinstance(obj, type.DictProxy):
        if isinstance(obj, types.DictProxyType):
          return dict(obj)
        else:
          return obj.__dict__


  def toJSON(self):
    # return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    # Support custom serialization of datetime
    return json.dumps(self.__dict__, default=JsonSerializable.json_serial, sort_keys=True, indent=4)



class SQLExpression(object):
  def __init__(self, stringExpression=""):
    super(SQLExpression, self).__init__()
    if stringExpression is None:
      self.str = ""
    else:
      self.str = str(stringExpression)

  def __str__(self):
    return self.str



# #The PathMaker class
#
# This class purpose to use as static method container to split FS directory pattern on two parts
# @param string is a directory pattern
# @param subdirLen is a number of characters in sub-directory item
#
class PathMaker(object):

  SUBDIR_LEVEL1_LEN = 2
  SUBDIR_CHAR = "/"

  def __init__(self, string, subdirLen=SUBDIR_LEVEL1_LEN):
    super(PathMaker, self).__init__()

    self.string = string
    self.subdirLen = subdirLen

    if "CONTENT_STORE_PATH" in os.environ and os.environ["CONTENT_STORE_PATH"] != "":
      logger.debug("os.environ[CONTENT_STORE_PATH]: set to %s" % (os.environ["CONTENT_STORE_PATH"]))
      self.string += "/"
      self.string += os.environ["CONTENT_STORE_PATH"]
    else:
      logger.debug("os.environ[CONTENT_STORE_PATH]: not set.")

  def getDir(self):
    if len(self.string) > self.subdirLen:
      return self.string[:self.subdirLen] + self.SUBDIR_CHAR + self.string[self.subdirLen:]
    else:
      return self.string



# #The function to get traceback information string prepared for logging
#
# This function collects traceback information and creates sreing representation ready to log it
# @param linesNumberMax max number of traceback lines to include in to the collection, None - signs all
# @ret return string
#
def getTracebackInfo(linesNumberMax=None):
  ret = ""
  n = 0

  type_, value_, traceback_ = sys.exc_info()
  stack = traceback.format_tb(traceback_)
  del type_
  del value_
  for item in stack:
    ret = ret + "\n" + (str(item))
    n = n + 1
    if linesNumberMax != None and n == linesNumberMax:
      break

  return ret



# #The function to get a printable representation of an object for debugging
#
#
# @param obj The object to print
# @ret return string dump
#
'''
def varDump(obj):
  if '__dict__' in dir(obj):
    return str(vars(obj))
  else:
    return str(obj)
'''
def varDump(obj, stringify=True):
  newobj = obj
  if isinstance(obj, list):
    newobj = []
    for item in obj:
      newobj.append(varDump(item, False))
  elif isinstance(obj, tuple):
    temp = []
    for item in obj:
      temp.append(varDump(item, False))
    newobj = tuple(temp)
  elif isinstance(obj, set):
    temp = []
    for item in obj:
      temp.append(str(varDump(item, False)))
    newobj = set(temp)
  elif isinstance(obj, dict):
    newobj = {}
    for key, value in obj.items():
      newobj[str(varDump(key, False))] = varDump(value, False)
  # elif isinstance(obj, types.FunctionType):
  #  newobj = repr(obj)
  elif '__dict__' in dir(obj):
    newobj = obj.__dict__.copy()
    if ' object at ' in str(obj) and not '__type__' in newobj:
      newobj['__type__'] = str(obj).replace(" object at ", " #").replace("__main__.", "")
    for attr in newobj:
      newobj[attr] = varDump(newobj[attr], False)

  if stringify:
    return str(newobj)
  else:
    return newobj



def memUsage(point=""):
  import resource
  usage = resource.getrusage(resource.RUSAGE_SELF)
  return '''%s: mem=%s mb
      ''' % (point, resource.getrusage(resource.RUSAGE_SELF).ru_maxrss / 1000)



# #class UrlParser makes URL operation
#
class UrlParser(object):


  def __init__(self):
    pass


  # #The function check protocol and dimain name in the incoming url
  #
  # url - param, incoming url
  # @ret boolean
  @staticmethod
  def isValidURL(url):
    ret = False
    parseUrl = urlparse.urlparse(url)
    if parseUrl.scheme != None and parseUrl.netloc != None:
      ret = True
    return ret


  # #The function generates base url
  #
  # url - param, incoming url
  # @ret boolean
  @staticmethod
  def generateDomainUrl(url):
    ret = ""
    parseUrl = urlparse.urlparse(url)
    if UrlParser.isValidURL(url):
      ret = parseUrl.scheme + "://" + parseUrl.netloc
    else:
      raise Exceptions.UrlParseException("Empty protocol or domain name")
    return ret



# #class UrlNormalizator makes URL normalization
#
class UrlNormalizator(object):

  OWN_ALGO = 0
  T100_ALGO = 1

  def __init__(self):
    pass


  # #The function normalize the incoming url according  RFC 3986
  #
  # url - param, incoming url
  # @ret normalized url
  @staticmethod
  def normalize(url):
    norm_url = url
    try:
      # logger.debug("input url: <%s>", url)
      # url.strip() removes trailing special characters like '\n'
      # without it urls inserted with '\n' symbol at the end
      # as result will be crawled only sites ended by '/' like 'html://www.foo.com/\n'
      # however full urls won't crawled like 'html:/www.foo.com/file.html\n'
      # norm_url = str(urlnorm.norm(url.strip()))
      # logger.debug("AAAAA: <%s>", norm_url)
      norm_url = str(urinormpath(url.strip()))
      #norm_url = str(canonicalize_url(url.strip()))
      # logger.debug("norm_url: <%s>", norm_url)
    # except urlnorm.InvalidUrl:
    #  logger.error("Normalization InvalidUrl")
    #  norm_url = ""
    except Exception as e:
      logger.error("Normalization error: " + str(e.message) + "\nURL: [" + url + "]")

    return norm_url



# ENV_SCRAPER_STORE_PATH = "ENV_SCRAPER_STORE_PATH"
# # storePickleOnDisk
#
def storePickleOnDisk(input_pickled_object, env_path, file_name):
  if env_path in os.environ and os.environ[env_path] != "":
    logger.debug("os.environ[%s]: set to %s" % (env_path, os.environ[env_path]))
    open(os.environ[env_path] + file_name, "wb").write(input_pickled_object)
  else:
    logger.debug("os.environ[%s]: not set.", env_path)



# This function taken from uritools module as it was removed from module
def urinormpath(path, algorithm=UrlNormalizator.T100_ALGO, stripWWW=False, useValidator=False):
  #Remove '.' and '..' path segments from a URI path.
  # RFC 3986 5.2.4. Remove Dot Segments
  ret = None
  ret1 = None

  if path is None or path == "":
    logger.debug(">>> Normilize empty path")
    ret1 = path
  else:
    out = []
    for s in path.split('/'):
      if s == '.':
        continue
      elif s != '..':
        out.append(s)
      elif out:
        out.pop()
    # Fix leading/trailing slashes
    if path.startswith('/') and (not out or out[0]):
      out.insert(0, '')
    if path.endswith('/.') or path.endswith('/..'):
      out.append('')
    ret = '/'.join(out)

    if stripWWW:
      splitPath = path.split("?")
      if len(splitPath) > 0:
        splitPath[0] = splitPath[0].replace("://www.", "://")
        localPath = splitPath[0]
        for elem in splitPath[1:]:
          localPath += "?"
          localPath += elem
    else:
      localPath = path
    resultUrlDict = Url.Url(localPath)
    if useValidator and not Url.Url.GetStats([resultUrlDict])[0]["valid"]:
      raise Exception(path + " NOT VALIDATE!")
    ret1 = Url.Url.GetStats([resultUrlDict])[0]["canonicalized"]

    if ret is not None and ret1 is not None and ret != ret1:
      logger.debug("--->>>> URLS DIFFERTNT <<<<---")
      logger.debug(ret)
      logger.debug(ret1)
  return ret1



# #Logger file name generator
#
#
class LoggerFileName(object):


  def __init__(self, loggerInst=None):
    pass


  def getFreeProcInstanceNumber(self, sock_prefix="module", min_number=0, max_number=32):
    ret = ""

    for i in range(min_number, max_number):
      try:
        import socket
        global lock_socket
        lock_socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        # Create an abstract socket
        lock_socket.bind('\0' + "dc_process_lock_" + sock_prefix + str(i))
        ret = str(i)
        break
      except socket.error:
        #Socket already exists
        continue

    return ret


  def findReplace(self, newFile=None):
    log_file = None

    rootlogger = logging.getLogger('')
    for h in rootlogger.__dict__['handlers']:
      if h.__class__.__name__ == 'FileHandler':
        log_file = h.baseFilename
        if newFile is not None:
          h.baseFilename = newFile
          if h.stream:
            h.stream.close()
            h.stream = None
          h.stream = h._open()
          h.doRollover()
        break
      elif h.__class__.__name__ == 'TimedRotatingFileHandler':
        log_file = h.baseFilename
        if newFile is not None:
          h.baseFilename = newFile
          if h.stream:
            h.stream.close()
            h.stream = None
          h.stream = h._open()
          h.doRollover()
        break

    return log_file



# #accumulateSubstrings accumulates substr list in one string and returns it, also adds prefixies between
#  substrings in resulting string. substrList and prefixes must be List[str] type with equal length
# @param substrList - substrings list
# @param prefixes - prefixies list
# @returns - accumulate string
def accumulateSubstrings(substrList, prefixes):
  ret = ""
  if substrList is None or type(substrList) is not types.ListType:
    raise Exception(">>> error substrList is None or not List type")
  if prefixes is None or type(prefixes) is not types.ListType:
    raise Exception(">>> error prefixes is None or not List type")
  if len(substrList) != len(prefixes):
    raise Exception(">>> error substrList and prefixes lists have different lengths")
  i = 0
  for substr in substrList:
    if type(substr) in types.StringTypes:
      if type(prefixes[i]) in types.StringTypes:
        ret += str(prefixes[i])
      ret += str(substr)
    i += 1
  return ret



class DataReplacementConstants(object):

  CUR_YEAR_FULL = "@CUR_YEAR_FULL"
  CUR_YEAR_SHORT = "@CUR_YEAR_SHORT"
  CUR_MONTH = "@CUR_MONTH"
  CUR_DAY = "@CUR_DAY"


# #generateReplacementDict method generates and returns replacement dict with current datatime
# @returns - replacement dict with current datatime
def generateReplacementDict():
  ret = {}
  ret[DataReplacementConstants.CUR_YEAR_FULL] = datetime.now().strftime("%Y")
  ret[DataReplacementConstants.CUR_YEAR_SHORT] = datetime.now().strftime("%y")
  ret[DataReplacementConstants.CUR_MONTH] = datetime.now().strftime("%m")
  ret[DataReplacementConstants.CUR_DAY] = datetime.now().strftime("%d")
  return ret


# #parseHost parse the root host name from url
# for example: the result of http://s1.y1.example.com/path/to is example.com
# @param url the full url
# @return host of the url, eg: example.com
def parseHost(url):
  host = None
  if urlparse.urlparse(url).hostname:
    host = '.'.join(urlparse.urlparse(url).hostname.split('.')[-2:])
  return host


# # convert date str to HTTP header format
# 2014-07-29 20:31:50 (GMT+8) to Tue, 29 Jul 2014 12:31:50 GMT
# @param date_str date str, 2014-07-29 20:31:50
# @return HTTP header formated date str : Tue, 29 Jul 2014 12:31:50 GMT
def convertToHttpDateFmt(date_str):
  stamp = time.mktime(date_str.timetuple())
  # stamp = time.mktime(time.strptime(date_str, '%Y-%m-%d %H:%M:%S'))
  return time.strftime('%a, %d %b %Y %H:%M:%S GMT', time.gmtime(stamp))


# # method returns siteId, substitutes to "0" value if incoming siteId is None
def autoFillSiteId(siteId):
  ret = siteId
  if siteId is None:
    ret = "0"
  return ret


# # method strips incoming html from html comments
# @param htmlBuf incoming content in string format
# @param soup incoming content as bs object
# @return clean html buff
def stripHTMLComments(htmlBuf=None, soup=None):
  ret = htmlBuf
  if soup is not None:
    for elem in soup.findAll(text=lambda text:isinstance(text, Comment)):
      elem.extract()
  elif htmlBuf is not None:
    ret = re.sub(r"\<![ \r\n\t]*(--([^\-]|[\r\n]|-[^\-])*--[ \r\n\t]*)\>", "", htmlBuf)
  return ret


# # function extracts text from html content, strips all tags
# @param htmlText incoming raw html
# @param stripComment is strip comment or not
# @param stripScript is strip script or not
# @return inner html text
def innerHTMLText(htmlBuf, stripComment=True, stripScript=True):
  soup = BeautifulSoup(htmlBuf)

  if stripScript:
    for elem in soup.findAll(name='script'):
      elem.extract()
  if stripComment:
    stripHTMLComments(htmlBuf=None, soup=soup)

  return ''.join(soup.findAll(text=True))


# # function call splitPairs for each element in incomeDict and fills return dict
# @param incomeDict incoming dict
# @param splitters incoming splitters
# @return result dict
def getPairsDicts(incomeDict, splitters=','):
  ret = {}
  if type(incomeDict) == types.DictType:
    for key in incomeDict:
      if type(incomeDict[key]) in types.StringTypes:
        ret[key] = splitPairs(incomeDict[key], splitters)
  return ret


# # function extracts splits incoming string by splitters into dict of name=value pairs
# @param buf incoming text buf
# @param splitters incoming splitters
# @return result dict
def splitPairs(buf, splitters=','):
  ret = {}
  splitStr = buf.split(splitters)
  for elem in splitStr:
    localStr = elem.split('=')
    if type(localStr) == types.ListType and len(localStr) >= 2:
      ret[localStr[0]] = localStr[1]
  return ret


def isTailSubstr(str1, str2):
  ret = False
  if str1.find(str2) > 0 and ((len(str1) - str1.find(str2)) == len(str2)):
    ret = True
  return ret



# #Pickle Serializer
#
class PickleSerializer(object):

  CODE_DIRECTION_ENCODE = 0
  CODE_DIRECTION_DECODE = 1

  # #constructor
  #
  def __init__(self, code=True):
    super(PickleSerializer, self).__init__()

    self.encode = code


  def serialize(self, obj, code=None):
    ret = None

    if self.isEncode(code) == True:
      print "QQQ"
      ret = self.code(pickle.dumps(obj), self.CODE_DIRECTION_ENCODE)
    else:
      print "WWW"
      ret = pickle.dumps(obj)

    return ret


  def unserialize(self, buf, code=None):
    ret = None

    if self.isEncode(code) == True:
      ret = pickle.loads(self.code(buf, self.CODE_DIRECTION_DECODE))
    else:
      ret = pickle.loads(buf)

    return ret


  def code(self, buf, direction=None):
    ret = buf

    if direction == self.CODE_DIRECTION_ENCODE:
      print "haha"
      ret = unicode(buf, 'ISO-8859-1')
    else:
      if direction == self.CODE_DIRECTION_DECODE:
        ret = buf.encode("ISO-8859-1")

    return ret


  def isEncode(self, code=None):
    if code is None:
      encode = self.encode
    else:
      encode = code

    return encode

