"""
HCE project, Python bindings, Distributed Tasks Manager application.
Event objects definitions.

@package: dc
@file CrawlerTask.py
@author Oleksii <developers.hce@gmail.com>
@author madk <developers.hce@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

# import os
import os.path
import sys
import time

# import pickle
try:
  import cPickle as pickle
except ImportError:
  import pickle

# import logging
import logging.config
import ConfigParser
import robotparser
import re
import requests
from dc_crawler.Fetcher import BaseFetcher
#from dc_crawler import LangDetector
import json
import datetime
import lxml.etree
import MySQLdb as mdb
import tidylib
import CollectURLs
import CollectProperties
import base64
from dateutil.parser import *
from contextlib import closing
from cement.core import foundation
from collections import namedtuple
from time import strftime

from app.Utils import varDump
from dc.EventObjects import URL
from dc.EventObjects import Site
from dc.EventObjects import Batch
from dc.EventObjects import BatchItem
from dc_crawler.CrawledResource import CrawledResource
from app.Utils import PathMaker
from dc_crawler.CrawlerException import CrawlerException
from dc_crawler.DBTasksWrapper import DBTasksWrapper
from app.Utils import SQLExpression
import app.Utils as Utils  # pylint: disable=F0401
import dc.Constants as DC_CONSTS
import dc_crawler.Constants as CONSTS
import dc_processor.Constants as PCONSTS
import app.Consts as APP_CONSTS
import dc.EventObjects as dc_event
import URLProcess
import ResourceProcess
from urlparse import urlparse


DB_SITES = "dc_sites"
DB_URLS = "dc_urls"

MSG_ERROR_LOAD_CONFIG = "Error loading config file. Exciting. "
MSG_ERROR_LOAD_OPTIONS = "Error loading options. Exciting. "
MSG_ERROR_LOAD_LOG_CONFIG_FILE = "Can't load logging config file. Exiting. "
MSG_ERROR_LOAD_SITE_DATA = "Can't load site data: "
MSG_ERROR_LOAD_URL_DATA = "Can't load url data: "
MSG_ERROR_PROCESS_BATCH_ITEM = "Can't process batch item "
MSG_ERROR_WRITE_CRAWLED_DATA = "Can't write crawled data "
MSG_ERROR_COLLECT_URLS = "Can't collect urls "
MSG_ERROR_ADD_URL_TO_BATCH_ITEM = "Can't add url to batch item "
MSG_ERROR_LOAD_SITE_PROPERTIES = "Can't load site properties "
MSG_ERROR_CRAWL_SITE = "Can't crawl site "
MSG_ERROR_CHECK_SITE = "Site don't passed check site "
MSG_ERROR_GET_DIR = "Can't get dir "
MSG_ERROR_READ_SITE_FROM_DB = "Can't read site data from db"
MSG_ERROR_EMPTY_RESPONSE_SIZE = "Empty response"
MSG_ERROR_CONTENT_TYPE = "Invalid content-type"

MSG_INFO_PROCESS_BATCH = "ProcessBatch "
MSG_INFO_STORE_COOKIES_FILE = "Store cookies file on disk."

MSG_DEBUG_NON_PROCESSING = "ProcessorName is NONE. Exclude batch item from further processing."

SITE_MD5_EMPTY="d41d8cd98f00b204e9800998ecf8427e"

DEFAULT_MAX_SIZE = 1000000
EMPTY_RESPONSE_SIZE = "0"

APP_NAME = "crawler-task"

HTTP_COOKIE = "HTTP_COOKIE"
DEFAULT_HTTP_COOKIE = ""
HTTP_HEADERS = "HTTP_HEADERS"
DEFAULT_HTTP_HEADER = ""

DC_URLS_DB_NAME = "dc_urls"
DC_URLS_TABLE_PREFIX = "urls_"
DC_SITES_DB_NAME = "dc_sites"
DC_SITES_PROPERTIES_TABLE_NAME = "sites_properties"
DC_SITES_TABLE_NAME = "sites"
DC_URLS_TABLE_NAME = "urls"
COOKIES_FILE_POSTFIX = ".cookies.txt"

NON_PROCESSING = "NONE"

HTTP_REDIRECT = "<Response [301]>"
HTML_REDIRECT = ""
MAX_HTTP_REDIRECTS_UNLIMITED = 0
MAX_HTML_REDIRECTS_UNLIMITED = 0
META_XPATH = "//meta[contains(@content, 'url')]/@content"

Results = namedtuple("Results", "exit_code, output, err")

ROBOTS_PATTERN = re.compile(r'(https?://[^/]+).*', re.I)

TEXT_CONTENT_TYPE_PATTERN = re.compile('text', re.I)

ENV_CRAWLER_STORE_PATH = "ENV_CRAWLER_STORE_PATH"
# SiteProperties = namedtuple("http_header", "http_cookie")
# CrawlResponse = namedtuple("html_content", "html_header", "html_request")
# named tuple for filters
#Filter = namedtuple("Filter", "pattern, pattern_name, type, state")

DETECT_MIME_MAIN_CONTENT = "1"
RECOVER_IF_FAILED = "2"

EXIT_SUCCESS = 0
EXIT_FAILURE = 1

# #The CrawlerTask class, is a interface for fetching content from the web
#
# This object is a run at once application
class CrawlerTask(foundation.CementApp):


  # Configuration settings options names
  DB_HOST_CFG_NAME = "db_host"
  DB_PORT_CFG_NAME = "db_port"
  DB_USER_CFG_NAME = "db_user"
  DB_PWD_CFG_NAME = "db_pwd"
  DB_SITES_CFG_NAME = "db_dc_sites"
  DB_URLS_CFG_NAME = "db_dc_urls"

  RAW_DATA_DIR = "raw_data_dir"
  SITE_TEMPLATES = "dc_site_template"
  KEY_VALUE_STORAGE_DIR = "key_value_storage_dir"
  DB_DATA_DIR = "db_data_dir"


  # Mandatory
  class Meta(object):
    label = APP_NAME
    def __init__(self):
      #self.site_table_row = None
      self.url_xpath_list = []


  # #constructor
  # initialize default fields
  def __init__(self):
    # call base class __init__ method
    foundation.CementApp.__init__(self)
    self.dc_urls_db_connect = None
    self.need_store_mime = None
    self.site = None
    self.raw_data_dir = None
    self.robotparser = None
    self.batchItem = None
    self.dc_sites_db_connect = None
    self.siteTable = None
    self.site_connector = None
    self.headersDict = None
    self.store_http_headers = None
    self.cookie = None
    self.urlConnector = None
    self.proxies = None
    self.logger = None
    self.urlTable = None
    self.realUrl = None
    self.url = None
    self.crawledResource = None
    self.batch = None
    self.headers = None
    self.crawledTime = None
    self.store_http_request = None
    self.dir = None
    self.default_header_file = None
    self.default_cookie_file = None
    self.kvConnector = None
    self.kvCursor = None
    self.processorName = None
    self.site_properties = None
    self.auto_remove_props = None
    self.store_cookies = None
    self.allow_http_redirects = True
    self.allow_html_redirects = True
    self.http_redirects = 0
    self.html_redirects = 0
    self.max_http_redirects = 1
    self.max_html_redirects = None
    self.dom = None
    self.time_postfix = ""
    self.res = None
    self.postForms = None
    self.authName = None
    self.authPwd = None
    self.kvDbDir = None
    self.defaultIcrCrawlTime = None
    self.wrapper = None
    self.store_items = []
    self.url_xpath_list = []
    self.errorMask = CONSTS.ERROR_OK
    self.feed = None
    self.feed_items = None
    self.max_fetch_time = CONSTS.FETCHER_TIME_LIMIT_MAX
    self.collect_additional_prop = False
    self.exit_code = EXIT_SUCCESS
    self.keep_old_resources = False
    self.urlProcess = None
    self.resourceProcess = None
    self.urlTempalteRegular = None
    self.urlTempalteRealtime = None
    self.urlTempalteRegularEncode = None
    self.urlTempalteRealtimeEncode = None
    self.tidyOptions = {'numeric-entities':1, 'char-encoding': 'utf8'}


  # #setup
  # setup application
  def setup(self):
    # call base class setup method
    foundation.CementApp.setup(self)
    self.args.add_argument('-c', '--config', action='store', metavar='config_file', help='config ini-file')


  # #run
  # run application
  def run(self):
    # call base class run method
    foundation.CementApp.run(self)

    # config section
    self.loadConfig()

    # load logger config file
    self.loadLogConfigFile()

    # load mandatory options
    self.loadOptions()

    # load db backend
    self.loadDBBackend()

    # load key-value db
    self.loadKeyValueDB()


  # #RLs collect URL from response body
  #
  # @return is collect successfully
  # bullshit, move it to the separate module (like URLCollector)
  def collectURLs(self):
    collectURLs = CollectURLs.CollectURLs()
    collectURLs.crawledResource = self.crawledResource
    collectURLs.url = self.url
    collectURLs.dom = self.dom
    collectURLs.realUrl = self.realUrl
    collectURLs.processorName = self.processorName
    collectURLs.batchItem = self.batchItem
    collectURLs.urlXpathList = self.url_xpath_list
    collectURLs.siteProperties = self.site_properties
    collectURLs.site = self.site
    collectURLs.dbWrapper = self.wrapper
    collectURLs.autoRemoveProps = self.auto_remove_props
    collectURLs.autoDetectMime = self.auto_detect_mime
    collectURLs.processContentTypes = self.process_content_types
    collectURLs.postForms = self.postForms
    collectURLs.urlProcess = self.urlProcess

    collectURLsResult, internalLinks, externalLinks = collectURLs.process()
    if collectURLsResult and self.collect_additional_prop:
      collectPropertiesObj = CollectProperties.CollectProperties()
      collectPropertiesObj.siteId = Utils.autoFillSiteId(self.batchItem.siteId)
      collectPropertiesObj.kvDbDir = self.kvDbDir
      collectPropertiesObj.res = self.res
      collectPropertiesObj.batchItem = self.batchItem
      collectPropertiesObj.realUrl = self.realUrl
      collectPropertiesObj.process(self.dom, internalLinks, externalLinks)

    return collectURLsResult


  # #writeData  write the response body and headers to file
  #
  # @param
  def writeData(self):
    # check wether need store to disk
    if self.need_store_mime and self.need_store_mime != '*':
      if self.crawledResource.content_type.lower() not in self.need_store_mime:
        self.logger.info("do not store resource files to disk")
        return
        # self.logger.info("do not store resource files to disk")
        # self.updateURLForFailed(CONSTS.ERROR_MIME_NOT_WRITE)
        # raise CrawlerException("MIME Content-Type not allow store content to disk")

    #self.logger.info("store resource files to disk")
    #self.logger.debug("urlMd5: %s" % str(self.batchItem.urlId))
    #self.makeDir()
    # html content
    #base_path = os.path.join(self.dir, self.batchItem.urlId + "_" + self.time_postfix)

    # save raw file
    #html_content_file_name = base_path + ".bin"
    #self.logger.debug("self.crawledResource: %s" % varDump(self.crawledResource))
    #with open(html_content_file_name, "wb") as f:

    urlPut_list = []

    if TEXT_CONTENT_TYPE_PATTERN.match(self.crawledResource.content_type):
      # save UTF-8 encoding text for text content types
      if self.crawledResource.dynamic_fetcher_type:
        raw_unicode_content = self.crawledResource.meta_content
      else:
        raw_unicode_content = self.crawledResource.html_content
      if raw_unicode_content:
        putDict = {"data": base64.b64encode(raw_unicode_content)}
        content_type = dc_event.Content.CONTENT_RAW_CONTENT
        urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, content_type, putDict))
        #f.write(raw_unicode_content)

        # save tidy recovered file
        #self.dom = self.resourceProcess.domParser(self.htmlRecover, res.rendered_unicode_content, resource.http_code)
        localDom = None
        if self.htmlRecover is not None and self.htmlRecover == "2":
          localDom = self.resourceProcess.domParser(None, raw_unicode_content, 200)
        if self.htmlRecover is not None and (self.htmlRecover == "1" or self.htmlRecover == "2" and localDom is None):
          #with open(base_path + ".tidy", "wb") as tidy_f:
          tidy_content, errors = tidylib.tidy_document(raw_unicode_content, self.tidyOptions)
          putDict = {"data": base64.b64encode(tidy_content)}
          content_type = dc_event.Content.CONTENT_TIDY_CONTENT
          urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, content_type, putDict))
          #tidy_f.write(tidy_content)
    else:
      # save origin binary data for non-text content types
      if self.crawledResource.binary_content:
        putDict = {"data": base64.b64encode(self.crawledResource.binary_content)}
        content_type = dc_event.Content.CONTENT_RAW_CONTENT
        urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, content_type, putDict))
        #f.write(self.crawledResource.binary_content)

    #detect mime type
    detected_mime = None

    # save rendered file
    if self.crawledResource.dynamic_fetcher_type and self.crawledResource.html_content:
      putDict = {"data": base64.b64encode(self.crawledResource.html_content)}
      content_type = dc_event.Content.CONTENT_DYNAMIC_CONTENT
      urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, content_type, putDict))
      #rendered_file_name = base_path + "." + self.crawledResource.dynamic_fetcher_type + ".dyn"
      #with open(rendered_file_name, "wb") as f:
        #f.write(self.crawledResource.html_content)

    # html header
    if self.store_http_headers and self.crawledResource.response_header:
      putDict = {"data": base64.b64encode(self.crawledResource.response_header)}
      content_type = dc_event.Content.CONTENT_HEADERS_CONTENT
      urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, content_type, putDict))
      #html_header_file_name = base_path + ".headers.txt"
      #with open(html_header_file_name, "wb") as f:
        #self.logger.warn("file is %s", html_header_file_name)
        #if self.crawledResource.response_header:
          #f.write(self.crawledResource.response_header)

    # html request
    if self.store_http_request and self.crawledResource.html_request:
      putDict = {"data": base64.b64encode(self.crawledResource.html_request)}
      content_type = dc_event.Content.CONTENT_REQUESTS_CONTENT
      urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, content_type, putDict))
      #html_request_file_name = base_path + ".request.txt"
      #with open(html_request_file_name, "wb") as f:
        #f.write("GET %s HTTP/1.0\r\n\r\n" % (self.realUrl,))
        #if self.crawledResource.html_request:
          #f.write(self.crawledResource.html_request)

    # Write raw content on disk via db-task
    # Check if Real-Time crawling
    self.wrapper.putURLContent(urlPut_list)
    
    #change url's contentMask
    self.url.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
    self.batchItem.urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
    return detected_mime


  # #getDir prepare dir
  #
  # @param
  def getDir(self):
    if len(self.batchItem.siteId):
      self.dir = os.path.join(self.raw_data_dir, self.batchItem.siteId, PathMaker(self.batchItem.urlId).getDir())
    else:
      self.dir = os.path.join(self.raw_data_dir, "0", PathMaker(self.batchItem.urlId).getDir())


  # #getDir prepare dir
  #
  # @param
  def makeDir(self):
    if not os.path.exists(self.dir):
      os.makedirs(self.dir)
    elif not os.path.isdir(self.dir):
      self.updateURLForFailed(CONSTS.ERROR_WRITE_FILE_ERROR)
      raise Exception("path %s exists, but is not a directory" % (self.dir,))


  # #updateURLForFailed
  #
  # @param error_bit BitMask of error
  def updateURLForFailed(self, errorBit, httpCode=None):
    self.urlProcess.siteId = self.batchItem.siteId
    self.urlProcess.updateURLForFailed(errorBit, self.batchItem, httpCode)


  # #crawl site
  #
  # @return should continue to write data and collect URLs
  def crawl(self):
    self.crawledResource = None
    # delay
    time.sleep(self.url.requestDelay / 1000.0)
    #(1) Decode url - [UrlProcess]
    # TODO move to - [UrlProcess]
    self.urlProcess.url = self.url.url
    #(1) END
    url = self.urlProcess.getRealUrl()
    self.realUrl = url
    self.logger.info("allowed in robots.txt")
    startTime = time.time()

    try:
      self.logger.info("start to fetch %s", url)
      #(2) Makes auth tuple - []
      # TODO move to - []
      if self.authName and self.authPwd:
        auth = (self.authName, self.authPwd)
        self.logger.info("using http basic auth %s:%s", self.authName, self.authPwd)
      else:
        auth = None
      #(2) END
      #(3) Resolve HTTP Method (move convertToHttpDateFmt to Utils) - [UrlProcess]
      # TODO move to - [UrlProcess]
      self.urlProcess.urlObj = self.url
      postData = self.urlProcess.resolveHTTP(self.postForms, self.headersDict)
      #(3) END
    
      # Workaround
      '''
      if self.site.fetchType == BaseFetcher.TYP_DYNAMIC:
        self.site.fetchType = BaseFetcher.TYP_NORMAL
        url_ = "http://demo.dc4.hce-project.com/TagsReaperUI/content?url="
        url_ = url_ + url
        url = url_ + "&contentOnly=true"
        self.logger.info("URL: %s" % url)
      '''
      url = self.urlProcess.urlTemplateApply(url, self.batch.crawlerType, self.urlTempalteRegular,
                                             self.urlTempalteRealtime, self.urlTempalteRegularEncode,
                                             self.urlTempalteRealtimeEncode)
      res = BaseFetcher.get_fetcher(self.site.fetchType).open(url,
        external_url=self.external_url,
        timeout=int(self.url.httpTimeout) / 1000.0,
        headers=self.headersDict,
        allow_redirects=self.allow_http_redirects,
        proxies=self.proxies, auth=auth,
        data=postData, logger=self.logger,
        allowed_content_types=self.process_content_types,
        max_resource_size=self.site.maxResourceSize,
        max_redirects=self.max_http_redirects)

      #(4) Checks and update url in db-task them - [UrlProcess]
      # TODO move to - [UrlProcess]
      if not res.headers:
        self.updateURLForFailed(res.error_mask)
        return False

      if "content-length" in res.headers and res.headers["content-length"] == EMPTY_RESPONSE_SIZE:
        self.logger.debug("content-length: %s", str(res.headers["content-length"]))
        self.logger.debug(MSG_ERROR_EMPTY_RESPONSE_SIZE)
        self.updateURLForFailed(CONSTS.ERROR_EMPTY_RESPONSE)
        return False
      #(4) END

      # save res to self, will use it in next steps
      #(5) Create and fill resource struct - [ResourceProcess]
      # TODO move to - [ResourceProcess]
      self.res = res
      self.logger.info("response code:%s", str(self.res.status_code))
      # use charset to improve encoding detect
      self.crawledTime = time.time()
      self.resourceProcess.urlObj = self.url
      resource = self.resourceProcess.generateResource(startTime, res, self.headers, self.crawledTime, 
                                                       self.defaultIcrCrawlTime)
      # updated by Oleksii
      # save cookies from response
      resource.cookies = res.cookies
      resource.dynamic_fetcher_type = res.dynamic_fetcher_type
      self.crawledResource = resource
      #(5) END

      if "content-type" in res.headers and (res.headers["content-type"] == "image/x-icon" or \
          res.headers["content-type"] == "application/pdf"):
        self.updateSite(0)
        return True
      # Added by Oleksii
      # Build DOM object
      #(6) DOM processing [???]
      self.resourceProcess.resource = resource
      self.dom = self.resourceProcess.domParser(self.htmlRecover, res.rendered_unicode_content, resource.http_code)
      #(6) END
      # Added by Oleksii
      # support of HTTP_REDIRECTS_MAX
      #(7) Check response for redirect count [???]
      self.checkResponse()
      if not self.allow_html_redirects:
        self.updateSite(CONSTS.ERROR_MAX_ALLOW_HTML_REDIRECTS)
        return False
      if not self.allow_http_redirects:
        self.updateSite(CONSTS.ERROR_MAX_ALLOW_HTTP_REDIRECTS)
        return False
      #(7) END

      #(8) Check Content size and response code - [ResourceProcess]
      # TODO move to - [ResourceProcess]
      self.resourceProcess.resource = resource
      self.resourceProcess.batchItem = self.batchItem
      if not self.resourceProcess.checkResourcesResponse(res, self.site.maxResourceSize, self.updateSite):
        return False
      #(8) END

    except requests.exceptions.Timeout, err:
      self.updateURLForFailed(CONSTS.ERROR_REQUEST_TIMEOUT)
      # tbi = Utils.getTracebackInfo()
      # self.logger.error(str(type(err)) + "\n")
      # self.logger.error(str(err.message) + "\n" + tbi)
      # self.logger.debug("Can't crawl resource.")
      return False
    except requests.exceptions.InvalidURL:
      self.updateURLForFailed(CONSTS.ERROR_BAD_URL)
      return False
    except requests.exceptions.TooManyRedirects:
      self.updateURLForFailed(CONSTS.ERROR_BAD_REDIRECTION)
      return False
    except requests.exceptions.ChunkedEncodingError:
      self.updateURLForFailed(CONSTS.ERROR_PAGE_CONVERT_ERROR)
      return False
    except requests.exceptions.ConnectionError:
      self.updateURLForFailed(CONSTS.ERROR_CONNECTION_ERROR)
      return False
    except requests.exceptions.ContentDecodingError:
      self.updateURLForFailed(CONSTS.ERROR_PAGE_CONVERT_ERROR)
      return False
    except lxml.etree.XMLSyntaxError:
      self.logger.warning("XML HTML syntax error")
      self.updateURLForFailed(CONSTS.ERROR_DTD_INVALID)
      return False
    except Exception, err:
      self.updateURLForFailed(CONSTS.ERROR_HTML_STRUCTURE)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.logger.error(str(type(err)) + "\n")
      self.logger.error(str(err.message) + "\n" + tbi)
      self.logger.debug("Can't crawl resource." + err.message)
      return False
    self.updateSite(CONSTS.ERROR_OK)
    return True


  # #parseRobotFile fetch the robot file for spefic url
  #
  # @param url the url to fetch(not the url of robots.txt)
  def parseRobotFile(self, url):
    robots_txt_url = ROBOTS_PATTERN.sub(r'\1/robots.txt', url)
    self.logger.info("robots_txt_url: %s" % robots_txt_url)
    self.robotparser = robotparser.RobotFileParser(robots_txt_url)
    self.robotparser.read()
    if self.robotparser.can_fetch("*", urlparse(url).path):
      self.logger.info("%s ALLOWED in robots.txt" % url)
    else:
      self.logger.info("%s DISALLOWED in robots.txt" % url)
    return True#is_allowed


  # updateAVGSpeed update sites.AVGSpeed property
  def updateAVGSpeed(self):
    if self.res.status_code == 304:
      # not modified will return empty response body
      return
    newSpeed = (self.site.avgSpeed * self.site.avgSpeedCounter + self.crawledResource.bps) / \
               (self.site.avgSpeedCounter + 1)
    localSiteUpdate = dc_event.SiteUpdate(self.batchItem.siteId)
    for attr in localSiteUpdate.__dict__:
      if hasattr(localSiteUpdate, attr):
        setattr(localSiteUpdate, attr, None)
    localSiteUpdate.id = self.batchItem.siteId
    localSiteUpdate.tcDate = SQLExpression("NOW()")
    localSiteUpdate.avgSpeed = newSpeed
    localSiteUpdate.avgSpeedCounter = SQLExpression("`AVGSpeedCounter` + 1")
    self.wrapper.siteNewOrUpdate(localSiteUpdate)


  # saveCookies
  def saveCookies(self):
    self.logger.debug(MSG_INFO_STORE_COOKIES_FILE)
    self.logger.debug("Store cookies properties:\n%s", varDump(self.store_cookies))
    if self.store_cookies and self.store_cookies == "1":
      cookies_str = [key + ": " + value + "; " for (key, value) in self.crawledResource.cookies.items()]
      self.makeDir()
      self.logger.debug("Response cookies dict: %s", str(self.crawledResource.cookies))
      self.logger.debug("Response cookies string: %s", str(cookies_str))
      self.logger.debug("self.batchItem.urlId: %s", str(self.batchItem.urlId))
      self.logger.debug("self.time_postfix: %s", str(self.time_postfix))
      base_path = os.path.join(self.dir, self.batchItem.urlId + "_" + self.time_postfix)
      cookies_file_name = base_path + COOKIES_FILE_POSTFIX
      with open(cookies_file_name, "wb") as f:
        f.write(''.join(sorted(cookies_str)))


  # #prepare request headers and cookie
  #
  # @param site - site object
  # TODO move to - [SiteProcess]
  def readSiteProperties(self):
    # TODO not sure is it right that fetch headers and cookie
    # by `Name` = 'headers'/'cookie'
    self.store_http_request = True
    self.store_http_headers = True
    self.headersDict = {}
    self.postForms = {}
    self.headers, self.cookie = None, None
    self.proxies = None
    self.need_store_mime = '*'
    self.authName, self.authPwd = None, None
    self.external_url = None
    self.auto_remove_props = {}
    self.htmlRecover = None
    self.auto_detect_mime = None
    self.process_content_types = []
    headers, cookie = None, None
    proxy_host, proxy_port = None, None

    try:
      #stub
      self.logger.info("self.site.properties: %s", varDump(self.site.properties))

      # Update site properties from batch item properties
      keys = [property["name"] for property in self.site.properties]
      self.logger.info("keys: %s" % str(keys))
      for key in self.batchItem.properties.keys():
        if key in keys:
          for property in self.site.properties:
            if property["name"] == key:
              self.logger.info("%s exist in site properties. Rewrite property" % key)
              self.logger.info("Old value: %s" % varDump(property["value"]))
              property["value"] = self.batchItem.properties[key]
              self.logger.info("New value: %s" % varDump(property["value"]))
        else:
          self.logger.info("%s don't exist in site properties. Add property" % key)
          self.site.properties.append({"name":key, "value":self.batchItem.properties[key], 
                                       "URLMd5":self.batchItem.urlId})
      self.logger.debug("Updated site's properties: " + str(self.site.properties))

      self.site_properties = {}
      for item in self.site.properties:
        self.site_properties[item["name"]] = item["value"]

      if 'HTTP_HEADERS' in self.site_properties:
        headers = self.site_properties['HTTP_HEADERS']
      if 'HTTP_COOKIE' in self.site_properties:
        cookie = self.site_properties['HTTP_COOKIE']
      if 'STORE_HTTP_REQUEST' in self.site_properties:
        self.store_http_request = int(self.site_properties['STORE_HTTP_REQUEST'])!=0
      if 'STORE_HTTP_HEADERS' in self.site_properties:
        self.store_http_headers = int(self.site_properties['STORE_HTTP_HEADERS'])!=0
      if 'HTTP_PROXY_HOST' in self.site_properties:
        proxy_host = self.site_properties['HTTP_PROXY_HOST']
      if 'HTTP_PROXY_PORT' in self.site_properties:
        proxy_port = self.site_properties['HTTP_PROXY_PORT']
      if 'MIME_TYPE_STORE_ON_DISK' in self.site_properties:
        allow_mimes = self.site_properties['MIME_TYPE_STORE_ON_DISK']
        if allow_mimes == '' or allow_mimes == '*':
          self.need_store_mime = '*'
        else:
          self.need_store_mime = set([mime.lower() for mime in allow_mimes.split(',')])
      if 'HTTP_AUTH_NAME' in self.site_properties:
        self.authName = self.site_properties['HTTP_AUTH_NAME']
      if 'HTTP_AUTH_PWD' in self.site_properties:
        self.authPwd = self.site_properties['HTTP_AUTH_PWD']
      for key in self.site_properties.keys():
        if key.startswith('HTTP_POST_FORM_'):
          self.postForms[key[15:]] = self.site_properties[key]
      if 'EXTERNAL_FETCHER_URL' in self.site_properties:
        self.external_url = self.site_properties['EXTERNAL_FETCHER_URL']
      if 'HTML_RECOVER' in self.site_properties:
        self.htmlRecover = self.site_properties['HTML_RECOVER']
      if 'MIME_TYPE_AUTO_DETECT' in self.site_properties:
        self.auto_detect_mime = self.site_properties['MIME_TYPE_AUTO_DETECT']
      if 'ALLOWED_CTYPES' in self.site_properties:
        self.process_content_types = self.site_properties['ALLOWED_CTYPES'].lower().split(',')
      if 'PROCESSOR_NAME' in self.site_properties:
        self.processorName = self.site_properties['PROCESSOR_NAME']
      if DC_CONSTS.SITE_PROP_SAVE_COOKIES in self.site_properties:
        self.store_cookies = self.site_properties[DC_CONSTS.SITE_PROP_SAVE_COOKIES]
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES in self.site_properties:
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES] = \
        self.site_properties[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES]
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER in self.site_properties:
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER] = \
        self.site_properties[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER]
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE in self.site_properties:
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE] = \
        self.site_properties[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE]
      if 'HTTP_REDIRECTS_MAX' in self.site_properties:
        self.max_http_redirects = int(self.site_properties['HTTP_REDIRECTS_MAX'])
      if 'HTML_REDIRECTS_MAX' in self.site_properties:
        self.max_html_redirects = int(self.site_properties['HTML_REDIRECTS_MAX'])
      if 'URL_XPATH_LIST' in self.site_properties:
        self.url_xpath_list = json.loads(self.site_properties['URL_XPATH_LIST'])['xpath_list']
      if 'URL_TEMPLATE_REGULAR' in self.site_properties:
        self.urlTempalteRegular = self.site_properties['URL_TEMPLATE_REGULAR']
      if 'URL_TEMPLATE_REALTIME' in self.site_properties:
        self.urlTempalteRealtime = self.site_properties['URL_TEMPLATE_REALTIME']
      if 'URL_TEMPLATE_REGULAR_ENCODE' in self.site_properties:
        self.urlTempalteRegularEncode = self.site_properties['URL_TEMPLATE_REGULAR_ENCODE']
      if 'URL_TEMPLATE_REALTIME_ENCODE' in self.site_properties:
        self.urlTempalteRealtimeEncode = self.site_properties['URL_TEMPLATE_REALTIME_ENCODE']
      if headers is None:
        # read request headers from crawler-task_headers.txt file
        headers = CrawlerTask.readSmallFileContent(self.default_header_file)
      if cookie is None:
        # read request cookies from crawler-task_headers.txt file
        cookie = CrawlerTask.readSmallFileContent(self.default_cookie_file)
      self.headers = headers
      if cookie and cookie.lower().startswith('cookie:'):
        self.cookie = cookie[len('cookie:'):]
      else:
        self.cookie = cookie

      for header in headers.splitlines():
        if not header:
          continue
        try:
          key, value = header[:header.index(':')].strip(), header[header.index(':') + len(':'):].strip()
        except Exception:
          self.logger.warn("header:%s", header)
        self.headersDict[key] = value
      if self.cookie:
        self.headersDict['Cookie'] = self.cookie

      if proxy_host and proxy_port:
        self.proxies = {'http' : 'http://%s:%s' % (proxy_host, proxy_port)}
      self.logger.info("proxies: %s", self.proxies)
    except Exception:
      self.logger.error(MSG_ERROR_LOAD_SITE_PROPERTIES, exc_info=True)


  # # readSmallFileContent read small file content
  #
  # @param path the file path to read
  @staticmethod
  def readSmallFileContent(path):
    with open(path, 'r') as f:
      return ''.join(f.readlines())


  # #update site params
  # update site params
  # @param mask ErrorMask
  # @param is_suspend should set state to suspend for this site
  def updateSite(self, mask, is_suspend=False):
    response = False
    try:
      localSiteUpdate = dc_event.SiteUpdate(self.batchItem.siteId)
      for attr in localSiteUpdate.__dict__:
        if hasattr(localSiteUpdate, attr):
          setattr(localSiteUpdate, attr, None)
      localSiteUpdate.id = self.batchItem.siteId
      #TODO: possible redundant for the URLFetch algorithm and need to be removed
      localSiteUpdate.tcDate = SQLExpression("NOW()")
      if mask:
        localSiteUpdate.errors = SQLExpression("`Errors` + 1")
        localSiteUpdate.errorMask = SQLExpression(("`ErrorMask` | %s" % mask))
        if is_suspend:
          localSiteUpdate.state = Site.STATE_SUSPENDED
      updated_count = self.wrapper.siteNewOrUpdate(localSiteUpdate)
      if updated_count > 0:
        response = True
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise
    return response


  # #check site params
  # check site params
  #
  # @return should continue execute this BatchItem
  def checkSite(self):
    # TODO: self.site and self.url not must be none
    if not self.site or not self.url:
      self.logger.error("Error: wrong condition,\nself.site:%s\nself.url:%s", varDump(self.site), varDump(self.url))
      return False
    self.logger.debug("url:" + str(self.url))

    if ((self.batch.crawlerType!=dc_event.Batch.TYPE_REAL_TIME_CRAWLER) and (self.site.state!=Site.STATE_ACTIVE)) or \
      ((self.batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER) and (self.site.state==Site.STATE_DISABLED)):
      self.logger.debug("Warning: Batch CrawlerType: %s, site state is %s but not STATE_ACTIVE!"
                        % (str(self.batch.crawlerType), str(self.site.state)))
      return False

    # if not root url
    #if self.url["ParentMd5"] != "":
    if self.url.parentMd5 != "":
      # url wasn't inserted before
      #if self.url["Crawled"] == 0:
      if self.url.crawled == 0:
        # if site not recrawl
        if self.site.recrawlPeriod == 0:
          if self.site.maxErrors and self.site.errors > self.site.maxErrors:
            self.logger.debug("Errors=" + str(self.site.errors) + " limit is reached maxErrors=" +
                              str(self.site.maxErrors))
            # TODO: possible improve suspend logic
            # self.updateSite(CONSTS.ERROR_SITE_MAX_ERRORS, True)
            self.updateSite(CONSTS.ERROR_SITE_MAX_ERRORS)
            return False
          elif self.site.maxURLs and self.site.resources > self.site.maxURLs:
            self.logger.debug("Resources=" + str (self.site.resources) + " limit is reached maxURLs=" +
                              str(self.site.maxURLs))
            # TODO: possible improve suspend logic
            # self.updateSite(CONSTS.ERROR_SITE_MAX_RESOURCES, True)
            self.updateSite(CONSTS.ERROR_SITE_MAX_RESOURCES)
            return False
    
    self.updateSite(0)
    return True


  # #load url
  # the site object to crawl
  # @param site - object to crawl
  # TODO move to - [SiteProcess]
  def loadSite(self):
    try:
      if not len(self.batchItem.siteId):
        self.batchItem.siteId = "0"
      self.readSiteFromDB()
      # Check if site not exist then read site for siteId=0
      if self.site.id == SITE_MD5_EMPTY:
        self.logger.debug("Site not found. Take site id 0")
        self.batchItem.siteId = "0"
        self.batchItem.urlObj.siteId = "0"
        self.readSiteFromDB()
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise


  # #load url
  # the site object to crawl
  # @param site - object to crawl
  def readSiteFromDB(self):
    siteStatus = dc_event.SiteStatus(self.batchItem.siteId)
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.SITE_STATUS, siteStatus)
    responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
    self.site = None
    self.siteTable = DC_SITES_TABLE_NAME
    try:
      self.site = responseDRCESyncTasksCover.eventObject
    except Exception as err:
      self.logger.error(MSG_ERROR_READ_SITE_FROM_DB + err.message)
      raise
    #self.logger.info("site object: " + varDump(self.site))


  # #load url
  # the site object to crawl
  # @param site - object to crawl
  def loadURL(self):
    # do siteId
    self.urlConnector = self.dc_urls_db_connect
    if len(self.batchItem.siteId):
      self.urlTable = DC_URLS_TABLE_PREFIX + self.batchItem.siteId
    else:
      self.urlTable = DC_URLS_TABLE_PREFIX + "0"
    self.logger.info("db backend Table for siteId %s is: %s" % (self.batchItem.siteId, self.urlTable))
    self.urlProcess.siteId = self.batchItem.siteId
    self.urlProcess.urlTable = self.urlTable
    self.url = self.urlProcess.readUrlFromDB(self.batchItem, self.batch.crawlerType, self.site.recrawlPeriod,
                                             self.auto_remove_props)
    if self.urlProcess.isUpdateCollection:
      self.updateCollectedURLs()


  # #resetVars
  def resetVars(self):
    self.need_store_mime = None
    self.site = None
    self.robotparser = None
    self.headersDict = None
    self.store_http_headers = None
    self.cookie = None
    self.proxies = None
    self.realUrl = None
    self.url = None
    self.crawledResource = None
    self.headers = None
    self.crawledTime = None
    self.store_http_request = None
    self.dir = None
    self.kvConnector = None
    self.kvCursor = None
    self.processorName = None
    self.site_properties = None
    self.auto_remove_props = None
    self.store_cookies = None
    self.allow_http_redirects = True
    self.allow_html_redirects = True
    self.http_redirects = 0
    self.html_redirects = 0
    self.max_http_redirects = CONSTS.MAX_HTTP_REDIRECTS_LIMIT
    self.max_html_redirects = None
    self.dom = None
    self.time_postfix = ""
    self.res = None
    self.postForms = None
    self.authName = None
    self.authPwd = None
    #self.site_table_row = None
    self.site_properties = {}
    self.url_xpath_list = []
    self.errorMask = CONSTS.ERROR_OK
    self.feed = None
    self.feed_items = None
    self.urlTempalteRegular = None
    self.urlTempalteRealtime = None
    self.urlTempalteRegularEncode = None
    self.urlTempalteRealtimeEncode = None


  # #process batch item
  # the batch item is the one from list of batch items within batch object
  # @param item - the batch item is the one from list of batch items within batch object
  def processBatchItem(self):
    self.resetVars()
    self.logger.info("Batch item: siteId: %s, urlId: %s" % (self.batchItem.siteId, self.batchItem.urlId))
    response = self.batchItem
    try:
      # TODO move to - [SiteProcess]
      self.logger.info("BatchItem: %s" % varDump(self.batchItem))
      self.logger.info("self.batch.crawlerType: %s" % str(self.batch.crawlerType))
      self.loadSite()
      self.logger.info("Batch: %s, URLMD5: %s, SITEID: %s, ULR: %s" % (self.batch.id, self.batchItem.urlObj.urlMd5,
                                          Utils.autoFillSiteId(self.batchItem.siteId), self.batchItem.urlObj.url))

      # prepare headers and cookie
      # TODO move to - [SiteProcess]
      self.readSiteProperties()
      # TODO move to - [UrlProcess]
      #Check if Real-Time crawling
      if self.batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER and self.batchItem.urlObj.urlPut is not None:
        self.url = self.batchItem.urlObj
        self.batchItem.urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
        # save tidy recovered file
        if "data" in self.batchItem.urlObj.urlPut.putDict:
          raw_unicode_content = self.batchItem.urlObj.urlPut.putDict["data"]
          localDom = None
          if self.htmlRecover is not None and self.htmlRecover == "2":
            localDom = self.resourceProcess.domParser(None, raw_unicode_content, 200)
          if self.htmlRecover is not None and (self.htmlRecover == "1" or self.htmlRecover == "2" and localDom is None):
            tidy_content, errors = tidylib.tidy_document(raw_unicode_content, self.tidyOptions)
            self.batchItem.urlObj.urlPut.putDict["data"] = base64.b64encode(tidy_content)
            self.logger.info(">>> Call tidy for ONLY_PROCESSING algorithm.")
        self.logger.info(">>> ONLY_PROCESSING algorithm. Skip batch item.")
        return self.batchItem

      elif self.batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER and \
        "DB_TASK_MODE" in self.batchItem.properties and \
        self.batchItem.properties["DB_TASK_MODE"] == "RO":
        self.url = self.batchItem.urlObj
        self.wrapper.affect_db = False
      else:
        self.loadURL()

      # TODO move to - [SiteProcess]
      if not self.checkSite():
        self.logger.debug("Site check was failed. ")
        self.batchItem = None
        return self.batchItem

      if not self.url:
        self.logger.debug("Can't load url. ")
        self.batchItem = None
        return self.batchItem

      #Check if Real-Time crawling resource already crawled
      if self.batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER and \
        self.url.crawled and \
        self.url.errorMask==CONSTS.ERROR_OK and \
        self.url.tagsCount>0 and \
        self.errorMask==CONSTS.ERROR_OK:
        self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty")
        self.logger.debug("Batch item properties: %s" % varDump(self.batchItem.properties))
        if PCONSTS.RECRAWL_KEY not in self.batchItem.properties or \
          self.batchItem.properties[PCONSTS.RECRAWL_KEY]==PCONSTS.RECRAWL_VALUE_NO:
          self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty. "+
                            "Don't need to recrawl.")
          # set contentMask stored on disk
          self.batchItem.urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
          return self.batchItem
        else:
          self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty but "+
                            "properties recrawl is <%s>. Send to recrawling." % \
                            str(self.batchItem.properties[PCONSTS.RECRAWL_KEY]))

      
      # reset error mask for current crawled url
      self.urlProcess.siteId = self.batchItem.siteId
      self.urlProcess.resetErrorMask(self.batchItem)

      # update state to crawling
      # TODO move to - [UrlProcess]
      self.urlProcess.urlObj = self.url
      self.urlProcess.siteId = self.batchItem.siteId
      self.urlProcess.updateURL(self.batchItem, self.batch.id)

      # crawl
      crawl_result = None
      if self.parseRobotFile(self.url.url):
        crawl_result = self.crawl()
      if self.crawledResource:
        # TODO move to - [UrlProcess]
        self.urlProcess.urlObj = self.url
        self.urlProcess.siteId = self.batchItem.siteId
        self.urlProcess.updateCrawledURL(self.crawledResource, self.batchItem, self.res.content_size)
        # TODO move to - [SiteProcess]
        self.updateAVGSpeed()
      else:
        self.logger.debug(">>> Wrong !!!! self.crawledResourc not set")

      if not crawl_result:
        self.logger.debug("Crawl results was failed. ")
        self.batchItem = None
        return self.batchItem

      # not save data to disk/collect URL/collect properties if not modified
      #(1) Processing content if resource not modified
      if self.crawledResource.http_code != 304:
        detected_mime = None
        self.getDir()
        try:
          # Keep old resources on disk
          if self.keep_old_resources:
            self.time_postfix = datetime.datetime.now().strftime("%Y%m%d%H%M%S")

          detected_mime = self.writeData()
          self.saveCookies()
        except CrawlerException, cerr:
          self.logger.debug(cerr.message)
          raise
        except Exception as err:
          self.updateURLForFailed(CONSTS.ERROR_WRITE_FILE_ERROR)
          self.logger.error(MSG_ERROR_WRITE_CRAWLED_DATA + err.message, exc_info=True)
        self.collectURLs()
        # TODO move to - [UrlProcess]
        self.urlProcess.siteId = self.batchItem.siteId
        self.urlProcess.updateCollectTimeAndMime(detected_mime, self.batchItem, self.crawledTime, self.auto_detect_mime)
      else:
        self.updateURLForFailed(CONSTS.ERROR_NOT_MODIFY, 304)
      #(1) END
      #self.addResource()
    except CrawlerException as err:
      self.logger.debug(MSG_ERROR_PROCESS_BATCH_ITEM)
    except Exception as err:
      self.exit_code = EXIT_FAILURE
      self.logger.error(MSG_ERROR_PROCESS_BATCH_ITEM)
      self.logger.error("xxx", exc_info=True)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)

    # Added by Oleksii
    # support PROCESSOR_NAME
    if self.processorName == NON_PROCESSING:
      self.logger.debug(MSG_DEBUG_NON_PROCESSING)
      self.batchItem = None

    self.logger.debug("Batch item:\n%s", varDump(response))
    return self.batchItem


  # #process batch
  # the main processing of the batch object
  def processBatch(self):
    try:
      # read pickled batch object from stdin and unpickle it
      input_pickled_object = sys.stdin.read()
      input_batch = pickle.loads(input_pickled_object)
      #self.logger.info("input_batch: " + varDump(input_batch))
      self.batch = input_batch
      Utils.storePickleOnDisk(input_pickled_object, ENV_CRAWLER_STORE_PATH, "crawler.in." + str(self.batch.id))
      batch_items = []
      for batchItem in self.batch.items:
        self.batchItem = batchItem
        self.processBatchItem()
        #Currently we update URL.ErrorMask for each url in batch
        if not self.batchItem:
          self.updateBatchItem(batchItem)
        else:
          self.updateBatchItem()
        if type(self.batchItem) == type([]):
          self.logger.debug("Batch item is list of items")
          for item in self.batchItem:
            batch_items.append(item)
        else:
          self.logger.debug("Batch item is not list")
          batch_items.append(self.batchItem)
      self.logger.debug("batch_items: " + str(batch_items))
      if input_batch.crawlerType == dc_event.Batch.TYPE_REAL_TIME_CRAWLER:
        process_task_batch = input_batch
        process_task_batch.items = self.store_items
      else:
        process_task_batch = Batch(input_batch.id, batch_items)
      output_pickled_object = pickle.dumps(process_task_batch)
      Utils.storePickleOnDisk(output_pickled_object, ENV_CRAWLER_STORE_PATH, "crawler.out." + str(self.batch.id))
      sys.stdout.write(output_pickled_object)
      sys.stdout.flush()
    except Exception, err:
      self.logger.error("Crawler has failed. %s", err.message)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)


  ##updateBatchItem
  # updateBatchItem
  def updateBatchItem(self, batchItem=None):
    #update batchItem properties
    # RSS feed items -> batch items
    if (self.processorName==PCONSTS.PROCESSOR_FEED_PARSER) and self.feed_items is not None:
      self.batchItem = self.createBatchItems()
      #self.batchItem.properties[dc_event.BatchItem.PROP_FEED] = self.feed_items
      #self.logger.debug("Batch property: %s" % varDump(self.batchItem.properties))
    elif self.url is not None:
      batchItem = self.batchItem if not batchItem else batchItem
      self.logger.debug("Before: batchItem urlObj errorMask: %s, url ErrorMask: %s" % (batchItem.urlObj.errorMask, 
                                                                                        self.url.errorMask))
      batchItem.urlObj.errorMask = self.url.errorMask
      self.logger.debug("After: batchItem urlObj errorMask: %s, url ErrorMask: %s" % (batchItem.urlObj.errorMask, 
                                                                                       self.url.errorMask))
    self.store_items.append(batchItem)


  # #createBatchItems
  #
  def createBatchItems(self):
    items = []
    for urlMd5, value in self.feed_items.items():
      self.logger.debug("URLMD5: %s" % str(urlMd5))
      self.logger.debug("value: %s" % str(value))
      siteId = self.batchItem.siteId
      urlObj = value["urlObj"]

      #serialize content
      value.pop("urlObj", None)

      # list of objects must be converted to string before json dumps
      dates = ["published_parsed", "updated_parsed"]
      for date in dates:
        if date in value["entry"]:
          value["entry"][date] = strftime("%a, %d %b %Y %H:%M:%S +0000", value["entry"][date])
      value["urlMd5"] = urlMd5

      #store raw content on disk
      try:
        self.crawledResource = CrawledResource()
        self.crawledResource.binary_content = json.dumps(value)
        self.batchItem.urlId = urlMd5
        self.getDir()
        self.writeData()
      except Exception, err:
        self.logger.debug("Can't save object on disk. Reason: %s" % str(err.message))
        continue

      urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
      batchItem = dc_event.BatchItem(siteId, urlMd5, urlObj)
      items.append(batchItem)
    return items


  # #load config from file
  # load from cli argument or default config file
  def loadConfig(self):
    try:
      self.config = ConfigParser.ConfigParser()
      self.config.optionxform = str
      if self.pargs.config:
        self.config.read(self.pargs.config)
    except:
      print MSG_ERROR_LOAD_CONFIG
      raise


  # #load db backend
  # load from cli argument or default config file
  def loadDBBackend(self):
    try:
      className = self.__class__.__name__
      dbHost = self.config.get(className, self.DB_HOST_CFG_NAME)
      dbPort = int(self.config.get(className, self.DB_PORT_CFG_NAME))
      dbUser = self.config.get(className, self.DB_USER_CFG_NAME)
      dbPWD = self.config.get(className, self.DB_PWD_CFG_NAME)

      db_dc_sites = self.config.get(className, self.DB_SITES_CFG_NAME)
      db_dc_urls = self.config.get(className, self.DB_URLS_CFG_NAME)

      self.dc_sites_db_connect = mdb.connect(dbHost, dbUser, dbPWD, db_dc_sites, dbPort, charset="utf8",
                                             use_unicode=True)
      self.dc_urls_db_connect = mdb.connect(dbHost, dbUser, dbPWD, db_dc_urls, dbPort, charset="utf8", use_unicode=True)
      # self.dc_sites_db_connect.set_character_set("utf8")
      # self.dc_urls_db_connect.set_character_set("utf8")
      self.site_connector = self.dc_sites_db_connect
      self.urlConnector = self.dc_urls_db_connect
      self.siteTable = None
      self.urlTable = None
    except Exception, err:
      self.logger.error("Error load config param: %s" % err.message)
      raise


  # #loadKeyValueDB load key-value db
  #
  #
  def loadKeyValueDB(self):
    try:
      className = self.__class__.__name__
      self.kvDbDir = self.config.get(className, self.DB_DATA_DIR)
    except Exception, err:
      self.logger.error("Error load config param: %s" % err.message)
      raise


  # #load logging
  # load logging configuration (log file, log level, filters)
  #
  def loadLogConfigFile(self):
    try:
      log_conf_file = self.config.get("Application", "log")
      logging.config.fileConfig(log_conf_file)
      self.logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
      # Replace logger name for processes instances
      lfn = Utils.LoggerFileName()
      fname = lfn.findReplace()
      if fname != "":
        pin = lfn.getFreeProcInstanceNumber(os.path.basename(fname))
        #self.logger.info("LOGGER>>>>: %s, %s, %s", str(fname), pin, os.path.basename(fname))
        if pin != "" and pin != "0":
          pin = "." + pin + ".log"
          fname = lfn.findReplace(fname + pin)
          self.logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
          #self.logger.info("LOGGER>>>>: %s", str(lfn.findReplace()))
    except:
      print MSG_ERROR_LOAD_LOG_CONFIG_FILE
      raise


  # #load mandatory options
  # load mandatory options
  #
  def loadOptions(self):
    try:
      self.raw_data_dir = self.config.get(self.__class__.__name__, "raw_data_dir")
      self.default_header_file = self.config.get(self.__class__.__name__, "headers_file")
      self.default_cookie_file = self.config.get(self.__class__.__name__, "cookie_file")
      self.defaultIcrCrawlTime = self.config.getfloat(self.__class__.__name__, "default_icr_crawl_time")
      db_task_ini = self.config.get(self.__class__.__name__, "db-task_ini")

      # Add support operations updateCollectedURLs and removeURLs
      cfgParser = ConfigParser.ConfigParser()
      cfgParser.read(db_task_ini)
      self.wrapper = DBTasksWrapper(cfgParser)
    
      # does call collectAddtionalProp and collectProperties methods
      self.max_fetch_time = self.config.getint(self.__class__.__name__, "max_fetch_time")
      
      # keep old resources on disk
      if self.config.has_option(self.__class__.__name__, "keep_old_resources"):
        self.keep_old_resources = self.config.getboolean(self.__class__.__name__, "keep_old_resources")
      self.collect_additional_prop = self.config.getboolean(self.__class__.__name__, "collect_additional_prop")
      self.urlProcess = URLProcess.URLProcess()
      self.urlProcess.dbWrapper = self.wrapper
      self.resourceProcess = ResourceProcess.ResourceProcess()
      self.resourceProcess.dbWrapper = self.wrapper
    except Exception, err:
      self.logger.error("Error load config param: %s" % err.message)
      raise


  # #checkResponse
  #
  def checkResponse(self):
    self.logger.debug("Requests response history: " + str(self.res.redirects))
    # calculate http redirect
    if HTTP_REDIRECT  in self.res.redirects:
      self.http_redirects = self.http_redirects + 1
      self.logger.debug("http redirect: " + str(self.http_redirects))
      self.logger.debug("http max redirect: " + str(self.max_http_redirects))
    # calculate html redirect
    if self.dom is not None and bool(self.dom.xpath(META_XPATH)):
      self.html_redirects = self.html_redirects + 1
      self.logger.debug("html redirect: " + str(self.html_redirects))
      self.logger.debug("html max redirect: " + str(self.max_html_redirects))
    # check http redirect
    if self.max_http_redirects and self.max_http_redirects != MAX_HTTP_REDIRECTS_UNLIMITED and \
    self.http_redirects >= self.max_http_redirects:
      self.allow_http_redirects = False
      self.logger.debug("http redirect limit was reached! Max http redirects: %s, encountered http redirects: %s." %
                        (str(self.max_http_redirects), str(self.http_redirects)))
    else:
      self.allow_http_redirects = True
    # check html redirect
    if self.max_html_redirects and int(self.max_html_redirects) != MAX_HTML_REDIRECTS_UNLIMITED and \
    int(self.html_redirects) >= int(self.max_html_redirects):
      self.allow_html_redirects = False
      self.logger.debug("html redirect limit was reached! Max html redirects: %s, encountered html redirects: %s." %
                        (str(self.max_html_redirects), str(self.html_redirects)))
    else:
      self.allow_html_redirects = True


  # #calcLastModified
  #
  def calcLastModified(self, resource, res):
    if resource.http_code == 304:
      resource.last_modified = self.url.tcDate
      #resource.last_modified = self.url["TcDate"]
    elif 'Last-Modified' in res.headers:
      resource.last_modified = res.headers['Last-Modified']
      resource.last_modified = parse(resource.last_modified).strftime('%Y-%m-%d %H:%M:%S')
    elif 'Date' in res.headers:
      resource.last_modified = res.headers['Date']
      resource.last_modified = parse(resource.last_modified).strftime('%Y-%m-%d %H:%M:%S')
    else:
      resource.last_modified = time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime(time.time() - self.defaultIcrCrawlTime))
    self.logger.debug("LastModified date:" + str(resource.last_modified))
    return str(resource.last_modified)


  # #updateCollectedURLs
  #
  def updateCollectedURLs(self):
    self.logger.debug("updateCollectedURLs")
    self.wrapper.fieldRecalculating(Utils.autoFillSiteId(self.batchItem.siteId))


  # #get exit code
  # return extt code
  # 0 if no critical errors was encountered during processing
  # 1 if smth bad happened
  #
  def getExitCode(self):
    return self.exit_code


# #RedirectHistory model,
# contains status code and response headers of one redirect response
class RedirectHistory(object):


  # #constructor
  # initialize fields
  def __init__(self, statusCode, responseHeader):
    self.statusCode = statusCode
    self.responseHeader = responseHeader
