"""
HCE project, Python bindings, Distributed Tasks Manager application.
Event objects definitions.

@package: dc
@file CrawlerTask.py
@author Oleksii <developers.hce@gmail.com>
@author madk <developers.hce@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

# import os
import os.path
import sys
import time

import magic
# import pickle
try:
  import cPickle as pickle
except ImportError:
  import pickle

# import logging
import logging.config
import ConfigParser
import robotparser
import urlparse
import re
import requests
from dc_crawler.Fetcher import BaseFetcher
#from dc_crawler import LangDetector
import json
import urllib
import hashlib
import datetime
from dateutil.parser import *
import lxml.html
import lxml.etree
import MySQLdb.cursors
import MySQLdb as mdb
import sqlite3
import tidylib
from contextlib import closing
from cement.core import foundation
from collections import namedtuple

from app.Utils import varDump
from dc.EventObjects import URL
from dc.EventObjects import Site
from dc.EventObjects import Batch
from dc.EventObjects import BatchItem
from dc.EventObjects import SiteFilter
import dc.EventObjects as dc_event
from dc_crawler.CrawledResource import CrawledResource
from app.Utils import PathMaker
import app.Utils as Utils  # pylint: disable=F0401
from dc_crawler.CrawlerException import CrawlerException
import dc.Constants as DC_CONSTS
from dc import EventObjects
import dc_db.Constants  as Constants
from dc_crawler.Filter import Filter
import dc_crawler.Constants as CONSTS
import dc_processor.Constants as PCONSTS
import app.Consts as APP_CONSTS
from dc_crawler.DBTasksWrapper import DBTasksWrapper
from app.Utils import SQLExpression


DB_SITES = "dc_sites"
DB_URLS = "dc_urls"

MSG_ERROR_LOAD_CONFIG = "Error loading config file. Exciting. "
MSG_ERROR_LOAD_OPTIONS = "Error loading options. Exciting. "
MSG_ERROR_LOAD_LOG_CONFIG_FILE = "Can't load logging config file. Exiting. "
MSG_ERROR_LOAD_SITE_DATA = "Can't load site data: "
MSG_ERROR_LOAD_URL_DATA = "Can't load url data: "
MSG_ERROR_PROCESS_BATCH_ITEM = "Can't process batch item "
MSG_ERROR_WRITE_CRAWLED_DATA = "Can't write crawled data "
MSG_ERROR_COLLECT_URLS = "Can't collect urls "
MSG_ERROR_ADD_URL_TO_BATCH_ITEM = "Can't add url to batch item "
MSG_ERROR_LOAD_SITE_PROPERTIES = "Can't load site properties "
MSG_ERROR_CRAWL_SITE = "Can't crawl site "
MSG_ERROR_CHECK_SITE = "Site don't passed check site "
MSG_ERROR_GET_DIR = "Can't get dir "
MSG_ERROR_READ_SITE_FROM_DB = "Can't read site data from db"
MSG_ERROR_EMPTY_RESPONSE_SIZE = "Empty response"
MSG_ERROR_CONTENT_TYPE = "Invalid content-type"

MSG_INFO_PROCESS_BATCH = "ProcessBatch "
MSG_INFO_STORE_COOKIES_FILE = "Store cookies file on disk."

MSG_DEBUG_NON_PROCESSING = "ProcessorName is NONE. Exclude batch item from further processing."

SITE_MD5_EMPTY="d41d8cd98f00b204e9800998ecf8427e"


DEFAULT_MAX_SIZE = 1000000
EMPTY_RESPONSE_SIZE = "0"

APP_NAME = "crawler-task"

HTTP_COOKIE = "HTTP_COOKIE"
DEFAULT_HTTP_COOKIE = ""
HTTP_HEADERS = "HTTP_HEADERS"
DEFAULT_HTTP_HEADER = ""

DC_URLS_DB_NAME = "dc_urls"
DC_URLS_TABLE_PREFIX = "urls_"
DC_SITES_DB_NAME = "dc_sites"
DC_SITES_PROPERTIES_TABLE_NAME = "sites_properties"
DC_SITES_TABLE_NAME = "sites"
DC_URLS_TABLE_NAME = "urls"
COOKIES_FILE_POSTFIX = ".cookies.txt"

NON_PROCESSING = "NONE"

HTTP_REDIRECT = "<Response [301]>"
HTML_REDIRECT = ""
MAX_HTTP_REDIRECTS_UNLIMITED = 0
MAX_HTML_REDIRECTS_UNLIMITED = 0
META_XPATH = "//meta[contains(@content, 'url')]/@content"

Results = namedtuple("Results", "exit_code, output, err")

ROBOTS_PATTERN = re.compile(r'(https?://[^/]+).*', re.I)

PATTERN_WITH_PROTOCOL = re.compile('[a-zA-Z]+:(//)?')

# URL_PATTERN = re.compile(r'https?://.*', re.I)
#BINARY_CONTENT_TYPE_PATTERN = re.compile('(image)|(application)', re.I)
BINARY_CONTENT_TYPE_PATTERN = re.compile('(text)|(xml)', re.I)
TEXT_CONTENT_TYPE_PATTERN = re.compile('text', re.I)

WITH_URL_XPATH_LIST = ["//a/@href", "//applet/@codebase", "//area/@href", "//base/@href", \
"//blockquote/@cite", "//body/@background", "//del/@cite", \
"//frame/@src", "//head/@profile", "//iframe/@src", "//input/@src", "//ins/@cite", \
"//object/@codebase", "//q/@cite", "//audio/@src", \
"//button/@formaction", "//command/@icon", "//embed/@src", "//html/@manifest", \
"//input/@formaction", "//source/@src", "//video/@src", "//video/@poster", "//category/@domain",
"//link", "//guid"]

KV_TABLE_TEMPLATES = {
    "titles": ''' CREATE TABLE titles (
    url_id VARCHAR(32) NOT NULL PRIMARY KEY,
    data VARCHAR(100) NOT NULL DEFAULT '')''',

    "redirects": ''' CREATE TABLE redirects (
    url_id VARCHAR(32) NOT NULL PRIMARY KEY,
    data VARCHAR(1000000) NOT NULL DEFAULT '') ''',

   "internal_links": ''' CREATE TABLE internal_links (
    url_id VARCHAR(32) NOT NULL PRIMARY KEY,
    data VARCHAR(1000000) NOT NULL DEFAULT '') ''',

   "external_links": ''' CREATE TABLE external_links (
    url_id VARCHAR(32) NOT NULL PRIMARY KEY,
    data VARCHAR(1000000) NOT NULL DEFAULT '') '''
}

ENV_CRAWLER_STORE_PATH = "ENV_CRAWLER_STORE_PATH"
# SiteProperties = namedtuple("http_header", "http_cookie")
# CrawlResponse = namedtuple("html_content", "html_header", "html_request")
# named tuple for filters
#Filter = namedtuple("Filter", "pattern, pattern_name, type, state")

DETECT_MIME_MAIN_CONTENT = "1"
DETECT_MIME_COLLECTED_URL = "2"
COLLECT_POST_DATA_NAME = "COLLECT_POST_DATA"
COLLECT_POST_DATA = "1"
RECOVER_IF_FAILED = "2"

#timeout(seconds) for mime type detecting
DETECT_MIME_TIMEOUT = 1

EXIT_SUCCESS = 0
EXIT_FAILURE = 1

# #The CrawlerTask class, is a interface for fetching content from the web
#
# This object is a run at once application
class CrawlerTask(foundation.CementApp):


  URL_EXTRACT_PA = re.compile(r'[\'"](https?://[^\'" ]*)[\'"]')

  # Configuration settings options names
  DB_HOST = "db_host"
  DB_PORT = "db_port"
  DB_USER = "db_user"
  DB_PWD = "db_pwd"
  DB_SITES = "db_dc_sites"
  DB_URLS = "db_dc_urls"

  RAW_DATA_DIR = "raw_data_dir"
  SITE_TEMPLATES = "dc_site_template"
  KEY_VALUE_STORAGE_DIR = "key_value_storage_dir"
  DB_DATA_DIR = "db_data_dir"


  # Mandatory
  class Meta(object):
    label = APP_NAME
    def __init__(self):
      self.site_table_row = None
      self.url_xpath_list = []



  # #constructor
  # initialize default fields
  def __init__(self):
    # call base class __init__ method
    foundation.CementApp.__init__(self)
    self.dc_urls_db_connect = None
    self.need_store_mime = None
    self.site = None
    self.raw_data_dir = None
    self.robotparser = None
    self.batch_item = None
    self.dc_sites_db_connect = None
    self.site_table = None
    self.site_connector = None
    self.headers_dict = None
    self.store_http_headers = None
    self.cookie = None
    self.url_connector = None
    self.proxies = None
    self.logger = None
    self.url_table = None
    self.real_url = None
    self.url = None
    self.crawledResource = None
    self.batch = None
    self.headers = None
    self.table_name = None
    self.crawled_time = None
    self.store_http_request = None
    self.dir = None
    self.default_header_file = None
    self.default_cookie_file = None
    self.kvConnector = None
    self.kvCursor = None
    self.processorName = None
    self.site_properties = None
    self.siteId = None
    self.auto_remove_props = None
    self.store_cookies = None
    self.allow_http_redirects = True
    self.allow_html_redirects = None
    self.http_redirects = 0
    self.html_redirects = 0
    self.max_http_redirects = 1
    self.max_html_redirects = None
    self.dom = None
    self.time_postfix = ""
    self.res = None
    self.post_forms = None
    self.auth_name = None
    self.auth_pwd = None
    self.kvDbDir = None
    self.default_icr_crawl_time = None
    self.wrapper = None
    self.store_items = []
    self.feed = None
    self.feed_items = None
    self.max_fetch_time = CONSTS.FETCHER_TIME_LIMIT_MAX
    self.collect_additional_prop = False
    self.exit_code = EXIT_SUCCESS
    self.keep_old_resources = False


  # #setup
  # setup application
  def setup(self):
    # call base class setup method
    foundation.CementApp.setup(self)
    self.args.add_argument('-c', '--config', action='store', metavar='config_file', help='config ini-file')


  # #run
  # run application
  def run(self):
    # call base class run method
    foundation.CementApp.run(self)

    # config section
    self.loadConfig()

    # load logger config file
    self.loadLogConfigFile()

    # load mandatory options
    self.loadOptions()

    # load db backend
    self.loadDBBackend()

    # load key-value db
    self.loadKeyValueDB()

  # #helper function for correct query execution
  #
  # @param query query for execution
  def executeQuery(self, connector, query):
    try:
      self.logger.info("executing query: %s", query)
      with closing(connector.cursor(MySQLdb.cursors.DictCursor)) as cursor:
        cursor.execute(query)
        connector.commit()
        return cursor.fetchall()
    except mdb.Error:  # @todo logging in db_task
      connector.rollback()
      raise


  # #helper function for correct update/insert/delete execution
  #
  # @param sql sql statement for execution
  def executeUpdate(self, connector, sql):
    try:
      self.logger.info("executing update: %s", sql)
      with closing(connector.cursor(MySQLdb.cursors.DictCursor)) as cursor:
        cursor.execute(sql)
        connector.commit()
        return cursor.rowcount
    except mdb.Error:  # @todo logging in db_task
      connector.rollback()
      raise


  # #helper function for execute batch update
  #
  # @param connector database connector
  # @param sql sql statement for execution
  # @param params argument sequence
  def executeMany(self, connector, sql, params):
    self.logger.warn("executing batch update, sql :%s, params:%s", sql, params)
    try:
      with closing(connector.cursor()) as cursor:
        cursor.executemany(sql, params)
        connector.commit()
        return cursor.rowcount
    except mdb.Error as err:  # @todo logging in db_task
      connector.rollback()
      self.logger.error("%s %s" % (err.args[0], err.args[1]))
      raise


  # #
  #
  # @param
  def addBatchItem(self, url):
    try:
      siteId = self.batch_item.siteId
      urlId = str(hashlib.md5(url).hexdigest())
      self.batch.append(BatchItem(siteId, urlId, None))
    except Exception as err:
      self.logger.error(MSG_ERROR_ADD_URL_TO_BATCH_ITEM + err.message)
      raise


  # #parseHost parse the root host name from url
  # for example: the result of http://s1.y1.example.com/path/to is example.com
  # @param url the full url
  # @return host of the url, eg: example.com
  @staticmethod
  def parseHost(url):
    host = None
    if urlparse.urlparse(url).hostname:
      host = '.'.join(urlparse.urlparse(url).hostname.split('.')[-2:])
    return host


  # #readCurrentCnt return urls cnt from db
  #
  def readCurrentCnt(self):
    current_cnt = 0
    if self.site.maxURLs:
      countsql = "SELECT COUNT(*) AS cnt FROM `%s` WHERE NOT (Status=4 AND Crawled=0 AND Processed=0)" % \
      (self.url_table,)
      result = self.wrapper.customRequest(countsql, DB_URLS)
      if len(result) > 0 and len(result[0]) > 0:
        current_cnt = result[0][0]
      else:
        current_cnt = 0
    return current_cnt
        

  # #update some fields in dc_urls
  #
  #url_md5 param - urls md5
  def updateUrlsStatusDate(self, url_md5):
    urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, url_md5, dc_event.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.status = 1
    self.wrapper.urlUpdate(urlUpdateObj, "`State`=0")


  # #makes urlStatus request and returns depth field
  #
  def getDepthFromUrl(self, url_md5):
    ret = 0
    urlStatusObj = dc_event.URLStatus(self.batch_item.siteId, url_md5)
    result = self.wrapper.urlStatus(urlStatusObj, True)
    if len(result) > 0 and type(result[0]) == dc_event.URL:
      ret = result[0].depth
    return ret


  # #makes UrlNew operation
  #params - list with new values of urlNew fields
  def urlNew(self, params):
    if params is not None:
      if hasattr(params, '__iter__'):
        urlsList = []
        for param in params:
          urlObject = dc_event.URL(param[0], param[1])
          urlObject.type = param[2]
          urlObject.urlMd5 = param[3]
          urlObject.requestDelay = param[4]
          urlObject.httpTimeout = param[5]
          urlObject.httpMethod = param[6]
          urlObject.parentMd5 = param[7]
          urlObject.maxURLsFromPage = param[8]
          urlObject.tcDate = SQLExpression("NOW()")
          urlObject.UDate = SQLExpression("NOW()")
          urlObject.depth = param[9]
          urlObject.contentType = param[10]
          urlObject.priority = param[11]
          #RSS Feed init url
          if len(param)==18:
            urlObject.status = param[12]
            urlObject.crawled = param[15]
            urlObject.contentType = param[13]
            urlObject.pDate = param[14]
            urlObject.type = param[16]
            urlObject.size = param[17]
          urlsList.append(urlObject)
          self.logger.debug("URL: %s" % varDump(urlObject))
          self.logger.debug("PARAM: %s" % varDump(param))
          self.logger.debug("PARAM LEN: %s" % str(len(param)))
        result = self.wrapper.urlNew(urlsList)
        self.logger.debug("rows_count: %s", result)
      self.updateCollectedURLs()


  # #insertNewSiteProperties update Site (add new properties in correspond table)
  #siteId - site's id
  #params - list with new values of urlNew fields
  def insertNewSiteProperties(self, siteId, params):
    if siteId is not None and hasattr(params, '__iter__') and len(params) > 0:
      localSiteUpdate = dc_event.SiteUpdate(siteId)
      for attr in localSiteUpdate.__dict__:
        if hasattr(localSiteUpdate, attr):
          setattr(localSiteUpdate, attr, None)
      localSiteUpdate.updateType=dc_event.SiteUpdate.UPDATE_TYPE_APPEND
      localSiteUpdate.id = siteId
      localSiteUpdate.properties = []
      for param in params:
        newPropElem = {}
        newPropElem["siteId"] = param[0]
        newPropElem["name"] = param[1]
        newPropElem["value"] = param[2]
        localSiteUpdate.properties.append(newPropElem)
      self.wrapper.siteNewOrUpdate(localSiteUpdate, stype=dc_event.SiteUpdate)


  # #collectURLs collect URL from response body
  #
  # @return is collect successfully
  # bullshit, move it to the separate module (like URLCollector)
  def collectURLs(self):
    if self.url.type == dc_event.URL.TYPE_SINGLE:
      self.logger.debug("URL type: single")
      return False

    if not BINARY_CONTENT_TYPE_PATTERN.search(self.crawledResource.content_type):
      self.logger.debug("Binary content: <<%s>>", self.crawledResource.content_type)
      return False

    # don't parse url for 4XX or 5XX response
    code_type = int(self.crawledResource.http_code) / 100
    if code_type == 4 or code_type == 5:
      return False
    if not self.crawledResource.html_content:
      return False
    internal_links, external_links = [], []
    host = CrawlerTask.parseHost(self.real_url)
    #results like filtes
    #(1) Read Filters
    results = [localFilter for localFilter in self.site.filters if \
    localFilter.state==SiteFilter.TYPE_ENABLED and localFilter.mode==SiteFilter.TYPE_URL]
    self.logger.debug("results: %s", varDump(results))
    filter_types = []
    filter_states = []
    filter_patterns = []
    filters = []
    if len(results):
      filters = [Filter(result.pattern, result.type) for result in results]
      self.logger.debug("filters: %s", varDump(filters))
    #(1) END
    if self.dom is None:
      return False
    # use set for avoid duplicate urls
    url_set = set()
    
    #(2) Parse rss feed
    # parse rss feed 
    # if site processing name is rss feed brief parsing
    # and url was not fetched yet
    if (self.processorName==PCONSTS.PROCESSOR_FEED_PARSER or self.processorName==PCONSTS.PROCESSOR_RSS) \
      and self.batch_item.urlObj.type!=dc_event.URL.TYPE_FETCHED:
      try:
        import feedparser
        self.feed_items = {}
        self.feed = feedparser.parse(self.crawledResource.html_content)
        #self.logger.debug("self.feed: %s" % varDump(self.feed))
        url_set.update(entry.link for entry in self.feed.entries)
      except TypeError, err:
        self.logger.debug("WRONG URL <%s> not rss feed. %s" % (str(self.batch_item.urlObj.url), str(err.message)))
    # won't collect urls from rss feed resources
    elif self.processorName!=PCONSTS.PROCESSOR_RSS:
      # Added support of url_xpath_list as site's properties
      if len(self.url_xpath_list):
        self.logger.debug("Site has URL_XPATH_LIST property: %s", str(self.url_xpath_list))
      else:
        self.url_xpath_list = WITH_URL_XPATH_LIST
        self.logger.debug("Site hasn't URL_XPATH_LIST property. Use default xpath's' list.")
      for xpath in self.url_xpath_list:
        elem = self.dom.xpath(xpath)
        elem_type = type(elem)
        if elem_type == list and len(elem) > 0 and hasattr(elem[0], "tail"):
          url_set.update([el.tail for el in elem])
        elif elem_type == list and len(elem) > 0 and isinstance(elem[0], lxml.html.HtmlElement):
          url_set.update([el.text for el in elem])
        else:
          url_set.update(elem)
    self.logger.debug("url_set: %s", str(url_set))
    #(2) END
    
    #(3) Extract POST Data
    #check if collect POST data
    form_urls, form_methods, form_fields = [], {}, {}
    if COLLECT_POST_DATA_NAME in self.site_properties and \
      self.site_properties['COLLECT_POST_DATA'] == COLLECT_POST_DATA:
      form_urls, form_methods, form_fields = self.extractFormURL(self.dom)
    url_set.update(form_urls)
    #(3) END

    # Updated by Oleksii
    # support maxURLsFromPage
    num_urls_from_page = 0

    current_cnt = self.readCurrentCnt()
    #(4) Loop by url_set
    params = []
    for url in url_set:
      _url = url
      #(5) Reassign url value
      if self.url.type == URL.TYPE_SINGLE or not url:
        self.logger.debug("URL type is TYPE_SINGLE - not collect urls. Skip url.")
        continue
      #if self.url["Type"] == URL.TYPE_SINGLE or not url: continue
      # url contains protocol, but protocol is not http/https(such as javascript,mailto),
      # just ignore it
      self.logger.debug("URL: %s", url)
      if PATTERN_WITH_PROTOCOL.match(url) and \
       not url.lower().startswith("http://") and not url.lower().startswith("https://"):
        continue
      if not url.lower().startswith("http://") and not url.lower().startswith("https://"):
        url = urlparse.urljoin(self.real_url, url)
        # normalization
        # self.logger.debug("NORMALIZATION: init url: %s", url)
        url = URL(0, url).getURL()
        # self.logger.debug("NORMALIZATION: norm url: %s", url)
        internal_links.append(url)
        self.logger.info("internal")
      else:
        # normalization
        # self.logger.debug("NORMALIZATION: init url: %s", url)
        url = URL(0, url).getURL()
        # self.logger.debug("NORMALIZATION: norm url: %s", url)
        if CrawlerTask.parseHost(url) == host:
          internal_links.append(url)
          self.logger.info("internal")
        elif CrawlerTask.parseHost(url):
          external_links.append(url)
          self.logger.info("external")
        else:  # not valid url like http://
          continue
      #(5) END

      #(6) Filter apply
      flag = False
      if isinstance(filters, list) and len(filters):
        # filter by Pattern
        for localFilter in filters:
          self.logger.debug("filter: %s", varDump(localFilter))
          match = localFilter.pattern.match(url)
          if localFilter.type == SiteFilter.TYPE_EXCLUDE and match:
            self.logger.debug("Exclude B url: %s", url)
            flag = True
            break
            # continue
          if localFilter.type == SiteFilter.TYPE_INCLUDE and not match and not localFilter.date:
            self.logger.debug("Exclude A url: %s", url)
            self.logger.debug("filter.type: %s", str(localFilter.type))
            self.logger.debug("match: %s", str(match))
            self.logger.debug("filter.date: %s", str(localFilter.date))
            flag = True
            break
            # continue
          if localFilter.date:
            self.logger.debug("Filter has date rule.")
            if localFilter.date_checker_pattern.match(url):
              self.logger.debug("url has date pattern")
              match = localFilter.pattern.match(url)
              if not match:
                self.logger.debug("Exclude by date url: %s", url)
                flag = True
                break
          if localFilter.max_depth is not None:
            self.logger.debug("Filter has MaxDepth constrain. Max Depth is: %s", str(localFilter.max_depth))
            self.logger.debug("URL Max Depth is: %s", str(self.url.depth))
            if int(localFilter.max_depth) <= int(self.url.depth):
              self.logger.debug("URL Max Depth limit was reached.")
              flag = True
              break
      if flag: continue
      #(6) END
      #(7) Update UrlStatusDate
      url_md5 = hashlib.md5(url).hexdigest()
      query = "SELECT COUNT(*), (DATE_ADD(UDate, INTERVAL %s MINUTE)-NOW()), `Type` FROM `%s` WHERE `URLMd5` = '%s'" % \
      (str(self.site.recrawlPeriod), self.url_table, url_md5)
      result = self.wrapper.customRequest(query, DB_URLS)
      if len(result) > 0 and len(result[0]) > 0 and result[0][0] > 0:
        if self.site.recrawlPeriod == 0 or \
          (len(result[0]) > 1 and result[0][1] > 0) or \
          (result[0][2] == dc_event.URL.TYPE_FETCHED) or \
          ("RECRAWL_NO_ROOT_URLS" in self.site_properties and self.site_properties["RECRAWL_NO_ROOT_URLS"]=="0"):
          self.logger.debug("URL skipped, exists and re-crawling not active, time not reached or URL Type is from "+
                            "RSS feed (not to fetch)\n %s %s", url, url_md5)
          continue
        else:
          self.updateUrlsStatusDate(url_md5)
          self.logger.debug("URL state updated to NEW because re-crawling\n %s %s", url, url_md5)
          continue
      else:
        self.logger.debug("URL passed check and treated as new\n %s %s", url, url_md5)
      #(7) END
      
      # MaxURLs limit
      if self.site.maxURLs:
        if self.site.maxURLs and current_cnt < self.site.maxURLs:
          self.logger.debug("Site MaxURLs limit need to be checked.")
          current_cnt += 1
        else:
          if self.autoRemoveURL() == 0:
            # should not use break, because need add to internal/external links although reach the MaxURL limit
            continue

      # Updated by Oleksii
      # support maxURLsFromPage
      if  self.site.maxURLsFromPage and num_urls_from_page >= self.site.maxURLsFromPage:
        self.logger.debug("Site maxURLsFromPage limit reached! Site.MaxURLsFromPage: %s, current_page_num: %s",
                          self.site.maxURLsFromPage, num_urls_from_page)
        continue

      #detect collected url mime type and ignore non-match URL
      #(7) Detect collected url mime type and ignore non-match URL
      detected_mime = ''
      if self.auto_detect_mime == DETECT_MIME_COLLECTED_URL:
        detected_mime = self.detectUrlMime(url)
        if detected_mime not in self.process_content_types:
          continue
      num_urls_from_page += 1
      #(7) END

      depth = self.getDepthFromUrl(self.batch_item.urlId)
      
      # RSS Feed parse
      #(8) Calculate params for RSS feed
      if self.processorName == PCONSTS.PROCESSOR_FEED_PARSER or \
        self.processorName == PCONSTS.PROCESSOR_RSS :
        _contentType = None
        _pubdate = None
        _status = dc_event.URL.STATUS_CRAWLED
        _crawled = 1
        _type = dc_event.URL.TYPE_FETCHED
        if self.processorName == PCONSTS.PROCESSOR_RSS :
          _status = dc_event.URL.STATUS_NEW
          _crawled = 0
          _type = dc_event.URL.TYPE_SINGLE
        for entry in self.feed.entries:
          if entry.link == _url:
            self.feed_items[url_md5] = {}
            self.feed_items[url_md5]["entry"] = entry
            self.feed_items[url_md5]["urlObj"] = URL(self.batch_item.siteId, url)
            self.feed_items[url_md5]["parent_rss_feed"] = self.url.url
            self.feed_items[url_md5]["parent_rss_feed_urlMd5"] = self.batch_item.urlId
            #pubdate
            from time import gmtime, strftime
            self.feed_items[url_md5]["pubdate"] = None
            dates = ["published_parsed", "updated_parsed"]
            for date in dates:
              if date in entry:
                try:
                  self.feed_items[url_md5]["pubdate"] = strftime("%Y-%m-%d %H:%M:%S", entry[date])
                except TypeError, err:
                  self.logger.debug("Unsupported date format: <%s>" % varDump(entry[date]))
            _pubdate = self.feed_items[url_md5]["pubdate"]
            _contentType = entry.links[0].type
            _size = len(str(self.feed_items[url_md5]))
        params.append((self.batch_item.siteId, mdb.escape_string(url), self.url.type,
        url_md5, self.url.requestDelay, self.url.httpTimeout,
        form_methods.get(url, "get"), self.batch_item.urlId, self.site.maxURLsFromPage, (depth + 1), detected_mime, 
                       self.url.priority, _status, _contentType, _pubdate, _crawled, _type, _size))
      #(8) END
      # regular parse
      else:
        params.append((self.batch_item.siteId, mdb.escape_string(url), self.url.type,
                       url_md5, self.url.requestDelay, self.url.httpTimeout,
                       form_methods.get(url, "get"), self.batch_item.urlId, self.site.maxURLsFromPage, (depth + 1),
                       detected_mime, self.url.priority))
    #(5) END
    self.urlNew(params)

    #(9) Processing post form
    if form_fields:
      field_params = []
      for field_name, field_value in form_fields.iteritems():
        if field_name in self.post_forms:
          continue
        self.logger.debug("field_name: %s", field_name)
        self.logger.debug("field_value: %s", field_value)
        field_params.append((self.batch_item.siteId, "HTTP_POST_FORM_" + field_name, field_value))
        
      self.insertNewSiteProperties(self.batch_item.siteId, field_params)
    #(9) END
    #(10) Addition prop
    if self.collect_additional_prop:
      try:
        self.collectProperties(self.dom, internal_links, external_links)
      except Exception:
        self.logger.warn("collect base properties to key-value db failed", exc_info=True)
      try:
        self.collectAddtionalProp(len(internal_links), len(external_links))
      except Exception:
        self.logger.warn("collect addtional propeties to main db failed", exc_info=True)
    #(10) END
    return True


  # # detect mime type for an URL using HEAD method
  #
  # @param url the target URL
  # @return mime_type detected mime type, empty string if failed
  def detectUrlMime(self, url):
    try:
      res = requests.head(url, timeout=DETECT_MIME_TIMEOUT)
      return res.headers.get('content-type', '').lower()
    except BaseException:
      self.logger.warn("detect mime type for %s failed", url, exc_info=True)
      return ''

  # #collectAddtionalProp collect addtional properties to main MySQL db
  #
  # @param internal_links_num the count of internal link list
  # @param external_links_num the count of external link list
  def collectAddtionalProp(self, internal_links_num, external_links_num):
    self.logger.debug("Response: %s", str(self.res))
    size = len(self.res.str_content)
    content_md5 = hashlib.md5(self.res.str_content).hexdigest()
    kv_sql = "SELECT data FROM internal_links WHERE url_id <> '%s'" % (self.batch_item.urlId,)
    self.kvCursor.execute(kv_sql)
    freq = 0
    for row in self.kvCursor.fetchall():
      url_internal_lists = row["data"]
      if self.real_url in url_internal_lists:
        freq += 1

    urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.size = size
    urlUpdateObj.linksI = internal_links_num
    urlUpdateObj.linksE = external_links_num
    urlUpdateObj.freq = freq
    urlUpdateObj.rawContentMd5 = content_md5
    self.wrapper.urlUpdate(urlUpdateObj)
    

  # # extrace URL, form action, and fields from html dom
  #
  # @param dom the dom tree
  # @return form_urls    sequence of urls extracted
  # @return form_methods  dict of form methods, {form_action: form_method}
  # @return form_fields  dict of fields {field_name: field_value}
  def extractFormURL(self, dom):
    form_urls, form_methods, form_fields = [], {}, {}
    for form in dom.xpath("//form"):
      form_action = None
      form_method = 'get'
      for attr in form.keys():
        if attr.lower() == "action":
          form_action = form.get(attr)
          form_urls.append(form_action)
        elif attr.lower() == "method":
          form_method = form.get(attr)
      if not form_action:
        continue
      form_methods[form_action] = form_method
      for field in form.getchildren():
        tag_name, tag_value = None, ""
        for field_tag in field.keys():
          if field_tag.lower() == "name":
            tag_name = field.get(field_tag)
          elif field_tag.lower() == "value":
            tag_value = field.get(field_tag)
        if tag_name:
          form_fields[tag_name] = tag_value
    self.logger.info("extracted form data, form_urls:%s, form_methods:%s, form_fields:%s", \
    form_urls, form_methods, form_fields)
    return form_urls, form_methods, form_fields


  # # checkKVTable check weather the sqlite table exists
  # if not, then create it
  #
  def checkKVTable(self):
    for table in ("titles", "redirects", "internal_links", "external_links"):
      sql = "SELECT COUNT(*) as cnt FROM sqlite_master WHERE type='table' AND name='%s'" \
      % table
      self.kvCursor.execute(sql)
      if self.kvCursor.fetchone()["cnt"] == 0:
        self.logger.info("kv table %s dose not exist, now create it", table)
        self.kvCursor.execute(KV_TABLE_TEMPLATES[table])
        self.kvConnector.commit()


  # #collectProperties collect page properties to Key-Value DB
  #
  # @param dom the dom tree of the page
  # @param internal_links internal link list
  # @param external_links external link list
  def collectProperties(self, dom, internal_links, external_links):
    self.logger.info("start collect properties")
    self.prepareKvDbConnector()
    self.checkKVTable()
    title = None
    dom_title = None
    tmp = dom.find(".//title")
    if tmp is not None:
      dom_title = tmp.text
    if isinstance(dom_title, lxml.etree._Element):
      title = dom_title.text
    if isinstance(title, str):
      title = title.decode('utf-8')

    histories = []
    for history in self.res.redirects:
      text_headers = '\r\n'.join(['%s: %s' % (k, v) for k, v in history.headers.iteritems()])
      history_item = {"status_code": history.status_code, "headers": text_headers}
      histories.append(history_item)
    histories_data = json.dumps(histories)
    internal_links_data = json.dumps(internal_links)
    external_links_data = json.dumps(external_links)
    # save title
    self.kvCursor.execute('''INSERT OR REPLACE INTO titles(url_id, data) VALUES(?, ?)''', \
      (self.batch_item.urlId, title))

    # save redirects
    self.kvCursor.execute('''INSERT OR REPLACE INTO redirects(url_id, data) VALUES(?, ?)''', \
      (self.batch_item.urlId, histories_data))

    # save internal links
    self.kvCursor.execute('''INSERT OR REPLACE INTO internal_links(url_id, data) VALUES(?, ?)''', \
      (self.batch_item.urlId, internal_links_data))

    # save external links
    self.kvCursor.execute('''INSERT OR REPLACE INTO external_links(url_id, data) VALUES(?, ?)''', \
      (self.batch_item.urlId, external_links_data))

    self.kvConnector.commit()


  # #writeData  write the response body and headers to file
  #
  # @param
  def writeData(self):
    # check wether need store to disk
    if self.need_store_mime and self.need_store_mime != '*':
      if self.crawledResource.content_type.lower() not in self.need_store_mime:
        self.logger.info("do not store resource files to disk")
        return
        # self.logger.info("do not store resource files to disk")
        # self.updateURLForFailed(CONSTS.ERROR_MIME_NOT_WRITE)
        # raise CrawlerException("MIME Content-Type not allow store content to disk")

    #self.logger.info("store resource files to disk")
    #self.logger.debug("urlMd5: %s" % str(self.batch_item.urlId))
    #self.makeDir()
    # html content
    #base_path = os.path.join(self.dir, self.batch_item.urlId + "_" + self.time_postfix)

    # save raw file
    #html_content_file_name = base_path + ".bin"
    #self.logger.debug("self.crawledResource: %s" % varDump(self.crawledResource))
    #with open(html_content_file_name, "wb") as f:

    urlPut_list = []

    if TEXT_CONTENT_TYPE_PATTERN.match(self.crawledResource.content_type):
      # save UTF-8 encoding text for text content types
      if self.crawledResource.dynamic_fetcher_type:
        raw_unicode_content = self.crawledResource.meta_content
      else:
        raw_unicode_content = self.crawledResource.html_content
      if raw_unicode_content:
        putDict = {"data": raw_unicode_content}
        content_type = EventObjects.Content.CONTENT_RAW_CONTENT
        urlPut_list.append(dc_event.URLPut(self.batch_item.siteId, self.batch_item.urlId, content_type, putDict))
        #f.write(raw_unicode_content)

        # save tidy recovered file
        if self.html_recover == "1":
          #with open(base_path + ".tidy", "wb") as tidy_f:
            tidy_content, errors = tidylib.tidy_document(raw_unicode_content)
            putDict = {"data": tidy_content}
            content_type = EventObjects.Content.CONTENT_TIDY_CONTENT
            urlPut_list.append(dc_event.URLPut(self.batch_item.siteId, self.batch_item.urlId, content_type, putDict))
            #tidy_f.write(tidy_content)
    else:
      # save origin binary data for non-text content types
      if self.crawledResource.binary_content:
        putDict = {"data": self.crawledResource.binary_content}
        content_type = EventObjects.Content.CONTENT_RAW_CONTENT
        urlPut_list.append(dc_event.URLPut(self.batch_item.siteId, self.batch_item.urlId, content_type, putDict))
        #f.write(self.crawledResource.binary_content)

    #detect mime type
    detected_mime = None
    if self.auto_detect_mime == DETECT_MIME_MAIN_CONTENT:
      detected_mime = magic.from_file(html_content_file_name, mime=True)
      self.logger.info("detected mime type:%s", detected_mime)

    # save rendered file
    if self.crawledResource.dynamic_fetcher_type and self.crawledResource.html_content:
      putDict = {"data": self.crawledResource.html_content}
      content_type = EventObjects.Content.CONTENT_DYNAMIC_CONTENT
      urlPut_list.append(dc_event.URLPut(self.batch_item.siteId, self.batch_item.urlId, content_type, putDict))
      #rendered_file_name = base_path + "." + self.crawledResource.dynamic_fetcher_type + ".dyn"
      #with open(rendered_file_name, "wb") as f:
        #f.write(self.crawledResource.html_content)

    # html header
    if self.store_http_headers and self.crawledResource.response_header:
      putDict = {"data": self.crawledResource.response_header}
      content_type = EventObjects.Content.CONTENT_HEADERS_CONTENT
      urlPut_list.append(dc_event.URLPut(self.batch_item.siteId, self.batch_item.urlId, content_type, putDict))
      #html_header_file_name = base_path + ".headers.txt"
      #with open(html_header_file_name, "wb") as f:
        #self.logger.warn("file is %s", html_header_file_name)
        #if self.crawledResource.response_header:
          #f.write(self.crawledResource.response_header)

    # html request
    if self.store_http_request and self.crawledResource.html_request:
      putDict = {"data": self.crawledResource.html_request}
      content_type = EventObjects.Content.CONTENT_REQUESTS_CONTENT
      urlPut_list.append(dc_event.URLPut(self.batch_item.siteId, self.batch_item.urlId, content_type, putDict))
      #html_request_file_name = base_path + ".request.txt"
      #with open(html_request_file_name, "wb") as f:
        #f.write("GET %s HTTP/1.0\r\n\r\n" % (self.real_url,))
        #if self.crawledResource.html_request:
          #f.write(self.crawledResource.html_request)

    # Write raw content on disk via db-task
    self.wrapper.putURLContent(urlPut_list)

    #change url's contentMask
    self.url.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
    self.batch_item.urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
    return detected_mime


  # #getDir prepare dir
  #
  # @param
  def getDir(self):
    if len(self.batch_item.siteId):
      self.dir = os.path.join(self.raw_data_dir, self.batch_item.siteId, PathMaker(self.batch_item.urlId).getDir())
    else:
      self.dir = os.path.join(self.raw_data_dir, "0", PathMaker(self.batch_item.urlId).getDir())


  # #getDir prepare dir
  #
  # @param
  def makeDir(self):
    if not os.path.exists(self.dir):
      os.makedirs(self.dir)
    elif not os.path.isdir(self.dir):
      self.updateURLForFailed(CONSTS.ERROR_WRITE_FILE_ERROR)
      raise Exception("path %s exists, but is not a directory" % (self.dir,))


  # #updateURLForFailed
  #
  # @param error_bit BitMask of error
  def updateURLForFailed(self, error_bit):
    urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.status = URL.STATUS_CRAWLED
    urlUpdateObj.errorMask = error_bit
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    self.wrapper.urlUpdate(urlUpdateObj)
    self.updateSite(error_bit)
    self.errorMask = error_bit

  # #crawl site
  #
  # @return should continue to write data and collect URLs
  def crawl(self):
    self.crawledResource = None
    # delay
    time.sleep(self.url.requestDelay / 1000.0)
    #(1) Decode url - [UrlProcess]
    # TODO move to - [UrlProcess]
    if self.url.url.startswith("http%3A") or self.url.url.startswith("https%3A"):
      url = urllib.unquote(self.url.url).decode('utf-8')
    else:
      url = self.url.url.decode('utf8')
    #(1) END
    self.real_url = url
    self.logger.info("allowed in robots.txt")
    start_time = time.time()

    try:
      self.logger.info("start to fetch %s", url)
      #(2) Makes auth tuple - [UrlProcess]
      # TODO move to - [UrlProcess]
      if self.auth_name and self.auth_pwd:
        auth = (self.auth_name, self.auth_pwd)
        self.logger.info("using http basic auth %s:%s", self.auth_name, self.auth_pwd)
      else:
        auth = None
      #(2) END
      #(3) Resolve HTTP Method (move convertToHttpDateFmt to Utils) - [UrlProcess]
      # TODO move to - [UrlProcess]
      try:
        method = self.url.httpMethod.lower()
      except Exception:
        method = "get"
      post_data = None
      if method == "post":
        post_data = self.post_forms
        self.logger.info("use post, post_data:%s", post_data)
      else:
        self.logger.info("last modified: <<%s>>", str(type(self.url.lastModified)))
        self.logger.info("last modified: <<%s>>", str((self.url.lastModified)))
        if str(self.url.lastModified)!="None":
          self.logger.info("If-Modified-Since: <<%s>>", self.url.lastModified)
          self.logger.info("If-Modified-Since type: <<%s>>", str(type(self.url.lastModified)))
          self.headers_dict["If-Modified-Since"] = \
          self.convertToHttpDateFmt(datetime.datetime.strptime(str(self.url.lastModified),"%Y-%m-%d %H:%M:%S"))
      #(3) END
    
      res = BaseFetcher.get_fetcher(self.site.fetchType).open(url,
        external_url=self.external_url,
        timeout=int(self.url.httpTimeout) / 1000.0,
        headers=self.headers_dict,
        allow_redirects=self.allow_http_redirects,
        proxies=self.proxies, auth=auth,
        data=post_data, logger=self.logger,
        allowed_content_types=self.process_content_types,
        max_resource_size=self.site.maxResourceSize,
        max_redirects=self.max_http_redirects)

      #(4) Checks and update url in db-task them - [UrlProcess]
      # TODO move to - [UrlProcess]
      if not res.headers:
        self.updateURLForFailed(res.error_mask)
        return False

      if "content-length" in res.headers and res.headers["content-length"] == EMPTY_RESPONSE_SIZE:
        self.logger.debug("content-length: %s", str(res.headers["content-length"]))
        self.logger.debug(MSG_ERROR_EMPTY_RESPONSE_SIZE)
        self.updateURLForFailed(CONSTS.ERROR_EMPTY_RESPONSE)
        return False
      #(4) END

      # save res to self, will use it in next steps
      #(5) Create and fill resource struct - [ResourceProcess]
      # TODO move to - [ResourceProcess]
      self.res = res
      self.logger.info("response code:%s", str(self.res.status_code))
      # use charset to improve encoding detect
      self.crawled_time = time.time()
      resource = CrawledResource()
      resource.meta_content = res.meta_res
      resource.crawling_time = int((self.crawled_time - start_time) * 1000)
      resource.bps = res.content_size / resource.crawling_time * 1000
      self.logger.warn("crawling_time: %s, bps: %s", resource.crawling_time, resource.bps)
      resource.http_code = res.status_code
      self.logger.warn("headers is :%s", res.headers)
      resource.content_type = res.headers.get('Content-Type', 'text/html').split(';')[0]
      if res.encoding:
        resource.charset = res.encoding
      else:
        resource.charset = ""
      resource.html_request = self.headers
      resource.response_header = '\r\n'.join(['%s: %s' % (k, v) for k, v in res.headers.iteritems()])
      resource.last_modified = self.calcLastModified(resource, res)
      self.logger.debug("request is: %s", resource.html_request)
      self.logger.debug("response is: %s", resource.response_header)

      # updated by Oleksii
      # save cookies from response
      resource.cookies = res.cookies
      resource.dynamic_fetcher_type = res.dynamic_fetcher_type
      self.crawledResource = resource
      #(5) END

      if "content-type" in res.headers and (res.headers["content-type"] == "image/x-icon" or \
          res.headers["content-type"] == "application/pdf"):
        self.updateSite(0)
        return True
      # Added by Oleksii
      # Build DOM object
      #(6) DOM processing [???]
      parser = lxml.etree.HTMLParser(encoding='utf-8')
      self.dom = None
      if resource.http_code == 304:
        self.dom = lxml.html.fromstring("<html></html>", parser=parser)
      else:
        try:
          self.dom = lxml.html.fromstring(res.rendered_unicode_content.encode("utf-8"), parser=parser)
        except Exception, err:
          self.logger.debug("Wrong DOM model structure.")
          if self.html_recover == RECOVER_IF_FAILED:
            self.logger.debug("Try to fix DOM by tidylib.")
            tidy_content, errors = tidylib.tidy_document(res.rendered_unicode_content)
            self.logger.debug("tidylib errors: %s", str(errors))
            self.dom = lxml.html.fromstring(tidy_content, parser=parser)
      #(6) END
      # Added by Oleksii
      # support of HTTP_REDIRECTS_MAX
      #(7) Check response for redirect count [???]
      self.checkResponse()
      if not self.allow_html_redirects:
        self.updateSite(CONSTS.ERROR_MAX_ALLOW_HTML_REDIRECTS)
        return False
      if not self.allow_http_redirects:
        self.updateSite(CONSTS.ERROR_MAX_ALLOW_HTTP_REDIRECTS)
        return False
      #(7) END

      #(8) Check Content size and response code - [ResourceProcess]
      # TODO move to - [ResourceProcess]
      resource_size = res.content_size
      self.logger.debug("MaxResourceSize: %s, ResourceSize: %s " % (self.site.maxResourceSize, resource_size))
      if resource_size == 0 and resource.http_code / 100 != 3:
        resource.error_mask = CONSTS.ERROR_EMPTY_RESPONSE
        self.updateSite(CONSTS.ERROR_EMPTY_RESPONSE)
        return False
      elif self.site.maxResourceSize and resource_size > self.site.maxResourceSize:
        resource.error_mask = CONSTS.ERROR_RESPONSE_SIZE_ERROR
        self.updateSite(CONSTS.ERROR_RESPONSE_SIZE_ERROR)
        self.logger.debug("Site MaxResourceSize limit overshooted.")
        return False
      else:
        resource.html_content = res.rendered_unicode_content
        resource.binary_content = res.str_content
      if res.status_code / 100 == 4 or res.status_code / 100 == 5:
        resource.error_mask = CONSTS.ERROR_HTTP_ERROR
        self.updateSite(CONSTS.ERROR_HTTP_ERROR)
        return False
      self.addSiteSize(resource_size)
      #(8) END

    except requests.exceptions.Timeout, err:
      self.updateURLForFailed(CONSTS.ERROR_REQUEST_TIMEOUT)
      # tbi = Utils.getTracebackInfo()
      # self.logger.error(str(type(err)) + "\n")
      # self.logger.error(str(err.message) + "\n" + tbi)
      # self.logger.debug("Can't crawl resource.")
      return False
    except requests.exceptions.InvalidURL:
      self.updateURLForFailed(CONSTS.ERROR_BAD_URL)
      return False
    except requests.exceptions.TooManyRedirects:
      self.updateURLForFailed(CONSTS.ERROR_BAD_REDIRECTION)
      return False
    except requests.exceptions.ChunkedEncodingError:
      self.updateURLForFailed(CONSTS.ERROR_PAGE_CONVERT_ERROR)
      return False
    except requests.exceptions.ConnectionError:
      self.updateURLForFailed(CONSTS.ERROR_CONNECTION_ERROR)
      return False
    except requests.exceptions.ContentDecodingError:
      self.updateURLForFailed(CONSTS.ERROR_PAGE_CONVERT_ERROR)
      return False
    except lxml.etree.XMLSyntaxError:
      self.logger.warning("XML HTML syntax error")
      self.updateURLForFailed(CONSTS.ERROR_DTD_INVALID)
      return False
    except Exception, err:
      self.updateURLForFailed(CONSTS.ERROR_HTML_STRUCTURE)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.logger.error(str(type(err)) + "\n")
      self.logger.error(str(err.message) + "\n" + tbi)
      self.logger.debug("Can't crawl resource." + err.message)
      return False
    self.updateSite(CONSTS.ERROR_OK)
    return True


  # # convert date str to HTTP header format
  # 2014-07-29 20:31:50 (GMT+8) to Tue, 29 Jul 2014 12:31:50 GMT
  # @param date_str date str, 2014-07-29 20:31:50
  # @return HTTP header formated date str : Tue, 29 Jul 2014 12:31:50 GMT
  def convertToHttpDateFmt(self, date_str):
    stamp = time.mktime(date_str.timetuple())
    # stamp = time.mktime(time.strptime(date_str, '%Y-%m-%d %H:%M:%S'))
    return time.strftime('%a, %d %b %Y %H:%M:%S GMT', time.gmtime(stamp))


  # #parseRobotFile fetch the robot file for spefic url
  #
  # @param url the url to fetch(not the url of robots.txt)
  def parseRobotFile(self, url):
    robots_txt_url = ROBOTS_PATTERN.sub(r'\1/robots.txt', url)
    self.robotparser = robotparser.RobotFileParser(robots_txt_url)
    self.robotparser.read()


  # updateAVGSpeed update sites.AVGSpeed property
  def updateAVGSpeed(self):
    if self.res.status_code == 304:
      # not modified will return empty response body
      return
    newSpeed = (self.site.avgSpeed * self.site.avgSpeedCounter + self.crawledResource.bps) / \
               (self.site.avgSpeedCounter + 1)
    localSiteUpdate = dc_event.SiteUpdate(self.batch_item.siteId)
    for attr in localSiteUpdate.__dict__:
      if hasattr(localSiteUpdate, attr):
        setattr(localSiteUpdate, attr, None)
    localSiteUpdate.id = self.batch_item.siteId
    localSiteUpdate.tcDate = SQLExpression("NOW()")
    localSiteUpdate.avgSpeed = newSpeed
    localSiteUpdate.avgSpeedCounter = SQLExpression("`AVGSpeedCounter` + 1")
    self.wrapper.siteNewOrUpdate(localSiteUpdate)


  # saveCookies
  def saveCookies(self):
    self.logger.debug(MSG_INFO_STORE_COOKIES_FILE)
    self.logger.debug("Store cookies properties:\n%s", varDump(self.store_cookies))
    if self.store_cookies and self.store_cookies == "1":
      cookies_str = [key + ": " + value + "; " for (key, value) in self.crawledResource.cookies.items()]
      self.makeDir()
      self.logger.debug("Response cookies dict: %s", str(self.crawledResource.cookies))
      self.logger.debug("Response cookies string: %s", str(cookies_str))
      self.logger.debug("self.batch_item.urlId: %s", str(self.batch_item.urlId))
      self.logger.debug("self.time_postfix: %s", str(self.time_postfix))
      base_path = os.path.join(self.dir, self.batch_item.urlId + "_" + self.time_postfix)
      cookies_file_name = base_path + COOKIES_FILE_POSTFIX
      with open(cookies_file_name, "wb") as f:
        f.write(''.join(sorted(cookies_str)))


  # #updateCrwledURL
  #
  # @param site - site object
  def updateCrawledURL(self):
    state = URL.STATE_ENABLED
    updated_count = self.url.mRate * self.url.mRateCounter
    if self.crawledResource.http_code != 304:
      updated_count += 1
    #total_count = self.url["MRateCounter"] + 1
    total_count = self.url.mRateCounter + 1
    mrate = updated_count / total_count
    urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.status = URL.STATUS_CRAWLED
    urlUpdateObj.state = state
    urlUpdateObj.contentType = self.crawledResource.content_type
    urlUpdateObj.charset = self.crawledResource.charset
    urlUpdateObj.errorMask = self.crawledResource.error_mask
    urlUpdateObj.crawlingTime = self.crawledResource.crawling_time
    urlUpdateObj.totalTime = self.crawledResource.crawling_time
    urlUpdateObj.UDate = SQLExpression("NOW()")
    urlUpdateObj.httpCode = self.crawledResource.http_code
    urlUpdateObj.lastModified = self.crawledResource.last_modified
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.mRate = mrate
    urlUpdateObj.size = self.res.content_size
    urlUpdateObj.mRateCounter = SQLExpression("`MRateCounter` + 1")
    updated_rows = self.wrapper.urlUpdate(urlUpdateObj, "`Status` = 3")
    return updated_rows > 0


  # #updateURL
  #
  # @param site - site object
  def updateURL(self, status=URL.STATUS_CRAWLING):
    urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
    if not self.url.httpMethod:
      urlUpdateObj.httpMethod = "get"
    else:
      urlUpdateObj.status = status
    urlUpdateObj.status = status
    urlUpdateObj.crawled = SQLExpression("`Crawled`+1")
    urlUpdateObj.batchId = self.batch.id
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    updated_rows = self.wrapper.urlUpdate(urlUpdateObj)
    return updated_rows > 0



  # #addSiteSize update sites table to increase size
  #
  # @param size content size of this crawler
  def addSiteSize(self, size):
    localSiteUpdate = dc_event.SiteUpdate(self.batch_item.siteId)
    for attr in localSiteUpdate.__dict__:
      if hasattr(localSiteUpdate, attr):
        setattr(localSiteUpdate, attr, None)
    localSiteUpdate.id = self.batch_item.siteId
    localSiteUpdate.tcDate = SQLExpression("NOW()")
    localSiteUpdate.size = SQLExpression(("`Size` + %s" % str(size)))
    self.wrapper.siteNewOrUpdate(localSiteUpdate)


  # #prepare request headers and cookie
  #
  # @param site - site object
  # TODO move to - [SiteProcess]
  def readSiteProperties(self):
    # TODO not sure is it right that fetch headers and cookie
    # by `Name` = 'headers'/'cookie'
    self.store_http_request = True
    self.store_http_headers = True
    self.headers_dict = {}
    self.post_forms = {}
    self.headers, self.cookie = None, None
    self.proxies = None
    self.need_store_mime = '*'
    self.auth_name, self.auth_pwd = None, None
    self.external_url = None
    self.auto_remove_props = {}
    self.html_recover = None
    self.auto_detect_mime = None
    self.process_content_types = []
    headers, cookie = None, None
    proxy_host, proxy_port = None, None

    try:
      #stub
      self.site_properties = {}
      for item in self.site.properties:
        self.site_properties[item["name"]] = item["value"]
      #self.logger.debug("self.site_properties: %s", varDump(self.site_properties))
      #rows = self.executeQuery(self.dc_sites_db_connect, query)
      #self.site_properties = rows
      #self.logger.debug("self.site.properties: %s", varDump(self.site.properties))
      #self.logger.debug("self.site.properties type: %s", str(type(self.site.properties)))
      if 'HTTP_HEADERS' in self.site_properties:
        headers = self.site_properties['HTTP_HEADERS']
      if 'HTTP_COOKIE' in self.site_properties:
        cookie = self.site_properties['HTTP_COOKIE']
      if 'STORE_HTTP_REQUEST' in self.site_properties:
        self.store_http_request = int(self.site_properties['STORE_HTTP_REQUEST'])!=0
      if 'STORE_HTTP_HEADERS' in self.site_properties:
        self.store_http_headers = int(self.site_properties['STORE_HTTP_HEADERS'])!=0
      if 'HTTP_PROXY_HOST' in self.site_properties:
        proxy_host = self.site_properties['HTTP_PROXY_HOST']
      if 'HTTP_PROXY_PORT' in self.site_properties:
        proxy_port = self.site_properties['HTTP_PROXY_PORT']
      if 'MIME_TYPE_STORE_ON_DISK' in self.site_properties:
        allow_mimes = self.site_properties['MIME_TYPE_STORE_ON_DISK']
        if allow_mimes == '' or allow_mimes == '*':
          self.need_store_mime = '*'
        else:
          self.need_store_mime = set([mime.lower() for mime in allow_mimes.split(',')])
      if 'HTTP_AUTH_NAME' in self.site_properties:
        self.auth_name = self.site_properties['HTTP_AUTH_NAME']
      if 'HTTP_AUTH_PWD' in self.site_properties:
        self.auth_pwd = self.site_properties['HTTP_AUTH_PWD']
      for key in self.site_properties.keys():
        if key.startswith('HTTP_POST_FORM_'):
          self.post_forms[key[15:]] = self.site_properties[key]
      if 'EXTERNAL_FETCHER_URL' in self.site_properties:
        self.external_url = self.site_properties['EXTERNAL_FETCHER_URL']
      if 'HTML_RECOVER' in self.site_properties:
        self.html_recover = self.site_properties['HTML_RECOVER']
      if 'MITE_TYPE_AUTO_DETECT' in self.site_properties:
        self.auto_detect_mime = self.site_properties['MITE_TYPE_AUTO_DETECT']
      if 'ALLOWED_CTYPES' in self.site_properties:
        self.process_content_types = self.site_properties['ALLOWED_CTYPES'].lower().split(',')
      if 'PROCESSOR_NAME' in self.site_properties:
        self.processorName = self.site_properties['PROCESSOR_NAME']
      if DC_CONSTS.SITE_PROP_SAVE_COOKIES in self.site_properties:
        self.store_cookies = self.site_properties[DC_CONSTS.SITE_PROP_SAVE_COOKIES]
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES in self.site_properties:
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES] = \
        self.site_properties[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES]
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER in self.site_properties:
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER] = \
        self.site_properties[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER]
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE in self.site_properties:
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE] = \
        self.site_properties[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE]
      if 'HTTP_REDIRECTS_MAX' in self.site_properties:
        self.max_http_redirects = int(self.site_properties['HTTP_REDIRECTS_MAX'])
      if 'HTML_REDIRECTS_MAX' in self.site_properties:
        self.max_html_redirects = int(self.site_properties['HTML_REDIRECTS_MAX'])
      if 'URL_XPATH_LIST' in self.site_properties:
        self.url_xpath_list = json.loads(self.site_properties['URL_XPATH_LIST'])['xpath_list']
      if headers is None:
        # read request headers from crawler-task_headers.txt file
        headers = CrawlerTask.readSmallFileContent(self.default_header_file)
      if cookie is None:
        # read request cookies from crawler-task_headers.txt file
        cookie = CrawlerTask.readSmallFileContent(self.default_cookie_file)
      self.headers = headers
      if cookie and cookie.lower().startswith('cookie:'):
        self.cookie = cookie[7:]
      else:
        self.cookie = cookie

      for header in headers.splitlines():
        if not header:
          continue
        try:
          key, value = header[:header.index(':')].strip(), header[header.index(':') + 1:].strip()
        except Exception:
          self.logger.warn("header:%s", header)
        self.headers_dict[key] = value
      if self.cookie:
        self.headers_dict['Cookie'] = self.cookie

      if proxy_host and proxy_port:
        self.proxies = {'http' : 'http://%s:%s' % (proxy_host, proxy_port)}
      self.logger.info("proxies: %s", self.proxies)
    except Exception:
      self.logger.error(MSG_ERROR_LOAD_SITE_PROPERTIES, exc_info=True)


  # # readSmallFileContent read small file content
  #
  # @param path the file path to read
  @staticmethod
  def readSmallFileContent(path):
    with open(path, 'r') as f:
      return ''.join(f.readlines())


  # #update site params
  # update site params
  # @param mask ErrorMask
  # @param is_suspend should set state to suspend for this site
  def updateSite(self, mask, is_suspend=False):
    response = False
    try:
      localSiteUpdate = dc_event.SiteUpdate(self.batch_item.siteId)
      for attr in localSiteUpdate.__dict__:
        if hasattr(localSiteUpdate, attr):
          setattr(localSiteUpdate, attr, None)
      localSiteUpdate.id = self.batch_item.siteId
      localSiteUpdate.tcDate = SQLExpression("NOW()")
      if mask:
        localSiteUpdate.errors = SQLExpression("`Errors` + 1")
        localSiteUpdate.errorMask = SQLExpression(("`ErrorMask` | %s" % mask))
        if is_suspend:
          localSiteUpdate.state = Site.STATE_SUSPENDED
      updated_count = self.wrapper.siteNewOrUpdate(localSiteUpdate)
      if updated_count > 0:
        response = True
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise
    return response


  # #check site params
  # check site params
  #
  # @return should continue execute this BatchItem
  def checkSite(self):
    # TODO: self.site and self.url not must be none
    if not self.site or not self.url:
      self.logger.error("Error: wrong condition,\nself.site:%s\nself.url:%s", varDump(self.site), varDump(self.url))
      return False
    self.logger.debug("url:" + str(self.url))

    if ((self.batch.crawlerType!=dc_event.Batch.TYPE_REAL_TIME_CRAWLER) and (self.site.state!=Site.STATE_ACTIVE)) or \
      ((self.batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER) and (self.site.state==Site.STATE_DISABLED)):
      self.logger.debug("Warning: Batch CrawlerType: %s, site state is %s but not STATE_ACTIVE!"
                        % (str(self.batch.crawlerType), str(self.site.state)))
      return False

    # if not root url
    #if self.url["ParentMd5"] != "":
    if self.url.parentMd5 != "":
      # url wasn't inserted before
      #if self.url["Crawled"] == 0:
      if self.url.crawled == 0:
        # if site not recrawl
        if self.site.recrawlPeriod == 0:
          if self.site.maxErrors and self.site.errors > self.site.maxErrors:
            self.logger.debug("Errors=" + str(self.site.errors) + " limit is reached maxErrors=" +
                              str(self.site.maxErrors))
            # TODO: possible improve suspend logic
            # self.updateSite(CONSTS.ERROR_SITE_MAX_ERRORS, True)
            self.updateSite(CONSTS.ERROR_SITE_MAX_ERRORS)
            return False
          elif self.site.maxURLs and self.site.resources > self.site.maxURLs:
            self.logger.debug("Resources=" + str (self.site.resources) + " limit is reached maxURLs=" +
                              str(self.site.maxURLs))
            # TODO: possible improve suspend logic
            # self.updateSite(CONSTS.ERROR_SITE_MAX_RESOURCES, True)
            self.updateSite(CONSTS.ERROR_SITE_MAX_RESOURCES)
            return False
    
    self.updateSite(0)
    return True


  # #load url
  # the site object to crawl
  # @param site - object to crawl
  # TODO move to - [SiteProcess]
  def loadSite(self):
    try:
      if not len(self.batch_item.siteId):
        self.batch_item.siteId = "0"
      self.readSiteFromDB()
      # Check if site not exist then read site for siteId=0
      if self.site.id == SITE_MD5_EMPTY:
        self.logger.debug("Site not found. Take site id 0")
        self.batch_item.siteId = "0"
        self.batch_item.urlObj.siteId = "0"
        self.siteId = "0"
        self.readSiteFromDB()
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise


  # #load url
  # the site object to crawl
  # @param site - object to crawl
  def readSiteFromDB(self):
    siteStatus = dc_event.SiteStatus(self.batch_item.siteId)
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.SITE_STATUS, siteStatus)
    responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
    self.site = None
    self.site_table = DC_SITES_TABLE_NAME
    try:
      self.site = responseDRCESyncTasksCover.eventObject
    except Exception as err:
      self.logger.error(MSG_ERROR_READ_SITE_FROM_DB + err.message)
      raise
    #self.logger.info("site object: " + varDump(self.site))


  def resetErrorMask(self, batch_item):
    urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.errorMask = 0
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    self.wrapper.urlUpdate(urlUpdateObj)


  # #readUrlFromDB read url object from database
  #
  #
  def readUrlFromDB(self):
    self.table_name = self.url_table
    self.url = None
    urlStatus = dc_event.URLStatus(self.batch_item.siteId, self.batch_item.urlId)
    urlStatus.urlType = dc_event.URLStatus.URL_TYPE_MD5
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_STATUS, [urlStatus])
    responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
    row = responseDRCESyncTasksCover.eventObject
    self.logger.info("responseDRCESyncTasksCover: %s", row)
    try:
      if len(row):
        self.url = row[0]
        #self.logger.debug("lastModified: %s", str(row[0]['LastModified']))
      # when url come from another dc cluster's host
      else:
        if self.addURL():
          self.readUrlFromDB()
        else:
          self.logger.debug("Can't add url from another host.")
          raise CrawlerException("Can't add url from another host.")
    except CrawlerException as err:
      self.logger.debug(MSG_ERROR_LOAD_URL_DATA + err.message)
      raise
    self.logger.info("url object: " + str(self.url))


  # #load url
  # the site object to crawl
  # @param site - object to crawl
  def addURL(self):
    # self.table_name = self.url_table???
    ret = True
    try:
      # add only if site accepted
      # self.site_table = DC_SITES_TABLE_NAME
      siteStatusObj = dc_event.SiteStatus(self.siteId)
      result = self.wrapper.siteStatus(siteStatusObj)
      if result is not None:
        maxURLs = result.maxURLs
        if ((self.batch.crawlerType!=dc_event.Batch.TYPE_REAL_TIME_CRAWLER) and (result.state!=Site.STATE_ACTIVE)) or \
          ((self.batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER) and (result.state==Site.STATE_DISABLED)):
          self.logger.debug("Warning: Batch CrawlerType: %s, site state is %s but not STATE_ACTIVE!", str(self.site.state))
          raise CrawlerException("Site state is not active, state=" + str(result.state))
        if result.maxErrors!=0 and (result.errors>result.maxErrors):
          raise CrawlerException("Site maxErrors limit " + str(result.maxErrors) + " reached " + str(result.errors))
        elif maxURLs>0 and (result.resources>maxURLs):
          if self.autoRemoveURL() == 0:
            return False
            # raise CrawlerException("Site MaxURLs limit " + str(maxURLs) + " reached " + str(row[0]["Resources"]))
          else:
            self.logger.debug("Resources>maxURLs check : Resource(s) auto removed to add one more from external host!")
        
        query = "SELECT COUNT(*) FROM `%s` WHERE `Status` IN (1,2)" % self.url_table
        result = self.wrapper.customRequest(query, DB_URLS)
        if len(result) > 0 and len(result[0]) > 0:
          self.logger.debug("url_count: %s, maxURLs: %s" % (result[0][0], maxURLs))
          if maxURLs>0 and (result[0][0]>maxURLs):
            if self.autoRemoveURL() == 0:
              raise CrawlerException("URLs count " + str(result[0][0]) + \
                                     " in state 1 and 2 is grater than limit sites.MaxURLs:" + str(maxURLs))
            else:
              self.logger.debug("urls>maxURLs check : Resource(s) auto removed to add one more from external host!")
        else:
          raise CrawlerException("Execute last SQL query, no rows returned:\n" + query)
        
        self.batch_item.urlObj.CDate = str(datetime.datetime.now())
        self.batch_item.urlObj.batchId = 0 #self.batch.id
        result = self.wrapper.urlNew([self.batch_item.urlObj])
        self.logger.debug("rows_count: %s", result)
        self.updateCollectedURLs()
        # TODO: result of INSERT query must be checked, wrong state must be logged!
      else:
        raise CrawlerException("Execute last SQL query(SiteStatus), no rows returned:\n")
    except CrawlerException as err:
      self.logger.debug("Error add new url from batch (another host source): " + err.message)
      ret = False
    except Exception as err:
      self.logger.error("Error add new url from batch (another host source): " + err.message)
      ret = False
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      raise err
    return ret


  # #load url
  # the site object to crawl
  # @param site - object to crawl
  def loadURL(self):
    # do siteId
    self.url_connector = self.dc_urls_db_connect
    if len(self.batch_item.siteId):
      self.url_table = DC_URLS_TABLE_PREFIX + self.batch_item.siteId
    else:
      self.url_table = DC_URLS_TABLE_PREFIX + "0"
    self.logger.info("db backend Table for siteId %s is: %s" % (self.batch_item.siteId, self.url_table))
    self.readUrlFromDB()


  # #resetVars
  def resetVars(self):
    self.need_store_mime = None
    self.site = None
    self.robotparser = None
    self.headers_dict = None
    self.store_http_headers = None
    self.cookie = None
    self.proxies = None
    self.real_url = None
    self.url = None
    self.crawledResource = None
    self.headers = None
    self.table_name = None
    self.crawled_time = None
    self.store_http_request = None
    self.dir = None
    self.kvConnector = None
    self.kvCursor = None
    self.processorName = None
    self.site_properties = None
    self.siteId = None
    self.auto_remove_props = None
    self.store_cookies = None
    self.allow_http_redirects = True
    self.allow_html_redirects = None
    self.http_redirects = 0
    self.html_redirects = 0
    self.max_http_redirects = CONSTS.MAX_HTTP_REDIRECTS_LIMIT
    self.max_html_redirects = None
    self.dom = None
    self.time_postfix = ""
    self.res = None
    self.post_forms = None
    self.auth_name = None
    self.auth_pwd = None
    self.site_table_row = None
    self.site_properties = {}
    self.url_xpath_list = []
    self.errorMask = CONSTS.ERROR_OK
    self.feed = None
    self.feed_items = None


  # #process batch item
  # the batch item is the one from list of batch items within batch object
  # @param item - the batch item is the one from list of batch items within batch object
  def processBatchItem(self):
    self.resetVars()
    self.siteId = self.batch_item.siteId if self.batch_item.siteId else "0"
    self.logger.info("Batch item: siteId: %s, urlId: %s" % (self.batch_item.siteId, self.batch_item.urlId))
    response = self.batch_item
    try:
      # TODO move to - [SiteProcess]
      self.loadSite()
      self.logger.info("Batch: %s, URLMD5: %s, SITEID: %s, ULR: %s" % (self.batch.id, self.batch_item.urlObj.urlMd5, 
                                                                       self.siteId, self.batch_item.urlObj.url))
      
      # prepare headers and cookie
      # TODO move to - [SiteProcess]
      self.readSiteProperties()
      # TODO move to - [UrlProcess]
      self.loadURL()
      
      # TODO move to - [SiteProcess]
      if not self.checkSite():
        #self.updateURL(status=URL.STATUS_NEW)
        self.logger.debug("Site check was failed. ")
        self.batch_item = None
        return self.batch_item

      if not self.url:
        self.logger.debug("Can't load url. ")
        self.batch_item = None
        return self.batch_item

      #Check if Real-Time crawling resource already crawled
      if self.batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER and \
        self.url.crawled and \
        self.url.errorMask==CONSTS.ERROR_OK and \
        self.url.tagsCount>0 and \
        self.errorMask==CONSTS.ERROR_OK:
        self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty")
        self.logger.debug("Batch item properties: %s" % varDump(self.batch_item.properties))
        if PCONSTS.RECRAWL_KEY not in self.batch_item.properties or \
          self.batch_item.properties[PCONSTS.RECRAWL_KEY]==PCONSTS.RECRAWL_VALUE_NO:
          self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty. "+
                            "Don't need to recrawl.")
          # set contentMask stored on disk
          self.batch_item.urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
          return self.batch_item
        else:
          self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty but "+
                            "properties recrawl is <%s>. Send to recrawling." % \
                            str(self.batch_item.properties[PCONSTS.RECRAWL_KEY]))

      
      # reset error mask for current crawled url
      self.resetErrorMask(self.batch_item)

      # update state to crawling
      # TODO move to - [UrlProcess]
      self.updateURL()

      # crawl
      crawl_result = self.crawl()
      if self.crawledResource:
        # TODO move to - [UrlProcess]
        self.updateCrawledURL()
        # TODO move to - [SiteProcess]
        self.updateAVGSpeed()

      if not crawl_result:
        self.logger.debug("Crawl results was failed. ")
        self.batch_item = None
        return self.batch_item

      # not save data to disk/collect URL/collect properties if not modified
      #(1) Processing content if resource not modified
      if self.crawledResource.http_code != 304:
        detected_mime = None
        self.getDir()
        try:
          # Keep old resources on disk
          if self.keep_old_resources:
            self.time_postfix = datetime.datetime.now().strftime("%Y%m%d%H%M%S")

          detected_mime = self.writeData()
          self.saveCookies()
        except CrawlerException, cerr:
          self.logger.debug(cerr.message)
          raise
        except Exception as err:
          self.updateURLForFailed(CONSTS.ERROR_WRITE_FILE_ERROR)
          self.logger.error(MSG_ERROR_WRITE_CRAWLED_DATA + err.message, exc_info=True)
        self.collectURLs()
        # TODO move to - [UrlProcess]
        self.updateCollectTimeAndMime(detected_mime)
      #(1) END
      #self.addResource()
    except CrawlerException as err:
      self.logger.debug(MSG_ERROR_PROCESS_BATCH_ITEM)
    except Exception as err:
      self.exit_code = EXIT_FAILURE
      self.logger.error(MSG_ERROR_PROCESS_BATCH_ITEM)
      self.logger.error("xxx", exc_info=True)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)

    # Added by Oleksii
    # support PROCESSOR_NAME
    if self.processorName == NON_PROCESSING:
      self.logger.debug(MSG_DEBUG_NON_PROCESSING)
      self.batch_item = None

    self.logger.debug("Batch item:\n%s", varDump(response))
    return self.batch_item


  # #updateCollectTimeAndMime
  # add time cost of collect url to URL.CrawlingTime
  # and update Detected MIME type
  #
  #
  def updateCollectTimeAndMime(self, detected_mime):
    collect_time = int((time.time() - self.crawled_time) * 1000)
    urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.crawlingTime = SQLExpression(("`CrawlingTime` + %s" % str(collect_time)))
    urlUpdateObj.totalTime = SQLExpression(("`TotalTime` + %s" % str(collect_time)))
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    if self.auto_detect_mime == DETECT_MIME_MAIN_CONTENT:
      urlUpdateObj.contentType = str(detected_mime)
    self.wrapper.urlUpdate(urlUpdateObj)


  # #process batch
  # the main processing of the batch object
  def processBatch(self):
    try:
      # read pickled batch object from stdin and unpickle it
      input_pickled_object = sys.stdin.read()
      input_batch = pickle.loads(input_pickled_object)
      #self.logger.info("input_batch: " + varDump(input_batch))
      self.batch = input_batch
      Utils.storePickleOnDisk(input_pickled_object, ENV_CRAWLER_STORE_PATH, "crawler.in." + str(self.batch.id))
      batch_items = []
      for batch_item in self.batch.items:
        self.batch_item = batch_item
        self.processBatchItem()
        #Currently we update URL.ErrorMask for each url in batch
        if not self.batch_item:
          self.updateBatchItem(batch_item)
        else:
          self.updateBatchItem()
        if type(self.batch_item) == type([]):
          self.logger.debug("Batch item is list of items")
          for item in self.batch_item:
            batch_items.append(item)
        else:
          self.logger.debug("Batch item is not list")
          batch_items.append(self.batch_item)
      self.logger.debug("batch_items: " + str(batch_items))
      if input_batch.crawlerType == dc_event.Batch.TYPE_REAL_TIME_CRAWLER:
        process_task_batch = input_batch
        process_task_batch.items = self.store_items
      else:
        process_task_batch = Batch(input_batch.id, batch_items)
      output_pickled_object = pickle.dumps(process_task_batch)
      Utils.storePickleOnDisk(output_pickled_object, ENV_CRAWLER_STORE_PATH, "crawler.out." + str(self.batch.id))
      sys.stdout.write(output_pickled_object)
      sys.stdout.flush()
    except Exception, err:
      self.logger.error("Crawler has failed. %s", err.message)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
    
    
  ##updateBatchItem
  # updateBatchItem
  def updateBatchItem(self, batch_item=None):
    #update batchItem properties
    # RSS feed items -> batch items
    if (self.processorName==PCONSTS.PROCESSOR_FEED_PARSER) and self.feed_items is not None:
      self.batch_item = self.createBatchItems()
      #self.batch_item.properties[dc_event.BatchItem.PROP_FEED] = self.feed_items
      #self.logger.debug("Batch property: %s" % varDump(self.batch_item.properties))
    elif self.url is not None:
      batch_item = self.batch_item if not batch_item else batch_item
      self.logger.debug("Before: batch_item urlObj errorMask: %s, url ErrorMask: %s" % (batch_item.urlObj.errorMask, 
                                                                                        self.url.errorMask))
      batch_item.urlObj.errorMask = self.url.errorMask
      self.logger.debug("After: batch_item urlObj errorMask: %s, url ErrorMask: %s" % (batch_item.urlObj.errorMask, 
                                                                                       self.url.errorMask))
    self.store_items.append(batch_item)


  # #createBatchItems
  #
  def createBatchItems(self):
    items = []
    for urlMd5, value in self.feed_items.items():
      self.logger.debug("URLMD5: %s" % str(urlMd5))
      self.logger.debug("value: %s" % str(value))
      siteId = self.batch_item.siteId
      urlObj = value["urlObj"]
      
      #serialize content
      value.pop("urlObj", None)
      from time import gmtime, strftime

      # list of objects must be converted to string before json dumps
      dates = ["published_parsed", "updated_parsed"]
      for date in dates:
        if date in value["entry"]:
          value["entry"][date] = strftime("%a, %d %b %Y %H:%M:%S +0000", value["entry"][date])
      value["urlMd5"] = urlMd5

      #store raw content on disk
      try:
        self.crawledResource = CrawledResource()
        self.crawledResource.binary_content = json.dumps(value)
        self.batch_item.urlId = urlMd5
        self.getDir()
        self.writeData()
      except Exception, err:
        self.logger.debug("Can't save object on disk. Reason: %s" % str(err.message))
        continue

      urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
      batch_item = dc_event.BatchItem(siteId, urlMd5, urlObj)
      items.append(batch_item)
    return items


  # #load config from file
  # load from cli argument or default config file
  def loadConfig(self):
    try:
      self.config = ConfigParser.ConfigParser()
      self.config.optionxform = str
      if self.pargs.config:
        self.config.read(self.pargs.config)
    except:
      print MSG_ERROR_LOAD_CONFIG
      raise


  # #load db backend
  # load from cli argument or default config file
  def loadDBBackend(self):
    try:
      className = self.__class__.__name__
      dbHost = self.config.get(className, self.DB_HOST)
      dbPort = int(self.config.get(className, self.DB_PORT))
      dbUser = self.config.get(className, self.DB_USER)
      dbPWD = self.config.get(className, self.DB_PWD)

      db_dc_sites = self.config.get(className, self.DB_SITES)
      db_dc_urls = self.config.get(className, self.DB_URLS)

      self.dc_sites_db_connect = mdb.connect(dbHost, dbUser, dbPWD, db_dc_sites, dbPort, charset="utf8",
                                             use_unicode=True)
      self.dc_urls_db_connect = mdb.connect(dbHost, dbUser, dbPWD, db_dc_urls, dbPort, charset="utf8", use_unicode=True)
      # self.dc_sites_db_connect.set_character_set("utf8")
      # self.dc_urls_db_connect.set_character_set("utf8")
      self.site_connector = self.dc_sites_db_connect
      self.url_connector = self.dc_urls_db_connect
      self.site_table = None
      self.url_table = None
    except Exception, err:
      self.logger.error("Error load config param: %s" % err.message)
      raise


  # #loadKeyValueDB load key-value db
  #
  #
  def loadKeyValueDB(self):
    try:
      className = self.__class__.__name__
      self.kvDbDir = self.config.get(className, self.DB_DATA_DIR)
    except Exception, err:
      self.logger.error("Error load config param: %s" % err.message)
      raise


  # prepare sqlite DB connector and cursor object
  #
  #
  def prepareKvDbConnector(self):
    if hasattr(self, "kvConnector") and self.kvConnector:
      self.kvConnector.close()
    dbFile = os.path.join(self.kvDbDir, "%s_fields.db" % (self.siteId,))
    self.kvConnector = sqlite3.connect(dbFile)
    self.kvConnector.row_factory = sqlite3.Row
    self.kvConnector.text_factory = unicode
    self.kvCursor = self.kvConnector.cursor()


  # #load logging
  # load logging configuration (log file, log level, filters)
  #
  def loadLogConfigFile(self):
    try:
      log_conf_file = self.config.get("Application", "log")
      logging.config.fileConfig(log_conf_file)
      self.logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
      # Replace logger name for processes instances
      lfn = Utils.LoggerFileName()
      fname = lfn.findReplace()
      if fname != "":
        pin = lfn.getFreeProcInstanceNumber(os.path.basename(fname))
        #self.logger.info("LOGGER>>>>: %s, %s, %s", str(fname), pin, os.path.basename(fname))
        if pin != "" and pin != "0":
          pin = "." + pin + ".log"
          fname = lfn.findReplace(fname + pin)
          self.logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
          #self.logger.info("LOGGER>>>>: %s", str(lfn.findReplace()))
    except:
      print MSG_ERROR_LOAD_LOG_CONFIG_FILE
      raise


  # #load mandatory options
  # load mandatory options
  #
  def loadOptions(self):
    try:
      self.raw_data_dir = self.config.get(self.__class__.__name__, "raw_data_dir")
      self.default_header_file = self.config.get(self.__class__.__name__, "headers_file")
      self.default_cookie_file = self.config.get(self.__class__.__name__, "cookie_file")
      self.default_icr_crawl_time = self.config.getfloat(self.__class__.__name__, "default_icr_crawl_time")
      db_task_ini = self.config.get(self.__class__.__name__, "db-task_ini")

      # Add support operations updateCollectedURLs and removeURLs
      cfgParser = ConfigParser.ConfigParser()
      cfgParser.read(db_task_ini)
      self.wrapper = DBTasksWrapper(cfgParser)
    
      # does call collectAddtionalProp and collectProperties methods
      self.max_fetch_time = self.config.getint(self.__class__.__name__, "max_fetch_time")
      
      # keep old resources on disk
      if self.config.has_option(self.__class__.__name__, "keep_old_resources"):
        self.keep_old_resources = self.config.getboolean(self.__class__.__name__, "keep_old_resources")
      self.collect_additional_prop = self.config.getboolean(self.__class__.__name__, "collect_additional_prop")
    except Exception, err:
      self.logger.error("Error load config param: %s" % err.message)
      raise


  # #Auto remove resources
  # Removes resources in condition based on dc_sites.sites_properties items
  #
  # @return number of URLs
  def autoRemoveURL(self):
    ret = 0
    try:
      self.logger.debug("Auto remove properties:\n%s", varDump(self.auto_remove_props))
      # If defined auto remove properties???
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES in self.auto_remove_props and\
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES] != "" and\
        DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE in self.auto_remove_props and\
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE] != "" and\
        DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER in self.auto_remove_props and\
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER] != "":
        # Select candidates to remove
        # CHANGE -> (Custom request)
        query = "SELECT Site_Id, URLMd5 FROM %s WHERE %s ORDER BY %s LIMIT %s" % (self.url_table,
                self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE].replace("%RecrawlPeriod%",
                                                                                      str(self.site.recrawlPeriod)),
                self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER],
                self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES])
        self.logger.debug("SQL to select auto remove candidates: %s", query)
        result = self.wrapper.customRequest(query, DB_URLS)
        if len(result) > 0:
          urlsToDelete = []
          for row in result:
            # Create new URLDelete object
            urlDelete = EventObjects.URLDelete(row[0], row[1], EventObjects.URLStatus.URL_TYPE_MD5,
                                               reason=dc_event.URLDelete.REASON_CRAWLER_AUTOREMOVE)
            urlsToDelete.append(urlDelete)
            self.logger.debug("URL added to auto remove URLMd5:[%s]", row[1])
          drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_DELETE, urlsToDelete)
          responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
          self.logger.debug("Response from db-task module on URLDelete operation:\n%s", \
                            varDump(responseDRCESyncTasksCover))
          ret = len(result)
        else:
          self.logger.debug("No auto remove candidates or SQL query error!")
      else:
        self.logger.debug("No mandatory auto remove properties in auto_remove_props:\n" + \
                          varDump(self.auto_remove_props))
    except Exception, err:
      self.logger.error("Error of auto remove operation: %s", str(err.message))

    return ret

  # #checkResponse
  #
  def checkResponse(self):
    self.logger.debug("Requests response history: " + str(self.res.redirects))
    # calculate http redirect
    if HTTP_REDIRECT  in self.res.redirects:
      self.http_redirects = self.http_redirects + 1
      self.logger.debug("http redirect: " + str(self.http_redirects))
      self.logger.debug("http max redirect: " + str(self.max_http_redirects))
    # calculate html redirect
    if self.dom is not None and bool(self.dom.xpath(META_XPATH)):
      self.html_redirects = self.html_redirects + 1
      self.logger.debug("html redirect: " + str(self.html_redirects))
      self.logger.debug("html max redirect: " + str(self.max_html_redirects))
    # check http redirect
    if self.max_http_redirects and self.max_http_redirects != MAX_HTTP_REDIRECTS_UNLIMITED and \
    self.http_redirects >= self.max_http_redirects:
      self.allow_http_redirects = False
      self.logger.debug("http redirect limit was reached! Max http redirects: %s, encountered http redirects: %s." %
                        (str(self.max_http_redirects), str(self.http_redirects)))
    else:
      self.allow_http_redirects = True
    # check html redirect
    if self.max_html_redirects and int(self.max_html_redirects) != MAX_HTML_REDIRECTS_UNLIMITED and \
    int(self.html_redirects) >= int(self.max_html_redirects):
      self.allow_html_redirects = False
      self.logger.debug("html redirect limit was reached! Max html redirects: %s, encountered html redirects: %s." %
                        (str(self.max_html_redirects), str(self.html_redirects)))
    else:
      self.allow_html_redirects = True


  # #calcLastModified
  #
  def calcLastModified(self, resource, res):
    if resource.http_code == 304:
      resource.last_modified = self.url.tcDate
      #resource.last_modified = self.url["TcDate"]
    elif 'Last-Modified' in res.headers:
      resource.last_modified = res.headers['Last-Modified']
      resource.last_modified = parse(resource.last_modified).strftime('%Y-%m-%d %H:%M:%S')
    elif 'Date' in res.headers:
      resource.last_modified = res.headers['Date']
      resource.last_modified = parse(resource.last_modified).strftime('%Y-%m-%d %H:%M:%S')
    else:
      resource.last_modified = time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime(time.time() - self.default_icr_crawl_time))
    self.logger.debug("LastModified date:" + str(resource.last_modified))
    return str(resource.last_modified)


  # #updateCollectedURLs
  #
  def updateCollectedURLs(self):
    self.logger.debug("updateCollectedURLs")
    self.wrapper.fieldRecalculating(self.siteId)


  # #get exit code
  # return extt code
  # 0 if no critical errors was encountered during processing
  # 1 if smth bad happened
  #
  def getExitCode(self):
    return self.exit_code


# #RedirectHistory model,
# contains status code and response headers of one redirect response
class RedirectHistory(object):


  # #constructor
  # initialize fields
  def __init__(self, status_code, response_header):
    self.status_code = status_code
    self.response_header = response_header

