# coding: utf-8

"""
  Natual Language auto detector
  support multiple detectors, but strongly recommon langid
  @file LangDetector.py
  @author Madk <developers.hce@gmail.com>
  @link http://hierarchical-cluster-engine.com/
  @copyright Copyright &copy; 2014 IOIX Ukraine
  @license http://hierarchical-cluster-engine.com/license/
  @package HCE project node API
  @since 0.1
"""

import logging
import random
import guess_language
import detectlanguage
import langid
import app.Consts as APP_CONSTS

CONFIG_SECTION = "LangDetector"
CONFIG_DETECTOR = "detector"
CONFIG_DETECTLANGUAGE_APIKEYS = "detectlanguage_apikeys"
CONFIG_RETRY_NUM = "retry_number"
CONFIG_DETECT_MAX_LENGTH = "max_length"
DETECTOR_GUESS_LANGUAGE = "guess_language"
DETECTOR_DETECTLANGUAGE = "detectlanguage"
DETECTOR_LANGID = "langid"

logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

detect_func = None
detectlanguage_apikeys = None

max_length = None

retry_num = 3

'''
init the LangDetector module
available detectors:
  1. langid (recommended)
  2. guess_language 
  3. detectlanguage
'''
def init(config):
  global detect_func
  global detectlanguage_apikeys
  global max_length
  global retry_num

  max_length = int(config.get(CONFIG_SECTION, CONFIG_DETECT_MAX_LENGTH))
  detector = config.get(CONFIG_SECTION, CONFIG_DETECTOR)
  if detector == DETECTOR_GUESS_LANGUAGE:
    detect_func = _guess_language_detect
  elif detector == DETECTOR_DETECTLANGUAGE:
    detect_func = _detectlang_detect
    detectlanguage_apikeys = config.get(CONFIG_SECTION, CONFIG_DETECTLANGUAGE_APIKEYS).split(",")
    if config.has_option(CONFIG_SECTION, CONFIG_RETRY_NUM):
      retry_num = int(config.get(CONFIG_SECTION, CONFIG_RETRY_NUM))
  elif detector == DETECTOR_LANGID:
    detect_func = _langid_detect
  else:
    logger.warn("cann't find appropriate language detector for %s", detector)


'''
detect language for given content

for better performance, will truncate content if size too much
@param content unicode content
@return language
'''
def detect(content):
  if not content:
    return
  if not isinstance(content, unicode):
    raise None
  length = len(content)
  if length >= max_length:
    content = content[(length-max_length)/2 : (length-max_length)/2 + max_length]   

  if detect_func is None:
    logger.warn("no appropriate detecotor")
    return None
  result = detect_func(content)
  if not isinstance(result, list):
    return [result]
  return result


def _guess_language_detect(content):
	return guess_language.guessLanguage(content)

def _langid_detect(content):
  return langid.classify(content)[0]

def _detectlang_detect(content, retry = None):
  if retry is None:
    retry = retry_num
  try:
    # detectlanguage.configuration.api_key = "a31d934a633d86cfe5a401340d98fb52"
    detectlanguage.configuration.api_key = random.choice(detectlanguage_apikeys)
    result = detectlanguage.detect(content)
    if isinstance(result, list):
      return [item['language'].encode('utf-8') for item in result]
    else:
      return result['language'].encode('utf-8')
  except detectlanguage.DetectLanguageError:
    if retry > 0:
      return _detectlang_detect(content, retry - 1)
    raise
