"""
@package: dc
@file URLProcess.py
@author Scorp <developers.hce@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""
import datetime
import logging
import MySQLdb as mdb
import re
import requests
import time
import urllib
import urlparse
import CrawlerTask
import app.Utils as Utils
import app.Consts as APP_CONSTS
import dc.EventObjects
import dc.Constants as DC_CONSTS
import dc_processor.Constants as PCONSTS
from dc_crawler.CrawlerException import CrawlerException
from app.Utils import SQLExpression
from time import strftime

logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


class URLProcess(object):

  DC_URLS_TABLE_PREFIX = "urls_"
  DETECT_MIME_TIMEOUT = 1
  PATTERN_WITH_PROTOCOL = re.compile('[a-zA-Z]+:(//)?')
  URL_TEMPLATE_CONST = "%URL%"

  def __init__(self):
    self.isUpdateCollection = False
    self.urlObj = None
    self.url = None
    self.dbWrapper = None
    self.siteId = None
    self.urlTable = None


  # #checkClassFields method checks all class's mandatory fields
  #
  def checkClassFields(self, checkList):
    for field in self.__dict__:
      if field in checkList and (not hasattr(self, field) or getattr(self, field) is None):
        raise Exception(">>> [CollectURLs] Mandatory field must be initialized, field Name = " + field)


  def resolveTableName(self, localSiteId):
    self.urlTable = self.DC_URLS_TABLE_PREFIX + localSiteId
    return self.urlTable


  @staticmethod
  def createURLObj(siteId, url):
    ret = dc.EventObjects.URL(siteId, mdb.escape_string(url))
    return ret


  # #readCurrentCnt return urls cnt from db
  #
  def readCurrentCnt(self, maxURLs):
    self.checkClassFields(["dbWrapper", "urlTable"])
    currentCnt = 0
    if maxURLs:
      countsql = "SELECT COUNT(*) AS cnt FROM `%s` WHERE NOT (Status=4 AND Crawled=0 AND Processed=0)" % \
                  (self.urlTable,)
      result = self.dbWrapper.customRequest(countsql, CrawlerTask.DB_URLS)
      if len(result) > 0 and len(result[0]) > 0:
        currentCnt = result[0][0]
      else:
        currentCnt = 0
    return currentCnt



  # #processURL processed incoming url and added it to the one of incoming lists [internalLinks or externalLinks]
  #
  def processURL(self, realUrl, internalLinks, externalLinks):
    self.checkClassFields(["urlObj"])
    retUrl = None
    retContinue = False
    if self.urlObj.type == dc.EventObjects.URL.TYPE_SINGLE or not self.url:
      logger.debug("URL type is TYPE_SINGLE - not collect urls. Skip url.")
      retContinue = True
    logger.debug("URL: %s", self.url)
    #TODO: must be replaced with more general check on supported protocols list for this site
    if not retContinue and self.PATTERN_WITH_PROTOCOL.match(self.url) and \
     not self.url.lower().startswith("http://") and not self.url.lower().startswith("https://"):
      retContinue = True

    if not retContinue:
      if not self.url.lower().startswith("http://") and not self.url.lower().startswith("https://"):
        self.url = urlparse.urljoin(realUrl, self.url)
        # normalization
        retUrl = dc.EventObjects.URL(0, self.url).getURL()
        internalLinks.append(retUrl)
        logger.info("internal")
      else:
        # normalization
        retUrl = dc.EventObjects.URL(0, self.url).getURL()
        if Utils.parseHost(retUrl) == Utils.parseHost(realUrl):
          internalLinks.append(retUrl)
          logger.info("internal")
        elif Utils.parseHost(retUrl):
          externalLinks.append(retUrl)
          logger.info("external")
        else:  # not valid url like http://
          retContinue = True
    return retUrl, retContinue



  # #isUrlExist slelect localUrl from dc_urls db if it exist
  #
  def isUrlExist(self, recrawlPeriod, siteProperties, urlMd5):
    self.checkClassFields(["url", "dbWrapper", "siteId", "urlTable"])
    ret = False
    if "RECRAWL_URL_AGE_EXPRESSION" in siteProperties and siteProperties["RECRAWL_URL_AGE_EXPRESSION"] != "":
      ageExpr = siteProperties["RECRAWL_URL_AGE_EXPRESSION"].replace("%RECRAWL_PERIOD%", str(recrawlPeriod))
    else:
      ageExpr = "(DATE_ADD(UDate, INTERVAL %s MINUTE)-NOW())" % (str(recrawlPeriod))
    query = "SELECT COUNT(*), %s, `Type` FROM `%s` WHERE `URLMd5` = '%s'" % \
            (ageExpr, self.urlTable, urlMd5)
    result = self.dbWrapper.customRequest(query, CrawlerTask.DB_URLS)
    if len(result) > 0 and len(result[0]) > 0 and result[0][0] > 0:
      if recrawlPeriod == 0 or \
      (len(result[0]) > 1 and result[0][1] > 0) or \
      (result[0][2] == dc.EventObjects.URL.TYPE_FETCHED) or \
      ("RECRAWL_NO_ROOT_URLS" in siteProperties and siteProperties["RECRAWL_NO_ROOT_URLS"] == "0"):
        logger.debug("URL skipped, exists and re-crawling not active, time not reached or URL Type is " +
                     "RSS feed (not to fetch)\n %s %s", self.url, urlMd5)
        ret = True
      else:
        self.updateURLFields(urlMd5, self.dbWrapper, self.siteId, siteProperties)
        logger.debug("URL state updated to NEW because re-crawling\n %s %s", self.url, urlMd5)
        ret = True
    else:
      logger.debug("URL %s treated as new\n %s", self.url, urlMd5)

    return ret



  # #update some fields in dc_urls
  #
  #urlMd5 param - urls md5
  #wrapper param - db_db wrapper
  #siteId param - site's id
  def updateURLFields(self, urlMd5, wrapper, siteId, siteProperties):
    urlUpdateObj = dc.EventObjects.URLUpdate(siteId, urlMd5, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    #Status field
    if "RECRAWL_URL_UPDATE_STATUS" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_STATUS"] != "-1":
      urlUpdateObj.status = int(siteProperties["RECRAWL_URL_UPDATE_STATUS"])
    else:
      if "RECRAWL_URL_UPDATE_STATUS" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_STATUS"] == "-1":
        urlUpdateObj.status = None
      else:
        urlUpdateObj.status = dc.EventObjects.URL.STATUS_NEW
    #TcDate field
    if "RECRAWL_URL_UPDATE_TCDATE" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_TCDATE"] != "":
      urlUpdateObj.tcDate = siteProperties["RECRAWL_URL_UPDATE_TCDATE"]
    else:
      if "RECRAWL_URL_UPDATE_TCDATE" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_TCDATE"] == "":
        urlUpdateObj.tcDate = None
      else:
        urlUpdateObj.tcDate = SQLExpression("NOW()")
    #CDate field
    if "RECRAWL_URL_UPDATE_CDATE" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_CDATE"] != "":
      urlUpdateObj.CDate = siteProperties["RECRAWL_URL_UPDATE_CDATE"]
    #UDate field
    if "RECRAWL_URL_UPDATE_UDATE" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_UDATE"] != "":
      urlUpdateObj.UDate = siteProperties["RECRAWL_URL_UPDATE_UDATE"]
    else:
      if "RECRAWL_URL_UPDATE_UDATE" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_UDATE"] == "":
        urlUpdateObj.UDate = None
      else:
        urlUpdateObj.UDate = SQLExpression("NOW()")

    wrapper.urlUpdate(urlUpdateObj, "`State`=0")



  # # detect mime type for an URL using HEAD method
  #
  # @return mime_type detected mime type, empty string if failed
  def detectUrlMime(self):
    self.checkClassFields(["url"])
    ret = ''
    try:
      res = requests.head(self.url, timeout=self.DETECT_MIME_TIMEOUT)
      ret = res.headers.get('content-type', '').lower()
    except BaseException:
      logger.warn("detect mime type for %s failed", self.url, exc_info=True)
    return ret


  # #getDepthFromUrl method
  #
  #urlMd5 - url's md5
  def getDepthFromUrl(self, urlMd5):
    self.checkClassFields(["dbWrapper", "siteId"])
    ret = 0
    urlStatusObj = dc.EventObjects.URLStatus(self.siteId, urlMd5)
    result = self.dbWrapper.urlStatus(urlStatusObj, True)
    if len(result) > 0 and type(result[0]) == dc.EventObjects.URL:
      ret = result[0].depth
    return ret


  # #updateURLForFailed
  #
  # @param errorBit BitMask of error
  # @param batchItem incoming batchItem instance
  def updateURLForFailed(self, errorBit, batchItem, httpCode=None):
    self.checkClassFields(["dbWrapper", "siteId"])
    logger.info(">>> ERROR bit = " + str(errorBit))
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.status = dc.EventObjects.URL.STATUS_CRAWLED
    urlUpdateObj.errorMask = errorBit
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    if httpCode is not None:
      urlUpdateObj.httpCode = httpCode
    self.dbWrapper.urlUpdate(urlUpdateObj)


  # #getRealUrl
  #
  def getRealUrl(self):
    self.checkClassFields(["url"])
    if self.url.startswith("http%3A") or self.url.startswith("https%3A"):
      ret = urllib.unquote(self.url.url).decode('utf-8')
    else:
      ret = self.url.decode('utf8')
    return ret


  # #resolveHTTP method resolves http method, and fill headersDict["If-Modified-Since"] element
  #
  # @param postForms incoming post forms
  # @param headersDict incoming headersDict dict, where filled element with "If-Modified-Since" key
  # return None or filled postData
  def resolveHTTP(self, postForms, headersDict):
    self.checkClassFields(["urlObj"])
    postData = None
    try:
      method = self.urlObj.httpMethod.lower()
    except Exception:
      method = "get"
    if method == "post":
      postData = postForms
      logger.info("use post, post_data:%s", postData)
    else:
      logger.info("last modified: <<%s>>", str(type(self.urlObj.lastModified)))
      logger.info("last modified: <<%s>>", str((self.urlObj.lastModified)))
      if str(self.urlObj.lastModified) != "None" and str(self.urlObj.lastModified) != "NULL":
        logger.info("If-Modified-Since: <<%s>>", self.urlObj.lastModified)
        logger.info("If-Modified-Since type: <<%s>>", str(type(self.urlObj.lastModified)))
        headersDict["If-Modified-Since"] = \
        Utils.convertToHttpDateFmt(datetime.datetime.strptime(str(self.urlObj.lastModified), "%Y-%m-%d %H:%M:%S"))
    return postData


  # #updateCrwledURL
  #
  # @param site - site object
  def updateCrawledURL(self, crawledResource, batchItem, contentSize):
    self.checkClassFields(["urlObj", "dbWrapper", "siteId"])
    logger.info(">>> Start urls update")
    state = dc.EventObjects.URL.STATE_ENABLED
    updatedCount = self.urlObj.mRate * self.urlObj.mRateCounter
    if crawledResource.http_code != 304:
      updatedCount += 1
    mrate = updatedCount / (self.urlObj.mRateCounter + 1)
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.status = dc.EventObjects.URL.STATUS_CRAWLED
    urlUpdateObj.state = state
    urlUpdateObj.contentType = crawledResource.content_type
    urlUpdateObj.charset = crawledResource.charset
    urlUpdateObj.errorMask = crawledResource.error_mask
    urlUpdateObj.crawlingTime = crawledResource.crawling_time
    urlUpdateObj.totalTime = crawledResource.crawling_time
    urlUpdateObj.UDate = SQLExpression("NOW()")
    urlUpdateObj.httpCode = crawledResource.http_code
    urlUpdateObj.lastModified = crawledResource.last_modified
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.mRate = mrate
    urlUpdateObj.size = contentSize
    urlUpdateObj.mRateCounter = SQLExpression("`MRateCounter` + 1")
    updatedRowsCount = self.dbWrapper.urlUpdate(urlUpdateObj, "`Status` = 3")
    return updatedRowsCount > 0


  # #updateURL
  #
  # @param batchItem - incoming batchItem
  # @param batchId - base batchId
  # @param status - value for status field updating
  def updateURL(self, batchItem, batchId, status=dc.EventObjects.URL.STATUS_CRAWLING):
    self.checkClassFields(["urlObj", "dbWrapper", "siteId"])
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    if not self.urlObj.httpMethod:
      urlUpdateObj.httpMethod = "get"
    else:
      urlUpdateObj.status = status
    urlUpdateObj.status = status
    urlUpdateObj.crawled = SQLExpression("`Crawled`+1")
    urlUpdateObj.batchId = batchId
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    updatedRowsCount = self.dbWrapper.urlUpdate(urlUpdateObj)
    return updatedRowsCount > 0


  # #resetErrorMask reset url's errorMask, using db-task wrapper
  #
  # @param batchItem - incoming batchItem
  def resetErrorMask(self, batchItem):
    self.checkClassFields(["dbWrapper", "siteId"])
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.errorMask = 0
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    self.dbWrapper.urlUpdate(urlUpdateObj)


  # #addURL makes some checks and makes URLNew operation
  #
  # @param batchItem - incoming BatchItem object
  # @param crawlerType - url's crawlingType, we have used for internal checks
  # @param recrawlPeriod - url's recrawlPeriod, we have used for internal checks
  # @param autoRemoveProps - param send into URLProcess.autoRemoveURL method call
  def addURL(self, batchItem, crawlerType, recrawlPeriod, autoRemoveProps):
    self.checkClassFields(["dbWrapper", "siteId", "urlTable"])
    ret = True
    try:
      siteStatusObj = dc.EventObjects.SiteStatus(Utils.autoFillSiteId(self.siteId))
      result = self.dbWrapper.siteStatus(siteStatusObj)
      if result is not None:
        maxURLs = result.maxURLs
        if ((crawlerType != dc.EventObjects.Batch.TYPE_REAL_TIME_CRAWLER) and \
        (result.state != dc.EventObjects.Site.STATE_ACTIVE)) or \
        ((crawlerType == dc.EventObjects.Batch.TYPE_REAL_TIME_CRAWLER) and \
        (result.state == dc.EventObjects.Site.STATE_DISABLED)):
          logger.debug("Warning: Batch CrawlerType: %s, site state is %s but not STATE_ACTIVE!", crawlerType,
                       str(result.state))
          raise CrawlerException("Site state is not active, state=" + str(result.state))
        if result.maxErrors != 0 and (result.errors > result.maxErrors):
          raise CrawlerException("Site maxErrors limit " + str(result.maxErrors) + " reached " + str(result.errors))
        elif maxURLs > 0 and (result.resources > maxURLs):
          if URLProcess.autoRemoveURL(autoRemoveProps, recrawlPeriod, self.urlTable, self.dbWrapper) == 0:
            raise CrawlerException("Site MaxURLs limit " + str(maxURLs) + " reached ")
          else:
            logger.debug("Resources>maxURLs check : Resource(s) auto removed to add one more from external host!")

        query = "SELECT COUNT(*) FROM `%s` WHERE `Status` IN (1,2)" % self.urlTable
        result = self.dbWrapper.customRequest(query, CrawlerTask.DB_URLS)
        if len(result) > 0 and len(result[0]) > 0:
          logger.debug("url_count: " + str(result[0][0]) + ", maxURLs: " + str(maxURLs))
          if maxURLs > 0 and (result[0][0] > maxURLs):
            if URLProcess.autoRemoveURL(autoRemoveProps, recrawlPeriod, self.urlTable, self.dbWrapper) == 0:
              raise CrawlerException("URLs count " + str(result[0][0]) + \
                                     " in state 1 and 2 is grater than limit sites.MaxURLs:" + str(maxURLs))
            else:
              logger.debug("urls>maxURLs check : Resource(s) auto removed to add one more from external host!")
        else:
          raise CrawlerException("Execute last SQL query, no rows returned:\n" + query)

        batchItem.urlObj.CDate = str(datetime.datetime.now())
        batchItem.urlObj.batchId = 0  #self.batch.id
        result = self.dbWrapper.urlNew([batchItem.urlObj])
        logger.debug("rows_count: %s", result)
        self.isUpdateCollection = True
        #self.updateCollectedURLs()
      else:
        raise CrawlerException("Execute last SQL query(SiteStatus), no rows returned:\n")
    except CrawlerException as err:
      logger.debug("Error add new url from batch (another host source): " + err.message)
      ret = False
    except Exception as err:
      logger.error("Error add new url from batch (another host source): " + err.message)
      ret = False
      tbi = Utils.getTracebackInfo()
      logger.error(str(vars(err)) + "\n" + tbi)
      raise err
    return ret


  # #Auto remove resources
  # Removes resources in condition based on dc_sites.sites_properties items
  #
  # @param autoRemoveProps - list of autoremove properties
  # @param recrawlPeriod - used only for replacing in autoRemoveProps field
  # @param urlTable - currently table in dc_urls db
  # @param wrapper - db_task wrapper
  # @return number of URLs
  @staticmethod
  def autoRemoveURL(autoRemoveProps, recrawlPeriod, urlTable, wrapper):
    ret = 0
    try:
      #logger.debug("Auto remove properties:\n%s", varDump(autoRemoveProps))
      # If defined auto remove properties???
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES in autoRemoveProps and\
        autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES] != "" and\
        DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE in autoRemoveProps and\
        autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE] != "" and\
        DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER in autoRemoveProps and\
        autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER] != "":
        # Select candidates to remove
        # CHANGE -> (Custom request)
        query = "SELECT Site_Id, URLMd5 FROM %s WHERE %s ORDER BY %s LIMIT %s" % (urlTable,
                autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE].replace("%RecrawlPeriod%",
                                                                                      str(recrawlPeriod)),
                autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER],
                autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES])
        logger.debug("SQL to select auto remove candidates: %s", query)
        result = wrapper.customRequest(query, CrawlerTask.DB_URLS)
        if len(result) > 0:
          urlsToDelete = []
          for row in result:
            # Create new URLDelete object
            urlDelete = dc.EventObjects.URLDelete(row[0], row[1], dc.EventObjects.URLStatus.URL_TYPE_MD5,
                                                  reason=dc.EventObjects.URLDelete.REASON_CRAWLER_AUTOREMOVE)
            urlsToDelete.append(urlDelete)
            logger.debug("URL added to auto remove URLMd5:[%s]", row[1])
          drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_DELETE, urlsToDelete)
          responseDRCESyncTasksCover = wrapper.process(drceSyncTasksCoverObj)
          logger.debug("Response from db-task module on URLDelete operation:\n%s", \
                            Utils.varDump(responseDRCESyncTasksCover))
          ret = len(result)
        else:
          logger.debug("No auto remove candidates or SQL query error!")
      else:
        logger.debug("No mandatory auto remove properties in auto_remove_props:\n" + Utils.varDump(autoRemoveProps))
    except Exception as err:
      logger.error("Error of auto remove operation: %s", str(err.message))
    return ret


  # #updateCollectTimeAndMime
  # add time cost of collect url to URL.CrawlingTime
  # and update Detected MIME type
  #
  # @param detectedMime - mine type from current page
  # @param batchItem - incoming batch item
  # @param crawledTime - datatime used for filling some fields
  # @param autoDetectMime - used for check detectedMime param before contentType field updating
  def updateCollectTimeAndMime(self, detectedMime, batchItem, crawledTime, autoDetectMime):
    self.checkClassFields(["dbWrapper", "siteId"])
    collectTime = int((time.time() - crawledTime) * 1000)
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.crawlingTime = SQLExpression(("`CrawlingTime` + %s" % str(collectTime)))
    urlUpdateObj.totalTime = SQLExpression(("`TotalTime` + %s" % str(collectTime)))
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    if autoDetectMime == CrawlerTask.DETECT_MIME_MAIN_CONTENT and detectedMime is not None:
      urlUpdateObj.contentType = str(detectedMime)
    self.dbWrapper.urlUpdate(urlUpdateObj)


  # #readUrlFromDB read url object from database
  #
  # @param batchItem - incoming batch item
  # @param crawlerType - url's crawlingType, we have used for internal checks
  # @param recrawlPeriod - url's recrawlPeriod, we have used for internal checks
  # @param autoRemoveProps - list of autoremove properties
  def readUrlFromDB(self, batchItem, crawlerType, recrawlPeriod, autoRemoveProps):
    self.urlObj = None
    self.checkClassFields(["dbWrapper", "siteId"])
    self.isUpdateCollection = False
    urlStatus = dc.EventObjects.URLStatus(self.siteId, batchItem.urlId)
    urlStatus.urlType = dc.EventObjects.URLStatus.URL_TYPE_MD5
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_STATUS, [urlStatus])
    responseDRCESyncTasksCover = self.dbWrapper.process(drceSyncTasksCoverObj)
    row = responseDRCESyncTasksCover.eventObject
    logger.info("responseDRCESyncTasksCover: %s", row)
    try:
      if len(row):
        self.urlObj = row[0]
        #self.logger.debug("lastModified: %s", str(row[0]['LastModified']))
      # when url come from another dc cluster's host
      else:
        if self.addURL(batchItem, crawlerType, recrawlPeriod, autoRemoveProps):
          self.readUrlFromDB(batchItem, crawlerType, recrawlPeriod, autoRemoveProps)
        else:
          logger.debug("Can't add url from another host.")
          raise CrawlerException("Can't add url from another host.")
    except CrawlerException as err:
      logger.debug("Can't load retUrl data: " + err.message)
      raise
    logger.info("retUrl object: " + str(self.urlObj))
    return self.urlObj


  # #updateAdditionProps update some fields for URL (makes URLUpdate operation)
  #
  # @param internalLinksCount - count of internal links from page
  # @param externalLinksCount - count of enternal links from page
  # @param batchItem - incoming batch item
  # @param size - value for .size field update
  # @param freq - value for .freq field update
  # @param contentMd5 - value for .contentMd5 field update
  def updateAdditionProps(self, internalLinksCount, externalLinksCount, batchItem, size, freq, contentMd5):
    self.checkClassFields(["dbWrapper", "siteId"])
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.size = size
    urlUpdateObj.linksI = internalLinksCount
    urlUpdateObj.linksE = externalLinksCount
    urlUpdateObj.freq = freq
    urlUpdateObj.rawContentMd5 = contentMd5
    self.dbWrapper.urlUpdate(urlUpdateObj)


  # #createUrlObjForCollectURLs - creates and returns urlObj for collectedURL operation
  #
  # @param urlMd5 - url's md5
  # @param formMethods - formMethods post/get form data
  # @param urlId - parent URL md5
  # @param depth - url's depth in site structure
  # @param detectedMime -
  # @param maxURLsFromPage
  def createUrlObjForCollectURLs(self, urlMd5, formMethods, urlId, depth, detectedMime, maxURLsFromPage):
    self.checkClassFields(["url", "siteId", "urlObj"])
    ret = dc.EventObjects.URL(self.siteId, mdb.escape_string(self.url))
    ret.type = self.urlObj.type
    ret.urlMd5 = urlMd5
    ret.requestDelay = self.urlObj.requestDelay
    ret.httpTimeout = self.urlObj.httpTimeout
    ret.httpMethod = formMethods.get(self.url, "get")
    ret.parentMd5 = urlId
    ret.maxURLsFromPage = maxURLsFromPage
    ret.tcDate = SQLExpression("NOW()")
    ret.UDate = SQLExpression("NOW()")
    ret.depth = (depth + 1)
    ret.contentType = detectedMime
    ret.priority = self.urlObj.priority
    return ret


  # #fillRssFieldInUrlObj - fills some fields, related with rss, in internal urlObj
  #
  # @param oldUrl - incoming previous ulr
  # @param objectUrlUlr - incoming url object
  # @param batchItem - incoming bachItem
  # @param processorName - processor name
  # @param feed - incoming source of rss data
  # @return dict with filled rss values
  def fillRssFieldInUrlObj(self, oldUrl, objectUrlUlr, batchItem, processorName, feed):
    self.checkClassFields(["url", "siteId", "urlObj"])
    ret = None
    contentType = None
    pubdate = None
    status = dc.EventObjects.URL.STATUS_CRAWLED
    crawled = 1
    localType = dc.EventObjects.URL.TYPE_FETCHED
    if processorName == PCONSTS.PROCESSOR_RSS:
      status = dc.EventObjects.URL.STATUS_NEW
      crawled = 0
      localType = dc.EventObjects.URL.TYPE_SINGLE
    for entry in feed.entries:
      if entry.link == oldUrl:
        ret = {}
        ret["entry"] = entry
        ret["urlObj"] = dc.EventObjects.URL(self.siteId, self.url)
        ret["parent_rss_feed"] = objectUrlUlr
        ret["parent_rss_feed_urlMd5"] = batchItem.urlId
        #pubdate
        ret["pubdate"] = None
        dates = ["published_parsed", "updated_parsed"]
        for date in dates:
          if date in entry:
            try:
              ret["pubdate"] = strftime("%Y-%m-%d %H:%M:%S", entry[date])
            except TypeError:
              logger.debug("Unsupported date format: <%s>", Utils.varDump(entry[date]))
        pubdate = ret["pubdate"]
        contentType = entry.links[0].type
        size = len(str(ret))
        self.urlObj.status = status
        self.urlObj.crawled = crawled
        self.urlObj.contentType = contentType
        self.urlObj.pDate = pubdate
        self.urlObj.type = localType
        self.urlObj.size = size
    return ret


  # #urlTemplateApply - method applies url's template for current url
  #
  # @param url - incoming url
  # @param crawlerType - crawler's type
  # @param urlTempalteRegular - url template, that applying in regularCrawl case
  # @param urlTempalteRealtime - url template, that applying in realTimeCrawl case
  # @param urlTempalteRegularEncode - make url enconing for regularCrawl or not
  # @param urlTempalteRealtimeEncode - make url enconing for realTimeCrawl or not
  # @return processed url, after url template applying
  def urlTemplateApply(self, url, crawlerType, urlTempalteRegular, urlTempalteRealtime, urlTempalteRegularEncode,
                       urlTempalteRealtimeEncode):
    ret = url
    if crawlerType == dc.EventObjects.Batch.TYPE_REAL_TIME_CRAWLER:
      if urlTempalteRealtime is not None:
        try:
          if urlTempalteRealtimeEncode is not None and bool(int(urlTempalteRealtimeEncode)):
            encodedUrl = urllib.quote(url)
          else:
            encodedUrl = url
        except ValueError:
          encodedUrl = url
        ret = urlTempalteRealtime.replace(self.URL_TEMPLATE_CONST, encodedUrl)
    else:
      if urlTempalteRegular is not None:
        try:
          if urlTempalteRegularEncode is not None and bool(int(urlTempalteRegularEncode)):
            encodedUrl = urllib.quote(url)
          else:
            encodedUrl = url
        except ValueError:
          encodedUrl = url
        ret = urlTempalteRegular.replace(self.URL_TEMPLATE_CONST, encodedUrl)
    if ret != url:
      logger.debug(">>> url was replaced ")
      logger.debug(">>> new url = " + ret)
    return ret
