'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

import logging
import dc.EventObjects
import Constants
import app.SQLCriterions
from dtm.EventObjects import GeneralResponse
from BaseTask import BaseTask
from SiteTask import SiteTask
from URLDeleteTask import URLDeleteTask

logger = logging.getLogger(Constants.LOGGER_NAME)

##sql query which checks existence of a table
#CHECK_TABLE_SQL_TEMPLATE = " SELECT COUNT(*) FROM sites WHERE Id = '%s'"

#@todo move to apropriate place
TASK_NOT_EXIST_ERR = 2020
TASK_NOT_EXISTS_ERR_MSG = "Site Id Not Found"\


class SiteUpdateTask(SiteTask):


  ##constructor
  #
  #@param dcSiteTemplate path to sql template for dc_urls_* tables
  def __init__(self, dcSiteTemplate, keyValueDefaultFile, keyValueStorageDir, rawDataDir, dBDataTask, dcStatTemplate,
               dcLogTemplate):
    super(SiteUpdateTask, self).__init__(dcSiteTemplate, keyValueDefaultFile, keyValueStorageDir, dBDataTask,
                                         dcStatTemplate, dcLogTemplate)
    self.urlDeleteTask = URLDeleteTask(keyValueStorageDir, rawDataDir, dBDataTask)


  ##make all necessary actions to update site into in mysql db
  #
  #@param siteUpdate instance of SiteUpdate object
  #@param queryCallback function for queries execution
  #@return generalResponse instance of GeneralResponse object
  def process(self, siteUpdate, queryCallback):
    siteIds = []
    response = GeneralResponse()
    if siteUpdate.id is None:
      siteIds.extend(self.fetchByCriterions(siteUpdate.criterions, queryCallback))
    else:
      siteIds.append(siteUpdate.id)

    for siteId in siteIds:
      siteUpdate.id = siteId
      if super(SiteUpdateTask, self).isSiteExist(siteUpdate.id, queryCallback):
        self.updateSite(siteUpdate, queryCallback)
        self.updateSitesFilter(siteUpdate, queryCallback)
        self.updateSiteProperties(siteUpdate, queryCallback)
        self.updateSiteURL(siteUpdate, queryCallback)
        self.restartSite(siteUpdate, queryCallback)
        sitePropValue = BaseTask.readValueFromSiteProp(siteUpdate.id, "UPDATE_NOT_INSERT_ROOT_URLS", queryCallback)
        if ((sitePropValue is None or not bool(int(sitePropValue))) and siteUpdate.urls is not None):
        #TODO
          logger.debug(">>> SiteUpdateTask start moving root urls")
          self.deleteRootUrlsUrls(siteUpdate.id, queryCallback)
          self.moveURLsFromSiteURlsToDcURLs(siteUpdate.id, queryCallback)
        #TODO END
        response.statuses.append(GeneralResponse.ERROR_OK)
      else:
        response.statuses.append(TASK_NOT_EXIST_ERR)

    return response


  ## update data in sites table
  #
  #@param siteUpdate instance of SiteUpdate object
  #@param queryCallback function for queries execution
  def updateSite(self, siteUpdate, queryCallback):
    SQL_UPDATE_SITE_SQL_TEMPLATE = "UPDATE sites SET %s WHERE `Id` = '%s'"
    fields, values = Constants.getFieldsValuesTuple(siteUpdate, Constants.siteDict)
    fieldValueString = Constants.createFieldsValuesString(fields, values, Constants.siteExcludeList)
    query = SQL_UPDATE_SITE_SQL_TEMPLATE % (fieldValueString, siteUpdate.id)
    queryCallback(query, Constants.PRIMARY_DB_ID)


  ##update data in sites_filters table
  #
  #@param siteUpdate instance of SiteUpdate object
  #@param queryCallback function for queries execution
  def updateSitesFilter(self, siteUpdate, queryCallback):
    if siteUpdate.filters is not None:
      if siteUpdate.updateType == dc.EventObjects.SiteUpdate.UPDATE_TYPE_APPEND:
        super(SiteUpdateTask, self).addSitesFilter(siteUpdate, queryCallback)
      if siteUpdate.updateType == dc.EventObjects.SiteUpdate.UPDATE_TYPE_OVERWRITE:
        query = Constants.DEL_BY_ID_QUERY_TEMPLATE % ("sites_filters", siteUpdate.id)
        queryCallback(query, Constants.PRIMARY_DB_ID)
        super(SiteUpdateTask, self).addSitesFilter(siteUpdate, queryCallback)
      if siteUpdate.updateType == dc.EventObjects.SiteUpdate.UPDATE_TYPE_UPDATE:
        super(SiteUpdateTask, self).updateSitesFilter(siteUpdate, queryCallback)


  ##update data in sites_properties table
  #
  #@param siteUpdate instance of SiteUpdate object
  #@param queryCallback function for queries execution
  def updateSiteProperties(self, siteUpdate, queryCallback):
    if siteUpdate.properties is not None:
      if siteUpdate.updateType == dc.EventObjects.SiteUpdate.UPDATE_TYPE_APPEND:
        super(SiteUpdateTask, self).addSiteProperties(siteUpdate, queryCallback)
      if siteUpdate.updateType == dc.EventObjects.SiteUpdate.UPDATE_TYPE_OVERWRITE:
        query = Constants.DEL_BY_ID_QUERY_TEMPLATE % ("sites_properties", siteUpdate.id)
        queryCallback(query, Constants.PRIMARY_DB_ID)
        super(SiteUpdateTask, self).addSiteProperties(siteUpdate, queryCallback)
      if siteUpdate.updateType == dc.EventObjects.SiteUpdate.UPDATE_TYPE_UPDATE:
        super(SiteUpdateTask, self).updateSiteProperties(siteUpdate, queryCallback)


  ##update data in urls_ table
  #
  #@param siteUpdate instance of SiteUpdate object
  #@param queryCallback function for queries execution
  def updateSiteURL(self, siteUpdate, queryCallback):
    if siteUpdate.urls is not None:
      site = dc.EventObjects.Site("")
      site.urls = siteUpdate.urls
      site.id = siteUpdate.id
      if siteUpdate.updateType == dc.EventObjects.SiteUpdate.UPDATE_TYPE_APPEND:
        super(SiteUpdateTask, self).addSiteURLSites(site, queryCallback)
      if siteUpdate.updateType == dc.EventObjects.SiteUpdate.UPDATE_TYPE_OVERWRITE:
        URL_TABLE_DEL_TEMPLATE = "DELETE FROM `sites_urls` WHERE `Site_Id` = '%s'"
        query = URL_TABLE_DEL_TEMPLATE % siteUpdate.id
        queryCallback(query, Constants.PRIMARY_DB_ID)
        super(SiteUpdateTask, self).addSiteURLSites(site, queryCallback)
      if siteUpdate.updateType == dc.EventObjects.SiteUpdate.UPDATE_TYPE_UPDATE:
        super(SiteUpdateTask, self).updateSiteURLSites(siteUpdate, queryCallback)


  ##update data in sites_urls and sites tables
  #
  #@param siteUpdate instance of SiteUpdate object
  #@param queryCallback function for queries execution
  def restartSite(self, siteUpdate, queryCallback):
    if siteUpdate.state == siteUpdate.STATE_RESTART:
      UPDATE_SITE_SQL_QUERY = ("UPDATE `sites` SET `State`=%s, `Udate`=NOW(0), " +
      "`Iterations`=`Iterations`+1, `ErrorMask`=0, `Errors`=0 WHERE id='%s'")
      query = UPDATE_SITE_SQL_QUERY % (str(dc.EventObjects.Site.STATE_ACTIVE), siteUpdate.id)
      queryCallback(query, Constants.PRIMARY_DB_ID)

      siteUpdate.urls = []
      SELECT_QUERY = "SELECT `URL` FROM `sites_urls` WHERE `Site_Id` = '%s'"
      query = SELECT_QUERY % siteUpdate.id
      res = queryCallback(query, Constants.PRIMARY_DB_ID)
      if hasattr(res, '__iter__'):
        for row in res:
          siteUpdate.urls.append(dc.EventObjects.URL(siteUpdate.id, row[0]))
      super(SiteUpdateTask, self).addSiteURLURLs(siteUpdate, queryCallback)


  ##deleteRootUrlsUrls method deletes all root urls from dc_urls dadabases[siteId's table]
  #
  #@param siteId current siteId
  #@param queryCallback function for queries execution
  def deleteRootUrlsUrls(self, siteId, queryCallback):
    urlDeleteObj = dc.EventObjects.URLDelete(siteId, None,
                                             reason=dc.EventObjects.URLDelete.REASON_SITE_UPDATE_ROOT_URLS)
    whereCriterionStr = \
          "`URLMd5` IN (SELECT `URLMd5` FROM dc_sites.sites_urls WHERE `Site_Id` = '%s')" % siteId
    urlDeleteObj.criterions = {app.SQLCriterions.CRITERION_WHERE : whereCriterionStr}
    self.urlDeleteTask.process([urlDeleteObj], queryCallback)


  ##moveURLsFromSiteURlsToDcURLs copies all root urls from sites_urls to the dc_urls
  #
  #@param siteId current siteId
  #@param queryCallback function for queries execution
  def moveURLsFromSiteURlsToDcURLs(self, siteId, queryCallback):
    if len(Constants.URLTableDict) > 0:
      tbName = Constants.DC_URLS_TABLE_NAME_TEMPLATE % siteId
      query = "INSERT INTO `%s` ("
      query = query % tbName
      fields = ''
      for elem in Constants.URLTableDict.values():
        fields += elem
        fields += ','

      fields = fields[:-1]
      query += fields
      query += (") SELECT ")
      query += fields
      query += (" FROM dc_sites.sites_urls WHERE `Site_Id`='%s' GROUP BY `URLMd5`" % siteId)

      queryCallback(query, Constants.SECONDARY_DB_ID)
