'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''
import logging
import MySQLdb.cursors
import MySQLdb as mdb
import dc.EventObjects as dc_event
import Constants
import ConfigParser
from contextlib import closing
from dc.Constants import EVENT_TYPES, DRCESyncTasksCover
from SiteTask import SiteTask
from SiteCleanUpTask import SiteCleanUpTask
from SiteDeleteTask import SiteDeleteTask
from SiteUpdateTask import SiteUpdateTask
from SiteStatusTask import SiteStatusTask
from SiteFindTask import SiteFindTask
from URLNewTask import URLNewTask
from URLStatusTask import URLStatusTask
from URLUpdateTask import URLUpdateTask
from URLFetchTask import URLFetchTask
from URLCleanupTask import URLCleanUpTask
from URLDeleteTask import URLDeleteTask
from URLContentTask import URLContentTask
from SQLCustomTask import SQLCustomTask
from URLPurgeTask import URLPurgeTask
from FieldRecalculatorTask import FieldRecalculatorTask
from URLVerifyTask import URLVerifyTask
from URLAgeTask import URLAgeTask
from DBDataTask import DBDataTask
from URLPutTask import URLPutTask
from Constants import LOGGER_NAME
from StatisticLogManager import StatisticLogManager

logger = logging.getLogger(LOGGER_NAME)

# #dict to check event
eventCheckDict = dict({EVENT_TYPES.SITE_NEW: dc_event.Site,
                     EVENT_TYPES.SITE_UPDATE: dc_event.SiteUpdate,
                     EVENT_TYPES.SITE_STATUS: dc_event.SiteStatus,
                     EVENT_TYPES.SITE_DELETE: dc_event.SiteDelete,
                     EVENT_TYPES.SITE_CLEANUP: dc_event.SiteCleanup,
                     EVENT_TYPES.SITE_FIND: dc_event.SiteFind,
                     EVENT_TYPES.URL_NEW: list,
                     EVENT_TYPES.URL_UPDATE: list,
                     EVENT_TYPES.URL_STATUS: list,
                     EVENT_TYPES.URL_DELETE: list,
                     EVENT_TYPES.URL_FETCH: list,
                     EVENT_TYPES.URL_CLEANUP: list,
                     EVENT_TYPES.URL_CONTENT: list,
                     EVENT_TYPES.SQL_CUSTOM: dc_event.CustomRequest,
                     EVENT_TYPES.URL_PURGE: list,
                     EVENT_TYPES.FIELD_RECALCULATE: list,
                     EVENT_TYPES.URL_VERIFY: list,
                     EVENT_TYPES.URL_AGE: list,
                     EVENT_TYPES.URL_PUT: list})

# #sql query which checks existence of a table
CHECK_TABLE_SQL_TEMPLATE = " SELECT COUNT(*) FROM `sites` WHERE Id = '%s'"


# LOGGER_NAME = "db_task"

class TasksManager(object):
  # Configuration settings options names
  DB_HOST = "db_host"
  DB_PORT = "db_port"
  DB_USER = "db_user"
  DB_PWD = "db_pwd"
  PRIMARY_DB = "primary_db"
  SECONDARY_DB = "secondary_db"
  THIRD_DB = "third_db"
  FOURTH_DB = "fourth_db"
  FIFTH_DB = "fifth_db"
  STAT_DB = "stat_db"
  LOG_DB = "log_db"
  SITE_TEMPLATE = "dc_site_template"
  KEY_VALUE_STORAGE_DIR = "key_value_storage_dir"
  KEY_VALUE_DEFAULT_FILE = "key_value_default_file"
  RAW_DATA_DIR = "raw_data_dir"
  CONTENT_STORAGE_TYPE = "content_storage_type"
  CONTENT_TEMPLATE = "dc_content_template"
  STAT_TEMPLATE = "dc_statistics_freq_template"
  LOG_TEMPLATE = "dc_statistics_log_template"
  SITE_PROP_NAME_FREQ = "stats_freq_enabled"
  SITE_PROP_NAME_LOG = "stats_log_enabled"

  # #constructor
  #
  # @param configParser config parser object
  def __init__(self, configParser):
    className = self.__class__.__name__
    # Get configuration settings
    dbHost = configParser.get(className, self.DB_HOST)
    dbPort = int(configParser.get(className, self.DB_PORT))
    dbUser = configParser.get(className, self.DB_USER)
    dbPWD = configParser.get(className, self.DB_PWD)
    dbPrimary = configParser.get(className, self.PRIMARY_DB)
    dbSecondary = configParser.get(className, self.SECONDARY_DB)
    dbThird = configParser.get(className, self.THIRD_DB)
    dbFourth = configParser.get(className, self.FOURTH_DB)
    dbFifth = configParser.get(className, self.FIFTH_DB)
    dbStat = configParser.get(className, self.STAT_DB)
    dbLog = configParser.get(className, self.LOG_DB)
    self.contentStorageType = int(configParser.get(className, self.CONTENT_STORAGE_TYPE))
    self.dbConnections = {}
    self.dbNames = {}

    self.dcSiteTemplate = configParser.get(className, self.SITE_TEMPLATE)
    self.dcStatTemplate = configParser.get(className, self.STAT_TEMPLATE)
    self.dcLogTemplate = configParser.get(className, self.LOG_TEMPLATE)
    self.keyValueStorageDir = configParser.get(className, self.KEY_VALUE_STORAGE_DIR)
    self.rawDataDir = configParser.get(className, self.RAW_DATA_DIR)
    self.keyValueDefaultFile = configParser.get(className, self.KEY_VALUE_DEFAULT_FILE)
    self.dcContentTemplate = configParser.get(className, self.CONTENT_TEMPLATE)
    self.dbConnections[Constants.PRIMARY_DB_ID] = mdb.connect(dbHost, dbUser, dbPWD, dbPrimary, dbPort)
    self.dbNames[dbPrimary] = Constants.PRIMARY_DB_ID
    self.dbConnections[Constants.SECONDARY_DB_ID] = mdb.connect(dbHost, dbUser, dbPWD, dbSecondary, dbPort)
    self.dbNames[dbSecondary] = Constants.SECONDARY_DB_ID
    if dbThird is not None and dbThird != '':
      self.dbConnections[Constants.THIRD_DB_ID] = mdb.connect(dbHost, dbUser, dbPWD, dbThird, dbPort)
      self.dbNames[dbThird] = Constants.THIRD_DB_ID
    else:
      logger.debug(">>> THIRD_DB_ID empty")
    if dbFourth is not None and dbFourth != '':
      self.dbConnections[Constants.FOURTH_DB_ID] = mdb.connect(dbHost, dbUser, dbPWD, dbFourth, dbPort)
      self.dbNames[dbFourth] = Constants.FOURTH_DB_ID
    else:
      logger.debug(">>> FOURTH_DB_ID empty")
    if dbFifth is not None and dbFifth != '':
      self.dbConnections[Constants.FIFTH_DB_ID] = mdb.connect(dbHost, dbUser, dbPWD, dbFifth, dbPort)
      self.dbNames[dbFifth] = Constants.FIFTH_DB_ID
    else:
      logger.debug(">>> FIFTH_DB_ID empty")
    self.dbConnections[Constants.STAT_DB_ID] = mdb.connect(dbHost, dbUser, dbPWD, dbStat, dbPort)
    self.dbNames[dbStat] = Constants.STAT_DB_ID
    self.dbConnections[Constants.LOG_DB_ID] = mdb.connect(dbHost, dbUser, dbPWD, dbLog, dbPort)
    self.dbNames[dbLog] = Constants.LOG_DB_ID
    self.SQLErrorCode = Constants.EXIT_CODE_OK
    self.SQLErrorString = ""
    self.dBDataTask = DBDataTask(self.contentStorageType, self.keyValueDefaultFile, self.dcContentTemplate, 
                                 self.keyValueStorageDir, self.rawDataDir)
    try:
      tmpVal = int(configParser.get(className, self.SITE_PROP_NAME_FREQ))
      StatisticLogManager.GLOBAL_FREQ_ENABLED = bool(tmpVal)
    except ConfigParser.NoOptionError:
      pass
    try:
      tmpVal = int(configParser.get(className, self.SITE_PROP_NAME_LOG))
      StatisticLogManager.GLOBAL_LOG_ENABLED = bool(tmpVal)
    except ConfigParser.NoOptionError:
      pass


  # #destructor
  #
  def __del__(self):
    for dbConnect in self.dbConnections.values():
      if dbConnect is not None:
        dbConnect.close()


  # #process input event
  #
  # @param drceSyncTasksCover instance of DRCESyncTasksCover
  # @return generalResponse instance of GeneralResponse object
  def process(self, drceSyncTasksCover):

    self.checkInputData(drceSyncTasksCover)
    responseObject = None
    responseEvent = None
    if drceSyncTasksCover.eventType == EVENT_TYPES.SITE_DELETE:
      siteDeleteTask = SiteDeleteTask(self.keyValueStorageDir, self.rawDataDir, self.dBDataTask)
      responseObject = siteDeleteTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_DELETE_RESPONSE
      self.SQLErrorCode = Constants.EXIT_CODE_OK

    elif drceSyncTasksCover.eventType == EVENT_TYPES.SITE_NEW:
      siteTask = SiteTask(self.dcSiteTemplate, self.keyValueDefaultFile, self.keyValueStorageDir, self.dBDataTask,
                          self.dcStatTemplate, self.dcLogTemplate)
      responseObject = siteTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_NEW_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.SITE_UPDATE:
      siteUpdateTask = SiteUpdateTask(self.dcSiteTemplate, self.keyValueDefaultFile, self.keyValueStorageDir,
                                      self.rawDataDir, self.dBDataTask, self.dcStatTemplate, self.dcLogTemplate)
      responseObject = siteUpdateTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_UPDATE_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.SITE_CLEANUP:
      siteCleanUpTask = SiteCleanUpTask(self.keyValueStorageDir, self.rawDataDir, self.dBDataTask)
      responseObject = siteCleanUpTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_CLEANUP_RESPONSE
      self.SQLErrorCode = Constants.EXIT_CODE_OK

    elif drceSyncTasksCover.eventType == EVENT_TYPES.SITE_STATUS:
      siteStatusTask = SiteStatusTask()
      responseObject = siteStatusTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_STATUS_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_NEW:
      localSiteTask = SiteTask(self.dcSiteTemplate, self.keyValueDefaultFile, self.keyValueStorageDir, self.dBDataTask,
                          self.dcStatTemplate, self.dcLogTemplate)
      urlNewTask = URLNewTask(self.keyValueStorageDir, self.rawDataDir, self.dBDataTask, localSiteTask)
      urlNewTask.siteTask = SiteTask(self.dcSiteTemplate, self.keyValueDefaultFile, self.keyValueStorageDir, 
                                     self.dBDataTask, self.dcStatTemplate, self.dcLogTemplate)
      responseObject = urlNewTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_NEW_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_STATUS:
      urlStatusTask = URLStatusTask()
      responseObject = urlStatusTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_STATUS_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_UPDATE:
      urlUpdateTask = URLUpdateTask(self.keyValueStorageDir, self.rawDataDir, self.dBDataTask)
      responseObject = urlUpdateTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_UPDATE_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_FETCH:
      urlFetchTask = URLFetchTask(self.keyValueStorageDir, self.rawDataDir, self.dBDataTask, self.dcSiteTemplate,
                                  self.keyValueDefaultFile, self.dcStatTemplate, self.dcLogTemplate)
      responseObject = urlFetchTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_FETCH_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_CONTENT:
      urlContentTask = URLContentTask(self.keyValueStorageDir, self.rawDataDir, self.contentStorageType, 
                                      self.dBDataTask)
      responseObject = urlContentTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_CONTENT_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_DELETE:
      urlDeleteTask = URLDeleteTask(self.keyValueStorageDir, self.rawDataDir, self.dBDataTask)
      responseObject = urlDeleteTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_DELETE_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_CLEANUP:
      urlCleanUpTask = URLCleanUpTask(self.keyValueStorageDir, self.rawDataDir, self.dBDataTask)
      responseObject = urlCleanUpTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_CLEANUP_RESPONSE

    # find site by root url and return list of siteId's with root urls
    elif drceSyncTasksCover.eventType == EVENT_TYPES.SITE_FIND:
      siteFindTask = SiteFindTask(self.dcSiteTemplate, self.keyValueDefaultFile, self.keyValueStorageDir,
                                  self.dBDataTask, self.dcStatTemplate, self.dcLogTemplate)
      responseObject = siteFindTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_FIND_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.SQL_CUSTOM:
      sqlCustomTask = SQLCustomTask()
      responseObject = sqlCustomTask.process(drceSyncTasksCover.eventObject, self.executeQuery, self.backDBResolve)
      responseEvent = EVENT_TYPES.SQL_CUSTOM_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_PURGE:
      urlPurgeTask = URLPurgeTask(self.keyValueStorageDir, self.rawDataDir, self.dBDataTask)
      responseObject = urlPurgeTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_PURGE_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.FIELD_RECALCULATE:
      fieldRecalculatorTask = FieldRecalculatorTask()
      responseObject = fieldRecalculatorTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.FIELD_RECALCULATE_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_VERIFY:
      urlVarifyTask = URLVerifyTask()
      responseObject = urlVarifyTask.process(drceSyncTasksCover.eventObject, self.executeQuery, self.backDBResolve)
      responseEvent = EVENT_TYPES.URL_VERIFY_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_AGE:
      urlAgeTask = URLAgeTask(self.keyValueStorageDir, self.rawDataDir, self.backDBResolve)
      responseObject = urlAgeTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_AGE_RESPONSE

    elif drceSyncTasksCover.eventType == EVENT_TYPES.URL_PUT:
      urlPutTask = URLPutTask(self.keyValueStorageDir, self.rawDataDir, self.dBDataTask)
      responseObject = urlPutTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_PUT_RESPONSE

    return self.createDRCESyncTasksCover(responseEvent, responseObject)


  # #executeQuery common entry point for SQL execution
  #
  # @param query query for execution
  # @param dbConnectionName MySQL connection Id
  # @executeType executeType - return set type
  # @returns fetching data or None
  def executeQuery(self, query, dbConnectionName, executeType=Constants.EXEC_INDEX, SQLErrorClear=False):
    ret = None
    if executeType == Constants.EXEC_INDEX:
      ret = self.executeQueryByIndex(query, dbConnectionName, SQLErrorClear)
    elif executeType == Constants.EXEC_NAME:
      ret = self.executeQueryByName(query, dbConnectionName, SQLErrorClear)
    return ret


  # #method performs common sql error handling
  #
  # @param err SQL exception class instance
  # @param dbConnectionName incoming dbConnection name
  # @param SQLErrorClear mean set errorCode or not
  def commonQueryErrorHandler(self, err, dbConnection, SQLErrorClear=False):
    dbConnection.rollback()
    self.SQLErrorString = "%s %s" % (err.args[0], err.args[1])
    if not SQLErrorClear:
      logger.error(self.SQLErrorString)
      self.SQLErrorCode = Constants.EXIT_CODE_MYSQL_ERROR
    else:
      logger.debug(self.SQLErrorString)


  # #helper function for correct query execution
  #
  # @param query query for execution
  # @param dbConnectionName incoming dbConnection name
  # @param SQLErrorClear mean set errorCode or not
  def executeQueryByIndex(self, query, dbConnectionName, SQLErrorClear=False):
    ret = None
    dbConnection = None
    if dbConnectionName in self.dbConnections:
      dbConnection = self.dbConnections[dbConnectionName]
    if dbConnection:
      try:
        with closing(dbConnection.cursor()) as cursor:
          logger.info(query)
          cursor.execute(query)
          dbConnection.commit()
          ret = cursor.fetchall()
      except mdb.Error as err:
        self.commonQueryErrorHandler(err, dbConnection, SQLErrorClear)
    else:
      logger.debug(">>> dbConnection Not Found = " + str(dbConnectionName))
    return ret


  # #helper function for correct query execution
  #
  # @param query query for execution
  # @param dbConnectionName incoming dbConnection name
  # @param SQLErrorClear mean set errorCode or not
  def executeQueryByName(self, query, dbConnectionName, SQLErrorClear=False):
    ret = None
    dbConnection = None
    if dbConnectionName in self.dbConnections:
      dbConnection = self.dbConnections[dbConnectionName]
    if dbConnection:
      try:
        with closing(dbConnection.cursor(MySQLdb.cursors.DictCursor)) as cursor:
          logger.info(query)
          cursor.execute(query)
          dbConnection.commit()
          ret = cursor.fetchall()
      except mdb.Error as err:
        self.commonQueryErrorHandler(err, dbConnection, SQLErrorClear)
    else:
      logger.debug(">>> dbConnection Not Found = " + str(dbConnectionName))
    return ret


  # #check  conformity data in input DRCESyncTasksCover object
  #
  # @param drceSyncTasksCover instance of DRCESyncTasksCover
  def checkInputData(self, drceSyncTasksCover):
    for inputType in eventCheckDict:
#      logger.info("eventCheckDict: "+ str(vars(eventCheckDict)))
#      logger.info("inputType: "+ str(vars(inputType)))
#      logger.info("drceSyncTasksCover.eventType: "+ str(drceSyncTasksCover.eventType))
      if drceSyncTasksCover.eventType == inputType and \
      isinstance(drceSyncTasksCover.eventObject, eventCheckDict[inputType]):
        return True
    raise Exception("Incorrect types of  input data")


  # #helper function to build response as DRCESyncTasksCover object
  #
  # @param eventType type of response event
  # @param eventObject instance of returning object
  def createDRCESyncTasksCover(self, eventType, eventObject):
    return DRCESyncTasksCover(eventType, eventObject)


  # #gets dbName and returns internar db index
  #
  # @param dbName db name
  def backDBResolve(self, dbName):
    ret = None
    if dbName in self.dbNames:
      ret = self.dbNames[dbName]
    return ret
    
