'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

import glob
import logging
import os
import Constants
import dc.EventObjects
import dc.Constants as DC_CONSTANTS
import sqlite3
import URLFetchTask
import base64
import DBDataTask
from app.Utils import PathMaker
from Constants import LOGGER_NAME
from datetime import datetime

logger = logging.getLogger(LOGGER_NAME)

##process urlContent event
class URLContentTask(object):


  ##constructor
  #
  #@param keyValueStorageDir path to keyValue storage work dir
  #@param rawDataDir path to raw data dir
  #@param dBDataTask instance of DBDataTask module
  def __init__(self, keyValueStorageDir, rawDataDir, contentStorageType, dBDataTask):
    self.keyValueStorageDir = keyValueStorageDir
    self.rawDataDir = rawDataDir
    self.processedContents = []
    self.rawContents = []
    self.headers = []
    self.requests = []
    self.meta = []
    self.cookies = []
    self.contentMask = None
    self.dbDataTask = dBDataTask


  ##method clears main contants
  def clearContents(self):
    self.processedContents = []
    self.rawContents = []
    self.headers = []
    self.requests = []
    self.meta = []
    self.cookies = []


  ##make all necessary actions to get urls content data from storages
  #
  #@param urlContentRequests list of URLContentRequest objects
  #@param queryCallback function for queries execution
  #@return list of URLContentResponse objects
  def process(self, urlContentRequests, queryCallback):  # pylint: disable=W0613
    urlContentResponses = []
    #@todo add more complex case, implemented only from rawDataDir
    for urlContentRequest in urlContentRequests:
      if urlContentRequest is None:
        urlContentResponses.append(None)
      elif hasattr(urlContentRequest, "urlFetch") and urlContentRequest.urlFetch is not None:
        urlFetchTask = URLFetchTask.URLFetchTask()
        urlFetches = []
        urlFetches.append(urlContentRequest.urlFetch)
        urls = urlFetchTask.process(urlFetches, queryCallback)
        for url in urls:
          urlContentRequest.urlMd5 = ""
          urlContentRequest.siteId = url.siteId
          urlContentRequest.url = url.url
          self.calcEmptyFields(urlContentRequest)
          urlContentResponses.append(self.getURLContent(urlContentRequest, queryCallback))
      else:
        self.calcEmptyFields(urlContentRequest)
        urlContentResponses.append(self.getURLContent(urlContentRequest, queryCallback))
      logging.debug(">>> COUNT = %s", str(len(urlContentResponses)))
    return urlContentResponses


  ##calcEmptyFields method calculate values of empty fields
  #
  #@param urlContentRequest object of  URLContentRequest type
  def calcEmptyFields(self, urlContentRequest):
    if urlContentRequest.siteId == "":
      urlContentRequest.siteId = "0"
    if urlContentRequest.urlMd5 is None or urlContentRequest.urlMd5 == "":
      urlContentRequest.urlMd5 = urlContentRequest.fillMD5(urlContentRequest.url)


  ##generates and returns dbFieldsDict
  #
  #@param dbFieldsList list of db fields names
  #@param row db row, not None
  #@return just generated dbFieldsDict dict
  def genDBFields(self, dbFieldsList, row):
    ret = {}
    for fName in dbFieldsList:
      if fName is not None:
        if fName in row:
          if fName in ["UDate", "CDate", "LastModified", "TcDate", "PDate"]:
            ret[str(fName)] = str(row[fName])
          else:
            ret[str(fName)] = row[fName]
        else:
          ret[str(fName)] = None
    return ret


  ##read content from KVDB if CONTENT_TYPE_PROCESSED have setted
  #
  #@param urlContentRequest object of  URLContentRequest type
  #@param dataDir - contains file directory
  #@return list of Content objects
  def contentProcessed(self, dataDir, urlContentRequest, queryCallback):  # pylint: disable=W0613
    ret = []
    DATA_FIELD = "data"
    CDATE_FIELD = "CDate"
    #dbDataTask = DBDataTask.DBDataTask()
    dataFetchRequest = dc.EventObjects.DataFetchRequest(urlContentRequest.siteId, urlContentRequest.urlMd5)
    dataFetchResponse = self.dbDataTask.process(dataFetchRequest, queryCallback)
    if dataFetchResponse is not None and len(dataFetchResponse.resultDict) > 0:
      if DATA_FIELD in dataFetchResponse.resultDict and dataFetchResponse.resultDict[DATA_FIELD] is not None and \
      CDATE_FIELD in dataFetchResponse.resultDict and dataFetchResponse.resultDict[CDATE_FIELD] is not None:
        ret.append(dc.EventObjects.Content(dataFetchResponse.resultDict[DATA_FIELD], 
                                           dataFetchResponse.resultDict[CDATE_FIELD], 
                                           dc.EventObjects.Content.CONTENT_PROCESSOR_CONTENT))
        logger.debug(">>> ret_content == " + str(ret))
    logger.debug(">>> UrlContent result = " + str(dataFetchResponse.__dict__))
    return ret


  ##extract url fields from Database
  #
  #@param siteId site Id
  #@param urlMD5 urls urlMD5
  #@return first row of SQL request
  def selectURLFromMySQL(self, siteId, urlMD5, queryCallback):
    row = None
    tableName = Constants.DC_URLS_TABLE_NAME_TEMPLATE % siteId
    SELECT_URL_QUERY = "SELECT * FROM %s WHERE `URLMd5` = '%s'"
    query = SELECT_URL_QUERY % (tableName, urlMD5)
    res = queryCallback(query, Constants.SECONDARY_DB_ID, Constants.EXEC_NAME)
    if hasattr(res, '__iter__') and len(res) >= 1:
      row = res[0]
    return row


  ##fillLists - fills incoming list of file's content
  #
  #@param filePath - path to the content file
  #@param elemList - incoming filled list
  def fillLists(self, filePath, elemList):
    if os.path.isfile(filePath):
      try:
        fd = open(filePath)
        raw_content = fd.read()
        localDate = datetime.fromtimestamp(os.path.getctime(filePath))
        elemList.append(dc.EventObjects.Content(base64.b64encode(raw_content), localDate.isoformat(' ')))
        fd.close()
      except IOError as err:
        elemList.append(None)
        logger.debug(">>> IOError with file = %s MSG = %s", str(filePath), str(err.message))
    else:
      elemList.append(None)
      logger.debug(">>> No file = %s", str(filePath))


  ##contentRaw - content reader
  #
  #@param fList - incoming file list
  #@param isBreak - break after firs element or not
  def contentRaw(self, fList, isBreak, contentTypeId, parseAdditionType):
    fd = None
    wasOpen = False
    for filePath in fList:
      if os.path.isfile(filePath):
        try:
          fd = open(filePath)
          raw_content = fd.read()
          localDate = datetime.fromtimestamp(os.path.getctime(filePath))
          self.rawContents.append(dc.EventObjects.Content(base64.b64encode(raw_content), localDate.isoformat(' '),
                                                          contentTypeId))
          wasOpen = True
          fd.close()
        except IOError as err:
          logger.debug(">>> IOError with file = %s MSG = %s", str(filePath), str(err.message))

        if wasOpen and parseAdditionType:
          filePath = filePath[0: len(DC_CONSTANTS.RAW_DATA_SUFF) * -1]
          filePath += DC_CONSTANTS.RAW_DATA_HEADERS_SUFF
          if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_HEADERS:
            self.fillLists(filePath, self.headers)
          filePath = filePath[0: len(DC_CONSTANTS.RAW_DATA_HEADERS_SUFF) * -1]
          filePath += DC_CONSTANTS.RAW_DATA_REQESTS_SUFF
          if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_REQUESTS:
            self.fillLists(filePath, self.requests)
          filePath = filePath[0: len(DC_CONSTANTS.RAW_DATA_REQESTS_SUFF) * -1]
          filePath += DC_CONSTANTS.RAW_DATA_META_SUFF
          if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_META:
            self.fillLists(filePath, self.meta)
          filePath = filePath[0: len(DC_CONSTANTS.RAW_DATA_META_SUFF) * -1]
          filePath += DC_CONSTANTS.RAW_DATA_COOKIES_SUFF
          if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_COOKIES:
            self.fillLists(filePath, self.cookies)
      if isBreak:
        break


  ##contentRawCommon - common content reader
  #
  #@param dataDir - contains file directory
  #@param localReverse - file reverse sorting (boolean)
  #@param allFiles - all files read or not (boolean)
  def contentRawCommon(self, dataDir, localReverse=False, allFiles=False, rawDataSuff=DC_CONSTANTS.RAW_DATA_SUFF,
                       contentTypeId=dc.EventObjects.Content.CONTENT_RAW_CONTENT, parseAdditionType=True):
    fileMask = (dataDir + "/*" + rawDataSuff)
    logger.debug(">>> contentRaw fList = " + str(fileMask))
    fList = sorted(glob.glob(fileMask), key=os.path.getctime, reverse=localReverse)
    self.contentRaw(fList, (not allFiles), contentTypeId, parseAdditionType)


  ##extract url content from mandatory storage - implemented RAW!!
  #
  #@param urlContentRequest instance of URLContentRequest objects
  #@param queryCallback function for queries execution
  #@return list of URLContentResponse objects
  def getURLContent(self, urlContentRequest, queryCallback):
    dataDir = self.rawDataDir + "/" + urlContentRequest.siteId + "/" + PathMaker(urlContentRequest.urlMd5).getDir()
    self.clearContents()
    self.contentMask = urlContentRequest.contentTypeMask

    if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_PROCESSED:
      self.processedContents.extend(self.contentProcessed(dataDir, urlContentRequest, queryCallback))

    if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_RAW:
      if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_RAW_LAST:
        self.contentRawCommon(dataDir, True, False)
      if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_RAW_FIRST:
        self.contentRawCommon(dataDir, False, False)
      if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_RAW_ALL:
        self.contentRawCommon(dataDir, False, True)

    if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_TIDY:
      if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_RAW_LAST:
        self.contentRawCommon(dataDir, True, False, DC_CONSTANTS.RAW_DATA_TIDY_SUFF,
                              dc.EventObjects.Content.CONTENT_TIDY_CONTENT, False)
      if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_RAW_FIRST:
        self.contentRawCommon(dataDir, False, False, DC_CONSTANTS.RAW_DATA_TIDY_SUFF,
                              dc.EventObjects.Content.CONTENT_TIDY_CONTENT, False)
      if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_RAW_ALL:
        self.contentRawCommon(dataDir, False, True, DC_CONSTANTS.RAW_DATA_TIDY_SUFF,
                              dc.EventObjects.Content.CONTENT_TIDY_CONTENT, False)

    if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_DYNAMIC:
      if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_RAW_LAST:
        self.contentRawCommon(dataDir, True, False, DC_CONSTANTS.RAW_DATA_DYNAMIC_SUFF,
                              dc.EventObjects.Content.CONTENT_DYNAMIC_CONTENT, False)
      if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_RAW_FIRST:
        self.contentRawCommon(dataDir, False, False, DC_CONSTANTS.RAW_DATA_DYNAMIC_SUFF,
                              dc.EventObjects.Content.CONTENT_DYNAMIC_CONTENT, False)
      if self.contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_RAW_ALL:
        self.contentRawCommon(dataDir, False, True, DC_CONSTANTS.RAW_DATA_DYNAMIC_SUFF,
                              dc.EventObjects.Content.CONTENT_DYNAMIC_CONTENT, False)

    ret = dc.EventObjects.URLContentResponse(urlContentRequest.url, self.rawContents, self.processedContents)
    ret.headers = self.headers
    ret.requests = self.requests
    ret.meta = self.meta
    ret.cookies = self.cookies
    row = self.selectURLFromMySQL(urlContentRequest.siteId, urlContentRequest.urlMd5, queryCallback)
    if row is not None:
      if "Status" in row:
        ret.status = row["Status"]
      if "URL" in row:
        ret.url = row["URL"]
      if "URLMd5" in row:
        ret.urlMd5 = row["URLMd5"]
      if "RawContentMd5" in row:
        ret.rawContentMd5 = row["RawContentMd5"]
      if "ContentURLMd5" in row:
        ret.contentURLMd5 = row["ContentURLMd5"]
      if "Site_Id" in row:
        ret.siteId = row["Site_Id"]
      if hasattr(urlContentRequest.dbFieldsList, '__iter__') and len(urlContentRequest.dbFieldsList) > 0:
        ret.dbFields = self.genDBFields(urlContentRequest.dbFieldsList, row)
    return ret
