'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''
from URLUpdateTask import URLUpdateTask
from SiteUpdateTask import SiteUpdateTask
from BaseTask import BaseTask
import logging
import dc.EventObjects
import Constants
import time
import datetime

logger = logging.getLogger(Constants.LOGGER_NAME)

##process urlFetch event
class URLFetchTask(BaseTask):


  ##constructor
  #
  def __init__(self, keyValueStorageDir, rawDataDir, dBDataTask, dcSiteTemplate, keyValueDefaultFile,
               dcStatTemplate, dcLogTemplate):
    super(URLFetchTask, self).__init__()
    self.quantMaxUrls = 0
    self.localSiteList = []
    self.uRLUpdateTask = URLUpdateTask(keyValueStorageDir, rawDataDir, dBDataTask)
    self.siteUpdateTask = SiteUpdateTask(dcSiteTemplate, keyValueDefaultFile, keyValueStorageDir, rawDataDir, 
                                         dBDataTask, dcStatTemplate, dcLogTemplate)


  ##make all necessary actions to fetch urls data from db, 
  #also locks db mutex (if locking ) between URLFetch operation
  #
  #@param urlFetches list of URLFetch objects
  #@param queryCallback function for queries execution
  #@return list of URL objects
  def process(self, urlFetches, queryCallback):
    urls = []
    needToLock = False
    try:
      for urlFetch in urlFetches:
        needToLock = urlFetch.isLocking
        if needToLock:
          self.dbLock(Constants.FETCH_LOCK_NAME, queryCallback, urlFetch.lockIterationTimeout)
        #@todo add more complex case
        if urlFetch.algorithm is None or urlFetch.algorithm == dc.EventObjects.URLFetch.DEFAULT_ALGORITHM:
          urls.extend(self.processSimple(urlFetch, self.uRLUpdateTask, queryCallback))
        elif urlFetch.algorithm == dc.EventObjects.URLFetch.PROPORTIONAL_ALGORITHM:
          urls.extend(self.processProportial(urlFetch, self.uRLUpdateTask, queryCallback))
        if needToLock:
          self.dbUnlock(Constants.FETCH_LOCK_NAME, queryCallback)
    except Exception:
      if needToLock:
        self.dbUnlock(Constants.FETCH_LOCK_NAME, queryCallback)
      raise
    return urls


  ##processProportial propostial algo implementing
  #
  #@param urlFetch incoming UrlFetch object
  #@param queryCallback function for queries execution
  #@param uRLUpdateTask incoming URLUpdateTask object
  #@return list of site_ids object
  def processProportial(self, urlFetch, uRLUpdateTask, queryCallback):
    ret = []
    offset = 0
    if urlFetch.maxURLs is not None and urlFetch.maxURLs > 0:
      urlFetch.sitesList = self.fillSiteList(urlFetch.sitesList, queryCallback, urlFetch.sitesCriterions)
#     look sites from SiteList for clearing (month ... mins) sites limits
#     self.limitsClearing(urlFetch.sitesList, queryCallback):
      if len(urlFetch.sitesList) > 0:
        self.quantMaxUrls = urlFetch.maxURLs / len(urlFetch.sitesList)
        if self.quantMaxUrls < 1:
          self.quantMaxUrls = 1

        while len(ret) < urlFetch.maxURLs and len(urlFetch.sitesList) > 0:
          self.fillCriterionLimits(urlFetch, offset)
          self.localSiteList = []
          logger.debug(">>> Debug-1 = %s %s %s %s",
                                      str(urlFetch.urlsCriterions[dc.EventObjects.URLFetch.CRITERION_LIMIT][0]),
                                      str(urlFetch.urlsCriterions[dc.EventObjects.URLFetch.CRITERION_LIMIT][1]),
                                      str(urlFetch.maxURLs),
                                      str(self.quantMaxUrls))
          ret.extend(self.getURLFromURLTable(urlFetch, len(ret), queryCallback))
          urlFetch.sitesList = self.localSiteList
          offset += self.quantMaxUrls
      else:
        logger.debug(">>> UrlFetch proportional >>> Empty SiteList")
    self.updateUrl(ret, urlFetch, uRLUpdateTask, queryCallback)
    return ret

  '''
  def limitClearing(self, site, curTime, fName0, fName1, queryCallback):
    sqlTemplate = "SELECT `Value` FROM `sites_properties` WHERE `Site_Id`='%s' AND `Name`='%s'"
    query = sqlTemplate % (site, fName0)
    res = queryCallback(query, Constants.PRIMARY_DB_ID, Constants.EXEC_INDEX)
    if hasattr(res, '__iter__') and len(res) > 0:
      value = res[0][0]
      if curTime > value:
        sqlTemplate = "UPDATE `sites_properties` SET `VALUE`=%s WHERE `Site_Id`='%s' AND `Name`='%s'"
        query = sqlTemplate % (str(curTime), site, fName0)
        queryCallback(query, Constants.PRIMARY_DB_ID, Constants.EXEC_INDEX)
        sqlTemplate = "UPDATE `sites_properties` SET `VALUE`=0 WHERE `Site_Id`='%s' AND `Name`='%s'"
        query = sqlTemplate % (site, fName1)
        queryCallback(query, Constants.PRIMARY_DB_ID, Constants.EXEC_INDEX)


  def limitsClearing(self, siteList, queryCallback):
    curTime = time.time()
    for site in siteList:
      self.limitClearing(site, int(curTime/60), "MIN_LIMIT_TIME", "MIN_LIMIT_VALUE", queryCallback)
      self.limitClearing(site, int(curTime/3600), "HOUR_LIMIT_TIME", "HOUR_LIMIT_VALUE", queryCallback)
      self.limitClearing(site, int(curTime/86400), "DAY_LIMIT_TIME", "DAY_LIMIT_VALUE", queryCallback)
      self.limitClearing(site, int(curTime/604800), "WEEK_LIMIT_TIME", "WEEK_LIMIT_VALUE", queryCallback)
      localDate = datetime.date.fromtimestamp(curTime)
      self.limitClearing(site, int((localDate.year - 1970) * 12 + localDate.month), \
                         "MONTH_LIMIT_TIME", "MONTH_LIMIT_VALUE", queryCallback)


  def limitURL(self, site, urlList, nameLimit, nameValue, queryCallback):
    ret = urlList
    sqlTemplate = "SELECT `Value` FROM `sites_properties` WHERE `Site_Id`='%s' AND `Name`='%s'"
    query = sqlTemplate % (site, nameValue)
    res = queryCallback(query, Constants.PRIMARY_DB_ID, Constants.EXEC_INDEX)
    if hasattr(res, '__iter__') and len(res) > 0:
      query = sqlTemplate % (site, nameLimit)
      res2 = queryCallback(query, Constants.PRIMARY_DB_ID, Constants.EXEC_INDEX)
      if hasattr(res2, '__iter__') and len(res2) > 0:
        value = res[0][0]
        limit = res2[0][0]
        if (value + len(urlList)) >= limit:
          prevLen = len(urlList)
          diff = limit - value
          if diff < 0:
            diff = 0
          ret = urlList[:diff]
          logger.debug(">>> LIMITs in Prop limits [%s trunc From %s to %s]", nameValue, str(prevLen), 
                                                                              str(len(urlList)))
        sqlTemplate = "UPDATE `sites_properties` SET `Value`=%s WHERE `Site_Id`='%s' AND `Name`='%s'"
        query = sqlTemplate % (str(value + len(urlList)), site, nameValue)
        queryCallback(query, Constants.PRIMARY_DB_ID, Constants.EXEC_INDEX)
    return ret


  def limitsURLs(self, site, urlList, queryCallback):
    ret = urlList
    ret = self.limitURL(site, ret, "MIN_LIMIT_LIMIT", "MIN_LIMIT_VALUE", queryCallback)
    ret = self.limitURL(site, ret, "HOUR_LIMIT_LIMIT", "HOUR_LIMIT_VALUE", queryCallback)
    ret = self.limitURL(site, ret, "DAY_LIMIT_LIMIT", "DAY_LIMIT_VALUE", queryCallback)
    ret = self.limitURL(site, ret, "WEEK_LIMIT_LIMIT", "WEEK_LIMIT_VALUE", queryCallback)
    ret = self.limitURL(site, ret, "MONTH_LIMIT_LIMIT", "MONTH_LIMIT_VALUE", queryCallback)
    return ret
  '''

  ##fillCriterionLimits calculates new limit and insert ot the urlsCriterions
  #
  #@param urlFetch incoming UrlFetch object
  #@return list of site_ids object
  def fillCriterionLimits(self, urlFetch, offset):
    if urlFetch.urlsCriterions is None:
      urlFetch.urlsCriterions = {}
    urlFetch.urlsCriterions[dc.EventObjects.URLFetch.CRITERION_LIMIT] = [offset, self.quantMaxUrls]


  ##processSimple simple algo implementing
  #
  #@param urlFetch incoming UrlFetch object
  #@param queryCallback function for queries execution
  #@param uRLUpdateTask incoming URLUpdateTask object
  #@return list of site_ids object
  def processSimple(self, urlFetch, uRLUpdateTask, queryCallback):
    urlFetch.sitesList = self.fillSiteList(urlFetch.sitesList, queryCallback, urlFetch.sitesCriterions)
#   look sites from SiteList for clearing (month ... mins) sites limits
#   self.limitsClearing(urlFetch.sitesList, queryCallback):
    ret = self.getURLFromURLTable(urlFetch, 0, queryCallback)

    self.updateUrl(ret, urlFetch, uRLUpdateTask, queryCallback)
    return ret


  ##updateUrl updates urls from urls list
  #
  #@param urls incoming urls list
  #@param urlFetch incoming UrlFetch object
  #@param queryCallback function for queries execution
  #@param uRLUpdateTask incoming URLUpdateTask object
  #@return list of site_ids object
  def updateUrl(self, urls, urlFetch, uRLUpdateTask, queryCallback):
    if len(urls) > 0:
      for localURL in urls:
        if localURL.urlMd5 is not None and urlFetch.urlUpdate is not None:
          urlUpdateList = []
          urlUpdateList.append(urlFetch.urlUpdate)
          urlUpdateList[0].urlMd5 = localURL.urlMd5
          urlUpdateList[0].siteId = localURL.siteId
          uRLUpdateTask.process(urlUpdateList, queryCallback)


  ##Generates SQL String request for using in the fillSiteList method
  #
  #@param sitesCriterions - income sitesCriterions field
  #@param userId - income userId field
  #@return query string
  def fillSiteListSQLGenerate(self, sitesCriterions, userId=None):
    query = Constants.SELECT_SQL_TEMPLATE_SIMPLE %("`Id`", "sites")
    additionQueryStr = ""
    additionWhereCause = ""
    if userId is not None:
      additionWhereCause += (Constants.CHECK_TABLE_SQL_ADDITION % str(userId))
    
    if sitesCriterions is None:
      if additionWhereCause != "":
        additionWhereCause += " AND "
      additionWhereCause += ("`state`=%d" % dc.EventObjects.Site.STATE_ACTIVE)
    
    if sitesCriterions is None:
      if additionWhereCause != "":
        additionQueryStr = " WHERE " + additionWhereCause
    else:
      if additionWhereCause == "":
        additionQueryStr = self.generateCriterionSQL(sitesCriterions)
      else:
        additionQueryStr = self.generateCriterionSQL(sitesCriterions, additionWhereCause)
    query += additionQueryStr

    return query


  ##get id from sites
  #
  #@parama incomeSiteList - income (from event) site list
  #@param queryCallback function for queries execution
  #@param sitesCriterions - income sitesCriterions field
  #@param userId - income userId field
  #@return list of site_ids object
  def fillSiteList(self, incomeSiteList, queryCallback, sitesCriterions, userId=None):
    ret = []
    newSiteList = []
    localSiteDict = {}
    query = self.fillSiteListSQLGenerate(sitesCriterions, userId)
    res = queryCallback(query, Constants.PRIMARY_DB_ID)
    if hasattr(res, '__iter__'):
      for row in res:
        localSiteDict[row[0]] = row[0]
        newSiteList.append(row[0])
        
      if len(incomeSiteList) == 0:
        ret = newSiteList
      else:
        newSiteList = []
        for site in incomeSiteList:
          if site in localSiteDict:
            newSiteList.append(site)
        ret = newSiteList
    return ret


  ##static fillUrlObj method, create and fills URL object
  #
  #@param row db select row with URL fields
  #@return URL object
  @staticmethod
  def fillUrlObj(row):
    url = dc.EventObjects.URL(row["Site_Id"], row["URL"])
    for field in Constants.URLTableDict.keys():
      if hasattr(url, field) and Constants.URLTableDict[field] in row:
        setattr(url, field, row[Constants.URLTableDict[field]])
    url.UDate = Constants.readDataTimeField("UDate", row)
    url.CDate = Constants.readDataTimeField("CDate", row)
    url.lastModified = Constants.readDataTimeField("LastModified", row)
    url.tcDate = Constants.readDataTimeField("TcDate", row)
    url.pDate = Constants.readDataTimeField("PDate", row)
    return url


  ##get URL from URL db
  #
  #@param urlFetch instance of URLFetch object
  #@param queryCallback function for queries execution
  #@return list of URL object
  def getURLFromURLTable(self, urlFetch, globalLen, queryCallback):
    urls = []

    for siteId in urlFetch.sitesList:
      #Update site TcDate to push rotate sites list
      if urlFetch.siteUpdate is not None:
        urlFetch.siteUpdate.id = siteId
        self.siteUpdateTask.process(urlFetch.siteUpdate, queryCallback)
        logger.debug(">>> FRLFetch - Site %s was updated", str(siteId))
      tableName = Constants.DC_URLS_TABLE_NAME_TEMPLATE % siteId
      additionQueryStr = self.generateCriterionSQL(urlFetch.urlsCriterions, None, siteId)
      query = Constants.SELECT_SQL_TEMPLATE_SIMPLE %("*", tableName)
      if len(additionQueryStr) > 0:
        query += " "
        query += additionQueryStr

      res = queryCallback(query, Constants.SECONDARY_DB_ID, Constants.EXEC_NAME)
      if not hasattr(res, '__iter__'):
        res = []

      #Add urlList truncating 
      #res = self.limitsURLs(siteId, res, queryCallback):
      if self.quantMaxUrls > 0 and len(res) == self.quantMaxUrls:
        self.localSiteList.append(siteId)

      for row in res:
        if urlFetch.maxURLs is not None and urlFetch.maxURLs != 0 and urlFetch.maxURLs == globalLen:
          break
        if "Site_Id" in row and "URL" in row:
          urls.append(URLFetchTask.fillUrlObj(row))
          globalLen += 1
      if urlFetch.maxURLs is not None and urlFetch.maxURLs != 0 and urlFetch.maxURLs == globalLen:
        break

    return urls
