"""
  HCE project, Python bindings, Distributed Tasks Manager application.
  Event objects definitions.
  
  @package: dc
  @file CollectionConverter.py
  @author Oleksii <developers.hce@gmail.com>
  @link: http://hierarchical-cluster-engine.com/
  @copyright: Copyright &copy; 2013-2014 IOIX Ukraine
  @license: http://hierarchical-cluster-engine.com/license/
  @since: 0.1
  """


import logging
import json
import copy
import dc.EventObjects as dc_event
import app.Consts as APP_CONSTS
import dc.Constants as DC_CONSTS
from app.Utils import varDump
from scraper_utils import encode
from scraper_utils import decode


logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


class CollectionConverter(object):


  def __init__(self, input_batch, batch_item, wrapper):
    self.input_batch = input_batch
    self.batch_item = batch_item
    self.processedContent = None
    self.urlContentResponse = None
    self.wrapper = wrapper
    self.collection = []
    self.buffer = None


  def process(self):
    self.getContent()
    self.convertContent2Collection()
    self.putContent()
    return self.batch_item
  
  
  def getContent(self):
    siteId = self.batch_item.siteId
    url = self.batch_item.urlObj.url
    urlId = self.batch_item.urlId
    logger.info("siteId: <<%s>>, urlId: <<%s>>, url: <<%s>>" % (siteId, urlId, url))
    
    #Check if Real-Time crawling
    if self.input_batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER and \
      "DB_TASK_MODE" in self.batch_item.properties and \
      self.batch_item.properties["DB_TASK_MODE"] == "RO":
      logger.info("self.batch_item.urlPutObj.putDict[data]: %s", varDump(self.batch_item.urlPutObj.putDict["data"]))
      if self.batch_item.urlContentResponse is not None:
        self.batch_item.urlContentResponse.processedContents = [dc_event.Content(self.batch_item.urlPutObj.putDict["data"], fieldId=dc_event.Content.CONTENT_PROCESSOR_CONTENT)]
      else:
        content = dc_event.Content(self.batch_item.urlPutObj.putDict["data"], fieldId=dc_event.Content.CONTENT_PROCESSOR_CONTENT)
        self.batch_item.urlContentResponse = dc_event.URLContentResponse(url, rawContents=None, processedContents=[content])
        self.batch_item.urlContentResponse.status = 7
      self.urlContentResponse = [self.batch_item.urlContentResponse]
    else:
      urlContentRequest = dc_event.URLContentRequest(siteId, url)
      drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_CONTENT, [urlContentRequest])
      responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
      self.urlContentResponse = responseDRCESyncTasksCover.eventObject
    logger.info("urlContentResponse: %s", varDump(self.urlContentResponse))
  
  
  
  def convertContent2Collection(self):
    # TODO: URLContentResponse object return list of contents
    # We assume that send / recieve only one object in the list
    # If not adjust response handling
    self.processedContent = self.urlContentResponse[0].processedContents
    logger.info("self.processedContent: %s", varDump(self.processedContent))
    self.buffer = json.loads(decode(self.processedContent[0].self.buffer))
    if "template" in self.batch_item.properties:
      logger.info("self.buffer: %s", varDump(self.buffer))
      logger.info("self.buffer's type: %s", str(type(self.buffer)))
      items = self.buffer["default"]["data"]["tagList"][0]
      logger.info("items: %s" % varDump(items))
      logger.info("self.batch_item.properties['template']: %s" % str(self.batch_item.properties['template']))
      #logger.info("self.batch_item.properties['template'] type: %s" % str(type(self.batch_item.properties['template'])))
      
      
      # Convert DL(Dict of Lists) -> LD(List of Dicts)
      keys = self.batch_item.properties["template"]["templates"][0]["tags"].keys()
      DL = {}
      for item in items:
        logger.info('len(item["data"]): %s' % str(len(item["data"])))
        logger.info('item["data"]: %s' % varDump(item["data"]))
        logger.info('item["name"]: %s' % varDump(item["name"]))
        logger.info('keys: %s' % varDump(keys))
        if (item["name"] in keys) and (len(item["data"]) > 0):
          DL[item["name"]] = item["data"]
      logger.info('DL.keys(): %s' % varDump(DL.keys()))
      if len(DL.keys())>0:
        self.collection = [{key:value[index] for key,value in DL.items()} for index in range(max(map(len,DL.values())))]
      logger.info("self.collection: %s" % varDump(self.collection))
      
      # Create list of result's objs
      collection_len = len(self.collection)
      self.buffer["default"]["data"]["tagList"] = self.buffer["default"]["data"]["tagList"]*collection_len
      
      for i in range(0,collection_len):
        A = []
        for tag_item in self.buffer["default"]["data"]["tagList"][i]:
          B = copy.copy(tag_item)
          B["data"] = [DL[tag_item["name"]][i]]
          A.append(B)
        self.buffer["default"]["data"]["tagList"][i] = A

    # Place back content
    logger.info("self.buffer: %s" % varDump(self.buffer))
    self.processedContent[0].self.buffer = encode(json.dumps(self.buffer))


  def putContent(self):
    # URLPut
    # Create new URLPut object
    putDict = {}
    putDict["id"] = self.batch_item.urlId
    putDict["data"] = self.processedContent[0].self.buffer
    urlPut = dc_event.URLPut(self.batch_item.siteId,
                                 self.batch_item.urlId,
                                 dc_event.Content.CONTENT_PROCESSOR_CONTENT,
                                 putDict)
    #Check if Real-Time crawling
    if self.input_batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER and \
      "DB_TASK_MODE" in self.batch_item.properties and \
      self.batch_item.properties["DB_TASK_MODE"] == "RO":
      self.batch_item.urlPutObj = urlPut
    else:
      drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_PUT, [urlPut])
      responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
      if responseDRCESyncTasksCover.eventType == DC_CONSTS.EVENT_TYPES.URL_PUT_RESPONSE:
        for obj in responseDRCESyncTasksCover.eventObject:
          logger.info("URL_PUT_RESPONSE: %s" % varDump(obj))
      else:
        logger.error("URL_PUT_RESPONSE >>> Wrong response type")
