# coding: utf-8  # pylint: disable-all

"""@package docstring
  @file Metrics.py
  @author Alexey <developers.hce@gmail.com>
  @link http://hierarchical-cluster-engine.com/
  @copyright Copyright &copy; 2013 IOIX Ukraine
  @license http://hierarchical-cluster-engine.com/license/
  @package HCE project node API
  @since 0.1
  """


import logging

import dc_processor.Constants as CONSTS
import app.Consts as APP_CONSTS


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


## BaseMetric
#
class BaseMetric(object):
  
  name = "Base metric"
  tresholdValue = CONSTS.DEFAULT_TRESHOLD_VALUE
  metricValue = CONSTS.DEFAULT_METRIC_VALUE
  comparator = CONSTS.DEFAULT_COMPARATOR
  functor = None
  
  ## ctor
  #
  def __init__(self):
    self.name = self.__class__.__name__
  

  ## calculator stub
  #
  def addThresholdValue(self, tresholdValue):
    self.tresholdValue = tresholdValue


  ## init functor
  #
  def initFunctor(self, functor):
    self.functor = functor
  
  
  ## calculator stub
  #
  def calculateMetricValue(self, dictionary):
    self.metricValue = CONSTS.DEFAULT_METRIC_VALUE


## WordsMetric
#
class WordsMetric(BaseMetric):
  
  tresholdValue = CONSTS.WORDS_TRESHOLD_VALUE
  comparator = CONSTS.WORDS_COMPARATOR
  
  # ctor
  def __init__(self):
    super(WordsMetric, self).__init__()

  ## calculator stub
  #
  def calculateMetricValue(self, dictionary):
    logger.info("dictionary: %s" % str(dictionary))
    if CONSTS.ARTICLE_CORPUS in dictionary and \
      (type(dictionary[CONSTS.ARTICLE_CORPUS])==type({})):
      article_corpus = dictionary[CONSTS.ARTICLE_CORPUS]["data"]
      logger.info("article_corpus: %s" % str(article_corpus))
      self.metricValue = len(article_corpus.split())
      logger.info("Words count: %s" % str(self.metricValue))


## WordsMetric
#
class SentencesMetric(BaseMetric):
  
  tresholdValue = CONSTS.SENTENCES_TRESHOLD_VALUE
  comparator = CONSTS.SENTENCES_COMPARATOR
  
  # ctor
  def __init__(self):
    super(SentencesMetric, self).__init__()
  
  ## calculator stub
  #
  def calculateMetricValue(self, dictionary):
    logger.info("dictionary: %s" % str(dictionary))
    if CONSTS.ARTICLE_CORPUS in dictionary and \
      (type(dictionary[CONSTS.ARTICLE_CORPUS])==type({})):
      article_corpus = dictionary[CONSTS.ARTICLE_CORPUS]["data"]
      logger.info("article_corpus: %s" % str(article_corpus))
      self.metricValue = len(article_corpus.split("."))
      logger.info("Sentences count: %s" % str(self.metricValue))


## ARIMetric
#
class ARIMetric(BaseMetric):
  
  tresholdValue = CONSTS.ARI_TRESHOLD_VALUE
  comparator = CONSTS.ARI_COMPARATOR
  
  # ctor
  def __init__(self):
    super(ARIMetric, self).__init__()
  
  ## calculator stub
  #
  #@param dictionary data returned by scraper's module
  def calculateMetricValue(self, dictionary):
    logger.info("dictionary: %s" % str(dictionary))
    if CONSTS.ARTICLE_CORPUS in dictionary and \
      (type(dictionary[CONSTS.ARTICLE_CORPUS])==type({})):
      article_corpus = dictionary[CONSTS.ARTICLE_CORPUS]["data"]
      logger.info("article_corpus: %s" % str(article_corpus))
      from textstat.textstat import textstat
      self.metricValue = textstat.automated_readability_index(article_corpus)
      logger.info("Automated Readability index: %s" % str(self.metricValue))


## Custom Metric
#
class CustomMetric(BaseMetric):
  
  tresholdValue = CONSTS.ARI_TRESHOLD_VALUE
  comparator = CONSTS.ARI_COMPARATOR
  
  # ctor
  def __init__(self):
    super(CustomMetric, self).__init__()
  
  ## calculator stub
  #
  #@param dictionary data returned by scraper's module
  def calculateMetricValue(self, dictionary):
    logger.info("dictionary: %s" % str(dictionary))
    if CONSTS.ARTICLE_CORPUS in dictionary and \
      (type(dictionary[CONSTS.ARTICLE_CORPUS])==type({})):
      text = dictionary[CONSTS.ARTICLE_CORPUS]["data"]
      logger.info("article_corpus: %s" % str(text))
      if self.functor is not None:
        logger.info("self.functor: %s" % self.functor)
        self.metricValue = eval(self.functor)
      logger.info("Custom index: %s" % str(self.metricValue))
