"""@package docstring
 @file Scraper.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


import pickle
import sys
import json
import hashlib
import sqlite3  # as sqlite3
import feedparser
import logging.config
import ConfigParser
from cement.core import foundation
import app.Utils as Utils  # pylint: disable=F0401
import dc_processor.Constants as CONSTS
from scraper_result import Result
from scraper_utils import encode
from app.Utils import varDump
import app.Utils
import dc.EventObjects as dc_event
from dc_processor.ScraperResponse import ScraperResponse


APP_NAME = "ProcessorFeedParser"


# #Scraper
#
#
class ProcessorFeedParser(foundation.CementApp):


  # Mandatory
  class Meta(object):
    label = APP_NAME
    def __init__(self):
      pass


  # #constructor
  # initialize default fields
  def __init__(self):
    # call base class __init__ method
    foundation.CementApp.__init__(self)
    self.exit_code = CONSTS.EXIT_SUCCESS
    self.logger = None
    self.config_db_dir = None
    self.sqliteTimeout = CONSTS.SQLITE_TIMEOUT
    self.entry = None
    self.tagsCount = None
    self.tagsMask = None
    self.pubdate = None
    self.processedContent = None
    self.store_xpath = False
    self.db_engine = CONSTS.SQLITE_ENGINE


  # #setup
  # setup application
  def setup(self):
    # call base class setup method
    foundation.CementApp.setup(self)
    self.args.add_argument('-c', '--config', action='store', metavar='config_file', help='config ini-file')


  # #run
  # run application
  def run(self):
    # call base class run method
    foundation.CementApp.run(self)

    # config section
    self.loadConfig()

    # load logger config file
    self.loadLogConfigFile()

    # load sqlite db backend
    # self.loadSqliteDBBackend()

    # sqlite
    # self.loadDBBackend()

    # options
    self.loadOptions()


  # #main content processing
  # main content processing
  #
  def createArticle(self):
    resid = self.entry["urlMd5"]
    self.article = Result(self.config, resid)

    #tags = ["tags", "media_thumbnail", "summary", "comments", "author", "media_content", "summary_detail", "link", "title", "content", "published", "updated", "updated_parsed"]#, "links"]
    #for tag in tags:
    for tag in self.entry["entry"]:
        data = {"extractor":"feedParser extractor", "data":"", "name":""}
        data["data"] = self.entry["entry"][tag]
        data["name"] = tag
        self.article.tags[tag] = data

    date_tags = ["published", "updated", "updated_parsed"]
    if len(set(self.entry["entry"].keys()).intersection(date_tags))==0:
      self.logger.debug("PUBDATE_EXTRACTOR: list of tags from rss feed: %s" % str(self.entry["entry"].keys()))

    if "pubdate" in self.entry and \
      self.article.tags["pubdate"]=="":
      data = {"extractor":"feedParser extractor", "data":"", "name":""}
      data["data"] = self.entry["pubdate"]
      data["name"] = "pubdate"
      self.article.tags["pubdate"] = data

    #parent rss feed
    data = {"extractor":"feedParser extractor", "data":"", "name":""}
    data["data"] = self.entry["parent_rss_feed"]
    data["name"] = "parent_rss_feed"
    data["xpath"] = ""
    data["extractor"] = self.__class__.__name__
    self.article.tags["parent_rss_feed"] = data

    #parent rss feed urlMd5
    data = {"extractor":"feedParser extractor", "data":"", "name":""}
    data["data"] = self.entry["parent_rss_feed_urlMd5"]
    data["name"] = "parent_rss_feed_urlMd5"
    data["xpath"] = ""
    data["extractor"] = self.__class__.__name__
    self.article.tags["parent_rss_feed_urlMd5"] = data
  
    #tags count
    self.article.tagsCount = len(self.article.tags.keys())





  # #main content processing
  # main content processing
  #
  def parseFeed(self):
    ret = True
    try:
      self.entry = json.loads(self.input_data.raw_content)
      self.createArticle()
      #self.putArticleToDB()
      self.putArticleToDB({"default":self.article})
    except ValueError, err:
      self.logger.debug("Bad raw content: %s" % varDump(self.input_data.raw_content))
      ret = False
    return ret


  # #main content processing
  # main content processing
  #
  def process(self):
    self.logger.debug("URL: %s" % str(self.input_data.url))
    self.logger.debug("URLMd5: %s" % str(self.input_data.urlId))
    self.logger.debug("SiteId: %s" % str(self.input_data.siteId))
    if self.parseFeed():
      self.tagsCount = self.article.tagsCount
      self.tagsMask = self.article.tagsMask
      self.processedContent = self.article.get()
      # correct pubdate
      if CONSTS.PUBLISHED in self.article.tags:
        from dateutil.parser import parse
        self.pubdate = parse(self.article.tags[CONSTS.PUBLISHED]["data"]).strftime('%Y-%m-%d %H:%M:%S')
      else:
        self.logger.debug("Resource %s hasn't publish date" % str(self.article.tags[CONSTS.TAG_LINK]["data"]))
      # for ml
      if self.store_xpath:
        self.storeXPath()
    else:
      self.logger.debug("Resource hasn't raw content. Exit.")


  ## For
  #
  def getQueryPrefix(self):
    query_prefix = None
    if self.db_engine == CONSTS.SQLITE_ENGINE:
      query_prefix = "INSERT OR REPLACE INTO `" + self.articles_tbl
    elif self.db_engine == CONSTS.MYSQL_ENGINE:
      query_prefix = "REPLACE INTO `contents_" + str(self.input_data.siteId if len(self.input_data.siteId) else 0)
    else:
      self.logger.info("DB Backend %s not supported" % str(self.db_engine))
    self.logger.info("db_name: " + query_prefix)
    return query_prefix


  ## getDataBuffer
  ## prepare data buffer
  #
  def getDataBuffer(self, data):
    buffer = {}
    for key in data.keys():
      data[key].get()
      buffer[key] = data[key].data
    ret = encode(json.dumps(buffer))
    self.logger.info("Result buffer: %s" % ret)
    return ret


  def putArticleToDB(self, result):
    # Check if Content extracted
    self.logger.info("Result: %s" % varDump(result))
    tags_count = int(result["default"].tagsCount)
    self.logger.info("Tags count: %s" % str(tags_count))
    if int(tags_count)>0:
      self.logger.info("Tags count OK")
      
      # Get DB table name
      query_prefix = self.getQueryPrefix()
      
      # Prepare query
      # Firstly, prepare data buffer
      data = self.getDataBuffer(result)
      # Then, insert data buffer to the query
      #query = query_prefix + "`(`id`,`data`, CDate) VALUES('" \
      #  + result.resId + "', '" + encode(result.get()) + "', strftime('%Y-%m-%d %H:%M:%f', 'now', 'localtime'))"
      query = query_prefix + "`(`id`,`data`, CDate) VALUES('" + result["default"].resId + "', '" + data + "', NOW())"
      
      # Initialize options for DB Backend
      options = {}
      # Genaral options
      options["query"] = query
      self.logger.info("DB Backend %s" % str(self.db_engine))
      
      if self.db_engine == CONSTS.SQLITE_ENGINE:
        # Get DB Backend name
        db_name = self.getDBName()
        # SQLite options
        options["db_name"] = db_name
        options["sqliteTimeout"] = self.sqliteTimeout
        options["PRAGMA_synchronous"] = self.PRAGMA_synchronous
        options["PRAGMA_journal_mode"] = self.PRAGMA_journal_mode
        options["PRAGMA_temp_store"] = self.PRAGMA_temp_store
        # Call db query
        Utils.putContentToSQLiteDB(result, options)
      elif self.db_engine == CONSTS.MYSQL_ENGINE:
        # MYSQL options
        options["dbHost"] = self.dbHost
        options["dbPort"] = self.dbPort
        options["dbUser"] = self.dbUser
        options["dbPWD"] = self.dbPWD
        options["MYSQLDBName"] = self.dc_contents_db
        Utils.putContentToMYSQLDB(result, options)
      else:
        self.logger.info("DB Backend %s not supported" % str(self.db_engine))


  """
  def putArticleToDB(self):
    # get appropriate db name, depending on siteId
    if len(self.input_data.siteId):
      db_name = self.config_db_dir + "/" + self.input_data.siteId + ".db"
    else:
      db_name = self.config_db_dir + "/0.db"
    #self.logger.info("db_name: " + db_name)
    connector = None
    try:
      # put parsed resource to the db
      connector = sqlite3.connect(db_name, timeout=self.sqliteTimeout)  # @UndefinedVariable
      with connector:
        cur = connector.cursor()
        query = "INSERT OR REPLACE INTO `" \
          + self.articles_tbl + \
            "`(`id`,`data`, CDate) VALUES('" \
              + self.article.resId + \
                "', '" \
                + encode(self.article.get()) + \
                "', strftime('%Y-%m-%d %H:%M:%f', 'now', 'localtime'))"
        cur.execute(query)
    except Exception as err:
      self.logger.error("%s " % (err.message))
      raise
    """

  # #process batch
  # the main processing of the batch object
  def processBatch(self):
    try:
      # read pickled batch object from stdin and unpickle it
      input_pickled_object = sys.stdin.read()
      stored_in_data = pickle.loads(input_pickled_object)
      self.input_data = stored_in_data
      # self.logger.info("input scraper object: " + str(vars(stored_in_data)))
      # TODO main processing over every url from list of urls in the batch object
      self.process()
      scraperResponse = ScraperResponse(self.tagsCount, self.tagsMask, self.pubdate, self.processedContent)
      output_pickled_object = pickle.dumps(scraperResponse)
      print output_pickled_object
      sys.stdout.flush()
    except Exception as err:
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.logger.error(CONSTS.MSG_ERROR_PROCESS + err.message)
      self.exit_code = CONSTS.EXIT_FAILURE


  ## storeXpath
  #
  def storeXPath(self):
    ret = False
    """
    from dc_processor.base_extractor import BaseExtractor
    # check content's presence in response
    if self.scraper_response.tagsMask & BaseExtractor.tagsMask[CONSTS.SUMMARY_DETAIL]:
      content = None
      tags = json.loads(self.scraper_response.processedContent)["data"]["tagList"]
      for tag in tags:
        if tag["name"]==CONSTS.SUMMARY_DETAIL:
          content = tag["data"]["value"]
          self.logger.debug("content: %s" % str(content))
          from scrapy.selector import Selector
          sel = Selector(text=self.raw_content)
          xpath_list = sel.xpath("//*[contains(., '" + content + "')]").extract()
          self.logger.debug("xpath_list: %s" % str(xpath_list))
    """
    return ret


  # #load config from file
  # load from cli argument or default config file
  def loadConfig(self):
    try:
      self.config = ConfigParser.ConfigParser()
      self.config.optionxform = str
      if self.pargs.config:
        self.config.read(self.pargs.config)
    except Exception as err:
      print CONSTS.MSG_ERROR_LOAD_CONFIG + err.message
      raise


  # #load logging
  # load logging configuration (log file, log level, filters)
  #
  def loadLogConfigFile(self):
    try:
      log_conf_file = self.config.get("Application", "log")
      logging.config.fileConfig(log_conf_file)
      self.logger = logging.getLogger(APP_NAME)
    except Exception as err:
      print CONSTS.MSG_ERROR_LOAD_LOG_CONFIG_FILE + err.message
      raise


  # #load mandatory options
  # load mandatory options
  #
  def loadOptions(self):
    class_name = self.__class__.__name__
    try:
      self.config_db_dir = self.config.get(class_name, "config_db_dir")
      self.articles_tbl = self.config.get("sqlite", "articles_tbl")
      self.sqliteTimeout = self.config.getint("sqlite", "timeout")

      # support sqlite optimizer options
      if self.config.has_option("sqlite", "PRAGMA_synchronous"):
        self.PRAGMA_synchronous = self.config.get("sqlite", "PRAGMA_synchronous")
      if self.config.has_option("sqlite", "PRAGMA_journal_mode"):
        self.PRAGMA_journal_mode = self.config.get("sqlite", "PRAGMA_journal_mode")
      if self.config.has_option("sqlite", "PRAGMA_temp_store"):
        self.PRAGMA_temp_store = self.config.get("sqlite", "PRAGMA_temp_store")
    
      # support db backend
      if self.config.has_option(class_name, "db_engine"):
        self.db_engine = self.config.get(class_name, "db_engine")

    except Exception as err:
      print CONSTS.MSG_ERROR_LOAD_OPTIONS + err.message
      raise


  # #
  #
  #
  def getExitCode(self):
    return self.exit_code
