"""
HCE project, Python bindings, Distributed Tasks Manager application.
Event objects definitions.

@package: dc
@file ProcessorTask.py
@author Oleksii <developers.hce@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""
import os
import sys
import time
import icu
import json
import hashlib
import re
import copy
import types
import base64
try:
  import cPickle as pickle
except ImportError:
  import pickle
# import logging
import logging.config
import ConfigParser
import MySQLdb as mdb
import MySQLdb.cursors
# import sqlite3 as lite
from cement.core import foundation
from subprocess import Popen
from subprocess import PIPE
from collections import namedtuple

from app.Utils import varDump
from app.Utils import PathMaker
import app.Utils
from scraper_utils import encode
from dc_processor.ScraperInData import ScraperInData
from dc.EventObjects import URL
from dc.EventObjects import Site
from dc.EventObjects import Batch
import dc.EventObjects as dc_event
from dc import EventObjects
from dc_db.TasksManager import TasksManager as DBTasksManager
from dc_processor.ProcessorException import ProcessorException
import app.Utils as Utils  # pylint: disable=F0401
from dc_crawler.Constants import ERROR_MIME_NOT_WRITE
from dc_crawler.Constants import DB_URLS
from dc_processor.ScraperResponse import ScraperResponse
import app.Consts as APP_CONSTS
import dc.Constants as DC_CONSTS
import dc_processor.Constants as CONSTS
from dc_crawler.DBTasksWrapper import DBTasksWrapper
from app.Utils import SQLExpression
from scraper_utils import decode
from dc_processor.CollectionConverter import CollectionConverter


APP_NAME = "processor-task"

DC_URLS_DB_NAME = "dc_urls"
DC_URLS_TABLE_PREFIX = "urls_"
DC_SITES_DB_NAME = "dc_sites"
DC_SITES_TABLE_NAME = "sites"
DC_URLS_TABLE_NAME = "urls"
DC_SITES_PROPERTIES_TABLE_NAME = "sites_properties"

MSG_ERROR_PROCESS_BATCH_ITEM = "Error process batch item "
MSG_ERROR_PROCESS_BATCH = "Error process batch. "
MSG_ERROR_LOAD_CONFIG = "Error loading config file. Exciting."
MSG_ERROR_LOAD_LOG_CONFIG_FILE = "Error loading logging config file. Exiting."
MSG_ERROR_LOAD_URL_DATA = "Can't load url data: "
MSG_ERROR_LOAD_SITE_DATA = "Error load site data: "
MSG_ERROR_READ_SITE_FROM_DB = "Error read site data from db"
MSG_ERROR_PROCESS_TASK = "Can't process task "
MSG_ERROR_SERIALISE_RESULT = "Error serialize result "
MSG_ERROR_GET_SITE_FILE_DB = "Error get site file db "
MSG_ERROR_UPDATE_RECORD = "Error update record "
MSG_ERROR_UPDATE_PROCESSED_URL = "Error update processed url "
MSG_ERROR_GET_RAW_CONTENT_FROM_DB = "Error get raw content from disk "
MSG_ERROR_PROCESS = "Error process "
MSG_ERROR_LOAD_SITE_PROPERTIES = "Error load site properties "
MSG_ERROR_CHECK_SITE = "Error check site. "
MSG_ERROR_LOAD_OPTIONS = "Error load options. "
MSG_ERROR_CONVERT_RAW_CONTENT_CHARSET = "Cannot convert raw content charset. "
MSG_ERROR_UPDATE_SITE_RESOURCES = "Error update site resources. "
MSG_ERROR_EMPTY_BATCH = "Error read input pickle from stdin."
MSG_ERROR_CHECK_CONTENT_HASH = "Fail to check content hash"
MSG_ERROR_CALC_CONTENT_HASH = "Fail to calc content hash"
MSG_ERROR_CHECK_CONTENT_HASH_DUPLICATE = "Can't check content hash duplicate"

MSG_INFO_PROCESSOR_CMD = "Processor cmd: "
MSG_INFO_LOAD_SITE_PROPERTIES = "Mismatch load site properties "
MSG_INFO_PROCESS_BATCH = "Skipped process batch. "
MSG_INFO_PROCESS_BATCH_ITEM = "Skipped process batch item "
MSG_INFO_PROCESSOR_EXIT_CODE = "Scraper exit_code: "
MSG_INFO_PROCESSOR_OUTPUT = "Scraper output: "
MSG_INFO_PROCESSOR_ERROR = "Scraper err: "

EXIT_SUCCESS = 0
EXIT_FAILURE = 1

ERROR_MASK_NO_ERRORS = 0
ERROR_MASK_SITE_OK = 0
ERROR_MASK_SITE_MAX_ERRORS = 1 << 32
ERROR_MASK_SITE_MAX_RESOURCES = 1 << 33
ERROR_MASK_SITE_UNSUPPORTED_CONTENT_TYPE = 1 << 34
ERROR_MASK_URL_ENCODING_ERROR = 1 << 35
ERROR_MASK_SCRAPER_ERROR = 1 << 36
ERROR_MASK_MISSED_RAW_CONTENT_ON_DISK = 1 << 37


TEMPLATE_SELECTION_TYPE_DEFAULT = 0
TEMPLATE_SELECTION_TYPE_FULL_URLMD5 = 1
TEMPLATE_SELECTION_TYPE_DOMAIN_URLMD5 = 2

URLS_OF_MEDIA_CONTENT = 1
ENV_PROCESSOR_STORE_PATH = "ENV_PROCESSOR_STORE_PATH"

Results = namedtuple("Results", "exit_code, output, err")
# scraper = None

# #The CrawlerTask class, is a interface for fetching content from the web
#
# This object is a run at once application
class ProcessorTask(foundation.CementApp):

  """
  # Configuration settings options names
  DB_HOST = "db_host"
  DB_PORT = "db_port"
  DB_USER = "db_user"
  DB_PWD = "db_pwd"
  DB_SITES = "db_dc_sites"
  DB_URLS = "db_dc_urls"
  """

  # Mandatory
  class Meta(object):
    label = APP_NAME
    def __init__(self):
      pass

  # #constructor
  # initialize default fields
  def __init__(self):
    # call base class __init__ method
    foundation.CementApp.__init__(self)
    self.exit_code = EXIT_SUCCESS
    self.logger = None
    self.result_object = None
    self.site_properties = None
    self.template = None
    self.raw_content = None
    self.ErrorMask = None
    self.site = None
    self.DBConnector = None
    self.table_name = None
    self.url = None
    self.process_time = None
    self.batch_item = None
    self.dc_sites_db_connect = None
    self.dc_urls_db_connect = None
    self.site_connector = None
    self.URLConnector = None
    self.site_table = None
    self.URLTable = None
    self.raw_data_dir = None
    self.processCTypes = []
    self.charset = None
    self.filters = None
    self.processorName = None
    self.processorProperties = None
    self.wrapper = None
    self.batch_sites = set()
    self.scraper_response = None
    self.input_batch = None
    self.pubdate = None
    self.timezone = None
    self.text_stats = True
    self.contentHash = None
    self.list_hashed_tags = None
    self.contentURLMd5 = ""
    self.contentURLMd5Algorithm = None
    self.contentURLMd5Action = None
    self.db_task_ini = None
    self.wrapper = None
    self.processedContent = None
    self.collection_support = False
    self.do_reduce = False
    self.htmlRecover = False

  # #setup
  # setup application
  def setup(self):
    # call base class setup method
    foundation.CementApp.setup(self)
    self.args.add_argument('-c', '--config', action='store', metavar='config_file', help='config ini-file')


  # #run
  # run application
  def run(self):
    # call base class run method
    foundation.CementApp.run(self)

    # config section
    self.loadConfig()

    # load scraper
    # self.loadScraper()

    # load logger config file
    self.loadLogConfigFile()

    # load mandatory options
    self.loadOptions()

    # load db backend
    self.loadDBBackend()



  # #loadSiteProperties
  #
  # @param site - site object
  def loadSiteProperties(self):
  # support of TEMPLATE_SELECTION_TYPE
    self.processorName = None
    self.site_properties = None
    self.processCTypes = []
    self.template = ""
    self.logger.debug("self.site.properties: " + str(self.site.properties))
    try:
      # Update site properties from batch item properties
      keys = [property["name"] for property in self.site.properties]
      self.logger.info("keys: %s" % str(keys))
      for key in self.batch_item.properties.keys():
        if key in keys:
          for property in self.site.properties:
            if property["name"]==key:
              self.logger.info("%s exist in site properties. Rewrite property" % key)
              self.logger.info("Old value: %s" % varDump(property["value"]))
              property["value"] = self.batch_item.properties[key]
              self.logger.info("New value: %s" % varDump(property["value"]))
        else:
          self.logger.info("%s don't exist in site properties. Add property" % key)
          self.site.properties.append({"name":key, "value":self.batch_item.properties[key], \
                                       "URLMd5":self.batch_item.urlId})
      self.logger.debug("Updated site's properties: " + str(self.site.properties))

      for property in self.site.properties:

        #TEMPLATE_SELECTION_TYPE
        if "TEMPLATE_SELECTION_TYPE" == property["name"]:
          self.template_selection_type = int(property["value"])
          self.logger.debug("TEMPLATE_SELECTION_TYPE: " + str(self.template_selection_type))
          self.logger.debug("HASH: %s, URL: %s" % (str(hashlib.md5(app.Utils.UrlParser.generateDomainUrl\
                                                                   (self.url.url)).hexdigest()),self.url.url))

        #PROCESS_CTYPES
        elif "PROCESS_CTYPES" == property["name"]:
          self.processCTypes = property["value"]
          self.logger.debug("PROCESS_CTYPES: " + str(self.processCTypes))

        #TIMEZONE
        elif "TIMEZONE" == property["name"]:
          self.timezone = property["value"]
          self.logger.debug("TIMEZONE: " + str(self.timezone))

        #REFINE_TAGS
        elif "TEXT_STATS" == property["name"]:
          #self.text_stats = property["value"].split(',')
          self.text_stats = int(property["value"])
          self.logger.debug("TEXT_STATS: " + str(self.text_stats))

        #PROCESSOR_NAME
        elif "PROCESSOR_NAME" == property["name"]:
          self.processorName = property["value"]
          self.logger.debug("PROCESSOR_NAME: " + str(self.processorName))

        #PROCESSOR_PROPERTIES
        elif "PROCESSOR_PROPERTIES" == property["name"]:
          self.processorProperties = property["value"]
          self.logger.debug("PROCESSOR_PROPERTIES: " + str(self.processorProperties))
      
        #CONTENT_HASH
        elif "CONTENT_HASH" == property["name"]:
          self.contentHash = property["value"]
          self.logger.debug("CONTENT_HASH: " + str(self.contentHash))

        #CONTENT_HASH
        elif "HTML_RECOVER" == property["name"]:
          self.htmlRecover = property["value"]
          self.logger.debug("HTML_RECOVER: " + str(self.htmlRecover))

        #template
        elif "template" == property["name"]:
          if self.template_selection_type == TEMPLATE_SELECTION_TYPE_DEFAULT:
            self.template = property["value"]
          elif (self.template_selection_type == TEMPLATE_SELECTION_TYPE_FULL_URLMD5) and \
            "URLMd5" in property and \
            property["URLMd5"] == self.batch_item.url.md5:
            self.template = property["value"]
          elif (self.template_selection_type == TEMPLATE_SELECTION_TYPE_DOMAIN_URLMD5) and \
            "URLMd5" in property and \
            property["URLMd5"] == hashlib.md5(app.Utils.UrlParser.generateDomainUrl(self.url.url)).hexdigest():
            self.logger.debug("HASH: %s, URl: %s" % (str(hashlib.md5(app.Utils.UrlParser.generateDomainUrl\
                                                                     (self.url.url)).hexdigest()),self.url.url))
            self.template = property["value"]
          self.logger.info("Template: " + varDump(self.template))

        # Collection support
        elif "COLLECTION_SUPPORT" == property["name"] and (int(property["value"])==1):
          self.collection_support = True
          self.logger.debug("COLLECTION_SUPPORT: " + str(self.collection_support))
      
      #debug info
      self.logger.debug("+++++++++++++++")
      self.logger.debug("TEMPLATE_SELECTION_TYPE: " + str(self.template_selection_type))
      self.logger.debug("HASH: %s, URL: %s" % (str(hashlib.md5(app.Utils.UrlParser.generateDomainUrl(self.url.url)).\
                                                               hexdigest()),self.url.url))
      self.logger.debug("PROCESS_CTYPES: " + str(self.processCTypes))
      self.logger.debug("TIMEZONE: " + str(self.timezone))
      self.logger.debug("TEXT_STATS: " + str(self.text_stats))
      self.logger.debug("PROCESSOR_NAME: " + str(self.processorName))
      self.logger.debug("CONTENT_HASH: " + str(self.contentHash))
      self.logger.info("Template: " + varDump(self.template))
      self.logger.info("Template type: " + str(type(self.template)))
      self.logger.debug("+++++++++++++++")

      # ckeck contentType and PROCESS_CTYPES
      if self.url.contentType not in self.processCTypes:
        self.ErrorMask |= ERROR_MASK_SITE_UNSUPPORTED_CONTENT_TYPE
        self.updateURLRecordFailed()
        raise ProcessorException("url ContentType not matched! url.contentType: <<%s>>, \
                                 site_properties.PROCESS_CTYPES: <<%s>>"\
                                 % (self.url.contentType, str(self.processCTypes)))
    except ProcessorException, err:
      self.logger.info(MSG_INFO_LOAD_SITE_PROPERTIES + str(err.message))
      raise err
    except Exception as err:
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.logger.error(MSG_ERROR_LOAD_SITE_PROPERTIES + str(err.message))
      raise err


  # #
  #
  def getProcessorCmd(self):
    # support PROCESSOR_NAME
    cmd = CONSTS.PYTHON_BINARY + " " + CONSTS.SCRAPER_BINARY + " " + CONSTS.SCRAPER_CFG
    if self.processorName is None or self.processorName == CONSTS.PROCESSOR_EMPTY:
      cmd = CONSTS.PYTHON_BINARY + " " + CONSTS.SCRAPER_BINARY + " " + CONSTS.SCRAPER_CFG
    elif self.processorName == CONSTS.PROCESSOR_STORE:
      cmd = CONSTS.PYTHON_BINARY + " " + CONSTS.STORE_PROCESSOR_BINARY + " " + CONSTS.STORE_PROCESSOR_CFG
    elif self.processorName == CONSTS.PROCESSOR_FEED_PARSER:
      cmd = CONSTS.PYTHON_BINARY + " " + CONSTS.SCRAPER_BINARY + " " + CONSTS.SCRAPER_CFG
      # Use default scraper for rss_feed site
      #cmd = CONSTS.PYTHON_BINARY + " " + CONSTS.PROCESSOR_FEED_PARSER_BINARY + " " + CONSTS.PROCESSOR_FEED_PARSER_CFG
    self.logger.debug(MSG_INFO_PROCESSOR_CMD + cmd)
    return cmd


  # #
  #
  def getProcessedContent(self):
    buffer = {}
    for key in self.scraper_response.processedContent.keys():
      self.scraper_response.processedContent[key].get()
      buffer[key] = self.scraper_response.processedContent[key].data
    self.processedContent = encode(json.dumps(buffer))


  # #
  #
  def textStats(self):
    flag = True
    try:
      self.logger.info("textStats has started")
      self.logger.info("Scraper's response: %s" % varDump(self.scraper_response))
      content_dict = self.scraper_response.processedContent
      self.logger.info("Returned content's keys: %s" % str(content_dict.keys()))
    except Exception as err:
      self.logger.debug("textStats has failed: %s", str(err))
    return flag


  # #
  #
  def doRefine(self, refiner):
    flag = True
    try:
      self.logger.debug("Refiner is: %s", str(refiner))
      cmd = ""
      process = Popen(cmd, stdout=PIPE, stdin=PIPE, stderr=PIPE, shell=True, close_fds=True)
      (output, err) = process.communicate(input=self.result_object)
    except Exception as err:
      self.logger.debug("Refine has failed: %s", str(err))
    return flag


  # #
  #
  def process(self):
    try:
      scraper_input_object = ScraperInData(self.url.url, \
                                           self.batch_item.urlId, \
                                           self.batch_item.siteId, \
                                           self.raw_content, \
                                           self.template["tags"], \
                                           self.filters, \
                                           self.url.lastModified, \
                                           self.timezone, \
                                           self.batch_item,
                                           self.processorProperties)
      # self.logger.info("scraper_input_object: " + str(vars(scraper_input_object)))
      input_pickled_object = pickle.dumps(scraper_input_object)
      Utils.storePickleOnDisk(input_pickled_object, ENV_PROCESSOR_STORE_PATH, "processor.out." + self.batch_item.urlId)
      # self.logger.info("input_pickled_object: " + str(input_pickled_object))

      # sleep to reduce system load
      time.sleep(self.url.processingDelay / 1000.0)

      # support PROCESSOR_NAME
      cmd = self.getProcessorCmd()
      process = Popen(cmd, stdout=PIPE, stdin=PIPE, stderr=PIPE, shell=True, close_fds=True)
      # scraper_response = Popen(cmd, stdin=PIPE, stdout=sys.stdout, shell=True).communicate(input=input_pickled_object)
      (output, err) = process.communicate(input=input_pickled_object)
      #self.content = pickle.loads(output)
      # self.logger.debug("scraper_response pickle: %s", str(output))
      self.logger.debug("scraper_response error: %s", str(err))
      # open("scraper_input.bin", "w").write(input_pickled_object)
      exit_code = process.wait()
      self.logger.debug("scraper_response exit_code: %s", str(exit_code))
      # finish processing if scraper return exit_code EXIT_FAILURE
      if exit_code == EXIT_FAILURE:
        self.logger.error("Scraper has failed.")
        raise ProcessorException("Scraper has failed.")

      self.scraper_response = pickle.loads(output)
      self.logger.debug("scraper_response: %s", varDump(self.scraper_response))
      self.logger.debug("tagsCount: %s", str(self.scraper_response.tagsCount))
      self.logger.debug("tagsMask: %s", str(self.scraper_response.tagsMask))
      #self.logger.debug("processedContent: %s", self.scraper_response.processedContent)
      self.logger.info(MSG_INFO_PROCESSOR_EXIT_CODE + str(exit_code))
      # self.logger.info(MSG_INFO_PROCESSOR_OUTPUT + str(output))
      self.logger.info(MSG_INFO_PROCESSOR_ERROR + str(err))


      self.result_object = output
    except Exception as err:
      tbi = Utils.getTracebackInfo()
      self.logger.error(MSG_ERROR_PROCESS + str(vars(err)) + "\n" + tbi)
      raise err
    return Results(exit_code, output, err)


  # #
  #
  def getRawContentFromFS(self):
    self.raw_content = None
    try:
      # Check if Real-Time crawling with ONLY_PROCESSING algorithm
      if self.input_batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER and \
        self.batch_item.urlObj.urlPut is not None:
        self.raw_content = self.batch_item.urlObj.urlPut.putDict["data"]
        if self.batch_item.urlObj.urlPut.contentType == dc_event.Content.CONTENT_RAW_CONTENT:
          self.raw_content = base64.b64decode(self.raw_content)
      else:
        if self.htmlRecover is not None and self.htmlRecover == "1":
          #---------------------------
          urlContentObj = dc_event.URLContentRequest(self.batch_item.siteId, self.batch_item.urlObj.url, 
                                                     contentTypeMask=dc_event.URLContentRequest.CONTENT_TYPE_RAW_LAST + \
                                                     dc_event.URLContentRequest.CONTENT_TYPE_TIDY)
          self.raw_content = self.wrapper.urlContent([urlContentObj])
          if len(self.raw_content[0].rawContents) > 0:
            self.logger.info(">>> YES tidy on disk")
          else:
            self.logger.info(">>> NO tidy on disk")
          #---------------------------
        else:
          urlContentObj = dc_event.URLContentRequest(self.batch_item.siteId, self.batch_item.urlObj.url, 
                                                     contentTypeMask=dc_event.URLContentRequest.CONTENT_TYPE_RAW_LAST + \
                                                     dc_event.URLContentRequest.CONTENT_TYPE_RAW)
          self.raw_content = self.wrapper.urlContent([urlContentObj])
        # Decode buffer
        self.raw_content = decode(self.raw_content[0].rawContents[0].buffer)
    except Exception as err:
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.ErrorMask = ERROR_MASK_MISSED_RAW_CONTENT_ON_DISK
      self.logger.error(MSG_ERROR_GET_RAW_CONTENT_FROM_DB + err.message)
      raise ProcessorException(MSG_ERROR_GET_RAW_CONTENT_FROM_DB)


  # #
  #
  def convertRawContentCharset(self):
    try:
      #self.logger.info("self.raw_content: %s" % str(self.raw_content))
      self.charset = icu.CharsetDetector(self.raw_content).detect().getName()  # pylint: disable=E1101
      self.logger.info("Charset incoming raw data: " + self.charset)
      self.raw_content = self.raw_content.decode(self.charset)
    except Exception, err:
      self.logger.debug(MSG_ERROR_CONVERT_RAW_CONTENT_CHARSET + err.message)
      self.ErrorMask |= ERROR_MASK_URL_ENCODING_ERROR
      self.updateURLRecordFailed()
      #raise ProcessorException(MSG_ERROR_CONVERT_RAW_CONTENT_CHARSET)


  # #
  #
  def updateURLCharset(self):
    try:
      urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
      #urlUpdateObj.UDate = SQLExpression("NOW()")
      urlUpdateObj.charset = self.charset
      updated_rows = self.wrapper.urlUpdate(urlUpdateObj)
    except Exception as err:
      self.logger.error(MSG_ERROR_UPDATE_PROCESSED_URL + err.message)
      raise err


  # readFilters read pattern from sites_filters
  #
  def readFilters(self):
    self.filters = self.site.filters
    self.logger.debug("sites_filters: " + varDump(self.filters))


  # #
  #
  def updateProcessedURL(self):
    # check url state
    state = URL.STATE_ENABLED
    processed_time = int((time.time() - self.process_time) * 1000)

    if self.scraper_response:
      tagsMask = self.scraper_response.tagsMask or 0
      tagsCount = self.scraper_response.tagsCount or 0
      self.pubdate = self.scraper_response.pubdate or None
    else:
      tagsMask = 0
      tagsCount = 0
    try:
      urlUpdateList = []
      urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
      urlUpdateObj.status = URL.STATUS_PROCESSED
      urlUpdateObj.state = state
      urlUpdateObj.errorMask = SQLExpression("`ErrorMask` + %s" % str(self.ErrorMask))
      urlUpdateObj.tagsMask = tagsMask
      urlUpdateObj.tagsCount = tagsCount
      urlUpdateObj.totalTime = SQLExpression(("`TotalTime` + %s" % str(processed_time)))
      urlUpdateObj.processingTime = processed_time
      urlUpdateObj.pDate = str(self.pubdate) if self.pubdate is not None else None
      urlUpdateObj.UDate = SQLExpression("NOW()")
      urlUpdateObj.tcDate = SQLExpression("NOW()")
      urlUpdateObj.contentURLMd5 = self.contentURLMd5
      urlUpdateList.append(urlUpdateObj)

      if (self.processorName == CONSTS.PROCESSOR_FEED_PARSER) \
        and (dc_event.BatchItem.PROP_FEED in self.batch_item.properties):
        for url, value in self.batch_item.properties[dc_event.BatchItem.PROP_FEED].items():
          urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, value["urlMd5"], dc_event.URLStatus.URL_TYPE_MD5)
          urlUpdateObj.status = URL.STATUS_PROCESSED
          urlUpdateObj.state = state
          urlUpdateObj.errorMask = self.ErrorMask
          urlUpdateObj.tagsMask = tagsMask
          urlUpdateObj.tagsCount = tagsCount
          urlUpdateObj.totalTime = SQLExpression(("`TotalTime` + %s" % str(processed_time)))
          urlUpdateObj.processingTime = processed_time
          urlUpdateObj.pDate = str(self.pubdate)
          urlUpdateObj.UDate = SQLExpression("NOW()")
          urlUpdateObj.contentURLMd5 = self.contentURLMd5
          urlUpdateList.append(urlUpdateObj)
      result = self.wrapper.urlUpdate(urlUpdateList)
    except Exception as err:
      self.logger.error(MSG_ERROR_UPDATE_PROCESSED_URL + err.message)
      raise err


  # addResource add Resources counter for sites table
  def updateSiteResources(self):
    pass
    '''
    try:
      localSiteUpdate = dc_event.SiteUpdate(self.batch_item.siteId)
      for attr in localSiteUpdate.__dict__:
        if hasattr(localSiteUpdate, attr):
          setattr(localSiteUpdate, attr, None)
      localSiteUpdate.id = self.batch_item.siteId
      #localSiteUpdate.tcDate = SQLExpression("NOW()")
      updated_count = self.wrapper.siteNewOrUpdate(localSiteUpdate)
    except Exception as err:
      self.logger.error(MSG_ERROR_UPDATE_SITE_RESOURCES + err.message)
      raise err
    '''


  # # processContentHash
  #
  def processContentHash(self):
    try:
      if self.contentHash:
        self.logger.debug("Site has content hash rule: %s", str(self.contentHash))
        self.contentHash = json.loads(self.contentHash)
        self.list_hashed_tags = self.contentHash["tags"].split(",")
        self.contentURLMd5Algorithm = self.contentHash["algorithm"]
        self.contentURLMd5Action = self.contentHash["delete"]
        self.logger.debug("List hashed tags: %s", str(self.list_hashed_tags))
        self.calcContentHash()
        self.checkContentHashDuplicate()
        self.logger.debug("content hash buffer: %s", str(self.hashed_content))
      else:
        self.logger.debug("Site hasn't content hash rule: %s", str(self.contentHash))
    except ProcessorException as err:
      self.logger.debug(MSG_ERROR_CALC_CONTENT_HASH + " : " + err.message)


  ## checkContentHashDuplicate
  #
  def checkContentHashDuplicate(self):
    try:
      query = "SELECT * FROM `%s` WHERE `ContentURLMd5`='%s' AND `ParentMd5`<>''" % (self.URLTable, self.contentURLMd5)
      row = self.wrapper.customRequest(query, DB_URLS)
      if len(row):
        self.logger.info("Duplicate exist: %s", str(row))
        if self.contentURLMd5Action == CONSTS.CONTENT_HASH_ACTION_DELETE and\
          self.batch_item.urlObj.parentMd5 != CONSTS.PARENT_URL_MD5:
	  # TODO reimplement self.deleteDuplicate() method later
          #self.deleteDuplicate()
	  pass
      else:
        self.logger.info("Duplicate doesn't exist: %s", str(row))
    except ProcessorException as err:
      self.logger.debug(MSG_ERROR_CHECK_CONTENT_HASH_DUPLICATE + " : " + err.message)


  # # checkContentHashDuplicate
  #
  def deleteDuplicate(self):
    # Create new URLDelete object
    urlDelete = EventObjects.URLDelete(self.batch_item.siteId, self.batch_item.urlId, 
                                       EventObjects.URLStatus.URL_TYPE_MD5,
				       reason=EventObjects.URLDelete.REASON_PROCESSOR_DUPLICATE)
    cfgParser = ConfigParser.ConfigParser()
    cfgParser.read(self.db_task_ini)
    dbTask = DBTasksManager(cfgParser)
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_DELETE, [urlDelete])
    responseDRCESyncTasksCover = dbTask.process(drceSyncTasksCoverObj)


  # # getContentHash
  #
  def getContentHash(self):
    if self.contentURLMd5Algorithm == CONSTS.CONTENT_HASH_ALGORITHM_MD5:
      self.contentURLMd5 = str(hashlib.md5(self.hashed_content).hexdigest())
    elif self.contentURLMd5Algorithm == CONSTS.CONTENT_HASH_ALGORITHM_MD5_WITHOUT_HTML:
      self.stripHTML()
      self.contentURLMd5 = str(hashlib.md5(self.hashed_content).hexdigest())
    else:
      self.logger.debug("Algorithm <<%s>> for calculating contentURLMd5 not supported.", 
                        str(self.contentURLMd5Algorithm))


  # # calcContentHash
  #
  def calcContentHash(self):
    self.hashed_content = ""
    self.contentURLMd5 = ""
    try:
      if self.contentURLMd5Algorithm != CONSTS.CONTENT_HASH_ALGORITHM_EMPTY:
        tag_list = []
        if len(self.scraper_response.processedContent["default"].data["data"]["tagList"])>0:
          tag_list = self.scraper_response.processedContent["default"].data["data"]["tagList"][0]
        for tag in tag_list:
          if tag["name"] in self.list_hashed_tags:
            self.logger.debug("Tag name added to hash: %s", str(tag["name"]))
            self.logger.debug("Tag value added to hash: %s", str(tag["data"]))
            self.hashed_content = self.hashed_content + str(tag["data"])
          else:
            self.logger.debug("Tag: %s not added to hash", tag["name"])
        self.getContentHash()
    except ProcessorException as err:
      self.logger.debug(MSG_ERROR_CHECK_CONTENT_HASH + " : " + err.message)


  # # calcContentHash
  #
  def stripHTML(self):
    # remove non-printable characters
    self.hashed_content = re.sub("(\n*)(\t*)(\r*)", "", self.hashed_content)
    # remove whole javascript
    self.hashed_content = re.sub("<script(.*?)>(.*?)</script>", "", self.hashed_content)
    # remove comments
    self.hashed_content = re.sub("<!--(.*?)-->", "", self.hashed_content)
    #remove HTML tags
    self.hashed_content = re.sub("<(.*?)>", "", self.hashed_content)
    #remove spaces
    self.hashed_content = re.sub(" ", "", self.hashed_content)


  ## convertTemplateFormat
  #
  def convertTemplateFormat(self):
    self.logger.info("Template before convertion: %s" % varDump(self.template))
    template = copy.copy(self.template)
    self.template = {"templates":[{"priority":100, "mandatory":1, "is_filled":0, "tags":template}]}
    self.batch_item.properties["template"] = self.template
    self.logger.info("Template after convertion: %s" % varDump(self.template))


  ## mapResponse
  #
  def mapResponse(self):
    # I. TODO reduce phase
    self.do_reduce = True
    self.logger.info("Output format: %s" % varDump(self.template["output_format"]))
    
    # II. Fill header
    self.template["response"] = self.template["output_format"]["header"]

    # III. Fill item
    self.template["response"] = self.template["response"] + self.template["output_format"]["items_header"]
    entry = self.template["output_format"]["item"]
    self.template["contentsCount"] = 0
    self.template["contentsLen"] = 0
    self.template["isEmpty"] = True
    for data in self.scraper_response.processedContent["default"].data["data"]["tagList"][0]:
      #data = self.scraper_response.processedContent["default"].data["data"]["tagList"][0][0]
      self.logger.info("Output: %s" % varDump(data))
      if self.template["output_format"]["item"].find(data["name"])!=-1:
        for item in data["data"]:
          pattern = "%"+data["name"]+"%"
          self.logger.info("pattern: %s" % str(pattern))
          self.logger.info("item: %s" % str(item))
          self.logger.info("A: %s" % str(self.template["output_format"]["item"]))
          pattern = pattern.replace("\\", "\\\\")
          item = item.replace("\\", "\\\\")
          self.template["contentsLen"] += len(item)
          if len(item) > 0:
            self.template["contentsCount"] += 1
            self.template["isEmpty"] = False
          entry = re.sub(pattern, item, entry)
    self.template["response"] = self.template["response"] + entry + self.template["output_format"]["items_footer"]
    # IV. Fill footer
    self.template["response"] = self.template["response"] + self.template["output_format"]["footer"]

    self.logger.info("Output response: %s" % varDump(self.template["response"]))


  ## reduceResponse
  #
  def reduceResponse(self, processingTamplatesDict, templateSelectType):
    self.logger.info("Output response: %s" % varDump(processingTamplatesDict))
    #self.processedContent = encode(self.template["response"])
    maxVal = -1
    ret = None
    for elem in processingTamplatesDict:
      if "mandatory" in elem and elem["mandatory"] == 1 and "isEmpty" in elem and elem["isEmpty"]:
        elem["response"] = "<<< TEMPLATE ERROR >>>"
        ret = elem
        break
      if "contentsCount" in elem and "contentsLen" in elem:
        if templateSelectType == "first_nonempty":
          if elem["contentsCount"] > maxVal:
            maxVal = elem["contentsCount"]
            ret = elem
        elif elem["contentsLen"] > maxVal:
          maxVal = elem["contentsLen"]
          ret = elem
      else:
        ret = elem
    if ret is not None:
      self.template = ret
    self.processedContent = encode(self.template["response"])


  # #processTask
  #
  def processTask(self):
    try:
      self.updateURLCharset()
      self.readFilters()

      # Parse templates
      self.logger.info("self.template: %s" % varDump(self.template))
      self.logger.info("type of self.template is = " + str(type(self.template)))
      if type(self.template) in types.StringTypes and self.template != "":
        self.template = json.loads(self.template)
      if "templates" in self.template:
        # If new template format do nothing
        self.logger.info("NEW template format")
        if "template" not in self.batch_item.properties:
          self.batch_item.properties["template"] = copy.deepcopy(self.template)
      else:
        # Conver old template format to new one
        self.logger.info("OLD template format")
        self.convertTemplateFormat()

      # Step I: reorder templates order by priority
      #self.logger.info("Template order before: %s" % varDump(self.template))
      self.template["templates"] = sorted(self.template["templates"], key=lambda template: template["priority"], 
                                          reverse=True)
      #self.logger.info("Template order after: %s" % varDump(self.template))

      # Step II: process each template
      processingTemplatesDict = []
      templateSelectType = self.template["select"] if "select" in self.template else "first_nonempty"
      for self.template in self.template["templates"]:
        self.logger.info(">>> Processing template %s" % varDump(self.template))
        if "state" in self.template and not bool(int(self.template["state"])):
          self.logger.info(">>> Template disable")
          continue
        # Anything can happen but
        # we don't bother what happeh - just go further
        try:
          self.process()
        except:
          continue

        # Step III Convert response to template's output format
        # extract content from scraper
        if "output_format" in self.template:
          self.logger.info("Template has output format")
          self.mapResponse()

        processingTemplatesDict.append(copy.deepcopy(self.template))
      # Step IV Reduce response to best one

      if self.do_reduce:
        self.reduceResponse(processingTemplatesDict, templateSelectType)
      else:
        self.getProcessedContent()

      # Send to text stats
      if self.text_stats:
        self.textStats()

      # Put URL's content to DB
      self.putContent()

      # COLLECTION SUPPORT
      if self.collection_support:
        self.convertContent2Collection()

    except ProcessorException as err:
      self.logger.debug(MSG_ERROR_PROCESS_TASK + " : " + err.message)
      self.ErrorMask = ERROR_MASK_SCRAPER_ERROR
      raise err
    except Exception as err:
      self.logger.error(MSG_ERROR_PROCESS_TASK + " : " + err.message)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.ErrorMask = ERROR_MASK_SCRAPER_ERROR


  ## Put content to DB
  #
  def putContent(self):
    # URLPut
    # Create new URLPut object
    putDict = {}
    putDict["id"] = self.batch_item.urlId
    putDict["data"] = self.processedContent
    urlPut = EventObjects.URLPut(self.batch_item.siteId,
                                 self.batch_item.urlId,
                                 dc_event.Content.CONTENT_PROCESSOR_CONTENT,
                                 putDict)
    #Check if Real-Time crawling
    if self.input_batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER and \
      "DB_TASK_MODE" in self.batch_item.properties and \
      self.batch_item.properties["DB_TASK_MODE"] == "RO":
      self.batch_item.urlPutObj = urlPut
    else:
      drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_PUT, [urlPut])
      responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
      if responseDRCESyncTasksCover.eventType == DC_CONSTS.EVENT_TYPES.URL_PUT_RESPONSE:
        for obj in responseDRCESyncTasksCover.eventObject:
          self.logger.info("URL_PUT_RESPONSE: %s" % varDump(obj))
      else:
        logger.error("URL_PUT_RESPONSE >>> Wrong response type")
  
  # #updateURL
  #
  def convertContent2Collection(self):
    collectionConverter = CollectionConverter(self.input_batch, self.batch_item, self.wrapper)
    self.batch_item = collectionConverter.process()


  # #updateURL
  #
  def updateURL(self):
    try:
      # updated by Oleksii
      # if insert into kv db wasn't successfully
      # not update Processed counter
      processed_counter = 1
      if self.ErrorMask == ERROR_MASK_SCRAPER_ERROR:
        processed_counter = 0
      urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
      urlUpdateObj.processed = SQLExpression("`Processed`+" + str(processed_counter))
      urlUpdateObj.tcDate = SQLExpression("NOW()")
      urlUpdateObj.status = URL.STATUS_PROCESSING
      self.wrapper.urlUpdate(urlUpdateObj)
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise err


  # #update site params
  # update site params
  # @param site - site object
  def updateSite(self):
    try:
      localSiteUpdate = dc_event.SiteUpdate(self.batch_item.siteId)
      for attr in localSiteUpdate.__dict__:
        if hasattr(localSiteUpdate, attr):
          setattr(localSiteUpdate, attr, None)
      localSiteUpdate.id = self.batch_item.siteId
      localSiteUpdate.tcDate = SQLExpression("NOW()")
      localSiteUpdate.errorMask = SQLExpression("`ErrorMask` | " + str(self.ErrorMask))
      localSiteUpdate.state = Site.STATE_SUSPENDED
      updated_count = self.wrapper.siteNewOrUpdate(localSiteUpdate)
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise err


  # #update site params
  # update site params
  # @param site - site object
  def updateURLRecordFailed(self):
    try:
      # updated by Oleksii
      # we shouldn't set state to error
      # cause it checked and rejected after many algorithms
      urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
      urlUpdateObj.errorMask = SQLExpression("`ErrorMask` | " + str(self.ErrorMask))
      urlUpdateObj.UDate = SQLExpression("NOW()")
      self.wrapper.urlUpdate(urlUpdateObj)
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise err


  # #check site params
  # check site params
  # @param site - site object
  def checkSite(self):
    #self.ErrorMask = ERROR_MASK_SITE_OK
    if self.site.state == Site.STATE_DISABLED:
      raise ProcessorException("Site state is not active! Actual site state is: %s. " % self.site.state)

    # pass checking if it is first processing, but not re-crawling
    if self.site.maxResources and self.site.contents >= self.site.maxResources and self.url.processed == 0:# pylint: disable=C0326
      self.ErrorMask |= ERROR_MASK_SITE_MAX_RESOURCES
      self.logger.debug("Site maxResources is reached! Site contents is: %s. Site maxResources: %s " \
                        % (self.site.contents, self.site.maxResources))
      if self.ErrorMask != ERROR_MASK_SITE_OK:
        raise ProcessorException(MSG_ERROR_CHECK_SITE)


  # #
  #
  def readSiteFromDB(self):
    try:
      #Add support db-task Site
      siteStatus = dc_event.SiteStatus(self.batch_item.siteId)
      drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.SITE_STATUS, siteStatus)
      responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
      self.site = responseDRCESyncTasksCover.eventObject
    except Exception, err:
      self.logger.error(MSG_ERROR_READ_SITE_FROM_DB + str(err.message))
      raise err
    #self.logger.info("site object: " + varDump(self.site))


  # #
  #
  def loadSite(self):
    try:
      if not len(self.batch_item.siteId):
        self.batch_item.siteId = "0"
      self.readSiteFromDB()
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise err


  # #
  #
  #
  def readUrlFromDB(self):
    urlStatus = dc_event.URLStatus(self.batch_item.siteId, self.batch_item.urlId)
    urlStatus.urlType = dc_event.URLStatus.URL_TYPE_MD5
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_STATUS, [urlStatus])
    responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
    row = responseDRCESyncTasksCover.eventObject
    self.table_name = self.URLTable
    try:
      if len(row):
        self.url = row[0]
        if self.url.errorMask & ERROR_MIME_NOT_WRITE:
          raise ProcessorException("MIME Content-Type not allow store content to disk")
      else:  # throw if url doesn't exists
        raise ProcessorException("URL entry doesn't exist in table " + str(self.table_name))
    except Exception, err:
      self.logger.debug(MSG_ERROR_LOAD_URL_DATA)
      raise err
    self.logger.info("url object: " + varDump(self.url))


  # #
  #
  def loadURL(self):
    # do siteId
    self.URLConnector = self.dc_urls_db_connect
    if len(self.batch_item.siteId):
      self.URLTable = DC_URLS_TABLE_PREFIX + self.batch_item.siteId
    else:
      self.URLTable = DC_URLS_TABLE_PREFIX + "0"
    self.logger.info("db backend Table for siteId %s is: %s" % (self.batch_item.siteId, self.URLTable))
    self.readUrlFromDB()


  def resetVars(self):
    self.scraper_response = None
    #self.input_batch = None
    self.pubdate = None
    self.timezone = None
    self.text_stats = True
    self.contentHash = None
    self.list_hashed_tags = None
    self.contentURLMd5 = ""
    self.contentURLMd5Algorithm = None
    self.contentURLMd5Action = None
    self.ErrorMask = ERROR_MASK_SITE_OK
    self.template_selection_type = TEMPLATE_SELECTION_TYPE_DEFAULT
    self.processorProperties = None
    self.collection_support = False
    self.do_reduce = False


  # #process task
  # main resource processing
  # item is the batch item
  # @param item - the batch item
  def processBatchItem(self, batch_item):
    if not batch_item: return batch_item
    start_batch_item_time = time.time()
    response = None
    self.process_time = time.time()
    self.batch_item = batch_item
    self.logger.info("Batch item: siteId: %s" % (batch_item.siteId))
    self.logger.info("Batch item: urlId: %s" % (batch_item.urlId))
    self.logger.info("Batch item: url: %s" % (batch_item.urlObj.url))
    try:
      # update site list from batch
      self.batch_sites.update([batch_item.urlObj.siteId])

      self.resetVars()
      self.loadSite()
      # Check if Real-Time crawling
      if self.input_batch.crawlerType==dc_event.Batch.TYPE_REAL_TIME_CRAWLER and \
        "DB_TASK_MODE" in self.batch_item.properties and \
        self.batch_item.properties["DB_TASK_MODE"] == "RO":
        self.url = self.batch_item.urlObj
        self.wrapper.affect_db = False
      else:
        self.loadURL()
      self.checkSite()

      #Check if Real-Time crawling resource already crawled
      if self.url.processed!=0 and \
        self.url.errorMask==ERROR_MASK_NO_ERRORS and \
        self.url.tagsCount>0 and \
        self.ErrorMask==ERROR_MASK_NO_ERRORS:
        self.logger.debug("Real time crawling. Check reprocessing.")
        self.logger.debug("Batch item properties: %s" % varDump(batch_item.properties))
        
        # Check if need to process resource
        if ("PROCESSOR_NAME" in batch_item.properties) and \
          (batch_item.properties["PROCESSOR_NAME"]=="NONE"):
          self.logger.debug("RealTime Crawling: Only crawling mode. Exit.")
          return batch_item
        
        
        if (CONSTS.REPROCESS_KEY in batch_item.properties) and \
          (batch_item.properties[CONSTS.REPROCESS_KEY]==CONSTS.REPROCESS_VALUE_NO):
          self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty."+
                            "Don't need to reprocess.")
          return batch_item
        else:
          self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty but"+
                             "properties reprocess is Yes or empty. Send to reprocessing.")

      #Check if content stored on disk
      if self.batch_item.urlObj.contentMask!=dc_event.URL.CONTENT_STORED_ON_DISK:
        self.logger.debug("Content not found on disk. Exit.")
        return batch_item

      self.loadSiteProperties()
      self.updateURL()
      self.getRawContentFromFS()
      self.convertRawContentCharset()
      self.processTask()
      self.processContentHash()
      self.updateSiteResources()
      response = batch_item
      batch_item_processing_time = int((time.time() - start_batch_item_time) * 1000)
      self.logger.debug("Batch item processing time: %s msec." % (batch_item_processing_time))
    except ProcessorException, err:
      self.logger.info(MSG_INFO_PROCESS_BATCH_ITEM + err.message)
    except Exception as err:
      self.logger.error(MSG_ERROR_PROCESS_BATCH_ITEM + err.message)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.exit_code = EXIT_FAILURE
    self.updateProcessedURL()
    #self.logger.info("response: " + varDump(response))
    return response


  # #process batch
  # the main processing of the batch object
  def processBatch(self):
    try:
      start_batch_time = time.time()

      # read pickled batch object from stdin and unpickle it
      #self.logger.debug("Start to read from stdin")
      input_pickled_object = sys.stdin.read()
      #self.logger.debug("Input pickled object:\n %s", str(input_pickled_object))
      #self.logger.debug("End to read from stdin")

      input_batch = pickle.loads(input_pickled_object)
      self.input_batch = input_batch
      Utils.storePickleOnDisk(input_pickled_object, ENV_PROCESSOR_STORE_PATH, "processor.in." + 
                              str(self.input_batch.id))
      if self.input_batch.id == 0:
        raise Exception("Empty batch items: maybe previous app had failed. ")
      self.logger.info("input_batch items: " + str(self.input_batch.items))
      # TODO main processing over every url from list of urls in the batch object
      batch_items = [self.processBatchItem(batch_item) for batch_item in self.input_batch.items]
      self.logger.info("Output batch items: " + str(batch_items))
      if input_batch.crawlerType == dc_event.Batch.TYPE_REAL_TIME_CRAWLER:
        input_batch.items = batch_items
        process_task_batch = input_batch
      else:
        process_task_batch = Batch(input_batch.id, batch_items)
      self.logger.info("process_task_batch: " + varDump(process_task_batch))
      # Recalculate Contents
      self.wrapper.ContentsRecalculating(self.batch_sites)

      # send response to the stdout
      batch_processing_time = int((time.time() - start_batch_time) * 1000)
      self.logger.debug("Batch processing time: %s msec." % (batch_processing_time))
    except ProcessorException, err:
      self.logger.info(MSG_INFO_PROCESS_BATCH + err.message)
    except Exception.EOFError, err:  # pylint: disable-all
      self.logger.error(MSG_ERROR_EMPTY_BATCH)
    except Exception as err:
      self.logger.error(MSG_ERROR_PROCESS_BATCH + err.message, exc_info=True)
    output_pickled_object = pickle.dumps(process_task_batch)
    Utils.storePickleOnDisk(output_pickled_object, ENV_PROCESSOR_STORE_PATH, "processor.out." + str(input_batch.id))
    sys.stdout.write(output_pickled_object)
    sys.stdout.flush()
    # print pickle.dumps(process_task_batch)


  # #load db backend
  # load from cli argument or default config file
  def loadDBBackend(self):
    try:
      className = self.__class__.__name__
      dbHost = self.config.get(className, CONSTS.DB_HOST)
      dbPort = int(self.config.get(className, CONSTS.DB_PORT))
      dbUser = self.config.get(className, CONSTS.DB_USER)
      dbPWD = self.config.get(className, CONSTS.DB_PWD)
      db_dc_sites = self.config.get(className, CONSTS.DB_SITES)
      db_dc_urls = self.config.get(className, CONSTS.DB_URLS)

      self.dc_sites_db_connect = mdb.connect(dbHost, dbUser, dbPWD, db_dc_sites, dbPort)
      self.dc_urls_db_connect = mdb.connect(dbHost, dbUser, dbPWD, db_dc_urls, dbPort)
      self.site_connector = self.dc_sites_db_connect
      self.URLConnector = self.dc_urls_db_connect
    except Exception, err:
      self.logger.error(CONSTS.MSG_ERROR_LOAD_DB_BACKEND)


  # #load config from file
  # load from cli argument or default config file
  def loadConfig(self):
    try:
      self.config = ConfigParser.ConfigParser()
      self.config.optionxform = str
      self.config.read(self.pargs.config)
    except Exception, err:
      print MSG_ERROR_LOAD_CONFIG + err.message
      raise err


  # #load logging
  # load logging configuration (log file, log level, filters)
  #
  def loadLogConfigFile(self):
    try:
      log_conf_file = self.config.get("Application", "log")
      logging.config.fileConfig(log_conf_file)
      # Logger initialization
      self.logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
      # Replace logger name for processes instances
      lfn = Utils.LoggerFileName()
      fname = lfn.findReplace()
      if fname != "":
        pin = lfn.getFreeProcInstanceNumber(os.path.basename(fname))
        if pin != "" and pin != "0":
          pin = "." + pin + ".log"
          fname = lfn.findReplace(fname + pin)
          self.logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

    except Exception, err:
      print MSG_ERROR_LOAD_LOG_CONFIG_FILE + err.message
      raise err


  # #load mandatory options
  # load mandatory options
  #
  def loadOptions(self):
    try:
      self.raw_data_dir = self.config.get(self.__class__.__name__, "raw_data_dir")
      self.db_task_ini = self.config.get(self.__class__.__name__, "db-task_ini")
      # Add support operations updateCollectedURLs and removeURLs
      cfgParser = ConfigParser.ConfigParser()
      cfgParser.read(self.db_task_ini)
      self.wrapper = DBTasksWrapper(cfgParser)
    except Exception, err:
      print MSG_ERROR_LOAD_OPTIONS + err.message
      raise err


  # #get exit code
  # return extt code
  # 0 if no critical errors was encountered during processing
  # 1 if smth bad happened
  #
  def getExitCode(self):
    return self.exit_code
