# coding: utf-8  # pylint: disable-all

"""@package docstring
 @file Scraper.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


import os
import re
import sys
import time
import json
import datetime
import types
from lepl.apps.rfc3696 import HttpUrl
import ast
import pickle
import logging.config
import ConfigParser
import MySQLdb as mdb
import MySQLdb.cursors
import xml.sax.saxutils
#import sqlite3  # as sqlite3
from email.Utils import formatdate
from dateutil.parser import *
from dateutil import parser
from urlparse import urljoin
from urlparse import urlparse
from contextlib import closing
import app.Utils as Utils  # pylint: disable=F0401
from app.Utils import varDump

from scrapy.selector import Selector
from cement.core import foundation

# scraper's modules
from newspaper_extractor import NewspaperExtractor
from goose_extractor import GooseExtractor
from scrapy_extractor import ScrapyExtractor
from ml_extractor import MLExtractor
# AlchemyAPI ectractor (proprietary)
try:
  from alchemy_extractor import AlchemyExtractor
except ImportError:
  pass
# Boilerpipe ectractor (need java)
try:
  from boilerpipe_extractor import BoilerpipeExtractor
except ImportError:
  pass

from scraper_utils import encode
from scraper_resource import Resource as Res
from scraper_result import Result as Result
from dc_processor.ScraperResponse import ScraperResponse
import app.Utils as Utils  # pylint: disable=F0401
from dc.EventObjects import SiteFilter
from dc_db.TasksManager import TasksManager as DBTasksManager
from dc import EventObjects
import app.Consts as APP_CONSTS
from scrapy.selector import Selector
import dc_processor.Constants as CONSTS
import dc.Constants as DC_CONSTS
from app.Utils import SQLExpression
from dc_processor.Metrics import WordsMetric
from dc_processor.Metrics import SentencesMetric
from dc_processor.Metrics import ARIMetric
from dc_processor.Metrics import CustomMetric

APP_NAME = "scraper"

MSG_ERROR_LOAD_CONFIG = "Error loading config file. Exciting."
MSG_ERROR_LOAD_LOG_CONFIG_FILE = "Error loading logging config file. Exiting."
MSG_ERROR_LOAD_EXTRACTORS = "Error load extractors "
MSG_ERROR_TEMPLATE_EXTRACTION = "Error template extraction "
MSG_ERROR_DYNAMIC_EXTRACTION = "Error dynamic extraction "
MSG_ERROR_LOAD_DB_BACKEND = "Error load db backend"
MSG_ERROR_LOAD_OPTIONS = "Error load options"
MSG_INFO_PREPARE_CONTENT = "Prepare content: "
MSG_ERROR_ADJUST_PR = "Error adjust partial references. "
MSG_ERROR_ADJUST_PUBDATE = "PUBDATE_ERROR "
MSG_ERROR_ADJUST_TITLE = "Can't adjust title. "

EXIT_SUCCESS = 0
EXIT_FAILURE = 1

SQLITE_TIMEOUT = 30

ENV_SCRAPER_STORE_PATH = "ENV_SCRAPER_STORE_PATH"
# #Scraper
#
#
class Scraper(foundation.CementApp):


  # Mandatory
  class Meta(object):
    label = APP_NAME
    def __init__(self):
      pass


  # #constructor
  # initialize default fields
  def __init__(self):
    # call base class __init__ method
    foundation.CementApp.__init__(self)
    self.exit_code = EXIT_SUCCESS
    self.itr = None
    self.extractor = None
    self._extractors = []
    self.input_data = None
    self.logger = None
    self.config_db_dir = None
    self.articles_tbl = None
    self.sqliteTimeout = SQLITE_TIMEOUT
    self.DBConnector = None
    self.db_dc_scraper = None
    self.db_dc_scraper_db_connect = None
    self.properties = {}
    self.algorithm_name = None
    self.pubdate = None
    self.timezone = None
    self.PRAGMA_synchronous = None
    self.PRAGMA_journal_mode = None
    self.PRAGMA_temp_store = None
    self.message_queue = []
    self.metrics_list = []
    self.SQLTmpTable = CONSTS.SQL_TMP_TABLE
    self.db_engine = CONSTS.SQLITE_ENGINE
    self.dbHost = None
    self.dbPort = None
    self.dbUser = None
    self.dbPWD = None
    self.dc_contents_db = None
    self.entry = None
    self.article = None
    self.outputFormat = None
  
  
  # #setup
  # setup application
  def setup(self):
    # call base class setup method
    foundation.CementApp.setup(self)
    self.args.add_argument('-c', '--config', action='store', metavar='config_file', help='config ini-file')


  # #run
  # run application
  def run(self):
    # call base class run method
    foundation.CementApp.run(self)

    # config section
    self.loadConfig()

    # load logger config file
    self.loadLogConfigFile()

    # load sqlite db backend
    # self.loadSqliteDBBackend()

    # mysql
    self.loadDBBackend()
  
    # mysql
    self.loadPropertiesFromDB()
    
    # options
    self.loadOptions()

    # init extractor's list
    #self.loadExtractors()
  


  # #adjust partial references
  # adjust partial references
  #
  def checkDOMElement(self, elem):
    ret = False
    #valid_http_url = HttpUrl()
    if re.search('<', elem):
        self.logger.debug("Media tag contain DOM element: %s", elem)
        ret = True
    return ret


  # #adjust partial references
  # adjust partial references
  #
  def adjustPartialReferences(self, response):
    if "links" in response.tags and type(response.tags["link"]) == types.DictType and \
      "media" in response.tags and type(response.tags["media"]) == types.DictType:
      try:
        url = None
        if self.input_data.template and "link" in self.input_data.template:
          self.logger.debug("url type: %s", str(type(response.tags["link"]["data"])))
          if type(response.tags["link"]["data"]) == str or type(response.tags["link"]["data"]) == unicode:
            url = response.tags["link"]["data"]
          else:
            url = response.tags["link"]["data"][0]
        else:
          url = self.input_data.url
        if self.input_data.template and "media" in self.input_data.template:
          self.logger.debug("resource has template with media tag. Try to adjust media.")
          #if type(response.tags["media"]) == str and response.tags["media"] == "": return
          self.logger.debug("response.tags['media']: " + str(response.tags["media"]))
          self.logger.info("media tag in response: <<%s>>" % str(response.tags["media"]["data"]))
          self.logger.info("link tag in response: <<%s>>" % str(url))
          res = []
  
          filter_patterns, filter_types = [], []
          if self.input_data.filters:
            #filter_types = [filter_item["Type"] for filter_item in self.input_data.filters]
            #filter_patterns = [re.compile(filter_item["Pattern"]) for filter_item in self.input_data.filters]
            filter_types = [filter_item.type for filter_item in self.input_data.filters]
            filter_patterns = [re.compile(filter_item.pattern) for filter_item in self.input_data.filters]
          self.logger.debug("filter: %s" % (str(self.input_data.filters)))
          for media in response.tags["media"]["data"]:
            self.logger.info("Media link: <<%s>>", media)
            # instead pure url
            if self.checkDOMElement(media):
              res.append(media)
              break
            media = urljoin(url, media)
            valid_http_url = HttpUrl()
            for filter_type, filter_pattern in zip(filter_types, filter_patterns):
              match = filter_pattern.match(media)
              if filter_type == SiteFilter.TYPE_EXCLUDE and match:
                break
              if filter_type == SiteFilter.TYPE_INCLUDE and match:
                res.append(media)
                break
            else:
              self.logger.debug("media: %s", media)
              self.logger.debug("url: %s", url)
              #if valid_http_url(media) and Scraper.parseHost(media) == Scraper.parseHost(url):
              if valid_http_url(media):
                res.append(media)
  
          # If media tag after adjusting is empty remove it from response
          if not len(res):
            self.logger.debug("media tag is empty. Remove media tag from response.")
            del response.tags["media"]
          else:
            self.logger.debug("media tag is adjusted. Copy media tag to response.")
            response.tags["media"]["data"] = res
          # End code block removing empty media tag
        else:
          self.logger.debug("resource hasn't template with media tag. adjustPartialReferences doesn't execute")
      except Exception as err:
        # self.exit_code = EXIT_FAILURE
        self.logger.debug(MSG_ERROR_ADJUST_PR + err.message)
        tbi = Utils.getTracebackInfo()
        self.logger.error(str(vars(err)) + "\n" + tbi)
        # raise
    else:
      self.logger.debug(">>> Response has not have link or media tag, Don't need adjust media")


  # adjustTitle
  #
  def adjustTitle(self, response):
    try:
      if self.input_data.template and "title" in self.input_data.template:
        self.logger.debug("resource has template with title tag. Try to adjust title.")
        self.logger.debug("response.tags['title']: " + str(response.tags["title"]))
        if not self.extractor:
          if len(self._extractors) > 2:
            self.extractor = self._extractors[2]
          else:
            raise Exception(" >>> Wrong! self._extractors list doesn't have 3'th element (index 2)")
        if type(response.tags["title"]) == str:
          self.logger.info("response has not have title tag")
          sel = Selector(text=self.input_data.raw_content)
          title = sel.xpath("//title/text()").extract()
          self.extractor.addTag(result=response, tag_name="title", tag_value=title)
        self.logger.info("TYPE response.tags['title']['data']" + str(type(response.tags["title"]["data"])))
      else:
        self.logger.debug("resource hasn't template with title tag. Don't need adjust title.")
    except Exception as err:
      self.logger.debug(MSG_ERROR_ADJUST_TITLE + err.message)


  # adjustLinkURL
  #
  def adjustLinkURL(self, response):
    flag = False
    try:
      if self.input_data.template and "link" in self.input_data.template:
        self.logger.debug("resource has template with link tag. Try to adjust link.")
        self.logger.debug("response.tags['link']: " + str(response.tags["link"]))
        self.logger.debug("self.extractor: %s", str(self.extractor))
        flag = True
        if self.extractor:
          self.logger.debug("Extractor exists")
          if type(response.tags["link"]) == str:
            self.logger.info("response has not have link tag")
            self.extractor.addTag(result=response, tag_name="link", tag_value=[self.input_data.url])
          # bypass
          else:
            response.tags["link"]["data"] = self.input_data.url
        else:
          if len(self._extractors) > 2:
            self._extractors[2].addTag(result=response, tag_name="link", tag_value=[self.input_data.url])
          else:
            self.logger.info(">>> Wrong! self._extractors list doesn't have 3'th element (index 2)")
        self.logger.info("TYPE response.tags['link']['data']" + str(type(response.tags["link"]["data"])))
      else:
        self.logger.debug("resource hasn't template with link tag. Don't need adjust link.")
    except Exception as err:
      self.logger.debug(MSG_ERROR_ADJUST_PR + err.message)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
    return flag


  # adjustLinkURL
  #
  def adjustJapaneseDate(self, raw_pubdate):
    # convert raw_pubdate in format like
    # 8月20日 20:01
    # 9月3日 11時41分
    # （8月20日 紙面から）
    # 2014年8月7日
    # 2014年8月20日 夕刊
    # 2014年8月21日 朝刊>
    # 2014/8/14付
    # （2014/08/16-14:46）
    # 2014/6/3 18:53 (2014/6/3 20:13更新)
    # 2014.08.20 Wed posted at 17:52 JST
    # 2014.08.01 Fri posted at 12:36 JST
    # 2014年 08月 20日 14:34 JST
    # 2014年08月20日 19時46分
    # 2014年8月29日16時30分
    # 8/20 21:00 更新
    # 11月２３日
    # to
    # 2014年8月7日 20:01
    # check if 日 exist in date
    self.logger.debug("pubdate has to be converted: <<%s>>", raw_pubdate)
    self.logger.debug("pubdate type: <<%s>>", str(type(raw_pubdate)))
    #self.logger.debug("pubdate charcode: <<%s>>", str(icu.CharsetDetector(raw_pubdate).detect().getName()))
    #raw_pubdate = raw_pubdate.decode("utf_8")
    #self.logger.debug("pubdate has to be converted: <<%s>>", raw_pubdate)
    #self.logger.debug("pubdate type: <<%s>>", str(type(raw_pubdate)))
    raw_pubdate = raw_pubdate.strip(" \r\t\n)（）\xe6\x9b\xb4\xe6\x96\xb0\xef"+
                                    "\xbc\x89\xe5\x88\x86\xe4\xbb\x98JST\xe7\xb4\x99\xe9\x9d\xa2\xe3\x81\x8b\xe3\x82"+
                                    "　\xe5\xa4\x95\xe5\x88\x8a\xe6\x9c\x9d\xe5\x88\x8a\xe3\x80\x80\xe5\x88\x86")
    # 2014.08.20 Wed posted at 17:52 JST
    # if "Wed posted at" in raw_pubdate:
    #  date_items = raw_pubdate.replace(" Wed posted at ", " ").replace(".", " ").split()
    #  raw_pubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + "\xe6\x97\xa5 " + date_items[3]
    if "\xe6\x97\xa5" in raw_pubdate:
      raw_pubdate = raw_pubdate.replace("\xe6\x97\xa5", "\xe6\x97\xa5 ")
      # check if date contain year
      # 2014年08月20日 19時46分
      if "\xe5\xb9\xb4" in raw_pubdate and "\xe6\x99\x82" in raw_pubdate:
        raw_pubdate = raw_pubdate.replace("\xe6\x99\x82", ":")
      elif "\xe5\xb9\xb4" in raw_pubdate and not ":" in raw_pubdate:
        raw_pubdate = raw_pubdate + " 00:00"
      # 2014年08月19日 11:53
      elif "\xe5\xb9\xb4" in raw_pubdate and ":" in raw_pubdate and raw_pubdate.count(" ") == 1:
        pass
      # 2014年 08月 20日 14:34 JST
      elif "\xe5\xb9\xb4" in raw_pubdate and ":" in raw_pubdate and raw_pubdate.count(" ") > 1:
        date_items = raw_pubdate.split()
        if isinstance(date_items, list) and len(date_items) == 2:
          raw_pubdate = date_items[0] + " " + date_items[1]
        else:
          raw_pubdate = date_items[0] + date_items[1] + date_items[2] + " " + date_items[3]
      elif "\xe5\xb9\xb4" in raw_pubdate:
        raw_pubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + raw_pubdate
    # 2014/8/14付
    if raw_pubdate.count("/") == 2:
      # date_items = raw_pubdate.split("/")
      # （2014/08/16-14:46）
      date_items = re.split("/|-", raw_pubdate)
      if isinstance(date_items, list) and len(date_items) == 4:
        raw_pubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + \
        "\xe6\x97\xa5 " + date_items[3]
      else:
        raw_pubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + \
        "\xe6\x97\xa5 00:00"
    # 8月20日 紙面から
    if not "\xe5\xb9\xb4" in raw_pubdate and not "/" in raw_pubdate:
      date_items = raw_pubdate.replace("\xe6\x9c\x88", " ").replace("\xe6\x97\xa5", " ").replace("\xe6\x99\x82", " ").split()
      if isinstance(date_items, list) and len(date_items) == 4:
        raw_pubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + date_items[0] + "\xe6\x9c\x88" + date_items[1] + \
        "\xe6\x97\xa5 " + date_items[2] + ":" + date_items[3]
      else:
        raw_pubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + date_items[0] + "\xe6\x9c\x88" + date_items[1] + \
        "\xe6\x97\xa5 00:00"
    # 8/20 21:00 更新
    if not "\xe5\xb9\xb4" in raw_pubdate and "/" in raw_pubdate:
      date_items = raw_pubdate.replace("/", " ").split()
      # 2014/6/3 18:53 (2014/6/3 20:13更新)
      if len(date_items) > 4:
        raw_pubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + \
        "\xe6\x97\xa5 " + date_items[3]
      else:
        raw_pubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + date_items[0] + "\xe6\x9c\x88" + date_items[1] + \
        "\xe6\x97\xa5 " + date_items[2]
    self.logger.debug("pubdate converted is: <<%s>>", raw_pubdate)
    return raw_pubdate


  # getTimeZone
  #
  def getTimeZone(self, raw_pubdate):
    # check if site has timezone rule
    if self.input_data.timezone is not None:
      self.timezone = self.input_data.timezone
      self.logger.debug("Resource has timezone properties. Timezone is: %s" % self.input_data.timezone)
    else:
      self.logger.debug("pubdate hasn't timezone")
    for tz in CONSTS.TIMEZONE_LIST:
      # check if date has timezone
      if tz in raw_pubdate:
        self.timezone = tz
        self.logger.debug("pubdate <<%s>> has timezone: <<%s>>" % (raw_pubdate, self.timezone))
        break
    self.logger.debug("Timezone is: %s" % self.timezone)


  # adjustTimeZone
  #
  def adjustTimeZone(self, pubdate):
    if self.timezone is not None:
      pubdate = re.sub("[+-]\d{4}", self.timezone, pubdate)
    return pubdate


  # convertPubDateToRFC2822
  #
  def convertPubDateToRFC2822(self, raw_pubdate):

    #replace all unicode digits to decimal (e.g. u'\uff13' to '3')
    import unicodedata
    for i in range(0, len(raw_pubdate)):
      if raw_pubdate[i].isdigit():
        raw_pubdate = re.sub(raw_pubdate[i], str(unicodedata.digit(raw_pubdate[i])), raw_pubdate)
    raw_pubdate = raw_pubdate.encode("utf_8")

    self.getTimeZone(raw_pubdate)
    import calendar
    try:
      pubdate_parts = raw_pubdate.split(",")
      if len(pubdate_parts)>1:
        raw_pubdate = pubdate_parts[0]
      raw_pubdate = raw_pubdate.replace("posted at", "")
      raw_pubdate = raw_pubdate.replace("Updated:", "")
      raw_pubdate = re.sub("\(1/\d{1}ページ\)", "", raw_pubdate)
      raw_pubdate = re.sub("平成26", "2014", raw_pubdate)
      raw_pubdate = re.sub("\(木\)", "", raw_pubdate)
      raw_pubdate = re.sub(" 年 ", "年", raw_pubdate)
      raw_pubdate = re.sub("@", "", raw_pubdate)
      parsed_time_candidate_str = float(calendar.timegm(parse(raw_pubdate).timetuple()))
      self.logger.debug("pubdate in seconds: %s", str(parsed_time_candidate_str))
    except ValueError:
      raw_pubdate = self.adjustJapaneseDate(raw_pubdate)
      parsed_time_candidate_str = float(calendar.timegm(time.strptime(raw_pubdate, 
                                  u"%Y\xe5\xb9\xb4%m\xe6\x9c\x88%d\xe6\x97\xa5 %H:%M")))
      self.logger.debug("parsed_time_candidate_str: %s", str(parsed_time_candidate_str))
    time_str = formatdate(parsed_time_candidate_str)
    self.logger.debug("pubdate converted is: %s", str(time_str))
    time_str = self.adjustTimeZone(time_str)
    self.logger.debug("pubdate adjusted timezone is: %s", str(time_str))
    return time_str


  # adjustPubDate
  #
  def adjustPubDate(self, response):
    # adjust date in format
    # Fri, 21 Nov 1997 09:55:06 -0600
    flag = False
    pubdate = None
    try:
      if (self.input_data.template and (CONSTS.TAG_PUB_DATE in self.input_data.template)) \
        or not self.input_data.template:
        self.logger.debug("resource has template with pubdate tag or dynamic scraping. Try to adjust pubdate.")
        extractor = self.getExtractorByName("ScrapyExtractor")
        if CONSTS.TAG_PUB_DATE in response.tags and response.tags[CONSTS.TAG_PUB_DATE] != "":
          self.logger.debug("response has 'pubdate' attr. pubdate is: %s", response.tags[CONSTS.TAG_PUB_DATE]["data"])
          response.tags[CONSTS.TAG_PUB_DATE]["data"] = self.convertPubDateToRFC2822(response.tags[CONSTS.TAG_PUB_DATE]\
                                                                                    ["data"])
          flag = True
        else:
          self.logger.debug("response hasn't pubdate. Try to extract one.")
          sel = Selector(text=self.input_data.raw_content)
          for path in extractor.template[CONSTS.TAG_PUB_DATE]:
            time_candidate_str = sel.xpath(path).extract()
            if type(time_candidate_str) == list and len(time_candidate_str) > 0 \
              and time_candidate_str[0].strip(' \t\n\r') != "" \
              and flag == False \
              and any(i.isdigit() for i in time_candidate_str):
              self.logger.debug("Time candidate extracted by xpath: <<%s>> is: <<%s>>" % (path, time_candidate_str))
              time_candidate_str_stripped = time_candidate_str[0].strip(' \t\n\r')
              self.logger.debug("Time candidate: %s", time_candidate_str_stripped)
              # time_candidate_str_encoded = time_candidate_str_stripped.encode("utf-8", "ignore")
              # self.logger.debug("Extracted date: %s", time_candidate_str_encoded)
              converted_pubdate = self.convertPubDateToRFC2822(time_candidate_str_stripped)
              flag = extractor.addTag(result=response, tag_name=CONSTS.TAG_PUB_DATE, tag_value=converted_pubdate, 
                                      xpath=path)
              self.logger.debug("tagsCount after: %s", response.tagsCount)
              self.logger.debug("response adjusted pubdate: %s", response.tags["pubdate"])
        if flag == False:
          self.logger.debug("Extracting pubdate has failed. Remove pubdate from response.")
          del response.tags["pubdate"]
          #converted_pubdate = self.convertPubDateToRFC2822(str(self.input_data.lastModified))
          #extractor.addTag(result=response, tag_name=CONSTS.TAG_PUB_DATE, tag_value=converted_pubdate)
      else:
        self.logger.debug("resource hasn't template with pubdate tag. Don't need adjust pubdate.")
      # self.logger.debug("pubdate: " + str(pubdate))
    except Exception as err:
      self.logger.debug(MSG_ERROR_ADJUST_PUBDATE + err.message)
      #tbi = Utils.getTracebackInfo()
      #self.logger.error(str(vars(err)) + "\n" + tbi)
      if CONSTS.TAG_PUB_DATE in response.tags:
        self.logger.debug("Extracting pubdate has failed but pubdate tag still exist in response. "+
                          "Remove pubdate from response.")
        del response.tags[CONSTS.TAG_PUB_DATE]
    # pubdate has been extracted from content properly
    if flag:
      pubdate = parser.parse(response.tags["pubdate"]["data"])
      # pubdate = datetime.datetime.strptime(response.tags["pubdate"]["data"], '%a, %d %b %Y %H:%M:%S %:z')
      self.logger.debug("pubdate: " + str(pubdate))
    return pubdate


  # #main content processing
  # main content processing
  #
  def process(self):
    # info input data
    self.logger.info("input_data.url: " + str(self.input_data.url))
    self.logger.info("input_data.urlId: " + str(self.input_data.urlId))
    self.logger.info("input_data.siteId: " + str(self.input_data.siteId))
    self.logger.info("input_data.filters: " + str(self.input_data.filters))
    # self.logger.info("input_data.template: " + str(self.input_data.template))
    self.logger.info("input_data.content len: %s", str(len(self.input_data.raw_content)))
    self.logger.info("input_data.lastModified data: %s", str(self.input_data.lastModified))
    self.logger.info("timezone: %s", str(self.input_data.timezone))

    # get iterator to ranked list of extractors
    self.itr = iter(sorted(self._extractors, key=lambda extractor: extractor.rank, reverse=True))
    self.logger.debug("Extractors: %s" % varDump(self.itr))
    # if for given resource exist template collection
    # then perform template extraction
    # else perform dynamic extraction
    if  self.input_data.siteId==CONSTS.GOOGLE_SEARCH_SITE_ID or \
      self.input_data.siteId==CONSTS.CABINET_SEARCH_SITE_ID or\
      self.input_data.siteId==CONSTS.OLD_GOOGLE_SEARCH_SITE_ID or \
      self.input_data.siteId==CONSTS.OLD_CABINET_SEARCH_SITE_ID:
      response = self.searchEngineScraping()
    elif self.input_data.template and self.algorithm_name == CONSTS.PROCESS_ALGORITHM_REGULAR:
      # Reconfigure processor's properties to involve only template scraper
      response = self.templateExtraction()
    else:
      response = self.dynamicExtraction()
    # put extracted article to rhe db
    # self.logger.debug("Response: " + response.get())
    # adjust partial references in media tag
    if self.input_data.siteId not in [CONSTS.OLD_GOOGLE_SEARCH_SITE_ID, CONSTS.OLD_CABINET_SEARCH_SITE_ID]:
      if self.algorithm_name != CONSTS.PROCESS_ALGORITHM_REGULAR:
        self.adjustTitle(response)
        self.adjustLinkURL(response)
        self.adjustPartialReferences(response)
        self.logger.debug("PDate: %s" % str(self.input_data.batch_item.urlObj.pDate))
        self.logger.debug("PDate type: %s" % str(type(self.input_data.batch_item.urlObj.pDate)))
        if (self.input_data.batch_item.urlObj.pDate is not None) and \
          (type(self.input_data.batch_item.urlObj.pDate) != type(SQLExpression())):
          data = {"extractor":"Taken from rss feed parser", "data":"", "name":"pubdate"}
          data["data"] = self.input_data.batch_item.urlObj.pDate
          response.tags[CONSTS.TAG_PUB_DATE] = data
        else:
          self.pubdate = self.adjustPubDate(response)
      self.preparseResponse(response)
      self.getProcessedContent({"default":response})
      self.tagsCount = response.tagsCount
      self.tagsMask = response.tagsMask
      #self.putArticleToDB({"default":response})
      self.logger.debug("self.tagsCount: %s", self.tagsCount)
      self.logger.debug("self.tagsMasks: %s", self.tagsMask)
    else:
      self.preparseResponse(response)

    self.logger.debug(">>> Resp: %s" % varDump(response))
    return response


  def preparseResponse(self, response):
    #self.logger.debug(">>> Resp: %s" % varDump(response))
    for key in response.tags:
      if "data" in response.tags[key]:
        if type(response.tags[key]["data"]) in types.StringTypes:
          localStr = response.tags[key]["data"]
          response.tags[key]["data"] = []
          response.tags[key]["data"].append(localStr)
    #self.logger.debug(">>> Resp: %s" % varDump(response))
    
    if self.outputFormat is not None:
      self.formatOutputData(response, self.outputFormat)
    else:
      self.logger.debug(">>> Warning, can't extracr output format")


  def formatOutpuElement(self, elem, localOutputFormat):
    ret = elem
    if localOutputFormat == "json":
      self.logger.info(">>> JSON HTML = " + elem)
      localStr = json.dumps(elem, ensure_ascii=False)
      if localStr[0] == '\"' or localStr[0] == '\'':
        localStr = localStr[1:]
      if localStr[-1] == '\"' or localStr[-1] == '\'':
        localStr = localStr[0:-1]
      ret = localStr
      self.logger.info(">>> JSON HTML = " + ret)
    elif localOutputFormat == "html":
      ret = xml.sax.saxutils.escape(elem, {"'": "&apos;", "\"" : "&quot;"})
    elif localOutputFormat == "sql":
      ret = mdb.escape_string(elem)
    return ret


  def formatOutputData(self, response, localOutputFormat):
    #result.tags[key]["data"]
    for key in response.tags:
      if "data" in response.tags[key]:
        if type(response.tags[key]["data"]) == types.ListType:
          for i, elem in enumerate(response.tags[key]["data"]):
            response.tags[key]["data"][i] = self.formatOutpuElement(elem, localOutputFormat)
        elif type(response.tags[key]["data"]) in types.StringTypes:
          response.tags[key]["data"] = self.formatOutpuElement(response.tags[key]["data"], localOutputFormat)


  def templateExtraction(self):
    self.extractor = ScrapyExtractor(self.config, self.input_data.template)
    resultsList = []
    # create empty result
    result = Result(None, self.input_data.urlId)
    sel = Selector(text=self.input_data.raw_content)
    if type(self.input_data.template)==type({}):
      template = self.input_data.template
    else:
      template = ast.literal_eval(self.input_data.template)
    #Andrey Add
    self.logger.info(">>> Dict == " + str(template))
    # Add End
    for key in template:
      self.logger.info("Template key: " + key)
      if "state" in template[key] and not bool(int(template[key]["state"])):
        self.logger.info(">>> Template disable: template name = " + str(key))
        continue
      for path in template[key]:
        if type(path) != type({}):
          self.logger.info(">>> WARNING path not DICT type ")
          continue
        pathDict = Utils.getPairsDicts(path)
        isExtract = True
        localResult = Result(None, self.input_data.urlId)
        # Added new template format conversion
        xpath = None
        xpathValue = None
          # Added new template type specification
        if path["type"] == "text":
          localXpath = sel.xpath(path["target"])
          xpathValue = self.innerText(localXpath)
          xpath = path["target"]
          #xpath = path["target"] + "/text()"
        # datetime tag's type
        elif path["type"] == "datetime":
          xpathValue, xpath = self.getXpathValueForDTime(path["target"], sel)
          self.logger.info(">>> final XPath = " + str(xpathValue))
        elif path["type"] == "image":
          self.logger.info(">>> img format = " + str(path["format"]))
          if path["format"] == "URL":
            xpath = path["target"] + "/@src"
          elif path["format"] == "DATA":
            pass
          elif path["format"] == "ALT":
            xpath = path["target"] + "/@alt"
          elif path["format"] == "TITLE":
            xpath = path["target"] + "/@title"
        elif path["type"] == "html":
          pass
        elif path["type"] == "link":
          if not Utils.isTailSubstr(path["target"], "/@href"):
            xpath = path["target"] + "/@href"

        if xpath is None:
          xpath = path["target"]

        if xpathValue is None:
          try:
            xpathValue = sel.xpath(xpath).extract()
          except Exception:
            self.logger.info(">>> Common xPath extractor exception")
            xpathValue = []

        self.logger.info(">>> xpathValue " + str(type(xpathValue)) + " " + str(xpathValue))
        self.logger.info(">>> xpath: %s" % str(xpath))
        if (type(xpathValue) == types.ListType and len(xpathValue) == 0) or\
        (type(xpathValue) in types.StringTypes and xpathValue == ''):
          self.logger.info(">>> set default xpathValue")
          xpathValue = []
          xpathValue.append(path["default"])
          isExtract = False
        self.logger.info("result before: %s" % varDump(localResult))
        self.extractor.addTag(result=localResult, tag_name=key, tag_value=xpathValue, xpath=xpath)
        self.logger.info("reslt after: %s" % varDump(localResult))

        self.logger.info(">>> tag type = " + str(type(localResult.tags)))
        self.logger.info(">>> tags data type = " + str(type(localResult.tags[key]["data"])))
        if key in localResult.tags and type(localResult.tags[key]["data"]) in types.StringTypes:
          self.logger.info(">>> Convert result = " + str(key))
          localString = localResult.tags[key]["data"]
          localResult.tags[key]["data"] = []
          localResult.tags[key]["data"].append(localString)

        self.formatTag(localResult, path, key, pathDict)
        
        resultsList.append({"obj": localResult, "join": path["join"], "isExtract": isExtract, "mandatory":
                            (bool(path["mandatory"]) if "mandatory" in path else False)})

      prepareResultsList = self.prepareResults(resultsList)
      self.compileResults(result, prepareResultsList, key)
      resultsList = []
        #self.formatOutputData(path[])
      #result = Result(self.config, self.input_data.urlId)
      #for 
      #result.tags.update(localResult.tags)
    return result


  def compileResults(self, result, resultsList, key):
    for elem in resultsList:
      if key in result.tags:
        if result.tags[key]["xpath"] == None:
          result.tags[key]["xpath"] = elem["obj"].tags[key]["xpath"]
        else:
          result.tags[key]["xpath"] += ' '
          result.tags[key]["xpath"] += elem["obj"].tags[key]["xpath"]
        if result.tags[key]["data"] == None or len(result.tags[key]["data"]) == 0:
          result.tags[key]["data"] = elem["obj"].tags[key]["data"]
        else:
          result.tags[key]["data"][0] += ' '
          result.tags[key]["data"][0] += elem["obj"].tags[key]["data"][0]
      else:
        result.tags.update(elem["obj"].tags)


  def prepareResults(self, resultsList):
    ret = []
    if len(resultsList) > 0:
      localElemWeight = 0
      firstElemWeight = 0
      firstElem = None
      tempList = []
      for elem in resultsList:
        localElemWeight = 0
        if elem["join"] == "concat":
          tempList.append(elem)
        else:
          if elem["mandatory"]:
            localElemWeight = localElemWeight | CONSTS.TAGS_RULES_MASK_MANDATORY_FIELD
          if elem["join"] == "best":
            localElemWeight = localElemWeight | CONSTS.TAGS_RULES_MASK_RULE_PRIORITY
          if elem["isExtract"]:
            localElemWeight = localElemWeight | CONSTS.TAGS_RULES_MASK_DEFAULT_VALUE

          self.logger.info(">>> Rule weight = " + str(localElemWeight))
          self.logger.info(">>> Rule join = " + elem["join"])
          if localElemWeight > firstElemWeight:
            firstElemWeight = localElemWeight
            firstElem = elem

      if firstElem is not None:
        tempList = [firstElem] + tempList
      '''
      isExtractResults = any([elem["isExtract"] for elem in resultsList])
      if isExtractResults:
        ret = [elem for elem in resultsList if elem["isExtract"]]
      else:
        ret.append(resultsList[0])
      '''
      isExtractResults = any([elem["isExtract"] for elem in tempList])
      if isExtractResults:
        ret = [elem for elem in tempList if elem["isExtract"]]
      else:
        ret.append(tempList[0])
    return ret


  def extractXpathFromSelectorList(self, sList, localXpath, lambdaCall):
    ret = []
    for elem in sList:
      if lambdaCall(elem):
        ret = elem.xpath(localXpath).extract()
        if len(ret) > 1 and any(True for ch in ret if (ch >= '0' and ch <= '9')):
          ret = [ret[0]]
          break
    return ret


  def innerText(self, selectorList):
    ret = ''
    for elem in selectorList.extract():
      localStr = Utils.innerHTMLText(str(elem))
      ret += localStr.strip()
      ret += ' '
    ret.strip()
    return ret


  def getXpathValueForDTime(self, initXpath, sel):
    xpathValue = []
    xpath = initXpath
  
    localXpath = sel.xpath(xpath)
    self.logger.info(">>> Datetime | Meta extraction")
    xpathValue = self.extractXpathFromSelectorList(localXpath, "@content", \
                                                   lambda elem: elem.extract().find("<meta") == 0)
    if len(xpathValue) == 0:
      self.logger.info(">>> Datetime | any tag @datetime argument extraction")
      xpathValue = self.extractXpathFromSelectorList(localXpath, "@datetime", \
                                                     lambda elem: elem.extract().find("<time") == 0)
      if len(xpathValue) == 0:
        self.logger.info(">>> Datetime | inner Text Extraction")
        localStr = self.innerText(localXpath)
        if localStr != '':
          xpathValue = [localStr]
    return xpathValue, xpath


  ## formatTag
  #
  def formatTag(self, result, path, key, pathDict):
    #Andrey Add
    self.logger.info("Tag type: %s, tag format: %s" % (str(path["type"]), str(path["format"])))
    #Add End
    if path["type"] == "text":
      localText = u''
      for elem in result.tags[key]["data"]:
        localText += elem.strip()
        localText += ' '
      localText = localText.strip()
      localMaxCh = None
      if "format" in pathDict and "maxCh" in pathDict["format"]:
        self.logger.info(">>> get LocalMaxCh from DICT[\"format\"][\"maxCh\"]")
        localMaxCh = pathDict["format"]["maxCh"]
      else:
        self.logger.info(">>> get LocalMaxCh from [\"format\"]")
        localMaxCh = path["format"]
      try:
        if localMaxCh is not None and int(localMaxCh) > 0 and len(localText) > int(localMaxCh):
          localText = localText[0: int(localMaxCh)]
      except ValueError:
        self.logger.info(">>> Wrong maxCh value")
      result.tags[key]["data"] = []
      result.tags[key]["data"].append(localText)
    if path["type"] == "title":
      pass
    elif path["type"] == "body":
      pass
    elif path["type"] == "html":
      #>>> html
      for i, elem in enumerate(result.tags[key]["data"]):
        result.tags[key]["data"][i] = re.sub("\<![ \r\n\t]*(--([^\-]|[\r\n]|-[^\-])*--[ \r\n\t]*)\>", "", elem)
      self.logger.info(">>> After RE = " + str(result.tags[key]["data"]))
      #>>> html END
    elif path["type"] == "datetime":
      #>>> datetime
      bestData = ''
      try:
        self.logger.info("Try to convert data")
        if result.tags[key]["data"][0] != path["default"]:
          bestData = self.getBestDatatimeData(result.tags[key]["data"])
          self.logger.info(">>> Time log Before = " + bestData)
          if path["format"] != "" and path["format"] != "FULL":
            result.tags[key]["data"][0] = datetime.datetime.strftime(parser.parse(bestData), path["format"])
          else:
            result.tags[key]["data"][0] = str(parser.parse(bestData))
        self.logger.info(">>> Time log after = " + result.tags[key]["data"][0])
      except Exception as err:
        self.logger.info("Can't convert data <<< " + str(result.tags) + " " + str(key) + " err = " + str(err))
        result.tags[key]["data"][0] = bestData
      if len(result.tags[key]["data"]) > 0:
        result.tags[key]["data"] = [result.tags[key]["data"][0]]
      #>>> datetime END
    elif path["type"] == "author":
      pass
    elif path["type"] == "image":
      pass

    localElem = u''
    for elem in result.tags[key]["data"]:
      localElem += elem
      localElem += ' '
    result.tags[key]["data"][0] = localElem
    result.tags[key]["data"][0] = result.tags[key]["data"][0].strip()


  def processingHTMLData(self, htmlBuf, bufFormat):
    ret = htmlBuf
    if bufFormat.find("NO_SCRIPT") >= 0:
      ret = Utils.stripHTMLComments(htmlBuf, soup=None)
    if bufFormat.find("NO_META") >= 0:
      pass
    if bufFormat.find("NO_COMMENTS") >= 0:
      pass
    if bufFormat.find("ENTITIES_ENCODED") >= 0:
      pass
    return ret


  def getBestDatatimeData(self, data):
    ret = None
    for elem in data:
      for ch in elem:
        if ch >= '0' and ch <= '9':
          ret = elem
          break
      if ret is not None:
        break
    if ret is None:
      ret = data[0]
    ret = ret.replace('\n', '')
    ret = ret.replace('\t', '')
    return ret


  def dynamicExtraction(self):
    # self.logger.info("self.input_data.urlId: " + self.input_data.urlId)
    # get resource as dictionary
    # resource_set = self.getResourceFromDB(resId)
    resource_set = {}
    resource_set["url"] = self.input_data.url
    resource_set["resId"] = self.input_data.urlId
    resource_set["siteId"] = self.input_data.siteId
    resource_set["raw_html"] = self.input_data.raw_content
    # self.logger.info("raw_content: " + str(self.input_data.raw_content))
    resource = Res(resource_set)

    # create empty result
    result = Result(self.config, self.input_data.urlId)
    result.metrics.append(WordsMetric())

    # get list of empty tags from result
    empty_tags = result.getEmptyTags()
    self.logger.info("get list of empty tags from result: " + str(empty_tags))

    # get best matching extractor
    self.extractor = self.getNextBestExtractor()
    self.logger.info("get best matching extractor: " + str(self.extractor))

    # search engine parsing
    if not resource.site_id:
      return self.extractor.extractTags(resource, result)

    # main loooop
    while len(empty_tags) and self.extractor:
      result = self.extractor.extractTags(resource, result)
      #result = Utils.timeLimit(self.extractor.extractTags(resource, result),10)
      # metric support
      self.extractor.calculateMetrics(result)
      # print result.tags
      empty_tags = result.getEmptyTags()
      self.logger.info("get list of empty tags from result: " + str(empty_tags))
      filled_tags = result.getFilledTags()
      self.logger.info("get list of filled_tags from result: " + str(filled_tags))
      self.extractor = self.getNextBestExtractor()
      self.logger.info("get best matching extractor: " + str(self.extractor))
    result.stripResult()
    return result


  def searchEngineScraping(self):
    resource_set = {}
    resource_set["url"] = self.input_data.url
    resource_set["resId"] = self.input_data.urlId
    resource_set["siteId"] = self.input_data.siteId
    resource_set["raw_html"] = self.input_data.raw_content
    resource = Res(resource_set)
    result = Result(self.config, self.input_data.urlId)

    self.extractor = ScrapyExtractor(self.config, self.input_data.template)
    return self.extractor.extractTags(resource, result)


  def getNextBestExtractor(self):
    # return extractor with highest rank
    try:
      extractor = next(self.itr)
    except StopIteration:
      extractor = None
    return extractor


  def getProcessedContent(self, result):
    # Add md5 hash of content
    result["default"].get()
    #self.logger.debug("result: %s" % varDump(result))
    #buffer = {}
    #for key in result.keys():
      #result[key].get()
      #buffer[key] = result[key].data
    #self.processedContent = encode(json.dumps(buffer))
    self.processedContent = result

  """
  ##
  #
  def getDBName(self):
    # get appropriate db name, depending on siteId
    if len(self.input_data.siteId):
      db_name = self.config_db_dir + "/" + self.input_data.siteId + ".db"
    else:
      db_name = self.config_db_dir + "/0.db"
    self.logger.info("db_name: " + db_name)
    return db_name
  """

  """
  ##
  #
  def getQueryPrefix(self):
    query_prefix = None
    if self.db_engine == CONSTS.SQLITE_ENGINE:
      query_prefix = "INSERT OR REPLACE INTO `" + self.articles_tbl
    elif self.db_engine == CONSTS.MYSQL_ENGINE:
      query_prefix = "REPLACE INTO `contents_" + str(self.input_data.siteId if len(self.input_data.siteId) else 0)
    else:
      self.logger.info("DB Backend %s not supported" % str(self.db_engine))
    self.logger.info("db_name: " + query_prefix)
    return query_prefix
  """
  
  
  """
  ## getDataBuffer
  ## prepare data buffer
  #
  def getDataBuffer(self, data):
    buffer = {}
    for key in data.keys():
      data[key].get()
      buffer[key] = data[key].data
    ret = encode(json.dumps(buffer))
    #self.logger.info("Result buffer: %s" % ret)
    return ret
  """

  """
  def putArticleToDB(self, result):
    # Check if Content extracted
    if type(result)==type({}):
      tags_count = int(result["default"].tagsCount)
    else:
      tags_count = int(result.tagsCount)
    self.logger.info("Tags count: %s" % str(tags_count))
    if int(tags_count)>0:
      self.logger.info("Tags count OK")
      
      # Firstly, prepare data buffer
      data = self.getDataBuffer(result)
      putDict = {}
      putDict["id"] = self.input_data.urlId
      putDict["data"] = data
      
      # URLPut
      # Create new URLPut object
      urlPut = EventObjects.URLPut(self.input_data.siteId, self.input_data.urlId, EventObjects.Content.CONTENT_PROCESSOR_CONTENT, putDict)
      cfgParser = ConfigParser.ConfigParser()
      cfgParser.read(self.db_task_ini)
      dbTask = DBTasksManager(cfgParser)
      drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_PUT, [urlPut])
      responseDRCESyncTasksCover = dbTask.process(drceSyncTasksCoverObj)
      if responseDRCESyncTasksCover.eventType == DC_CONSTS.EVENT_TYPES.URL_PUT_RESPONSE:
        for obj in responseDRCESyncTasksCover.eventObject:
          self.logger.info("URL_PUT_RESPONSE: %s" % varDump(obj))
      else:
        logger.error("URL_PUT_RESPONSE >>> Wrong response type")
      # Get DB table name
      query_prefix = self.getQueryPrefix()
      
      # Prepare query
      # Firstly, prepare data buffer
      data = self.getDataBuffer(result)
      # Then, insert data buffer to the query
      #query = query_prefix + "`(`id`,`data`, CDate) VALUES('" \
      #  + result.resId + "', '" + encode(result.get()) + "', strftime('%Y-%m-%d %H:%M:%f', 'now', 'localtime'))"
      query = query_prefix + "`(`id`,`data`, CDate) VALUES('" + result["default"].resId + "', '" + data + "', NOW())"

      # Initialize options for DB Backend
      options = {}
      # Genaral options
      options["query"] = query
      self.logger.info("DB Backend %s" % str(self.db_engine))
      
      if self.db_engine == CONSTS.SQLITE_ENGINE:
        # Get DB Backend name
        db_name = self.getDBName()
        # SQLite options
        options["db_name"] = db_name
        options["sqliteTimeout"] = self.sqliteTimeout
        options["PRAGMA_synchronous"] = self.PRAGMA_synchronous
        options["PRAGMA_journal_mode"] = self.PRAGMA_journal_mode
        options["PRAGMA_temp_store"] = self.PRAGMA_temp_store
        # Call db query
        Utils.putContentToSQLiteDB(result, options)
      elif self.db_engine == CONSTS.MYSQL_ENGINE:
        # MYSQL options
        options["dbHost"] = self.dbHost
        options["dbPort"] = self.dbPort
        options["dbUser"] = self.dbUser
        options["dbPWD"] = self.dbPWD
        options["MYSQLDBName"] = self.dc_contents_db
        Utils.putContentToMYSQLDB(result, options)
      else:
        self.logger.info("DB Backend %s not supported" % str(self.db_engine))
    self.logger.info("Tags count: %s" % str(result.tagsCount))
    if int(result.tagsCount)>0:
      self.logger.info("Tags count OK")
      # get appropriate db name, depending on siteId
      if len(self.input_data.siteId):
        db_name = self.config_db_dir + "/" + self.input_data.siteId + ".db"
      else:
        db_name = self.config_db_dir + "/0.db"
      self.logger.info("db_name: " + db_name)
      connector = None
      try:
        # put parsed resource to the db
        connector = sqlite3.connect(db_name, timeout=self.sqliteTimeout)  # @UndefinedVariable
        with connector:
          cur = connector.cursor()
          if self.PRAGMA_synchronous is not None:
            cur.execute("PRAGMA synchronous = %s" %(self.PRAGMA_synchronous))
          if self.PRAGMA_journal_mode is not None:
            cur.execute("PRAGMA journal_mode = %s" %(self.PRAGMA_journal_mode))
          if self.PRAGMA_temp_store is not None:
            cur.execute("PRAGMA temp_store = %s" %(self.PRAGMA_temp_store))
          query = "INSERT OR REPLACE INTO `" \
                  + self.articles_tbl + \
                  "`(`id`,`data`, CDate) VALUES('" \
                  + result.resId + \
                  "', '" \
                  + encode(result.get()) + \
                  "', strftime('%Y-%m-%d %H:%M:%f', 'now', 'localtime'))"
          cur.execute(query)
      except Exception as err:
        # Connection objects can be used as context managers that automatically commit or rollback transactions.
        # In the event of an exception, the transaction is rolled back; otherwise, the transaction is committed:
        # connector.rollback()
        self.logger.error("%s " % (err.message))
        raise
  """

  # #load extractors
  #
  def loadExtractors(self):
    try:
      # modules
      modules = self.properties[CONSTS.MODULES_KEY][self.algorithm_name]

      self.logger.info("Algorithm name: <%s>" % (self.algorithm_name))
      self.logger.info("Modules: %s" % modules)

      for module in modules:
        exrtactor = self.createModule(module)
        # Check if module was created successfully and then insert it to extractors
        if exrtactor is not None:
          self._extractors.append(exrtactor)
    
      # Info show extractors loaded
      self.logger.info("===================")
      self.logger.info("Loaded extractors:")
      for exrtactor in self._extractors:
        self.logger.info(exrtactor.name)
      self.logger.info("===================")

    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_EXTRACTORS + err.message)
      raise


  # #process batch
  # the main processing of the batch object
  def processBatch(self):
    # logger
    for entry in self.message_queue:
      self.logger.info(entry)
    try:
      # read pickled batch object from stdin and unpickle it
      input_pickled_object = sys.stdin.read()
      scraper_in_data = pickle.loads(input_pickled_object)
      self.input_data = scraper_in_data
      self.logger.debug("Batch item properties: %s" % varDump(self.input_data.batch_item.properties))
      
      if ("templates" in self.input_data.batch_item.properties["template"] and \
          len(self.input_data.batch_item.properties["template"]["templates"]) > 0 and \
          "output_format" in self.input_data.batch_item.properties["template"]["templates"][0] and \
          "name" in self.input_data.batch_item.properties["template"]["templates"][0]["output_format"]):
        self.outputFormat = self.input_data.batch_item.properties["template"]["templates"][0]["output_format"]["name"]
      # check properties in input data
      try:
        if (self.input_data is not None) and \
          self.input_data.processor_properties is not None:
          processor_properties = self.input_data.processor_properties
          self.logger.debug("Processor's properties was taken from input data: %s" % processor_properties)
          self.logger.debug("Processor's properties type: %s" % str(type(processor_properties)))
          if type(processor_properties)!=type({}):
            processor_properties = json.loads(self.input_data.processor_properties)
          self.logger.debug("Processor's properties was taken from input data: %s" % processor_properties)
          self.properties = processor_properties
      except Exception, err:
        self.logger.error("Error load properties from input data: %s" % str(err.message))

      self.algorithm_name = self.properties[CONSTS.ALGORITHM_KEY][CONSTS.ALGORITHM_NAME_KEY]
      self.logger.debug("Algorithm : %s" % self.algorithm_name)
      Utils.storePickleOnDisk(input_pickled_object, ENV_SCRAPER_STORE_PATH, "scraper.in." + str(self.input_data.urlId))
      # self.logger.info("input scraper object: " + str(vars(scraper_in_data)))
      # TODO main processing over every url from list of urls in the batch object
      tmp = sys.stdout
      sys.stdout = open("/dev/null", "wb")

      # initialization of scraper
      # load scraper's modules
      self.loadExtractors()

      self.logger.debug("Process: %s" % str(self.algorithm_name))
      if self.algorithm_name == CONSTS.PROCESS_ALGORITHM_REGULAR:
        response = self.process()

        """
      elif self.algorithm_name == CONSTS.PROCESS_ALGORITHM_TRAINING:
        response = self.processTraining()
      elif self.algorithm_name == CONSTS.PROCESS_ALGORITHM_CONCURRENCY:
        response = self.processConcurrency()
      elif self.algorithm_name == CONSTS.PROCESS_ALGORITHM_PREDICTION:
        response = self.process()
        """

      # for debugging single module lide goose or ml extractor
      elif (self.algorithm_name==CONSTS.PROCESS_ALGORITHM_NEWSPAPER) or \
        (self.algorithm_name==CONSTS.PROCESS_ALGORITHM_GOOSE) or \
        (self.algorithm_name==CONSTS.PROCESS_ALGORITHM_SCRAPY) or \
        (self.algorithm_name==CONSTS.PROCESS_ALGORITHM_ML):
        response = self.process()

      # SUPPORT METRICS ALGORITHTM
      elif self.algorithm_name==CONSTS.PROCESS_ALGORITHM_METRIC:
        response = self.processMetrics()

      # SUPPORT FEED_PARSER ALGORITHTM
      elif self.algorithm_name==CONSTS.PROCESS_ALGORITHM_FEED_PARSER:
        response = self.feedParserProcess()
    
      # ALGORITHM NOT SUPPORTED
      else:
        self.logger.debug("Process algorithm unknown or not defined: %s" % str(self.algorithm_name))
        response = self.process()

      # send response to the stdout
      sys.stdout = tmp
      if self.input_data.siteId in [CONSTS.OLD_GOOGLE_SEARCH_SITE_ID, CONSTS.OLD_CABINET_SEARCH_SITE_ID]:
        print response.get()
      else:
        scraperResponse = ScraperResponse(self.tagsCount, self.tagsMask, self.pubdate, self.processedContent)
        output_pickled_object = pickle.dumps(scraperResponse)
        Utils.storePickleOnDisk(output_pickled_object, ENV_SCRAPER_STORE_PATH, 
                                "scraper.out." + str(self.input_data.urlId))
        print output_pickled_object
      sys.stdout.flush()
    except Exception as err:
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.logger.error("Scraper process batch error: " + err.message)
      self.exit_code = EXIT_FAILURE


  # #load config from file
  # load from cli argument or default config file
  def loadConfig(self):
    try:
      self.config = ConfigParser.ConfigParser()
      self.config.optionxform = str
      if self.pargs.config:
        self.config.read(self.pargs.config)
      else:
        self.config.read(APP_NAME)
    except:
      print MSG_ERROR_LOAD_CONFIG
      raise


  # #load logging
  # load logging configuration (log file, log level, filters)
  #
  def loadLogConfigFile(self):
    try:
      log_conf_file = self.config.get("Application", "log")
      logging.config.fileConfig(log_conf_file)
      # Logger initialization
      self.logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

      # Replace logger name for processes instances
      lfn = Utils.LoggerFileName()
      fname = lfn.findReplace()
      if fname != "":
        pin = lfn.getFreeProcInstanceNumber(os.path.basename(fname))
        if pin != "" and pin != "0":
          pin = "." + pin + ".log"
          fname = lfn.findReplace(fname + pin)
          self.logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

    except:
      print MSG_ERROR_LOAD_LOG_CONFIG_FILE
      raise


  # #load mandatory options
  # load mandatory options
  #
  def loadOptions(self):
    try:
      class_name = self.__class__.__name__
      self.db_task_ini = self.config.get(class_name, "db-task_ini")

      self.config_db_dir = self.config.get(class_name, "config_db_dir")
      self.articles_tbl = self.config.get("sqlite", "articles_tbl")
      self.sqliteTimeout = self.config.getint("sqlite", "timeout")
      """
      # support sqlite optimizer options
      if self.config.has_option("sqlite", "PRAGMA_synchronous"):
        self.PRAGMA_synchronous = self.config.get("sqlite", "PRAGMA_synchronous")
        self.message_queue.append("PRAGMA_synchronous from config file: %s" % self.PRAGMA_synchronous)
      if self.config.has_option("sqlite", "PRAGMA_journal_mode"):
        self.PRAGMA_journal_mode = self.config.get("sqlite", "PRAGMA_journal_mode")
        self.message_queue.append("PRAGMA_journal_mode from config file: %s" % self.PRAGMA_journal_mode)
      if self.config.has_option("sqlite", "PRAGMA_temp_store"):
        self.PRAGMA_temp_store = self.config.get("sqlite", "PRAGMA_temp_store")
        self.message_queue.append("PRAGMA_temp_store from config file: %s" % self.PRAGMA_temp_store)
  
      # support db backend
      if self.config.has_option(class_name, "db_engine"):
        self.db_engine = self.config.get(class_name, "db_engine")
        self.message_queue.append("DB backend from config file: %s" % self.db_engine)
      """
    except:
      print MSG_ERROR_LOAD_OPTIONS
      raise

  
  """
  ## processTraining
  #
  def processTraining(self):
    #TODO: add processTraining method implementation
    #stub
    self.modules[CONSTS.TRAINING_QUEUE] = [
      nex(self.config),
      gex(self.config)
    ]
    self.modules[CONSTS.TRAINED_QUEUE] = [
      sex(self.config)
    ]

    self._extractors = self.modules[CONSTS.TRAINING_QUEUE]
    self.response = Utils.timeLimit(self.dynamicExtraction(),3)
    self.getTags()
    return Result(self.config, self.input_data.urlId)
  """
  """
  ## processConcurrency
  #
  def processConcurrency(self):
    #preamble
    resource_set = {}
    resource_set["url"] = self.input_data.url
    resource_set["resId"] = self.input_data.urlId
    resource_set["siteId"] = self.input_data.siteId
    resource_set["raw_html"] = self.input_data.raw_content
    
    result = {}
    resource = {}
    for extractor in self._extractors:
      resource[extractor.__class__.__name__] = Res(resource_set)
      result[extractor.__class__.__name__] = Result(self.config, self.input_data.urlId)
      extractor.extractTags(resource[extractor.__class__.__name__], result[extractor.__class__.__name__])
    
    response = Result(self.config, self.input_data.urlId)
    response.mergeResults(result)

    self.getProcessedContent({"default": response})
    self.tagsCount = response.tagsCount
    self.tagsMask = response.tagsMask
    #self.putArticleToDB({"default":response})
    return response
  """

  ## processMetrics
  #
  def processMetrics(self):
    # load metrics
    self.loadMetrics()

    #preamble
    resource_set = {}
    resource_set["url"] = self.input_data.url
    resource_set["resId"] = self.input_data.urlId
    resource_set["siteId"] = self.input_data.siteId
    resource_set["raw_html"] = self.input_data.raw_content
    
    # create empty result
    result = Result(self.config, self.input_data.urlId)

    result = {}
    resource = {}

    for extractor in self._extractors:
      extractor_name = extractor.__class__.__name__
      resource[extractor_name] = Res(resource_set)
      result[extractor_name] = Result(self.config, self.input_data.urlId)
      
      # add metrics to the each extractor's response
      for metric in self.metrics_list:
        self.logger.info("Add %s metric with threshold %s to %s."
                         % (metric["name"], metric["threshold"], extractor_name))
        metric_instance = self.createMetricModule(metric["name"])
        metric_instance.addThresholdValue(metric["threshold"])
        if "functor" in metric:
          metric_instance.initFunctor(metric["functor"])
        result[extractor_name].metrics.append(metric_instance)
      extractor.extractTags(resource[extractor_name], result[extractor_name])
      extractor.calculateMetrics(result[extractor_name])

    # Put metrics to temporary SQL table
    # Later we can retrieve it by QL
    self.putMetricsToSQL(result)
    
    # Prepare some default response
    response = Result(self.config, self.input_data.urlId)
    response.mergeResults(result)
    result["default"] = response
    
    # Apply adjusted content
    self.logger.info("Apply content by metrics")
    # get exrtactor name with the opitmal content
    extractor_name = self.getContentFromSQL()
    self.logger.info("get exrtactor name with the opitmal content: %s" % extractor_name)
    self.logger.info("result: %s" % result)
    if "content_encoded" in result[extractor_name].tags and \
      type(result[extractor_name].tags["content_encoded"])==type({}):
      self.logger.info("Result object for %s has content_encoded item" % extractor_name)
      content = result[extractor_name].tags["content_encoded"]["data"]
      self.logger.info("Content of result object for %s : " % content)
      old_extractor_name = result[extractor_name].tags["content_encoded"]["extractor"]
      self.logger.info("old_extractor_name: %s" % old_extractor_name)
      old_content = response.tags["content_encoded"]["data"]
      self.logger.info("old_content: %s" % old_content)
      response.tags["content_encoded"]["data"] = content
      response.tags["content_encoded"]["extractor"] = old_extractor_name
    
    # Afterword
    result["default"] = response
    self.getProcessedContent(result)
    self.tagsCount = response.tagsCount
    self.tagsMask = response.tagsMask
    # Put dict of results
    #self.putArticleToDB(result)
    #self.putArticleToDB(response)
    return response


  ## putMetricsToSQL
  #
  def putMetricsToSQL(self, result):
    # Assign uniq name to the SQL temporary table (URLMd5)
    self.SQLTmpTable = self.input_data.siteId + "_" + self.input_data.urlId
    # Drop table if exist
    query = "DROP TABLE IF EXISTS `%s`" % self.SQLTmpTable
    self.logger.info("query: %s" % query)
    res = self.executeQuery(query)
    
    # Create temporary table
    query = "CREATE TABLE IF NOT EXISTS `%s`(`ModuleName` VARCHAR(32)" % self.SQLTmpTable
    for metric in self.metrics_list:
      query = query + ", `%s` DECIMAL(64,4)" % (metric["name"])
    query = query + ")"
    self.logger.info("query: %s" % query)
    res = self.executeQuery(query)

    # Put data to db
    for key in result.keys():
      query = "INSERT INTO `%s`VALUES('%s'" % (self.SQLTmpTable, key)
      for metric in result[key].metrics:
        query = query + ", %s" % (metric.metricValue)
      query = query + ")"
      self.logger.info("query: %s" % query)
      res = self.executeQuery(query)


  ## getContentFromSQL
  #
  def getContentFromSQL(self):
    metrics = ""
    order = " ORDER BY "
    limit = " LIMIT 1"
    for metric in self.metrics_list:
      metrics = metrics + ", abs(`%s`-%s) as %s " % (metric["name"], metric["threshold"], metric["name"])
      order = order + " %s ," % (metric["name"])
    clause = " WHERE 1 "
    query = "SELECT `ModuleName` %s FROM `%s`  %s %s %s" % (metrics, self.SQLTmpTable, clause, order[:-1], limit)
    self.logger.info("query: %s" % query)
    res = self.executeQuery(query)
    self.logger.info("MySQL Result: %s" % str(res))
    
    # Extractor's module name
    module_name = res[0]["ModuleName"]
    
    # Update hits counter in main scraper table for that module
    #ALTER TABLE `scraper_main` ADD `Hits` INT(32) NOT NULL DEFAULT '0' COMMENT 'Count of hints by the article''s corpus' AFTER `Name`;
    query = "UPDATE `%s` SET `Hits`=`Hits`+1, `UDate`=NOW() WHERE `Name`='%s'" % (self.db_dc_scraper_table, module_name)
    self.logger.info("query: %s" % query)
    res = self.executeQuery(query)
    
    # Drop table after
    query = "DROP TABLE IF EXISTS `%s`" % self.SQLTmpTable
    self.logger.info("query: %s" % query)
    self.executeQuery(query)

    return module_name


  ## loadMetrics
  #
  def loadMetrics(self):
    if "metrics" in self.properties and len(self.properties["metrics"]):
      self.metrics_list = self.properties["metrics"]
      self.logger.info("self.metrics_list: %s" % str(self.metrics_list))


  # #
  #
  #
  def getTags(self):
    #TODO: add processTraining method implementation
    #stub
    return None


  # #helper function for correct query execution
  #
  # @param query query for execution
  def executeQuery(self, query):
    self.logger.info("Connector: " + str(self.DBConnector))
    try:
      with closing(self.DBConnector.cursor(MySQLdb.cursors.DictCursor)) as cursor:
        cursor.execute(query)
        self.DBConnector.commit()
        return cursor.fetchall()
    except mdb.Error as err:  # @todo logging in db_task
      self.DBConnector.rollback()
      self.logger.error("%s %s" % (err.args[0], err.args[1]))
      raise err


  ## loadDBBackend
  #
  #
  def loadDBBackend(self):
    try:
      className = self.__class__.__name__
      self.dbHost = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_HOST)
      self.dbPort = int(self.config.get(CONSTS.DB_SECTION, CONSTS.DB_PORT))
      self.dbUser = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_USER)
      self.dbPWD = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_PWD)
      self.db_dc_scraper = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_SCRAPERS)
      self.db_dc_scraper_table = self.config.get(className, CONSTS.DB_SCRAPERS)
      self.dc_contents_db = self.config.get(className, CONSTS.DC_CONTENTS_DB_NAME)
      self.db_dc_scraper_db_connect = mdb.connect(self.dbHost, self.dbUser, self.dbPWD, self.db_dc_scraper, self.dbPort)
    except Exception, err:
      self.logger.debug(CONSTS.MSG_ERROR_LOAD_DB_BACKEND + str(err.message))


  ## loadPropertiesFromDB
  #
  def loadPropertiesFromDB(self):
    self.logger.debug("Start load Scraper's properties from DB")
    self.DBConnector = self.db_dc_scraper_db_connect
    self.logger.debug("self.DBConnector: %s" % str(self.DBConnector))
    try:
      query = "SELECT `Value` FROM `%s` WHERE `Name`='%s'" % (self.db_dc_scraper_table, self.__class__.__name__)
      self.logger.debug("query: %s" % query)
      res = self.executeQuery(query)
      if len(res)>0:
        self.properties = json.loads(res[0]["Value"])[CONSTS.PROPERTIES_KEY]
      else:
        self.logger.debug("properties wasn't found in %s table" % self.db_dc_scraper)
    except Exception, err:
      self.logger.debug("Error load Scraper's properties from DB: %s" % str(err.message))
    self.logger.debug("Properties: %s" % varDump(self.properties))
    self.logger.debug("End load Scraper's properties from DB")


  # #createApp
  # create application's pool
  #
  # @param app_name application name which instance will be created
  # @return instance of created application
  def createModule(self, module_name):
    app = None
    try:
      app = (module_name, eval(module_name)(self.config))[1]
      self.logger.debug("%s has been created!" % module_name)
    except Exception as err:
      self.logger.error("Can't create module %s. Error is: %s" % (module_name, err.message))
      #tbi = Utils.getTracebackInfo()
      #self.logger.error(str(vars(err)) + "\n" + tbi)
      #raise
    return app


  # #createApp
  # create application's pool
  #
  # @param app_name application name which instance will be created
  # @return instance of created application
  def createMetricModule(self, module_name):
    app = None
    try:
      app = (module_name, eval(module_name)())
      self.logger.debug("%s has been created!" % module_name)
    except Exception as err:
      self.logger.error("Exception has been thrown during %s creation. %s" % (module_name, err.message))
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      raise
    return app[1]


  # #createApp
  # create application's pool
  #
  # @param app_name application name which instance will be created
  # @return instance of created application
  def getExtractorByName(self, extractor_name):
    for extractor in self._extractors:
      if extractor.__class__.__name__ == extractor_name:
        return extractor


  # #
  #
  #
  def getExitCode(self):
    return self.exit_code

  # #parseHost parse the root host name from url
  # for example: the result of http://s1.y1.example.com/path/to is example.com
  # @param url the full url
  # @return host of the url, eg: example.com
  @staticmethod
  def parseHost(url):
    host = None
    if urlparse(url).hostname:
      host = '.'.join(urlparse(url).hostname.split('.')[-2:])
    return host

  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # FeedParser section
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  # #main content processing
  # main content processing
  #
  def feedParserProcess(self):
    self.logger.debug("URL: %s" % str(self.input_data.url))
    self.logger.debug("URLMd5: %s" % str(self.input_data.urlId))
    self.logger.debug("SiteId: %s" % str(self.input_data.siteId))
    if self.parseFeed():
      self.tagsCount = self.article.tagsCount
      self.tagsMask = self.article.tagsMask
      self.processedContent = self.article.get()
      # correct pubdate
      if CONSTS.PUBLISHED in self.article.tags:
        self.pubdate = parse(self.article.tags[CONSTS.PUBLISHED]["data"]).strftime('%Y-%m-%d %H:%M:%S')
      else:
        self.logger.debug("Resource %s hasn't publish date" % str(self.article.tags[CONSTS.TAG_LINK]["data"]))
    else:
      self.logger.debug("Resource hasn't raw content. Exit.")


  # #main content processing
  # main content processing
  #
  def createArticle(self):
    resid = self.entry["urlMd5"]
    self.article = Result(self.config, resid)
    
    for tag in self.entry["entry"]:
      data = {"extractor":"feedParser extractor", "data":"", "name":""}
      data["data"] = self.entry["entry"][tag]
      data["name"] = tag
      self.article.tags[tag] = data
  
    date_tags = ["published", "updated", "updated_parsed"]
    if len(set(self.entry["entry"].keys()).intersection(date_tags))==0:
      self.logger.debug("PUBDATE_ERROR: list of tags from rss feed: %s" % str(self.entry["entry"].keys()))

    if "pubdate" in self.entry and \
      self.article.tags["pubdate"]=="":
      data = {"extractor":"feedParser extractor", "data":"", "name":""}
      data["data"] = self.entry["pubdate"]
      data["name"] = "pubdate"
      self.article.tags["pubdate"] = data
  
    #parent rss feed
    data = {"extractor":"feedParser extractor", "data":"", "name":""}
    data["data"] = self.entry["parent_rss_feed"]
    data["name"] = "parent_rss_feed"
    data["xpath"] = ""
    data["extractor"] = self.__class__.__name__
    self.article.tags["parent_rss_feed"] = data
    
    #parent rss feed urlMd5
    data = {"extractor":"feedParser extractor", "data":"", "name":""}
    data["data"] = self.entry["parent_rss_feed_urlMd5"]
    data["name"] = "parent_rss_feed_urlMd5"
    data["xpath"] = ""
    data["extractor"] = self.__class__.__name__
    self.article.tags["parent_rss_feed_urlMd5"] = data

    #tags count
    self.article.tagsCount = len(self.article.tags.keys())


  # #main content processing
  # main content processing
  #
  def parseFeed(self):
    ret = True
    try:
      self.entry = json.loads(self.input_data.raw_content)
      self.createArticle()
      self.putArticleToDB({"default":self.article})
    except ValueError, err:
      self.logger.debug("Bad raw content: %s" % varDump(self.input_data.raw_content))
      ret = False
    return ret
