"""@package docstring
 @file goose_extractor.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


# import json
import logging
from goose import Goose

from dc_processor.base_extractor import BaseExtractor as bex
import signal
from base_extractor import signal_handler
import dc_processor.Constants as CONSTS
import app.Consts as APP_CONSTS
from app.Utils import varDump


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


class GooseExtractor(bex):

  goose = None


  def __init__(self, config):
    try:
      bex.__init__(self, config)
      logger.debug("Properties: %s" % varDump(self.properties))
    
      if (CONSTS.RANK_KEY in self.properties):
        self.rank = self.properties[CONSTS.RANK_KEY]
        logger.debug("rank was setted: %s" % self.rank)
      self.name = "Goose extractor"
      self.goose = Goose()
      self.data["extractor"] = "Goose extractor"
    except Exception as err:
      logger.info("Goose extractor constructor error: possible /tmp not permitted to write " + err.message)
      raise


  def extractTags(self, resource, reslt):
    # support time execution limit
    signal.signal(signal.SIGALRM, signal_handler)
    signal.alarm(CONSTS.TIME_EXECUTION_LIMIT)
    try:
      article = self.goose.extract(raw_html=str(resource.raw_html), url=resource.url)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_TITLE, tag_value=article.title)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_LINK, tag_value=article.canonical_link)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_PUB_DATE, tag_value=article.publish_date)
      #self.addTag(result=reslt, tag_name=CONSTS.TAG_GUID, tag_value=article.link_hash)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_DC_DATE, tag_value=article.additional_data)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_CONTENT_UTF8_ENCODED, tag_value=article.cleaned_text)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_KEYWORDS, tag_value=article.meta_keywords)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_MEDIA, tag_value=getattr(article, "top_image.src", None))
    except IOError as err:
      logger.debug("Goose open file error. It may be unsupported encoding like jp: " + str(err.message))
    except Exception as err:
      logger.debug("Goose parse error: " + str(err.message))
    return reslt
