"""@package docstring
 @file scraper_result.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


import json
import time
import copy
import logging
import types
from dc_processor.base_extractor import BaseExtractor
import dc_processor.Constants as CONSTS
import app.Consts as APP_CONSTS
from app.Utils import varDump


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


class Result(object):


  def __init__(self, config, resId):
    self.name = "Scraper result object"
    self.data = {}
    self.tags = {}
    self.resId = resId
    if config is None:
      self.article_tags = []
    else:
      self.article_tags = [tag for tag in config.options("article_tags")]
    for tag in self.article_tags:
      self.addTag(tag)
    self.name = "Scraper result object"
    self.start = time.clock()
    self.errorCode = CONSTS.ERROR_OK
    self.errorMessage = CONSTS.MSG_ERROR_OK
    self.tagsCount = 0
    self.tagsMask = 0
    # Added metrics support
    self.metrics = []


  def getEmptyTags(self):
    empty_tags = [key for key, value in self.tags.items() if key in self.article_tags and not value]
    return empty_tags


  def getFilledTags(self):
    filled_tags = [key for key, value in self.tags.items() if key in self.article_tags and value]
    return filled_tags


  def addTag(self, tag):
    self.tags[tag] = ""


  def get(self):
    data = {}
    data["resId"] = self.resId
    data["tagList"] = []

    # Convert old format to new collection format
    data["tagList"].append([])
    
    # Metrics support
    # Add metric's info to the content's data
    M = {}
    for metric in self.metrics:
      M[metric.name] = metric.metricValue
    data["metrics"] = M
    for key, value in self.tags.items():
      if value!="":
        tag = {}
        for key  in value:
          tag[key] = value[key]
        self.tagsCount += 1
        # set tag's mask only if tag is registered
        if tag["name"] in BaseExtractor.tagsMask:
          self.tagsMask = self.tagsMask | BaseExtractor.tagsMask[tag["name"]]

        data["tagList"][0].append(copy.copy(tag))
    self.data["data"] = data
    self.data["error_code"] = self.errorCode
    self.data["error_message"] = self.errorMessage
    self.data["time"] = "%s sec." % (time.clock() - self.start)
    return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ":"))


  def mergeResults(self, results_dict):
    logging.debug("results_dict: %s" % varDump(results_dict))
    tmp = {}
    for item_name, item_value in self.tags.items():
      logging.debug("item_name: %s, item_value: %s" % (item_name, item_value))
      tmp[item_name] = []
      for key, value in results_dict.items():
        logging.debug("key: %s, value: %s" % (key, value))
        tmp[item_name].append(value.tags[item_name])

      logging.debug("BEFORE: %s" % varDump(tmp[item_name]))
      tmp[item_name] = self.getBestValue(tmp[item_name])
      logging.debug("AFTER: %s" % varDump(tmp[item_name]))
    logging.debug("SUMMARY: %s" % varDump(tmp))
    self.tags = tmp


  def getBestValue(self, items_list):
    tmp = [item for item in items_list if item!=""]
    response = ""
    # if more than one suggestions try to select best one
    if len(tmp)>1:
      # for each tag own rule
      # for content_encoded select biggest text
      if tmp[0]["name"]=="content_encoded":
        response = max(tmp, key = lambda x: x["data"])
      # for any else apply the same rule
      else:
        response = max(tmp, key = lambda x: x["data"])
    # if only one suggestion return it
    elif len(tmp)>0:
      response = tmp[0]
    # if no one suggestions return empty string
    else:
      response = ""
    return response


  def stripResult(self):
    for key in self.tags:
      if type(self.tags[key]) in types.StringTypes:
        self.tags[key] = self.tags[key].strip()
      elif (type(self.tags[key]) is types.DictType and "data" in self.tags[key] and \
      type(self.tags[key]["data"]) in types.StringTypes):
        self.tags[key]["data"] = self.tags[key]["data"].strip()
