'''
Created on Apr 10, 2014

@package: dc
@author: scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''
from DCCObjectsSerializator import DCCObjectsSerializator
from transport.ConnectionBuilderLight import ConnectionBuilderLight
from cement.core import foundation
from transport.Event import EventBuilder
from dc.Constants import EVENT_TYPES as EVENT_TYPES
import app.Utils as Utils  # pylint: disable=F0401
import app.Exceptions as Exceptions
import transport.Consts
import Constants as CONSTANTS
import ConfigParser
import dc.EventObjects
import logging
import logging.config
import json
import sys
import app.Consts as APP_CONSTS
import inspect
import types


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
#logger = logging.getLogger(CONSTANTS.APP_NAME)


##DCC Class contents main functional of DCC application, class inherits from foundation.CementApp
#
class DCC(foundation.CementApp):

  class Meta:  # pylint: disable=W0232, C1001
    label = CONSTANTS.APP_NAME

  ##constructor
  #initialise all class variable and recieve connectionBuilder as param(not mandatory)
  def __init__(self, connectionBuilder=None):
    foundation.CementApp.__init__(self)
    self.config = ConfigParser.ConfigParser()
    self.connectionBuilder = connectionBuilder
    self.localConnection = None
    self.eventBuilder = EventBuilder()
    self.errorCode = CONSTANTS.ERROR_NOERROR
    self.errorStr = ""
    self.sendEventType = None
    self.sendEventObject = None


  ##fillError method
  #calls from error-code point from main processing (...from event handlers)
  #errorStr - external errorMessage
  #errorCode - external errorCode
  def fillError(self, errorStr, errorCode, isLogging=True):
    self.errorCode = errorCode
    self.errorStr = errorStr
    if isLogging:
      logger.error(self.errorStr)


  ##connectionInit method
  #initializes internal variables that containts network connections/communications
  def connectionInit(self):
    if self.connectionBuilder == None:
      self.connectionBuilder = ConnectionBuilderLight()
    host = str(self.config.get(CONSTANTS.APP_NAME, CONSTANTS.DTM_HOST))
    if len(host) > 1 and host[0] == "\"":
      host = host[1:]
    if host[-1] == "\"":
      host = host[0:-1]
    port = str(self.config.get(CONSTANTS.APP_NAME, CONSTANTS.DTM_PORT))
    addr = host + ":" + port
    self.localConnection = self.connectionBuilder.build(transport.Consts.CLIENT_CONNECT, addr,
                          transport.Consts.TCP_TYPE)


  ##generateEmptyResponse method
  #If here was some critical error, we generate empty response here , instead real response
  #task - task arg
  def generateEmptyResponse(self):
    obj = dc.EventObjects.ClientResponse([])
    obj.error_code = self.errorCode
    obj.error_message = self.errorStr
    return obj.toJSON()


  ##taskProcessingDeserialize method
  #Method serializes incoming task to the JSON string
  #task - task arg
  #eventObj - response event object
  def transportCommunications(self, task, eventObj):
    timeout = None
    retEvent = None
    ret = None
    try:
      timeout = self.config.get(CONSTANTS.APP_NAME, CONSTANTS.TCP_TIMEOUT_CONFIG_NAME)
    except ConfigParser.NoSectionError:
      timeout = CONSTANTS.TCP_TIMEOUT
    except ConfigParser.NoOptionError:
      timeout = CONSTANTS.TCP_TIMEOUT
    eventType = None
    
    '''
    START_TASK_NUM = 0
    FINISH_TASK_NUM = 19
    for num in xrange(START_TASK_NUM, FINISH_TASK_NUM):
      if task == CONSTANTS.TASKS[num]:
        eventType = EVENT_TYPES.SITE_NEW + num
        break
    '''

    if task == CONSTANTS.TASKS[0]:
      eventType = EVENT_TYPES.SITE_NEW
    elif task == CONSTANTS.TASKS[1]:
      eventType = EVENT_TYPES.SITE_UPDATE
    elif task == CONSTANTS.TASKS[2]:
      eventType = EVENT_TYPES.SITE_STATUS
    elif task == CONSTANTS.TASKS[3]:
      eventType = EVENT_TYPES.SITE_DELETE
    elif task == CONSTANTS.TASKS[4]:
      eventType = EVENT_TYPES.SITE_CLEANUP
    elif task == CONSTANTS.TASKS[5]:
      eventType = EVENT_TYPES.URL_NEW
    elif task == CONSTANTS.TASKS[6]:
      eventType = EVENT_TYPES.URL_STATUS
    elif task == CONSTANTS.TASKS[7]:
      eventType = EVENT_TYPES.URL_UPDATE
    elif task == CONSTANTS.TASKS[8]:
      eventType = EVENT_TYPES.URL_FETCH
    elif task == CONSTANTS.TASKS[9]:
      eventType = EVENT_TYPES.URL_DELETE
    elif task == CONSTANTS.TASKS[10]:
      eventType = EVENT_TYPES.URL_CLEANUP
    elif task == CONSTANTS.TASKS[11]:
      eventType = EVENT_TYPES.URL_CONTENT
    elif task == CONSTANTS.TASKS[12]:
      eventType = EVENT_TYPES.SITE_FIND
    elif task == CONSTANTS.TASKS[13]:
      eventType = EVENT_TYPES.SQL_CUSTOM
    elif task == CONSTANTS.TASKS[14]:
      eventType = EVENT_TYPES.BATCH
    elif task == CONSTANTS.TASKS[15]:
      eventType = EVENT_TYPES.URL_PURGE
    elif task == CONSTANTS.TASKS[16]:
      eventType = EVENT_TYPES.FIELD_RECALCULATE
    elif task == CONSTANTS.TASKS[17]:
      eventType = EVENT_TYPES.URL_VERIFY
    elif task == CONSTANTS.TASKS[18]:
      eventType = EVENT_TYPES.URL_AGE
    elif task == CONSTANTS.TASKS[19]:
      eventType = EVENT_TYPES.URL_PUT
    self.sendEventType = eventType
    self.sendEventObject = eventObj
    mergeVal = None
    try:
      if self.pargs.merge != None:
        if int(self.pargs.merge) > 0:
          mergeVal = True
        else:
          mergeVal = False
    except ValueError as err:
      pass

    event = self.eventBuilder.build(eventType, eventObj)
    if mergeVal != None:
      event.cookie = {}
      if mergeVal:
        event.cookie[CONSTANTS.COOKIE_MERGE_NAME] = 1
      else:
        event.cookie[CONSTANTS.COOKIE_MERGE_NAME] = 0
    self.localConnection.send(event)

    try:
      if self.localConnection.poll(timeout) == 0:
        self.fillError(CONSTANTS.ERROR_STR7.format(str(timeout)), CONSTANTS.ERROR_NETWORK)
      else:
        retEvent = self.localConnection.recv()
      if retEvent != None:
        ret = retEvent.eventObj
    except Exception as err:
      self.fillError("Error: " + str(err.message), CONSTANTS.ERROR_EXCEPTION)

    return ret


  ##configReader method
  #Method try to read config file by prereared paths, return count of readed configs
  def configReader(self):
    configReadList = []
    if self.pargs.config == None:
      configReadList = self.config.read(CONSTANTS.DEFAULT_CONFIG_NAME1)
      if len(configReadList) == 0:
        configReadList = self.config.read(CONSTANTS.DEFAULT_CONFIG_NAME2)
    else:
      configReadList = self.config.read(self.pargs.config)
    return len(configReadList)


  ##load logging
  #load logging configuration (log file, log level, filters)
  #
  def loadLogConfigFile(self):
    global logger  # pylint: disable=W0603
    try:
      logIniFileName = self.config.get(CONSTANTS.LOG_CONFIG_SECTION_NAME, CONSTANTS.LOG_CONFIG_OPTION_NAME)
      if logIniFileName != None:
        logging.config.fileConfig(logIniFileName)
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
      sys.stdout.write(CONSTANTS.ERROR_STR10)
      self.errorCode = CONSTANTS.ERROR_LOG_SECTION_ERROR
    except Exception:
      sys.stdout.write(CONSTANTS.ERROR_STR11)
      self.errorCode = CONSTANTS.ERROR_LOG_INIT
    #logger = logging.getLogger(CONSTANTS.APP_NAME)
    logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


  ##commandProcessingDeserialize method
  #Reads task from file and deserializes it.
  #command - command arg
  #fileName - json filename
  def commandProcessingDeserialize(self, command, fileName):
    ffile = open(fileName, "r")
    data = ffile.read()
    ffile.close()
    jsonData = json.loads(data)
    dCCObjectsSerializator = DCCObjectsSerializator()
    eventObj = None
    if command == CONSTANTS.TASKS[0]:
      eventObj = dCCObjectsSerializator.siteNewDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[1]:
      eventObj = dCCObjectsSerializator.siteUpdateDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[2]:
      eventObj = dCCObjectsSerializator.siteStatusDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[3]:
      eventObj = dCCObjectsSerializator.siteDeleteDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[4]:
      eventObj = dCCObjectsSerializator.siteCleanupDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[5]:
      eventObj = dCCObjectsSerializator.URLNewDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[6]:
      eventObj = dCCObjectsSerializator.URLStatusDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[7]:
      eventObj = dCCObjectsSerializator.URLUpdateDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[8]:
      eventObj = dCCObjectsSerializator.URLFetchDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[9]:
      eventObj = dCCObjectsSerializator.URLDeleteDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[10]:
      eventObj = dCCObjectsSerializator.URLCleanupDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[11]:
      eventObj = dCCObjectsSerializator.URLContentDeserialize(jsonData)
    # find site by root url
    elif command == CONSTANTS.TASKS[12]:
      #logger.debug("jsonData: %s", str(jsonData))
      eventObj = dCCObjectsSerializator.siteFindDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[13]:
      eventObj = dCCObjectsSerializator.SQLCustomDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[14]:
      eventObj = dCCObjectsSerializator.BatchDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[15]:
      eventObj = dCCObjectsSerializator.URLPurgeDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[16]:
      eventObj = dCCObjectsSerializator.FieldRecalculatorDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[17]:
      eventObj = dCCObjectsSerializator.URLVerifyDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[18]:
      eventObj = dCCObjectsSerializator.URLAgeDeserialize(jsonData)
    elif command == CONSTANTS.TASKS[19]:
      eventObj = dCCObjectsSerializator.URLPutDeserialize(jsonData)
    return eventObj


  ##commandListSerialize method
  #Method implements common serialize processing for list of elements
  #command - command arg
  #eventObj - response event object
  #eventObjClassName - list class name
  #listElementClassName - list element class name
  #excpString - exception substring
  def commandListSerialize(self, command, eventObj, eventObjClassName, listElementClassName, excpString):
    if eventObjClassName != [].__class__.__name__:
      raise Exceptions.WrongEventObjectTypeException(CONSTANTS.ERROR_STR8.format(command, "list[]"))
    i = 0
    for listElement in eventObj:
      i += 1
      eventObjClassName = listElement.__class__.__name__
      if eventObjClassName != listElementClassName:
        raise Exceptions.WrongEventObjectTypeException(CONSTANTS.ERROR_STR8.format(command, excpString + str(i) + "]"))


  ##commandProcessingSerialize method
  #Method serializes incoming task to the JSON string
  #command - command arg
  #eventObj - response event object
  def commandProcessingSerialize(self, command, eventObj):
    ret = None
    eventObjClassName = eventObj.__class__.__name__
    if eventObjClassName != dc.EventObjects.ClientResponse([]).__class__.__name__:
      raise Exceptions.WrongEventObjectTypeException(CONSTANTS.ERROR_STR8.format(command, "ClientResponse"))
    try:
      ret = eventObj.toJSON()
    except Exception as exp:
      logger.error(">>> Serialize error %s %s", Utils.varDump(eventObj), str(exp.message))
      self.fillError("Error: " + CONSTANTS.ERROR_STR13, CONSTANTS.ERROR_JSON)
    return ret


  ##setup method
  #Method calls before run application
  def setup(self):
    foundation.CementApp.setup(self)
    self.args.add_argument("-c", "--config", action="store")
    additionHelpStr = CONSTANTS.HELP_COMMAND_TEMPLATE + str(CONSTANTS.TASKS)
    self.args.add_argument("-cmd", "--command", action="store", help=additionHelpStr)
    self.args.add_argument("-f", "--file", action="store")
    self.args.add_argument("-mrg", "--merge", action="store")
    self.args.add_argument("-ff", "--fields", action="store")
    self.args.add_argument("-mode", "--mode", action="store")


  # #Command processing build object instance by command name
  # initialize fields with None value
  # @param objectName name of an object class
  #
  def commandProcessingBuild(self, objectName):
    objectInst = None

    for args in range(0, CONSTANTS.OBJECT_MAX_INIT_ARGUMENTS):
      try:
        #Iteratively try to create instance with mandatory constructor arguments
        if args > 0:
          argsList = ",".join(['None'  for argsList in xrange(args)])
        else:
          argsList = ""
        #Try to create object instance by name
        logger.debug("Try to instantiate as: %s", "dc.EventObjects." + objectName + "(" + argsList + ")")
        objectInst = eval("dc.EventObjects." + objectName + "(" + argsList + ")")
        break
      except TypeError as exp:
        continue
      except Exception as exp:
        logger.error("Create instance of `%s`: %s : type is `%s`", str(objectName), str(exp.message), str(type(exp)))
        raise Exceptions.DeserilizeException("Create instance of object error: " + str(exp.message))

    #Fill with None value all fields
    try:
      for name in dir(objectInst):
        if not name.startswith('__'):
          setattr(objectInst, name, None)
    except Exception as exp:
      logger.error("Build object `%s`: %s", str(objectName), str(exp.message))
      raise Exceptions.DeserilizeException("Build object error: " + str(exp.message))

    return objectInst


  # #Command processing fill object instance attributes from fields set json
  # initialize fields with None value
  # @param objectInst object instance
  # @param fieldsJson fields json
  #
  def commandProcessingFill(self, objectInst, fieldsJson):
    try:
      #De-serialize fields set json
      fieldsDict = json.loads(fieldsJson)

      #Process load from file reference
      fieldsDict = self.dictFillFromFile(fieldsDict)
      logger.debug("fieldsDict: %s", str(fieldsDict))
      logger.debug("objectInst: %s", Utils.varDump(objectInst))
      logger.debug("mode: %s", self.pargs.mode)

      if isinstance(objectInst, list):
        newList = []
        #Set fields in object if it is list of objects
        for itemObj in objectInst:
          #logger.debug("Type of item is: %s", str(type(itemObj)))
          for name in dir(itemObj):
            if not name.startswith('__') and name in fieldsDict:
              self.setAttr(itemObj, name, fieldsDict[name], self.pargs.mode)
          newList.append(itemObj)
        objectInst = newList
      else:
        #Set fields in object if it is single
        for name in dir(objectInst):
          if not name.startswith('__') and name in fieldsDict:
            #  setattr(objectInst, name, fieldsDict[name])
            self.setAttr(objectInst, name, fieldsDict[name], self.pargs.mode)

    except Exception as exp:
      logger.error("Fill object: `%s`,\ntype: `%s`,\nfields: `%s`,\nerror: %s, %s", Utils.varDump(objectInst), \
		   str(type(objectInst)), str(fieldsJson), str(exp.message), Utils.getTracebackInfo())
      raise Exceptions.DeserilizeException("Fill object error: " + str(exp.message))

    return objectInst


  # #Sets object attributes without inheritance overwrite or not
  #
  # @param obj object instance
  # @param name attribute name of object
  # @param valsDict dict of attribute values
  # @param mode 0 - not nested overwrite mode, 1 - nested overwrite, 2 - nested overwrite with append
  #
  def setAttr(self, obj, name, valsDict, mode):
    if mode == None or mode == "" or mode == "0" or (not hasattr(obj, name)):
      #Without nesting, only one first level of fields overwrite
      setattr(obj, name, valsDict)
    else:
      #With nesting support, overwrite and/or append
      objAttr=getattr(obj, name)
      #if hasattr(objAttr, '__class__'):
      if str(type(objAttr)).startswith('<class'):
        #If attribute is a class
        for name2 in dir(objAttr):
          if not name2.startswith('__'):
            #print str(name2) + " | " + str(valsDict)
            if isinstance(valsDict, dict) and name2 in valsDict:
              objAttr2=getattr(objAttr, name2)
              #if hasattr(objAttr2, "__class__"):
              if str(type(objAttr2)).startswith('<class'):
                self.setAttr(objAttr, name2, valsDict[name2], mode)
              else:
                setattr(objAttr, name2, self.mergeDictList(objAttr, valsDict[name2], mode))
        setattr(obj, name, objAttr)
      else:
        #If attribute is not a class
        setattr(obj, name, self.mergeDictList(objAttr, valsDict, mode))


  # #Merges two dictionaries with nesting of levels
  #
  # @param iter1 dictionary or list to merge
  # @param iter2 dictionary or list to merge with
  # @param mode 0 - not nested overwrite mode, 1 - nested for dicts and overwrite for lists, 2 - nested overwrite with append if not present for dicts and append for lists
  # If iter1 and iter2 is not the same types or not dict and list type - iter2 overwrites iter1 not nested way (mode 0), and nested way (mode 1 and 2)
  #
  def mergeDictList(self, iter1, iter2, mode):
    retIter=None

    if isinstance(iter1, dict) and isinstance(iter2, dict):
      #Both iterable are dicts
      if mode == None or mode == "" or mode == "0":
        #Just overwrite with copy
        retIter=dict(iter2)
      else:
        #Init with source copy
        retIter=dict(iter1)
        #Process source items
        for key, value in iter2.iteritems():
          #If key exists in source and destination or mode 2 (need to be appended)
          if key in retIter or mode == "2":
            retIter[key]=self.mergeDictList(retIter[key], value, mode)
    else:
      if isinstance(iter1, list) and isinstance(iter2, list):
        #Both iterable are lists
        if mode == None or mode == "" or mode == "0" or mode == "1":
          #Just overwrite with copy
          retIter=list(iter2)
        else:
          #Init with source copy
          retIter=list(iter1)
          #Process source items
          for item in iter2:
            retIter.append(item)
      else:
        #Types of iter1 and iter2 are not the same
        if isinstance(iter2, dict):
          #Overwrite with dict copy
          retIter=dict(iter2)
        else:
          if isinstance(iter2, list):
            #Overwrite with list copy
            retIter=list(iter2)
          else:
            #Overwrite with copy of none iterable type or reference of the object
            retIter=iter2

    return retIter


  # #Command processing fill object instance attributes from fields set json
  # initialize fields with None value
  # @param objectInst object instance
  # @param fieldsJson fields json
  #
  def dictFillFromFile(self, d):
    for k, v in d.iteritems():
      if isinstance(v, dict):
        d[k] = self.dictFillFromFile(v)
      else:
        if (isinstance(v, str) or isinstance(v, unicode)) and v.startswith(CONSTANTS.FILE_PROTOCOL_SIGNATURE):
          logger.debug("File:%s", str(v[len(CONSTANTS.FILE_PROTOCOL_SIGNATURE):]))
          d[k] = open(v[len(CONSTANTS.FILE_PROTOCOL_SIGNATURE):], 'r').read()

    return d


  ##run method
  #Method contains main application functionality
  def run(self):
    foundation.CementApp.run(self)
    isHelpArg = False
    if '-h' in self.argv or '--help' in self.argv:
      isHelpArg = True
    eventObj = None
    retEventObj = None
    jsonBuf = None

    if self.configReader() > 0:
      self.loadLogConfigFile()

      if self.errorCode == CONSTANTS.ERROR_NOERROR:
        try:
          self.connectionInit()
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
          self.fillError(CONSTANTS.ERROR_STR10, CONSTANTS.ERROR_CONFIG_SECTION)
        if isHelpArg == False and self.pargs.command == None:
          self.fillError(CONSTANTS.ERROR_STR1, CONSTANTS.ERROR_ARGS1)
        elif self.localConnection == None:
          self.fillError(CONSTANTS.ERROR_STR12, CONSTANTS.ERROR_CONNECTION)
        elif self.pargs.command != None:
          if self.pargs.command in CONSTANTS.TASKS:
            if self.pargs.file != None or self.pargs.fields != None:
              try:
                if self.pargs.file != None:
                  #De-serialize object from json file
                  eventObj = self.commandProcessingDeserialize(self.pargs.command, self.pargs.file)
                else:
                  #Dynamic object creation and initialize all fields with None
                  eventObj = self.commandProcessingBuild(
                                  CONSTANTS.TASKS_OBJECTS[CONSTANTS.TASKS.index(self.pargs.command)])
                #Cover some fields with values from command line argument
                if self.pargs.fields != None:
                  #Fill some object's fields with values
                  eventObj = self.commandProcessingFill(eventObj, self.pargs.fields)
                  #logger.debug("resulted eventObj: %s", str(eventObj))
                  logger.debug("resulted eventObj: %s", Utils.varDump(eventObj))
              except IOError:
                self.fillError(CONSTANTS.ERROR_STR4, CONSTANTS.ERROR_BAD_FILE_NAME)
              except ValueError:
                self.fillError(CONSTANTS.ERROR_STR5, CONSTANTS.ERROR_BAD_JSON)
              except Exceptions.DeserilizeException as excp:
                self.fillError(CONSTANTS.ERROR_STR6.format(excp.message), CONSTANTS.ERROR_DCC)
              if eventObj != None:
                retEventObj = self.transportCommunications(self.pargs.command, eventObj)
                if retEventObj != None:
                  try:
                    jsonBuf = self.commandProcessingSerialize(self.pargs.command, retEventObj)
                  except Exceptions.WrongEventObjectTypeException as excp:
                    self.fillError(excp.message, CONSTANTS.ERROR_WRONG_RESPONSE)
              elif self.errorCode == CONSTANTS.ERROR_NOERROR:
                self.fillError(CONSTANTS.ERROR_STR14, CONSTANTS.ERROR_OBJECT_CREATE)
            else:
              self.fillError(CONSTANTS.ERROR_STR2, CONSTANTS.ERROR_ARGS2)
          else:
            self.fillError(CONSTANTS.ERROR_STR3, CONSTANTS.ERROR_BAD_TASK)
    else:
      self.fillError(CONSTANTS.ERROR_STR9, CONSTANTS.ERROR_NO_CONFIG, False)

    if jsonBuf == None:
      jsonBuf = self.generateEmptyResponse()
    sys.stdout.write(jsonBuf)
    sys.exit(self.errorCode)


  ##close method
  #Method calls after application run
  def close(self):
    foundation.CementApp.close(self)
