'''
Created on Feb 13, 2014

@author: igor
'''


import json

from transport.Request import Request
from transport.Response import ResponseFormatErr
from transport.UIDGenerator import UIDGenerator
from CommandConvertor import CommandConvertorError, TaskExecuteStructEncoder
from Commands import DRCECover
import logging
import app.Consts as APP_CONSTS


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


#import base64
##Exception which encapsulates errors related to convertatin operations
#
class CommandExecutorErr(Exception):

  def __init__(self, msg):
    Exception.__init__(self, msg)


##Main point for execution commands related to the drce
#It hides all complexities of command execution
#
class CommandExecutor(object):


  ##constructor
  #
  #@param connection an instance of transport.Connection object
  #@param cmd_convertor an instance of CommandConvertor object
  def __init__(self, connection, cmd_convertor):
    self.connection = connection
    self.cmd_convertor = cmd_convertor
    #@var id_generator
    #a member variable, used to generate uid for messages
    self.id_generator = UIDGenerator()


  ##simple wrapper to explicit express operation of replace connection
  #add ability to execute commands on many connections
  #
  #@param connection an instance of transport.Connection object
  #@return None
  def replace_connection(self, connection):
    self.connection = connection



  ##execute Task*Request command
  #
  #because a connection is external dependence - doesn't catch it exceptions
  #@param command an instance of Task*Request object
  #@param timeout timeout in msec
  #@param ttl time of task live in msec
  #@param maxTries max tries to receive message, if zero and received message that is not the same id as requested
  #                it will be returned, if grater than zero than N tries to receive message will be performed. If
  #                timeout reached it breaks loop and exception raised
  #@return TaskResponse or throw CommandExecutorErr
  def execute(self, command, timeout=1000, ttl=30000, maxTries=100):
    if maxTries < 0:
      maxTries = 0

    try:
      cmd_json = self.cmd_convertor.to_json(command)
      drce_cover_envelop = DRCECover(ttl, cmd_json)  #json.dumps(cmd_json))
      request = Request(self.id_generator.get_uid())
      request.add_data(json.dumps(drce_cover_envelop, cls=TaskExecuteStructEncoder))
      request.route = command.route

      logger.debug("Send DRCE request msg Id:" + request.get_uid())

      self.connection.send(request)

      for i in range(maxTries + 1):
        response = self.connection.recv(timeout)
        logger.debug("Received DRCE response msg Id:" + response.get_uid())
        if response.get_uid() == request.get_uid() or maxTries == 0:
          cover_envelop_response = json.loads(response.get_body())
          return self.cmd_convertor.from_json(cover_envelop_response["data"])
        else:
          logger.error("DRCE response msg Id:" + response.get_uid() + \
                       " not matched with the request msg Id:" + request.get_uid() + \
                       ", try: " + str(i))
    except (ResponseFormatErr, CommandConvertorError, KeyError, CommandExecutorErr) as err:
      raise CommandExecutorErr(err.message)

