'''
Created on Mar 24, 2014

@author: igor
'''
import os
import ConfigParser
import logging
import json
import base64
import signal
from app.BaseServerManager import BaseServerManager
from transport.ConnectionBuilder import ConnectionBuilder
from transport.ConnectionBuilderLight import ConnectionBuilderLight
from transport.IDGenerator import IDGenerator
from transport.Connection import ConnectionParams
from transport.Request import Request
import transport.Consts as tr_consts
import admin.Constants as admin_const


logger = logging.getLogger(__name__)

FAKE_EVENT_TYPE = 10000

TASK_EXECUTE_TYPE = 0
TASK_CHECK_TYPE = 1
TASK_TERMINATE_TYPE = 2
TASK_GET_DATA_TYPE = 3

operation_names = dict({TASK_EXECUTE_TYPE:"new_task",
                                         TASK_CHECK_TYPE:"check",
                                         TASK_TERMINATE_TYPE:"delete",
                                         TASK_GET_DATA_TYPE:"update"})


class EESimulator(BaseServerManager):

        
    #Configuration settings options names    
    DRCE_SERVER = "drce_server"
    ADMIN_SERVER = "admin_server"
    
    CONFIG_DRCE_SERVER_HOST = "drceServerHost"
    CONFIG_DRCE_SERVER_PORT = "drceServerPort"
    CONFIG_ADMIN_SERVER_HOST = "adminServerHost"
    CONFIG_ADMIN_SERVER_PORT = "adminServerPort"
    

    ##constructor 
    #@param configParser config parser object
    def __init__(self, configParser, path_to_data):
        super(EESimulator, self).__init__()
        #path to directory with response files        
        self.path_to_data = path_to_data
        self.connetBuilder = ConnectionBuilder(IDGenerator())
        
        cgf_section = self.__class__.__name__        
        drceServerHost = configParser.get(cgf_section, self.CONFIG_DRCE_SERVER_HOST)
        drceServerPort = configParser.get(cgf_section, self.CONFIG_DRCE_SERVER_PORT)
        adminServerHost = configParser.get(cgf_section, self.CONFIG_ADMIN_SERVER_HOST)
        adminServerPort = configParser.get(cgf_section, self.CONFIG_ADMIN_SERVER_PORT)
        
        drceServer = self.connetBuilder.build(tr_consts.ADMIN_CONNECT_TYPE, 
                                                                     ConnectionParams(drceServerHost, drceServerPort), 
                                                                     tr_consts.SERVER_CONNECT)

        adminServer = self.connetBuilder.build(tr_consts.ADMIN_CONNECT_TYPE, 
                                                                       ConnectionParams(adminServerHost, adminServerPort), 
                                                                       tr_consts.SERVER_CONNECT)
       
        self.addConnection(self.DRCE_SERVER, drceServer)
        self.addConnection(self.ADMIN_SERVER, adminServer)
        
        
    def on_unhandled_event(self, event):
        response = None
        if event.connect_name == self.DRCE_SERVER:
            response = self.processNewTaskCommand(event)
        else:        
            response  = self.processDRCEAdminCommand(event)
                    
        event.connect_identity = event.eventObj.connect_identity            
        eventResponse = self.eventBuilder.build(FAKE_EVENT_TYPE, response)
        self.reply(event, eventResponse)
            
            
    def processNewTaskCommand(self, event):
        json_string = json.loads(event.eventObj.get_body())        
        print json_string["data"]
        json_body = json.loads(base64.b64decode(json_string["data"]))        
        request_id = str(int(json_body["id"]))
        request = Request(request_id)
        if json_body["type"] == TASK_EXECUTE_TYPE:            
            response_body = self.load_from_file(request_id, "new_task")
            request.add_data(response_body)
        else:
            response_body = self.load_from_file(0, "new_task_err")
            request.add_data(response_body)
        
        return request
           
    
    def processDRCEAdminCommand(self, event):        
        admin_cmd = event.eventObj.get_body().split(admin_const.COMMAND_DELIM)
        print admin_cmd
        if admin_cmd[1] == admin_const.COMMAND_NAMES.DRCE:            
            json_body = json.loads(admin_cmd[2])            
            request_id = str(int(json_body["id"]))
            request = Request(request_id)            
            if json_body["type"] in (TASK_EXECUTE_TYPE, TASK_CHECK_TYPE, TASK_TERMINATE_TYPE, TASK_GET_DATA_TYPE):
                response_body = self.load_from_file(request_id, operation_names[ json_body["type"]])
                request.add_data(response_body)
            else:
                response_body = self.load_from_file(0, "err")
                request.add_data(response_body)
            
            return request         
        else:
            logger.error("Get wrong command ", event.eventObj.get_body())
    
    
    def load_from_file(self, uid, operation_name):  
        file_name = self.path_to_data + "/dtmds_" + operation_name + "_" + str(uid) + "_response.json"        
        return open(file_name).read()
        


if __name__ == "__main__":
    FORMAT = '%(asctime)s - %(thread)ld - %(threadName)s - %(name)s - %(funcName)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.DEBUG, format=FORMAT) #basic logging configuration

    connectBuilder = ConnectionBuilderLight()
    serverConnectAdmin = connectBuilder.build(tr_consts.SERVER_CONNECT, BaseServerManager.ADMIN_CONNECT_ENDPOINT)
    
    cur_dir =  os.path.dirname(os.path.abspath(__file__))
    config = ConfigParser.ConfigParser()
    
    config.read(cur_dir + "/" +"ees.cfg")
    
    ees = EESimulator(config, cur_dir)        
    ees.start()
    
    def onSigIntHandler(signum, stackFrame):
        print "Got sigint"
        ees.exit_flag = True
    
    signal.signal(signal.SIGINT, onSigIntHandler) # install signal handler
    
    signal.pause() 
    ees.join()
    print "OK"