'''
Created on Feb 4, 2014

@author: igor
'''


from collections import namedtuple
import zmq

import Consts as consts
from Response import Response
import logging
import app.Consts as APP_CONSTS


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


##Class is used to inform about expiration of operation timeout
#
class ConnectionTimeout(Exception):

  def __init__(self, message):
    Exception.__init__(self, message)


##Exception which express some critical errors in the network engine
#
#At the moment critical errors are zmq.ZMQError, zmq.Again
class TransportInternalErr(Exception):

  def __init__(self, message):
    Exception.__init__(self, message)


##Structure which describes all data need to establish connection
#
#@todo - add a protocol param as independent part
ConnectionParams = namedtuple("ConnectionParams", "host port")


##Main class of the transport layer.
#
#The class has two main methods - send, recv which are used to
#exchange messages
class Connection(object):
  '''
  Transport class
  '''

  RECV_DEFAULT_TIMEOUT = 5000

  ##constructor
  #
  #@param zmq_socket an instance of zmq.socket object
  #@param zmq_poller an instance of zmq.poller object
  #@param socket_type type of socket(seerver or client side)
  def __init__(self, zmq_socket, zmq_poller, socket_type=consts.CLIENT_CONNECT):
    self.zmq_socket = zmq_socket
    self.zmq_poller = zmq_poller
    self.socket_type = socket_type
    #@todo remove poller
    #self.zmq_poller.register(self.zmq_socket, zmq.POLLIN)


  ##send request
  #
  #@param request an instance of Request object
  #@return None or throw TransportInternalErr
  def send(self, request):
    try:
      if self.socket_type == consts.CLIENT_CONNECT:
        return self.zmq_socket.send_multipart(request.get_body())
      else:
        #TODO: old version, can be removed in stable revision
        #snd_data = list(request.eventObj.get_body())
        #snd_data.insert(0, request.connect_identity)
        #return self.zmq_socket.send_multipart(snd_data)

        if request.route is not None:
          snd_data = [request.connect_identity] + request.eventObj.get_body() + [request.route]
        else:
          snd_data = [request.connect_identity] + request.eventObj.get_body()
        return self.zmq_socket.send_multipart(snd_data)

    except zmq.ZMQError as err:  # pylint: disable-msg=E1101
      raise TransportInternalErr(err.message)


  ##recieve data from connection
  #
  #@param timeout timeout in msec, default RECV_DEFAULT_TIMEOUT msec
  #@return Response or throw ConnectionTimeout, TransportInternalErr
  def recv(self, timeout=RECV_DEFAULT_TIMEOUT):
    try:
      events = self.zmq_socket.poll(timeout, zmq.POLLIN)  # pylint: disable=E0602,E1101
      #self.zmq_poller.register(self.zmq_socket, zmq.POLLIN)
      #socks = dict(self.zmq_poller.poll(timeout))
      #if  self.zmq_socket in socks and socks[self.zmq_socket] == zmq.POLLIN:
      if  events == zmq.POLLIN:  # pylint: disable-msg=E1101
        if self.socket_type == consts.CLIENT_CONNECT:
          return Response(self.zmq_socket.recv_multipart())
        else:
          data = self.zmq_socket.recv_multipart()
          identity = data[:1]
          return Response(data[1:], identity[0])
      else:
        raise ConnectionTimeout("recv timeout " + str(timeout))
    except (zmq.ZMQError, zmq.Again) as err:  # pylint: disable-msg=E1101
      raise TransportInternalErr(err.mesage)
    #finally:
#      self.zmq_poller.unregister(self.zmq_socket)


  #close zqm.socket
  #
  #@return None
  def close(self):
    #self.zmq_poller.unregister(self.zmq_socket) #is right place??
    self.zmq_socket.close()


  ##check status of zqm.socket
  #
  #@return bool
  def is_closed(self):
    return self.zmq_socket.closed


  #Destroys the object, close connection
  #
  #@return None
def __del__(self):
  try:
    self.close()
  except Exception:
    pass

