#!/bin/bash

PROCESS=dtm-daemon.py
MANAGE=dtm-client.py

LOGDIR='../log/'
LOG=$0.log
LOG_TMP="$0.log.tmp"

if [[ "$1" == "--help" || "$1" == "-h" ]]
then
  echo "Usage:"
  echo "$0 - full log to "$LOGDIR$LOG
  echo "$0 [size] - print number of queue to stdout"
  echo "$0 [queue id] [field] - print value of field queue to stdout"
  exit 1
fi

TEMPFILE=`mktemp /tmp/dtm-daemon_tasks.XXXXXX`
trap "rm -f $TEMPFILE" 0 1 2 5 15

function check() {
  # $1 - path to json
  while read LINE
  do
    if [[ $LINE =~ '"ids": [],' ]]
    then
      echo 0
      exit 1
    fi
    if [[ $LINE =~ "ids" ]]
    then
      COUNT_IDS=-2
      CHECK_IDS="TRUE"
    fi
    COUNT_IDS=$((COUNT_IDS+1))
    if [[ $LINE =~ "]" ]]
    then
      break
    fi
  done < $1
  if [[ $CHECK_IDS == "TRUE" ]]
  then
    echo $COUNT_IDS
  else
    echo 0
  fi
}

function find() {
  # $1 - id of queue
  # $2 - field of queue
  # $3 - path to json
  readarray JSON < $3
  for ((LINE=1;LINE<=${#JSON[@]]};++LINE))
  do
    if [[ ${JSON[$LINE]} =~ '"id": '$1 ]]
    then
      for ((INLINE=$LINE;INLINE<=${#JSON[@]]};++INLINE))
      do
        if [[ ${JSON[$INLINE]} =~ "$2" ]]
        then
          echo ${JSON[$INLINE]} | awk -F\"$2\"\: '{print $2}' | sed 's/\"\|\,//g'
          exit 1
        fi
        if [[ ${JSON[$INLINE]} =~ "}" ]]
        then
          break
        fi
      done
      for ((INLINE=$LINE;INLINE>=0;--INLINE))
      do
        if [[ ${JSON[$INLINE]} =~ "$2" ]]
        then
          echo ${JSON[$INLINE]} | awk -F\"$2\"\: '{print $2}' | sed 's/\"\|\,//g'
          exit 1
        fi
        if [[ ${JSON[$INLINE]} =~ "{" ]]
        then
          break
        fi
      done
    fi
  done
  echo 'Field not found'
}

PID=$(pgrep $PROCESS)

if [ $PID ]
then

  if [[ $1 == "size" ]]
  then
    ARG='--config=../ini/dtm-client.ini -t GET_TASKS --file=../data/manage/dtm-daemon_tasks.json'
    cd  ../bin && ./$MANAGE $ARG > $TEMPFILE
    SIZE=$(check $TEMPFILE)
    echo $SIZE
    exit 1
  elif [[ "$1" && "$2" ]]
  then
    ARG='--config=../ini/dtm-client.ini -t GET_TASKS --file=../data/manage/dtm-daemon_tasks_full.json'
    cd  ../bin && ./$MANAGE $ARG > $TEMPFILE
    FIND=$(find $1 $2 $TEMPFILE)
    echo $FIND
    exit 1
  else
    ARG='--config=../ini/dtm-client.ini -t GET_TASKS --file=../data/manage/dtm-daemon_tasks_full.json'
    echo "Service running, pid="$PID
    echo "Checking statistics..."
    cd  ../bin && ./$MANAGE $ARG > $LOGDIR$LOG_TMP
    mv $LOGDIR$LOG_TMP $LOGDIR$LOG
    echo "Saved in $LOGDIR$LOG"
  fi
else
    echo "Process [$PROCESS] stopped..."
fi

exit 0
