#!/bin/bash

DIALOG=${DIALOG=dialog}
BACKTITLE='Install Dependencies for HCE project components: hce-node, DTM and DC'
HCE_DEV_FILE='/etc/hce-dev'


RED='\e[0;31m'
BLUE='\e[1;34m'
DEF='\e[0m'

HCE_USER=hce
MYSQL_USER=hce
MYSQL_PWD=hce12345

SCRIPT_NAME=$(basename $0)

TEMPFILE=`mktemp /tmp/$SCRIPT_NAME.XXXXX`
TMPDIR=`mktemp -d /tmp/$SCRIPT_NAME.XXXXXX`
trap "rm -f $TEMPFILE; rm -rf $TMPDIR" 0 1 2 5 15

# FUNCTIONS #

function infoDialog() {
    # $1 - title
    # $2 - info messages
    # $3 - height
    if [[ -z $3 ]]
    then
        HEIGHT=5
    else
        HEIGHT=$3
    fi
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
            --colors --msgbox "$2" $HEIGHT 80
}

function yesNo() {
    # $1 - title
    # $2 - info messages
    $DIALOG --backtitle "$BACKTITLE" --title "$1" --defaultno \
            --yesno "$2" 20 80
    case $? in
        0)
            CHECK="YES";;
        1)
            CHECK="NO";;
        255)
            infoDialog "$1" "Cancel. Exit" && clear && exit 1;;
    esac
}

function textDialog() {
    # $1 - title
    # $2 - info messages
    $DIALOG --backtitle "$BACKTITLE" --title "$1" --exit-label "NEXT"\
            --textbox "$2" 20 80
}

function indicatorDialog() {
    # $1 - info messages
    # $2 - info %%
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
            --gauge "$1" 20 80 "$2"

}

function checkSudo() {
    if [[ $(whoami) != root ]]
    then
        infoDialog "Error" "Run script with root permissions or sudo: sudo $0 \"USERNAME\""
        clear
        exit 1
    fi
}

function checkOs() {
    if [[ ! -f '/etc/debian_version' ]]
    then
        infoDialog "Error" "This script is only for the Debian system! Cancelled."
        clear
        exit 1
    fi
}

function checkHomeDir() {
    # $1 = hce user
    if [[ ! -d /home/$1 ]]
    then
        infoDialog "Error" "Home dir (/home/${HCE_USER}) for user ${HCE_USER} not exists! Cancelled."
        clear
        exit 1
    fi
}

function checkInstalledPackage() {
    # $1 - package name
    if [[ $(dpkg-query -l "$1" 2>/dev/null) ]]
    then
        echo 'INSTALLED'
    else
        exit 1
    fi
}

function start() {
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
            --checklist "Check HCE-project dependencies to install" 20 80 10 \
                                                0 "The en_US.UTF-8 locale (if not present)" on \
                                                1 "Install mysql-server" $MYSQL_CHECKBOX \
                                                2 "PHP language API and management tools" on \
                                                3 "DC service" on \
                                                4 "Python language API and management tools" on \
                                                5 "The pyzmq library with zmq version 4.0.3 from source" on \
                                                6 "The phpmyadmin, latest" on \
                                                7 "Copy test site files to the apache virtual host DocumentRoot" on \
                                                8 "The mysql DB support (server, client, databases, user and permissions)" on \
                                                9 "Key-Value storage support for default site" on 2> $TEMPFILE
    case $? in
        0)
            CHECK=$(cat $TEMPFILE);;
        1)
            infoDialog "Select components" "Cancel. Exit" && clear && exit 1;;
        255)
            infoDialog "Select components" "Cancel. Exit" && clear && exit 1;;
    esac
}

function locales() {
    if [[ -z $(checkLocale) ]]
    then
        echo -e '#\nLANG=en_US.UTF-8\n' >> /etc/default/locale
        sed -i 's/# en_US.UTF-8 UTF-8/en_US.UTF-8 UTF-8/' /etc/locale.gen
        (
            COUNT=99
            echo "XXX"
            echo $COUNT
            echo "generate locales..."
            echo "XXX"
            locale-gen
        ) | indicatorDialog "Added locales" "Generate locales..."
        infoDialog "Added locales" "Locale en_US.UTF-8 added" && clear
    else
        infoDialog "Added locales" "Locale en_US.UTF-8 present" && clear
    fi
}

function mysql_server() {
    if [[ $MYSQL_SERVER ]]
    then
        yesNo "Mysql server installation" "Mysql server installed on you system. Reinstall (not recommended)?"
        if [[ $CHECK == "YES" ]]
        then
            apt-get update > $TEMPFILE
            apt-get -y install mysql-server-5.5 >> $TEMPFILE
            textDialog "Mysql server installation..." "$TEMPFILE" && clear
        fi
    else
        apt-get update > $TEMPFILE
        apt-get -y install mysql-server-5.5 >> $TEMPFILE
        textDialog "Mysql server installation..." "$TEMPFILE" && clear
    fi
}

function dtse_php() {
    (
        COUNT=30
        echo "XXX"
        echo $COUNT
        echo "run apt-get update..."
        echo "XXX"
        apt-get update -qq
        COUNT=60
        echo "XXX"
        echo $COUNT
        echo "run apt-get install..."
        echo "XXX"
        apt-get install -y --force-yes bsdutils unzip libzmq3-dev apache2 php5 php5-dev php5-sqlite php-pear pkg-config libpgm-dev sphinxsearch make > $TEMPFILE
        COUNT=80
        echo "XXX"
        echo $COUNT
        echo "run install zmq-beta..."
        echo "XXX"
        echo '' >> $TEMPFILE
        printf "\n" | pecl install zmq-beta >> $TEMPFILE
        if [[ -z $(checkZmqSo) ]]
        then
            COUNT=90
            echo "XXX"
            echo $COUNT
            echo "add zmq module to php ini and reload apache"
            echo "XXX"
            echo -e "; configuration for php ZMQ module \n; priority=20 \nextension=zmq.so" > /etc/php5/cli/conf.d/20-zmq.ini
            service apache2 reload
        fi
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
    ) | indicatorDialog "Install PHP language API and management tools..." ""
    textDialog "Install PHP language API and management tools..." "$TEMPFILE" && clear
}

function dtse_dc() {
    (
        COUNT=50
        echo "XXX"
        echo $COUNT
        echo "run apt-get update..."
        echo "XXX"
        apt-get update -qq
        COUNT=100
        echo "XXX"
        echo $COUNT
        echo "run apt-get install..."
        echo "XXX"
        apt-get install -y bc openjdk-7-jdk ruby php5-gmp g++ > $TEMPFILE
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
    ) | indicatorDialog "Install DC service dependencies..." ""
    textDialog "Install DC service dependencies..." "$TEMPFILE" && clear
}

function dtse_python() {
    (
        COUNT=30
        echo "XXX"
        echo $COUNT
        echo "run apt-get update..."
        echo "XXX"
        apt-get update -qq
        COUNT=70
        echo "XXX"
        echo $COUNT
        echo "run apt-get install..."
        echo "XXX"
        apt-get install -y libpgm-dev python-pip python-dev python-flask python-flaskext.wtf libffi-dev libxml2-dev libxslt1-dev mysql-client libmysqlclient-dev python-mysqldb libicu-dev libgmp3-dev libtidy-dev libjpeg-dev > $TEMPFILE
        COUNT=80
        echo "XXX"
        echo $COUNT
        echo "run pip install..."
        echo "XXX"
        echo '' >> $TEMPFILE
        pip install cement sqlalchemy Flask-SQLAlchemy scrapy gmpy pyzmq lepl requests==2.5 mysql-python pyicu newspaper goose-extractor pytidylib uritools python-magic feedparser Ghost.py pillow beautifulsoup4 w3lib >> $TEMPFILE
        COUNT=90
        echo "XXX"
        echo $COUNT
        echo "run pip install pyzmq (with option zmq=bundle)..."
        echo "XXX"
        echo '' >> $TEMPFILE
        pip install pyzmq --install-option="--zmq=bundled" >> $TEMPFILE
        COUNT=100
        echo "XXX"
        echo $COUNT
        echo "run pip install urlnorm..."
        echo "XXX"
        echo '' >> $TEMPFILE
        pip install -U urlnorm >> $TEMPFILE
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
    ) | indicatorDialog "Install Python language API and management tools..." ""
    textDialog "Install Python language API and management tools..." "$TEMPFILE" && clear
}

function pyzmq() {
    (
        COUNT=20
        echo "XXX"
        echo $COUNT
        echo "download zeromq v 4.0.4..."
        echo "XXX"
        cd $TMPDIR
        wget -q http://download.zeromq.org/zeromq-4.0.4.tar.gz
        COUNT=40
        echo "XXX"
        echo $COUNT
        echo "untar zeromq v 4.0.4..."
        echo "XXX"
        tar -xf zeromq-4.0.4.tar.gz > $TEMPFILE
        cd zeromq-4.0.4 2>>$TEMPFILE
        COUNT=60
        echo "XXX"
        echo $COUNT
        echo "configure, make and make install zeromq..."
        echo "XXX"
        echo '' >> $TEMPFILE
        ./configure -q 2>>$TEMPFILE >> $TEMPFILE && make -s >> $TEMPFILE && make -s install >> $TEMPFILE
        COUNT=90
        echo "XXX"
        echo $COUNT
        echo "install pyzmq with zmq version 4.0.3..."
        echo "XXX"
        echo '' >> $TEMPFILE
        ldconfig >> $TEMPFILE
        echo '' >> $TEMPFILE
        pip install -q pyzmq --install-option="--zmq=4.0.3" >> $TEMPFILE
        COUNT=100
        echo "XXX"
        echo $COUNT
        echo "remove tmp files..."
        echo "XXX"
        cd $TMPDIR
        rm -rf $TMPDIR/*
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
     ) | indicatorDialog "Install pyzmq with zmq v4.0.3 from source..." ""
    textDialog "Install pyzmq with zmq v4.0.3 from source..." "$TEMPFILE" && clear
}

function phpmyadmin() {
    (
        COUNT=10
        echo "XXX"
        echo $COUNT
        echo "run apt-get update and install php5-mysqlnd..."
        echo "XXX"
        apt-get update
        apt-get install -y php5-mysqlnd
        COUNT=30
        echo "XXX"
        echo $COUNT
        echo "download phpmyadmin latest..."
        echo "XXX"
        cd $TMPDIR
        wget -q https://sourceforge.net/projects/phpmyadmin/files/latest/download?source=files -O $TMPDIR/phpmyadmin-latest.zip
        if [[ -z $(file $TMPDIR/phpmyadmin-latest.zip | grep empty) ]]
        then
            COUNT=60
            echo "XXX"
            echo $COUNT
            echo "unzip phpmyadmin..."
            echo "XXX"
            unzip -q phpmyadmin-latest.zip > $TEMPFILE
            rm -f phpmyadmin-latest.zip >> $TEMPFILE
            COUNT=90
            echo "XXX"
            echo $COUNT
            echo "install phpmyadmin..."
            echo "XXX"
            echo '' >> $TEMPFILE
            PHPMYADMINDIR=$(ls)
            if [[ -d /var/www/pma ]]
            then
                rm -rf /var/www/pma
            fi
            mkdir /var/www/pma
            mv $TMPDIR/$PHPMYADMINDIR/* /var/www/pma/ 2>>$TEMPFILE
            mv $TMPDIR/$PHPMYADMINDIR/.[!.]* /var/www/pma/ 2>>$TEMPFILE
            cp /var/www/pma/config.sample.inc.php /var/www/pma/config.inc.php 2>>$TEMPFILE
            if [[ -z $(grep "LogiDEFookieValidity" /var/www/pma/config.inc.php) ]]
            then
                sed -i "s/?>/\$cfg['LogiDEFookieValidity'] = '86400'\;\n?>/" /var/www/pma/config.inc.php
            fi
            cd /tmp
        else
            echo "phpmyadmin was not downloaded and was not installed" > $TEMPFILE
        fi
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
     ) | indicatorDialog "Install phpmyadmin..." ""
    textDialog "Install phpmyadmin..." "$TEMPFILE" && clear
}

function mvTestSite() {
    cp /home/${HCE_USER}/hce-node-bundle/api/python/data/ftests/test_site/* /var/www/ 2>$TEMPFILE
    chown -R www-data:www-data /var/www
    infoDialog "Set up test web site pages" "Test web site copied and file permissions set\n$(<$TEMPFILE)" 8 && clear
}

function mysqlSetup() {
    set -f
    if [[ $(pgrep mysql) ]]
    then
        MYPW=$(cat /etc/mysql/debian.cnf |grep password|uniq|awk '{ print ($3)}')
        CHECK_MYSQL_USER_LOCALHOST=$(echo "SHOW GRANTS FOR '$MYSQL_USER'@'localhost';"| mysql -udebian-sys-maint -p$MYPW 2>/dev/null && : || echo "NONE")
        CHECK_MYSQL_USER_ALL=$(echo "SHOW GRANTS FOR '$MYSQL_USER'@'%';"| mysql -udebian-sys-maint -p$MYPW 2>/dev/null && : || echo "NONE")

        if [[ $CHECK_MYSQL_USER_LOCALHOST == "NONE" ]]
        then
            echo "CREATE USER '$MYSQL_USER'@'localhost' IDENTIFIED BY '$MYSQL_PWD';"| mysql -udebian-sys-maint -p$MYPW
            echo "GRANT ALL ON *.* TO '$MYSQL_USER'@'localhost';"| mysql -udebian-sys-maint -p$MYPW
            echo "FLUSH PRIVILEGES;"| mysql -udebian-sys-maint -p$MYPW
        fi
        if [[ $CHECK_MYSQL_USER_ALL == "NONE" ]]
        then
            echo "CREATE USER '$MYSQL_USER'@'%' IDENTIFIED BY '$MYSQL_PWD';"| mysql -udebian-sys-maint -p$MYPW
            echo "GRANT ALL ON *.* TO '$MYSQL_USER'@'%';"| mysql -udebian-sys-maint -p$MYPW
            echo "FLUSH PRIVILEGES;"| mysql -udebian-sys-maint -p$MYPW
        fi
        sed -i '47d' /etc/mysql/my.cnf
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_sites.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_urls.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_sites_default.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_sites_mutexes_api.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_processor.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_contents.sql

        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_urls_deleted.sql

        infoDialog "Setup mysql support (server, client, databases, user, permissions)" "Databases successfully created" && clear
    else
        infoDialog "Setup mysql support (server, client, databases, user, permissions)" "Mysql server is not worked. User and databases not created" && clear
    fi
}

function kvStorage() {
    cp /home/${HCE_USER}/hce-node-bundle/api/python/ini/db-task_keyvalue_template.db /home/${HCE_USER}/hce-node-bundle/api/python/data/dc_dbdata/0.db 2>$TEMPFILE
    infoDialog "Creating KV-DB storage for default site" "Done\n $(<$TEMPFILE)" 8 && clear
}

function checkZmqSo() {
    #if [[ $(grep 'extension=zmq.so' /etc/php5/cli/php.ini) ]]
    if [[ $(php -m | grep zmq) ]]
    then
        echo 'Y'
    else
        exit 1
    fi
}

function checkLocale() {
    if [[ $(locale -a 2>/dev/null | grep 'en_US.utf8') ]]
    then
        echo 'OK'
    fi
    exit 1
}

function force() {
    if [[ $(whoami) != root ]]
    then
        echo "Error" "Run script with root permissions or sudo: sudo $0 \"USERNAME\""
        exit 1
    fi
    if [[ ! -f '/etc/debian_version' ]]
    then
        echo "Error" "This script is only for the Debian system! Cancelled."
        exit 1
    fi
    if [[ ! -d /home/$HCE_USER ]]
    then
        echo "Error" "Home dir (/home/${HCE_USER}) for user ${HCE_USER} not exists! Cancelled."
        exit 1
    fi
    # locales #
    echo -e '#\nLANG=en_US.UTF-8\n' >> /etc/default/locale
    sed -i 's/# en_US.UTF-8 UTF-8/en_US.UTF-8 UTF-8/' /etc/locale.gen
    locale-gen
    # dtse_php #
    apt-get update -qq
    apt-get install -y --force-yes bsdutils unzip libzmq3-dev apache2 php5 php5-dev php5-sqlite php-pear pkg-config libpgm-dev sphinxsearch make
    printf "\n" | pecl install zmq-beta
    if [[ -z $(checkZmqSo) ]]
    then
        echo -e "; configuration for php ZMQ module \n; priority=20 \nextension=zmq.so" > /etc/php5/cli/conf.d/20-zmq.ini
        service apache2 reload
    fi
    # dtse_dc #
    apt-get update -qq
    apt-get install -y bc openjdk-7-jdk ruby php5-gmp g++
    # dtse_python #
    apt-get update -qq
    apt-get install -y libpgm-dev mysql-server-5.5 python-pip python-dev python-flask python-flaskext.wtf libffi-dev libxml2-dev libxslt1-dev mysql-client libmysqlclient-dev python-mysqldb libicu-dev libgmp3-dev libtidy-dev libjpeg-dev
    pip install cement sqlalchemy Flask-SQLAlchemy scrapy gmpy pyzmq lepl requests==2.5 mysql-python pyicu newspaper goose-extractor pytidylib uritools python-magic feedparser Ghost.py pillow beautifulsoup4
    pip install pyzmq --install-option="--zmq=bundled"
    pip install -U urlnorm
    # pyzmq #
    cd $TMPDIR
    wget -q http://download.zeromq.org/zeromq-4.0.4.tar.gz
    tar -xf zeromq-4.0.4.tar.gz
    cd zeromq-4.0.4
    ./configure -q && make -s && make -s install
    ldconfig
    pip install -q pyzmq --install-option="--zmq=4.0.3"
    cd $TMPDIR
    rm -rf $TMPDIR/*
    # phpmyadmin #
    apt-get install -y php5-mysqlnd
    cd $TMPDIR
    wget -q https://sourceforge.net/projects/phpmyadmin/files/latest/download?source=files -O $TMPDIR/phpmyadmin-latest.zip
    if [[ -z $(file $TMPDIR/phpmyadmin-latest.zip | grep empty) ]]
    then
        unzip -q phpmyadmin-latest.zip
        rm -f phpmyadmin-latest.zip
        PHPMYADMINDIR=$(ls)
        if [[ -d /var/www/pma ]]
        then
            rm -rf /var/www/pma
        fi
        mkdir /var/www/pma
        mv $TMPDIR/$PHPMYADMINDIR/* /var/www/pma/
        mv $TMPDIR/$PHPMYADMINDIR/.[!.]* /var/www/pma/
        cp /var/www/pma/config.sample.inc.php /var/www/pma/config.inc.php
        if [[ -z $(grep "LogiDEFookieValidity" /var/www/pma/config.inc.php) ]]
        then
            sed -i "s/?>/\$cfg['LogiDEFookieValidity'] = '86400'\;\n?>/" /var/www/pma/config.inc.php
        fi
        cd /tmp
    else
        echo "phpmyadmin was not downloaded and was not installed"
    fi
    # mvTestSite #
    cp /home/${HCE_USER}/hce-node-bundle/api/python/data/ftests/test_site/* /var/www/
    chown -R www-data:www-data /var/www
    # mysqlSetup #
    set -f
    if [[ $(pgrep mysql) ]]
    then
        MYPW=$(cat /etc/mysql/debian.cnf |grep password|uniq|awk '{ print ($3)}')
        CHECK_MYSQL_USER_LOCALHOST=$(echo "SHOW GRANTS FOR '$MYSQL_USER'@'localhost';"| mysql -udebian-sys-maint -p$MYPW 2>/dev/null && : || echo "NONE")
        CHECK_MYSQL_USER_ALL=$(echo "SHOW GRANTS FOR '$MYSQL_USER'@'%';"| mysql -udebian-sys-maint -p$MYPW 2>/dev/null && : || echo "NONE")
        if [[ $CHECK_MYSQL_USER_LOCALHOST == "NONE" ]]
        then
            echo "CREATE USER '$MYSQL_USER'@'localhost' IDENTIFIED BY '$MYSQL_PWD';"| mysql -udebian-sys-maint -p$MYPW
            echo "GRANT ALL ON *.* TO '$MYSQL_USER'@'localhost';"| mysql -udebian-sys-maint -p$MYPW
            echo "FLUSH PRIVILEGES;"| mysql -udebian-sys-maint -p$MYPW
        fi
        if [[ $CHECK_MYSQL_USER_ALL == "NONE" ]]
        then
            echo "CREATE USER '$MYSQL_USER'@'%' IDENTIFIED BY '$MYSQL_PWD';"| mysql -udebian-sys-maint -p$MYPW
            echo "GRANT ALL ON *.* TO '$MYSQL_USER'@'%';"| mysql -udebian-sys-maint -p$MYPW
            echo "FLUSH PRIVILEGES;"| mysql -udebian-sys-maint -p$MYPW
        fi
        sed -i '47d' /etc/mysql/my.cnf
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_sites.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_urls.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_sites_default.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_sites_mutexes_api.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_processor.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_contents.sql

        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_urls_deleted.sql

        echo "Setup mysql support (server, client, databases, user, permissions)" "Databases successfully created"
    else
        echo "Setup mysql support (server, client, databases, user, permissions)" "Mysql server is not worked. User and databases not created"
    fi
    # kvStorage #
    cp /home/${HCE_USER}/hce-node-bundle/api/python/ini/db-task_keyvalue_template.db /home/${HCE_USER}/hce-node-bundle/api/python/data/dc_dbdata/0.db
    echo "Done"
    exit 0
}

# END OF FUNCTIONS #

# START #

# CHECK HCE-DEVELOPMENTS CONTAINER #
if [[ -f $HCE_DEV_FILE ]]
then
    echo "It's a container for hce-developments, template created at  $(awk -F= '{print $2}' /etc/hce-dev), all dependencies installed"
fi

for ARG in $*
do
    case $ARG in
    'force'|'-force'|'--force')
        force
        exit 0;;
    'username='*|'-username='*|'--username='*)
        HCE_USER=$(echo $ARG | awk -F\= '{print $2}');;
    'help'|'-help'|'--help')
        echo "Usage $0 [username=user] [force] [help]"
        exit 0;;
    esac
done

# CHECK DIALOG INSTALL #
if [[ -z $(which $DIALOG) ]]
then
    if [[ -f '/etc/debian_version' ]]
    then
        echo -e $RED"Dialog not installed, please install dialog using ("$BLUE"sudo apt-get install dialog"$RED") and run "$BLUE"$0"$RED" again"$DEF
        exit 1
    elif [[ -f '/etc/redhat-release' || '/etc/centos-release' ]]
    then
        echo -e $RED"Dialog not installed, please install dialog ("$BLUE"sudo yum install dialog"$RED") and run "$BLUE"$0"$RED" again"$DEF
        exit 1
    else
        echo -e $RED"Dialog not installed, please install dialog and run "$BLUE"$0"$RED" again"$DEF
        exit 1
    fi
fi

checkSudo
checkOs
checkHomeDir ${HCE_USER}
MYSQL_SERVER=$(checkInstalledPackage "mysql-server")
if [[ $MYSQL_SERVER ]]
then
    MYSQL_CHECKBOX='off'
else
    MYSQL_CHECKBOX='on'
fi
start "Select components"

for CHOISE in ${CHECK[@]}
do
    case $CHOISE in
        '"0"')
            locales;;
        '"1"')
            mysql_server;;
        '"2"')
            dtse_php;;
        '"3"')
            dtse_dc;;
        '"4"')
            dtse_python;;
        '"5"')
            pyzmq;;
        '"6"')
            phpmyadmin;;
        '"7"')
            mvTestSite;;
        '"8"')
            mysqlSetup;;
        '"9"')
            kvStorage;;
    esac
done

infoDialog "Finish" "All selected dependencies installed" && clear

exit 0
