/**
 * @file admin.hpp
 *
 * HCE project node admin handler.
 * Implements functionality for admin handler server.
 * Processes admin management requests and process them or proxy to the target handlers that are connected by zmq admin socket.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef __HCE_ADMIN_INCLUDED__
#define __HCE_ADMIN_INCLUDED__

#include <sstream>
#include <map>

#include "Handler.hpp"

namespace HCE {
  namespace handlers {

    class Admin: public HCE::handlers::Handler {
      public:
        Admin(std::string name, zmq::context_t& ctx, const std::string& clientIdentity, std::string connectionString, int64_t startedAt, unsigned char logPriority);
        ~Admin();
        void stopClients(void);

      protected:
        int initialize(void);
        void deinitialize(void);
        void* process(void);

        void handleInternalMessage(void);
        void handleExternalMessage(void);

        void processDataMessage(zmsg& msg);

        void processControlMessageInproc(zmsg& msg);
        void processDataMessageInproc(zmsg& msg);
        void registerClientInproc(const std::string& identity);

        unsigned char processAdminRequest(zmsg& msg);
        static std::vector<std::string> parseAdminRequest(const std::string& command);
        std::string processAdminCommands(std::string& command, std::string& parameters);

        //!<Get property information
        std::string getPropertyInformation(void);

        //!<Sockets
        zmq::socket_t* _adminServerSock;

        //!<Client ID
        std::string _clientIdentity;

        //!<Queue of connected clients for inproc_admin
        std::vector<ClientWorkerItem> _queueInproc;
        //!<Connected clients queue manager
        ClientsQueueManager* _clientsManagerInproc;

        //!<%Stat vars
        unsigned int _statRequestsTotal;
    };
  }
}

#endif /* __HCE_ADMIN_INCLUDED__ */
