/**
 * @file AdminClientsSerializator.hpp
 *
 * Implementation class AdminClientsSerializator for HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef ADMIN_CLIENTS_SERIALIZATOR_HPP
#define ADMIN_CLIENTS_SERIALIZATOR_HPP

#include "JsonSerializable.hpp"
#include "ClientWorkerItem.hpp"

namespace HCE
{
namespace handlers
{
//-----------------------------------------------------------------------------
class AdminClientsSerializator : public IJsonSerializable
{
  // json route message names
  const std::string PURGE_COUNTER = "purgeCounter";
  const std::string REFRESH_NOT_FOUND_COUNTER = "refreshNotFoundCounter";
  const std::string CLIENTS_COUNT = "clientsCount";
  const std::string CLIENTS = "clients";
  const std::string IDENTITY = "identity";
  const std::string COUNT = "count";
  const std::string FREQUENCY = "frequency";
  const std::string EXPIRY = "expiry";
  const std::string CPU = "cpu";
  const std::string IOWAIT = "iowait";
public:
  enum ErrorCodes
  {
    NO_ERROR=0,
    ERROR_SERIALIZE=1,
    ERROR_UNSERIALIZE=2
  };
public:
  explicit AdminClientsSerializator(std::vector<ClientWorkerItem>& clients_);
  AdminClientsSerializator(std::vector<ClientWorkerItem>& clients_, unsigned long long purgeCounter_, unsigned long long refreshNotFoundCounter_);
  ~AdminClientsSerializator(void) {}

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void clear(void);

  void setPurgeCounter(unsigned long long purgeCounter_) {purgeCounter=purgeCounter_;}
  unsigned long long getPurgeCounter(void) const {return purgeCounter;}

  void setRefreshNotFoundCounter(unsigned long long refreshNotFoundCounter_) {refreshNotFoundCounter=refreshNotFoundCounter_;}
  unsigned long long getRefreshNotFoundCounter(void) const {return refreshNotFoundCounter;}

  size_t getClientsCount(void) const {return clients.size();}
private:
  std::vector<ClientWorkerItem>& clients;
  unsigned long long purgeCounter;
  unsigned long long refreshNotFoundCounter;

  typedef IJsonSerializable inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace handlers
} // end namespace HCE 

#endif // ADMIN_CLIENTS_SERIALIZATOR_HPP
