#include <Poco/JSON/Object.h>
#include <Poco/JSON/Array.h>
#include <Poco/JSON/JSON.h>
#include <Poco/JSON/Stringifier.h>
#include <Poco/JSON/Parser.h>
#include <Poco/JSON/JSONException.h>
#include <Poco/Dynamic/Var.h>

#include "AdminCommandParameters.hpp"
#include "HandlerTypes.hpp"

namespace HCE
{
namespace handlers
{
//-----------------------------------------------------------------------------
AdminCommandParameters::AdminCommandParameters(const std::string& str)
: inherited(), parameters(""), realtime(PROPERTY_REALTIME_DEFAULT), _isJson(false)
{
  if (!str.empty())
  {
    if (isJsonString(str))
    {
      unserialize(str);
    }
    else
    {
      parameters = str;
    }
  }
}
//-----------------------------------------------------------------------------
bool AdminCommandParameters::serialize(std::string& json)
{
  resetError();
  try
  {
    Poco::JSON::Object::Ptr pObj = new Poco::JSON::Object();
    if (pObj.isNull())
      throw Poco::Exception("Object instance was not created");

    pObj->set(PARAMETERS, parameters);
    pObj->set(REALTIME, realtime);

    std::stringstream ostr;
    Poco::JSON::Stringifier::stringify(pObj, ostr);
    json = ostr.str();

    _isError = false;
  }
  catch(Poco::Exception& e)
  {
    _isError = true;
    errorMsg = e.displayText();
    errorCode = ERROR_UNSERIALIZE;
  }
  catch(std::exception& e)
  {
    _isError = true;
    errorMsg = e.what();
    errorCode = ERROR_SERIALIZE;
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
bool AdminCommandParameters::unserialize(const std::string& json)
{
  clear();
  try
  {
    Poco::JSON::Parser parser;
    Poco::Dynamic::Var res = parser.parse(json);

    Poco::JSON::Object::Ptr pObj = res.extract<Poco::JSON::Object::Ptr>();

    if (!pObj.isNull())
    {
      Poco::Dynamic::Var tmp = pObj->get(PARAMETERS);
      if (tmp.isString())
        parameters = tmp.convert<std::string>();

      tmp = pObj->get(REALTIME);
      realtime = convertVarToNumeric<unsigned int>(tmp, 0);
    }

    _isError = false;
  }
  catch(Poco::JSON::JSONException& e)
  {
    _isError = true;
    errorMsg = e.displayText();
    errorCode = ERROR_UNSERIALIZE;
  }
  catch(Poco::Exception& e)
  {
    _isError = true;
    errorMsg = e.displayText();
    errorCode = ERROR_UNSERIALIZE;
  }
  catch(std::exception& e)
  {
    _isError = true;
    errorMsg = e.what();
    errorCode = ERROR_UNSERIALIZE;
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
void AdminCommandParameters::clear(void)
{
  resetError();
  parameters.clear();
  realtime = PROPERTY_REALTIME_DEFAULT;
}
//-----------------------------------------------------------------------------
bool AdminCommandParameters::isRealtime(void) const
{
  return (realtime);
}
//-----------------------------------------------------------------------------
bool AdminCommandParameters::isJsonString(const std::string& str)
{
  _isJson = (str.front()=='{' && str.back()=='}');
  return _isJson;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace handlers
} // end namespace HCE 
