/**
 * @file AdminCommandParameters.hpp
 *
 * Implementation class AdminCommandParameters for HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef ADMIN_COMMAND_PARAMETERS_HPP
#define ADMIN_COMMAND_PARAMETERS_HPP

#include "JsonSerializable.hpp"

namespace HCE
{
namespace handlers
{
//-----------------------------------------------------------------------------
class AdminCommandParameters : public IJsonSerializable
{
  // json route message names
  const std::string PARAMETERS = "parameters";
  const std::string REALTIME = "realtime";
public:
  enum ErrorCodes
  {
    NO_ERROR=0,
    ERROR_SERIALIZE=1,
    ERROR_UNSERIALIZE=2
  };
public:
  explicit AdminCommandParameters(const std::string& str="");
  ~AdminCommandParameters(void) {}

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void clear(void);

  void setParameters(const std::string& parameters_) {parameters=parameters_;}
  std::string getParameters(void) const {return parameters;}

  void setRealtime(unsigned int realtime_) {realtime=realtime_;}
  unsigned int getRealtime(void) const {return realtime;}

  bool isRealtime(void) const;
  bool isJsonString(void) const {return _isJson;}
private:
  bool isJsonString(const std::string& str);
private:
  std::string parameters;
  unsigned int realtime;
  bool _isJson;

  typedef IJsonSerializable inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace handlers
} // end namespace HCE 

#endif // ADMIN_COMMAND_PARAMETERS_HPP
