/**
 * @file ClientWorkerItem.hpp
 *
 * Implementation class ClientWorkerItem for HCE cluster node.
 * This defines one active worker in our worker queue
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef CLIENT_WORKER_ITEM_HPP
#define CLIENT_WORKER_ITEM_HPP

#include <iostream>
#include <vector>
#include <deque>

#include "Dictionary.hpp"

namespace HCE
{
namespace handlers
{
//-----------------------------------------------------------------------------
class TimeoutShiftCounter
{
public:
  TimeoutShiftCounter(void);

  TimeoutShiftCounter& operator=(const TimeoutShiftCounter& rhs);
  TimeoutShiftCounter& operator=(TimeoutShiftCounter&& rhs);

  void reset(void);
  void responceTimeStamp(size_t rtimestamp);
  void recount(unsigned int timeout);

  size_t getRcounter(void) const {return rcounter;}
  size_t getRdiff(void) const {return rdiff;}
  unsigned int getShift(void) const {return shift;}
 private:
   size_t rcounter;
   size_t rdiff;
   unsigned int shift;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
typedef HCE::handlers::Dictionary <std::string, double> ResourceUsageData;
typedef std::deque<ResourceUsageData> ResourceUsageCollection;
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
typedef struct ClientWorkerItem
{
  ClientWorkerItem(void);
  explicit ClientWorkerItem(const std::string& identity_, int64_t expiry_=0, unsigned long long frequency_=0, unsigned long long count_=0);
  ClientWorkerItem(const ClientWorkerItem& rhs);
  ClientWorkerItem(ClientWorkerItem&& rhs);

  ClientWorkerItem& operator=(const ClientWorkerItem& rhs);
  ClientWorkerItem& operator=(ClientWorkerItem&& rhs);

  void clear(void);

  static bool sortByFrequencyPredicate(const ClientWorkerItem& d1, const ClientWorkerItem& d2);

  std::string identity;                 //!< Identity
  int64_t expiry;                       //!< Expiration time
  unsigned long long frequency;         //!< Frequency of usage, used to balance load round-robin way, but reset after some refresh not found or add new item actions
  unsigned long long count;             //!< Counter of usage, differs from frequency that is reset when new worker added or not found during the refresh action
  std::vector<ClientWorkerItem> route;  //!< List node names use for routing (key is identity)
  ResourceUsageCollection resources;    //!< Resource usage collection
  size_t rtimestamp;                    //!< Request timestamp
  TimeoutShiftCounter shiftCounter;     //!< Timeout shift counter
  std::string property;                 //!< Property data
  double cpu;                           //!< Cpu usage value
  double iowait;                        //!< Iowait value

} ClientWorkerItem;
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace handlers
} // end namespace HCE 

#endif // CLIENT_WORKER_ITEM_HPP
