/**
 * @file DataClientData.hpp
 *
 * HCE project node dataClientData handler.
 * Implements functionality for data client handler server.
 * Implements data server functionality for data client's requests handling and HCE project data node construction.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef __HCE_DATA_CLIENT_DATA_INCLUDED__
#define __HCE_DATA_CLIENT_DATA_INCLUDED__

#include <iostream>
#include <sstream>

#include "Handler.hpp"
#include "DataNodeOptions.hpp"

//Node ini
#include <Poco/Util/IniFileConfiguration.h>
#include "Poco/String.h"

namespace HCE {
  namespace handlers {

    class DataClientData: public HCE::handlers::Handler, public HCE::handlers::DataNodeOptions {
      public:
        DataClientData(std::string name, zmq::context_t& ctx, const std::string& clientIdentity, std::string connectionString, unsigned char dataClientMode, int64_t startedAt,
            unsigned char logPriority, const std::string& iniFile="");
        ~DataClientData();

      protected:
        int initialize(void);
        void deinitialize(void);
        void* process(void);

        void rebuildDataClientSocket(void);
        void handleExternalMessage(void);
        void handleInternalMessage(void);
        void heartbit(void);
        void sendHeartbeat(void);
        void logPeriodicStatistics(void);
        void externalConnectionRefresh(void);
        std::string processAdminCommands(std::string& command, std::string& parameters);

        //!<Get resource usage message
        std::string getResourceUsageMessage(void) const;

        //!<Get property information
        std::string getPropertyInformation(void);

        //!<Sockets
        zmq::socket_t* _inprocDataSock;
        zmq::socket_t* _dataClientSock;
        bool _dataClientSockConnected = false;

        //!<Client ID
        std::string _clientIdentity;

        //!<Data client mode
        unsigned char _dataClientSockMode;

        //!<Cluster node ini file name.
        std::string _iniFile;

        //!<Timeout shift counter
        TimeoutShiftCounter _shiftCounter;

        //!<Stat vars
        unsigned int _statRequests;
        unsigned int _statResponses;
        unsigned int _statRequestsProcessed;
        unsigned int _statResponsesProcessed;
        unsigned int _statRequestsProcessedSize;
        unsigned int _statResponsesProcessedSize;
        unsigned int _statRequestsSize;
        unsigned int _statResponsesSize;
        float _statQPS;
        float _statRPS;
        float _statRequestsBandwidth;
        float _statResponsesBandwidth;

        //!<Send out periodical stats at regular intervals
        int64_t _pstatsAt;

        //!<Time when heartbeat message for external connection need to be sent next time
        int64_t _heartbeatAt;
        //!<Last time when external connectionhave received heartbit message
        int64_t _heartbeatedAt;
    };

  }
}

#endif /* __HCE_DATA_CLIENT_DATA_INCLUDED__ */
