/**
 * @file DataClientProxy.hpp
 *
 * HCE project node DataClientProxy handler.
 * Implements functionality for data client handler server.
 * Implements proxy server functionality for proxy client's requests handling and HCE project proxy type (shard manager) node construction.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef __HCE_DATA_CLIENT_PROXY_INCLUDED__
#define __HCE_DATA_CLIENT_PROXY_INCLUDED__

#include <iostream>
#include <sstream>

#include "Handler.hpp"
#include "DataNodeOptions.hpp"

namespace HCE {
  namespace handlers {

    class DataClientProxy: public HCE::handlers::Handler, public HCE::handlers::DataNodeOptions {
      public:
        DataClientProxy(std::string name, zmq::context_t& ctx, const std::string& clientIdentity, std::string connectionString, int64_t startedAt,
            unsigned char logPriority, const std::string& iniFile="");
        virtual ~DataClientProxy();

      protected:
        int initialize(void);
        void deinitialize(void);
        void* process(void);

        void handleExternalMessage(void);
        void handleInternalMessage(void);
        void handleRouteMessage(void);
        void processControlMessage(zmsg& msg);
        void processDataMessage(zmsg& msg);

        std::string processAdminCommands(std::string& command, std::string& parameters);
        void rebuildDataClientSocket(void);
        void heartbit(void);
        void sendHeartbeat(void);
        void externalConnectionRefresh(void);
        //!<Get property information
        std::string getPropertyInformation(void);

        //!<ZMQ sockets
        zmq::socket_t* _inprocProxySock;
        zmq::socket_t* _inprocReducerOutSock;
        zmq::socket_t* _dataClientSock;
        bool _dataClientSockConnected = false;

        //!<Client ID
        std::string _clientIdentity;

        //!<Cluster node ini file name.
        std::string _iniFile;

        //!<Timeout shift counter
        TimeoutShiftCounter _shiftCounter;

        //!<Queue of data about connected clients use for make heartbeat message
        std::vector<ClientWorkerItem> _clientsQueue;

        //!<Heartbeat at time mark
        int64_t _heartbeatAt;
        //!<Last time when external connectionhave received heartbit message
        int64_t _heartbeatedAt;

        //!<Stat vars
        unsigned int _statRequests;
        unsigned int _statResponses;
    };

  }
}

#endif /* __HCE_DATA_CLIENT_PROXY_INCLUDED__ */
