#include <Poco/AutoPtr.h>
#include <Poco/Configurable.h>
#include <Poco/Util/IniFileConfiguration.h>
#include <Poco/String.h>

#include "HandlerTypes.hpp"
#include "DataNodeOptions.hpp"

namespace HCE
{
namespace handlers
{
//-----------------------------------------------------------------------------
DataNodeOptions::DataNodeOptions(void)
: heartbeatDelay(HEARTBEAT_DELAY), heartbeatTimeout(HEARTBEAT_TIMEOUT), heartbeatMode(HEARBEAT_MODE_DEFAULT)
{
}
//-----------------------------------------------------------------------------
void DataNodeOptions::initNode(const std::string& iniFile, LoggerStream& logger)
{
  if (!iniFile.empty())
  {
    logger.log(NODE_LOG_MODE_INFORMATION) << "load node settings from '" << iniFile << "' file" << flush;

    unsigned int hb_delay = HEARTBEAT_DELAY/1000;
    unsigned int hb_timeout = HEARTBEAT_TIMEOUT/1000;
    unsigned int hb_mode = HEARBEAT_MODE_DEFAULT;

    //!<Process Node's ini file
    try
    {
      Poco::AutoPtr<Poco::Util::IniFileConfiguration> pConf(new Poco::Util::IniFileConfiguration(iniFile));
      try
      {
        hb_delay = pConf->getInt("node.hb_delay");
        hb_timeout = pConf->getInt("node.hb_timeout");
        hb_mode = pConf->getInt("node.hb_mode");
      }
      catch(Poco::SyntaxException& e)
      {
        //!<Log some error of config file processing
        logger.log(NODE_LOG_MODE_ERROR) << "nodeInit() - data node options ini file processing error: '" << e.displayText() << "'" << flush;
      }
    }
    catch(Poco::Exception& e)
    {
      //!<Log some error of config file processing
      logger.log(NODE_LOG_MODE_ERROR) << "nodeInit() - data node options ini file access error: '" << e.displayText() << "'" << flush;
    }
    //!<Apply parameters
    setHeartbeatDelay(hb_delay*1000);
    setHeartbeatTimeout(hb_timeout*1000);
    setHeartbeatMode(hb_mode);

    logger.log(NODE_LOG_MODE_INFORMATION) << "hb_delay = " << hb_delay << flush;
    logger.log(NODE_LOG_MODE_INFORMATION) << "hb_timeout = " << hb_timeout << flush;
    logger.log(NODE_LOG_MODE_INFORMATION) << "hb_mode = " << hb_mode << flush;
  }
}
//-----------------------------------------------------------------------------
void DataNodeOptions::setHeartbeatDelay(unsigned int heartbeatDelay_)
{
  if (heartbeatDelay_ >= MIN_ALLOWED_HEARTBEAT_DELAY)
    heartbeatDelay=heartbeatDelay_;
}
//-----------------------------------------------------------------------------
void DataNodeOptions::setHeartbeatTimeout(unsigned int heartbeatTimeout_)
{
  if (heartbeatTimeout_ >= MIN_ALLOWED_HEARTBEAT_TIMEOUT)
    heartbeatTimeout=heartbeatTimeout_;
}
//-----------------------------------------------------------------------------
void DataNodeOptions::setHeartbeatMode(unsigned int heartbeatMode_)
{
  if (heartbeatMode_ <= HEARBEAT_MODE_MAX_NUMBER && heartbeatMode_ >= HEARBEAT_MODE_DEFAULT)
    heartbeatMode=heartbeatMode_;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE 
