/**
 * @file LoggerConfigLoader.hpp
 *
 * Implementation class LoggerConfigLoader need for initialization Poco::Logger as ROOT channel in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef LOGGER_CONFIG_LOADER_HPP
#define LOGGER_CONFIG_LOADER_HPP

#include <iostream>
#include <Poco/AutoPtr.h>
#include <Poco/Configurable.h>

namespace HCE
{
namespace handlers
{
//-----------------------------------------------------------------------------
class LoggerConfigLoader
{
  // list names of constants using for initialization Poco::Logger
  static const std::string sectionPatternFormatter;
  static const std::string sectionFileChannel;
  static const std::string sectionLogger;
  static const std::string delimiter;
public:
  LoggerConfigLoader(const std::string& nodeName, const Poco::Util::IniFileConfiguration::Keys& loggers, const std::string& configFileName="");
  explicit LoggerConfigLoader(const std::string& nodeName, const std::string& configFileName="");
  virtual ~LoggerConfigLoader(void);

  bool loadConfig(const std::string& configFileName);
  static void defaultInitialization(void);

  std::string getErrorMsg(void) const {return errorMsg;}
  bool isError(void) const {return _isError;}
  static unsigned int getDebugLevel(void);

protected:
  void loadSection(Poco::Util::IniFileConfiguration& iniFileConfiguration,
      const std::string& sectionName,
      Poco::Configurable& configurable) throw (Poco::Exception);

  void loggerInitialization(Poco::Util::IniFileConfiguration& iniFileConfiguration,
      const std::string& loggerName) throw (Poco::Exception);

  static std::string getLoggerSectionName(const std::string& sectionName,
      const std::string& loggerName);

  static void getLoggers(Poco::Util::IniFileConfiguration& iniFileConfiguration,
      Poco::Util::IniFileConfiguration::Keys& loggerNames);

  void checkExistSection(Poco::Util::IniFileConfiguration& iniFileConfiguration,
      const Poco::Util::IniFileConfiguration::Keys& loggerNames) throw (Poco::Exception);

  static void checkExistSection(const Poco::Util::IniFileConfiguration::Keys& sections,
      const std::string& sectionName) throw (Poco::Exception);

protected:
  std::string nodeName;
  std::string errorMsg;
  bool _isError;
  Poco::Util::IniFileConfiguration::Keys loggers;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace handlers
} // end namespace HCE 

#endif // LOGGER_CONFIG_LOADER_HPP
