/*
 * ResourceUsageLimitsChecker.cpp
 *
 *  Created on: Mar 17, 2015
 *      Author: alexander
 */

#include "ResourceUsageLimitsChecker.hpp"
#include "JsonSerializable.hpp"

namespace HCE
{
namespace handlers
{
//-----------------------------------------------------------------------------
ResourceUsageLimitsChecker::ResourceUsageLimitsChecker(ResourceUsageLimitsData& limitsData_)
: limitsData(limitsData_)
{
}
//-----------------------------------------------------------------------------
void ResourceUsageLimitsChecker::executeCheck(const CalculatorNodeResourceData& resourceData) throw (Poco::Exception)
{
  HCE::TrimFormatter <std::string> trim;
  for (ResourceUsageLimitsData::ConstIterator iter=limitsData.begin();iter!=limitsData.end();++iter)
  {
    const double resourceUsage = resourceData.getItem((*iter).first);
    if ((*iter).second && (*iter).second < resourceUsage)
    {
      std::stringstream errMsg;
      errMsg << "Node '" << resourceData.getIdentity() << "' has exceeded max allowed "
          << resourcesUsageMessage[(*iter).first] << " resource usage. Limits: " << trim((*iter).second) << " %. Current usage: " << trim(resourceUsage) << " %.";
      throw Poco::Exception(errMsg.str());
    }
  }
}
//-----------------------------------------------------------------------------
bool ResourceUsageLimitsChecker::safeCheck(const CalculatorNodeResourceData& resourceData, std::string& errorMsg)
{
  bool result = false;
  errorMsg.clear();
  try
  {
    executeCheck(resourceData);
    result = true;
  }
  catch(Poco::Exception& e)
  {
    errorMsg = e.message();
  }
  return result;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
}// end namespace handlers 
} // end namespace HCE 
