/**
 * @file ResourceUsageLimitsChecker.hpp
 *
 * Implementation class ResourceUsageLimitsChecker of HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef RESOURCE_USAGE_LIMITS_CHECKER_HPP
#define RESOURCE_USAGE_LIMITS_CHECKER_HPP

#include "CalculatorNodeExecution.hpp"

namespace HCE
{
namespace handlers
{
//-----------------------------------------------------------------------------
typedef Dictionary <std::string, double> ResourceUsageLimitsData;
//-----------------------------------------------------------------------------
class ResourceUsageLimitsChecker
{
  std::map<std::string, std::string> resourcesUsageMessage={
      {"1", "CPU usage"},
      {"2", "iowait load"},
      {"3", "virtual RAM"},
      {"4", "resource RAM"},
      {"5", "hard disk"},
      {"6", "processes"},
      {"7", "threads"},
      {"8", "load average"}
  };
public:
  ResourceUsageLimitsChecker(ResourceUsageLimitsData& limitsData_);
  ~ResourceUsageLimitsChecker(void) {}

  void executeCheck(const CalculatorNodeResourceData& resourceData) throw (Poco::Exception);
  bool safeCheck(const CalculatorNodeResourceData& resourceData, std::string& errorMsg);
private:
  ResourceUsageLimitsData& limitsData;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace handlers
} // end namespace HCE 

#endif // RESOURCE_USAGE_LIMITS_CHECKER_HPP
