/**
 * @file ResourceUsageManager.hpp
 *
 * Implementation class ResourceUsageManager of HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef RESOURCE_USAGE_COLLECTION_HPP
#define RESOURCE_USAGE_COLLECTION_HPP

#include "ClientWorkerItem.hpp"
#include "LoggerStream.hpp"

namespace HCE
{
namespace handlers
{
//-----------------------------------------------------------------------------
class ResourceUsageManager
{
  enum {MAX_SIZE=10};

  struct Weights
  {
  	Weights(void) :prev(0.0), last(0.0), diff(0.0), stepIndex(0), listWeights() {}
  	double prev;
  	double last;
  	double diff;
  	size_t stepIndex;
  	std::vector<double> listWeights;
  };
public:
  ResourceUsageManager(HCE::LoggerStream& logger_);
  ~ResourceUsageManager(void) {}

  void add(const std::string& identity, ResourceUsageCollection& collection, const ResourceUsageData& resourceUsageData, size_t maxSize=MAX_SIZE);
  void add(const std::string& identity, ResourceUsageCollection& collection, const std::string& resources, size_t maxSize=MAX_SIZE);

  ResourceUsageData calculateAverage(const std::string& identity, ResourceUsageCollection& collection);

  bool isAllowedRange(const std::string& identity, double maxAllowedRange);

  void setMaxSize(size_t maxSize_);
  size_t getMaxSize(void) const {return maxSize;}

  size_t nextStep(const std::string& identity);
  size_t prevStep(const std::string& identity);

  void setLastWeight(const std::string& identity, double weight);
  double getLastWeight(const std::string& identity);

  void setListWeights(const std::string& identity, const std::vector<double>& listWeights);
  std::vector<double> getListWeights(const std::string& identity);

protected:
  ResourceUsageData averageValue(ResourceUsageCollection& collection, size_t count);
private:
  //!<Max allowed size of resource usage collection
  size_t maxSize;
  //!<Steps allowed intervals for culculation
  std::vector<size_t> steps;
  //!<Register weights of nodes
  std::map<std::string, Weights> weights; // key is identity
  //Logger stream use now as log buffer
  HCE::LoggerStream& logger;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace handlers
} // end namespace HCE 

#endif // RESOURCE_USAGE_COLLECTION_HPP
