/**
 * @nodemcpp.hpp
 *
 * HCE project node model application.
 * Implements complete functional model of cluster node application.
 * Supports four basic runtime modes: router, proxy shard, data shard and data node.
 * Uses zmq PPP-based protocol and connection schema, heartbit algorithm, admin management handler and so on.
 * This is an atomic brick for HCE project architecture structure schema construction.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013-2015 HCE project, IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef __HCE_NODEM_INCLUDED__
#define __HCE_NODEM_INCLUDED__

#include <sys/utsname.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>

#include <string.h>
#include <iostream>
#include <map>

#include <Poco/Util/OptionSet.h>
#include <Poco/Util/OptionProcessor.h>
#include <Poco/Path.h>

#include "Admin.hpp"
#include "DataClientProxy.hpp"
#include "RouterServerProxy.hpp"
#include "DataReducerProxy.hpp"
#include "DataClientData.hpp"
#include "DataServerProxy.hpp"
#include "DataProcessorData.hpp"
#include "LoggerConfigLoader.hpp"
#include "LoggerStream.hpp"
#include "ServiceMessages.hpp"

//!<hce-node application version string
const std::string APP_VERSION = "1.4.2-stable, build " + std::string(__DATE__) + " " + std::string(__TIME__);

//!<CLI arguments names definitions
const std::string NODE_CLI_NAME = "name";
const std::string NODE_CLI_ADMIN = "admin";
const std::string NODE_CLI_CLIENT = "client";
const std::string NODE_CLI_SERVER = "server";
const std::string NODE_CLI_DATAMODE = "mode";
const std::string NODE_CLI_ROLE = "role";
const std::string NODE_CLI_INI_FILE = "ini";
const std::string NODE_CLI_LOG_MODE = "log";
const std::string NODE_CLI_HELP = "help";
const std::string NODE_CLI_VERSION = "ver";
const std::string NODE_CLI_DEMONIZE = "demonize";

//!<CLI argument role values names definitions
const std::string NODE_CLI_ROLE_ROUTER = "router";
const std::string NODE_CLI_ROLE_SMANAGER = "smanager";
const std::string NODE_CLI_ROLE_RMANAGER = "rmanager";
const std::string NODE_CLI_ROLE_RMANAGER_ROUND_ROBIN = "rmanager-round-robin";
const std::string NODE_CLI_ROLE_RMANAGER_RND = "rmanager-rnd";
const std::string NODE_CLI_ROLE_RMANAGER_RESOURCES_USAGE = "rmanager-resources-usage";
const std::string NODE_CLI_ROLE_REPLICA = "replica";

//!<HCE-node application exist logger names
const std::string HCE_MAIN_LOGGER = Poco::Logger::ROOT;
const std::string DRCE_TASKS_LOGGER = "DRCEFO-Tasks";

#endif /* __HCE_NODEM_INCLUDED__ */
