/**
 * @file DRCEAvgCountsConverter.hpp
 *
 * Implementation class AvgCountsConverter of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_AVG_COUNTS_CONVERTER_HPP
#define DRCE_AVG_COUNTS_CONVERTER_HPP

#include <map>
#include <Poco/Dynamic/Var.h>

#include "DRCEAvgCounts.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class AvgCountsConverter
{
public:
  typedef std::map<std::string, Poco::Dynamic::Var> Dictionary;

  AvgCountsConverter(void);
  explicit AvgCountsConverter(const AvgCounts& rhs);
  explicit AvgCountsConverter(const Dictionary& rhs);

  AvgCounts toObject(void) const {return avgCounts;}
  Dictionary toDictionary(void);

  void fromObject(const AvgCounts& rhs);
  void fromDictionary(const Dictionary& rhs);
private:
  static void toDictionary(const std::string& key, const size_t& value, Dictionary& dic);
  static void fromDictionary(const std::string& key, size_t& value, const Dictionary& dic);
private:
  AvgCounts avgCounts;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE 

#endif // DRCE_AVG_COUNTS_CONVERTER_HPP
