/**
 * @file DRCECommonList.hpp
 *
 * Implementation class DRCECommonList of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_COMMON_LIST_HPP
#define DRCE_COMMON_LIST_HPP

#include <iostream>
#include <vector>

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
template <typename _Item>
class DRCECommonList
{
public:
  DRCECommonList(void) : items() {}
  DRCECommonList(const DRCECommonList& rhs);
  DRCECommonList(DRCECommonList&& rhs);
  virtual ~DRCECommonList(void) {}

  DRCECommonList& operator=(const DRCECommonList& rhs);
  DRCECommonList& operator=(DRCECommonList&& rhs);

  void addItem(const _Item& item);
  void addItem(_Item&& item);

  const std::vector<_Item>& getItems(void) const {return items;}

  size_t getItemsCount(void) const;
  const _Item& getItem(size_t index) const;
  void setItem(size_t index, const _Item& item);
  void setItem(size_t index, _Item&& item);

  void clear(void);
protected:
  std::vector<_Item> items;
};
//-----------------------------------------------------------------------------
template <class _Item> DRCECommonList<_Item>::DRCECommonList(const DRCECommonList& rhs)
: items()
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
template <class _Item> DRCECommonList<_Item>::DRCECommonList(DRCECommonList&& rhs)
: items()
{
  (*this) = std::forward<DRCECommonList>(rhs);
}
//-----------------------------------------------------------------------------
template <class _Item> DRCECommonList<_Item>& DRCECommonList<_Item>::operator=(const DRCECommonList& rhs)
{
  if (this!=&rhs)
  {
    items = rhs.getItems();
  }
  return *this;
}
//-----------------------------------------------------------------------------
template <class _Item> DRCECommonList<_Item>& DRCECommonList<_Item>::operator=(DRCECommonList&& rhs)
{
  if (this!=&rhs)
  {
    items = std::move(rhs.getItems());
  }
  return *this;
}
//-----------------------------------------------------------------------------
template <class _Item> void DRCECommonList<_Item>::addItem(const _Item& item)
{
  items.push_back(item);
}
//-----------------------------------------------------------------------------
template <class _Item> void DRCECommonList<_Item>::addItem(_Item&& item)
{
  items.push_back(std::move(std::forward<_Item>(item)));
}
//-----------------------------------------------------------------------------
template <class _Item> size_t DRCECommonList<_Item>::getItemsCount(void) const
{
  return items.size();
}
//-----------------------------------------------------------------------------
template <class _Item> const _Item& DRCECommonList<_Item>::getItem(size_t index) const
{
  return items[index];
}
//-----------------------------------------------------------------------------
template <class _Item> void DRCECommonList<_Item>::setItem(size_t index, const _Item& item)
{
  if (items.size() > index)
    items[index] = item;
}
//-----------------------------------------------------------------------------
template <class _Item> void DRCECommonList<_Item>::setItem(size_t index, _Item&& item)
{
  if (items.size() > index)
    items[index] = std::forward<_Item>(item);
}
//-----------------------------------------------------------------------------
template <class _Item> void DRCECommonList<_Item>::clear(void)
{
  items.clear();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_COMMON_LIST_HPP
