#include "DRCEResultDataSerializator.hpp"
#include "DRCEResultData.hpp"
#include "DRCEError.hpp"
#include "DRCEDefaultJSON.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
DRCEDefaultJSON::DRCEDefaultJSON(unsigned int dataItemsCount, unsigned int filesCount)
: json(""), errorMsg(""), errorCode(NO_ERROR), _isError(false), jsonErrorMessage(""), jsonErrorCode(NO_ERROR),
  jsonNodeHost(""), jsonNodeName(""), jsonNodePort(""), jsonState(0), jsonRequestType(0)
{
  makeJSON(dataItemsCount, filesCount);
}
//-----------------------------------------------------------------------------
bool DRCEDefaultJSON::fillResultData(DRCEResultData& resultData, unsigned int dataItemsCount, unsigned int filesCount)
{
  try
  {
    if (dataItemsCount == 0)
    {
      DRCEResultDataItem resultDataItem;
      resultDataItem.setStdoutStream("It's default json");
      resultDataItem.addFileItem("", "", 0);
      resultDataItem.setErrorCode(jsonErrorCode);
      resultDataItem.setErrorMessage(jsonErrorMessage);
      resultDataItem.setNodeHost(jsonNodeHost);
      resultDataItem.setNodeName(jsonNodeName);
      resultDataItem.setNodePort(jsonNodePort);
      resultDataItem.setState(jsonState);
      resultDataItem.setRequestType(static_cast<DRCETaskRequest::RequestType>(jsonRequestType));
      resultData.addDataItem(std::forward<DRCEResultDataItem>(resultDataItem));
    }
    else
    {
      for (unsigned int i=1;i<=dataItemsCount;++i)
      {
        DRCEResultDataItem resultDataItem;
        resultDataItem.setNodeName("node_"+std::to_string(i));
        resultDataItem.setNodeHost("host_"+std::to_string(i));
        resultDataItem.setNodePort("host_"+std::to_string(i));
        resultDataItem.setRequestId(i);
        resultDataItem.setRequestType(DRCETaskRequest::RequestType::rtGetTaskData);
        resultDataItem.setState(static_cast<unsigned int>(DRCETaskRequest::TaskState::IN_PROGRESS));
        resultDataItem.setStdoutStream("stdout_stream_"+std::to_string(i));
        resultDataItem.setStderrStream("stderr_stream_"+std::to_string(i));
        resultDataItem.setTime(i);
        resultDataItem.setErrorCode(jsonErrorCode);
        resultDataItem.setErrorMessage(jsonErrorMessage);

        for (size_t k=0;k<filesCount;++k)
          resultDataItem.addFileItem("file_"+std::to_string(i), "data_"+std::to_string(i),
              FileItem::CREATE_BEFORE_EXEC_AND_WRITE_DATA|FileItem::READ_AFTER_EXEC_AND_RETURN_RESULT|FileItem::DELETE_AFTER_EXEC);

        resultData.addDataItem(std::forward<DRCEResultDataItem>(resultDataItem));
      }
    }
    _isError = false;
  }
  catch(std::exception& e)
  {
    _isError = true;
    errorMsg = e.what();
    errorCode = ERROR_MAKE_DEFAULT_JSON;
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
bool DRCEDefaultJSON::makeJSON(unsigned int dataItemsCount, unsigned int filesCount)
{
  json.clear();
  errorMsg.clear();
  errorCode = NO_ERROR;
  _isError = false;
  DRCEResultData resultData;
  DRCEResultDataSerializator resultSerializator(resultData);

  _isError = !fillResultData(resultData, dataItemsCount, filesCount);

  if (!_isError)
  {
    if (!resultSerializator.serialize(json))
    {
      _isError = resultSerializator.isError();
      errorMsg = resultSerializator.getErrorMsg();
      errorCode = resultSerializator.getErrorCode();
    }
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const DRCEDefaultJSON& defaultJson)
{
  return os << defaultJson.getJSON();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE
