/**
 * @file DRCEDefaultJSON.hpp
 *
 * Implementation class DRCEDefaultJSON of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_DEFAULT_JSON_HPP
#define DRCE_DEFAULT_JSON_HPP

#include <iostream>

namespace HCE
{
namespace drce
{
class DRCEResultData;
//-----------------------------------------------------------------------------
class DRCEDefaultJSON
{
public:
  explicit DRCEDefaultJSON(unsigned int dataItemsCount=0, unsigned int filesCount=0);
  ~DRCEDefaultJSON() {}

  bool makeJSON(unsigned int dataItemsCount=0, unsigned int filesCount=0);

  void setJsonErrorMessage(const std::string& jsonErrorMessage_) {jsonErrorMessage=jsonErrorMessage_;}
  std::string getJsonErrorMessage(void) const {return jsonErrorMessage;}

  void setJsonErrorCode(unsigned int jsonErrorCode_) {jsonErrorCode=jsonErrorCode_;}
  unsigned int getJsonErrorCode(void) const {return jsonErrorCode;}

  void setJsonNodeHost(const std::string& jsonNodeHost_) {jsonNodeHost=jsonNodeHost_;}
  std::string getJsonNodeHost(void) const {return jsonNodeHost;}

  void setJsonNodeName(const std::string& jsonNodeName_) {jsonNodeName=jsonNodeName_;}
  std::string getJsonNodeName(void) const {return jsonNodeName;}

  void setJsonNodePort(const std::string& jsonNodePort_) {jsonNodePort=jsonNodePort_;}
  std::string getJsonNodePort(void) const {return jsonNodePort;}

  void setJsonState(unsigned int jsonState_) {jsonState=jsonState_;}
  unsigned int getJsonState(void) const {return jsonState;}

  void setJsonRequestType(unsigned int jsonRequestType_) {jsonRequestType=jsonRequestType_;}
  unsigned int getJsonRequestType(void) const {return jsonRequestType;}

  const std::string& getJSON(void) const {return json;}
  const std::string& getErrorMsg(void) const {return errorMsg;}
  unsigned int getErrorCode(void) const {return errorCode;}
  bool isError(void) const {return _isError;}
private:
  bool fillResultData(DRCEResultData& resultData, unsigned int dataItemsCount, unsigned int filesCount);
private:
  std::string json;
  std::string errorMsg;
  unsigned int errorCode;
  bool _isError;

  std::string jsonErrorMessage;
  unsigned int jsonErrorCode;
  std::string jsonNodeHost;
  std::string jsonNodeName;
  std::string jsonNodePort;
  unsigned int jsonState;
  unsigned int jsonRequestType;

  friend std::ostream& operator<<(std::ostream& os, const DRCEDefaultJSON& defaultJson);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE


#endif // DRCE_DEFAULT_JSON_HPP
