/**
 * @file DRCEFileBuilder.hpp
 *
 * Implementation class DRCEFileBuilder of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_FILE_BUILDER_HPP
#define DRCE_FILE_BUILDER_HPP

#include "CustomMessage.hpp"
#include "DRCEResultData.hpp"
#include "DRCEInputJsonMessage.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class FileBuilder
{
public:
  FileBuilder(CustomMessage& message_, const std::string& dir_);
  virtual ~FileBuilder(void) {}

  void setDir(const std::string& dir_) {dir=dir_;}
  std::string getDir(void) const {return dir;}

protected:
  CustomMessage& message;
  std::string dir;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class DataFileBuilder : public FileBuilder
{
public:
  DataFileBuilder(CustomMessage& message_, const std::string& dir_);
  virtual ~DataFileBuilder(void) {}

  void build(const DRCEResultData& resultData) throw (Poco::Exception);

  static std::string makeJson(const DRCEResultData& resultData) throw (Poco::Exception);
private:
  typedef FileBuilder inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class StatusFileBuilder : public DataFileBuilder
{
public:
  StatusFileBuilder(CustomMessage& message_, const std::string& dir_);
  virtual ~StatusFileBuilder(void) {}

  void build(const DRCEResultData& resultData) throw (Poco::Exception);
  static std::string getStatusJson(const DRCEResultData& resultData) throw (Poco::Exception);
private:
  static void cleanup(DRCEResultDataItem& resultDataItem);
private:
  typedef DataFileBuilder inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class RequestFileBuilder : public FileBuilder
{
public:
  RequestFileBuilder(CustomMessage& message_, const std::string& dir_);
  virtual ~RequestFileBuilder(void) {}

  void build(DRCEInputJsonMessage& inputJsonMessage) throw (Poco::Exception);
private:
  typedef FileBuilder inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_FILE_BUILDER_HPP
