#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <Poco/File.h>

#include "DRCEFileStream.hpp"
#include "DRCEMessageConst.hpp"
#include "DRCEError.hpp"
#include "DRCEOutputJsonMessage.hpp"
#include "DRCEResultDataSerializator.hpp"
#include "DRCEFileExtractor.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
DRCEResultDataItem DataFileExtractor::extract(const std::string& dataFileName) throw (Poco::Exception)
{
  std::string dataFileData;
  FileStream dataFile(dataFileName, FileStream::read);
  if (!dataFile.isOpen())
  {
    std::stringstream errMsg;
    FILE* pFile = fopen(dataFileName.c_str(), "r");
    if (pFile==NULL)
      errMsg << message(message_const::ERROR_READ_DATA_FILE, dataFileName) << " (" << strerror(errno) << ")";
    else
      fclose(pFile);

    throw Poco::Exception(errMsg.str(), ERROR_READ_DATA_FILE);
  }

  dataFile >> dataFileData;

  DRCEResultData resultData;

  if (!dataFileData.empty())
  {
    DRCEResultDataSerializator resultSerializator(resultData);
    if (!resultSerializator.unserialize(dataFileData))
      throw Poco::Exception(resultSerializator.getErrorMsg(), resultSerializator.getErrorCode());

    if (resultData.getItemsCount()!=1)
      throw Poco::Exception(message(message_const::BAD_FILE_CONTENT, dataFileName), ERROR_BAD_DATA_CONTENT);
  }
  else
  {
    DRCEResultDataItem resultDataItem;
    resultDataItem.setErrorCode(ERROR_BAD_DATA_CONTENT);
    resultDataItem.setErrorMessage(message(message_const::DATA_FILE_IS_EMPTY, dataFileName));
    resultDataItem.setState(static_cast<unsigned int>(DRCETaskRequest::TaskState::UNDEFINED));
    resultData.addDataItem(std::forward<DRCEResultDataItem>(resultDataItem));
  }
  return resultData.getDataItem(0);
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
DRCEInputJsonMessage RequestFileExtractor::extract(const std::string& requestFileName) throw (Poco::Exception)
{
  std::string requestFileData;
  FileStream requestFile(requestFileName, FileStream::read);
  if (!requestFile.isOpen())
  {
    std::stringstream errMsg;
    FILE* pFile = fopen(requestFileName.c_str(), "r");
    if (pFile==NULL)
      errMsg << "Errno: " << strerror(errno) << std::endl;
    else
      fclose(pFile);

    errMsg << message(message_const::ERROR_READ_REQUEST_FILE, requestFileName);
    throw Poco::Exception(errMsg.str(), ERROR_READ_REQUEST_FILE);
  }

  requestFile >> requestFileData;

  if (requestFileData.empty())
    throw Poco::Exception(message(message_const::EMPTY_FILE_CONTENT, requestFileName), ERROR_BAD_DATA_CONTENT);

  DRCEInputJsonMessage inputJsonMessage(requestFileData);
  if (inputJsonMessage.isError())
    throw Poco::Exception(message(message_const::BAD_FILE_CONTENT, requestFileName)+
        message(message_const::ERROR, inputJsonMessage.getErrorMsg()), inputJsonMessage.getErrorCode());

  return inputJsonMessage;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 
