#include <Poco/Path.h>
#include <sstream>
#include <ctype.h>

#include "DRCEMessageConst.hpp"
#include "DRCEFileStream.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
FileStream::FileStream(void)
: fs()
{
}
//-----------------------------------------------------------------------------
FileStream::FileStream(const std::string& fileName, OpenMode openMode)
: fs(fileName.c_str(), openMode)
{
}
//-----------------------------------------------------------------------------
FileStream::~FileStream(void)
{
  close();
}
//-----------------------------------------------------------------------------
bool FileStream::isOpen(void) const
{
  return fs.is_open();
}
//-----------------------------------------------------------------------------
void FileStream::open(const std::string& fileName, OpenMode openMode)
{
  fs.open(fileName.c_str(), openMode);
}
//-----------------------------------------------------------------------------
void FileStream::close(void)
{
  if (fs.is_open())
    fs.close();
}
//-----------------------------------------------------------------------------
FileStream& FileStream::operator<<(const std::string& data)
{
  fs << data;
  fs.flush();
  return *this;
}
//-----------------------------------------------------------------------------
FileStream& FileStream::operator>>(std::string& data)
{
  fs.seekg(0, std::ios::end);
  data.resize(fs.tellg());
  fs.seekg(0, std::ios::beg);
  fs.read(const_cast<char*>(data.c_str()), data.size());
  return *this;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
std::string FileStream::getDataFileName(const std::string& dir, unsigned int taskId)
{
  std::ostringstream ostr;
  ostr << dir << "/" << taskId << "." << drce_const::dataFileExt;
  return ostr.str();
}
//-----------------------------------------------------------------------------
std::string FileStream::getStatusFileName(const std::string& dir, unsigned int taskId)
{
  std::ostringstream ostr;
  ostr << dir << "/" << taskId;
  return ostr.str();
}
//-----------------------------------------------------------------------------
std::string FileStream::getRequestFileName(const std::string& dir, unsigned int taskId)
{
  std::ostringstream ostr;
  ostr << dir << "/" << taskId << "." << drce_const::requestFileExt;
  return ostr.str();
}
//-----------------------------------------------------------------------------
std::string FileStream::getDumpFileName(const std::string& dir, const std::string& nodeName)
{
  return dir+"/"+nodeName+"_"+drce_const::dumpFileName;
}
//-----------------------------------------------------------------------------
bool FileStream::isStatusFileName(const std::string& fileName)
{
  Poco::Path path(fileName);
  char* str = const_cast<char*>(path.getFileName().c_str());
  int symbols = 0;
  int digits = 0;
  while(*str != 0)
  {
    ++symbols;
    if(isdigit(*str++))
      ++digits;
  }
  return (symbols==digits);
}
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 
