/**
 * @file DRCEFileStream.hpp
 *
 * Implementation class FileStream of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_FILE_STREAM_HPP
#define DRCE_FILE_STREAM_HPP

#include <fstream>
#include <iostream>

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class FileStream
{
public:
  typedef std::ios_base::openmode OpenMode;

  static const OpenMode write = (std::fstream::trunc|std::fstream::binary|std::fstream::out);
  static const OpenMode read = (std::fstream::binary|std::fstream::in);

  FileStream(void);
  FileStream(const std::string& fileName, OpenMode openMode=write);
   ~FileStream(void);

   bool isOpen(void) const;
   void open(const std::string& fileName, OpenMode openMode=write);
   void close(void);

   FileStream& operator<<(const std::string& data);
   FileStream& operator>>(std::string& data);

   static std::string getDataFileName(const std::string& dir, unsigned int taskId);
   static std::string getStatusFileName(const std::string& dir, unsigned int taskId);
   static std::string getRequestFileName(const std::string& dir, unsigned int taskId);
   static std::string getDumpFileName(const std::string& dir, const std::string& nodeName);

   static bool isStatusFileName(const std::string& fileName);
private:
   std::fstream fs;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_FILE_STREAM_HPP
