#include <utility>

#include "DRCEFilesList.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
FileItem::FileItem(void)
: name(""), data(""), actionType(0)
{
}
//-----------------------------------------------------------------------------
FileItem::FileItem(const std::string& name_, const std::string& data_, unsigned int actionType_)
: name(name_), data(data_), actionType(actionType_)
{
}
//-----------------------------------------------------------------------------
FileItem::FileItem(const FileItem& rhs)
: name(""), data(""), actionType(0)
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
FileItem::FileItem(FileItem&& rhs)
: name(""), data(""), actionType(0)
{
  (*this) = std::forward<FileItem>(rhs);
}
//-----------------------------------------------------------------------------
FileItem& FileItem::operator=(const FileItem& rhs)
{
  if (this!=&rhs)
  {
    name = rhs.name;
    data = rhs.data;
    actionType = rhs.actionType;
  }
  return *this;
}
//-----------------------------------------------------------------------------
FileItem& FileItem::operator=(FileItem&& rhs)
{
  if (this!=&rhs)
  {
    name = std::move(rhs.name);
    data = std::move(rhs.data);
    actionType = std::move(rhs.actionType);
  }
  return *this;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
DRCEFilesList::DRCEFilesList(void)
: files()
{
}
//-----------------------------------------------------------------------------
DRCEFilesList::DRCEFilesList(const DRCEFilesList& rhs)
: files()
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
DRCEFilesList::DRCEFilesList(DRCEFilesList&& rhs)
: files()
{
  (*this) = std::forward<DRCEFilesList>(rhs);
}
//-----------------------------------------------------------------------------
DRCEFilesList& DRCEFilesList::operator=(const DRCEFilesList& rhs)
{
  if (this!=&rhs)
  {
    files = rhs.getFiles();
  }
  return *this;
}
//-----------------------------------------------------------------------------
DRCEFilesList& DRCEFilesList::operator=(DRCEFilesList&& rhs)
{
  if (this!=&rhs)
  {
    files = std::move(rhs.getFiles());
  }
  return *this;
}
//-----------------------------------------------------------------------------
void DRCEFilesList::addFileItem(const FileItem& fileItem)
{
  files.push_back(fileItem);
}
//-----------------------------------------------------------------------------
void DRCEFilesList::addFileItem(FileItem&& fileItem)
{
  files.push_back(std::move(std::forward<FileItem>(fileItem)));
}
//-----------------------------------------------------------------------------
void DRCEFilesList::addFileItem(const std::string& name, const std::string& data, unsigned int actionType)
{
  files.push_back(std::move(std::forward<FileItem>(FileItem(name, data, actionType))));
}
//-----------------------------------------------------------------------------
size_t DRCEFilesList::getFilesCount(void) const
{
  return files.size();
}
//-----------------------------------------------------------------------------
const FileItem& DRCEFilesList::getFileItem(size_t index) const
{
  return files[index];
}
//-----------------------------------------------------------------------------
// cppcheck-suppress unusedFunction
void DRCEFilesList::setFileItem(size_t index, const FileItem& fileItem)
{
  files[index] = fileItem;
}
//-----------------------------------------------------------------------------
void DRCEFilesList::setFileItem(size_t index, FileItem&& fileItem)
{
  files[index] = std::forward<FileItem>(fileItem);
}
//-----------------------------------------------------------------------------
void DRCEFilesList::clear(void)
{
  files.clear();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE
