/**
 * @file DRCEFilesList.hpp
 *
 * Implementation class DRCEFilesList of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_FILES_LIST_HPP
#define DRCE_FILES_LIST_HPP

#include <vector>
#include <iostream>

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
struct FileItem
{
  /*!
   *  The const list for action that will be done with file
   */
  enum
  {
    CREATE_BEFORE_EXEC_AND_WRITE_DATA = 1,    /*!< create before command execution and write data stream. */
    DELETE_BEFORE_EXEC = 2,                   /*!< delete before command execution.  */
    READ_AFTER_EXEC_AND_RETURN_RESULT = 4,    /*!< read data stream after execution and return in result response. */
    DELETE_AFTER_EXEC = 8,                    /*!< delete after command execution. */

    ENCODE_DECODE_CONTENT_BASE64 = 2147483648 /*!<  encode/decode content with base64 algorithm. */
  };

  FileItem(void);
  FileItem(const std::string& name_, const std::string& data_, unsigned int actionType_);
  FileItem(const FileItem& rhs);
  FileItem(FileItem&& rhs);

  FileItem& operator=(const FileItem& rhs);
  FileItem& operator=(FileItem&& rhs);

  std::string name;         /*!< name of file that will be processed. */
  std::string data;         /*!< data stream that can be written to the file. */
  unsigned int actionType;  /*!< action that will be done with file. */
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class DRCEFilesList
{
public:
  DRCEFilesList(void);
  DRCEFilesList(const DRCEFilesList& rhs);
  DRCEFilesList(DRCEFilesList&& rhs);
  virtual ~DRCEFilesList(void) {}

  DRCEFilesList& operator=(const DRCEFilesList& rhs);
  DRCEFilesList& operator=(DRCEFilesList&& rhs);

  void addFileItem(const FileItem& fileItem);
  void addFileItem(FileItem&& fileItem);
  void addFileItem(const std::string& name, const std::string& data, unsigned int actionType);

  const std::vector<FileItem>& getFiles(void) const {return files;}

  size_t getFilesCount(void) const;
  const FileItem& getFileItem(size_t index) const;
  void setFileItem(size_t index, const FileItem& fileItem);
  void setFileItem(size_t index, FileItem&& fileItem);

  void clear(void);
protected:
  std::vector<FileItem> files;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE

#endif // DRCE_FILES_LIST_HPP
