/**
 * @file DRCEFunctionalObjectBase.hpp
 *
 * Implementation class DRCEFunctionalObjectBase of a Distributed Remote Command Execution functional object in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_FUNCTIONAL_OBJECT_BASE_HPP
#define DRCE_FUNCTIONAL_OBJECT_BASE_HPP

#include <Poco/Timestamp.h>
#include <sstream>
#include <iostream>

#include "LoggerStream.hpp"
#include "DRCETaskRequest.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class DRCEFunctionalObjectBase
{
public:
  DRCEFunctionalObjectBase(void);
  virtual ~DRCEFunctionalObjectBase(void) {}

  void setErrorMsg(const std::string& errorMsg_) {errorMsg=errorMsg_;}
  std::string getErrorMsg(void) const {return errorMsg;}

  void setErrorCode(unsigned int errorCode_) {errorCode=errorCode_;}
  unsigned int getErrorCode(void) const {return errorCode;}

  void isError(bool isError_) {_isError=isError_;}
  bool isError(void) const {return _isError;}

  void resetLogStream(void);
  void setLogStream(std::ostream& os);

  std::string logMsg(bool isReset=true);
  std::ostream& log(LoggerStream::Priority logPriority);

  void setLoggable(Loggable loggable);
  void resetLoggable(void);

  void setEnvironments(const std::string& env);
  void applyEnvironments(DRCETaskRequest* pTaskRequest);

protected:
  static bool insensitiveCompare(const std::string& lhs, const std::string& rhs);
  static size_t getElapsedTimeMsec(const Poco::Timestamp& ts); // return msec
protected:
  std::string errorMsg;
  unsigned int errorCode;
  bool _isError;

  LoggerStream logger;

  std::vector<std::pair<std::string, std::string> > environments;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE 

#endif // DRCE_FUNCTIONAL_OBJECT_BASE_HPP
