/**
 * @file DRCEInputJsonMessage.hpp
 *
 * Implementation class DRCEInputJsonMessage of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_INPUT_JSON_MESSAGE_HPP
#define DRCE_INPUT_JSON_MESSAGE_HPP

#include <Poco/JSON/Object.h>
#include <Poco/JSON/Array.h>
#include <Poco/SharedPtr.h>
#include <vector>

#include "JsonSerializable.hpp"
#include "DRCEJsonMessageConst.hpp"
#include "DRCETaskRequest.hpp"
#include "DRCESubtasksList.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class DRCEInputJsonMessage;
typedef DRCESubtasksList <DRCEInputJsonMessage> Subtasks;
//-----------------------------------------------------------------------------
class DRCEInputJsonMessage : public IJsonSerializable, public Subtasks
{
public:
  explicit DRCEInputJsonMessage(const std::string& json="");
  DRCEInputJsonMessage(const DRCEInputJsonMessage& rhs);
  DRCEInputJsonMessage(DRCEInputJsonMessage&& rhs);
  virtual ~DRCEInputJsonMessage(void) {}

  DRCEInputJsonMessage& operator=(const DRCEInputJsonMessage& rhs);
  DRCEInputJsonMessage& operator=(DRCEInputJsonMessage&& rhs);

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void setRequestType(DRCETaskRequest::RequestType requestType_) {requestType=requestType_;}
  DRCETaskRequest::RequestType getRequestType(void) const {return requestType;}

  void setRequestData(const std::string& requestData_);
  std::string getRequestData(void) const {return requestData;}

  void setRequestId(unsigned int requestId_) {requestId=requestId_;}
  unsigned int getRequestId(void) const {return requestId;}

  void setTaskRequest(Poco::SharedPtr<DRCETaskRequest> pTaskRequest_);
  Poco::SharedPtr<DRCETaskRequest> getTaskRequest(void) const {return pTaskRequest;}

  void clear(void);
private:
  DRCETaskRequest::RequestType requestType;
  std::string requestData;
  unsigned int requestId;
  Poco::SharedPtr<DRCETaskRequest> pTaskRequest;
private:
  void packToObject(Poco::JSON::Object& obj) throw (Poco::Exception);
  void packToArray(Poco::JSON::Array& arr) throw (Poco::Exception);
  void unpackFromObject(Poco::JSON::Object& obj) throw (Poco::Exception);
  void unpackFromArray(Poco::JSON::Array& arr) throw (Poco::Exception);
private:
  friend std::istream& operator>>(std::istream& is, DRCEInputJsonMessage& inputJsonMessage);
  friend std::ostream& operator<<(std::ostream& os, const DRCEInputJsonMessage& inputJsonMessage);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_INPUT_JSON_MESSAGE_HPP
