/**
 * @file DRCEJsonMessageConst.hpp"
 *
 * Declaration const list need for Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_JSON_MESSAGE_CONST_HPP
#define DRCE_JSON_MESSAGE_CONST_HPP

#include <iostream>

namespace HCE
{
namespace drce
{
namespace drce_json_message_const
{
//-----------------------------------------------------------------------------
  // session options
  const std::string sessionType = "type";
  const std::string port = "port";
  const std::string userName = "user";
  const std::string userPassword = "password";
  const std::string shellName = "shell";
  const std::string environment = "environment";
  const std::string homeDir = "home_directory";
  const std::string timeout = "timeout";
  const std::string tmode = "tmode";
  const std::string timeMax = "time_max";
  const std::string cleanup = "cleanup";
  // resource limits
  const std::string procMax = "proc_max";
  const std::string threadsMax = "threads_max";
  const std::string cpuMax = "cpu_max";
  const std::string iowaitMax = "iowait_max";
  const std::string vramFree = "vram_free";
  const std::string vramFreeMin = "vram_free_min";
  const std::string rramFree = "rram_free";
  const std::string rramFreeMin = "rram_free_min";
  const std::string diskFree = "disk_free";
  const std::string diskFreeMin = "disk_free_min";
  const std::string iowaitUsageMax = "iowait_usage_max";
  const std::string vramUsageMax = "vram_usage_max";
  const std::string rramUsageMax = "rram_usage_max";
  const std::string diskUsageMax = "disk_usage_max";
  const std::string diskUsageMaxBytes = "disk_usage_max_bytes";
  // file item
  const std::string fileName = "name";
  const std::string fileData = "data";
  const std::string actionTypeMask = "action";
  // input json message
  const std::string requestType = "type";
  const std::string requestData = "data";
  const std::string requestId = "id";
  const std::string subtasksArray = "subtasks";
  // task execute request
  const std::string sessionOptions = "session";
  const std::string commandLine = "command";
  const std::string inputStream = "input";
  const std::string resourceLimits = "limits";
  // task check state
  const std::string checkType = "type";
  // task fetch type
  const std::string fetchType = "type";
  // task terminate
  const std::string algorithmType = "alg";
  const std::string delayValue = "delay";
  const std::string repeatValue = "repeat";
  const std::string signalValue = "signal";
  // output json message
  const std::string errorCode = "error_code";
  const std::string errorMessage = "error_message";
  const std::string state = "state";
  const std::string pid = "pid";
  const std::string responseBody = "data";
  const std::string elapsedTime = "time";
  const std::string nodeHost = "host";
  const std::string nodePort = "port";
  // result data
  const std::string stdoutStream = "stdout";
  const std::string stderrStream = "stderror";
  const std::string filesArray = "files";
  const std::string nodeName = "node";
  const std::string time = "time";
  const std::string exitStatus = "exit_status";
  const std::string fields = "fields";
  // task status data structure
  const std::string id = "id";
  const std::string sdate = "sdate";
  // task info data structure
  const std::string taskId = "id";
  const std::string info = "info";
  const std::string modified = "modified";
  // resource usage data structure
  const std::string CPU = "CPU";
  const std::string IOWAIT = "IOWAIT";
  const std::string VRAM = "VRAM";
  const std::string RRAM = "RRAM";
  const std::string DISK = "DISK";
  const std::string PROCESSES = "PROCESSES";
  const std::string THREADS = "THREADS";
  const std::string LOADAVG = "LOADAVG";
  // stat variable data data structure
  const std::string MIN = "min";
  const std::string AVG = "avg";
  const std::string MAX = "max";
  const std::string STAT_DELIMITER = "_";
  // stat variables data structure
  const std::string TIME_ASYNC_TASK = "time_async_task";
  const std::string TIME_SYNC_TASK = "time_sync_task";
  const std::string SIZE_INPUT_BUFFER_SYNC_TASKS = "size_input_buffer_sync_tasks";
  const std::string SIZE_OUTPUT_BUFFER_SYNC_TASKS = "size_output_buffer_sync_tasks";
  const std::string SIZE_INPUT_BUFFER_ASYNC_TASKS = "size_input_buffer_async_tasks";
  const std::string SIZE_OUTPUT_BUFFER_ASYNC_TASKS = "size_output_buffer_async_tasks";
  const std::string CPU_USAGE_ASYNC_TASKS = "cpu_usage_async_tasks";
  const std::string MEMORY_USAGE_ASYNC_TASKS = "memory_usage_async_tasks";
  const std::string COUNT_SYNC_TASKS_FOR_MINUTE = "count_sync_tasks_for_minute";
  const std::string COUNT_ASYNC_TASKS_FOR_MINUTE = "count_async_tasks_for_minute";
  const std::string COUNT_SYNC_TASKS = "count_sync_tasks";
  const std::string COUNT_ASYNC_TASKS = "count_async_tasks";
  const std::string COUNT_SYNC_TASKS_FAIL = "count_sync_tasks_fail";
  const std::string COUNT_ASYNC_TASKS_FAIL = "count_async_tasks_fail";
  // average count variables data structure
  const std::string TIME_ASYNC_TASK_COUNT = "time_async_task_count";
  const std::string TIME_SYNC_TASK_COUNT = "time_sync_task_count";
  const std::string SIZE_INPUT_BUFFER_SYNC_TASKS_COUNT = "size_input_buffer_sync_tasks_count";
  const std::string SIZE_OUTPUT_BUFFER_SYNC_TASKS_COUNT = "size_output_buffer_sync_tasks_count";
  const std::string SIZE_INPUT_BUFFER_ASYNC_TASKS_COUNT = "size_input_buffer_async_tasks_count";
  const std::string SIZE_OUTPUT_BUFFER_ASYNC_TASKS_COUNT = "size_output_buffer_async_tasks_count";
  const std::string CPU_USAGE_ASYNC_TASKS_COUNT = "cpu_usage_async_tasks_count";
  const std::string MEMORY_USAGE_ASYNC_TASKS_COUNT = "memory_usage_async_tasks_count";
  const std::string COUNT_SYNC_TASKS_FOR_MINUTE_COUNT = "count_sync_tasks_for_minute_count";
  const std::string COUNT_ASYNC_TASKS_FOR_MINUTE_COUNT = "count_async_tasks_for_minute_count";

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce_json_message_const
} // end namespace drce
} // end namespace HCE

#endif // DRCE_JSON_MESSAGE_CONST_HPP
