/**
 * @file DRCEListAllTasks.hpp
 *
 * Implementation class DRCEListAllTasks of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_LIST_ALL_TASKS_HPP
#define DRCE_LIST_ALL_TASKS_HPP

#include <iostream>

#include "DRCECommonList.hpp"
#include "JsonSerializable.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
struct TaskInfoData
{
  TaskInfoData(void);
  TaskInfoData(const TaskInfoData& rhs);
  TaskInfoData(TaskInfoData&& rhs);
  ~TaskInfoData(void) {}

  TaskInfoData& operator=(const TaskInfoData& rhs);
  TaskInfoData& operator=(TaskInfoData&& rhs);

  void clear(void);

  unsigned int taskId;
  std::string infoData;
  time_t timeModified;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class DRCEListAllTasks : public DRCECommonList <TaskInfoData>, public IJsonSerializable
{
public:
  explicit DRCEListAllTasks(void);
  ~DRCEListAllTasks(void) {};

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  bool rebuild(const std::string& sourceDir);
  std::string toString(void);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
}// end namespace drce 
} // end namespace HCE 

#endif // DRCE_LIST_ALL_TASKS_HPP
