/**
 * @file DRCENodeOptions.hpp
 *
 * Implementation class DRCENodeOptions of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_NODE_OPTIONS_HPP
#define DRCE_NODE_OPTIONS_HPP

#include <string>

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class DRCENodeOptions
{
public:
  DRCENodeOptions(const std::string& nodeName_,
      const std::string& homeDir_,
      const std::string& tasksDataDir_,
      const std::string& tasksStatusDir_,
      const std::string& nodeHost_="",
      const std::string& nodePort_="");
  DRCENodeOptions(const DRCENodeOptions& rhs);
  DRCENodeOptions(DRCENodeOptions&& rhs);
  virtual ~DRCENodeOptions(void) {}

  DRCENodeOptions& operator=(const DRCENodeOptions& rhs);
  DRCENodeOptions& operator=(DRCENodeOptions&& rhs);

  void setNodeName(const std::string& nodeName_) {nodeName=nodeName_;}
  std::string getNodeName(void) const {return nodeName;}

  void setHomeDir(const std::string& homeDir_) {homeDir=homeDir_;}
  std::string getHomeDir(void) const {return homeDir;}

  void setTasksDataDir(const std::string& tasksDataDir_) {tasksDataDir=tasksDataDir_;}
  std::string getTasksDataDir(void) const {return tasksDataDir;}

  void setTasksStatusDir(const std::string& tasksStatusDir_) {tasksStatusDir=tasksStatusDir_;}
  std::string getTasksStatusDir(void) const {return tasksStatusDir;}

  void setNodeHost(const std::string& nodeHost_) {nodeHost=nodeHost_;}
  std::string getNodeHost(void) const {return nodeHost;}

  void setNodePort(const std::string& nodePort_) {nodePort=nodePort_;}
  std::string getNodePort(void) const {return nodePort;}

protected:
  std::string nodeName;
  std::string homeDir;
  std::string tasksDataDir;
  std::string tasksStatusDir;
  std::string nodeHost;
  std::string nodePort;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_NODE_OPTIONS_HPP
