#include "DRCEOutputJsonMessage.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
DRCEOutputJsonMessage::DRCEOutputJsonMessage(const std::string& json)
: inherited(), resultData()
{
  if (!json.empty())
    unserialize(json);
}
//-----------------------------------------------------------------------------
void DRCEOutputJsonMessage::clear(void)
{
  resetError();
  resultData.clear();
}
//-----------------------------------------------------------------------------
void DRCEOutputJsonMessage::setResultData(const DRCEResultData& rhs)
{
  resultData = rhs;
}
//-----------------------------------------------------------------------------
void DRCEOutputJsonMessage::setResultData(DRCEResultData&& rhs)
{
  resultData = std::forward<DRCEResultData>(rhs);
}
//-----------------------------------------------------------------------------
bool DRCEOutputJsonMessage::serialize(std::string& json)
{
  resetError();
  DRCEResultDataSerializator resultDataSerializator(resultData);
  if (!resultDataSerializator.serialize(json))
  {
    _isError = resultDataSerializator.isError();
    errorMsg = resultDataSerializator.getErrorMsg();
    errorCode = resultDataSerializator.getErrorCode();
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
bool DRCEOutputJsonMessage::unserialize(const std::string& json)
{
  clear();
  DRCEResultDataSerializator resultDataSerializator(resultData);
  if (!resultDataSerializator.unserialize(json))
  {
    _isError = resultDataSerializator.isError();
    errorMsg = resultDataSerializator.getErrorMsg();
    errorCode = resultDataSerializator.getErrorCode();
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
std::istream& operator>>(std::istream& is, DRCEOutputJsonMessage& outputJsonMessage)
{
  std::string json;
  is.seekg(0, std::ios::end);
  json.resize(is.tellg());
  is.seekg(0, std::ios::beg);
  is.read(const_cast<char*>(json.c_str()), json.size());
  outputJsonMessage.unserialize(json);
  return is;
}
//-----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const DRCEOutputJsonMessage& outputJsonMessage)
{
  std::string json;
  const_cast<DRCEOutputJsonMessage&>(outputJsonMessage).serialize(json);
  return os << json;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE
