/**
 * @file DRCEProcessData.hpp
 *
 * Implementation class DRCEProcessData of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_PROCESS_DATA_HPP
#define DRCE_PROCESS_DATA_HPP

#include <string>

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
struct ProcessData
{
  ProcessData(void);
  ProcessData(const ProcessData& rhs);
  ProcessData(ProcessData&& rhs);

  ProcessData& operator=(const ProcessData& rhs);
  ProcessData& operator=(ProcessData&& rhs);

  void clear(void);

  pid_t pid;            // process id
  pid_t ppid;           // parent process id
  size_t vram;          // vsize memory (in bytes)
  size_t rram;          // rss memory (in bytes)
  unsigned int threads; // number of threads
  size_t utime;         // user-mode CPU time accumulated by process
  size_t stime;         // kernel-mode CPU time accumulated by process
  size_t cutime;        // cumulative utime of process and reaped children
  size_t cstime;        // cumulative stime of process and reaped children
  std::string cmd;      // basename of executable file in call to exec(2)
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_PROCESS_DATA_HPP
