#ifndef DRCE_READ_PROCESS_DATA
#define DRCE_READ_PROCESS_DATA

#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>

#ifdef  __cplusplus
#define EXTERN_C_BEGIN extern "C" {
#define EXTERN_C_END }
#define EXTERN_C extern "C"
#else
#define EXTERN_C_BEGIN
#define EXTERN_C_END
#define EXTERN_C
#endif

EXTERN_C_BEGIN

#define P_G_SZ 20

#define restrict __restrict__

#define likely(x)       __builtin_expect(!!(x),1)
#define unlikely(x)     __builtin_expect(!!(x),0)
#define expected(x,y)   __builtin_expect((x),(y))

typedef struct proc_t
{
// 1st 16 bytes
  int
    tid,		// (special)       task id, the POSIX thread ID (see also: tgid)
    ppid;		// stat,status     pid of parent process
  unsigned
    pcpu;   // stat (special)  %CPU usage (is not filled in by readproc!!!)
  char
    state,		// stat,status     single-char code for process state (S=sleeping)
    pad_1,		// n/a             padding
    pad_2,		// n/a             padding
    pad_3;		// n/a             padding
// 2nd 16 bytes
  unsigned long long
    utime,		// stat            user-mode CPU time accumulated by process
    stime,		// stat            kernel-mode CPU time accumulated by process
// and so on...
    cutime,		// stat            cumulative utime of process and reaped children
    cstime,		// stat            cumulative stime of process and reaped children
    start_time;	// stat            start time of process -- seconds since 1-1-70

  char
// Linux 2.1.7x and up have 64 signals. Allow 64, plus '\0' and padding.
    signal[18],	  // status          mask of pending signals, per-task for readtask() but per-proc for readproc()
    blocked[18],	// status          mask of blocked signals
    sigignore[18],// status          mask of ignored signals
    sigcatch[18],	// status          mask of caught  signals
    _sigpnd[18];	// status          mask of PER TASK pending signals

  unsigned long
    start_code,	// stat            address of beginning of code segment
    end_code,	  // stat            address of end of code segment
    start_stack,// stat            address of the bottom of stack for the process
    kstk_esp,	  // stat            kernel stack pointer
    kstk_eip,	  // stat            kernel instruction pointer
    wchan;		  // stat (special)  address of kernel wait channel proc is sleeping in
  long
    priority,	// stat            kernel scheduling priority
    nice,		  // stat            standard unix nice level of process
    rss,		  // stat            resident set size from /proc/#/stat (pages)
    alarm,		// stat            ?
  // the next 7 members come from /proc/#/statm
    size,		  // statm           total # of pages of memory
    resident,	// statm           number of resident set (non-swapped) pages (4k)
    share,		// statm           number of pages of shared (mmap'd) memory
    trs,		  // statm           text resident set size
    lrs,		  // statm           shared-lib resident set size
    drs,		  // statm           data resident set size
    dt;		    // statm           dirty pages
  unsigned long
    vm_size,  // status          same as vsize in kb
    vm_lock,  // status          locked pages in kb
    vm_rss,   // status          same as rss in kb
    vm_data,  // status          data size
    vm_stack, // status          stack size
    vm_exe,   // status          executable size
    vm_lib,   // status          library size (all pages, not just used ones)
    rtprio,		// stat            real-time priority
    sched,		// stat            scheduling class
    vsize,		// stat            number of pages of virtual memory ...
    rss_rlim,	// stat            resident set size limit?
    flags,		// stat            kernel flags for the process
    min_flt,	// stat            number of minor page faults since process start
    maj_flt,	// stat            number of major page faults since process start
    cmin_flt,	// stat            cumulative min_flt of process and child processes
    cmaj_flt;	// stat            cumulative maj_flt of process and child processes
  char
    **environ,	// (special)       environment string vector (/proc/#/environ)
    **cmdline;	// (special)       command line string vector (/proc/#/cmdline)
  char
// Be compatible: Digital allows 16 and NT allows 14 ???
    euser[P_G_SZ],	// stat(),status   effective user name
    ruser[P_G_SZ],	// status          real user name
    suser[P_G_SZ],	// status          saved user name
    fuser[P_G_SZ],	// status          filesystem user name
    rgroup[P_G_SZ],	// status          real group name
    egroup[P_G_SZ],	// status          effective group name
    sgroup[P_G_SZ],	// status          saved group name
    fgroup[P_G_SZ],	// status          filesystem group name
    cmd[16];	// stat,status     basename of executable file in call to exec(2)
  struct proc_t
    *ring,		// n/a             thread group ring
    *next;		// n/a             various library uses
  int
    pgrp,		    // stat            process group id
    session,	  // stat            session id
    nlwp,		    // stat,status     number of threads, or 0 if no clue
    tgid,		    // (special)       task group ID, the POSIX PID (see also: tid)
    tty,		    // stat            full device number of controlling terminal
    euid, egid, // stat(),status   effective
    ruid, rgid, // status          real
    suid, sgid, // status          saved
    fuid, fgid, // status          fs (used for file access only)
    tpgid,		  // stat            terminal process group id
    exit_signal,// stat            might not be SIGCHLD
    processor;  // stat            current (or most recent?) CPU
} proc_t;

// PROCTAB: data structure holding the persistent information readproc needs
// from openproc().  The setup is intentionally similar to the dirent interface
// and other system table interfaces (utmp+wtmp come to mind).

#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>

#define PROCPATHLEN 64

typedef struct PROCTAB
{
  DIR*	procfs;
  DIR*	taskdir;  // for threads
  pid_t	taskdir_user;  // for threads
  int   did_fake; // used when taskdir is missing
  int(*finder)(struct PROCTAB *restrict const, proc_t *restrict const);
  proc_t*(*reader)(struct PROCTAB *restrict const, proc_t *restrict const);
  int(*taskfinder)(struct PROCTAB *restrict const, const proc_t *restrict const, proc_t *restrict const, char *restrict const);
  proc_t*(*taskreader)(struct PROCTAB *restrict const, const proc_t *restrict const, proc_t *restrict const, char *restrict const);
  pid_t*	pids;	// pids of the procs
  uid_t*	uids;	// uids of procs
  int		nuid;	  // cannot really sentinel-terminate unsigned short[]
  int         i;  // generic
  unsigned	flags;
  unsigned    u;  // generic
  void *      vp; // generic
  char        path[PROCPATHLEN];  // must hold /proc/2000222000/task/2000222000/cmdline
  unsigned pathlen; // length of string in the above (w/o '\0')
} PROCTAB;

// initialize a PROCTAB structure holding needed call-to-call persistent data
//PROCTAB* openproc(int flags, ... /* pid_t*|uid_t*|dev_t*|char* [, int n] */ );

// clean-up open files, etc from the openproc()
//void closeproc(PROCTAB* PT);

// retrieve the next process matching the criteria set by the openproc()
//proc_t* readproc(PROCTAB *restrict const PT, proc_t *restrict p);

// deallocate space allocated by readproc
//void freeproc(proc_t* p);

//fill out a proc_t for a single task
//proc_t * getProcStats(pid_t pid, proc_t *p);

#define PROC_FILLMEM         0x0001 // read statm
#define PROC_FILLSTATUS      0x0020 // read status -- currently unconditional
#define PROC_FILLSTAT        0x0040 // read stat -- currently unconditional
#define PROC_FILLARG         0x0100 // alloc and fill in `cmdline'
// Obsolete, consider only processes with one of the passed:
#define PROC_PID             0x1000  // process id numbers ( 0   terminated)
#define PROC_UID             0x4000  // user id numbers    ( length needed )

EXTERN_C_END
//-----------------------------------------------------------------------------
#include <Poco/ThreadPool.h>
#include <Poco/Task.h>
#include <Poco/TaskManager.h>
#include <Poco/Mutex.h>
#include <Poco/ScopedLock.h>
#include <Poco/Logger.h>
#include <vector>

#include "DRCEProcessData.hpp"
#include "DRCEMessageConst.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class DRCEReadProcessData
{
public:
  static DRCEReadProcessData& getInstance(void);
  static void shutdown(void);

  static std::vector<ProcessData> getProcessDataList(void);
  static ProcessData getProcessData(pid_t pid);
  static std::vector<pid_t> getAllProcessIds(pid_t pid);

  static void getProcessAndThreads(unsigned int& processCount, unsigned int& threadsCount);
protected:
  static DRCEReadProcessData* pHandler;
  static Poco::Mutex mutex;
private:
  DRCEReadProcessData(void) { Poco::Logger::root().log(Poco::Message(drce_const::moduleName, "DRCEReadProcessData was created", Poco::Message::PRIO_INFORMATION)); };
  ~DRCEReadProcessData(void) { Poco::Logger::root().log(Poco::Message(drce_const::moduleName, "DRCEReadProcessData was deleted", Poco::Message::PRIO_INFORMATION)); }
  DRCEReadProcessData(const DRCEReadProcessData&);
  DRCEReadProcessData& operator=(const DRCEReadProcessData&);
  static void getChildrenList(pid_t pid, std::vector<pid_t>& children);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE

#endif // DRCE_READ_PROCESS_DATA
