#include <utility>

#include "DRCEResourceLimits.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
ResourceUsageLimits::ResourceUsageLimits(void)
: iowaitMax(0.0), vramMax(0.0), rramMax(0.0), diskMax(0.0), diskMaxBytes(0)
{
}
//-----------------------------------------------------------------------------
ResourceUsageLimits::ResourceUsageLimits(const ResourceUsageLimits& rhs)
: iowaitMax(0.0), vramMax(0.0), rramMax(0.0), diskMax(0.0), diskMaxBytes(0)
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
ResourceUsageLimits::ResourceUsageLimits(ResourceUsageLimits&& rhs)
: iowaitMax(0.0), vramMax(0.0), rramMax(0.0), diskMax(0.0), diskMaxBytes(0)
{
  (*this) = std::forward<ResourceUsageLimits>(rhs);
}
//-----------------------------------------------------------------------------
ResourceUsageLimits& ResourceUsageLimits::operator=(const ResourceUsageLimits& rhs)
{
  if (this!=&rhs)
  {
    iowaitMax = rhs.iowaitMax;
    vramMax = rhs.vramMax;
    rramMax = rhs.rramMax;
    diskMax = rhs.diskMax;
    diskMaxBytes = rhs.diskMaxBytes;
  }
  return *this;
}
//-----------------------------------------------------------------------------
ResourceUsageLimits& ResourceUsageLimits::operator=(ResourceUsageLimits&& rhs)
{
  if (this!=&rhs)
  {
    iowaitMax = std::move(rhs.iowaitMax);
    vramMax = std::move(rhs.vramMax);
    rramMax = std::move(rhs.rramMax);
    diskMax = std::move(rhs.diskMax);
    diskMaxBytes = std::move(rhs.diskMaxBytes);
  }
  return *this;
}
//-----------------------------------------------------------------------------
void ResourceUsageLimits::clear(void)
{
  iowaitMax = 0.0;
  vramMax = 0.0;
  rramMax = 0.0;
  diskMax = 0.0;
  diskMaxBytes = 0;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ResourceLimits::ResourceLimits(void)
: procMax(0), threadsMax(0), cpuMax(0.0), iowaitMax(0.0), vramFree(0.0), vramFreeMin(0),
  rramFree(0.0), rramFreeMin(0), diskFree(0.0), diskFreeMin(0),
  usageLimits()
{
}
//-----------------------------------------------------------------------------
ResourceLimits::ResourceLimits(const ResourceLimits& rhs)
: procMax(0), threadsMax(0), cpuMax(0.0), iowaitMax(0.0), vramFree(0.0), vramFreeMin(0),
  rramFree(0.0), rramFreeMin(0), diskFree(0.0), diskFreeMin(0),
  usageLimits()
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
ResourceLimits::ResourceLimits(ResourceLimits&& rhs)
: procMax(0), threadsMax(0), cpuMax(0.0), iowaitMax(0.0), vramFree(0.0), vramFreeMin(0),
  rramFree(0.0), rramFreeMin(0), diskFree(0.0), diskFreeMin(0),
  usageLimits()
{
  (*this) = std::forward<ResourceLimits>(rhs);
}
//-----------------------------------------------------------------------------
ResourceLimits& ResourceLimits::operator=(const ResourceLimits& rhs)
{
  if (this!=&rhs)
  {
    procMax = rhs.procMax;
    threadsMax = rhs.threadsMax;
    cpuMax = rhs.cpuMax;
    iowaitMax = rhs.iowaitMax;
    vramFree = rhs.vramFree;
    vramFreeMin = rhs.vramFreeMin;
    rramFree = rhs.rramFree;
    rramFreeMin = rhs.rramFreeMin;
    diskFree = rhs.diskFree;
    diskFreeMin = rhs.diskFreeMin;
    usageLimits = rhs.usageLimits;
  }
  return *this;
}
//-----------------------------------------------------------------------------
ResourceLimits& ResourceLimits::operator=(ResourceLimits&& rhs)
{
  if (this!=&rhs)
  {
    procMax = std::move(rhs.procMax);
    threadsMax = std::move(rhs.threadsMax);
    cpuMax = std::move(rhs.cpuMax);
    iowaitMax = std::move(rhs.iowaitMax);
    vramFree = std::move(rhs.vramFree);
    vramFreeMin = std::move(rhs.vramFreeMin);
    rramFree = std::move(rhs.rramFree);
    rramFreeMin = std::move(rhs.rramFreeMin);
    diskFree = std::move(rhs.diskFree);
    diskFreeMin = std::move(rhs.diskFreeMin);
    usageLimits = std::move(rhs.usageLimits);
  }
  return *this;
}
//-----------------------------------------------------------------------------
void ResourceLimits::clear(void)
{
  procMax = 0;
  threadsMax = 0;
  cpuMax = 0.0;
  iowaitMax = 0.0;
  vramFree = 0.0;
  vramFreeMin = 0;
  rramFree = 0.0;
  rramFreeMin = 0;
  diskFree = 0.0;
  diskFreeMin = 0;
  usageLimits.clear();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 
