/**
 * @file DRCEResourceLimits.hpp
 *
 * Implementation structure ResourceLimits of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_RESOURCE_LIMITS_HPP
#define DRCE_RESOURCE_LIMITS_HPP

#include <utility>
#include <stddef.h>

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
struct ResourceUsageLimits
{
  ResourceUsageLimits(void);
  ResourceUsageLimits(const ResourceUsageLimits& rhs);
  ResourceUsageLimits(ResourceUsageLimits&& rhs);

  ResourceUsageLimits& operator=(const ResourceUsageLimits& rhs);
  ResourceUsageLimits& operator=(ResourceUsageLimits&& rhs);

  void clear(void);

  double iowaitMax;
  double vramMax;
  double rramMax;
  double diskMax;
  size_t diskMaxBytes;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
struct ResourceLimits
{
  ResourceLimits(void);
  ResourceLimits(const ResourceLimits& rhs);
  ResourceLimits(ResourceLimits&& rhs);

  ResourceLimits& operator=(const ResourceLimits& rhs);
  ResourceLimits& operator=(ResourceLimits&& rhs);

  void clear(void);

  unsigned int procMax;
  unsigned int threadsMax;
  double cpuMax;
  double iowaitMax;
  double vramFree;
  size_t vramFreeMin;
  double rramFree;
  size_t rramFreeMin;
  double diskFree;
  size_t diskFreeMin;

  ResourceUsageLimits usageLimits;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_RESOURCE_LIMITS_HPP
