/**
 * @file DRCEResourceLimitsChecker.hpp
 *
 * Implementation class DRCEResourceLimitsChecker of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_RESOURCE_LIMITS_CHECKER_HPP
#define DRCE_RESOURCE_LIMITS_CHECKER_HPP

#include <iostream>
#include <utility>
#include <Poco/Exception.h>

#include "DRCEResourceLimits.hpp"
#include "CustomMessage.hpp"

namespace HCE
{
namespace drce
{
class AsyncTasks;
//-----------------------------------------------------------------------------
class DRCEResourceLimitsChecker
{
public:
  DRCEResourceLimitsChecker(CustomMessage& message_);
  ~DRCEResourceLimitsChecker(void) {}

  void checkLimits(const ResourceLimits& resourceLimit) throw (Poco::Exception);
  void checkLimits(const AsyncTasks& asyncTask, const ResourceUsageLimits& resourceUsageLimit) throw (Poco::Exception);
private:
  static void getUsingProcessAndThreads(unsigned int& processCount, unsigned int& threadsCount);

private:
  CustomMessage& message;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_RESOURCE_LIMITS_CHECKER_HPP
