/**
 * @file DRCEResourceUsage.hpp
 *
 * Implementation structure ResourceUsage of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_RESOURCE_USAGE_HPP
#define DRCE_RESOURCE_USAGE_HPP

#include <utility>
#include <stddef.h>

#include "JsonSerializable.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
struct ResourceUsage
{
  ResourceUsage(void);
  ResourceUsage(const ResourceUsage& rhs);
  ResourceUsage(ResourceUsage&& rhs);

  ResourceUsage& operator=(const ResourceUsage& rhs);
  ResourceUsage& operator=(ResourceUsage&& rhs);

  void clear(void);

  double cpu;
  double iowait;

  size_t vramUsed;
  double vramUsedPercent;
  size_t rramUsed;
  double rramUsedPercent;
  size_t diskUsed;
  double diskUsedPercent;

  unsigned int processes;
  double processesPercent;
  unsigned int threads;
  double threadsPercent;

  double loadAverage;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class ResourceUsageConverter
{
  static const char DELIMITER = ',';
public:
  ResourceUsageConverter(void);
  explicit ResourceUsageConverter(const ResourceUsage& rhs);
  explicit ResourceUsageConverter(const std::string& rhs);

  ResourceUsage toObject(void) const {return resourceUsage;}
  std::string toString(const char delimiter=DELIMITER);

  void fromObject(const ResourceUsage& rhs);
  void fromString(const std::string& rhs, const char delimiter=DELIMITER);
private:
  ResourceUsage resourceUsage;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class ResourceUsageSerializator : public IJsonSerializable
{
public:
  explicit ResourceUsageSerializator(ResourceUsage& resourceUsage_);
  ResourceUsageSerializator(ResourceUsage& resourceUsage_, const std::string& json);

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void clear(void);
private:
  ResourceUsage& resourceUsage;
  typedef IJsonSerializable inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE 

#endif // DRCE_RESOURCE_USAGE_HPP
