#include <utility>

#include "DRCEResultData.hpp"
#include "DRCEError.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
DRCEResultDataItem::DRCEResultDataItem(void)
: DRCEFilesList(), DRCESubtasksList <DRCEResultDataItem>(), errorCode(NO_ERROR), errorMessage(""), requestId(0), requestType(DRCETaskRequest::RequestType::rtSetTaskExecute),
  nodeHost(""), nodePort(""), state(0), pid(0), stdoutStream(""), stderrStream(""), nodeName(""), time(0), exitStatus(0), fields()
{
}
//-----------------------------------------------------------------------------
DRCEResultDataItem::DRCEResultDataItem(const DRCEResultDataItem& rhs)
: DRCEFilesList(), DRCESubtasksList <DRCEResultDataItem>(), errorCode(NO_ERROR), errorMessage(""), requestId(0), requestType(DRCETaskRequest::RequestType::rtSetTaskExecute),
  nodeHost(""), nodePort(""), state(0), pid(0), stdoutStream(""), stderrStream(""), nodeName(""), time(0), exitStatus(0), fields()
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
DRCEResultDataItem::DRCEResultDataItem(DRCEResultDataItem&& rhs)
: DRCEFilesList(), DRCESubtasksList <DRCEResultDataItem>(), errorCode(NO_ERROR), errorMessage(""), requestId(0), requestType(DRCETaskRequest::RequestType::rtSetTaskExecute),
  nodeHost(""), nodePort(""), state(0), pid(0), stdoutStream(""), stderrStream(""), nodeName(""), time(0), exitStatus(0), fields()
{
  (*this) = std::forward<DRCEResultDataItem>(rhs);
}
//-----------------------------------------------------------------------------
DRCEResultDataItem& DRCEResultDataItem::operator=(const DRCEResultDataItem& rhs)
{
  if (this!=&rhs)
  {
    errorCode = rhs.getErrorCode();
    errorMessage = rhs.getErrorMessage();
    requestId = rhs.getRequestId();
    requestType = rhs.getRequestType();
    nodeHost = rhs.getNodeHost();
    nodePort = rhs.getNodePort();
    state = rhs.getState();
    pid = rhs.getPid();
    stdoutStream = rhs.getStdoutStream();
    stderrStream = rhs.getStderrStream();
    nodeName = rhs.getNodeName();
    time = rhs.getTime();
    exitStatus = rhs.getExitStatus();
    fields = rhs.getFields();
    files = rhs.getFiles();
    (*dynamic_cast<DRCESubtasksList <DRCEResultDataItem>* >(this)) = static_cast<DRCESubtasksList <DRCEResultDataItem> >(rhs);
  }
  return *this;
}
//-----------------------------------------------------------------------------
DRCEResultDataItem& DRCEResultDataItem::operator=(DRCEResultDataItem&& rhs)
{
  if (this!=&rhs)
  {
    errorCode = std::move(rhs.getErrorCode());
    errorMessage = std::move(rhs.getErrorMessage());
    requestId = std::move(rhs.getRequestId());
    requestType = std::move(rhs.getRequestType());
    nodeHost = std::move(rhs.getNodeHost());
    nodePort = std::move(rhs.getNodePort());
    state = std::move(rhs.getState());
    pid = std::move(rhs.getPid());
    stdoutStream = std::move(rhs.getStdoutStream());
    stderrStream = std::move(rhs.getStderrStream());
    nodeName = std::move(rhs.getNodeName());
    time = std::move(rhs.getTime());
    exitStatus = std::move(rhs.getExitStatus());
    fields = std::move(rhs.getFields());
    files = std::move(rhs.getFiles());
    items = std::move(rhs.getSubtasks());
    (*dynamic_cast<DRCESubtasksList <DRCEResultDataItem>* >(this)) = std::move(static_cast<DRCESubtasksList <DRCEResultDataItem> >(rhs));
  }
  return *this;
}
//-----------------------------------------------------------------------------
void DRCEResultDataItem::clear(void)
{
  errorCode = NO_ERROR;
  errorMessage.clear();
  requestId = 0;
  requestType = DRCETaskRequest::RequestType::rtCheckTaskState;
  nodeHost.clear();
  nodePort.clear();
  state = 0;
  pid = 0;
  stdoutStream.clear();
  stderrStream.clear();
  nodeName.clear();
  time = 0;
  exitStatus = 0;
  fields.clear();
  files.clear();
  DRCESubtasksList <DRCEResultDataItem>::clear();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
DRCEResultData::DRCEResultData(void)
: items()
{
}
//-----------------------------------------------------------------------------
DRCEResultData::DRCEResultData(const DRCEResultData& rhs)
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
DRCEResultData::DRCEResultData(DRCEResultData&& rhs)
{
  (*this) = std::forward<DRCEResultData>(rhs);
}
//-----------------------------------------------------------------------------
DRCEResultData& DRCEResultData::operator=(const DRCEResultData& rhs)
{
  if (this!=&rhs)
  {
    items = rhs.getItems();
  }
  return *this;
}
//-----------------------------------------------------------------------------
DRCEResultData& DRCEResultData::operator=(DRCEResultData&& rhs)
{
  if (this!=&rhs)
  {
    items = std::move(rhs.getItems());
  }
  return *this;
}
//-----------------------------------------------------------------------------
void DRCEResultData::addDataItem(const DRCEResultDataItem& dataItem)
{
  items.push_back(dataItem);
}
//-----------------------------------------------------------------------------
void DRCEResultData::addDataItem(DRCEResultDataItem&& dataItem)
{
  items.push_back(std::move(std::forward<DRCEResultDataItem>(dataItem)));
}
//-----------------------------------------------------------------------------
size_t DRCEResultData::getItemsCount(void) const
{
  return items.size();
}
//-----------------------------------------------------------------------------
const DRCEResultDataItem& DRCEResultData::getDataItem(size_t index) const
{
  return items[index];
}
//-----------------------------------------------------------------------------
// cppcheck-suppress unusedFunction
void DRCEResultData::setDataItem(size_t index, const DRCEResultDataItem& dataItem)
{
  items[index] = dataItem;
}
//-----------------------------------------------------------------------------
void DRCEResultData::setDataItem(size_t index, DRCEResultDataItem&& dataItem)
{
  items[index] = std::forward<DRCEResultDataItem>(dataItem);
}
//-----------------------------------------------------------------------------
void DRCEResultData::clear(void)
{
  items.clear();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE
