/**
` * @file DRCEResultData.hpp
 *
 * Implementation class DRCEResultData of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_RESULT_DATA_HPP
#define DRCE_RESULT_DATA_HPP

#include <map>

#include "DRCEFilesList.hpp"
#include "DRCETaskRequest.hpp"
#include "DRCESubtasksList.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class DRCEResultDataItem : public DRCEFilesList, public DRCESubtasksList <DRCEResultDataItem>
{
public:
  DRCEResultDataItem(void);
  DRCEResultDataItem(const DRCEResultDataItem& rhs);
  DRCEResultDataItem(DRCEResultDataItem&& rhs);
  ~DRCEResultDataItem(void) {}

  DRCEResultDataItem& operator=(const DRCEResultDataItem& rhs);
  DRCEResultDataItem& operator=(DRCEResultDataItem&& rhs);

  void setErrorCode(unsigned int errorCode_) {errorCode=errorCode_;}
  unsigned int getErrorCode(void) const {return errorCode;}

  void setErrorMessage(const std::string& errorMessage_) {errorMessage=errorMessage_;}
  std::string getErrorMessage(void) const {return errorMessage;}

  void setRequestId(unsigned int requestId_) {requestId=requestId_;}
  unsigned int getRequestId(void) const {return requestId;}

  void setRequestType(DRCETaskRequest::RequestType requestType_) {requestType=requestType_;}
  DRCETaskRequest::RequestType getRequestType(void) const {return requestType;}

  void setNodeHost(const std::string& nodeHost_) {nodeHost=nodeHost_;}
  std::string getNodeHost(void) const {return nodeHost;}

  void setNodePort(const std::string& nodePort_) {nodePort=nodePort_;}
  std::string getNodePort(void) const {return nodePort;}

  void setState(unsigned int state_) {state=state_;}
  unsigned int getState(void) const {return state;}

  void setPid(unsigned int pid_) {pid=pid_;}
  unsigned int getPid(void) const {return pid;}

  void setStdoutStream(const std::string& stdoutStream_) {stdoutStream=stdoutStream_;}
  std::string getStdoutStream(void) const {return stdoutStream;}

  void setStderrStream(const std::string& stderrStream_) {stderrStream=stderrStream_;}
  std::string getStderrStream(void) const {return stderrStream;}

  void setNodeName(const std::string& nodeName_) {nodeName=nodeName_;}
  std::string getNodeName(void) const {return nodeName;}

  void setTime(unsigned int time_) {time=time_;}
  unsigned int getTime(void) const {return time;}

  void setExitStatus(unsigned int exitStatus_) {exitStatus=exitStatus_;}
  unsigned int getExitStatus(void) const {return exitStatus;}

  void setFields(const std::map<std::string, std::string>& fields_) {fields=fields_;}
  const std::map<std::string, std::string>& getFields(void) const {return fields;}

  void clear(void);
private:
  unsigned int errorCode;
  std::string errorMessage;
  unsigned int requestId;
  DRCETaskRequest::RequestType requestType;
  std::string nodeHost;
  std::string nodePort;
  unsigned int state;
  unsigned int pid;
  std::string stdoutStream;
  std::string stderrStream;
  std::string nodeName;
  unsigned int time;
  unsigned int exitStatus;
  std::map<std::string, std::string> fields;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class DRCEResultData
{
public:
  DRCEResultData(void);
  DRCEResultData(const DRCEResultData& rhs);
  DRCEResultData(DRCEResultData&& rhs);
  ~DRCEResultData(void) {}

  DRCEResultData& operator=(const DRCEResultData& rhs);
  DRCEResultData& operator=(DRCEResultData&& rhs);

  void addDataItem(const DRCEResultDataItem& dataItem);
  void addDataItem(DRCEResultDataItem&& dataItem);

  const std::vector<DRCEResultDataItem>& getItems(void) const {return items;}

  size_t getItemsCount(void) const;
  const DRCEResultDataItem& getDataItem(size_t index) const;
  void setDataItem(size_t index, const DRCEResultDataItem& dataItem);
  void setDataItem(size_t index, DRCEResultDataItem&& dataItem);

  void clear(void);
private:
  std::vector<DRCEResultDataItem> items;

  typedef DRCEFilesList inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE

#endif // DRCE_RESULT_DATA_HPP
