/**
 * @file DRCEResultDataSerializator.hpp
 *
 * Implementation class DRCEResultDataSerializator of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_RESULT_DATA_SERIALIZATOR_HPP
#define DRCE_RESULT_DATA_SERIALIZATOR_HPP

#include <Poco/JSON/Object.h>
#include <Poco/JSON/Array.h>

#include "DRCEJsonMessageConst.hpp"
#include "JsonSerializable.hpp"
#include "DRCEResultData.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class DRCEResultDataSerializator : public IJsonSerializable
{
public:
  explicit DRCEResultDataSerializator(DRCEResultData& resultData);
  ~DRCEResultDataSerializator(void) {}

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

private:
  void packToObject(Poco::JSON::Object& obj, DRCEResultDataItem& resultDataItem) throw (Poco::Exception);
  void packToArray(Poco::JSON::Array& arr, DRCEResultDataItem& resultDataItem) throw (Poco::Exception);
  void unpackFromObject(Poco::JSON::Object& obj, DRCEResultDataItem& resultDataItem) throw (Poco::Exception);
  void unpackFromArray(Poco::JSON::Array& arr, DRCEResultDataItem& resultDataItem) throw (Poco::Exception);
private:
  DRCEResultData& resultData;

  typedef IJsonSerializable inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE

#endif // DRCE_RESULT_DATA_SERIALIZATOR_HPP
