#include <utility>

#include "DRCESessionOptions.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
SessionOptions::SessionOptions(void)
: sessionType(SessionType::stHostShell), port(0), userName(""), userPassword(""), shellName(""),
  environments(), homeDir(""), timeout(0), tmode(ThreadMode::tmSync), timeMax(0), cleanup(CleanupFlag::cfNotDelete)
{
}
//-----------------------------------------------------------------------------
SessionOptions::SessionOptions(const SessionOptions& rhs)
: sessionType(SessionType::stHostShell), port(0), userName(""), userPassword(""), shellName(""),
  environments(), homeDir(""), timeout(0), tmode(ThreadMode::tmSync), timeMax(0), cleanup(CleanupFlag::cfNotDelete)
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
SessionOptions::SessionOptions(SessionOptions&& rhs)
: sessionType(SessionType::stHostShell), port(0), userName(""), userPassword(""), shellName(""),
  environments(), homeDir(""), timeout(0), tmode(ThreadMode::tmSync), timeMax(0), cleanup(CleanupFlag::cfNotDelete)
{
  (*this) = std::forward<SessionOptions>(rhs);
}
//-----------------------------------------------------------------------------
SessionOptions& SessionOptions::operator=(const SessionOptions& rhs)
{
  if (this!=&rhs)
  {
    sessionType = rhs.sessionType;
    port = rhs.port;
    userName = rhs.userName;
    userPassword = rhs.userPassword;
    shellName = rhs.shellName;
    environments = rhs.environments;
    homeDir = rhs.homeDir;
    timeout = rhs.timeout;
    tmode = rhs.tmode;
    timeMax = rhs.timeMax;
    cleanup = rhs.cleanup;
  }
  return *this;
}
//-----------------------------------------------------------------------------
SessionOptions& SessionOptions::operator=(SessionOptions&& rhs)
{
if (this!=&rhs)
  {
    sessionType = std::move(rhs.sessionType);
    port = std::move(rhs.port);
    userName = std::move(rhs.userName);
    userPassword = std::move(rhs.userPassword);
    shellName = std::move(rhs.shellName);
    environments = std::move(rhs.environments);
    homeDir = std::move(rhs.homeDir);
    timeout = std::move(rhs.timeout);
    tmode = std::move(rhs.tmode);
    timeMax = std::move(rhs.timeMax);
    cleanup = std::move(rhs.cleanup);
  }
  return *this;
}
//-----------------------------------------------------------------------------
void SessionOptions::clear(void)
{
  sessionType = SessionType::stHostShell;
  port = 0;
  userName.clear();
  userPassword.clear();
  shellName.clear();
  environments.clear();
  homeDir.clear();
  timeout = 0;
  tmode = ThreadMode::tmSync;
  timeMax = 0;
  cleanup = CleanupFlag::cfNotDelete;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE 
