/**
 * @file DRCESessionOptions.hpp
 *
 * Implementation structure SessionOptions of a Distributed Remote Command Execution functional object in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_SESSION_OPTIONS_HPP
#define DRCE_SESSION_OPTIONS_HPP

#include <iostream>
#include <vector>

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
struct SessionOptions
{
  enum class SessionType
  {
    stHostShell=0,
    stSsh=1
  };

  enum class ThreadMode
  {
    tmSync=1,
    tmAsync=2
  };

  enum class CleanupFlag
  {
    cfNotDelete=0,
    cfDelete=1
  };

  SessionOptions(void);
  SessionOptions(const SessionOptions& rhs);
  SessionOptions(SessionOptions&& rhs);

  SessionOptions& operator=(const SessionOptions& rhs);
  SessionOptions& operator=(SessionOptions&& rhs);

  void clear(void);

  SessionType sessionType;
  unsigned int port;
  std::string userName;
  std::string userPassword;
  std::string shellName;
  std::vector<std::pair<std::string, std::string> > environments;
  std::string homeDir;
  unsigned int timeout;
  ThreadMode tmode;
  unsigned int timeMax;
  CleanupFlag cleanup;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE 

#endif // DRCE_SESSION_OPTIONS_HPP
