/**
 * @file DRCEStatVariables.hpp
 *
 * Implementation structure DRCEStatVariables of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_STAT_VARIABLES_HPP
#define DRCE_STAT_VARIABLES_HPP

#include <iostream>

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
template <class T>
struct StatVariableData
{
  StatVariableData(void);
  StatVariableData(const StatVariableData& rhs);
  StatVariableData(StatVariableData&& rhs);

  StatVariableData& operator=(const StatVariableData& rhs);
  StatVariableData& operator=(StatVariableData&& rhs);

  void clear(void);

  T min; // minimum value
  T avg; // average value
  T max; // maximum value
};
//-----------------------------------------------------------------------------
struct StatVariables
{
  StatVariables(void);
  StatVariables(const StatVariables& rhs);
  StatVariables(StatVariables&& rhs);

  StatVariables& operator=(const StatVariables& rhs);
  StatVariables& operator=(StatVariables&& rhs);

  void clear(void);

  StatVariableData <double> timeAsyncTask;              // time working of async task
  StatVariableData <double> timeSyncTask;               // time working of sync task
  StatVariableData <size_t> sizeInputBufferSyncTasks;   // size of input buffer stdin for sync tasks
  StatVariableData <size_t> sizeOutputBufferSyncTasks;  // size of output buffer stdout for sync tasks
  StatVariableData <size_t> sizeInputBufferAsyncTasks;  // size of input buffer stdin for async tasks
  StatVariableData <size_t> sizeOutputBufferAsyncTasks; // size of output buffer stdout for async tasks
  StatVariableData <double> cpuUsageAsyncTasks;         // cpu usage only async tasks
  StatVariableData <size_t> memoryUsageAsyncTasks;      // memory usage only async tasks
  StatVariableData <double> countSyncTasksForMinute;    // count of sync tasks for minute
  StatVariableData <double> countAsyncTasksForMinute;   // count of async tasks for minute

  size_t countSyncTasks;        // full count sync tasks for all time
  size_t countAsyncTasks;       // full count async tasks for all time
  size_t countSyncTasksFail;    // full count sync tasks returned error for all time
  size_t countAsyncTasksFail;   // full count async tasks returned error for all time
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_STAT_VARIABLES_HPP
