#include "DRCEStatVariablesConverter.hpp"
#include "DRCEJsonMessageConst.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
StatVariablesConverter::StatVariablesConverter(void)
: statVariables()
{
}
//-----------------------------------------------------------------------------
StatVariablesConverter::StatVariablesConverter(const StatVariables& rhs)
: statVariables(rhs)
{
}
//-----------------------------------------------------------------------------
StatVariablesConverter::StatVariablesConverter(const Dictionary& rhs)
: statVariables()
{
  fromDictionary(rhs);
}
//-----------------------------------------------------------------------------
template <typename T> void StatVariablesConverter::toDictionary(const std::string& key, const StatVariableData<T>& data, Dictionary& dic)
{
  dic[key+HCE::drce::drce_json_message_const::STAT_DELIMITER+HCE::drce::drce_json_message_const::MIN] = data.min;
  dic[key+HCE::drce::drce_json_message_const::STAT_DELIMITER+HCE::drce::drce_json_message_const::AVG] = data.avg;
  dic[key+HCE::drce::drce_json_message_const::STAT_DELIMITER+HCE::drce::drce_json_message_const::MAX] = data.max;
}
//-----------------------------------------------------------------------------
template <typename T> void StatVariablesConverter::fromDictionary(const std::string& key, StatVariableData<T>& data, const Dictionary& dic)
{
  Dictionary dictionary(dic);
  Poco::Dynamic::Var tmp = dictionary[key+HCE::drce::drce_json_message_const::STAT_DELIMITER+HCE::drce::drce_json_message_const::MIN];
  if (tmp.isInteger())
    data.min = tmp.convert<T>();

  tmp = dictionary[key+HCE::drce::drce_json_message_const::STAT_DELIMITER+HCE::drce::drce_json_message_const::AVG];
  if (tmp.isInteger())
    data.avg = tmp.convert<T>();

  tmp = dictionary[key+HCE::drce::drce_json_message_const::STAT_DELIMITER+HCE::drce::drce_json_message_const::MAX];
  if (tmp.isInteger())
    data.max = tmp.convert<T>();
}
//-----------------------------------------------------------------------------
void StatVariablesConverter::toDictionary(const std::string& key, const size_t& value, Dictionary& dic)
{
  dic[key] = value;
}
//-----------------------------------------------------------------------------
void StatVariablesConverter::fromDictionary(const std::string& key, size_t& value, const Dictionary& dic)
{
  Dictionary dictionary(dic);
  Poco::Dynamic::Var tmp = dictionary[key];
  if (tmp.isInteger())
    value = tmp.convert<size_t>();
}
//-----------------------------------------------------------------------------
StatVariablesConverter::Dictionary StatVariablesConverter::toDictionary(void)
{
  Dictionary dictionary;
  toDictionary(HCE::drce::drce_json_message_const::TIME_ASYNC_TASK, statVariables.timeAsyncTask, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::TIME_SYNC_TASK, statVariables.timeSyncTask, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::SIZE_INPUT_BUFFER_SYNC_TASKS, statVariables.sizeInputBufferSyncTasks, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::SIZE_OUTPUT_BUFFER_SYNC_TASKS, statVariables.sizeOutputBufferSyncTasks, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::SIZE_INPUT_BUFFER_ASYNC_TASKS, statVariables.sizeInputBufferAsyncTasks, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::SIZE_OUTPUT_BUFFER_ASYNC_TASKS, statVariables.sizeOutputBufferAsyncTasks, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::CPU_USAGE_ASYNC_TASKS, statVariables.cpuUsageAsyncTasks, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::MEMORY_USAGE_ASYNC_TASKS, statVariables.memoryUsageAsyncTasks, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::COUNT_SYNC_TASKS_FOR_MINUTE, statVariables.countSyncTasksForMinute, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::COUNT_ASYNC_TASKS_FOR_MINUTE, statVariables.countAsyncTasksForMinute, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::COUNT_SYNC_TASKS, statVariables.countSyncTasks, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::COUNT_ASYNC_TASKS, statVariables.countAsyncTasks, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::COUNT_SYNC_TASKS_FAIL, statVariables.countSyncTasksFail, dictionary);
  toDictionary(HCE::drce::drce_json_message_const::COUNT_ASYNC_TASKS_FAIL, statVariables.countAsyncTasksFail, dictionary);

  return dictionary;
}
//-----------------------------------------------------------------------------
void StatVariablesConverter::fromObject(const StatVariables& rhs)
{
  statVariables = rhs;
}
//-----------------------------------------------------------------------------
void StatVariablesConverter::fromDictionary(const Dictionary& rhs)
{
  fromDictionary(HCE::drce::drce_json_message_const::TIME_ASYNC_TASK, statVariables.timeAsyncTask, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::TIME_SYNC_TASK, statVariables.timeSyncTask, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::SIZE_INPUT_BUFFER_SYNC_TASKS, statVariables.sizeInputBufferSyncTasks, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::SIZE_OUTPUT_BUFFER_SYNC_TASKS, statVariables.sizeOutputBufferSyncTasks, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::SIZE_INPUT_BUFFER_ASYNC_TASKS, statVariables.sizeInputBufferAsyncTasks, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::SIZE_OUTPUT_BUFFER_ASYNC_TASKS, statVariables.sizeOutputBufferAsyncTasks, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::CPU_USAGE_ASYNC_TASKS, statVariables.cpuUsageAsyncTasks, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::MEMORY_USAGE_ASYNC_TASKS, statVariables.memoryUsageAsyncTasks, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::COUNT_SYNC_TASKS_FOR_MINUTE, statVariables.countSyncTasksForMinute, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::COUNT_ASYNC_TASKS_FOR_MINUTE, statVariables.countAsyncTasksForMinute, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::COUNT_SYNC_TASKS, statVariables.countSyncTasks, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::COUNT_ASYNC_TASKS, statVariables.countAsyncTasks, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::COUNT_SYNC_TASKS_FAIL, statVariables.countSyncTasksFail, rhs);
  fromDictionary(HCE::drce::drce_json_message_const::COUNT_ASYNC_TASKS_FAIL, statVariables.countAsyncTasksFail, rhs);
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE 
