/**
 * @file DRCEStatVariablesConverter.hpp
 *
 * Implementation class StatVariablesConverter of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_STAT_VARIABLES_CONVERTER_HPP
#define DRCE_STAT_VARIABLES_CONVERTER_HPP

#include <map>
#include <Poco/Dynamic/Var.h>

#include "DRCEStatVariables.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class StatVariablesConverter
{
public:
  typedef std::map<std::string, Poco::Dynamic::Var> Dictionary;

  StatVariablesConverter(void);
  explicit StatVariablesConverter(const StatVariables& rhs);
  explicit StatVariablesConverter(const Dictionary& rhs);

  StatVariables toObject(void) const {return statVariables;}
  Dictionary toDictionary(void);

  void fromObject(const StatVariables& rhs);
  void fromDictionary(const Dictionary& rhs);
private:
  template <typename T> void toDictionary(const std::string& key, const StatVariableData<T>& data, Dictionary& dic);
  template <typename T> void fromDictionary(const std::string& key, StatVariableData<T>& data, const Dictionary& dic);

  static void toDictionary(const std::string& key, const size_t& value, Dictionary& dic);
  static void fromDictionary(const std::string& key, size_t& value, const Dictionary& dic);
private:
  StatVariables statVariables;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE 

#endif // DRCE_STAT_VARIABLES_CONVERTER_HPP
