/**
 * @file DRCESubtasksList.hpp
 *
 * Implementation class DRCESubtasksList of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_SUBTASKS_LIST_HPP
#define DRCE_SUBTASKS_LIST_HPP

#include "DRCECommonList.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
template <class _Subtask>
class DRCESubtasksList : public DRCECommonList <_Subtask>
{
public:
  DRCESubtasksList(void) : inherited() {}
  DRCESubtasksList(const DRCESubtasksList& rhs);
  DRCESubtasksList(DRCESubtasksList&& rhs);
  virtual ~DRCESubtasksList(void) {}

  DRCESubtasksList& operator=(const DRCESubtasksList& rhs);
  DRCESubtasksList& operator=(DRCESubtasksList&& rhs);

  void addSubtaskItem(const _Subtask& item);
  void addSubtaskItem(_Subtask&& item);

  const std::vector<_Subtask>& getSubtasks(void) const;

  size_t getSubtasksCount(void) const;
  const _Subtask& getSubtaskItem(size_t index) const;
  void setSubtaskItem(size_t index, const _Subtask& item);
  void setSubtaskItem(size_t index, _Subtask&& item);

  void clear(void);
private:
  typedef DRCECommonList<_Subtask> inherited;
};
//-----------------------------------------------------------------------------
template <class _Subtask> DRCESubtasksList<_Subtask>::DRCESubtasksList(const DRCESubtasksList& rhs)
: inherited()
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
template <class _Subtask> DRCESubtasksList<_Subtask>::DRCESubtasksList(DRCESubtasksList&& rhs)
: inherited()
{
  (*this) = std::forward<DRCESubtasksList>(rhs);
}
//-----------------------------------------------------------------------------
template <class _Subtask> DRCESubtasksList<_Subtask>& DRCESubtasksList<_Subtask>::operator=(const DRCESubtasksList& rhs)
{
  if (this!=&rhs)
  {
    inherited::items = rhs.getSubtasks();
  }
  return *this;
}
//-----------------------------------------------------------------------------
template <class _Subtask> DRCESubtasksList<_Subtask>& DRCESubtasksList<_Subtask>::operator=(DRCESubtasksList&& rhs)
{
  if (this!=&rhs)
  {
    inherited::items = std::move(rhs.getSubtasks());
  }
  return *this;
}
//-----------------------------------------------------------------------------
template <class _Subtask> void DRCESubtasksList<_Subtask>::addSubtaskItem(const _Subtask& item)
{
  inherited::addItem(item);
}
//-----------------------------------------------------------------------------
template <class _Subtask> void DRCESubtasksList<_Subtask>::addSubtaskItem(_Subtask&& item)
{
  inherited::addItem(std::forward<_Subtask>(item));
}
//-----------------------------------------------------------------------------
template <class _Subtask> const std::vector<_Subtask>& DRCESubtasksList<_Subtask>::getSubtasks(void) const
{
  return inherited::getItems();
}
//-----------------------------------------------------------------------------
template <class _Subtask> size_t DRCESubtasksList<_Subtask>::getSubtasksCount(void) const
{
  return inherited::getItemsCount();
}
//-----------------------------------------------------------------------------
template <class _Subtask> const _Subtask& DRCESubtasksList<_Subtask>::getSubtaskItem(size_t index) const
{
  return inherited::getItem(index);
}
//-----------------------------------------------------------------------------
template <class _Subtask> void DRCESubtasksList<_Subtask>::setSubtaskItem(size_t index, const _Subtask& item)
{
  inherited::setItem(index, item);
}
//-----------------------------------------------------------------------------
template <class _Subtask> void DRCESubtasksList<_Subtask>::setSubtaskItem(size_t index, _Subtask&& item)
{
  inherited::setItem(index, std::forward<_Subtask>(item));
}
//-----------------------------------------------------------------------------
template <class _Subtask> void DRCESubtasksList<_Subtask>::clear(void)
{
  inherited::clear();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE

#endif // DRCE_SUBTASKS_LIST_HPP

