#include "DRCEMessageConst.hpp"
#include "DRCEError.hpp"
#include "DRCETaskShellHost.hpp"
#include "DRCETaskSsh.hpp"
#include "DRCETaskFactory.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
DRCECommonTask* DRCETaskFactory::create(SessionOptions::SessionType sessionType,
    const std::string& taskName_,
    DRCENodeOptions& nodeOptions_,
    CustomMessage& message_,
    const DRCETaskRequestSetExecute& taskRequestSetExecute_,
    DRCEAsyncTasksQueue& asyncTasksQueue_,
    DRCEResourceMonitor& resourceMonitor_,
    DRCEInputJsonMessage& inputJsonMessage_,
    unsigned int parentTaskId_)
{
  DRCECommonTask* pCommonTask = nullptr;
  switch(sessionType)
  {
    case SessionOptions::SessionType::stHostShell:
      {
        pCommonTask = new DRCETaskShellHost(taskName_, nodeOptions_, message_, taskRequestSetExecute_, asyncTasksQueue_, resourceMonitor_, inputJsonMessage_, parentTaskId_);
      }
      break;
    case SessionOptions::SessionType::stSsh:
      {
        pCommonTask = new DRCETaskSsh(taskName_, nodeOptions_, message_, taskRequestSetExecute_, asyncTasksQueue_, resourceMonitor_, inputJsonMessage_, parentTaskId_);
      }
      break;
    default: throw Poco::Exception(message_(message_const::WRONG_TYPE_REMOTE_EXCUTION), ERROR_CREATE_NEW_TASK);
  }
  return pCommonTask;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 
