/**
 * @file DRCETaskRequest.hpp
 *
 * Base interface DRCETaskRequest need to a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_TASK_REQUEST_HPP
#define DRCE_TASK_REQUEST_HPP

#include "JsonSerializable.hpp"
#include "DRCEJsonMessageConst.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class DRCETaskRequest : public IJsonSerializable
{
public:
  enum TaskState
  {
    FINISHED=0,           // 0 – finished,
    IN_PROGRESS=1,        // 1 – in progress,
    SET_AS_NEW=2,         // 2 – set as new,
    NOT_FOUND=3,          // 3 – not found,
    TERMINATED=4,         // 4 – terminated before normal finish,
    CRASHED=5,            // 5 – crashed
    NOT_SET_AS_NEW=6,     // 6 – not set as new cause to limits of system resources (CPU, RAM, Disk, threads, processes, etc)
    UNDEFINED=7,          // 7 – undefined
    QUEUED_TO_RUN=8,      // 8 – queued to run
    DELETED=9,            // 9 – deleted
    BUSY=10,              // 10 – busy
    TERMINATED_BY_TTL=11  // 11 - task was terminated by TTL reason (internal event)
  };

  enum RequestType
  {
    rtSetTaskExecute=0,
    rtCheckTaskState=1,
    rtTerminateTask=2,
    rtGetTaskData=3,
    rtDeleteTaskData=4,
    rtTaskStateNotification=100,
    rtResourceMonitorNotification=101
  };

  explicit DRCETaskRequest(DRCETaskRequest::RequestType requestType_=DRCETaskRequest::RequestType::rtSetTaskExecute)
  : IJsonSerializable(), taskId(0), parentTaskId(0), taskState(TaskState::UNDEFINED), requestType(requestType_) {}
  virtual ~DRCETaskRequest(void) {}

  virtual bool serialize(std::string& json) {return false; }
  virtual bool unserialize(const std::string& json) {return false;}

  void setTaskId(unsigned int taskId_) {taskId=taskId_;}
  unsigned int getTaskId(void) const {return taskId;}

  void setParentTaskId(unsigned int parentTaskId_) {parentTaskId=parentTaskId_;}
  unsigned int getParentTaskId(void) const {return parentTaskId;}

  void setTaskState(TaskState taskState_) {taskState=taskState_;}
  TaskState getTaskState(void) const {return taskState;}

  void setRequestType(RequestType requestType_) {requestType=requestType_;}
  RequestType getRequestType(void) const {return requestType;}

protected:
  unsigned int taskId;
  unsigned int parentTaskId;
  TaskState taskState;
  RequestType requestType;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE

#endif // DRCE_TASK_REQUEST_HPP
