/**
 * @file DRCETaskRequestCheckState.hpp
 *
 * Implementation class DRCETaskRequestCheckState of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_TASK_REQUEST_CHECK_STATE_HPP
#define DRCE_TASK_REQUEST_CHECK_STATE_HPP

#include <string>

#include "DRCETaskRequest.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class DRCETaskRequestCheckState : public DRCETaskRequest
{
public:
  enum CheckType
  {
    ctUninitialized=0,
    ctSimple=1,
    ctExtended=2
  };

  explicit DRCETaskRequestCheckState(const std::string& json="");
  virtual ~DRCETaskRequestCheckState(void) {}

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void setCheckType(CheckType checkType_) {checkType=checkType_;}
  CheckType getCheckType(void) const {return checkType;}

  void clear(void);
private:
  CheckType checkType;
private:
  friend std::istream& operator>>(std::istream& is, DRCETaskRequestCheckState& rhs);
  friend std::ostream& operator<<(std::ostream& os, const DRCETaskRequestCheckState& rhs);
  typedef DRCETaskRequest inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_TASK_REQUEST_CHECK_STATE_HPP
