#include "DRCEError.hpp"
#include "DRCEFileStream.hpp"
#include "DRCETaskRequestExecutor.hpp"
#include "DRCEOutputJsonMessage.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
DRCETaskRequestExecutor::DRCETaskRequestExecutor(DRCEAsyncTasksQueue& asyncTaskQueue_,
    DRCENodeOptions& nodeOptions_,CustomMessage& message_, DRCEResourceMonitor& resourceMonitor_)
: asyncTaskQueue(asyncTaskQueue_), nodeOptions(nodeOptions_), message(message_), resourceMonitor(resourceMonitor_)
{
}
//-----------------------------------------------------------------------------
DRCEResultDataItem DRCETaskRequestExecutor::getResultDataItem(const DRCETaskRequest& taskRequest, DRCETaskRequest::TaskState state)
{
  return getResultDataItem(taskRequest, nodeOptions, state);
}
//-----------------------------------------------------------------------------
DRCEResultDataItem DRCETaskRequestExecutor::getResultDataItem(const DRCETaskRequest& taskRequest, DRCENodeOptions& nodeOptions, DRCETaskRequest::TaskState state)
{
  return getResultDataItem(taskRequest.getTaskId(), taskRequest.getRequestType(), nodeOptions, state);
}
//-----------------------------------------------------------------------------
DRCEResultDataItem DRCETaskRequestExecutor::getResultDataItem(unsigned int taskId, DRCETaskRequest::RequestType requestType,
    DRCENodeOptions& nodeOptions, DRCETaskRequest::TaskState state)
{
  DRCEResultDataItem resultDataItem;
  resultDataItem.setRequestId(taskId);
  resultDataItem.setRequestType(requestType);
  resultDataItem.setNodeName(nodeOptions.getNodeName());
  resultDataItem.setNodeHost(nodeOptions.getNodeHost());
  resultDataItem.setNodePort(nodeOptions.getNodePort());
  resultDataItem.setState(static_cast<unsigned int>(state));
  return resultDataItem;
}
//-----------------------------------------------------------------------------
DRCEResultDataItem DRCETaskRequestExecutor::getResultDataItem(const DRCEInputJsonMessage& inputJsonMessage, DRCETaskRequest::TaskState state)
{
  return getResultDataItem(inputJsonMessage, nodeOptions, state);
}
//-----------------------------------------------------------------------------
DRCEResultDataItem DRCETaskRequestExecutor::getResultDataItem(const DRCEInputJsonMessage& inputJsonMessage, const DRCENodeOptions& nodeOptions, DRCETaskRequest::TaskState state)
{
  DRCEResultDataItem resultDataItem;
  resultDataItem.setRequestId(inputJsonMessage.getRequestId());
  resultDataItem.setRequestType(inputJsonMessage.getRequestType());
  resultDataItem.setNodeName(nodeOptions.getNodeName());
  resultDataItem.setNodeHost(nodeOptions.getNodeHost());
  resultDataItem.setNodePort(nodeOptions.getNodePort());
  resultDataItem.setState(static_cast<unsigned int>(state));

  getSubtasks(inputJsonMessage, nodeOptions, resultDataItem);

  return resultDataItem;
}
//-----------------------------------------------------------------------------
void DRCETaskRequestExecutor::getSubtasks(const DRCEInputJsonMessage& inputJsonMessage, const DRCENodeOptions& nodeOptions, DRCEResultDataItem& resultDataItem)
{
  for (size_t i=0;i<inputJsonMessage.getSubtasksCount();++i)
  {
    DRCEResultDataItem subtasksResultItem;
    subtasksResultItem.setRequestId(inputJsonMessage.getSubtaskItem(i).getRequestId());
    subtasksResultItem.setRequestType(inputJsonMessage.getRequestType());
    subtasksResultItem.setState(static_cast<unsigned int>(DRCETaskRequest::TaskState::QUEUED_TO_RUN));
    subtasksResultItem.setNodeName(nodeOptions.getNodeName());
    subtasksResultItem.setNodeHost(nodeOptions.getNodeHost());
    subtasksResultItem.setNodePort(nodeOptions.getNodePort());

    getSubtasks(inputJsonMessage.getSubtaskItem(i), nodeOptions, subtasksResultItem);

    resultDataItem.addSubtaskItem(std::forward<DRCEResultDataItem>(subtasksResultItem));
  }
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE
