/**
 * @file DRCETaskRequestExecutor.hpp
 *
 * Implementation class DRCETaskRequestSetExecute of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_TASK_REQUEST_EXECUTOR_HPP
#define DRCE_TASK_REQUEST_EXECUTOR_HPP

#include <vector>

#include "DRCEAsyncTasksQueue.hpp"
#include "DRCEResourceMonitor.hpp"
#include "DRCEStatVariables.hpp"
#include "DRCEResultData.hpp"
#include "DRCETaskRequest.hpp"
#include "DRCENodeOptions.hpp"
#include "CustomMessage.hpp"
#include "DRCEInputJsonMessage.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class DRCETaskRequestExecutor
{
public:
  DRCETaskRequestExecutor(DRCEAsyncTasksQueue& asyncTaskQueue_, DRCENodeOptions& nodeOptions_, CustomMessage& message_, DRCEResourceMonitor& resourceMonitor_);
  virtual ~DRCETaskRequestExecutor(void) {}

  virtual DRCEResultDataItem execute(DRCETaskRequest* pTaskRequest, DRCEInputJsonMessage& inputJsonMessage) = 0;

  DRCEResultDataItem getResultDataItem(const DRCETaskRequest& taskRequest, DRCETaskRequest::TaskState state);
  static DRCEResultDataItem getResultDataItem(const DRCETaskRequest& taskRequest, DRCENodeOptions& nodeOptions, DRCETaskRequest::TaskState state);
  static DRCEResultDataItem getResultDataItem(unsigned int taskId, DRCETaskRequest::RequestType requestType, DRCENodeOptions& nodeOptions, DRCETaskRequest::TaskState state);

  DRCEResultDataItem getResultDataItem(const DRCEInputJsonMessage& inputJsonMessage, DRCETaskRequest::TaskState state);
  static DRCEResultDataItem getResultDataItem(const DRCEInputJsonMessage& inputJsonMessage, const DRCENodeOptions& nodeOptions, DRCETaskRequest::TaskState state);
protected:
  static void getSubtasks(const DRCEInputJsonMessage& inputJsonMessage, const DRCENodeOptions& nodeOptions, DRCEResultDataItem& resultDataItem);
protected:
  DRCEAsyncTasksQueue& asyncTaskQueue;
  DRCENodeOptions& nodeOptions;
  CustomMessage& message;
  DRCEResourceMonitor& resourceMonitor;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_TASK_REQUEST_EXECUTOR_HPP
