#include "DRCETaskRequestExecutorFactory.hpp"
#include "DRCETaskRequestSetExecuteExecutor.hpp"
#include "DRCETaskRequestCheckStateExecutor.hpp"
#include "DRCETaskRequestTerminateExecutor.hpp"
#include "DRCETaskRequestGetDataExecutor.hpp"
#include "DRCETaskRequestDeleteDataExecutor.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
Poco::SharedPtr<DRCETaskRequestExecutor> DRCETaskRequestExecutorFactory::create(DRCETaskRequest::RequestType requestType,
    DRCEAsyncTasksQueue& asyncTaskQueue_, DRCENodeOptions& nodeOptions_, CustomMessage& message_, DRCEResourceMonitor& resourceMonitor_)
{
  Poco::SharedPtr<DRCETaskRequestExecutor> pTaskRequestExecutor = nullptr;
  switch(requestType)
  {
    case DRCETaskRequest::RequestType::rtSetTaskExecute: pTaskRequestExecutor = new DRCETaskRequestSetExecuteExecutor(asyncTaskQueue_, nodeOptions_, message_, resourceMonitor_);
      break;
    case DRCETaskRequest::RequestType::rtCheckTaskState: pTaskRequestExecutor = new DRCETaskRequestCheckStateExecutor(asyncTaskQueue_, nodeOptions_, message_, resourceMonitor_);
      break;
    case DRCETaskRequest::RequestType::rtTerminateTask: pTaskRequestExecutor = new DRCETaskRequestTerminateExecutor(asyncTaskQueue_, nodeOptions_, message_, resourceMonitor_);
      break;
    case DRCETaskRequest::RequestType::rtGetTaskData: pTaskRequestExecutor = new DRCETaskRequestGetDataExecutor(asyncTaskQueue_, nodeOptions_, message_, resourceMonitor_);
      break;
    case DRCETaskRequest::RequestType::rtDeleteTaskData: pTaskRequestExecutor = new DRCETaskRequestDeleteDataExecutor(asyncTaskQueue_, nodeOptions_, message_, resourceMonitor_);
      break;
    default: throw Poco::Exception(message_(message_const::WRONG_TYPE_TASK_REQUEST), ERROR_BAD_TYPE_TASK_REQUEST);
  }
  return pTaskRequestExecutor;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce
} // end namespace HCE

