#include "DRCEMessageConst.hpp"
#include "DRCEError.hpp"
#include "DRCETaskRequestFactory.hpp"
#include "DRCETaskRequestSetExecute.hpp"
#include "DRCETaskRequestCheckState.hpp"
#include "DRCETaskRequestTerminate.hpp"
#include "DRCETaskRequestGetData.hpp"
#include "DRCETaskRequestDeleteData.hpp"


namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
Poco::SharedPtr<DRCETaskRequest> DRCETaskRequestFactory::create(DRCETaskRequest::RequestType requestType,
    CustomMessage& message_) throw (Poco::Exception)
{
  Poco::SharedPtr<DRCETaskRequest> pTaskRequest = create(requestType);
  if (pTaskRequest.isNull())
    throw Poco::Exception(message_(message_const::WRONG_TYPE_TASK_REQUEST), ERROR_BAD_TYPE_TASK_REQUEST);

  return pTaskRequest;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
Poco::SharedPtr<DRCETaskRequest> DRCETaskRequestFactory::create(DRCETaskRequest::RequestType requestType)
{
  Poco::SharedPtr<DRCETaskRequest> pTaskRequest = nullptr;
  switch(requestType)
  {
    case DRCETaskRequest::RequestType::rtSetTaskExecute: pTaskRequest = new DRCETaskRequestSetExecute();
      break;
    case DRCETaskRequest::RequestType::rtCheckTaskState: pTaskRequest = new DRCETaskRequestCheckState();
      break;
    case DRCETaskRequest::RequestType::rtTerminateTask: pTaskRequest = new DRCETaskRequestTerminate();
      break;
    case DRCETaskRequest::RequestType::rtGetTaskData: pTaskRequest = new DRCETaskRequestGetData();
      break;
    case DRCETaskRequest::RequestType::rtDeleteTaskData: pTaskRequest = new DRCETaskRequestDeleteData();
      break;
    default:;
  }
  if (!pTaskRequest.isNull())
    pTaskRequest->setRequestType(requestType);

  return pTaskRequest;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 
