#include <Poco/JSON/Object.h>
#include <Poco/JSON/Array.h>
#include <Poco/JSON/JSON.h>
#include <Poco/JSON/Stringifier.h>
#include <Poco/JSON/Parser.h>
#include <Poco/JSON/JSONException.h>
#include <Poco/Dynamic/Var.h>
#include <utility>

#include "DRCEError.hpp"
#include "DRCETaskRequestGetData.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
DRCETaskRequestGetData::DRCETaskRequestGetData(const std::string& json)
: inherited(DRCETaskRequest::RequestType::rtGetTaskData), fetchType(FetchType::ftUninitialized)
{
  if (!json.empty())
    unserialize(json);
}
//-----------------------------------------------------------------------------
bool DRCETaskRequestGetData::serialize(std::string& json)
{
  errorCode = NO_ERROR;
  errorMsg.clear();
  try
  {
    Poco::JSON::Object::Ptr obj = new Poco::JSON::Object();

    obj->set(drce_json_message_const::fetchType, static_cast<unsigned int>(fetchType));

    std::stringstream ostr;
    Poco::JSON::Stringifier::stringify(obj, ostr);
    json = ostr.str();

    _isError = false;
  }
  catch(std::exception& e)
  {
    _isError = true;
    errorMsg = e.what();
    errorCode = ERROR_SERIALIZE;
  }
  return !_isError;}
//-----------------------------------------------------------------------------
bool DRCETaskRequestGetData::unserialize(const std::string& json)
{
  clear();
  errorCode = NO_ERROR;
  errorMsg.clear();

  try
  {
    Poco::JSON::Parser parser;
    Poco::Dynamic::Var res = parser.parse(json);
    Poco::JSON::Object::Ptr obj = res.extract<Poco::JSON::Object::Ptr>();

    Poco::Dynamic::Var tmp = obj->get(drce_json_message_const::fetchType);
    fetchType = static_cast<FetchType>(convertVarToNumeric<unsigned int>(tmp, 0));

    _isError = false;
  }
  catch(Poco::JSON::JSONException& e)
  {
    _isError = true;
    errorMsg = e.message();
    errorCode = ERROR_UNSERIALIZE;
  }
  catch(Poco::Exception& e)
  {
    _isError = true;
    errorMsg = e.message();
    errorCode = ERROR_UNSERIALIZE;
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
void DRCETaskRequestGetData::clear(void)
{
  fetchType = FetchType::ftUninitialized;
}
//-----------------------------------------------------------------------------
std::istream& operator>>(std::istream& is, DRCETaskRequestGetData& rhs)
{
  std::string json;
  is.seekg(0, std::ios::end);
  json.resize(is.tellg());
  is.seekg(0, std::ios::beg);
  is.read(const_cast<char*>(json.c_str()), json.size());
  rhs.unserialize(json);
  return is;
}
//-----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const DRCETaskRequestGetData& rhs)
{
  std::string json;
  const_cast<DRCETaskRequestGetData&>(rhs).serialize(json);
  return os << json;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 
