/**
 * @file DRCETaskRequestSetExecute.hpp
 *
 * Implementation class DRCETaskRequestSetExecute of a Distributed Remote Command Execution functional object in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_TASK_REQUEST_SET_EXECUTE_HPP
#define DRCE_TASK_REQUEST_SET_EXECUTE_HPP

#include <iostream>

#include "DRCEJsonMessageConst.hpp"
#include "DRCESessionOptions.hpp"
#include "DRCEResourceLimits.hpp"
#include "DRCEFilesList.hpp"
#include "DRCETaskRequest.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class DRCETaskRequestSetExecute : public DRCETaskRequest, public DRCEFilesList
{
public:
  explicit DRCETaskRequestSetExecute(const std::string& json="");
  DRCETaskRequestSetExecute(const DRCETaskRequestSetExecute& rhs);
  DRCETaskRequestSetExecute(DRCETaskRequestSetExecute&& rhs);
  virtual ~DRCETaskRequestSetExecute(void) {}

  DRCETaskRequestSetExecute& operator=(const DRCETaskRequestSetExecute& rhs);
  DRCETaskRequestSetExecute& operator=(DRCETaskRequestSetExecute&& rhs);

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void setSessionOptions(const SessionOptions& sessionOptions_) {sessionOptions=sessionOptions_;}
  void setSessionOptions(SessionOptions&& sessionOptions_);
  const SessionOptions& getSessionOptions(void) const {return sessionOptions;}

  void setCommandLine(const std::string& commandLine_) {commandLine=commandLine_;}
  std::string getCommandLine(void) const {return commandLine;}

  void setInputStream(const std::string& inputStream_) {inputStream=inputStream_;}
  std::string getInputStream(void) const {return inputStream;}

  void setResourceLimits(const ResourceLimits& resourceLimits_) {resourceLimits=resourceLimits_;}
  void setResourceLimits(ResourceLimits&& resourceLimits_);
  const ResourceLimits& getResourceLimits(void) const {return resourceLimits;}

  void clear(void);
private:
  SessionOptions sessionOptions;
  std::string commandLine;
  std::string inputStream;
  ResourceLimits resourceLimits;

private:
  friend std::istream& operator>>(std::istream& is, DRCETaskRequestSetExecute& rhs);
  friend std::ostream& operator<<(std::ostream& os, const DRCETaskRequestSetExecute& rhs);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace drce
} // namespace HCE

#endif // DRCE_TASK_REQUEST_SET_EXECUTE_HPP
