#include <Poco/JSON/Object.h>
#include <Poco/JSON/Array.h>
#include <Poco/JSON/JSON.h>
#include <Poco/JSON/Stringifier.h>
#include <Poco/JSON/Parser.h>
#include <Poco/JSON/JSONException.h>
#include <Poco/Dynamic/Var.h>
#include <utility>

#include "DRCEError.hpp"
#include "DRCETaskRequestTerminate.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
DRCETaskRequestTerminate::DRCETaskRequestTerminate(const std::string& json)
: inherited(DRCETaskRequest::RequestType::rtTerminateTask), algorithmType(AlgorithmType::atUninitialized),
  delayValue(0), repeatValue(0), signalValue(0), cleanup(CleanupFlag::cfNotDelete)
{
  if (!json.empty())
    unserialize(json);
}
//-----------------------------------------------------------------------------
void DRCETaskRequestTerminate::clear(void)
{
  algorithmType = AlgorithmType::atUninitialized;
  delayValue = 0;
  repeatValue = 0;
  signalValue = 0;
  cleanup = CleanupFlag::cfNotDelete;
}
//-----------------------------------------------------------------------------
bool DRCETaskRequestTerminate::serialize(std::string& json)
{
  errorCode = NO_ERROR;
  errorMsg.clear();
  try
  {
    Poco::JSON::Object::Ptr obj = new Poco::JSON::Object();

    obj->set(drce_json_message_const::algorithmType, static_cast<unsigned int>(algorithmType));
    obj->set(drce_json_message_const::delayValue, delayValue);
    obj->set(drce_json_message_const::repeatValue, repeatValue);
    obj->set(drce_json_message_const::signalValue, signalValue);
    obj->set(drce_json_message_const::cleanup, static_cast<unsigned int>(cleanup));

    std::stringstream ostr;
    Poco::JSON::Stringifier::stringify(obj, ostr);
    json = ostr.str();

    _isError = false;
  }
  catch(std::exception& e)
  {
    _isError = true;
    errorMsg = e.what();
    errorCode = ERROR_SERIALIZE;
  }
  return !_isError;}
//-----------------------------------------------------------------------------
bool DRCETaskRequestTerminate::unserialize(const std::string& json)
{
  clear();
  errorCode = NO_ERROR;
  errorMsg.clear();

  try
  {
    Poco::JSON::Parser parser;
    Poco::Dynamic::Var res = parser.parse(json);
    Poco::JSON::Object::Ptr obj = res.extract<Poco::JSON::Object::Ptr>();

    Poco::Dynamic::Var tmp = obj->get(drce_json_message_const::algorithmType);
    algorithmType = static_cast<AlgorithmType>(convertVarToNumeric<unsigned int>(tmp, 0));

    tmp = obj->get(drce_json_message_const::delayValue);
    delayValue = convertVarToNumeric<unsigned int>(tmp, 0);

    tmp = obj->get(drce_json_message_const::repeatValue);
    repeatValue = convertVarToNumeric<unsigned int>(tmp, 0);

    tmp = obj->get(drce_json_message_const::signalValue);
    signalValue = convertVarToNumeric<unsigned int>(tmp, 0);

    tmp = obj->get(drce_json_message_const::cleanup);
    cleanup = static_cast<CleanupFlag>(convertVarToNumeric<unsigned int>(tmp, 0));

    _isError = false;
  }
  catch(Poco::JSON::JSONException& e)
  {
    _isError = true;
    errorMsg = e.message();
    errorCode = ERROR_UNSERIALIZE;
  }
  catch(Poco::Exception& e)
  {
    _isError = true;
    errorMsg = e.message();
    errorCode = ERROR_UNSERIALIZE;
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
std::istream& operator>>(std::istream& is, DRCETaskRequestTerminate& rhs)
{
  std::string json;
  is.seekg(0, std::ios::end);
  json.resize(is.tellg());
  is.seekg(0, std::ios::beg);
  is.read(const_cast<char*>(json.c_str()), json.size());
  rhs.unserialize(json);
  return is;
}
//-----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const DRCETaskRequestTerminate& rhs)
{
  std::string json;
  const_cast<DRCETaskRequestTerminate&>(rhs).serialize(json);
  return os << json;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 
