/**
 * @file DRCETaskRequestTerminate.hpp
 *
 * Implementation class DRCETaskRequestTerminate of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_TASK_REQUEST_TERMINATE_HPP
#define DRCE_TASK_REQUEST_TERMINATE_HPP

#include "DRCETaskRequest.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
class DRCETaskRequestTerminate : public DRCETaskRequest
{
public:
  enum class AlgorithmType
  {
    atUninitialized=0,
    atDefault=1,
    atCustom=2
  };

  enum class CleanupFlag
  {
    cfNotDelete=0,
    cfDelete=1
  };

  explicit DRCETaskRequestTerminate(const std::string& json="");
  virtual ~DRCETaskRequestTerminate(void) {}

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void setAlgorithmType(AlgorithmType algorithmType_) {algorithmType=algorithmType_;}
  AlgorithmType getAlgorithmType(void) const {return algorithmType;}

  void setDelayValue(unsigned int delayValue_) {delayValue=delayValue_;}
  unsigned int getDelayValue(void) const {return delayValue;}

  void setRepeatValue(unsigned int repeatValue_) {repeatValue=repeatValue_;}
  unsigned int getRepeatValue(void) const {return repeatValue;}

  void setSignalValue(unsigned int signalValue_) {signalValue=signalValue_;}
  unsigned int getSignalValue(void) const {return signalValue;}

  void setCleanupFlag(CleanupFlag cleanup_) {cleanup=cleanup_;}
  CleanupFlag getCleanupFlag(void) const {return cleanup;}

  void clear(void);
private:
  AlgorithmType algorithmType;
  unsigned int delayValue;
  unsigned int repeatValue;
  unsigned int signalValue;
  CleanupFlag cleanup;
private:
  friend std::istream& operator>>(std::istream& is, DRCETaskRequestTerminate& rhs);
  friend std::ostream& operator<<(std::ostream& os, const DRCETaskRequestTerminate& rhs);
  typedef DRCETaskRequest inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_TASK_REQUEST_TERMINATE_HPP
