/**
 * @file DRCETaskRequestTerminateExecutor.hpp
 *
 * Implementation class DRCETaskRequestTerminateExecutor of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_TASK_REQUEST_TERMINATE_EXECUTOR_HPP
#define DRCE_TASK_REQUEST_TERMINATE_EXECUTOR_HPP

#include "DRCEError.hpp"
#include "DRCEMessageConst.hpp"
#include "DRCETaskRequestExecutor.hpp"
#include "DRCECommonTask.hpp"

namespace HCE
{
namespace drce
{
class DRCETaskRequestTerminate;
//-----------------------------------------------------------------------------
class DRCETaskRequestTerminateExecutor : public DRCETaskRequestExecutor
{
public:
  DRCETaskRequestTerminateExecutor(DRCEAsyncTasksQueue& asyncTaskQueue_,
      DRCENodeOptions& nodeOptions_,
      CustomMessage& message_,
      DRCEResourceMonitor& resourceMonitor_);
  virtual ~DRCETaskRequestTerminateExecutor(void) {}

  DRCEResultDataItem execute(DRCETaskRequest* pTaskRequest,
      DRCEInputJsonMessage& inputJsonMessage);

  static void terminateExpiredTask(const AsyncTasks& asyncTask,
      DRCEAsyncTasksQueue& asyncTaskQueue,
      DRCENodeOptions& nodeOptions,
      CustomMessage& message);

  static void executeTerminate(unsigned int taskId,
      DRCEAsyncTasksQueue& asyncTaskQueue,
      DRCENodeOptions& nodeOptions,
      CustomMessage& message,
      const std::string& errorMsg="",
      unsigned int errorCode=0,
      DRCETaskRequest::TaskState taskState=DRCETaskRequest::TaskState::TERMINATED);
private:
  static void executeDefaultAlgorithm(DRCETaskRequestTerminate& taskRequestTerminate,
      pid_t pid,
      DRCEAsyncTasksQueue& asyncTaskQueue,
      DRCENodeOptions& nodeOptions,
      CustomMessage& message,
      DRCEResultDataItem& resultDataItem) throw (Poco::Exception);

  static void executeCustomAlgorithm(DRCETaskRequestTerminate& taskRequestTerminate,
      pid_t pid,
      DRCEAsyncTasksQueue& asyncTaskQueue,
      DRCENodeOptions& nodeOptions,
      CustomMessage& message,
      DRCEResultDataItem& resultDataItem) throw (Poco::Exception);

  static bool loadStatusFile(DRCEAsyncTasksQueue& asyncTaskQueue,
      DRCENodeOptions& nodeOptions,
      CustomMessage& message,
      DRCEResultDataItem& resultDataItem);

  static void terminateTask(unsigned int taskId,
      DRCETaskRequestTerminate& taskRequestTerminate,
      DRCEAsyncTasksQueue& asyncTaskQueue,
      DRCENodeOptions& nodeOptions,
      CustomMessage& message,
      DRCEResultDataItem& resultDataItem,
      const std::string& errorMsg="",
      unsigned int errorCode=0,
      DRCETaskRequest::TaskState taskState=DRCETaskRequest::TaskState::TERMINATED) throw (Poco::Exception);

  DRCEResultDataItem makeResultDataItem(DRCEInputJsonMessage& inputJsonMessage,
      unsigned int exitStatus=0);

  void prepareCleanup(unsigned int taskId,
      DRCEInputJsonMessage& inputJsonMessage);

  static DRCEResultDataItem executeTerminate(DRCETaskRequest* pTaskRequest,
      DRCEInputJsonMessage& inputJsonMessage,
      DRCEAsyncTasksQueue& asyncTaskQueue,
      DRCENodeOptions& nodeOptions,
      CustomMessage& message,
      const std::string& errorMsg="",
      unsigned int errorCode=0,
      DRCETaskRequest::TaskState taskState=DRCETaskRequest::TaskState::TERMINATED);

  typedef DRCETaskRequestExecutor inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_TASK_REQUEST_TERMINATE_EXECUTOR_HPP
