/**
 * @file DRCETasksQueue.hpp
 *
 * Implementation class DRCETasksQueue of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_TASKS_QUEUE_HPP
#define DRCE_TASKS_QUEUE_HPP

#include <iostream>

#include "DRCECommonList.hpp"
#include "JsonSerializable.hpp"
#include "DRCETaskRequest.hpp"
#include "DRCESessionOptions.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
struct TaskStatusData
{
public:
  TaskStatusData(void);
  TaskStatusData(unsigned int id_, size_t sdate_, DRCETaskRequest::TaskState state_, SessionOptions::ThreadMode tmode);
  TaskStatusData(const TaskStatusData& rhs);
  TaskStatusData(TaskStatusData&& rhs);

  TaskStatusData& operator=(const TaskStatusData& rhs);
  TaskStatusData& operator=(TaskStatusData&& rhs);

  void clear(void);

  unsigned int id;
  size_t sdate;
  DRCETaskRequest::TaskState state;
  SessionOptions::ThreadMode tmode;

  friend std::ostream& operator<<(std::ostream& os, const TaskStatusData& rhs);
  friend std::istream& operator>>(std::istream& is, TaskStatusData& rhs);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class DRCETasksQueue : public DRCECommonList <TaskStatusData>, public IJsonSerializable
{
public:
  DRCETasksQueue(void);
  virtual ~DRCETasksQueue(void) {}

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_TASKS_QUEUE_HPP
