/**
 * @file DRCECleanupTest.hpp
 *
 * Implementation class DRCECleanupTest of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_CLEANUP_TEST_HPP
#define DRCE_CLEANUP_TEST_HPP

#include <iostream>

#include "DRCESessionOptions.hpp"
#include "DRCEInputJsonMessage.hpp"

#define CLEANUP_TEST 1

namespace HCE
{
namespace drce
{
namespace tests
{
//-----------------------------------------------------------------------------
class DRCECleanupTest
{
public:
  static void testAsyncTaskAsyncSubtasks(void);
  static void testAsyncTaskSyncSubtasks(void);
  static void testSyncTaskAsyncSubtasks(void);
  static void testSyncTaskSyncSubtasks(void);

  static void testAsyncTaskMixSubtasks(void);
  static void testSyncTaskMixSubtasks(void);

  static void testAsyncTaskMixSubtasksCleanup(void);
  static void testSyncTaskMixSubtasksCleanup(void);

  static void testRequestGetDataCleanup(void);
  static void testRequestDeleteDataCleanup(void);
  static void testRequestTerminateCleanup(void);

  static void testRequestDeleteData(void);

private:

  static std::string getInputJson(unsigned int startTaskId,
      SessionOptions::ThreadMode taskThreadMode,
      SessionOptions::ThreadMode subtasksThreadMode);

  static std::string getInputJson(unsigned int startTaskId,
      SessionOptions::ThreadMode taskThreadMode);

  static std::string getInputJsonCleanup(unsigned int startTaskId,
      SessionOptions::ThreadMode taskThreadMode,
      const std::string& commandLine,
      unsigned int timeMax=200000);

  static std::string getInputJsonCleanup(unsigned int startTaskId,
        SessionOptions::ThreadMode taskThreadMode,
        SessionOptions::CleanupFlag cleanupFlag,
        const std::string& commandLine);

  static DRCEInputJsonMessage getInputJsonMessage(unsigned int taskId,
      SessionOptions::ThreadMode threadMode,
      SessionOptions::CleanupFlag cleanupFlag,
      const std::string& commandLine="",
      unsigned int timeMax=200000);

  static bool isExistTaskData(unsigned int taskId, DRCENodeOptions& nodeOptions);
  static bool isNotExistTaskData(unsigned int taskId, DRCENodeOptions& nodeOptions);

  static void cleanupDir(const std::string& dirName);
public:
  static void cleanupTestData(DRCENodeOptions& nodeOptions);
  static std::string getInputJson(unsigned int startTaskId,
        SessionOptions::ThreadMode threadMode,
        SessionOptions::CleanupFlag cleanupFlag,
        const std::string& commandLine,
        unsigned int timeMax);
  static DRCEResultDataItem extractResultData(unsigned int taskId,
      DRCENodeOptions& nodeOptions);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace tests
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_CLEANUP_TEST_HPP
