#include <iomanip>
#include <sstream>

#include <Poco/ConsoleChannel.h>
#include <Poco/FileChannel.h>
#include <Poco/FormattingChannel.h>
#include <Poco/PatternFormatter.h>
#include <Poco/Logger.h>
#include <Poco/AutoPtr.h>

#include "DRCEPrintStatus.hpp"

namespace HCE
{
namespace drce
{
namespace tests
{
//-----------------------------------------------------------------------------
DRCEPrintStatus::DRCEPrintStatus(const std::string& debugLevel_, const std::string& postfix_)
: debugLevel(debugLevel_), postfix(postfix_)
{
  Poco::AutoPtr<Poco::ConsoleChannel> pCons(new Poco::ConsoleChannel());
  pCons->open();

  Poco::Logger::get(debugLevel).setChannel(pCons);
  Poco::Logger::get(debugLevel).setLevel(debugLevel);
}
//-----------------------------------------------------------------------------
void DRCEPrintStatus::operator ()(const std::string& msg, unsigned int width)
{
  std::stringstream outMsg, fmtMsg;
  outMsg << std::setw(width) << msg;

  if (!postfix.empty())
    outMsg << postfix;

  fmtMsg << std::string(outMsg.str().length()+1,'=') << "\n" << outMsg.str() << "\n" << std::string(outMsg.str().length()+1,'=');
  int level = Poco::Logger::get(debugLevel).getLevel();
  Poco::Logger::get(debugLevel).log(Poco::Message("", fmtMsg.str(), static_cast<Poco::Message::Priority>(level)));
}
//-----------------------------------------------------------------------------
std::string DRCEPrintStatus::operator ()(IJsonSerializable& rhs)
{
  std::ostringstream outMsg;
  outMsg << "ErrorMsg: " << rhs.getErrorMsg() << "\tErrorCode: " << rhs.getErrorCode() << std::endl;
  return outMsg.str();
}
//-----------------------------------------------------------------------------
std::string DRCEPrintStatus::operator ()(DRCEFunctionalObjectBase& rhs)
{
  std::ostringstream outMsg;
  outMsg << "ErrorMsg: " << rhs.getErrorMsg() << "\tErrorCode: " << rhs.getErrorCode() << std::endl;
  return outMsg.str();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace tests
} // namespace drce
} // namespace HCE
