/**
 * @file DRCEStressTest.hpp
 *
 * Implementation class DRCEStressTest of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_STRESS_TEST_HPP
#define DRCE_STRESS_TEST_HPP

#include <iostream>
#include <string>
#include <map>

#include "DRCEFunctionalObject.hpp"
#include "DRCESessionOptions.hpp"
#include "DRCEResourceLimits.hpp"
#include "DRCETaskRequestSetExecute.hpp"
#include "DRCETaskRequestGetData.hpp"
#include "DRCETaskRequestCheckState.hpp"
#include "DRCETaskRequestTerminate.hpp"

namespace HCE
{
namespace drce
{
namespace tests
{
//-----------------------------------------------------------------------------
class DRCEStressTest
{
public:
  static void testMultipleRequests(DRCEFunctionalObject& fObj, SessionOptions::SessionType sessionType, unsigned int requestCount);
  static void testRequestsTimeDelay(DRCEFunctionalObject& fObj, SessionOptions::SessionType sessionType, unsigned int requestCount, unsigned int repeat);
  static void testResourceLimits(DRCEFunctionalObject& fObj, SessionOptions::SessionType sessionType, SessionOptions::ThreadMode threadMode);
  static void testResourceUsageLimits(DRCEFunctionalObject& fObj, SessionOptions::SessionType sessionType, SessionOptions::ThreadMode threadMode);

private:
  static void testResourceLimits(DRCEFunctionalObject& fObj,
      SessionOptions::SessionType sessionType,
      SessionOptions::ThreadMode threadMode,
      const ResourceLimits& resourceLimits,
      unsigned int& taskId,
      bool exceededLimits,
      const std::string& commandLine="");

  static void testRequestsTimeDelay(DRCEFunctionalObject& fObj, SessionOptions::SessionType sessionType, unsigned int requestCount, unsigned int& taskId,
      std::map<unsigned int, unsigned int>& delayTimes, std::map<unsigned int, unsigned int>& times);
  static unsigned int getRequestTime(DRCEFunctionalObject& fObj, unsigned int requestId);

  static void executeRequest(DRCEFunctionalObject& fObj, const std::string& json);
  static std::string getJsonRequestSetExecute(DRCETaskRequestSetExecute& requestSetExecute, unsigned int requestId);

  static std::string getJsonRequestSetExecuteAsyncExpitedTask(SessionOptions::SessionType sessionType,unsigned int requestId);
  static std::string getJsonRequestSetExecuteAsyncLongTask(SessionOptions::SessionType sessionType,unsigned int requestId);
  static std::string getJsonRequestSetExecuteAsyncShortTask(SessionOptions::SessionType sessionType,unsigned int requestId);
  static std::string getJsonRequestSetExecuteSyncShortTask(SessionOptions::SessionType sessionType,unsigned int requestId);


  static std::string generateJsonRequest(unsigned int caseNumber, unsigned int requestId);
  static std::string getJsonRequestCheckState(DRCETaskRequestCheckState::CheckType checkType, unsigned int requestId);
  static std::string getJsonRequestGetData(DRCETaskRequestGetData::FetchType fetchType, unsigned int requestId);
  static std::string getJsonRequestTerminate(DRCETaskRequestTerminate::AlgorithmType algorithmType, unsigned int requestId);

  static std::string getJsonRequestResourceLimits(SessionOptions::SessionType sessionType,
      SessionOptions::ThreadMode threadMode,
      unsigned int requestId,
      const ResourceLimits& resourceLimits,
      const std::string& commandLine="");
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace tests
} // end namespace drce 
} // end namespace HCE 

#endif // DRCE_STRESS_TEST_HPP
