#include <gtest/gtest.h>

#include "DRCEError.hpp"
#include "DRCEPrintStatus.hpp"
#include "DRCEFunctionalObject.hpp"
#include "DRCETaskRequestSetExecute.hpp"
#include "DRCETaskRequestGetData.hpp"
#include "DRCETaskRequestCheckState.hpp"
#include "DRCETaskRequestTerminate.hpp"
#include "DRCETaskRequestsTest.hpp"
#include "DRCENotificationExecutor.hpp"
#include "DRCETaskThreadingModeTest.hpp"

namespace HCE
{
namespace drce
{
namespace tests
{
//-----------------------------------------------------------------------------
void DRCETaskRequestsTest::testSerializeUnserializeSetTaskToExecuteRequest(void)
{
  std::string json = "{\"session\":{\"type\":5,\"tmode\":1,\"time_max\":750,\"port\":8080,\"user\":\"admin\",\"password\":\"42\",\"shell\":\"bash\",\"environment\":[{\"LINES\":\"80\",\"HOSTNAME\":\"node22\"}]},\"command\":\"sort\",\"input\":\"def\\nabc\\n123\\n\",\"files\":[{\"name\":\"file1\",\"data\":\"test content of file1\\n\",\"action\":13},{\"name\":\"file2\",\"data\":\"test content of file2\\n\",\"action\":1}]}";

  DRCETaskRequestSetExecute taskRequestSetExecute(json);
  ASSERT_FALSE(taskRequestSetExecute.isError());
  ASSERT_EQ(taskRequestSetExecute.getErrorCode(), NO_ERROR);
  ASSERT_TRUE(taskRequestSetExecute.getErrorMsg().empty());
  ASSERT_EQ(taskRequestSetExecute.getCommandLine(), "sort");
  ASSERT_EQ(taskRequestSetExecute.getInputStream(), "def\nabc\n123\n");
  ASSERT_EQ(static_cast<unsigned int>(taskRequestSetExecute.getSessionOptions().sessionType), 5);
  ASSERT_EQ(static_cast<unsigned int>(taskRequestSetExecute.getSessionOptions().tmode), 1);
  ASSERT_EQ(taskRequestSetExecute.getSessionOptions().port, 8080);
  ASSERT_EQ(taskRequestSetExecute.getSessionOptions().userName, "admin");
  ASSERT_EQ(taskRequestSetExecute.getSessionOptions().userPassword, "42");
  ASSERT_EQ(taskRequestSetExecute.getSessionOptions().shellName, "bash");
  ASSERT_EQ(taskRequestSetExecute.getSessionOptions().timeMax, 750);
  ASSERT_EQ(taskRequestSetExecute.getSessionOptions().environments.size(), 2);
  ASSERT_EQ(taskRequestSetExecute.getSessionOptions().environments[0].first, "HOSTNAME");
  ASSERT_EQ(taskRequestSetExecute.getSessionOptions().environments[0].second, "node22");
  ASSERT_EQ(taskRequestSetExecute.getSessionOptions().environments[1].first, "LINES");
  ASSERT_EQ(taskRequestSetExecute.getSessionOptions().environments[1].second, "80");
  ASSERT_EQ(taskRequestSetExecute.getFilesCount(), 2);
  ASSERT_EQ(taskRequestSetExecute.getFileItem(0).actionType, 13);
  ASSERT_EQ(taskRequestSetExecute.getFileItem(0).name, "file1");
  ASSERT_EQ(taskRequestSetExecute.getFileItem(0).data, "test content of file1\n");
  ASSERT_EQ(taskRequestSetExecute.getFileItem(1).actionType, 1);
  ASSERT_EQ(taskRequestSetExecute.getFileItem(1).name, "file2");
  ASSERT_EQ(taskRequestSetExecute.getFileItem(1).data, "test content of file2\n");
  printSuccess("Unserialize DRCETaskRequestSetExecute");

  std::string newJson;
  ASSERT_TRUE(taskRequestSetExecute.serialize(newJson));
  ASSERT_FALSE(taskRequestSetExecute.isError());
  ASSERT_EQ(taskRequestSetExecute.getErrorCode(), NO_ERROR);
  ASSERT_TRUE(taskRequestSetExecute.getErrorMsg().empty());
  printSuccess("Serialize DRCETaskRequestSetExecute");

  DRCETaskRequestSetExecute taskRequestSetExecuteRes(newJson);
  ASSERT_FALSE(taskRequestSetExecuteRes.isError());
  ASSERT_EQ(taskRequestSetExecuteRes.getErrorCode(), NO_ERROR);
  ASSERT_TRUE(taskRequestSetExecuteRes.getErrorMsg().empty());
  ASSERT_EQ(taskRequestSetExecuteRes.getCommandLine(), "sort");
  ASSERT_EQ(taskRequestSetExecuteRes.getInputStream(), "def\nabc\n123\n");
  ASSERT_EQ(static_cast<unsigned int>(taskRequestSetExecuteRes.getSessionOptions().sessionType), 5);
  ASSERT_EQ(static_cast<unsigned int>(taskRequestSetExecuteRes.getSessionOptions().tmode), 1);
  ASSERT_EQ(taskRequestSetExecuteRes.getSessionOptions().port, 8080);
  ASSERT_EQ(taskRequestSetExecuteRes.getSessionOptions().userName, "admin");
  ASSERT_EQ(taskRequestSetExecuteRes.getSessionOptions().userPassword, "42");
  ASSERT_EQ(taskRequestSetExecuteRes.getSessionOptions().shellName, "bash");
  ASSERT_EQ(taskRequestSetExecuteRes.getSessionOptions().timeMax, 750);
  ASSERT_EQ(taskRequestSetExecuteRes.getSessionOptions().environments.size(), 2);
  ASSERT_EQ(taskRequestSetExecuteRes.getSessionOptions().environments[0].first, "HOSTNAME");
  ASSERT_EQ(taskRequestSetExecuteRes.getSessionOptions().environments[0].second, "node22");
  ASSERT_EQ(taskRequestSetExecuteRes.getSessionOptions().environments[1].first, "LINES");
  ASSERT_EQ(taskRequestSetExecuteRes.getSessionOptions().environments[1].second, "80");
  ASSERT_EQ(taskRequestSetExecuteRes.getFilesCount(), 2);
  ASSERT_EQ(taskRequestSetExecuteRes.getFileItem(0).actionType, 13);
  ASSERT_EQ(taskRequestSetExecuteRes.getFileItem(0).name, "file1");
  ASSERT_EQ(taskRequestSetExecuteRes.getFileItem(0).data, "test content of file1\n");
  ASSERT_EQ(taskRequestSetExecuteRes.getFileItem(1).actionType, 1);
  ASSERT_EQ(taskRequestSetExecuteRes.getFileItem(1).name, "file2");
  ASSERT_EQ(taskRequestSetExecuteRes.getFileItem(1).data, "test content of file2\n");
  printSuccess("Test Serialize/Unserialize SetTaskToExecuteRequest");
}
//-----------------------------------------------------------------------------
void DRCETaskRequestsTest::testSerializeUnserializeCheckTaskRequest(void)
{
  std::string json;
  DRCETaskRequestCheckState taskRequestCheckState;
  taskRequestCheckState.setCheckType(DRCETaskRequestCheckState::CheckType::ctExtended);
  ASSERT_TRUE(taskRequestCheckState.serialize(json));
  ASSERT_FALSE(taskRequestCheckState.isError());
  ASSERT_EQ(taskRequestCheckState.getErrorCode(), NO_ERROR);
  ASSERT_TRUE(taskRequestCheckState.getErrorMsg().empty());
  printSuccess("DRCETaskRequestCheckState serialize");

  DRCETaskRequestCheckState taskRequestCheckStateRes(json);
  ASSERT_FALSE(taskRequestCheckStateRes.isError());
  ASSERT_EQ(taskRequestCheckStateRes.getErrorCode(), NO_ERROR);
  ASSERT_TRUE(taskRequestCheckStateRes.getErrorMsg().empty());
  printSuccess("DRCETaskRequestCheckState unserialize");

  ASSERT_EQ(taskRequestCheckState.getCheckType(), taskRequestCheckStateRes.getCheckType());
  printSuccess("Test Serialize/Unserialize DRCETaskRequestSetExecute");
}
//-----------------------------------------------------------------------------
void DRCETaskRequestsTest::testSerializeUnserializeGetTaskDataRequest(void)
{
  std::string json;
  DRCETaskRequestGetData taskRequestGetData;
  taskRequestGetData.setFetchType(DRCETaskRequestGetData::FetchType::ftNotDeleteDataAfterFetch);
  ASSERT_TRUE(taskRequestGetData.serialize(json));
  ASSERT_FALSE(taskRequestGetData.isError());
  ASSERT_EQ(taskRequestGetData.getErrorCode(), NO_ERROR);
  ASSERT_TRUE(taskRequestGetData.getErrorMsg().empty());
  printSuccess("DRCETaskRequestGetData serialize");

  DRCETaskRequestGetData taskRequestGetDataRes(json);
  ASSERT_FALSE(taskRequestGetDataRes.isError());
  ASSERT_EQ(taskRequestGetDataRes.getErrorCode(), NO_ERROR);
  ASSERT_TRUE(taskRequestGetDataRes.getErrorMsg().empty());
  printSuccess("DRCETaskRequestGetData unserialize");

  ASSERT_EQ(taskRequestGetData.getFetchType(), taskRequestGetDataRes.getFetchType());
  printSuccess("Test Serialize/Unserialize DRCETaskRequestGetData");
}
//-----------------------------------------------------------------------------
void DRCETaskRequestsTest::testSerializeUnserializeTerminateTaskRequest(void)
{
  std::string json;
  DRCETaskRequestTerminate taskRequestTerminate;
  taskRequestTerminate.setAlgorithmType(DRCETaskRequestTerminate::AlgorithmType::atDefault);
  taskRequestTerminate.setDelayValue(500);
  taskRequestTerminate.setRepeatValue(5);
  taskRequestTerminate.setSignalValue(15);
  ASSERT_TRUE(taskRequestTerminate.serialize(json));
  ASSERT_FALSE(taskRequestTerminate.isError());
  ASSERT_EQ(taskRequestTerminate.getErrorCode(), NO_ERROR);
  ASSERT_TRUE(taskRequestTerminate.getErrorMsg().empty());
  printSuccess("DRCETaskRequestTerminate serialize");

//  std::cout << "JSON: " << json << std::endl;

  DRCETaskRequestTerminate taskRequestTerminateRes(json);
  ASSERT_FALSE(taskRequestTerminateRes.isError());
  ASSERT_EQ(taskRequestTerminateRes.getErrorCode(), NO_ERROR);
  ASSERT_TRUE(taskRequestTerminateRes.getErrorMsg().empty());
  printSuccess("DRCETaskRequestTerminate unserialize");

  ASSERT_EQ(static_cast<unsigned int>(taskRequestTerminate.getAlgorithmType()), static_cast<unsigned int>(taskRequestTerminateRes.getAlgorithmType()));
  ASSERT_EQ(taskRequestTerminate.getDelayValue(), taskRequestTerminateRes.getDelayValue());
  ASSERT_EQ(taskRequestTerminate.getRepeatValue(), taskRequestTerminateRes.getRepeatValue());
  ASSERT_EQ(taskRequestTerminate.getSignalValue(), taskRequestTerminateRes.getSignalValue());
  printSuccess("Test Serialize/Unserialize DRCETaskRequestTerminate");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace tests
} // end namespace drce 
} // end namespace HCE 
