/**
 * @file DRCETaskThreadingModeTest.hpp
 *
 * Implementation class DRCETaskThreadingModeTest of a Distributed Remote Command Execution functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef DRCE_TASK_THREADING_MODE_TEST_HPP
#define DRCE_TASK_THREADING_MODE_TEST_HPP

#include <functional>
#include "DRCENodeOptions.hpp"
#include "DRCEFunctionalObject.hpp"
#include "DRCETaskRequestTerminate.hpp"
#include "DRCETaskRequestSetExecute.hpp"

namespace HCE
{
namespace drce
{
namespace tests
{
//-----------------------------------------------------------------------------
class DRCETaskThreadingModeTest
{
public:
  static DRCENodeOptions getNodeOptions(const std::string& homeDir="");
  static bool isExistTask(DRCEFunctionalObject& fObj, unsigned int requestId);
public:
  static void testParallelAsyncExecution(DRCEFunctionalObject& fObj,
      unsigned int requestId,
      unsigned int repeatCount=1);

  static void testMaxThreadLimits(DRCEFunctionalObject& fObj,
      unsigned int requestId,
      unsigned int maxAllowedThreadsCount,
      unsigned int repeatCount);

  static void testExecuteSyncRequest(DRCEFunctionalObject& fObj,
      unsigned int requestId);

  static void testSetTaskToExecuteRequest(DRCEFunctionalObject& fObj,
      unsigned int requestId,
      const std::string& requestData="");

  static void testCheckTaskRequest(DRCEFunctionalObject& fObj,
      unsigned int requestId,
      const std::string& requestData="");

  static void testGetTaskDataRequest(DRCEFunctionalObject& fObj,
      unsigned int requestId);

  static void testTerminateTaskRequest(DRCEFunctionalObject& fObj,
      unsigned int requestId,
      DRCETaskRequestTerminate::AlgorithmType algorithmType,
      DRCETaskRequestTerminate::CleanupFlag cleanupFlag=DRCETaskRequestTerminate::CleanupFlag::cfNotDelete);

  static void testDeleteTaskDataRequest(DRCEFunctionalObject& fObj,
      unsigned int requestId);

  static void testExecuteNotification(DRCEFunctionalObject& fObj,
      unsigned int requestId);

  static void testTerminateExpiredTaskAsync(DRCEFunctionalObject& fObj,
      unsigned int requestId);

  static void testTerminateExpiredTaskSync(DRCEFunctionalObject& fObj,
      unsigned int requestId);

  static void testTaskRequestTimeLimits(DRCEFunctionalObject& fObj,
      unsigned int requestId,
      SessionOptions::ThreadMode threadMode,
      unsigned int timeMax);

  static void testTaskRequestExpiredTime(DRCEFunctionalObject& fObj,
      unsigned int requestId,
      SessionOptions::ThreadMode threadMode,
      SessionOptions::CleanupFlag cleanupFlag);

  // functional test of cleanup
  static void testRequestLocalShellAsyncCleanUp(void);
  static void testTerminateTaskRequestCleanUp(void);
  // functional test of shell request
  static void testRequestLocalShellAsync(void);
  static void testStressMultipleRequestsShellHost(void);
  // functional test of resource monitor
  static void testResourceMonitor(void);
  // functional test of getter current tasks queue
  static void testCurrentTasksQueue(void);
  // functional test of getter list info all exist data about tasks
  static void testListAllTasks(void);

  std::string operator()(const std::string& data)
  {
    return data;
  }
  void setNotificationFunctor(std::function<std::string(const std::string&)> notificationFunctor_) {notificationFunctor=notificationFunctor_;}
  std::string executeNotification(const std::string& data)
  {
    std::string result;
    if (notificationFunctor)
      result = notificationFunctor(data);
    return result;
  }
private:
  std::function<std::string(const std::string&)> notificationFunctor;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace tests
} // namespace drce
} // namespace HCE

#endif // DRCE_TASK_THREADING_MODE_TEST_HPP
