#include <gtest/gtest.h>

#include "DRCEProtocolsTest.hpp"
#include "DRCEFunctionalObjectTest.hpp"
#include "DRCEActionTypesTest.hpp"
#include "DRCERequestLocalShellTest.hpp"
#include "DRCETaskThreadingModeTest.hpp"
#include "DRCETaskRequestsTest.hpp"
#include "DRCECleanupTest.hpp"
#include "DRCEMessageConst.hpp"

#include <Poco/Channel.h>
#include <Poco/ConsoleChannel.h>
#include <Poco/FileChannel.h>
#include <Poco/FormattingChannel.h>
#include <Poco/PatternFormatter.h>
#include <Poco/Logger.h>
#include <Poco/AutoPtr.h>


#define TEST_DEBUG 0

namespace HCE
{
namespace drce
{
namespace tests
{
//-----------------------------------------------------------------------------
void loggerInitialization(const std::string& loggerName, Poco::AutoPtr<Poco::Channel> pCustomerChannel, const std::string& level)
{
  Poco::AutoPtr<Poco::PatternFormatter> pPatternFormatter(new Poco::PatternFormatter("%Y-%m-%d %H:%M:%S.%i - %N[%P] - %s - %q - %t"));
  if (pPatternFormatter)
  {
    Poco::AutoPtr<Poco::FormattingChannel> pChannel = new Poco::FormattingChannel(pPatternFormatter);
    if (pChannel)
    {
      Poco::AutoPtr<Poco::Channel> pCustomer(pCustomerChannel);
      pChannel->setChannel(pCustomer);
      pChannel->open();
      Poco::Logger::get(loggerName).setChannel(pChannel);
      Poco::Logger::get(loggerName).setLevel(level);
    }
  }
}
//-----------------------------------------------------------------------------
} // end namespace tests
} // end namespace drce
} // end namespace HCE

#if TEST_DEBUG == 1

int main(int argc, char** argv)
{
  // loggers intialization
  HCE::drce::tests::loggerInitialization(Poco::Logger::ROOT, new Poco::ConsoleChannel(), "trace");

  Poco::AutoPtr<Poco::FileChannel> pFileChannel = new Poco::FileChannel();
  pFileChannel->setProperty("path", "./tasks.log");
  HCE::drce::tests::loggerInitialization(HCE::drce::drce_const::DRCE_TASKS_LOGGER, pFileChannel, "information");
/*
  HCE::drce::tests::DRCEProtocolsTest::testResultDataSerializator();
  HCE::drce::tests::DRCEProtocolsTest::testInputJsonMessage();
  HCE::drce::tests::DRCEProtocolsTest::testOutputJsonMessage();

  HCE::drce::tests::DRCEProtocolsTest::testSessionOptions();
  HCE::drce::tests::DRCEProtocolsTest::testFilesItemList();

  HCE::drce::tests::DRCEActionTypesTest::testCreateBeforeExec();
  HCE::drce::tests::DRCEActionTypesTest::testDeleteBeforeExec();
  HCE::drce::tests::DRCEActionTypesTest::testReadAfterExec();
  HCE::drce::tests::DRCEActionTypesTest::testDeleteAfterExec();
  HCE::drce::tests::DRCEActionTypesTest::testEncodeDecodeContent();

  HCE::drce::tests::DRCERequestLocalShellTest::testEmptyCommand();
  HCE::drce::tests::DRCERequestLocalShellTest::testInputStreamWithCommand();

  HCE::drce::tests::DRCERequestLocalShellTest::testInputStreamWithoutCommand();
  HCE::drce::tests::DRCERequestLocalShellTest::testCommandSort();
  HCE::drce::tests::DRCERequestLocalShellTest::testCommandLs();
  HCE::drce::tests::DRCERequestLocalShellTest::testBadCommandLs();

  HCE::drce::tests::DRCERequestLocalShellTest::testCommandPs();
  HCE::drce::tests::DRCERequestLocalShellTest::testCommandPhp();
  HCE::drce::tests::DRCERequestLocalShellTest::testCommandPerl();
  HCE::drce::tests::DRCERequestLocalShellTest::testCommandJava();

  HCE::drce::tests::DRCEFunctionalObjectTest::testApplyEnviroments();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellSortInput();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellSortOutput();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellSort();

  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPsInput();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPsOutput();

  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPs();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPhpVersion();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPhpScript();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPerlVersion();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPerlScript();

  HCE::drce::tests::DRCETaskRequestsTest::testSerializeUnserializeSetTaskToExecuteRequest();
  HCE::drce::tests::DRCETaskRequestsTest::testSerializeUnserializeCheckTaskRequest();
  HCE::drce::tests::DRCETaskRequestsTest::testSerializeUnserializeGetTaskDataRequest();
  HCE::drce::tests::DRCETaskRequestsTest::testSerializeUnserializeTerminateTaskRequest();

  HCE::drce::tests::DRCETaskThreadingModeTest::testTerminateTaskRequestCleanUp();
  HCE::drce::tests::DRCETaskThreadingModeTest::testRequestLocalShellAsyncCleanUp();
*/
//  HCE::drce::tests::DRCETaskThreadingModeTest::testRequestLocalShellAsync();
//  HCE::drce::tests::DRCETaskThreadingModeTest::testStressMultipleRequestsShellHost();
/*
  HCE::drce::tests::DRCETaskThreadingModeTest::testResourceMonitor(); // resource monitor to stdout
  HCE::drce::tests::DRCETaskThreadingModeTest::testCurrentTasksQueue();
  HCE::drce::tests::DRCETaskThreadingModeTest::testListAllTasks();

  HCE::drce::tests::DRCECleanupTest::testAsyncTaskAsyncSubtasks();
  HCE::drce::tests::DRCECleanupTest::testAsyncTaskSyncSubtasks();
  HCE::drce::tests::DRCECleanupTest::testSyncTaskAsyncSubtasks();
  HCE::drce::tests::DRCECleanupTest::testSyncTaskSyncSubtasks();
  HCE::drce::tests::DRCECleanupTest::testAsyncTaskMixSubtasks();
  HCE::drce::tests::DRCECleanupTest::testSyncTaskMixSubtasks();
  HCE::drce::tests::DRCECleanupTest::testAsyncTaskMixSubtasksCleanup();
  HCE::drce::tests::DRCECleanupTest::testSyncTaskMixSubtasksCleanup();
  HCE::drce::tests::DRCECleanupTest::testRequestGetDataCleanup();
  HCE::drce::tests::DRCECleanupTest::testRequestDeleteDataCleanup();
  HCE::drce::tests::DRCECleanupTest::testRequestTerminateCleanup();
  HCE::drce::tests::DRCECleanupTest::testRequestDeleteData();
*/

  Poco::Logger::root().information("Done");
  Poco::Logger::shutdown();
  return 0;
}
#else
  //  svn://192.168.1.201/hce/hce_dts/trunk/api/php/bin/drce_fetch_resources.sh

int main (int argc, char** argv)
{
  // loggers intialization
  HCE::drce::tests::loggerInitialization(Poco::Logger::ROOT, new Poco::ConsoleChannel(), "trace");
  Poco::AutoPtr<Poco::FileChannel> pFileChannel = new Poco::FileChannel();
  pFileChannel->setProperty("path", "./tasks.log");
  HCE::drce::tests::loggerInitialization(HCE::drce::drce_const::DRCE_TASKS_LOGGER, pFileChannel, "information");

  testing::InitGoogleTest(&argc, argv);
  return RUN_ALL_TESTS();
}

#endif // end TEST_DEBUG
