#ifndef COMPONENT_MANAGER_HPP
#define COMPONENT_MANAGER_HPP

#include <Poco/SharedPtr.h>
#include <Poco/Condition.h>
#include <Poco/Mutex.h>
#include <Poco/Runnable.h>
#include <Poco/TaskManager.h>
#include <Poco/Thread.h>
#include <Poco/ThreadPool.h>
#include <queue>
#include <vector>

#include "ComponentBase.hpp"
#include "TaskWaitObject.hpp"

namespace HCE
{
  namespace component
  {
    class ComponentManager : public Poco::Runnable
    {
      protected:
        Poco::SharedPtr<ComponentBase> getFreeComponent();
      protected:
        bool exitFlag;
        std::vector<Poco::SharedPtr<ComponentBase> > componentsList;
        Poco::Thread ownThread;
        Poco::SharedPtr<Poco::TaskManager> _taskManager;
        Poco::FastMutex insertComponentMutex;
        Poco::FastMutex incomeDataMutex;
        Poco::Condition conditionVar;
        std::queue<Poco::SharedPtr<TaskWaitObject> > taskQueue;
        Poco::SharedPtr<Poco::ThreadPool> _threadPool;
      public:
        ComponentManager():exitFlag(false), _taskManager(nullptr) {}
        void addComponent(Poco::SharedPtr<ComponentBase> componentPtr);
        void setTaskManager(Poco::SharedPtr<Poco::TaskManager> taskManager);
        void addTask(Poco::SharedPtr<TaskWaitObject> task);
        void setThreadPool(Poco::SharedPtr<Poco::ThreadPool> threadPool) {_threadPool = threadPool;}
        void clearTaskQueue();
        void start();
        void stop();
        void join() {ownThread.join();}
        bool isRunning() {return ownThread.isRunning();}
        virtual void run();
        virtual ~ComponentManager() {}
    };
  }
}


#endif