#include <Poco/ScopedLock.h>

#include "ComponentManagerPool.hpp"
#include "ComponentNotFoundException.hpp"

namespace HCE
{
  using namespace HCE::exception;
  namespace component
  {   
    void ComponentManagerPool::addManager(ComponentType componentType, Poco::SharedPtr<ComponentManager> cManagerPtr, bool _useCommonPool)
    {
      Poco::ScopedLock<Poco::FastMutex> sLock(globalComponentMutex);
      ManagerIterator it = tManagerMap.find(componentType);
      if(it != tManagerMap.end())
      {
    //  exception - manager already exist    
      }
      Poco::SharedPtr<Poco::ThreadPool> _threadPoolPtr;
      if(!_useCommonPool)
      {
        _threadPoolPtr = Poco::SharedPtr<Poco::ThreadPool>(new Poco::ThreadPool());
        cManagerPtr->setThreadPool(_threadPoolPtr);
      }
      Poco::ThreadPool &localPool = (_useCommonPool ? thPool : (*_threadPoolPtr));
      cManagerPtr->setTaskManager(Poco::SharedPtr<Poco::TaskManager> (new Poco::TaskManager(localPool)));
      tManagerMap[componentType] = cManagerPtr;
    }

    Poco::SharedPtr<TaskWaitObject> ComponentManagerPool::addInData(Poco::SharedPtr<DataBase> inData)
    {
      Poco::ScopedLock<Poco::FastMutex> sLock(globalComponentMutex);  
      Poco::SharedPtr<TaskWaitObject> task = Poco::SharedPtr<TaskWaitObject>(new TaskWaitObject(inData));
      ManagerIterator it = tManagerMap.find(inData->getType());
      if(it == tManagerMap.end())
      {
        task->setOutData(NULL, HCECTE_EXP2, NULL);
      }
      else
      {
        (*it).second->addTask(task);
      } 
      return task;  
    }
    
    void ComponentManagerPool::clearTaskQueue(ComponentType componentType)
    {
      Poco::ScopedLock<Poco::FastMutex> sLock(globalComponentMutex);
      ManagerIterator it = tManagerMap.find(componentType);
      if(it == tManagerMap.end())
      {
        throw ComponentNotFoundException("ComponentNotFound");        
      }
      else
      {
        (*it).second->clearTaskQueue();
      }
    }
    
    
    ComponentManagerPool::~ComponentManagerPool()
    {
      for(ManagerIterator it = tManagerMap.begin(); it != tManagerMap.end(); it++)
      {
        if((*it).second)
        {
          if(((*it).second)->isRunning())
          {
            ((*it).second)->stop();
            ((*it).second)->join();
          }
        }
      }
    }
  }
}