#ifndef COMPONENT_MANAGER_POOL_HPP
#define COMPONENT_MANAGER_POOL_HPP

#include <Poco/SharedPtr.h>
#include <Poco/Mutex.h>
#include <Poco/ThreadPool.h>
#include <map>

#include "HCEConstants.hpp"
#include "ComponentManager.hpp"
#include "ComponentManagerPoolIf.hpp"
#include "DataBase.hpp"
#include "TaskWaitObject.hpp"

namespace HCE
{
  namespace component
  {
    class ComponentManagerPool : public ComponentManagerPoolIf
    {
      private:
        virtual void createAndAddComponent(Poco::SharedPtr<ComponentBase> component){};    
      protected:
        typedef std::map<ComponentType,  Poco::SharedPtr<ComponentManager> >::iterator ManagerIterator;
        Poco::FastMutex globalComponentMutex;
        std::map<ComponentType, Poco::SharedPtr<ComponentManager> > tManagerMap;
        Poco::ThreadPool thPool;
      public:
        ComponentManagerPool() {}
        virtual void addManager(ComponentType componentType, Poco::SharedPtr<ComponentManager> cManagerPtr, bool _useCommonPool);
        virtual Poco::SharedPtr<TaskWaitObject> addInData(Poco::SharedPtr<DataBase> inData);
        virtual void clearTaskQueue(ComponentType componentType);
        virtual ~ComponentManagerPool();
    };
  }
}

#endif