#include <AdminClient.h>
AdminClient::AdminClient(const char *host, int port, List<BaseHandler*> *handlerList)
{
  int status = 0;
  this->handlerList = handlerList;
  listenSock = new(std::nothrow) Socket();
  if(!listenSock)lastErrNo = ENOMEM;
  listenSock->ReuseAddr(status);
  lastErrNo = 0;
  listenSock->bind(host, port, status);
  if(status == SOCK_OK)
  {
    listenSock->listen(status, 1);
    if(status != SOCK_OK)
    {
      lastErrNo = errno;
    }
  }
  else
  {
    lastErrNo = errno;
  }
}

AdminClient::~AdminClient()
{
  delete listenSock;
}

char* AdminClient::urlEncode(const char*url)
{
  char *encoded = NULL;
  register int enc_len = 0;
  encoded = (char*)malloc( strlen(url) * 3 );
  for(unsigned int i = 0; i < strlen(url); i++)
  {
    if( url[i] < 45 || (url[i] == 47) || (url[i] > 57 && url[i] < 65) || (url[i] > 90 && url[i] < 95) || (url[i] == 96) || (url[i] > 122))
    {
      enc_len += sprintf((char*)encoded + enc_len, "%%%02X", url[i]);
    }
    else
    {
      encoded[enc_len++] = url[i];
    }
  }
  encoded = (char*)realloc(encoded, enc_len + 1);
  encoded[enc_len] = 0;
  return encoded;
}

void* AdminClient::run()
{
  int status = 0;
  size_t requestLen = 0;
  char *request;
  char *buf = NULL;
  while(!exitFlag)
  {
    while(!listenSock->selectRead(status))
    {
      if(exitFlag)
      {
        return NULL;
      }
      usleep(1000);
    }
    sock_fd_t sock = listenSock->accept(status);
    if(status == SOCK_OK)
    {
      clientSock = new(std::nothrow) Socket(sock);
      if(!clientSock)
      {
        lastErrNo = ENOMEM;
        return NULL;
      }
      clientSock->setNonBlock(status);
      request = readRequest(requestLen);
      //printf("%d\n%s\n", requestLen, request );
      if(request)
      {
        char *response = NULL;
        httpParser = new(std::nothrow) HttpParser(request, requestLen);
        if(!httpParser)
        {
          lastErrNo = ENOMEM;
          return NULL;
        }
        char *command = httpParser->getByName("c");
        if(command != NULL)
        { 
          unsigned int handlerId = atoi(command);
          while(*command >= '0' && *command <= '9')command++;
          if(*command)
          {
            if(handlerList)
            {
              mutex.lock();
              for(unsigned long i = 0; i < handlerList->getCount(); i++)
              {
                if((*handlerList)[i]->getId() == handlerId)
                {
                  switch(handlerId)                
                  {
                    case 121:
                    {
                      if(*command == 'a')
                      {
                        char *ret = (char*)(*handlerList)[i]->handle(*command);
                        if(ret)
                        {
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(strlen(buf) + 50);
                          sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                          free(buf);
                          delete []ret;
                        }
                      }
                      break;
                    }
                    case 122:
                    {
                      if(*command == 'a')
                      {
                        char *p1 = httpParser->getByName("p1");
                        if(p1)
                        {
                          bool *ret = (bool*)(*handlerList)[i]->handle(*command, p1);
                          if(ret)
                          {
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc(strlen(buf) + 50);
                            sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                            free(buf);
                            delete ret;
                          }
                        }
                        else
                        {
                          buf = urlEncode(ES_WRONGPARAM);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                          free(buf);
                        }

                      }
                      if(*command == 'c' || *command == 'e')
                      {
                        char *p1 = httpParser->getByName("p1");
                        if(p1)
                        {
                          bool *ret = (bool*)(*handlerList)[i]->handle(*command, atoi(p1));
                          if(ret)
                          {
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc(strlen(buf) + 50);
                            sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                            free(buf);
                            delete ret;
                          }
                        }
                        else
                        {
                          buf = urlEncode(ES_WRONGPARAM);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                          free(buf);
                        }

                      }
                      if(*command == 'b' || *command == 'd' || *command == 'f' || *command == 'g')
                      {
                        int *v1 = (int*)(*handlerList)[i]->handle(*command);
                        if(v1)
                        {
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(strlen(buf) + 50);
                          sprintf(response, "n=1&e=%d&d=%s&v1=%d", E_OK, buf, *v1);
                          free(buf);
                          delete v1;
                        }
                      }
                      if(*command == 'h')
                      {
                        char *title = httpParser->getByName("p1");
                        char *h = httpParser->getByName("p2");
                        char *body = httpParser->getByName("p3");
                        char *topic = httpParser->getByName("p4");
                        if(title && h && body && topic)
                        {
                          unsigned int *ret = (unsigned int*)(*handlerList)[i]->handle(*command, atoi(topic), title, h, body);
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(strlen(buf) + 50 + 16 * 12);
                          unsigned int len = sprintf(response, "n=19&e=%d&d=%s", E_OK, buf);
                          for(int i = 0; i < 19; i++)
                          {
                            len += sprintf(response + len, "&v%d=%u", i + 1, ret[i]);
                          }
                          free(buf);
                          delete []ret;
                        }
                      }
                      if(*command == 'j' || *command == 'k')
                      {
                        char *content = httpParser->getByName("p1");
                        if(content)
                        {
                          char *ret = (char*)(*handlerList)[i]->handle(*command, content);
                          buf = urlEncode(ES_OK);
                          size_t rLen = (ret != NULL ? strlen(ret): 0);
                          response = (char*)malloc(strlen(buf) + 50 + rLen);
                          unsigned int len = sprintf(response, "n=1&e=%d&d=%s&v1=", E_OK, buf);
                          memcpy(response + len, ret, rLen);
                          response[len + rLen] = '\0';
                          free(buf);
                          if(ret)free(ret);
                        }
                      }
                      if(*command == 'l')
                      {
                        bool *ret = (bool*)(*handlerList)[i]->handle(*command);
                        if(ret)
                        {
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(50);
                          sprintf(response, "n=1&e=%d&d=%s&v1=%d", E_OK, buf, *ret ? 1:0);
                          delete ret;
                          free(buf);
                        }
                      }
                      if(*command == 'm')
                      {
                        char *ret = (char*)(*handlerList)[i]->handle(*command);
                        if(ret)
                        {
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(50 + strlen(ret));
                          sprintf(response, "n=1&e=%d&d=%s&v1=%s", E_OK, buf, ret);
                          free(ret);
                          free(buf);
                        }
                      }

                      break;
                    }

                    case 112:
                    {
                      if(*command == 'a')
                      {
                        unsigned long *ret = (unsigned long*)(*handlerList)[i]->handle(*command);
                        if(ret)
                        {
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(strlen(buf) + 50);
                          sprintf(response, "n=4&e=%d&d=%s&v1=%lu&v2=%lu&v3=%lu&v4=%lu", E_OK, buf, ret[0], ret[1], ret[2], ret[3]);
                          free(buf);
                          delete []ret;
                        }
                      }
                      else
                      {
                        if(*command == 'b')
                        {
                          char *p1 = httpParser->getByName("p1");
                          char *p2 = httpParser->getByName("p2");
                          char *p3 = httpParser->getByName("p3");
                          char *p4 = httpParser->getByName("p4");
                          if(p1 && p2 && p3 && p4)
                          {
                            char *ret = (char*)(*handlerList)[i]->handle(*command, (unsigned long)atol(p1), (unsigned long)atol(p2), (unsigned long)atol(p3), (unsigned long)atol(p4));
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              response =  (char*)malloc(strlen(buf) + 30);
                              sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                              free(buf);
                              delete ret;
                            }
                          }
                          else
                          {
                            buf = urlEncode(ES_WRONGPARAM);
                            response = (char*)malloc(512);
                            sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                            free(buf);
                          }
                        }
                        else
                        {
                          buf = urlEncode(ES_WRONGPARAM);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                          free(buf);
                        }
                      }
                      break;
                    }
                    case 140:
                    {
                      if(*command == 'a')
                      {
                        char *ret = (char*)(*handlerList)[i]->handle(*command);
                        if(ret)
                        {
                          int len = *((int*)ret);                        
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(strlen(buf) + 30 + len);
                          int pos = sprintf(response, "n=1&e=%d&d=%s&v1=", E_OK, buf);
                          free(buf);
                          memcpy(response + pos, ret + sizeof(int), len);
                          response[pos + len] = '\0';
                          free(ret);
                        }
                      }
                      else
                      {
                        if(*command == 'b')
                        {
                          char *value = httpParser->getByName("p1");
                          int *ret = (int*)(*handlerList)[i]->handle(*command, value);
                          if(ret)
                          {
                            buf = urlEncode(ES_OK);
                            response =  (char*)malloc(strlen(buf) + 30);
                            sprintf(response, "n=1&e=%d&d=%s&v1=%d", E_OK, buf, *ret);
                            free(ret);
                            free (buf);
                          }
                        }
                        else if (*command == 'c')
                        {
                          char *value = httpParser->getByName("p1");
                          if(value)
                          {
                            int *ret = (int*)(*handlerList)[i]->handle(*command, atoi(value));
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              response =  (char*)malloc(strlen(buf) + 64);
                              sprintf(response, "n=1&e=%d&d=%s&v1=%d", E_OK, buf, *ret);
                              free(ret);
                              free (buf);
                            }
                          }
                        }
                        else if(*command == 'd')
                        {
                          char *value = httpParser->getByName("p1");
                          if(value)
                          {
                            char *ret = (char*)(*handlerList)[i]->handle(*command, atoi(value));
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              response =  (char*)malloc(strlen(buf) + 30 + strlen(ret));
                              sprintf(response, "n=3&e=%d&d=%s&%s", E_OK, buf, ret);
                              free(ret);
                              free (buf);
                            }
                          }
                        }
                        else if(*command == 'e')
                        {
                          char *value = httpParser->getByName("p1");
                          if(value)
                          {
                            bool *ret = (bool*)(*handlerList)[i]->handle(*command, value);
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              response =  (char*)malloc(strlen(buf) + 30 + sizeof (u_int32_t));
                              sprintf(response, "n=1&e=%d&d=%s&v1=%u", E_OK, "", *ret ? 0 : 1);
                              free(ret);
                              free (buf);
                            }
                          }
                        }
                        else if(*command == 'f')
                        {
                          char *value = httpParser->getByName("p1");
                          if(value)
                          {
                            bool *ret = (bool*)(*handlerList)[i]->handle(*command, atoi(value));
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              response =  (char*)malloc(strlen(buf) + 30 + sizeof (u_int32_t));
                              sprintf(response, "n=1&e=%d&d=%s&v1=%u", E_OK, "", *ret ? 0 : 1);
                              free(ret);
                              free (buf);
                            }
                          }
                        }                        
                        else
                        {
                          buf = urlEncode(ES_WRONGPARAM);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                          free(buf);
                        }
                      }
                      break;
                    }
                    case 111:
                    {
                      switch(*command)
                      {
                        case 'a':
                        {
                          char *ret = (char*)(*handlerList)[i]->handle(*command);
                          if(ret)
                          {
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc(strlen(buf) + 30);
                            sprintf(response, "n=1&e=%d&d=%s&v1=%c", E_OK, buf, *ret);
                            free(buf); 
                            delete ret;
                          }
                          break;
                        }                      
                        case 'b':
                        {
                          char *p1 = httpParser->getByName("p1");
                          if(p1)
                          {
                            char *ret = (char*)(*handlerList)[i]->handle(*command, atoi(p1));
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              response = (char*)malloc(strlen(buf) + 30);
                              sprintf(response, "n=1&e=%d&d=%s&v1=%c", E_OK, buf, *ret);
                              free(buf); 
                              delete ret;
                            }
                          }
                          break;
                        }
                        case 'c':
                        {
                          char *p1 = httpParser->getByName("p1");
                          if(p1)
                          {
                            char *ret = (char*)(*handlerList)[i]->handle(*command, atoi(p1));
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              response = (char*)malloc(strlen(buf) + 30);
                              sprintf(response, "n=1&e=%d&d=%s&v1=%d", E_OK, buf, *ret);
                              free(buf); 
                              delete ret;
                            }
                          }
                          break;
                        }                      
                        case 'd':
                        {
                          char *ret = (char*)(*handlerList)[i]->handle(*command);
                          if(ret)
                          {
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc(strlen(buf) + 30);
                            sprintf(response, "n=1&e=%d&d=%s&v1=%d", E_OK, buf, *ret);
                            free(buf); 
                            delete ret;
                          }
                          break;
                        }
                        default:
                        {
                          buf = urlEncode(ES_WRONGPARAM);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                          free(buf);
                        }
                      }
                      break;
                    }
                    case 110:
                    {                    
                      char *value = httpParser->getByName("p1");
                      if(value && (*command >= 'a' && *command <= 'd'))
                      {                      
                        unsigned long *ret;
                        char *url = *command == 'a' ? httpParser->getByName("p2") : NULL;
                        ret = (unsigned long*)(*handlerList)[i]->handle(*command, (u_int32_t)atoi(value), url);
                        if(ret)
                        {
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(strlen(buf) + 30);
                          sprintf(response, "n=1&e=%d&d=%s&v1=%lu", E_OK, buf, *ret);
                          free(buf); 
                          delete ret;
                        }
                        else
                        {                              
                          char *errbuf;
                          int code = (*handlerList)[i]->getError(&errbuf);
                          buf = urlEncode(errbuf);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", code, buf);
                          free(buf);
                        }
                      } 
                      else
                      {
                        if(*command == 'e' || *command == 'g')
                        {
                          u_int32_t site = 0;
                          if(*command == 'g')
                          {
                            char *p1 = httpParser->getByName("p1");
                            if(p1)site = atoi(p1);
                          }
                          char *ret = (char*)(*handlerList)[i]->handle(*command, site);
                          if(ret)
                          {
                            buf = urlEncode(ES_OK);
                            unsigned int len = strlen(ret);
                            response = (char*)malloc(strlen(buf) + 30 + len);
                            sprintf(response, "n=1&e=%d&d=%s&v1=%s", E_OK, buf, ret);
                            free(buf); 
                            free(ret);
                          }
                          else
                          {
                            char *errbuf;
                            int code = (*handlerList)[i]->getError(&errbuf);
                            buf = urlEncode(errbuf);
                            response = (char*)malloc(512);
                            sprintf(response, "n=0&e=%d&d=%s", code, buf);
                            free(buf);
                          }
                        }
                        else
                        {
                          if(*command == 'f')
                          {
                            char *p1 = httpParser->getByName("p1");
                            if(p1)
                            {
                              char *ret = (char*)(*handlerList)[i]->handle(*command, atoi(p1));
                              if(ret)
                              {
                                int len = *((int*)ret);                        
                                buf = urlEncode(ES_OK);
                                response = (char*)malloc(strlen(buf) + 30 + len);
                                int pos = sprintf(response, "n=1&e=%d&d=%s&v1=", E_OK, buf);
                                free(buf);
                                memcpy(response + pos, ret + sizeof(int), len);
                                response[pos + len] = '\0';
                                free(ret);
                              }
                            }
                          }
                          else
                          {
                            if(*command == 'h')
                            {
                              char *p1 = httpParser->getByName("p1");
                              if(p1)
                              {
                                String *ret = (String*)(*handlerList)[i]->handle(*command, p1);
                                if(ret)
                                {
                                  buf = urlEncode(ES_OK);
                                  response = (char*)malloc(strlen(buf) + 30 + ret->length());
                                  int pos = sprintf(response, "n=1&e=%d&d=%s&v1=", E_OK, buf);
                                  free(buf);
                                  memcpy(response + pos, ret->c_str(), ret->length());
                                  response[pos + ret->length()] = '\0';
                                  delete ret;
                                }
                              }
                            }
                            else
                            {
                              if (*command == 'j')
                              {
                                char *p1 = httpParser->getByName("p1");
                                if(p1)
                                {
                                  char *ret = (char*)(*handlerList)[i]->handle(*command, p1);
                                  if(ret)
                                  {
                                    response = (char*)malloc(strlen(ret) + 16);
                                    sprintf(response, "n=1&e=0&d=&v1=%s", ret);
                                    delete ret;                                   
                                  }
                                  else
                                  {
                                    response = (char*)malloc(16);
                                    strcpy(response, "n=0&e=1&d=&v1=");
                                  }
                                }                                
                              }
                              else
                              {
                                buf = urlEncode(ES_WRONGPARAM);
                                response = (char*)malloc(512);
                                sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                                free(buf);
                             }
                            }
                          }
                        }
                      }
                      break;
                    }

                    case 106:
                    {                    
                      char *value = httpParser->getByName("p1");
                      if(value || *command == 'b' || *command == 'd' || *command == 'f')
                      {                      
                        unsigned long *ret;
                        if(value)
                        { 
                          ret = (unsigned long*)(*handlerList)[i]->handle(*command, (unsigned long)atol(value));
                        }
                        else
                        {
                          ret = (unsigned long*)(*handlerList)[i]->handle(*command);
                        }
                        if(ret)
                        {
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(strlen(buf) + 30);
                          sprintf(response, "n=1&e=%d&d=%s&v1=%lu", E_OK, buf, *ret);
                          free(buf); 
                          delete ret;
                        }
                        else
                        {                              
                          char *errbuf;
                          int code = (*handlerList)[i]->getError(&errbuf);
                          buf = urlEncode(errbuf);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", code, buf);
                          free(buf);
                        }
                      }
                      else
                      {
                        buf = urlEncode(ES_WRONGPARAM);
                        response = (char*)malloc(512);
                        sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                        free(buf);
                      }
                      break;
                    }
                    //***************************Dump handler for CDH**********************************************************************
                    case 105:
                    {
                      char *value = httpParser->getByName("p1");
                      if(value || *command == 'f' || *command == 'i' || *command == 'j')
                      {
                        switch(*command)                          
                        {
                          case 'a':
                          case 'b':
                          case 'c':
                          {
                            bool *ret = (bool*)(*handlerList)[i]->handle(*command, value);
                            if(ret)
                            {                         
                              if(*ret)
                              {
                                buf = urlEncode(ES_OK);
                                response = (char*)malloc(strlen(buf) + 15);
                                sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                                free(buf);
                              }
                              else
                              {                              
                                char *errbuf;
                                int code = (*handlerList)[i]->getError(&errbuf);
                                buf = urlEncode(errbuf);
                                response = (char*)malloc(512);
                                sprintf(response, "n=0&e=%d&d=%s", code, buf);
                                free(buf);
                              }
                              delete ret;
                            }
                            break;
                          }
                          case 'f':
                          {
                            char *ret = (char*)(*handlerList)[i]->handle(*command, value);
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              response = (char*)malloc((buf ? strlen(buf) : 0) + 256);
                              int state = *ret;
                              char *start = ret;
                              ret++;
                              int errNo = *((int*)ret);
                              ret += sizeof(int);
                              char *errorStr = ret + sizeof(int);
                              ret += sizeof(int) + *((int*)ret) + 1;
                              char *fileName = ret + sizeof(int);
                              ret += sizeof(int) + *((int*)ret) + 1;
                              time_t t = *((time_t*)ret);
                              sprintf(response, "n=5&e=%d&d=%s&v1=%u&v2=%d&v3=%s&v4=%s&v5=%u", E_OK, (buf ? buf : ""), state, 
                                      errNo, errorStr, fileName, (unsigned int)t);
                              if(buf)
                              {
                                free(buf);
                              }
                              delete []start;
                            }
                            break;
                          }
                          case 'g':
                          {
                            char *ret = (char*)(*handlerList)[i]->handle(*command, atoi(value));
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc((buf ? strlen(buf) : 0) + (ret ? strlen(ret) : 0) + 256);
                            sprintf(response, "n=1&e=%d&d=%s&v1=%s", E_OK, (buf ? buf : ""), (ret ? ret : ""));
                            if(buf)
                            {
                              free(buf);
                            }
                            if(ret)
                            {
                              free(ret);
                            }
                            break;
                          }
                          case 'h':
                          {
                            bool *ret = (bool*)(*handlerList)[i]->handle(*command, atoi(value));
                            if(ret)
                            {
                              if(*ret)
                              {
// Add code herre
                              }
                              else
                              {
// Add code here
                              }
                              delete ret;
                            }
//--------------------------------------------------------------------------
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc(512);
                            sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                            free(buf);
//--------------------------------------------------------------------------
                            break;
                          }
                          case 'i':
                          {
                            (*handlerList)[i]->handle(*command);
//--------------------------------------------------------------------------
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc(512);
                            sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                            free(buf);
//--------------------------------------------------------------------------  
                            break;
                          }
                          case 'j':
                          {
                            bool *ret = (bool*)(*handlerList)[i]->handle(*command);
//--------------------------------------------------------------------------
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc(512);
                            sprintf(response, "n=0&e=%d&d=%s&v1=%c", E_OK, buf, ((ret && *ret) ? '1' : '0'));
                            free(buf);
                            if(ret)
                            {
                              delete ret;
                            }
//--------------------------------------------------------------------------
                            break;
                          }
                          case 'k':
                          {
                            (*handlerList)[i]->handle(*command, atoi(value));
//--------------------------------------------------------------------------
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc(512);
                            sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                            free(buf);
//--------------------------------------------------------------------------
                            break;
                          }                          

                          default:
                          {
                            buf = urlEncode(ES_WRONGPARAM);
                            response = (char*)malloc(512);
                            sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                            free(buf);
                            break;
                          }
                        }
                      }
                      else
                      {
                        buf = urlEncode(ES_WRONGPARAM);
                        response = (char*)malloc(512);
                        sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                        free(buf);
                      }
                      break;
                    }
                    //***************************Resource Handler***************************************************************************
                    case 104:
                    {
                      char *value = httpParser->getByName("p1");
                      if(value || *command == 'f')
                      {
                        switch(*command)
                        {
                          case 'a':
                          case 'h':
                          {
                            char **ret = (char**)(*handlerList)[i]->handle(*command, strtoull(value, NULL, 10));
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              size_t rLen = 2048;
                              response = (char*)malloc(rLen + 5);
                              size_t rPos = sprintf(response, "n=17&e=%d&d=%s", E_OK, buf);
                              free(buf);
                              buf = (char*)malloc(10);
                              for(unsigned int r = 0; r < 17; r++)
                              {
                                size_t sLen = strlen(ret[r]);
                                if(rLen < rPos + sLen + 4)
                                {
                                  rLen += rPos + sLen + 4;
                                  response = (char*)realloc(response, rLen + 5);
                                }
                                size_t bufLen = sprintf(buf, "&v%u=", r + 1);
                                memcpy(response + rPos, buf, bufLen);
                                rPos += bufLen;
                                memcpy(response + rPos, ret[r], sLen);
                                rPos += sLen;
                                delete []ret[r];
                              }
                              response[rPos] = '\0';
                              free(buf);
                              delete []ret;
                            }
                            else
                            {
                              char *errbuf;
                              int code = (*handlerList)[i]->getError(&errbuf);
                              buf = urlEncode(errbuf);
                              response = (char*)malloc(512);
                              sprintf(response, "n=0&e=%d&d=%s", code, buf);
                              free(buf);
                            }
                            break;
                          }
                          case 'b':
                          case 'c':
                          case 'd':
                          case 'e':
                          {
                            bool *ret = (bool*)(*handlerList)[i]->handle(*command, value);
                            if(ret)
                            {
                              if(*ret)
                              {
                                buf = urlEncode(ES_OK);
                                response = (char*)malloc(strlen(buf) + 15);
                                sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                                free(buf);
                              }
                              else
                              {                              
                                char *errbuf;
                                int code = (*handlerList)[i]->getError(&errbuf);
                                buf = urlEncode(errbuf);
                                response = (char*)malloc(512);
                                sprintf(response, "n=0&e=%d&d=%s", code, buf);
                                free(buf);
                              }
                              delete ret;
                            }
                            break;
                          }
                          case 'k':
                          {
                            bool *ret = (bool*)(*handlerList)[i]->handle(*command, strtoull(value, NULL, 10));
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              response = (char*)malloc(strlen(buf) + 15);
                              sprintf(response, "n=1&e=%d&d=%s&v1=%d", E_OK, buf, *ret ? 1:0);
                              free(buf);
                              delete ret;
                            }
                            break;
                          }
                          case 'i':
                          case 'j':
                          {
                            char *smId = httpParser->getByName("p2");
                            if(smId)
                            {
                              bool *ret = (bool*)(*handlerList)[i]->handle(*command, value, atoi(smId));
                              if(ret)
                              {
                                if(*ret)
                                {
                                  buf = urlEncode(ES_OK);
                                  response = (char*)malloc(strlen(buf) + 15);
                                  sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                                  free(buf);
                                }
                                else
                                {                              
                                  char *errbuf;
                                  int code = (*handlerList)[i]->getError(&errbuf);
                                  buf = urlEncode(errbuf);
                                  response = (char*)malloc(512);
                                  sprintf(response, "n=0&e=%d&d=%s", code, buf);
                                  free(buf);
                                }
                                delete ret;
                              }
                            }
                            break;
                          }
                          case 'f':
                          {
                            char *ret = (char*)(*handlerList)[i]->handle(*command, value);
                            if(ret)
                            {
                              buf = urlEncode(ES_OK);
                              response = (char*)malloc(strlen(buf) + 256);
                              int state = *ret;
                              char *start = ret;
                              ret++;
                              int errNo = *((int*)ret);
                              ret += sizeof(int);
                              char *errorStr = ret + sizeof(int);
                              ret += sizeof(int) + *((int*)ret) + 1;
                              char *fileName = ret + sizeof(int);
                              ret += sizeof(int) + *((int*)ret) + 1;
                              time_t t = *((time_t*)ret);
                              sprintf(response, "n=5&e=%d&d=%s&v1=%u&v2=%d&v3=%s&v4=%s&v5=%u", E_OK, buf, state, errNo, errorStr, fileName, (unsigned int)t);
                              free(buf);
                              delete []start;
                            }
                            break;
                          }
                          case 'g':
                          {
                            u_int32_t *count = (u_int32_t*)(*handlerList)[i]->handle(*command, atoi(value));
                            if(count)
                            {
                              buf = urlEncode(ES_OK);
                              response = (char*)malloc(strlen(buf) + 256);
                              sprintf(response, "n=1&e=%d&d=%s&v1=%u", E_OK, buf, *count);
                              free(buf);
                              delete count;
                            }
                            break;
                          }
                        }
                        
                      }
                      else
                      {
                        buf = urlEncode(ES_WRONGPARAM);
                        response = (char*)malloc(512);
                        sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                        free(buf);
                      } 
                      break;
                    }
                    //***************************Log handler**********************************************************************************
                    case 101:
                    {
                      char *value = httpParser->getByName("p1");
                      if(value)
                      {
                        int *ret = (int*)(*handlerList)[i]->handle(*command, atoi(value));
                        if(ret)
                        {
                          if(*command == 'a' || *command == 'b')
                          {
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc(512);
                            sprintf(response, "n=1&e=%d&d=%s&v1=%d", E_OK, buf, *ret);
                            free(buf);
                            delete ret;
                          }
                          if(*command == 'c')
                          {
                            int errNo = *ret;
                            char *retData = (char*)(ret + 1);
                            buf = urlEncode(ES_OK);
                            response = (char*)malloc(strlen(retData) + 128);
                            char *errText = urlEncode(retData);
                            sprintf(response, "n=2&e=%d&d=%s&v1=%d&v2=%s", E_OK, buf, errNo, errText);
                            free(errText);
                            free(buf);
                            delete []ret;
                          }
                        }
                        else
                        {
                          char *errbuf;
                          int code = (*handlerList)[i]->getError(&errbuf);
                          buf = urlEncode(errbuf);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", code, buf);
                          free(buf);
                        }
                      }
                      else
                      {
                        buf = urlEncode(ES_WRONGPARAM);
                        response = (char*)malloc(512);
                        sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                        free(buf);
                      }
                      break;
                    }
                    //************************************Storage**************************************************************************
                    case 102:
                    {
                      char *name = httpParser->getByName("p1");
                      if(name && *name)
                      {
                        char *ret = (char*)(*handlerList)[i]->handle(*command, name);
                        if(ret)
                        {                        
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(512 + strlen(ret));
                          sprintf(response, "n=1&e=%d&d=%s&v1=%s", E_OK, buf, ret);
                          free(buf);
                          delete []ret;
                        }
                        else
                        {                        
                          char *errbuf;
                          int code = (*handlerList)[i]->getError(&errbuf);
                          buf = urlEncode(errbuf);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", code, buf);
                          free(buf);
                        }
                      }
                      else
                      {
                        buf = urlEncode(ES_WRONGPARAM);
                        response = (char*)malloc(512);
                        sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                        free(buf);
                      }
                      break;
                    }
                    //************************************COMMON***************************************************************************
                    case 100:
                    {
                      if(*command >= 'a' && *command <= 'f')
                      {
                        time_t *retval = (time_t*)(*handlerList)[i]->handle(*command);
                        if(retval)
                        {
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(512);
                          sprintf(response, "n=1&e=%d&d=%s&v1=%ld", E_OK, buf, *retval);
                          free(buf);
                          delete retval;
                        }
                        else
                        {
                          char *errbuf;
                          int code = (*handlerList)[i]->getError(&errbuf);
                          buf = urlEncode(errbuf);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", code, buf);
                          free(buf);
                        }
                      }
                      if(*command == 'g')
                      {
                        int *retval = (int*)(*handlerList)[i]->handle(*command);
                        if(retval)
                        {
                          buf = urlEncode(ES_OK);
                          response = (char*)malloc(512);
                          sprintf(response, "n=1&e=%d&d=%s&v1=%d", E_OK, buf, *retval);
                          free(buf);
                          delete retval;
                        }
                        else
                        {
                          char *errbuf;
                          int code = (*handlerList)[i]->getError(&errbuf);
                          buf = urlEncode(errbuf);
                          response = (char*)malloc(512);
                          sprintf(response, "n=0&e=%d&d=%s", code, buf);
                          free(buf);
                        }

                      }
                      break;
                    }
                    //************************************Admin Part***************************************************************************
                    case 13:
                    {
                      bool *retval = NULL;
                      unsigned long long *intval = NULL;
                      if(*command == 's')
                      {
                        char *p1 = httpParser->getByName("p1");
                        if(p1)
                        {
                          retval = new bool;
                          *retval = true;
                          intval = (unsigned long long*)(*handlerList)[i]->handle(*command, atoi(p1));
                        }
                      }
                      else
                      {
                        if(*command == 'd' || *command == 'f')
                        {
                          char *p1 = httpParser->getByName("p1");
                          char *p2 = httpParser->getByName("p2");
                          if(p1 && p2)
                          {
                            retval = (bool*)(*handlerList)[i]->handle(*command, atoi(p1), atoi(p2));
                          }
                        }
                        else
                        {
                          char *p1 = httpParser->getByName("p1");
                          if(p1)
                          {
                            retval = (bool*)(*handlerList)[i]->handle(*command, atoi(p1));
                          }
                        }
                      }
                      response = (char*)malloc(512);
                      if(retval)
                      {
                        if(*retval)
                        {
                          buf = urlEncode(ES_OK);
                          if(intval)
                            sprintf(response, "n=0&e=%d&d=%s&v1=%u&v2=%u", E_OK, buf, (unsigned int)(*intval >> 32), (unsigned int)(*intval & 0xffffffff));
                          else 
                            sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                          free(buf);
                        }
                        else
                        {
                          buf = urlEncode(ES_UNKNOWN);
                          sprintf(response, "n=0&e=%d&d=%s", E_UNKNOWN, buf);
                          free(buf);
                        }
                        delete retval;
                      }
                      else
                      {
                        buf = urlEncode(ES_WRONGPARAM);
                        sprintf(response, "n=0&e=%d&d=%s", E_WRONGPARAM, buf);
                        free(buf);
                      }
                      if(intval)delete intval;
                      break;
                    }
                    case 141:
                    {
                      if(*command == 'a' || *command == 'b' || *command == 'c')
                      {
                        char *p1 = httpParser->getByName("p1");
                        if(p1)
                        {
                          char *ret = (char*)(*handlerList)[i]->handle(*command, p1);
                          response = (char*)malloc(512);
                          if(ret)
                          {
                            buf = urlEncode(ES_OK);
                            sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                            free(buf);
                            delete ret;
                          }
                          else
                          {
                            char *errbuf;
                            int code = (*handlerList)[i]->getError(&errbuf);
                            buf = urlEncode(errbuf);
                            response = (char*)malloc(512);
                            sprintf(response, "n=0&e=%d&d=%s", code, buf);
                            free(buf);
                          }
                        }
                      }
                      break;
                    }
                    case 142:
                    {
                      switch(*command)
                      {
                        case 'a':
                        case 'b':
                        {
                          char *p1 = httpParser->getByName("p1");
                          if(p1)
                          {
                            response = (char*)malloc(512);
                            (*handlerList)[i]->handle(*command, atoi(p1));
                            buf = urlEncode(ES_OK);
                            sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                            free(buf);
                          }
                          break;
                        }
                        case 'c':
                        {
                          char *p1 = httpParser->getByName("p1");
                          char *p2 = httpParser->getByName("p2");
                          char *p3 = httpParser->getByName("p3");
                          if(p1 && p2 && p3)
                          {
                            char *ret = (char*)(*handlerList)[i]->handle(*command, atoi(p1), atoi(p2), p3);
                            if(ret)
                            {
                              size_t len = strlen(ret);
                              response = (char *)malloc(len + 512);
                              buf = urlEncode(ES_OK);
                              int pos = sprintf(response, "n=1&e=%d&d=%s&v1=", E_OK, buf);                              
                              free(buf);
                              memcpy(response + pos, ret, len + 1);
                              free(ret);
                            }
                          }
                          break;
                        }
                      }
                      break;
                    }
                    case 12:
                    {
                      switch(*command)
                      {
                        case 'c':
                        case 'a':
                        {
                          char *p1 = httpParser->getByName("p1");
                          if(p1)
                          {
                            (*handlerList)[i]->handle(*command, atoi(p1));
                            response = (char*)malloc(512);
                            buf = urlEncode(ES_OK);
                            sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                            free(buf);
                          }
                          break;
                        }
                        case 'e':
                        case 'd':
                        case 'b':
                        {
                          (*handlerList)[i]->handle(*command);
                          response = (char*)malloc(512);
                          buf = urlEncode(ES_OK);
                          sprintf(response, "n=0&e=%d&d=%s", E_OK, buf);
                          free(buf);
                          break;
                        }
                      }
                      break;
                    }

                    //***************************************************************************************************************            
                    default:
                      response = (char*)malloc(512);
                      buf = urlEncode(ES_BADCOMMAND);
                      sprintf(response, "n=0&e=%d&d=%s", E_BADCOMMAND, buf);
                      free(buf);
                  }
                  break;
                }
              }
              mutex.unlock();
            }
          }
        }
        if(!response)
        {
          response = (char*)malloc(512);
          buf = urlEncode(ES_BADCOMMAND);
          sprintf(response, "n=0&e=%d&d=%s", E_BADCOMMAND, buf);
          free(buf);
        }
        sendResponse(response);
        free(response);
        delete httpParser;
        free(request);
      }
      delete clientSock;
    }
  }
  return NULL;
}

void AdminClient::sendResponse(const char *response)
{
  size_t contentLen = strlen(response);
  size_t headerLen = 0;
  char header[512];
  int status;
  size_t sendSize = 0;
  time_t startTime = time(NULL), timeOut = 30;
  headerLen = sprintf(header,"HTTP/1.0 200 OK\r\nConnection: close\r\nContent-Type: text/html; charset=utf-8\r\nAccept-Ranges: bytes\r\nServer: hceAdminPart\r\nContent-Length: %lu\r\n\r\n",(unsigned long)contentLen);
  do
  {
    while(!clientSock->selectWrite(status) && !exitFlag)
    {
      if(status != SOCK_OK)break;
      if(startTime + timeOut < time(NULL))break;
    }
    if(status == SOCK_OK && !exitFlag)
    {
      sendSize += clientSock->write(header + sendSize, headerLen - sendSize, status);
    }
  }while(sendSize < headerLen && status == SOCK_OK && !exitFlag && startTime + timeOut > time(NULL));
  sendSize = 0;
  while(sendSize < contentLen && status == SOCK_OK && !exitFlag && startTime + timeOut > time(NULL))
  {
    while(!clientSock->selectWrite(status) && !exitFlag)
    {
      if(status != SOCK_OK)break;
      if(startTime + timeOut < time(NULL))break;
    }
    if(status == SOCK_OK && !exitFlag)
    {
      sendSize += clientSock->write(response + sendSize, contentLen - sendSize, status);
    }
  }
}

char* AdminClient::readRequest(size_t &size)
{
  size_t partSize = 1024;
  char *data = (char*)malloc(partSize + 1);
  size_t dataSize = partSize;
  bool endRequest = false;
  int status = 0;
  bool isGet = false, isPost = false;
  size_t contentLenght = 0;
  time_t startTime = time(NULL), timeOut = 5;
  size = 0;
  do
  {
    while(!clientSock->selectRead(status))
    {
      if(exitFlag || status != SOCK_OK || startTime + timeOut < time(NULL))
      {
        size = 0;
        free(data);
        return NULL;
      }
    }
    if(status == SOCK_OK)
    {
      if(dataSize < size + partSize)
      {
        dataSize += partSize;
        data = (char*)realloc(data, dataSize + 1);
      }
      size += clientSock->readPart(data + size, partSize, status);
      if(status != SOCK_OK)
      {
        free(data);
        size = 0;
        return NULL;
      }
      data[size] = '\0';
      if(!isGet && !isPost)
      {
        char *str = strstr(data, "GET ");
        isGet = (str == data);
        if(!isGet)
        {
          str = strstr(data, "POST ");
          isPost = (str == data);
        }
      }
      if(isGet && size > 4)
      {
        endRequest = (data[size - 4] == '\r' && data[size - 3] == '\n' && data[size - 2] == '\r' && data[size - 1] == '\n');
      }
      if(isPost)
      {
        char *str = NULL;
        if(!contentLenght)
        {
          str = strstr(data, "Content-Length:");
          if(str)
          {
            str += strlen("Content-Lenght:");
            while(*str == ' ' || *str == '\t')str++;
            contentLenght = atol(str);
          }
        }
        str = strstr(data, "\r\n\r\n");
        if(!str)
        {          
          str = strstr(data, "\n\r\n\r");
          if(!str)
          {
            str = strstr(data, "\n\n");
            if(!str)
            {
              str = strstr(data, "\r\r");
            }
          }
        }
        if(str != NULL)
        {
          endRequest = (size - (str - data) >= contentLenght);
        }
      }
    }
  }while(size < MAX_REQUEST_SIZE && status == SOCK_OK && !endRequest && startTime + timeOut > time(NULL));
  if(size)
  {
    data = (char*)realloc(data, size + 1);
  }
  else
  {
    free(data);
    data = NULL;
  }
  return data;
}
