#ifndef MYSQL_SESSION
#define MYSQL_SESSION

#include <iostream>
#include <string>
#include <Poco/Data/MySQL/MySQL.h>
#include <Poco/Data/SQLite/Connector.h>
#include <Poco/Data/MySQL/Connector.h>
#include <Poco/Data/Session.h>
#include <Poco/Data/SessionFactory.h>
#include <Poco/SharedPtr.h>
#include <Poco/ScopedLock.h>
#include <Poco/Mutex.h>
#include <Poco/Bugcheck.h>
#include <Poco/Data/MySQL/MySQLException.h>
#include <Poco/Util/AbstractConfiguration.h>

#define DataBase_KEY "DataBase"
#define DataBase_HOST "DataBase.host"
#define DataBase_USER "DataBase.user"
#define DataBase_PASSWORD "DataBase.password"
#define DataBase_DB "DataBase.db"
#define DataBase_COMPRESS "DataBase.compress"
#define DataBase_COMPRESS_DEFAULT "true"
#define DataBase_AUTO_RECONNECT "DataBase.auto-reconnect"
#define DataBase_AUTO_RECONNECT_DEFAULT "true"
#define DataBase_TYPE "DataBase.type"

#define MYSQL "mysql"
#define SQLITE "sqlite"

//MYSQL - mysql
#define MYSQL_KEY Poco::Data::MySQL::Connector::KEY
//SQLite - SQLite
#define SQLITE_KEY Poco::Data::SQLite::Connector::KEY

namespace HCE {
    namespace database {
    	//thread-safe database
        class DBSession {
        private:
            Poco::SharedPtr<Poco::Data::Session> _pSession;
            Poco::Mutex mutex;
        public:
            //constructor can throw an exception
            DBSession (const std::string& KEY,const std::string& dbConnString);
            ~DBSession () throw ();
        public:
            template <typename T>
            Poco::Data::Statement operator << (const T& t) {
            	// Creates a Statement with the given data as SQLContent
                Poco::Mutex::ScopedLock lock(this->mutex);
                return *_pSession << t;
            }
            // Closes the connection
            void close() throw ();
            // Return true if session is connected.
            bool isConnected() throw();
        public:
            static std::string getConnectionString (Poco::AutoPtr<Poco::Util::AbstractConfiguration> _config); 
        };
    }
}
#endif
