#include <FileWriter.h>
FileWriter::FileWriter(ssize_t bufSize)
{
  lastErrNo = 0;
  int retry = 3;
  do{
  buffer = (char*)malloc(bufSize);
  if(!buffer)bufSize >>= 1;
  }while(--retry > 0 && buffer == NULL);
  if(!buffer)lastErrNo = errno;
  size = bufSize;
  file = 0;
  isOpen = false;
  pos = 0;
}

FileWriter::~FileWriter()
{
  this->close();
  free(buffer);
}

bool FileWriter::open(const char *name, mode_t mode)
{
  file = ::open(name, O_WRONLY|O_CREAT|O_TRUNC, mode);
  if(file < 0)
  {
    lastErrNo = errno;
    return false;
  }
  isOpen = true;
  return true;
}

bool FileWriter::flush()
{
  if(!buffer)return false;
  if(pos > 0)
  {
    if(::write(file, buffer, pos) != pos)
    {
      lastErrNo = errno;
      pos = 0;
      return false;
    }
    pos = 0;
  }
  return true;
}

bool FileWriter::write(const void *src, ssize_t wsize)
{
  char *buf = (char*)src;
  ssize_t delta = 0;
  if(!buffer)return false;
  while(wsize)
  {
    delta = (size - pos) > wsize ? wsize : (size - pos);
    memcpy(buffer + pos, buf, delta);
    buf += delta;
    pos += delta;
    wsize -= delta;
    if(size == pos)
    {
      if(::write(file, buffer, size) != size)
      {
        lastErrNo = errno;
        return false;
      }
      pos = 0;
    }
  }
  return true;
}

void FileWriter::close()
{
  if(isOpen)
  {
    this->flush();
    ::close(file);
    isOpen = false;
  }
}

