#include <HttpParser.h>
HttpParser::HttpParser(const char *content, size_t contentLen)
{
  method = -1;
  this->content = content;
  this->contentLen = contentLen;
  if(strstr(content, "GET "))
  {
    method = HTTP_GET;
    parseGET();
  }
  else
  {
    if(strstr(content, "POST "))
    {
      method = HTTP_POST;
      parsePOST();
    }
  }
}

void HttpParser::parseGET()
{
  const char *pos = strstr(content, "GET ");
  size_t idx = 0;
  if(pos)
  {
    pos += 4;
    idx = pos - content;
    while((*pos == ' ' || *pos == '\t') && idx < contentLen)
    {
      pos++;
      idx++;
    }
    while(idx < contentLen && *pos != '?' && !delimiter(*pos))
    {
      pos++;
      idx++;
    }
    if(*pos == '?')
    {
      parse(++pos);
    }
  }
}

void HttpParser::parsePOST()
{
  const char *pos = strstr(content, "\r\n\r\n");
  if(pos)
  {
    pos += 4;
    if(pos < content + contentLen)
    {
      parse(pos);
    }
  }
}

void HttpParser::parse(const char *pos)
{ 
  size_t idx = pos - content;
  while(idx < contentLen && !delimiter(*pos))
  {
    Variable *var = new(std::nothrow) Variable();
    if(!var)
    {
      break;
    }
    size_t len = 0;
    const char *buf = pos;        
    while(*pos != '=' && idx < contentLen && !delimiter(*pos))
    {
      pos++;
      idx++;        
    }
    len = pos - buf;
    var->name = (char*)malloc(len + 1);
    if(!var->name)
    {
      break;
    }
    if(len)
    {
      memcpy(var->name, buf, len);
    }
    var->name[len] = '\0';
    pos++;
    idx++;
    buf = pos;
    while(*pos != '&' && idx < contentLen && !delimiter(*pos))
    {
      pos++;
      idx++;
    }
    len = pos - buf;
    var->value = (char*)malloc(len + 1);
    if(!var->value)
    {
      break;
    }
    if(len)
    {
      memcpy(var->value, buf, len);
    }
    var->value[len] = '\0';
    vars.pushDown(var);
    if(idx < contentLen && *pos != '&')
    {
      break;
    }
    pos++;
    idx++;
  }
}

HttpParser::~HttpParser()
{
  for(unsigned long i = 0; i < vars.getCount(); i++)
  {
    free(vars[i]->name);
    free(vars[i]->value);
    delete vars[i];
  }
}

char* HttpParser::getByName(const char *var)
{
  for(unsigned long i = 0; i < vars.getCount(); i++)
  {
    if(strcmp(vars[i]->name, var) == 0)
    {
      return vars[i]->value;
    }
  }
  return NULL;
}


