#ifndef LOGHANDLER_H
#define LOGHANDLER_H
#include <stdarg.h>
#include <Syslog.h>
#include <new>
#include <BaseHandler.h>
class LogHandler:public BaseHandler
{
  public:
    LogHandler(SysLog *sysLog):BaseHandler(101),log(sysLog){}
    void* handle(char command, ...)
    {
      va_list list;
      int *maxLevel = NULL;
      setError(0, NULL);
      va_start(list, command);
      switch(command)
      {
        case 'a':
        {//set max level
          int state = va_arg(list, int);
          maxLevel = new(std::nothrow) int;
          if(!maxLevel)
          {
            setError(E_MEMORY, "Out of memory");
            return NULL;
          }
          *maxLevel = log->getMaxLevel();
          if(state >= LEV_EMERG && state <= LEV_DEBUG)
          {
            log->setMaxLevel(state);
          }
          else
          {
            setError(2, "Wrong level, value must be between 0 and 7");
          }
          break;
        }
        case 'b':
        {//set level mask
          int mask = va_arg(list, int);
          maxLevel = new(std::nothrow) int;
          if(!maxLevel)
          {
            setError(E_MEMORY, "Out of memory");
            return NULL;
          }
          *maxLevel = log->getLevelMask();
          if(mask <= 0xff)log->setLevelMask((u_int8_t)mask);
          break;
        }
        case 'c':
        {//get last error on level
          int level = va_arg(list, int);
          if(level >= LEV_EMERG && level <= LEV_DEBUG)
          {
            int errNo = 0;
            char *data = log->getLastLog(level, errNo);
            if(!data)data = strdup("");
            char *ret = new(std::nothrow) char[strlen(data) + 1 + sizeof(int)];
            if(!ret)
            {
              setError(E_MEMORY, "Out of memory");
              return NULL;
            }
            *((int*)ret) = errNo;
            memcpy(ret + sizeof(int), data, strlen(data) + 1);
            free(data);
            return ret;
          }
          else
          {
            setError(2, "Wrong level, value must be between 0 and 7");
          }
          break;
        }
        default:
        {
          setError(1, "Wrong command code");
        }
      }
      va_end(list);
      return maxLevel;
    }

  private:
    SysLog *log;
};

#endif
