#ifndef MULTI_MEDIA_COUNTERS_H
#define MULTI_MEDIA_COUNTERS_H

#include <AVLTree.h>

enum MultiMediaTypes {IMG_TYPE = 1, VIDEO_TYPE, SOUND_TYPE};

template <class idType>
class MultiMediaCounters
{
  public:
    MultiMediaCounters();
    ~MultiMediaCounters();

    void addMultimedia(idType resId, u_int16_t multiMediaMask);
    void delMultimedia(idType resId);
    u_int32_t getImgCount();
    u_int32_t getVideoCount();
    u_int32_t getSoundCount();

  private:
    AVLTree<idType, bool> resourcesImg;
    AVLTree<idType, bool> resourcesVideo;
    AVLTree<idType, bool> resourcesSound;
    
    void updateMultiMedia(MultiMediaTypes multiMediaType, idType resId);
};

template <class idType>
MultiMediaCounters<idType>::MultiMediaCounters(){}

template <class idType>
MultiMediaCounters<idType>::~MultiMediaCounters(){}

template <class idType>
void MultiMediaCounters<idType>::addMultimedia(idType resId, u_int16_t multiMediaMask)
{
  if (multiMediaMask & 0xFF){
    updateMultiMedia(IMG_TYPE, resId);       
  }
  if (multiMediaMask & 0x0100){
    updateMultiMedia(VIDEO_TYPE, resId);
  }
  if (multiMediaMask & 0x0200){
    updateMultiMedia(SOUND_TYPE, resId);
  }
}

template <class idType>
void MultiMediaCounters<idType>::updateMultiMedia(MultiMediaTypes multiMediaType, idType resId)
{
  switch (multiMediaType){
    case IMG_TYPE:
      if (!resourcesImg.find(resId)){
        resourcesImg.add(resId, true);
      }
      break;

    case VIDEO_TYPE:
      if (!resourcesVideo.find(resId)){
        resourcesVideo.add(resId, true);
      }
      break;

    case SOUND_TYPE:
      if (!resourcesSound.find(resId)){
        resourcesSound.add(resId, true);
      }
      break;
  }
}

template <class idType>
void MultiMediaCounters<idType>::delMultimedia(idType resId)
{
  resourcesImg.del(resId);
  resourcesVideo.del(resId);
  resourcesSound.del(resId);
}

template <class idType>
u_int32_t MultiMediaCounters<idType>::getImgCount()
{
  return resourcesImg.getNodesCount();
}

template <class idType>
u_int32_t MultiMediaCounters<idType>::getVideoCount()
{
  return resourcesVideo.getNodesCount();
}

template <class idType>
u_int32_t MultiMediaCounters<idType>::getSoundCount()
{
  return resourcesSound.getNodesCount();
}
#endif

