#ifndef MYSQLLITE_H
#define MYSQLLITE_H
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <mysql.h>
#include <string.h>

class PreparedQuery;

class MysqlLite
{
public:
  MysqlLite(const char *host, const char *user, const char *passwd,
            const char *db, unsigned int port = 0, const char *socket = NULL, bool connectNow = true);
   ~MysqlLite();

  bool query(const char *pattern, ...);
  MYSQL_RES *store();
  MYSQL_RES *use();
  MYSQL_ROW fetch(MYSQL_RES *);
  bool is_connect();
  my_ulonglong insertId();
  const char *errorStr();
  void freeResult(MYSQL_RES * res)
  {
    mysql_free_result(res);
  }
  bool setCharacterSet(const char *character);
  const char *errStr();
  unsigned int errNo();
  char *escapeStr(const char *str);
  my_ulonglong affectedRows();
  my_ulonglong numRows(MYSQL_RES * res);
  void setTimeout(unsigned int readTimeout, unsigned int writeTimeout);
  bool ping();
  bool reconnect();
  void disconnect();
private:
  bool connected;
  MYSQL *sql;
  friend class PreparedQuery;
  char *host;
  char *user;
  char *passwd;
  char *db;
  unsigned int port;
  char *socket;
};

class PreparedQuery
{
  public:
    PreparedQuery(const char *query, MysqlLite *sql);
    ~PreparedQuery();
    bool execute();
    MYSQL_BIND* operator[](int index);
    bool bind();
    const char *errorStr();
    unsigned int errNo(){return mysql_stmt_errno(stmt);}

  private:
    MYSQL_STMT    *stmt;
    MYSQL_BIND    *bindParam;
    int paramCount;
};
#endif
