#ifndef PERSON
#define PERSON

#include <string>
#include <Poco/Data/TypeHandler.h>
#include <Poco/Data/AbstractExtractor.h>
#include <Poco/Data/AbstractBinder.h>
#include <Poco/Data/AbstractPreparation.h>

using namespace Poco::Data;

struct Person {
    int id;
    std::string name;
public:
    Person():id(0),name("") {}
    Person(int Id,const std::string& Name):id(Id),name(Name){}
    
    bool operator==(const Person& other) const {
        return name == other.name;
    }

    bool operator < (const Person& p) const {
        if (this->name < p.name)
            return true;
        return false;
    }
};

namespace Poco {
namespace Data {

template <>
class TypeHandler<Person>
{
public:
        static void bind(std::size_t pos, const Person& obj, Poco::Data::AbstractBinder* pBinder)
        {
                // the table is defined as person (id int, name VARCHAR)
                poco_assert_dbg (pBinder != 0);
                pBinder->bind(pos++, obj.id);
                pBinder->bind(pos++, obj.name);
        }

        static void prepare(std::size_t pos, const Person& obj, Poco::Data::AbstractPreparation* pPrepare)
        {
                // the table is defined as person (id int, name VARCHAR)
                poco_assert_dbg (pPrepare != 0);
                pPrepare->prepare(pos++, obj.id);
                pPrepare->prepare(pos++, obj.name);
        }

        static std::size_t size()
        {
                return 2;
        }

        static void extract(std::size_t pos, Person& obj, const Person& defVal, Poco::Data::AbstractExtractor* pExt)
        {
                poco_assert_dbg (pExt != 0);
                if (!pExt->extract(pos++, obj.id))
                        obj.name = defVal.id;
                if (!pExt->extract(pos++, obj.name))
                        obj.name = defVal.name;
        }

private:
        TypeHandler();
        ~TypeHandler();
        TypeHandler(const TypeHandler&);
        TypeHandler& operator=(const TypeHandler&);
};

}
}

#endif
