#include <ProcFileSystem.h>

char *ProcFileSystem::file_get_contents(const char *filename, size_t & size)
{
  FILE *file;
  char *data = NULL;
  size_t data_size = 1024;
  size = 0;
  if ((file = fopen(filename, "rb")) != NULL)
  {
    data = (char *) malloc(data_size);
    if(!data)
    {
      return NULL;
    }
    while (!feof(file))
    {
      size += fread(data + size, 1, 512, file);
      if (size + 512 > data_size)
      {
        data_size += 512;
        data = (char *) realloc(data, data_size);
        if(!data)
        {
          fclose(file);
          return NULL;
        }
      }
    }
    fclose(file);
    data[size] = '\0';
  }
  return data;
}

char *ProcFileSystem::getUptime()
{
  size_t size;
  return file_get_contents("/proc/loadavg", size);
}

unsigned long long ProcFileSystem::getTotalMem()
{
  size_t size = 0;
  unsigned long long total_mem = (unsigned long long)(-1);
  char *data = NULL;
  int pos;
  data = file_get_contents("/proc/meminfo", size);
  if (!data)
    return (unsigned long long)(-1);
  pos = fast_str_pos(data, "MemTotal:");
  if (pos >= 0)
  {
    pos += 9;
    for (; pos < (int) size && (data[pos] == ' ' || data[pos] == '\t'); pos++); //skip spaces
    int numeric_start = pos;
    for (; data[pos] >= '0' && data[pos] <= '9' && pos < (int) size; pos++);
    data[pos] = '\0';
    total_mem = atoll(data + numeric_start);
  }
  free(data);
  return total_mem;
}

unsigned long long ProcFileSystem::getVmSize()
{
  size_t size = 0;
  unsigned long long total_mem = (unsigned long long)(-1);
  char *data = NULL;
  int pos;
  data = file_get_contents("/proc/self/status", size);
  if (!data)
    return (unsigned long long)(-1);
  pos = fast_str_pos(data, "VmSize:");
  if (pos >= 0)
  {
    pos += 7;
    for (; pos < (int) size && (data[pos] == ' ' || data[pos] == '\t'); pos++); //skip spaces
    int numeric_start = pos;
    for (; data[pos] >= '0' && data[pos] <= '9' && pos < (int) size; pos++);
    data[pos] = '\0';
    total_mem = atoll(data + numeric_start);
  }
  free(data);
  return total_mem;
}

unsigned long long ProcFileSystem::getFreeMem()
{
  size_t size = 0;
  unsigned long long total_mem = (unsigned long long)(-1);
  char *data = NULL;
  int pos;
  data = file_get_contents("/proc/meminfo", size);
  if (!data)
    return (unsigned long long)(-1);
  pos = fast_str_pos(data, "MemFree:");
  if (pos >= 0)
  {
    pos += 9;
    for (; pos < (int) size && (data[pos] == ' ' || data[pos] == '\t'); pos++); //skip spaces
    int numeric_start = pos;
    for (; data[pos] >= '0' && data[pos] <= '9' && pos < (int) size; pos++);
    total_mem = atoll(data + numeric_start);
  }
  pos = fast_str_pos(data, "Cached:");
  if (pos >= 0)
  {
    pos += 8;
    for (; pos < (int) size && (data[pos] == ' ' || data[pos] == '\t'); pos++); //skip spaces
    int numeric_start = pos;
    for (; data[pos] >= '0' && data[pos] <= '9' && pos < (int) size; pos++);
    data[pos] = '\0';
    total_mem += atoll(data + numeric_start);
  }
  pos = fast_str_pos(data, "Buffers:");
  if (pos >= 0)
  {
    pos += 9;
    for (; pos < (int) size && (data[pos] == ' ' || data[pos] == '\t'); pos++); //skip spaces
    int numeric_start = pos;
    for (; data[pos] >= '0' && data[pos] <= '9' && pos < (int) size; pos++);
    data[pos] = '\0';
    total_mem += atoll(data + numeric_start);
  }

  free(data);
  return total_mem;
}

unsigned long ProcFileSystem::getUTime()
{
  size_t size = 0;
  char *data = file_get_contents("/proc/self/stat", size);
  if (!data)
    return (unsigned long)(-1);
  int pid = 0;
  char com[256];
  char state;
  int ppid, pgrp, session, tty_nr, tpgid;
  unsigned int flags;
  unsigned long minflt, cminflt, majflt, cmajflt, utime, stime;
  sscanf(data, "%d %s %c %d %d %d %d %d %u %lu %lu %lu %lu %lu %lu", &pid, com, &state, &ppid, &pgrp, &session, &tty_nr, &tpgid, &flags, &minflt, &cminflt, &majflt, &cmajflt, &utime, &stime);
  free(data);
  return utime;
}

unsigned long ProcFileSystem::getSTime()
{
  size_t size = 0;
  char *data = file_get_contents("/proc/self/stat", size);
  if (!data)
    return (unsigned long)(-1);
  int pid = 0;
  char com[256];
  char state;
  int ppid, pgrp, session, tty_nr, tpgid;
  unsigned int flags;
  unsigned long minflt, cminflt, majflt, cmajflt, utime, stime;
  sscanf(data, "%d %s %c %d %d %d %d %d %u %lu %lu %lu %lu %lu %lu", &pid, com, &state, &ppid, &pgrp, &session, &tty_nr, &tpgid, &flags, &minflt, &cminflt, &majflt, &cmajflt, &utime, &stime);
  free(data);
  return stime; 
}

unsigned long long ProcFileSystem::getRunTime()
{
  char *data = NULL;
  long long uptime = 0;
  size_t size = 0;
  size_t pos = 0;
  data = file_get_contents("/proc/uptime", size);
  if (!data)
    return 0;
  for (; data[pos] >= '0' && data[pos] <= '9' && pos < size; pos++);
  data[pos] = '\0';
  uptime = atoll(data);
  free(data);
  return uptime;
}

char *ProcFileSystem::getHostName()
{
  size_t size;
  return file_get_contents("/proc/sys/kernel/hostname", size);
}

int ProcFileSystem::setHostName(char *hostname)
{
  FILE *file;
  size_t size = 0;
  file = fopen("/proc/sys/kernel/hostname", "wb");
  if (file)
  {
    size = fwrite(hostname, 1, strlen(hostname), file);
    fclose(file);
  }
  return size;
}

unsigned long long ProcFileSystem::getShmAll()
{
  char *data;
  size_t size;
  long long shmall = 0;
  data = file_get_contents("/proc/sys/kernel/shmall", size);
  if (data)
  {
    shmall = atoll(data);
    free(data);
  }
  return shmall;
}

int ProcFileSystem::setShmAll(unsigned long long shmall)
{
  FILE *file;
  char body[20];
  size_t size = 0;
  file = fopen("/proc/sys/kernel/shmall", "wb");
  if (file)
  {
    sprintf(body, "%llu", shmall);
    size = fwrite(body, 1, strlen(body), file);
    fclose(file);
  }
  return size;
}

unsigned long long ProcFileSystem::getShmMax()
{
  char *data;
  size_t size;
  long long shmall = 0;
  data = file_get_contents("/proc/sys/kernel/shmmax", size);
  if (data)
  {
    shmall = atoll(data);
    free(data);
  }
  return shmall;
}

int ProcFileSystem::setShmMax(unsigned long long shmmax)
{
  FILE *file;
  char body[20];
  size_t size = 0;
  file = fopen("/proc/sys/kernel/shmmax", "wb");
  if (file)
  {
    sprintf(body, "%llu", shmmax);
    size = fwrite(body, 1, strlen(body), file);
    fclose(file);
  }
  return size;
}

int ProcFileSystem::fast_str_pos(const char *str, const char *pattern)
{
  int plen;
  int d[256];
  register int c;
  int slen = strlen(str);
  register int pindx;
  register int cmppos;
  register int endpos;
  plen = strlen(pattern);
  for (c = 0; c < 256; c++)
    d[c] = plen;
  for (c = 0; c < plen - 1; c++)
    d[pattern[c] & 0377] = plen - 1 - c;
  for (endpos = plen - 1; endpos < slen;)
  {
    for (cmppos = endpos, pindx = (plen - 1); pindx >= 0; cmppos--, pindx--)
      if (str[cmppos] != pattern[pindx])
      {
        endpos += d[str[endpos] & 0377];
        break;
      }
    if (pindx < 0)
      return endpos - (plen - 1);
  }
  return -1;
}

unsigned long long ProcFileSystem::getVmRSS()
{
  size_t size = 0;
  unsigned long long total_mem = (unsigned long long)(-1);
  char *data = NULL;
  int pos;
  data = file_get_contents("/proc/self/status", size);
  if (!data)
    return (unsigned long long)(-1);
  pos = fast_str_pos(data, "VmRSS:");
  if (pos >= 0)
  {
    pos += 6;
    for (; pos < (int) size && (data[pos] == ' ' || data[pos] == '\t'); pos++); //skip spaces
    int numeric_start = pos;
    for (; data[pos] >= '0' && data[pos] <= '9' && pos < (int) size; pos++);
    data[pos] = '\0';
    total_mem = atoll(data + numeric_start);
  }
  free(data);
  return total_mem;
}

unsigned long long ProcFileSystem::getFreeMemSelf()
{
  unsigned long long total = getTotalMem();
  unsigned long long vm = getVmSize();
  if(total > vm) return total - vm;
  return 0;
}
