#ifndef STATUSHANDLER_H
#define STATUSHANDLER_H
#include <time.h>
#include <new>
#include <BaseHandler.h>
#include <AStorageHandler.h>
class StatusHandler:public BaseHandler
{
  public:
    enum State
    {
      RUN = 1, SUSPEND, STOP, SHUTDOWN, RESTART, RECOVER, NOTLOADED, LISTEN
    };
    StatusHandler(AStorageHandler *statistic, State state = RUN):BaseHandler(100),current(state), changeDate(0), statisticHandler(statistic)
    {
      statisticHandler->setValue("ApplicationState", (long long)state);
    }
    State getState(){return current;}
    bool recover(){return current == RECOVER;}
    bool shutdown(){return current == SHUTDOWN;}
    bool restart(){return current == RESTART;}
    bool stop(){return current == STOP;}
    bool suspend(){return current == SUSPEND;}
    bool run(){return current == RUN;}
    void setState(State state)
    {
      current = state;
      changeDate = time(NULL);
      statisticHandler->setValue("ApplicationState", (long long)current);
    }
    void* handle(char command, ...)
    {
      setError(0, NULL);
      time_t *date = NULL;
      switch(command)
      {
        case 'a':
        {
          if(current == RUN)
          { 
            setError(1, "Already run");
          }
          else
          {
            date = new(std::nothrow) time_t;
            if(!date)
            {
              setError(E_MEMORY, "Out of memory");
              return NULL;
            }
            changeDate = *date = time(NULL);
            current = RUN;
            statisticHandler->setValue("ApplicationState", 1ll);
          }
          break;
        }
        case 'b':
        {
          if(current != RUN && current != LISTEN)
          { 
            setError(2, "Not runing");
          }
          else
          {
            date = new(std::nothrow) time_t;
            if(!date)
            {
              setError(E_MEMORY, "Out of memory");
              return NULL;
            }
            changeDate = *date = time(NULL);
            current = SUSPEND;
            statisticHandler->setValue("ApplicationState", 2ll);
          }
          break;
        }
        case 'c':
        {
          if(current != RUN || current != SUSPEND)
          { 
            setError(3, "Not runing and not in suspend");
          }
          else
          {
            date = new(std::nothrow) time_t;
            if(!date)
            {
              setError(E_MEMORY, "Out of memory");
              return NULL;
            }
            changeDate = *date = time(NULL);
            current = STOP;
            statisticHandler->setValue("ApplicationState", 3ll);
          }
          break;
        }
        case 'd':
        {
          if(current == SHUTDOWN)
          { 
            setError(4, "Already shutdown");
          }
          else
          {
            date = new(std::nothrow) time_t;
            if(!date)
            {
              setError(E_MEMORY, "Out of memory");
              return NULL;
            }
            changeDate = *date = time(NULL);
            current = SHUTDOWN;
            statisticHandler->setValue("ApplicationState", 4ll);
          }
          break;
        }
        case 'e':
        {
          if(current == RESTART)
          { 
            setError(5, "Already in restart state");
          }
          else
          {
            date = new(std::nothrow) time_t;
            if(!date)
            {
              setError(E_MEMORY, "Out of Memory");
              return NULL;
            }
            changeDate = *date = time(NULL);
            current = RESTART;
            statisticHandler->setValue("ApplicationState", 5ll);
          }
          break;
        }
        case 'f':
        {
          date = new(std::nothrow) time_t;
          if(!date)
          {
            setError(E_MEMORY, "Out of memory");
            return NULL;
          }
          changeDate = *date = time(NULL);
          current = RECOVER;
          break;
        }
        default:
        {
          setError(1, "Wrong command code");
          break;
        }
      }
      return date;
    }

  private:
    State current;
    time_t changeDate;
    AStorageHandler *statisticHandler;
};

#endif
