/**
 * @file Thread.hpp
 *
 * ServerApplication - base class for application
 *
 * @author Eduard <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef THREAD
#define THREAD

#include <Poco/Thread.h>
#include "./exceptions/ThreadAlreadyRunningException.hpp"
#include "./exceptions/ThreadAlreadyJoinedException.hpp"
#include <Poco/Runnable.h>

namespace HCE {
namespace Utils {
	//!< class Poco::Runnable does not provide flag which
    //!< monitors thread state so I created own class Thread which provides all functions for runnable class.
    class Thread : public Poco::Runnable {
    private:
    	Poco::Thread _ownThread;
    protected:
    	bool _exitFlag;
    public:
    	Thread ();
    	virtual ~Thread();
    public:
    	bool isRunning ();
    	void start ();
    	void stop ();
    	void join ();
	public:
    	//!< must be overridden in inheritance classes
	    virtual void run () = 0;
    };
}
} 

#endif
