#include <DataBaseSessionTest.h>

void DataBaseSessionTest::setConnection (const std::string& key,const std::string& parameters) {
	try {
		_pSession = Poco::SharedPtr<HCE::database::DBSession>(new HCE::database::DBSession(key,parameters));
	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::closeConnection () {
	_pSession->close();
}

bool DataBaseSessionTest::isConnected () {
	return _pSession->isConnected();
}

void DataBaseSessionTest::selectTupleDataFromDataBase () {
	try {
		*_pSession << "SELECT * FROM `Person` WHERE id = 1", Poco::Data::Keywords::into(people) ,Poco::Data::Keywords::now;
	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::createDataBase () {
	try {
		*_pSession << "CREATE TABLE `Person` (Id INTEGER(3),Name VARCHAR(30))", Poco::Data::Keywords::now;
	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::dropDataBase () {
	try {
		*_pSession << "DROP TABLE IF EXISTS `Person`" , Poco::Data::Keywords::now;
	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::clearDataBase () {
	try {
		*_pSession << "DELETE FROM `Person` WHERE 1" , Poco::Data::Keywords::now;
	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::insertTupleInDataBase () {
	try {
		*_pSession << "INSERT INTO `Person` VALUES (?,?)" , Poco::Data::Keywords::use(people), Poco::Data::Keywords::now;
		people.clear();
	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::insertObjectInDataBase() {
	try {
		*_pSession << "INSERT INTO `Person` VALUES (?,?)" , Poco::Data::Keywords::use(objects), Poco::Data::Keywords::now;
		objects.clear();
	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::selectAllTuplesDataFromDataBase () {
	try {
		*_pSession << "SELECT * FROM `Person`", Poco::Data::Keywords::into(people) ,Poco::Data::Keywords::now;
	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::selectOneFieldFormDataBaseInTuple () {
	try {
		*_pSession << "SELECT Name FROM `Person`", Poco::Data::Keywords::into(namesOfPersons) ,Poco::Data::Keywords::now;
	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::selectObjectFromDataBase () {
	try {
		*_pSession << "SELECT id,Name FROM `Person` WHERE id = 3", Poco::Data::Keywords::into(objects) ,Poco::Data::Keywords::now;
 	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::countDataInDataBase (int& count) {
	try {
		*_pSession << "SELECT COUNT(*) FROM `Person`", Poco::Data::Keywords::into(count) , Poco::Data::Keywords::now;
	}
	catch (const Poco::Exception& ex) {
		std::cout << ex.displayText() << std::endl;
		throw;
	}
}

void DataBaseSessionTest::clearVector () throw() {
	this->people.clear();
	this->objects.clear();
}

void DataBaseSessionTest::SetUp () {
	this->people.push_back(person(1,"Person1"));
	this->people.push_back(person(2,"Person2"));

	this->objects.push_back(Object(3,"Object3"));
	this->objects.push_back(Object(4,"Object4"));
}

void DataBaseSessionTest::TearDown () {
		this->clearVector();
}
