#include <MySQLSessionTests.h>
#include <SQLiteSessionTests.h>

/*
 * MySQL tests
 */
TEST (MySQLSessionTest,CheckMySQLConnectionWithCorrectConnectionParameters) {
	MySQLSessionTests mysql;
	ASSERT_NO_THROW(mysql.setConnection(MYSQL_KEY,CORRECT_PARAMETERS_MySQL));
	bool isConnected = false;
	isConnected = mysql.isConnected();
	EXPECT_EQ (isConnected,true);
	ASSERT_NO_THROW(mysql.closeConnection());
}

TEST (MySQLSessionTest,CheckMySQLConnectionWithIncorrectConnectionParameters) {
	MySQLSessionTests mysql;
	ASSERT_THROW(mysql.setConnection(MYSQL_KEY,INCORRECT_PARAMETERS_MySQL), Poco::Exception);
}

TEST_F (MySQLSessionTests,insertTupleInDataBase) {
	ASSERT_NO_THROW(MySQLSessionTests::insertTupleInDataBase());
	ASSERT_NO_THROW(MySQLSessionTests::selectAllTuplesDataFromDataBase());
	std::vector<person>::const_iterator it = people.begin();
	EXPECT_EQ (it->get<0>(),1);
	EXPECT_STREQ (it->get<1>().c_str(),"Person1");
	++it; //go to the next element
	EXPECT_EQ (it->get<0>(),2);
	EXPECT_STREQ (it->get<1>().c_str(),"Person2");
}

TEST_F (MySQLSessionTests,selectOneFieldFromDataBase) {
	ASSERT_NO_THROW(MySQLSessionTests::insertTupleInDataBase());
	ASSERT_NO_THROW(MySQLSessionTests::selectOneFieldFormDataBaseInTuple ());
	std::vector<person_name>::const_iterator it = namesOfPersons.begin();
	EXPECT_STREQ (it->get<0>().c_str(),"Person1");
	++it; //go to the next element
	EXPECT_STREQ (it->get<0>().c_str(),"Person2");
}

TEST_F (MySQLSessionTests,CountDataInEmptyDataBase) {
	int count = -1;
	ASSERT_NO_THROW(MySQLSessionTests::countDataInDataBase (count));
	EXPECT_EQ (count,0);
}

TEST_F (MySQLSessionTests,CountDataInDataBase) {
	int count = -1;
	MySQLSessionTests::insertTupleInDataBase();
	ASSERT_NO_THROW(MySQLSessionTests::countDataInDataBase(count));
	EXPECT_EQ (count,2);
}

TEST_F (MySQLSessionTests,DeleteDataFromDataBase) {
	MySQLSessionTests::insertTupleInDataBase();
	ASSERT_NO_THROW (MySQLSessionTests::clearDataBase ());
	int count = -1;
	MySQLSessionTests::countDataInDataBase (count);
	EXPECT_EQ (count,0);
}

TEST_F (MySQLSessionTests,UseTypeHandler) {
	ASSERT_NO_THROW(MySQLSessionTests::insertObjectInDataBase());
	ASSERT_NO_THROW(MySQLSessionTests::selectObjectFromDataBase());
	std::vector<Object>::const_iterator it = objects.begin();
	EXPECT_EQ (it->id,3);
	EXPECT_STREQ (it->name.c_str(),"Object3");
}
/*
 * end MySQL tests
 */

/*
 * SQLite tests
 */
TEST (SQLiteSessionTest,CheckSQLiteConnectionWithCorrectConnectionParameters) {
	SQLiteSessionTests sqlite;
	ASSERT_NO_THROW(sqlite.setConnection(SQLITE_KEY,CORRECT_PARAMETERS_SQLite));
	bool isConnected = false;
	isConnected = sqlite.isConnected();
	EXPECT_EQ (isConnected,true);
	ASSERT_NO_THROW(sqlite.closeConnection());
}

TEST_F (SQLiteSessionTests,insertTupleInDataBase) {
	ASSERT_NO_THROW(SQLiteSessionTests::insertTupleInDataBase());
	ASSERT_NO_THROW(SQLiteSessionTests::selectAllTuplesDataFromDataBase());
	std::vector<person>::const_iterator it = people.begin();
	EXPECT_EQ (it->get<0>(),1);
	EXPECT_STREQ (it->get<1>().c_str(),"Person1");
	++it; //go to the next element
	EXPECT_EQ (it->get<0>(),2);
	EXPECT_STREQ (it->get<1>().c_str(),"Person2");
}

TEST_F (SQLiteSessionTests,selectOneFieldFromDataBase) {
	ASSERT_NO_THROW(SQLiteSessionTests::insertTupleInDataBase());
	ASSERT_NO_THROW(SQLiteSessionTests::selectOneFieldFormDataBaseInTuple ());
	std::vector<person_name>::const_iterator it = namesOfPersons.begin();
	EXPECT_STREQ (it->get<0>().c_str(),"Person1");
	++it; //go to the next element
	EXPECT_STREQ (it->get<0>().c_str(),"Person2");
}

TEST_F (SQLiteSessionTests,CountDataInEmptyDataBase) {
	int count = -1;
	ASSERT_NO_THROW(SQLiteSessionTests::countDataInDataBase (count));
	EXPECT_EQ (count,0);
}

TEST_F (SQLiteSessionTests,CountDataInDataBase) {
	int count = -1;
	SQLiteSessionTests::insertTupleInDataBase();
	ASSERT_NO_THROW(SQLiteSessionTests::countDataInDataBase(count));
	EXPECT_EQ (count,2);
}

TEST_F (SQLiteSessionTests,DeleteDataFromDataBase) {
	SQLiteSessionTests::insertTupleInDataBase();
	ASSERT_NO_THROW (SQLiteSessionTests::clearDataBase ());
	int count = -1;
	SQLiteSessionTests::countDataInDataBase (count);
	EXPECT_EQ (count,0);
}

TEST_F (SQLiteSessionTests,UseTypeHandler) {
	ASSERT_NO_THROW(SQLiteSessionTests::insertObjectInDataBase());
	ASSERT_NO_THROW(SQLiteSessionTests::selectObjectFromDataBase());
	std::vector<Object>::const_iterator it = objects.begin();
	EXPECT_EQ (it->id,3);
	EXPECT_STREQ (it->name.c_str(),"Object3");
}
/*
 * end SQLite tests
 */

int main(int argc, char **argv) {
  ::testing::InitGoogleTest(&argc, argv);
  return RUN_ALL_TESTS();
}
