#include <MultiMediaCounters.h>
#include <gtest/gtest.h>
#include <iostream>

template <typename T>
class MultiMediaCountersTest: public ::testing::Test{
  protected:

    MultiMediaCounters<T> multiMediaCounters;
};

typedef ::testing::Types<u_int16_t, u_int32_t, u_int64_t> TestTypes;
TYPED_TEST_CASE(MultiMediaCountersTest, TestTypes);

TYPED_TEST(MultiMediaCountersTest, init)
{
  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getImgCount());
  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getVideoCount());
  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getSoundCount());
}


TYPED_TEST(MultiMediaCountersTest, addOneImg)
{
  TestFixture::multiMediaCounters.addMultimedia(1, 127);

  EXPECT_EQ(1U, TestFixture::multiMediaCounters.getImgCount());
  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getVideoCount());
  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getSoundCount());
}


TYPED_TEST(MultiMediaCountersTest, addOneVideo)
{
  TestFixture::multiMediaCounters.addMultimedia(1, 256);

  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getImgCount());
  EXPECT_EQ(1U, TestFixture::multiMediaCounters.getVideoCount());
  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getSoundCount());
}


TYPED_TEST (MultiMediaCountersTest, addOneSound)
{
  TestFixture::multiMediaCounters.addMultimedia(1, 512);

  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getImgCount());
  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getVideoCount());
  EXPECT_EQ(1U, TestFixture::multiMediaCounters.getSoundCount());
}


TYPED_TEST (MultiMediaCountersTest, addVideoSound)
{
  TestFixture::multiMediaCounters.addMultimedia(1, 256 + 512);

  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getImgCount());
  EXPECT_EQ(1U, TestFixture::multiMediaCounters.getVideoCount());
  EXPECT_EQ(1U, TestFixture::multiMediaCounters.getSoundCount());
}


TYPED_TEST (MultiMediaCountersTest, addManyEqualResources)
{
  TestFixture::multiMediaCounters.addMultimedia(1, 256 + 512);
  TestFixture::multiMediaCounters.addMultimedia(1, 255 + 256 + 512);

  EXPECT_EQ(1U, TestFixture::multiMediaCounters.getImgCount());
  EXPECT_EQ(1U, TestFixture::multiMediaCounters.getVideoCount());
  EXPECT_EQ(1U, TestFixture::multiMediaCounters.getSoundCount());
}


TYPED_TEST (MultiMediaCountersTest, addManyResources)
{
  TestFixture::multiMediaCounters.addMultimedia(1, 32);
  TestFixture::multiMediaCounters.addMultimedia(2, 32 + 256 + 512);
  TestFixture::multiMediaCounters.addMultimedia(3, 512);
  TestFixture::multiMediaCounters.addMultimedia(4, 32 + 512);

  EXPECT_EQ(3U, TestFixture::multiMediaCounters.getImgCount());
  EXPECT_EQ(1U, TestFixture::multiMediaCounters.getVideoCount());
  EXPECT_EQ(3U, TestFixture::multiMediaCounters.getSoundCount());
}


TYPED_TEST (MultiMediaCountersTest, addAndDeleteResource)
{
  TestFixture::multiMediaCounters.addMultimedia(1, 32 + 256 + 512);
  TestFixture::multiMediaCounters.delMultimedia(1);

  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getImgCount());
  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getVideoCount());
  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getSoundCount());
}


TYPED_TEST (MultiMediaCountersTest, addAndDeleteResources)
{
  TestFixture::multiMediaCounters.addMultimedia(1, 32);
  TestFixture::multiMediaCounters.addMultimedia(2, 32 + 256 + 512);
  TestFixture::multiMediaCounters.addMultimedia(3, 512);
  TestFixture::multiMediaCounters.addMultimedia(4, 32 + 512);
  TestFixture::multiMediaCounters.delMultimedia(2);
  TestFixture::multiMediaCounters.delMultimedia(3);

  EXPECT_EQ(2U, TestFixture::multiMediaCounters.getImgCount());
  EXPECT_EQ(0U, TestFixture::multiMediaCounters.getVideoCount());
  EXPECT_EQ(1U, TestFixture::multiMediaCounters.getSoundCount());
}
