#ifndef TYPE_HANDLER
#define TYPE_HANDLER

#include <Poco/SharedPtr.h>
#include <Poco/DateTimeFormatter.h>
#include <Poco/Data/SessionFactory.h>
#include <Poco/Data/Session.h>
#include <Poco/Data/TypeHandler.h>
#include <Poco/Data/SQLite/Connector.h>
#include <vector>
#include <iostream>


using namespace Poco::Data::Keywords;
using Poco::DateTime;
using Poco::DateTimeFormatter;
using Poco::Data::Session;
using Poco::Data::Statement;


struct Object {
public:
	int id;
	std::string name;
public:
	Object ():id(0),name(""){}
	Object (int _id,const std::string& _name):id(_id),name(_name){}
	~Object () {
		id = 0;
		name = "";
	}
};


namespace Poco {
namespace Data {

template <>
class TypeHandler<Object> {
public:
	static std::size_t size()
	{
		return 2;
	}
	
	static void bind(std::size_t pos, const Object& person, AbstractBinder* pBinder, AbstractBinder::Direction dir)
	{
		TypeHandler<int>::bind(pos++, person.id, pBinder, dir);
		TypeHandler<std::string>::bind(pos++, person.name, pBinder, dir);
	}
	
	static void extract(std::size_t pos, Object& person, const Object& deflt, AbstractExtractor* pExtr)
	{
		TypeHandler<int>::extract(pos++, person.id, deflt.id, pExtr);
		TypeHandler<std::string>::extract(pos++, person.name, deflt.name, pExtr);
	}
	
	static void prepare(std::size_t pos, const Object& person, AbstractPreparator* pPrep)
	{
		TypeHandler<int>::prepare(pos++, person.id, pPrep);
		TypeHandler<std::string>::prepare(pos++, person.name, pPrep);
	}
};


} } // namespace Poco::Data

#endif

/*int main(int argc, char** argv)
{
	// register SQLite connector
	Poco::Data::SQLite::Connector::registerConnector();
	
	// create a session
	Session session("SQLite", "sample.db");

	// drop sample table, if it exists
	session << "DROP TABLE IF EXISTS Person", now;
	
	// (re)create table
	session << "CREATE TABLE Person (Name VARCHAR(30), Address VARCHAR, Age INTEGER(3), Birthday DATE)", now;
	
	// insert some rows
	Person person = 
	{
		"Bart Simpson",
		"Springfield",
		10,
		DateTime(1980, 4, 1)
	};
	
	Statement insert(session);
	insert << "INSERT INTO Person VALUES(?, ?, ?, ?)",
		use(person);
		
	insert.execute();
	
	person.name    = "Lisa Simpson";
	person.address = "Springfield";
	person.age     = 8;
	person.birthday = DateTime(1982, 5, 9);

	insert.execute();
	
	// a simple query
	Statement select(session);
	select << "SELECT Name, Address, Age, Birthday FROM Person",
		into(person),
		range(0, 1); //  iterate over result set one row at a time
		
	while (!select.done())
	{
		select.execute();
		std::cout << person.name << "\t"
			<< person.address << "\t"
			<< person.age << "\t"
			<< DateTimeFormatter::format(person.birthday, "%b %d %Y") 
		<< std::endl;
	}
	
	// another query - store the result in a container
	std::vector<Person> persons;
	session << "SELECT Name, Address, Age, Birthday FROM Person",
		into(persons),
		now;
		
	for (std::vector<Person>::const_iterator it = persons.begin(); it != persons.end(); ++it)
	{
		std::cout << it->name << "\t"
			<< it->address << "\t"
			<< it->age << "\t"
			<< DateTimeFormatter::format(it->birthday, "%b %d %Y") 
		<< std::endl;
	}
	
	return 0;
}*/
