/**
 * @file sample.cpp
 *
 * Example Application main module. Example of how use encodeBase64 and decodeBase64 global functions.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#include <sstream>
#include <iostream>

#include "EncodeDecodeBase64.hpp"

void printTitle(const std::string& title)
{
	std::cout << std::string(title.length(), '=') << std::endl << title << std::endl << std::string(title.length(), '=') << std::endl;
} 

void sampleEncodeBase64(void)
{
	printTitle("Sample use encodeBase64");

	const std::string data = "Hello world!";
	std::string encodeString = HCE::encodeBase64(data);

	std::cout << "Input string: " << data << std::endl;
	std::cout << "Encode string: " << encodeString << std::endl;

#if JSON_USE_BASE64 == 1
	std::cout << "JSON_USE_BASE64 == 1" << std::endl;
	std::cout << ((encodeString=="SGVsbG8gd29ybGQh")?"Success":"Fail") << std::endl;
#else
	std::cout << "JSON_USE_BASE64 != 1" << std::endl;
	std::cout << ((encodeString==data)?"Success":"Fail") << std::endl;
#endif
}

void sampleDecodeBase64(void)
{
	printTitle("Sample use decodeBase64");

	const std::string data = "Hello world!";
	std::string encodeString;

#if JSON_USE_BASE64 == 1
	std::cout << "JSON_USE_BASE64 == 1" << std::endl;
	encodeString = "SGVsbG8gd29ybGQh";
#else
	std::cout << "JSON_USE_BASE64 != 1" << std::endl;
	encodeString = data;
#endif

	std::string decodeString = HCE::decodeBase64(encodeString);

	std::cout << "Input string: " << encodeString << std::endl;
	std::cout << "Decode string: " << decodeString << std::endl;

	std::cout << ((decodeString==data)?"Success":"Fail") << std::endl;
}

int main(void)
{
	sampleEncodeBase64();
	sampleDecodeBase64();

	return 0;
}
