/**
 * @file main.cpp
 *
 * Test Application main module. Tests of how use encodeBase64 and decodeBase64 global functions.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#include <gtest/gtest.h>
#include <Poco/Process.h>
#include <sstream>

#include "EncodeDecodeBase64.hpp"

const std::string decodeData = "Hello world!";
const std::string encodeData = "SGVsbG8gd29ybGQh";

TEST(EncodeDecodeBase64, encodeBase64)
{
	std::string encodeString = HCE::encodeBase64(decodeData);

	ASSERT_FALSE(encodeString.empty());

#if JSON_USE_BASE64 == 1
	ASSERT_EQ(encodeString, encodeData);
#else
	ASSERT_EQ(encodeString, decodeData);
#endif
}

TEST(EncodeDecodeBase64, decodeBase64)
{
	std::string encodeString;

#if JSON_USE_BASE64 == 1
	encodeString = encodeData;
#else
	encodeString = decodeData;
#endif

	std::string decodeString = HCE::decodeBase64(encodeString);

	ASSERT_FALSE(decodeString.empty());
	ASSERT_EQ(decodeString, decodeData);
}

int main (int argc, char** argv)
{
  testing::InitGoogleTest(&argc, argv);
  return RUN_ALL_TESTS();
}
