/**
 * @file Command.hpp
 *
 * Implementation class Command of a exec tools need to HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef COMMAND_HPP
#define COMMAND_HPP

#include <sstream>
#include <vector>

namespace HCE
{
//-----------------------------------------------------------------------------
class Command
{
public:
  typedef std::vector<std::pair<std::string, std::string> > Env;

  explicit Command(const std::string& commandLine, const std::string& shellName="");
  Command(const std::string& commandLine, const std::string& initialDirectory, const std::string& inputStream_, const std::string& shellName="");
  Command(const std::string& commandLine, const std::string& inputStream_, const Command::Env& env, const std::string& shellName="");
  Command(const std::string& commandLine, const std::string& initialDirectory, const std::string& inputStream_, const Command::Env& env, const std::string& shellName="");
  ~Command(void) {}

  void setShellName(const std::string& shellName_) {shellName=shellName_;}
  std::string getShellName(void) const {return shellName;}

  void setCommandLine(const std::string& commandLine_) {commandLine=commandLine_;}
  std::string getCommandLine(void) const {return commandLine;}

  void setInitialDirectory(const std::string& initialDirectory_) {initialDirectory=initialDirectory_;}
  std::string getInitialDirectory(void) const {return initialDirectory;}

  void setInputStream(const std::string& inputStream_) {inputStream=inputStream_;}
  std::string getInputStream(void) const {return inputStream;}

  void setEnvironments(const Command::Env& env_) {env=env_;}
  const Command::Env& getEnvironments(void) const {return env;}

private:
  std::string shellName;
  std::string commandLine;
  std::string initialDirectory;
  std::string inputStream;
  Env env;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE 

#endif // COMMAND_HPP
