#include <string.h>
#include <fstream>

#include "CommandExecutor.hpp"

namespace HCE
{
//-----------------------------------------------------------------------------
Poco::SharedPtr<CommandResultData> CommandExecutor::execute(const Command& command, Process::CallbackFunction fn, unsigned int allowedTime)
{
  Poco::SharedPtr<CommandResultData> pCommandResultData = new CommandResultData();
  if (!pCommandResultData.isNull())
  {
    try
    {
      Poco::Process::Env env;
      const Command::Env& refEnv = command.getEnvironments();
      for (size_t i=0;i<refEnv.size();++i)
        env[refEnv[i].first] = refEnv[i].second;

      HCE::StarterExecutor starterExecutor(command.getInitialDirectory(), command.getShellName());
      pid_t pid = starterExecutor.execute(command.getCommandLine(), command.getInputStream(), env);

      if (fn)
        fn(pid);

      unsigned int exitStatus = starterExecutor.wait(allowedTime);

      pCommandResultData->setExitStatus(exitStatus);
      pCommandResultData->setProcessId(pid);
      pCommandResultData->setOutStream(starterExecutor.getOutStream());
      pCommandResultData->setErrStream(starterExecutor.getErrStream());
      statusExecution = starterExecutor.getStatusExecution();
    }
    catch(Poco::Exception& e)
    {
      pCommandResultData->setIsError(true);
      pCommandResultData->getErrStream() << " " << e.displayText();
    }
    catch(std::exception& e)
    {
      pCommandResultData->setIsError(true);
      pCommandResultData->getErrStream() << " " << e.what();
    }
  }
  return pCommandResultData;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE 
