/**
 * @file CommandResultData.hpp
 *
 * Implementation class CommandResultData of a exec tools need to HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef COMMAND_RESULT_DATA_HPP
#define COMMAND_RESULT_DATA_HPP

#include <sstream>

namespace HCE
{
//-----------------------------------------------------------------------------
class CommandResultData
{
public:
  CommandResultData(void);
  CommandResultData(const CommandResultData&);
  ~CommandResultData(void) {}

  CommandResultData& operator=(const CommandResultData&);

  void clear(void);

  void setOutStream(const std::string& stdOut);
  void setOutStream(const std::stringstream& outStream_);
  std::stringstream& getOutStream(void) {return outStream;}

  void setErrStream(const std::string& stdErr);
  void setErrStream(const std::stringstream& errStream_);
  std::stringstream& getErrStream(void) {return errStream;}

  void setExitStatus(unsigned int exitStatus_) {exitStatus=exitStatus_;}
  unsigned int getExitStatus(void) const {return exitStatus;}

  void setIsError(bool isError_) {isError=isError_;}
  bool getIsError(void) const {return isError;}

  void setProcessId(pid_t processId_) {processId=processId_;}
  pid_t getProcessId(void) const {return processId;}
private:
  std::stringstream outStream;
  std::stringstream errStream;
  unsigned int exitStatus;
  bool isError;
  pid_t processId;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE 

#endif // COMMAND_RESULT_DATA_HPP
