#include <sstream>
#include <iterator>
#include <string.h>
#include <errno.h>
#include <stdexcept>
#include <Poco/Process.h>
#include <Poco/PipeStream.h>
#include <Poco/Exception.h>

#include "ProcExec.hpp"

namespace HCE
{
//-----------------------------------------------------------------------------
ProcExec::ProcExec(const std::string& command_)
: command(""), args(), env(), initialDirectory(""), _IsError(false), exitStatus(0)
{
  std::stringstream istr(command_), ostr;
  std::istream_iterator<std::string> eos;
  std::istream_iterator<std::string> iit(istr);

  ostr << *iit;
  command = ostr.str();

  while(++iit!=eos)
  {
    args.push_back(*iit);
  }
}
//-----------------------------------------------------------------------------
ProcExec::ProcExec(const std::string& command_, const Poco::Process::Args& args_)
: command(command_), args(args_), env(), initialDirectory(""), _IsError(false), exitStatus(0)
{
}
//-----------------------------------------------------------------------------
ProcExec::ProcExec(const std::string& command_, const Poco::Process::Args& args_, const Poco::Process::Env& env_)
: command(command_), args(args_), env(env_), initialDirectory(""), _IsError(false), exitStatus(0)
{
}
//-----------------------------------------------------------------------------
bool ProcExec::exec(std::ostream& os, std::ostream& es)
{
  _IsError = true;

  try
  {
    Poco::Pipe outPipe, errPipe;
    Poco::ProcessHandle ph = Poco::Process::launch(command, args, initialDirectory, 0, &outPipe, &errPipe, env);

    Poco::PipeInputStream ostr(outPipe);
    Poco::PipeInputStream estr(errPipe);

    exitStatus = ph.wait();
    if (exitStatus != 0)
      throw std::runtime_error(std::string("ProcExec::exec: ")+strerror(errno));

    os << ostr.rdbuf();
    es << estr.rdbuf();

    _IsError = false;
  }
  catch(Poco::Exception& e)
  {
    es << " " << e.displayText();
  }
  catch(std::exception& e)
  {
    es << " " << e.what();
  }
  return !_IsError;
}
//-----------------------------------------------------------------------------
std::ostream& operator << (std::ostream& os, const ProcExec& procExec)
{
  const_cast<ProcExec&>(procExec).exec(os, os);
  return os;
}
//-----------------------------------------------------------------------------
std::string& operator << (std::string& str, const ProcExec& procExec)
{
  std::stringstream outMsg;
  outMsg << procExec;
  if (!procExec.isError())
    str = outMsg.str();
  return str;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE



