/**
 * @file ProcExec.hpp
 *
 * Implementation of a ProcExec tools, possibility execute process and control it worked.
 *
 * @author Alexander Vybornyh <alexander.asm.cluster@gmail.com>
 * @link http://asm-search-engine.com/cluster/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://asm-search-engine.com/cluster/license/
 * @package ASM Cluster node API
 * @since 0.1
 */

#ifndef PROC_EXEC_HPP
#define PROC_EXEC_HPP

#include <iostream>
#include <Poco/Process.h>

namespace HCE
{
//-----------------------------------------------------------------------------
class ProcExec
{
public:
  explicit ProcExec(const std::string& command_);
  ProcExec(const std::string& command_, const Poco::Process::Args& args_);
  ProcExec(const std::string& command_, const Poco::Process::Args& args_, const Poco::Process::Env& env_);
  virtual ~ProcExec(void) {};

  bool exec(std::ostream& os=std::cout, std::ostream& es=std::cerr);

  void setCommand(const std::string& command_) {command=command_;}
  std::string getCommand(void) const {return command;}

  void setArgs(const Poco::Process::Args& args_) {args=args_;}
  Poco::Process::Args& getArgs(void) {return args;}

  void setEnv(const Poco::Process::Env& env_) {env=env_;}
  Poco::Process::Env& getEnv(void) {return env;}

  void setInitialDirectory(const std::string& initialDirectory_) {initialDirectory=initialDirectory_;}
  std::string getInitialDirectory(void) const {return initialDirectory;}

  bool isError(void) const {return _IsError;}
  unsigned int getExitStatus(void) const {return exitStatus;}
private:
  std::string command;
  Poco::Process::Args args;
  Poco::Process::Env env;
  std::string initialDirectory;
  bool _IsError;
  unsigned int exitStatus;


  friend std::ostream& operator << (std::ostream& os, const ProcExec& procExec);
  friend std::string& operator << (std::string& str, const ProcExec& procExec);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE

#endif // PROC_EXEC_HPP

