/**
 * @file Process.hpp
 *
 * Implementation class Process of a exec tools need to HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef PROCESS_HPP
#define PROCESS_HPP

#include <signal.h>
#include <functional>
#include <Poco/Process.h>

#include "ProcessHandle.hpp"

void signalHandler(int signum);

namespace HCE
{
//-----------------------------------------------------------------------------
/* list all possible exit status use in launch */
enum ExitStatus
{
  SUCCESS=0,                // exit status success (no errors)
  ERROR_FDOPEN_WRITE=1,     // error fdopen() write
  ERROR_FDOPEN_READ=2,      // error fdopen() read
  ERROR_WAITPID=3           // error waitpid()
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class Process
{
public:
  typedef std::function<void(pid_t)> CallbackFunction;

  static ProcessHandle launch(
    const std::string& command,
    const Poco::Process::Args& args,
    const std::string& initialDirectory,
    std::istream& in,
    std::ostream& out,
    std::ostream& err,
    const Poco::Process::Env& env,
    CallbackFunction fn);

  static ProcessHandle launch(
    const std::string& command,
    const Poco::Process::Args& args,
    const std::string& initialDirectory,
    std::istream& in,
    std::ostream& out,
    std::ostream& err,
    const Poco::Process::Env& env);

  static ProcessHandle launch(
    const std::string& command,
    const Poco::Process::Args& args,
    std::istream& in,
    std::ostream& out,
    std::ostream& err,
    const Poco::Process::Env& env);

  static ProcessHandle launch(
    const std::string& command,
    const Poco::Process::Args& args,
    const std::string& initialDirectory,
    std::istream& in,
    std::ostream& out,
    std::ostream& err);

  static ProcessHandle launch(
    const std::string& command,
    const Poco::Process::Args& args,
    std::istream& in,
    std::ostream& out,
    std::ostream& err);

  static ProcessHandle launch(
    const std::string& command,
    const Poco::Process::Args& args,
    const std::string& initialDirectory);

  static ProcessHandle launch(
    const std::string& command,
    const Poco::Process::Args& args);

  static void requestKill(ProcessHandle& handle, unsigned int signum);
  static void requestKill(pid_t pid, unsigned int signum);

  static void requestKill(ProcessHandle& handle);
  static void requestKill(pid_t pid);

  static void requestTermination(ProcessHandle& handle);
  static void requestTermination(pid_t pid);

  static std::string processStatus(ProcessHandle& handle);
  static std::string processStatus(pid_t pid);

  static ProcessHandle processParent(ProcessHandle& handle);
  static ProcessHandle processParent(pid_t pid);

  static bool isExistProcess(ProcessHandle& handle);
  static bool isExistProcess(pid_t pid);

  static pid_t id(void);
private:
  static void writeToPipe(int fd, const std::string& inputData="");
  static void readFromPipe(int fd, std::stringstream& os);
  static std::string toString(std::istream& in);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE 

#endif // PROCESS_HPP
