/**
 * @file StarterExecutor.hpp
 *
 * Implementation class StarterExecutor of a exec tools need to HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef STARTER_EXECUTOR_HPP
#define STARTER_EXECUTOR_HPP

#include <Poco/Pipe.h>
#include <Poco/PipeStream.h>

#include <iostream>
#include <sstream>

namespace HCE
{
//-----------------------------------------------------------------------------
namespace starter_const
{
  const std::string starterFileName = "hce-starter-bash.sh";
  const std::string starterPathName = "/usr/bin";
  const std::string starterTemplate = "#!/bin/bash\neval \"$1 < $2\"";
} // end namespace starter_const
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class StarterExecutor
{
public:
  enum StatusExecution
  {
    seUnknown=0,
    seFinished=1,
    seTerminated=2
  };
public:
  explicit StarterExecutor(const std::string& homeDir_, const std::string& starterFileName_="");
  virtual ~StarterExecutor(void);

  pid_t execute(const std::string& cmd, const std::string& inputStream, const Poco::Process::Env& env);
  unsigned int wait(unsigned int waitTime=0);

  void setStarterFileName(const std::string& starterFileName_);
  std::string getStarterFileName(void) const {return starterFileName;}

  void setHomeDir(const std::string& homeDir_);
  std::string getHomeDir(void) const {return homeDir;}
  std::string getStarterFullName(void) const;

  std::stringstream& getOutStream(void) {return outStream;}
  std::stringstream& getErrStream(void) {return errStream;}

  pid_t getPid(void) const {return pid;}
  StatusExecution getStatusExecution(void) const {return statusExecution;}

  static bool isExistShellScript(const std::string& homeDir, const std::string& scriptName);
  static bool canExecuteShellScript(const std::string& homeDir, const std::string& scriptName);
private:
  static std::string getStarterFullName(const std::string& homeDir, const std::string& scriptName);
private:
  Poco::Pipe outPipe;
  Poco::Pipe errPipe;
  Poco::PipeInputStream outInputStream;
  Poco::PipeInputStream errInputStream;

  std::stringstream outStream;
  std::stringstream errStream;

  std::string starterFileName;
  std::string homeDir;
  std::string inputStreamFileName;
  pid_t pid;

  StatusExecution statusExecution;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE 

#endif // STARTER_EXECUTOR_HPP
