/**
 * @file main.cpp
 *
 * Test Application main module. Tests of HCE::Process class.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#include <gtest/gtest.h>
#include <iostream>
#include <sstream>

#include "Process.hpp"
#include "ProcExec.hpp"
#include "CommandExecuter.hpp"

namespace HCE
{
namespace tests
{
//-----------------------------------------------------------------------------
std::string command = "sort";
HCE::Process::Args args = {"-r"};
std::string initialDirectory = "/tmp"; 
std::stringstream inMsg("def\nabc\n123\n");
HCE::Process::Env env = {{"key1", "value1"}};

const std::string reverseInputStream = "123\nabc\ndef\n";
//-----------------------------------------------------------------------------
TEST(ProcessTest, first)		
{ 
	bool noErrors = true;
	try
	{
		std::stringstream outMsg, errMsg;
		HCE::ProcessHandle ph = HCE::Process::launch(command, args, initialDirectory, inMsg, outMsg, errMsg, env);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);
		ASSERT_EQ(outMsg.str(), inMsg.str());
		ASSERT_TRUE(errMsg.str().empty());

		outMsg.str("");
		HCE::Process::Args argsEmpty;
		ph = HCE::Process::launch(command, argsEmpty, initialDirectory, inMsg, outMsg, errMsg, env);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);
		ASSERT_EQ(outMsg.str(), reverseInputStream);
		ASSERT_TRUE(errMsg.str().empty());
	}
	catch(std::exception& e)
	{
		std::cerr << "Exception: " << e.what() << std::endl;
		noErrors = false;
	}
	ASSERT_EQ(noErrors, true);
}
//-----------------------------------------------------------------------------
TEST(ProcessTest, second)		
{ 
	bool noErrors = true;
	try
	{
		std::stringstream outMsg, errMsg;
		HCE::ProcessHandle ph = HCE::Process::launch(command, args, inMsg, outMsg, errMsg, env);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);
		ASSERT_EQ(outMsg.str(), inMsg.str());
		ASSERT_TRUE(errMsg.str().empty());

		outMsg.str("");
		HCE::Process::Args argsEmpty;
		ph = HCE::Process::launch(command, argsEmpty, inMsg, outMsg, errMsg, env);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);
		ASSERT_EQ(outMsg.str(), reverseInputStream);
		ASSERT_TRUE(errMsg.str().empty());
	}
	catch(std::exception& e)
	{
		std::cerr << "Exception: " << e.what() << std::endl;
		noErrors = false;
	}
	ASSERT_EQ(noErrors, true);
}
//-----------------------------------------------------------------------------
TEST(ProcessTest, three)		
{ 
	bool noErrors = true;
	try
	{
		std::stringstream outMsg, errMsg;
		HCE::ProcessHandle ph = HCE::Process::launch(command, args, initialDirectory, inMsg, outMsg, errMsg);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);
		ASSERT_EQ(outMsg.str(), inMsg.str());
		ASSERT_TRUE(errMsg.str().empty());

		outMsg.str("");
		HCE::Process::Args argsEmpty;
		ph = HCE::Process::launch(command, argsEmpty, initialDirectory, inMsg, outMsg, errMsg);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);
		ASSERT_EQ(outMsg.str(), reverseInputStream);
		ASSERT_TRUE(errMsg.str().empty());
	}
	catch(std::exception& e)
	{
		std::cerr << "Exception: " << e.what() << std::endl;
		noErrors = false;
	}
	ASSERT_EQ(noErrors, true);
}
//-----------------------------------------------------------------------------
TEST(ProcessTest, four)		
{ 
	bool noErrors = true;
	try
	{
		std::stringstream outMsg, errMsg;
		HCE::ProcessHandle ph = HCE::Process::launch(command, args, inMsg, outMsg, errMsg);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);
		ASSERT_EQ(outMsg.str(), inMsg.str());
		ASSERT_TRUE(errMsg.str().empty());

		outMsg.str("");
		HCE::Process::Args argsEmpty;
		ph = HCE::Process::launch(command, argsEmpty, inMsg, outMsg, errMsg);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);
		ASSERT_EQ(outMsg.str(), reverseInputStream);
		ASSERT_TRUE(errMsg.str().empty());
	}
	catch(std::exception& e)
	{
		std::cerr << "Exception: " << e.what() << std::endl;
		noErrors = false;
	}
	ASSERT_EQ(noErrors, true);
}
//-----------------------------------------------------------------------------
TEST(ProcessTest, five)
{
	bool noErrors = true;
	try
	{
		HCE::ProcessHandle ph = HCE::Process::launch(command, args, initialDirectory);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);

		HCE::Process::Args argsEmpty;
		ph = HCE::Process::launch(command, argsEmpty, initialDirectory);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);
	}
	catch(std::exception& e)
	{
		std::cerr << "Exception: " << e.what() << std::endl;
		noErrors = false;
	}
	ASSERT_EQ(noErrors, true);
}
//-----------------------------------------------------------------------------
TEST(ProcessTest, six)
{
	bool noErrors = true;
	try
	{
		HCE::ProcessHandle ph = HCE::Process::launch(command, args);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);

		HCE::Process::Args argsEmpty;
		ph = HCE::Process::launch(command, argsEmpty);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);
	}
	catch(std::exception& e)
	{
		std::cerr << "Exception: " << e.what() << std::endl;
		noErrors = false;
	}
	ASSERT_EQ(noErrors, true);
}
//-----------------------------------------------------------------------------
TEST(ProcessTest, seven)
{
	bool noErrors = true;
	try
	{
		HCE::ProcessHandle ph = HCE::Process::launch(command, args);

		ASSERT_TRUE(ph.id());
		ASSERT_EQ(ph.wait(), ExitStatus::SUCCESS);

		HCE::Process::requestKill(ph);
    HCE::Process::requestTermination(ph);
    ASSERT_TRUE(HCE::Process::id());
	}
	catch(std::exception& e)
	{
		std::cerr << "Exception: " << e.what() << std::endl;
		noErrors = false;
	}
	ASSERT_EQ(noErrors, true);
}
//-----------------------------------------------------------------------------
TEST(ProcExecTest, exec)
{
	std::string command = "sort";
  HCE::Process::Args args = {"-r"};
  std::stringstream inMsg("def\nabc\n123\n");

	std::stringstream outMsg, errMsg;
	HCE::ProcExec procExec(command, args);
	bool ret = procExec.exec(inMsg, outMsg, errMsg);
	ASSERT_EQ(ret, true);
	ASSERT_EQ(procExec.isError(), false);
	ASSERT_EQ(outMsg.str(), inMsg.str());
	ASSERT_TRUE(errMsg.str().empty());
}
//-----------------------------------------------------------------------------
TEST(CommandExecutorTest, execute)
{
	std::string command = "sort -r";
	std::string shell = "bash -c";
  std::stringstream inMsg("def\nabc\n123\n");

	HCE::Command cmd(command);
	cmd.setShell(shell);
	cmd.setInputStream(inMsg);
	HCE::CommandExecuter commandExecuter;
	HCE::CommandResultData commandResultData = commandExecuter.execute(cmd);

	ASSERT_EQ(commandResultData.getIsError(), false);
	ASSERT_EQ(commandResultData.getExitStatus(), 0);
	ASSERT_EQ(commandResultData.getOutStream().str(), inMsg.str());
	ASSERT_TRUE(commandResultData.getErrStream().str().empty());
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace tests
} // end namespace HCE

int main (int argc, char** argv)
{
  testing::InitGoogleTest(&argc, argv);
  return RUN_ALL_TESTS();
}

