/**
 * @file Aerospike.hpp
 *
 * AerospikeDB class
 *
 * @author Alexey <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */


#ifndef AEROSPIKE_DB_HPP
#define AEROSPIKE_DB_HPP


#include <string>
#include <Poco/URI.h>
#include "BaseDB.hpp"

#include "citrusleaf/citrusleaf.h"


namespace HCE {

class AerospikeDB : public BaseDB {
public:
    AerospikeDB();
    virtual ~AerospikeDB();
public:
    void setDriverUrl( const std::string & dbUrl);
    void put(const std::string&, const std::string&);
    std::string get(const std::string&);
    virtual std::vector<std::string> get_all();
    void del(const std::string&);
    inline int getLastError() const { return _error; }
private:
    cl_cluster *_clc;
    cl_write_parameters _clwp;
public:
    static std::vector<std::string> documents;
private:
    enum {TIMEOUT=2000};
    enum {DEFAULT_DOCUMENT=0};
    const std::string docs[1] = {"document"};
    enum {DEFAULT_SET=0};
    const std::string sets[1] = {"default_set"};
private:
    template<class T>
    inline void SAFE_DELETE(T* ptr) {if(ptr){free(ptr);ptr=0;}}

    //int cb (char*, cf_digest*, char*, uint32_t, uint32_t, cl_bin*, int, bool, void*);
};

}


#endif //AEROSPIKE_DB_HPP
