/**
 * @file DataBaseDriverConfig.hpp
 * DataBaseDriverConfig class - class which contains data for connection with data base (voldemort, and so on..)
 * @author Oleksii Nidzelskyi<developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */


#ifndef DATA_BASE_DRIVER_CONFIG_HPP
#define DATA_BASE_DRIVER_CONFIG_HPP

//!<
//!< which ONE!!! Storage Engine driver does compile with
//!<
#include <iostream>

#ifdef _USE_VOLDEMORT_DB_
    #include "Voldemort.hpp"
#elif _USE_LEVEL_DB_
    #include "LevelDB.hpp"
#elif _USE_AEROSPIKE_DB_
    #include "Aerospike.hpp"
#endif

namespace HCE {

//<!
//<! class which contains data for connection with data base (voldemort, and so on..)
//<!

class DataBaseDriverConfig {
public:
	DataBaseDriverConfig ();
	~DataBaseDriverConfig ();
private:
	static std::string driverName;
private:
	std::string _driverName;
	std::string _dbName;
    std::string _dbUrl;
    bool _writePolicy; //!< Does the data will be inserted sync or async
    bool _writeUnique; //!< Does the data will be inserted only unique (e.g. try to insert the same data again will fail)
public:
    void setDriverName (const std::string& driverName) { this->_driverName = driverName; }
    void setDataBaseName (const std::string& dbName) { this->_dbName = dbName; }
    void setDataBaseUrl (const std::string& dbUrl) { this->_dbUrl = dbUrl; }
    void setWritePolicy(const bool write_policy) {this->_writePolicy = write_policy;}
    void setWriteUnique (const bool write_unique) { this->_writeUnique = write_unique; }
public:
    std::string getDriverName() const { return this->_driverName; }
    std::string getDataBaseName() const { return this->_dbName; }
    std::string getDataBaseUrl() const { return this->_dbUrl; }
    const bool getWritePolicy() const { return this->_writePolicy; }
    const bool getWriteUnique() const { return this->_writeUnique; }
};

} //!< namespace HCE

#endif //!< DATA_BASE_DRIVER_CONFIG_HPP
