/**
 * @file LevelDB.cpp
 *
 * LevelDB class
 *
 * @author Alexey <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */


//!<
//!< does compile with that driver
//!<
#ifdef _USE_LEVEL_DB_


#include "LevelDB.hpp"


namespace HCE
{

LevelDB::LevelDB():_db(nullptr){
	leveldb::Status status;
	_options.create_if_missing = true;
	assert( status.ok() );
	if ( !status.ok() ) {
		Poco::Message msg ("LevelDB",status.ToString(),Poco::Message::PRIO_CRITICAL,__FILE__,__LINE__);
		HCE::Logger::writeLog(msg);
	}
}

LevelDB::~LevelDB() {
	if (this->_db != nullptr) {
		delete this->_db;
	}
}

void LevelDB::setDatabaseName(const std::string& dbName) {
	leveldb::Status status;
	BaseDB::_dbName = dbName;
	status = leveldb::DB::Open( _options, _dbName, this->&_db );
	assert(status.ok());
	if (!status.ok()) {
		Poco::Message msg ("LevelDB",status.ToString(),Poco::Message::PRIO_CRITICAL,__FILE__,__LINE__);
		HCE::Logger::writeLog(msg);
	}
}

void LevelDB::put(const std::string& key, const std::string& value) {
	leveldb::Status status;
	BaseDB::_error = ACTION_SUCCESS;
	if ( status.ok() ) {
		status = _db->Put(leveldb::WriteOptions(), key, value);
	}
	if (!status.ok()) {
		BaseDB::_error = ACTION_GENERAL_FAILURE;
		Poco::Message msg ("LevelDB",status.ToString(),Poco::Message::PRIO_CRITICAL,__FILE__,__LINE__);
		HCE::Logger::writeLog(msg);
	}
}

std::string LevelDB::get(const std::string& key) {
	leveldb::Status status;
	BaseDB::_error = ACTION_GENERAL_FAILURE;
	std::string value;
	if ( status.ok() ) {
		BaseDB::_error = DOCUMENT_NOT_PRESENT;
		status = _db->Get(leveldb::ReadOptions(), key, &value);
		if ( status.ok() ) {
			BaseDB::_error = DOCUMENT_PRESENT;
		}
	}
	return value;
}

std::vector<std::string> LevelDB::get_all() {}

void LevelDB::del(const std::string& key) {
	leveldb::Status status;
	BaseDB::_error = ACTION_SUCCESS;
	if ( status.ok() ) {
		status = _db->Delete(leveldb::WriteOptions(), key);
	}
	if (!status.ok()) {
		BaseDB::_error = ACTION_GENERAL_FAILURE;
		Poco::Message msg ("LevelDB",status.ToString(),Poco::Message::PRIO_CRITICAL,__FILE__,__LINE__);
		HCE::Logger::writeLog(msg);
	}
}


} /// namespace HCE

#endif //!< _USE_LEVEL_DB_
