/**
 * @file Voldemort.cpp
 *
 * VoldemortDB class
 *
 * @author Alexey <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */


#ifdef _USE_VOLDEMORT_DB_


//!<
//!< app headers
//!<
#include "Voldemort.hpp"

//!<
//!< lib headers
//!<
#include "voldemort/VersionedValue.h"

//!<
//!< std headers
//!<
#include <list>
#include <string>

namespace HCE
{


	VoldemortDB::VoldemortDB() {
	   _config.setBootstrapUrls(&_bootstrapUrls);
	   _factory = Poco::SharedPtr<Voldemort::SocketStoreClientFactory>(new Voldemort::SocketStoreClientFactory(_config)) ;
	}


	VoldemortDB::~VoldemortDB() {}


	void VoldemortDB::setDatabaseName(const std::string& dbName) {
		_dbName = dbName;
		_client = Poco::SharedPtr<Voldemort::StoreClient>( _factory->getStoreClient(_dbName) );
	}


	void VoldemortDB::put(const std::string& key, const std::string& value) {
	    _error = ACTION_SUCCESS;
	    try {
	        _client->put(&key, &value);
	    } catch(Voldemort::VoldemortException& e) {
	       _error = ACTION_GENERAL_FAILURE;
	    }
	}


	std::string VoldemortDB::get(const std::string& key) {
        _error = DOCUMENT_NOT_PRESENT;
        std::string str;
        const Voldemort::VersionedValue* value;
        try {
            value = _client->get(&key);
        } catch(...) {
#ifdef _D_DEBUG_
            std::cout << "Voldemort::VersionedValue Exception" << std::endl;
#endif
        }
	    if(value) {
	        _error = DOCUMENT_PRESENT;
	        str = *((value)->getValue());
	    }
		return str;
	}


	std::vector<std::string> VoldemortDB::get_all() {

	}


	void VoldemortDB::del(const std::string& key) {
        bool status = _client->deleteKey(&key);
        _error = (status) ? ACTION_SUCCESS : ACTION_GENERAL_FAILURE;
	}


} //!< namespace HCE


#endif //!< _USE_VOLDEMORT_DB_
