/**
 * @file Voldemort.hpp
 *
 * VoldemortDB class
 *
 * @author Alexey <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */


#ifndef VOLDEMORT_DB_HPP
#define VOLDEMORT_DB_HPP


//!<
//!< app headers
//!<
#include "BaseDB.hpp"

//!<
//!< std headers
//!<
#include <list>
#include <string>
#include <iostream>

//!<
//!< Poco headers
//!<
#include <Poco/SharedPtr.h>

//!<
//!< libheaders
//!<
#include "voldemort/voldemort.h"


namespace HCE
{


class VoldemortDB : public BaseDB
{
public:
	VoldemortDB();
	virtual ~VoldemortDB();
public:
	void setDatabaseName(const std::string&);
	void put(const std::string& key, const std::string& value);
	virtual std::string get(const std::string& key);
	virtual std::vector<std::string> get_all();
	virtual void del(const std::string& key);
	inline int getLastError() const {return _error;}
private:
	Voldemort::ClientConfig _config;
	Poco::SharedPtr<Voldemort::SocketStoreClientFactory> _factory;
	Poco::SharedPtr<Voldemort::StoreClient> _client;
}; //!< class VoldemortDB


} //!< namespace HCE


#endif //!< VOLDEMORT_DB_HPP
