#include <sstream>

#include "CustomMessage.hpp"

namespace HCE
{
//-----------------------------------------------------------------------------
CustomMessage::CustomMessage(MessagesCollection& collection_)
: msgConst(), collection(collection_)
{
}
//-----------------------------------------------------------------------------
CustomMessage::CustomMessage(const CustomMessage& rhs)
: msgConst(), collection()
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
CustomMessage::CustomMessage(CustomMessage&& rhs)
: msgConst(), collection()
{
  (*this) = std::forward<CustomMessage>(rhs);
}
//-----------------------------------------------------------------------------
CustomMessage::~CustomMessage(void)
{
}
//-----------------------------------------------------------------------------
CustomMessage& CustomMessage::operator=(const CustomMessage& rhs)
{
  if (this != &rhs)
  {
    setMsgConst(rhs.getMsgConst());
    collection = rhs.getMessagesCollection();
  }
  return *this;
}
//-----------------------------------------------------------------------------
CustomMessage& CustomMessage::operator=(CustomMessage&& rhs)
{
  if (this != &rhs)
  {
    setMsgConst(std::move(rhs.getMsgConst()));
    collection = std::move(rhs.getMessagesCollection());
  }
  return *this;
}
//-----------------------------------------------------------------------------
void CustomMessage::setMsgConst(const MsgConst& msgConst_)
{
  msgConst.space = msgConst_.space;
  msgConst.point = msgConst_.point;
  msgConst.delimiter = msgConst_.delimiter;
}
//-----------------------------------------------------------------------------
std::string CustomMessage::makeMessage(unsigned int type)
{
  std::ostringstream outMsg;
  outMsg << collection.getMessage(type) << msgConst.point << msgConst.space;
  return outMsg.str();
}
//-----------------------------------------------------------------------------
std::string CustomMessage::makeMessage(unsigned int type, const std::string& arg)
{
  std::ostringstream outMsg;
  outMsg << collection.getMessage(type);
  if (!outMsg.str().empty() && !arg.empty())
    outMsg << msgConst.delimiter << msgConst.space << arg << msgConst.space;
  return outMsg.str();
}
//-----------------------------------------------------------------------------
std::string CustomMessage::operator()(unsigned int type)
{
  return makeMessage(type);
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

} // end namespace HCE 
