/**
 * @file CustomMessage.hpp
 *
 * Implementation class CustomMessage of a MessageStream tools in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef CUSTOM_MESSAGE_HPP
#define CUSTOM_MESSAGE_HPP

#include <sstream>

#include "MessagesCollection.hpp"

namespace HCE
{
//-----------------------------------------------------------------------------
class CustomMessage
{
public:
  struct MsgConst
  {
    MsgConst(void) {}
    MsgConst(char space_, char point_, char delimiter_) : space(space_), point(point_), delimiter(delimiter_) {}

    char space = ' ';
    char point = '.';
    char delimiter = ':';
  };
public:
  explicit CustomMessage(MessagesCollection& collection);
  CustomMessage(const CustomMessage& rhs);
  CustomMessage(CustomMessage&& rhs);
  virtual ~CustomMessage(void);

  CustomMessage& operator=(const CustomMessage& rhs);
  CustomMessage& operator=(CustomMessage&& rhs);

  void setMsgConst(const MsgConst& msgConst);
  const MsgConst& getMsgConst(void) const {return msgConst;}

  std::string operator()(unsigned int type);

  template<class T>
  std::string operator()(unsigned int type, const T& t)
  {
    std::stringstream outMsg;
    outMsg << t;
    return makeMessage(type, outMsg.str());
  }

  const MessagesCollection getMessagesCollection(void) const {return collection;}
private:
  std::string makeMessage(unsigned int type);
  std::string makeMessage(unsigned int type, const std::string& arg);

  MsgConst msgConst;
  MessagesCollection collection;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE 

#endif // CUSTOM_MESSAGE_HPP
