/**
 * @file MessagesCollection.hpp
 *
 * Implementation class MessagesCollection of a MessageStream tools in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef MESSAGES_COLLECTION_HPP
#define MESSAGES_COLLECTION_HPP

#include <vector>
#include <map>
#include <string>

namespace HCE
{
//-----------------------------------------------------------------------------
class MessagesCollection
{
  typedef std::pair<unsigned int, std::string> MsgItem;
  typedef std::map<unsigned int, std::string> MSG_MAP;
public:
  MessagesCollection(void);
  explicit MessagesCollection(const std::vector<MsgItem>& msgItems);
  MessagesCollection(const MessagesCollection& rhs);
  MessagesCollection(MessagesCollection&& rhs);
  ~MessagesCollection(void);

  MessagesCollection& operator=(const MessagesCollection& rhs);
  MessagesCollection& operator=(MessagesCollection&& rhs);

  void set(const std::vector<MsgItem>& msgItems);
  std::string getMessage(unsigned int type);

  const MSG_MAP getMessages(void) const {return Messages;}
private:
  MSG_MAP Messages;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE 

#endif // MESSAGES_COLLECTION_HPP
