#include <gtest/gtest.h>
#include <gmock/gmock.h>
#include "ReduceTaskCore.hpp"
#include "CoreMocks.hpp"
#include "ScheduleStrategySingle.hpp"

using namespace HCE::reduce::core;
using namespace Poco;
using namespace std;

using ::testing::_;
using ::testing::InSequence;
using ::testing::Return;
using ::testing::ReturnPointee;
using ::testing::SetArgReferee;

TEST(scheduleStrategySingle, sequenceCallEmptyObjects)
{
  ScheduleStrategySingle scheduleStrategySingle;

  SharedPtr<IterableMock>reduceStorageMock(new IterableMock());
  SharedPtr<ReduceResultMock>reduceResultMock(new ReduceResultMock());
  SharedPtr<ReduceTaskMock>reduceTaskMock(new ReduceTaskMock());
  SharedPtr<ReduceJobMock>reduceJobMock(new ReduceJobMock());

  InSequence s;

  EXPECT_CALL(*reduceJobMock.get(), getDataStorage())
             .WillOnce(ReturnPointee(&reduceStorageMock));
  EXPECT_CALL(*reduceJobMock.get(), getReduceTask())
             .WillOnce(ReturnPointee(&reduceTaskMock));
  EXPECT_CALL(*reduceStorageMock.get(), reset())
             .Times(1);
  EXPECT_CALL(*reduceStorageMock.get(), hasNext())
             .WillOnce(Return(false));
  EXPECT_CALL(*reduceJobMock.get(), getReduceResult())
             .WillOnce(ReturnPointee(&reduceResultMock));

  EXPECT_CALL(*reduceJobMock.get(), getReduceTask())
             .WillOnce(ReturnPointee(&reduceTaskMock));

  EXPECT_CALL(*reduceTaskMock.get(), fillReduceResult(_))
             .Times(1);

  scheduleStrategySingle.executeJob(reduceJobMock);
}


TEST(scheduleStrategySingle, sequenceCallProcessSomeData)
{
  ScheduleStrategySingle scheduleStrategySingle;

  SharedPtr<IterableMock>reduceStorageMock(new IterableMock());
  SharedPtr<ReduceResultMock>reduceResultMock(new ReduceResultMock());
  SharedPtr<ReduceTaskMock>reduceTaskMock(new ReduceTaskMock());
  SharedPtr<ReduceJobMock>reduceJobMock(new ReduceJobMock());

  string val = "";

  InSequence s;

  EXPECT_CALL(*reduceJobMock.get(), getDataStorage())
             .WillOnce(ReturnPointee(&reduceStorageMock));
  EXPECT_CALL(*reduceJobMock.get(), getReduceTask())
             .WillOnce(ReturnPointee(&reduceTaskMock));
  EXPECT_CALL(*reduceStorageMock.get(), reset())
             .Times(1);
  EXPECT_CALL(*reduceStorageMock.get(), hasNext())
             .WillOnce(Return(true));
  //data fill loop
  EXPECT_CALL(*reduceStorageMock.get(), getNext(_, _))
             .WillOnce(DoAll(SetArgReferee<0>(val), SetArgReferee<1>(val)));

  EXPECT_CALL(*reduceTaskMock.get(), addKeyValue(_, _))
             .Times(1);
  EXPECT_CALL(*reduceStorageMock.get(), hasNext())
               .WillOnce(Return(false));

  EXPECT_CALL(*reduceJobMock.get(), getReduceResult())
             .WillOnce(ReturnPointee(&reduceResultMock));
  EXPECT_CALL(*reduceJobMock.get(), getReduceTask())
             .WillOnce(ReturnPointee(&reduceTaskMock));
  EXPECT_CALL(*reduceTaskMock.get(), fillReduceResult(_))
             .Times(1);

  scheduleStrategySingle.executeJob(reduceJobMock);
}
