/**
 * @file DRCETaskReducersManager.hpp
 *
 * DRCETaskReducersManager manages the sphinx reducing tasks.
 *
 * @author Steven Ou
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef DRCETASKREDUCERSMANAGER_HPP_
#define DRCETASKREDUCERSMANAGER_HPP_

#include "TaskReducersManagerIf.hpp"
#include "Exceptions.hpp"
#include "DRCEResultData.hpp"
#include <map>
#include <Poco/SharedPtr.h>

using HCE::drce::DRCEResultData;

namespace HCE {
namespace drce{
namespace reduce_task {

class DRCETaskReducersManager: public HCE::reduce::additions::TaskReducersManagerIf{
public:
  virtual ~DRCETaskReducersManager();

  types::MessageType getType();
  bool isReducerExist(unsigned long long reduceTaskId);
  void createReducer(unsigned long long reduceTaskId);
  void addDataInReducer(unsigned long long reduceTaskId, const std::string& json);
  std::string runReduceTaskForTaskId(unsigned long long reduceTaskId);
  void deleteReducerBy(unsigned long long reduceTaskId);

private:
  std::map< unsigned long long, Poco::SharedPtr< DRCEResultData > > taskReduceResults_; 
};

}
}
}

#endif
