#include <ReducingHandler.h>
#include <ReducingPartsCountersStorage.h>
#include <ReducingOutputMessageBuilder.h>
#include <ConsoleSupportNotifier.h>
#include <ReducingExceptionsTranslator.h>
#include <SphinxTaskReducersManagerBuilder.h>
#include <PocoJSONReducingInputMessageConvertor.h>
#include <PocoJSONReducingOutputMessageConvertor.h>

using namespace HCE::reduce;
using namespace HCE::reduce_types;
using namespace HCE::reduce_convertors;
using namespace HCE::sphinx;
using namespace HCE::sphinx::reduce_task;
using namespace Poco;
using namespace std;

int main(int argc, char** argv)
{
  SphinxTaskReducersManagerBuilder<> sphinxTaskReducersManagerBuilder;
  SharedPtr<ReducingPartsCountersStorage> reducingPartsCountersStorage(new ReducingPartsCountersStorage());  
  SharedPtr<SphinxTaskReducersManagerIf<SphinxReduceJobBuilder<> > >sphinxTaskReducersManager = sphinxTaskReducersManagerBuilder.build();
  SharedPtr<ReducingExceptionsTranslator>reducingExceptionsTranslator(new ReducingExceptionsTranslator());
  SharedPtr<ReducingOutputMessageBuilder>reducingOutputMessageBuilder(new ReducingOutputMessageBuilder(reducingExceptionsTranslator));  
  SharedPtr<SupportNotifierIf> supportNotifier(new ConsoleSupportNotifier());

  ReducingHandler<> reducingHandler(reducingPartsCountersStorage, sphinxTaskReducersManager, reducingOutputMessageBuilder, supportNotifier);
  
  PocoJSONReducingInputMessageConvertor pocoJSONReducingInputMessageConvertor;
  PocoJSONReducingOutputMessageConvertor pocoJSONReducingOutputMessageConvertor;

  string sphinxResultJSON = "";
  string reducingResponseJSON = "";
  int itemId = 2;
  int nodesCount = 1;
  bool done = false;
  SharedPtr<ReducingInputMessage>reducingInputMessage;
  SharedPtr<ReducingOutputMessage> reducingOutputMessage;
  
  try{
    reducingInputMessage = pocoJSONReducingInputMessageConvertor.convertToReducingInputMessageFrom(sphinxResultJSON);
  }
  catch(WrongJSONStructureException& e){
    cout << e.what() << endl;
    return 1;
  }
    
  
  reducingHandler.accumulateReducingData(itemId, reducingInputMessage);
  
  try{
    if(reducingHandler.getAccumulateReducingPartsNumberBy(itemId) == nodesCount){
      done = true;
    }
  }
  catch(NotFoundByKeyException& e){
    cout << e.what() << endl;
    return 2;
  }
  
  if (done){
    reducingOutputMessage = reducingHandler.makeReducing(itemId);
    reducingHandler.deleteReducingTaskBy(itemId);
  }
  
  try{
    reducingResponseJSON = pocoJSONReducingOutputMessageConvertor.convertToJSONFrom(reducingOutputMessage);
  }
  catch(WrongJSONStructureException& e){
    cout << e.what() << endl;
    return 3;
  }
  
  return 0;
}