#include "PocoJSONReducingInputMessageConvertor.hpp"
#include "Exceptions.hpp"
#include <Poco/JSON/JSONException.h>
#include "PocoJSONExtractor.hpp"
#include "PocoJSONBuilder.hpp"

namespace HCE
{
namespace reduce_convertors
{

using namespace Poco;
using namespace std;
using namespace HCE::reduce_types;

PocoJSONReducingInputMessageConvertor::PocoJSONReducingInputMessageConvertor() {
}

PocoJSONReducingInputMessageConvertor::~PocoJSONReducingInputMessageConvertor() {
}

SharedPtr<ReducingInputMessage> PocoJSONReducingInputMessageConvertor::
                          convertToReducingInputMessageFrom(const std::string & json)
{
  try{
    PocoJSONExtractor pocoJSONExtractor(json);
    types::MessageType processingType =
    static_cast<types::MessageType>(pocoJSONExtractor.extractAsInt(PROCESSING_TYPE_NAME));
    int ttl = pocoJSONExtractor.extractAsInt(TTL_NAME);
    string body = pocoJSONExtractor.extractAsString(BODY_NAME);

    return SharedPtr<ReducingInputMessage>(new ReducingInputMessage(processingType, ttl, body));
  }
  catch(JSON::JSONException& e){
    throw reduce::WrongJSONStructureException(e.displayText());
  }
  catch(Exception& e){
    throw reduce::WrongJSONStructureException(e.displayText());
  }
}

string PocoJSONReducingInputMessageConvertor::convertToJSONFrom(
                            const SharedPtr<ReducingInputMessage> & inputMessage)
{
  try{
    PocoJSONBuilder pocoJSONBuilder;
    pocoJSONBuilder.add(PROCESSING_TYPE_NAME, static_cast<int>(inputMessage->getProcessingType()));
    pocoJSONBuilder.add(TTL_NAME, inputMessage->getTTL());
    pocoJSONBuilder.add(BODY_NAME, inputMessage->getBody());

    return pocoJSONBuilder.build();
  }
  catch(std::exception& e){
    throw reduce::WrongJSONStructureException(e.what());
  }
}

}
}
