#include "PocoJSONReducingOutputMessageConvertor.hpp"
#include "Exceptions.hpp"
#include <Poco/JSON/JSONException.h>
#include "PocoJSONExtractor.hpp"
#include "PocoJSONBuilder.hpp"

namespace HCE {

namespace reduce_convertors {

using namespace Poco;
using namespace std;
using namespace HCE::reduce_types;

PocoJSONReducingOutputMessageConvertor::PocoJSONReducingOutputMessageConvertor() {
}

PocoJSONReducingOutputMessageConvertor::~PocoJSONReducingOutputMessageConvertor() {
}

SharedPtr<ReducingOutputMessage> PocoJSONReducingOutputMessageConvertor::
                                convertToReducingOutputMessageFrom(string & json)
{
  try{
    PocoJSONExtractor pocoJSONExtractor(json);
    types::MessageType processingType =
    static_cast<types::MessageType>(pocoJSONExtractor.extractAsInt(PROCESSING_TYPE_NAME));
    int ttl = pocoJSONExtractor.extractAsInt(TTL_NAME);
    string body = pocoJSONExtractor.extractAsString(BODY_NAME);
    int processingErrorCode = pocoJSONExtractor.extractAsInt(PROCESSING_ERROR_CODE_NAME);
    int processingElapsedTime = pocoJSONExtractor.extractAsInt(PROCESSING_ELAPSED_TIME_NAME);
    string processingErrorMsg = pocoJSONExtractor.extractAsString(PROCESSING_ERROR_MSG_NAME);

    return SharedPtr<ReducingOutputMessage>(new ReducingOutputMessage(processingType, ttl,
                body, processingErrorCode, processingErrorMsg, processingElapsedTime));
  }
  catch(JSON::JSONException& e){
    throw reduce::WrongJSONStructureException(e.displayText());
  }
  catch(Exception& e){
    throw reduce::WrongJSONStructureException(e.displayText());
  }
}

string PocoJSONReducingOutputMessageConvertor::convertToJSONFrom(
                             const SharedPtr<ReducingOutputMessage> & message)
{
  try{
    PocoJSONBuilder pocoJSONBuilder;

    pocoJSONBuilder.add(PROCESSING_TYPE_NAME, static_cast<int>(message->getProcessingType()));
    pocoJSONBuilder.add(TTL_NAME, message->getTTL());
    pocoJSONBuilder.add(BODY_NAME, message->getBody());
    pocoJSONBuilder.add(PROCESSING_ERROR_CODE_NAME, message->getProcessingErrorCode());
    pocoJSONBuilder.add(PROCESSING_ERROR_MSG_NAME, message->getProcessingErrorMsg());
    pocoJSONBuilder.add(PROCESSING_ELAPSED_TIME_NAME, message->getProcessingElapsedTime());

    return pocoJSONBuilder.build();
  }
  catch(std::exception& e){
    throw reduce::WrongJSONStructureException(e.what());
  }
}


}
}
