#include "ReduceAdditionsStorage.hpp"

namespace HCE{
namespace reduce{

ReduceAdditionsStorage::ReduceAdditionsStorage(){}

ReduceAdditionsStorage::~ReduceAdditionsStorage(){}

Poco::SharedPtr<additions::TaskReducersManagerIf>ReduceAdditionsStorage::findBy
                                                 (HCE::types::MessageType key)
{
  StorageIterator storageIterator = findIteratorBy(key);
  return storageIterator->second;
}

Poco::SharedPtr<additions::TaskReducersManagerIf>ReduceAdditionsStorage::findBy
                                                 (unsigned long long secondKey)
{
  SecondKeyStorageIterator storageIterator = findIteratorBy(secondKey);
  return storageIterator->second;
}

void ReduceAdditionsStorage::deleteBy(unsigned long long secondKey)
{
  SecondKeyStorageIterator storageIterator = findIteratorBy(secondKey);
  _secondKeyStorage.erase(storageIterator);
}

void ReduceAdditionsStorage::addWith(HCE::types::MessageType key,
                 Poco::SharedPtr<additions::TaskReducersManagerIf> taskReducersManager)
{
  _storage.insert(std::make_pair(key, taskReducersManager));
}

void ReduceAdditionsStorage::addWith(unsigned long long secondKey,
                 Poco::SharedPtr<additions::TaskReducersManagerIf> taskReducersManager)
{
  _secondKeyStorage.insert(std::make_pair(secondKey, taskReducersManager));
}


ReduceAdditionsStorage::StorageIterator ReduceAdditionsStorage::findIteratorBy
                                       (HCE::types::MessageType key)
{
  StorageIterator storageIterator = _storage.find(key);
  if (storageIterator != _storage.end()){
    return storageIterator;
  }
  throw HCE::reduce::NotFoundByKeyException
            ("ReduceAdditionsStorage - not found key " +
                        std::to_string(static_cast<int>(key)));
}

ReduceAdditionsStorage::SecondKeyStorageIterator ReduceAdditionsStorage::findIteratorBy
                                                               (unsigned long long key)
{
  SecondKeyStorageIterator storageIterator = _secondKeyStorage.find(key);
  if (storageIterator != _secondKeyStorage.end()){
    return storageIterator;
  }
  throw HCE::reduce::NotFoundByKeyException
            ("ReduceAdditionsStorage - not found key " + std::to_string(key));
}


}
}
