/**
 * @file ReducingBaseMessage.hpp
 *
 * ReducingBaseMessage defines a base message of hierarchy messages
 * which can be processed the ReducingHandler class.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef REDUCINGBASEMESSAGE_HPP_
#define REDUCINGBASEMESSAGE_HPP_

#include <string>
#include "MessageType.hpp"

namespace HCE{
namespace reduce_types{

const int FAKE_MSG_TYPE = -1;
const int FAKE_TTL = -1;
const int SPHINX_SEARCH_PROCESSING_TYPE = 0;


class ReducingBaseMessage{
public:
  ReducingBaseMessage(const types::MessageType processingType, const int ttl,
                      const std::string& body);
  virtual ~ReducingBaseMessage();

  types::MessageType getProcessingType() const;
  int getTTL() const;
  std::string getBody() const;

private:
  types::MessageType _processingType;
  int _ttl;
  std::string _body;

  ReducingBaseMessage();
};


}
}

#endif
