#include "ReducingHandlerBuilder.hpp"
#include "ReducingPartsCountersStorage.hpp"
#include "ReduceAdditionsStorage.hpp"
#include "TaskReducersManagerFactory.hpp"
#include "SphinxTaskReducersManager.hpp"
#include "ReducingOutputMessageBuilder.hpp"
#include "ConsoleSupportNotifier.hpp"
#include "ReducingExceptionsTranslator.hpp"

using namespace Poco;
using namespace HCE::reduce;
using namespace HCE::sphinx;
using namespace HCE::sphinx::reduce_task;

namespace HCE{
namespace reduce{

Poco::SharedPtr<HCE::reduce::ReducingHandler> ReducingHandlerBuilder::build()
{
  SharedPtr<ReducingPartsCountersStorage> reducingPartsCountersStorage(new
                                              ReducingPartsCountersStorage() );
  SharedPtr<ReduceAdditionsStorage> reduceAdditionsStorage(new
                                              ReduceAdditionsStorage());
  SharedPtr<TaskReducersManagerFactory> taskReducersManagerFactory(new
                                              TaskReducersManagerFactory());
  SharedPtr<ReducingExceptionsTranslator>reducingExceptionsTranslator(new
                                              ReducingExceptionsTranslator());
  SharedPtr<ReducingOutputMessageBuilder> reducingOutputMessageBuilder(new
                   ReducingOutputMessageBuilder(reducingExceptionsTranslator));
  SharedPtr<SupportNotifierIf> supportNotifier(new ConsoleSupportNotifier());

  return SharedPtr<ReducingHandler>(new ReducingHandler(reducingPartsCountersStorage,
                                                        reduceAdditionsStorage,
                                                        taskReducersManagerFactory,
                                                        reducingOutputMessageBuilder,
                                                        supportNotifier));
}

}
}
