/**
 * @file ReducingOutputMessage.hpp
 *
 * The ReducingOutputMessage class defines the ReducingHandler output message
 * which contains data generated during executing a reducing task.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef REDUCINGOUTPUTMESSAGE_HPP_
#define REDUCINGOUTPUTMESSAGE_HPP_

#include "ReducingBaseMessage.hpp"

namespace HCE{
namespace reduce_types{

class ReducingOutputMessage: public ReducingBaseMessage{
public:
  ReducingOutputMessage(const types::MessageType processingType, const int ttl,
                        const std::string& body, const int processingErrorCode,
                        const std::string& processingErrorMsg,
                        int processingElapsedTime);
  ~ReducingOutputMessage();

  int getProcessingErrorCode() const;
  std::string getProcessingErrorMsg() const;
  int getProcessingElapsedTime() const;

private:
  int _processingErrorCode;
  int _processingElapsedTime;
  std::string _processingErrorMsg;

  ReducingOutputMessage();
};

}
}

#endif
