/**
 * @file ReducingOutputMessageBuilder.hpp
 *
 * The ReducingOutputMessageBuilder class builds the ReducingOutputMessage class
 * from work result of a ReducingHandler class.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef REDUCINGOUTPUTMESSAGEBUILDER_HPP_
#define REDUCINGOUTPUTMESSAGEBUILDER_HPP_

#include "ReducingOutputMessageBuilderIf.hpp"
#include "ReducingExceptionsTranslatorIf.hpp"

namespace HCE{
namespace reduce{

class ReducingOutputMessageBuilder: public ReducingOutputMessageBuilderIf{
public:
  ReducingOutputMessageBuilder(
    const Poco::SharedPtr<ReducingExceptionsTranslatorIf>& reducingExceptionsTranslator);
  virtual ~ReducingOutputMessageBuilder();

  Poco::SharedPtr<reduce_types::ReducingOutputMessage>build(
                                       const types::MessageType& msgType,
                                       const int ttl, const std::string& json);
  Poco::SharedPtr<reduce_types::ReducingOutputMessage>
                          build(const NotFoundByKeyException& e);

private:
  Poco::SharedPtr<ReducingExceptionsTranslatorIf> _reducingExceptionsTranslator;

  ReducingOutputMessageBuilder();
};

}
}

#endif
