/**
 * @file ReducingPartsCountersStorage.hpp
 *
 * The ReducingPartsCountersStorage class is used to storage
 * counts number of data parts were added in a reducingHandler.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef REDUCINGPARTSCOUNTERSSTORAGE_HPP_
#define REDUCINGPARTSCOUNTERSSTORAGE_HPP_

#include "ReducingPartsCountersStorageIf.hpp"
#include <map>

namespace HCE
{
namespace reduce
{

class ReducingPartsCountersStorage: public ReducingPartsCountersStorageIf {
public:
  typedef std::map<unsigned long long, int>::iterator StorageIterator;

  ReducingPartsCountersStorage();
  virtual ~ReducingPartsCountersStorage();

  bool findBy(unsigned long long keyId);
  void deleteBy(unsigned long long keyId);
  void addReducingPartsCounterWithKey(unsigned long long keyId);
  void incrementBy(unsigned long long keyId);
  int  getAccumulateReducingParts(unsigned long long keyId);
  unsigned int getTotalTasksNumber();

private:
  std::map<unsigned long long, int> keyIdCountersStorage;

  StorageIterator findIteratorBy(unsigned long long keyId);
};

}
}

#endif
