/*
 * SphinxTaskReducersStorageTest.cpp
 *
 *  Created on: May 20, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include "ReduceAdditionsStorage.hpp"
#include "Exceptions.hpp"

using namespace HCE::reduce;
using namespace Poco;

TEST(ReduceAdditionsStorage, returnReducerByValidKey)
{
  ReduceAdditionsStorage reduceAdditionsStorage;
  SharedPtr<additions::TaskReducersManagerIf>taskReducersManager;
  unsigned int taskId = 1;

  reduceAdditionsStorage.addWith(taskId, taskReducersManager);

  ASSERT_EQ(taskReducersManager, reduceAdditionsStorage.findBy(taskId));
}

TEST(ReduceAdditionsStorage, returnReducerByValidSecondKey)
{
  ReduceAdditionsStorage reduceAdditionsStorage;
  SharedPtr<additions::TaskReducersManagerIf>taskReducersManager;
  unsigned long long secondKey = 1;

  reduceAdditionsStorage.addWith(secondKey, taskReducersManager);

  ASSERT_EQ(taskReducersManager, reduceAdditionsStorage.findBy(secondKey));
}


TEST(ReduceAdditionsStorage, throwNotFoundSphinxTaskReducerExceptionWhileDeleteByInvalidKey)
{
  ReduceAdditionsStorage reduceAdditionsStorage;
  int invalidTaskId = 8;

  ASSERT_THROW(reduceAdditionsStorage.deleteBy(invalidTaskId), NotFoundByKeyException);
}


TEST(ReduceAdditionsStorage, throwNotFoundSphinxTaskReducerExceptionWhileSearchByInvalidSecondKey)
{
  ReduceAdditionsStorage reduceAdditionsStorage;
  unsigned long long invalidTaskId = 8;

  ASSERT_THROW(reduceAdditionsStorage.findBy(invalidTaskId), NotFoundByKeyException);
}


TEST(ReduceAdditionsStorage, correctDeleteSphinxTaskReducerByValidSecondKey)
{
  ReduceAdditionsStorage reduceAdditionsStorage;
  SharedPtr<additions::TaskReducersManagerIf>taskReducersManager;

  unsigned long long secondKey = 1;
  reduceAdditionsStorage.addWith(secondKey, taskReducersManager);
  reduceAdditionsStorage.deleteBy(secondKey);

  ASSERT_THROW(reduceAdditionsStorage.findBy(secondKey), NotFoundByKeyException);
}
