/*
 * ReducingOutputMessageBuilderTest.cpp
 *
 *  Created on: Jun 5, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include "ReducingOutputMessageBuilder.hpp"
#include "HandlerMocks.hpp"
#include "TestUtils.hpp"

using namespace HCE::reduce;
using namespace Poco;
using namespace std;

using ::testing::_;
using ::testing::Return;

class ReducingOutputMessageBuilderTest: public ::testing::Test{
  protected:
    virtual void SetUp(){
      reducingExceptionsTranslator.assign(new ReducingExceptionsTranslatorMock());
    }

    virtual void TearDown(){
      reducingExceptionsTranslator.assign(NULL);
      etalonOutputMessage.assign(NULL);
      reducingOutputMessage.assign(NULL);
    }

    static SharedPtr<ReducingExceptionsTranslatorMock> reducingExceptionsTranslator;
    static SharedPtr<ReducingOutputMessage> etalonOutputMessage;
    static SharedPtr<ReducingOutputMessage> reducingOutputMessage;
};
SharedPtr<ReducingExceptionsTranslatorMock>ReducingOutputMessageBuilderTest::
                                                reducingExceptionsTranslator;
SharedPtr<ReducingOutputMessage>ReducingOutputMessageBuilderTest::etalonOutputMessage;
SharedPtr<ReducingOutputMessage>ReducingOutputMessageBuilderTest::reducingOutputMessage;


TEST_F(ReducingOutputMessageBuilderTest, buildOutputMessageFromJSON)
{
  HCE::types::MessageType msgType = HCE::types::MessageType::mtSphinx;
  string reducingResultJSON = "test";
  int ttl = 7;

  etalonOutputMessage = buildEtalonFrom(msgType, ttl, reducingResultJSON);

  ReducingOutputMessageBuilder reducingOutputMessageBuilder(reducingExceptionsTranslator);

  reducingOutputMessage = reducingOutputMessageBuilder.build(msgType, ttl, reducingResultJSON);

  ASSERT_TRUE(isEqualReducingOutputMessage(*reducingOutputMessage.get(),
                                           *etalonOutputMessage.get()));
}


TEST_F(ReducingOutputMessageBuilderTest, buildOutputMessageFromNotFoundByKeyException)
{
  NotFoundByKeyException notFoundByKeyException("key 2 is wrong");
  etalonOutputMessage = buildEtalonFrom(notFoundByKeyException);

  EXPECT_CALL(*reducingExceptionsTranslator.get(), translateToErrorCode(_))
              .WillOnce(Return(INTERNAL_ERROR));
  EXPECT_CALL(*reducingExceptionsTranslator.get(), translateToErrorMsg(_))
              .WillOnce(Return(notFoundByKeyException.what()));

  ReducingOutputMessageBuilder reducingOutputMessageBuilder(reducingExceptionsTranslator);

  reducingOutputMessage = reducingOutputMessageBuilder.build(notFoundByKeyException);

  ASSERT_TRUE(isEqualReducingOutputMessage(*reducingOutputMessage.get(),
                                           *etalonOutputMessage.get()));
}
