#include <gtest/gtest.h>
#include "ReducingPartsCountersStorage.hpp"
#include "Exceptions.hpp"

using namespace HCE::reduce;

TEST(ReducingPartsCountersStorage, corectProcessFindRequests)
{
  int validKey = 2442352714LLU;
  int invalidKey = 7;

  ReducingPartsCountersStorage reducingPartsCountersStorage;

  reducingPartsCountersStorage.addReducingPartsCounterWithKey(validKey);

  EXPECT_TRUE(reducingPartsCountersStorage.findBy(validKey) == true);
  EXPECT_TRUE(reducingPartsCountersStorage.findBy(invalidKey) == false);
}


TEST(ReducingPartsCountersStorage, correctAccumulateReducingPartsNumber)
{
  int key = 2442352714LLU;

  ReducingPartsCountersStorage reducingPartsCountersStorage;

  reducingPartsCountersStorage.addReducingPartsCounterWithKey(key);
  reducingPartsCountersStorage.incrementBy(key);

  ASSERT_EQ(reducingPartsCountersStorage.getAccumulateReducingParts(key), 1);
}


TEST(ReducingPartsCountersStorage, throwExceptionWhileTryIncrementByInvalidKey)
{
  int invalidKey = 1;

  ReducingPartsCountersStorage reducingPartsCountersStorage;

  ASSERT_THROW(reducingPartsCountersStorage.incrementBy(invalidKey),
               NotFoundByKeyException);
}


TEST(ReducingPartsCountersStorage, correctDeleteCountersByKey)
{
  int key = 3;

  ReducingPartsCountersStorage reducingPartsCountersStorage;

  reducingPartsCountersStorage.addReducingPartsCounterWithKey(key);
  reducingPartsCountersStorage.deleteBy(key);

  ASSERT_TRUE(reducingPartsCountersStorage.findBy(key) == false);
}


TEST(ReducingPartsCountersStorage, calculateNumbersOfTasksProcessingAtTheMoment)
{
  int key1 = 4;
  int key2 = 2;
  unsigned int taskNumbers = 2;

  ReducingPartsCountersStorage reducingPartsCountersStorage;

  reducingPartsCountersStorage.addReducingPartsCounterWithKey(key1);
  reducingPartsCountersStorage.addReducingPartsCounterWithKey(key2);

  ASSERT_EQ(reducingPartsCountersStorage.getTotalTasksNumber(), taskNumbers);
}
