/*
 * ReducingTaskTTLManagerTest.cpp
 *
 *  Created on: Nov 24, 2013
 *      Author: Steven
 */

#include <gtest/gtest.h>
#include "ReducingTaskTTLManager.hpp"
#include <Poco/Thread.h>

using namespace std;
using HCE::reduce::ReducingTaskTTLManager;

class ReducingTaskTTLManagerTest: public ::testing::Test {
protected:
    virtual void SetUp(){
      taskId = 1;
    }
    
    virtual void TearDown(){
    }
    
    void sleep_milliseconds( unsigned int milliseconds ) {
      Poco::Thread::sleep( milliseconds );
    }
    
    unsigned int milliseconds( int millis ) {
      return millis;
    }
    
    ReducingTaskTTLManager taskTTLManager;
    unsigned long long taskId;
};

TEST_F( ReducingTaskTTLManagerTest, GetTaskTTL ) {
  taskTTLManager.addTaskTTL( taskId, 10 );
  
  EXPECT_EQ( taskTTLManager.getTaskTTL( taskId ), 10U);
}

TEST_F( ReducingTaskTTLManagerTest, NotExistTaskTTL ) {
  EXPECT_EQ( taskTTLManager.getTaskTTL( taskId ), 0U);
}

TEST_F( ReducingTaskTTLManagerTest, AddTaskMultiTimes ) {
  taskTTLManager.addTaskTTL( taskId, 10 );
  taskTTLManager.addTaskTTL( taskId, 20 );
  
  EXPECT_EQ( taskTTLManager.getTaskTTL( taskId ), 20U);
}

TEST_F( ReducingTaskTTLManagerTest, RemoveNotExpiredTask ) {
  taskTTLManager.addTaskTTL( taskId, 20 );
  EXPECT_EQ( taskTTLManager.getTaskTTL( taskId ), 20U);
  taskTTLManager.removeTaskTTL( taskId );  
  EXPECT_FALSE( taskTTLManager.isTaskTerminatedByTTL( taskId ) );
}

TEST_F( ReducingTaskTTLManagerTest, RemoveExpiredTask ) {
  
  taskTTLManager.addTaskTTL( taskId, 10 );
  sleep_milliseconds(20);
  EXPECT_TRUE( taskTTLManager.isTaskExpired( taskId ) );
  taskTTLManager.removeTaskTTL( taskId );
  EXPECT_TRUE( taskTTLManager.isTaskTerminatedByTTL( taskId ) );
}

TEST_F( ReducingTaskTTLManagerTest, NotExpiredWithoutParam ) {
  taskTTLManager.addTaskTTL( taskId, 100 );
  
  EXPECT_FALSE( taskTTLManager.isTaskExpired( taskId ) );
  sleep_milliseconds( 10 );
  EXPECT_FALSE( taskTTLManager.isTaskExpired( taskId ) );
}

TEST_F( ReducingTaskTTLManagerTest, NotExpiredWithParam ) {
  taskTTLManager.addTaskTTL( taskId, 100 );
  
  sleep_milliseconds( 10 );
  EXPECT_FALSE( taskTTLManager.isTaskExpired( taskId, 20 ) );  
}

TEST_F( ReducingTaskTTLManagerTest, ExpiredWithoutParam ) {
  taskTTLManager.addTaskTTL( taskId, 10 );
  
  EXPECT_FALSE( taskTTLManager.isTaskExpired( taskId ) );
  sleep_milliseconds( 10 );
  EXPECT_TRUE( taskTTLManager.isTaskExpired( taskId ) );
}

TEST_F( ReducingTaskTTLManagerTest, ExpiredWithParam ) {
  taskTTLManager.addTaskTTL( taskId, 100 );
  
  sleep_milliseconds( 10 );
  EXPECT_TRUE( taskTTLManager.isTaskExpired( taskId, 10 ) );
}


TEST_F( ReducingTaskTTLManagerTest, ExpiredTasksCleanup ) {
    taskTTLManager.addTaskTTL( taskId, 10 );
    taskTTLManager.addTaskTTL( taskId + 1, 20 );
    taskTTLManager.addTaskTTL( taskId + 2, 30 );
    taskTTLManager.addTaskTTL( taskId + 4, 40 );
    taskTTLManager.addTaskTTL( taskId + 5, 50 );
    
    sleep_milliseconds( 11 );
    taskTTLManager.removeTaskTTL( taskId );
    taskTTLManager.removeTaskTTL( taskId + 1);
    EXPECT_TRUE( taskTTLManager.isTaskTerminatedByTTL( taskId ) );
    EXPECT_FALSE( taskTTLManager.isTaskTerminatedByTTL( taskId + 1 ) );
    sleep_milliseconds( 11 );
    EXPECT_EQ( taskTTLManager.cleanupExpiredTasksByTTLQueue( 10, 10 ), 1U);
    EXPECT_FALSE( taskTTLManager.isTaskTerminatedByTTL( taskId ) );
    EXPECT_FALSE( taskTTLManager.isTaskTerminatedByTTL( taskId + 1 ) );
}

TEST_F( ReducingTaskTTLManagerTest, ExceededTTLTasksWithDefault ) {
    std::vector<unsigned long long> exceededTTLTasks;
    taskTTLManager.addTaskTTL( taskId, 10 );
    taskTTLManager.addTaskTTL( taskId + 1, 20 );
    taskTTLManager.addTaskTTL( taskId + 2, 30 );
    taskTTLManager.addTaskTTL( taskId + 4, 40 );
    taskTTLManager.addTaskTTL( taskId + 5, 50 );
    sleep_milliseconds( 21 );
    taskTTLManager.getExceededTTLTasks( 10, exceededTTLTasks );
    
    EXPECT_EQ( exceededTTLTasks.size(), 2U);
}

TEST_F( ReducingTaskTTLManagerTest, ExceededTTLTasksWithParam ) {
    std::vector<unsigned long long> exceededTTLTasks;
    taskTTLManager.addTaskTTL( taskId, 10 );
    taskTTLManager.addTaskTTL( taskId + 1, 20 );
    taskTTLManager.addTaskTTL( taskId + 2, 30 );
    taskTTLManager.addTaskTTL( taskId + 4, 40 );
    taskTTLManager.addTaskTTL( taskId + 5, 50 );
    sleep_milliseconds( 20 );
    taskTTLManager.getExceededTTLTasks( 10, exceededTTLTasks, 10 );
    
    EXPECT_EQ( exceededTTLTasks.size(), 5U);
}

TEST_F( ReducingTaskTTLManagerTest, NoTerminatedTasks ) {
    taskTTLManager.addTaskTTL( taskId, 10 );
    sleep_milliseconds( 20 );
    EXPECT_FALSE( taskTTLManager.isTaskTerminatedByTTL( taskId ) );
}

TEST_F( ReducingTaskTTLManagerTest, TaskIsTerminated ) {
    taskTTLManager.addTaskTTL( taskId, 10 );
    sleep_milliseconds( 11 );
    taskTTLManager.removeTaskTTL( taskId );
    EXPECT_TRUE( taskTTLManager.isTaskTerminatedByTTL( taskId ) );
}

