/**
 * @file SphinxDataStorage.hpp
 *
 * The SphinxReduceDataStorage class defines storage which keeps data
 * from sphinx search engine.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef SPHINXREDUCEDATASTORAGE_HPP_
#define SPHINXREDUCEDATASTORAGE_HPP_

#include "ReduceTaskCore.hpp"
#include "SphinxMatchInfo.hpp"
#include <map>
#include <utility>

namespace HCE
{
namespace sphinx
{
  namespace reduce_task
  {
     class SphinxReduceDataStorage: public HCE::reduce::core::IterableIf{
       public:
         typedef std::multimap<std::string, SphinxMatchInfo>::iterator DataBoundIterator;
         typedef std::pair<DataBoundIterator, DataBoundIterator> DataRangeIterator;

         SphinxReduceDataStorage();
         ~SphinxReduceDataStorage();

         bool hasNext();
         void reset();
         void getNext(std::string & key, std::string& value);

         void addData(const SphinxMatchInfo& sphinxMatchInfo);
         DataRangeIterator getData(const std::string& key);

       private:
         std::multimap<std::string, SphinxMatchInfo> _data;
         DataBoundIterator _keyValueIterator;
     };
  }
}
}
#endif
