/**
 * @file SphinxReduceJob.hpp
 *
 *  The SphinxReduceJob class defines the job to reduce data from
 *  the sphinx search engine.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef SPHINXREDUCEJOB_HPP_
#define SPHINXREDUCEJOB_HPP_

#include "ReduceTaskCore.hpp"
#include "SphinxReduceDataStorage.hpp"
#include "SphinxReduceTask.hpp"
#include "SphinxReduceResult.hpp"

namespace HCE{
namespace sphinx{
namespace reduce_task
{

class SphinxReduceJob: public HCE::reduce::core::ReduceJobIf{
public:
   SphinxReduceJob(Poco::SharedPtr<SphinxReduceDataStorage>& sphinxReduceDataStorage,
                   Poco::SharedPtr<SphinxReduceTask>& sphinxReduceTask,
                   Poco::SharedPtr<SphinxReduceResult>& sphinxReduceResult);
   ~SphinxReduceJob();

   Poco::SharedPtr<HCE::reduce::core::IterableIf> getDataStorage() const;
   Poco::SharedPtr<HCE::reduce::core::ReduceResultIf> getReduceResult() const;
   Poco::SharedPtr<HCE::reduce::core::ReduceTaskIf> getReduceTask() const;

private:
   SphinxReduceJob();
   Poco::SharedPtr<SphinxReduceDataStorage> _sphinxReduceDataStorage;
   Poco::SharedPtr<SphinxReduceTask> _sphinxReduceTask;
   Poco::SharedPtr<SphinxReduceResult> _sphinxReduceResult;
};

}
}
}

#endif
