/**
 * @file SphinxReduceTask.hpp
 *
 * The SphinxReduceTask class makes the reduce task for data from sphinx
 * search engine
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef SPHINXREDUCETASK_HPP_
#define SPHINXREDUCETASK_HPP_

#include "ReduceTaskCore.hpp"
#include "SphinxReduceResult.hpp"
#include <map>

namespace HCE{
namespace sphinx{
namespace reduce_task{

class SphinxReduceTask: public HCE::reduce::core::ReduceTaskIf{
public:
  SphinxReduceTask();
  ~SphinxReduceTask();

  void addKeyValue(const std::string& key, const std::string& value);
  void fillReduceResult(Poco::SharedPtr<HCE::reduce::core::ReduceResultIf> reduceResult);

private:
  std::map<std::string, std::string> _reduceData;
  std::map<std::string, std::string>::iterator _reduceDataIterator;

  bool hasKeyInData(const std::string& mapKey);
  void saveBestWeight(const std::string& key, const std::string& value);
  void addNewResource(const std::string& key, const std::string& value);
};

}
}
}

#endif
