/**
 * @file SphinxResultBuilder.hpp
 *
 * The SphinxResultBuilder class builds the SphinxResult class from data are taken after
 * the reducing task.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef SPHINXRESULTBUILDER_HPP_
#define SPHINXRESULTBUILDER_HPP_

#include "SphinxResultBuilderIf.hpp"
#include <vector>

namespace HCE {
namespace sphinx{
namespace reduce_task {

class SphinxResultBuilder: public SphinxResultBuilderIf {
public:
  SphinxResultBuilder();
  virtual ~SphinxResultBuilder();

  Poco::SharedPtr<SphinxResultData> build(
          Poco::SharedPtr<SphinxReduceDataStorage> &sphinxDataStorage,
          const Poco::SharedPtr<HCE::reduce::core::ReduceResultIf> &sphinxReduceResult,
          const std::vector<SphinxRequestInfo>& requestInfo);

  Poco::SharedPtr<SphinxResultData> buildFromUsingKeys(
                 const Poco::SharedPtr<SphinxResultData> &sphinxResultData,
                 std::vector<std::pair<std::string, unsigned long long> > &selectWeightKeys);

private:
  static void fillMatchInfo(Poco::SharedPtr<SphinxReduceDataStorage> &sphinxDataStorage,
                     Poco::SharedPtr<HCE::reduce::core::ReduceResultIf> sphinxReduceResult,
                     Poco::SharedPtr<SphinxResultData>& newSphinxResultData);

  static void fillRequestInfo(const std::vector<SphinxRequestInfo>& requestInfo,
                    Poco::SharedPtr<SphinxResultData>& newSphinxResultData);

  static void fillMatchDataBaseOnKeys(const Poco::SharedPtr<SphinxResultData> &sphinxResultData,
                  std::vector<std::pair<std::string, unsigned long long> > &selectWeightKeys,
                  Poco::SharedPtr<SphinxResultData> &newSphinxResult);

  static void fillRequestInfoBaseOnResultData(const Poco::SharedPtr<SphinxResultData> &sphinxResultData,
                                       Poco::SharedPtr<SphinxResultData> &newSphinxResult);

  static void buildMatchInfoMap(std::map<unsigned long long, SphinxMatchInfo> &keysMatchInfo,
                         const Poco::SharedPtr<SphinxResultData> &sphinxResultData);
};

}
}
}

#endif
