/**
 * @file SphinxResultBuilderIf.hpp
 *
 * The SphinxResultBuilderIf is an interface which describes behavior of abstract type.
 * The type is responsible for build the SphinxResult class from data are taken after
 * the reducing task.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef SPHINXRESULTBUILDERIF_HPP_
#define SPHINXRESULTBUILDERIF_HPP_

#include "SphinxReduceDataStorage.hpp"
#include "SphinxResultData.hpp"
#include <Poco/SharedPtr.h>
#include <vector>

namespace HCE{
namespace sphinx{
namespace reduce_task{

class SphinxResultBuilderIf{
  public:
    virtual ~SphinxResultBuilderIf(){}

    virtual Poco::SharedPtr<SphinxResultData> build(
            Poco::SharedPtr<SphinxReduceDataStorage> &sphinxDataStorage,
            const Poco::SharedPtr<HCE::reduce::core::ReduceResultIf> &sphinxReduceResult,
            const std::vector<SphinxRequestInfo>& requestInfo) = 0;

    virtual Poco::SharedPtr<SphinxResultData> buildFromUsingKeys(
                 const Poco::SharedPtr<SphinxResultData> &sphinxResultData,
                 std::vector<std::pair<std::string, unsigned long long> > &selectWeightKeys) = 0;
  };

}
}
}

#endif
