#include "SphinxResultTruncater.hpp"
#include "SphinxResultTransformer.hpp"
#include "SphinxResultBuilder.hpp"

namespace HCE
{
namespace sphinx
{
namespace reduce_task
{

using namespace Poco;
using namespace std;

SphinxResultTruncater::SphinxResultTruncater() {
}

SphinxResultTruncater::~SphinxResultTruncater() {
}

SharedPtr<SphinxResultData> SphinxResultTruncater::trancate(
                                        SharedPtr<SphinxResultData>& sphinxResultData)
{
  SphinxResultTransformer sphinxResultTransformer;
  SphinxResultBuilder sphinxResultBuilder;
  vector<pair<string, unsigned long long> > weightKeyPairs;
  SphinxResultSorterIf& sorter = selectResultSorter(sphinxResultData);

  sphinxResultTransformer.extractWeightKeys(sphinxResultData, weightKeyPairs);
  sorter.sortByWeight(weightKeyPairs);
  trancateByWeight(weightKeyPairs, sphinxResultData);
  return  sphinxResultBuilder.buildFromUsingKeys(sphinxResultData, weightKeyPairs);
}

SphinxResultSorterIf& SphinxResultTruncater::selectResultSorter(SharedPtr<SphinxResultData>& sphinxResultData)
{
  unsigned int sortMode = 0;
  if (sphinxResultData->getRequestInfoCount() > 0){
    sortMode = sphinxResultData->getRequestInfoItem(0).getOrderBy();
  }
  switch (sortMode){
    case ASC_SORT:
      return _sphinxResultAscSorter;

    case DESC_SORT:
      return _sphinxResultDescSorter;
  }
  return _sphinxResultNullSorter;
}


void SphinxResultTruncater::trancateByWeight
         (vector<pair<string, unsigned long long> >& sortedWeightDocIds,
          SharedPtr<SphinxResultData>& sphinxResultData)
{
  unsigned int maxNumberToSave = 0;
  if (sphinxResultData->getRequestInfoCount() > 0){
    maxNumberToSave = sphinxResultData->getRequestInfoItem(0).getMaxResultsNumber();
  }
  if (sortedWeightDocIds.size() > maxNumberToSave){
    sortedWeightDocIds.erase(sortedWeightDocIds.begin() + maxNumberToSave,
                             sortedWeightDocIds.end());
  }
}

}
}
}
