/**
 * @file SphinxResultTrancater.hpp
 *
 * The SphinxResultTruncater truncates number of returned
 * documents after the reducing task.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef SPHINXRESULTTRUNCATER_HPP_
#define SPHINXRESULTTRUNCATER_HPP_

#include "SphinxResultTruncaterIf.hpp"
#include "SphinxResultSorterIf.hpp"
#include "SphinxResultNullSorter.hpp"
#include "SphinxResultAscSorter.hpp"
#include "SphinxResultDescSorter.hpp"

namespace HCE
{
namespace sphinx
{
namespace reduce_task
{

class SphinxResultTruncater: public SphinxResultTruncaterIf {
public:
  SphinxResultTruncater();
  virtual ~SphinxResultTruncater();

  Poco::SharedPtr<SphinxResultData> trancate(Poco::SharedPtr<SphinxResultData>& sphinxResultData);

private:
  SphinxResultNullSorter _sphinxResultNullSorter;
  SphinxResultAscSorter _sphinxResultAscSorter;
  SphinxResultDescSorter _sphinxResultDescSorter;

  SphinxResultSorterIf& selectResultSorter(Poco::SharedPtr<SphinxResultData> &sphinxResultData);
  void trancateByWeight(
           std::vector<std::pair<std::string, unsigned long long> > &sortedWeightDocIds,
           Poco::SharedPtr<SphinxResultData>& sphinxResultData);
};

}
}
}

#endif
