#include "SphinxTaskReducer.hpp"

namespace HCE{
namespace sphinx{
namespace reduce_task{

SphinxTaskReducer::SphinxTaskReducer(SphinxResultBuilderIf& sphinxResultBuilder,
                SphinxReduceJobBuilderIf& reduceJobBuilder,
                SphinxResultDataAccumulatorFactoryIf& sphinxResultDataAccumulatorFactory):
               _sphinxResultBuilder(sphinxResultBuilder)
{
  _reduceJob = reduceJobBuilder.build();
  Poco::SharedPtr<SphinxReduceDataStorage> dataStorage =
          _reduceJob->getDataStorage().unsafeCast<SphinxReduceDataStorage>();
  _sphinxResultDataAccumulator = sphinxResultDataAccumulatorFactory.build(dataStorage);
}

SphinxTaskReducer::~SphinxTaskReducer(){}

void SphinxTaskReducer::addData(SphinxResultData& sphinxResultData)
{
  _sphinxResultDataAccumulator->accumulate(sphinxResultData);
}

void SphinxTaskReducer::reduce(HCE::reduce::core::ScheduleStrategyIf& scheduleStrategy)
{
  scheduleStrategy.executeJob(_reduceJob);
}

Poco::SharedPtr<SphinxResultData> SphinxTaskReducer::getResult()
{
  Poco::SharedPtr<HCE::reduce::core::ReduceResultIf> reduceResult =
                      _reduceJob->getReduceResult();
  Poco::SharedPtr<SphinxReduceDataStorage> dataStorage =
     _sphinxResultDataAccumulator->getSphinxDataStorage();

  std::vector<SphinxRequestInfo>& requestInfo =
                        _sphinxResultDataAccumulator->getAllSphinxRequestInfo();

  Poco::SharedPtr<SphinxResultData> reduceDataResult = _sphinxResultBuilder.build(
                                              dataStorage, reduceResult, requestInfo);
  return reduceDataResult;
}

}
}
}
