/**
 * @file SphinxTaskReducer.hpp
 *
 *  SphinxTaskReducer makes the reducing job for data given from
 *  the sphinx search engine.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef SPHINXREDUCER_HPP_
#define SPHINXREDUCER_HPP_

#include "SphinxTaskReducerIf.hpp"
#include "SphinxResultDataAccumulatorFactory.hpp"
#include "SphinxReduceJobBuilder.hpp"
#include "SphinxResultData.hpp"
#include "SphinxResultBuilderIf.hpp"
#include "ScheduleStrategySingle.hpp"
#include <vector>

namespace HCE{
namespace sphinx{
namespace reduce_task{

class SphinxTaskReducer: public SphinxTaskReducerIf{
public:
  SphinxTaskReducer(SphinxResultBuilderIf& sphinxResultBuilder,
                SphinxReduceJobBuilderIf& reduceJobBuilder,
                SphinxResultDataAccumulatorFactoryIf& sphinxResultDataAccumulatorFactory);
  virtual ~SphinxTaskReducer();

  void addData(SphinxResultData& sphinxResultData);
  void reduce(HCE::reduce::core::ScheduleStrategyIf& scheduleStrategy);
  Poco::SharedPtr<SphinxResultData> getResult();

private:
  SphinxTaskReducer();
  SphinxResultBuilderIf& _sphinxResultBuilder;
  Poco::SharedPtr<HCE::reduce::core::ReduceJobIf> _reduceJob;
  Poco::SharedPtr<SphinxResultDataAccumulatorIf>  _sphinxResultDataAccumulator;
};
}
}
}

#endif
