/**
 * @file SphinxTaskReducerFactory.hpp
 *
 * SphinxTaskReducerFactory creates  all classes needed to make a reducing job(sphinx).
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef SPHINXTASKREDUCERFACTORY_HPP_
#define SPHINXTASKREDUCERFACTORY_HPP_

#include "SphinxTaskReducerFactoryIf.hpp"
#include "SphinxReduceJobBuilder.hpp"
#include "SphinxTaskReducer.hpp"
#include "SphinxResultBuilder.hpp"
#include "SphinxResultTransformer.hpp"
#include "ScheduleStrategySingle.hpp"
#include "SphinxResultTruncater.hpp"

namespace HCE{
namespace sphinx{
namespace reduce_task{


class SphinxTaskReducerFactory: public SphinxTaskReducerFactoryIf{
public:
  SphinxTaskReducerFactory();
  virtual ~SphinxTaskReducerFactory();

  Poco::SharedPtr<SphinxTaskReducerIf> create();
  HCE::reduce::core::ScheduleStrategyIf& getScheduleStrategy();
  SphinxResultTruncaterIf& getSphinxResultTrancater();

private:
  SphinxResultBuilder sphinxResultBuilder;
  SphinxReduceJobBuilder sphinxReduceJobBuilder;
  SphinxResultDataAccumulatorFactory sphinxResultDataAccumulatorFactory;
  HCE::reduce::core::ScheduleStrategySingle scheduleStrategy;
  SphinxResultTruncater sphinxResultTrancater;
};

}
}
}

#endif
