/**
 * @file SphinxTaskReducersManager.hpp
 *
 * SphinxTaskReducersManager manages the sphinx reducing tasks.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef SPHINXTASKREDUCERSMANAGER_HPP_
#define SPHINXTASKREDUCERSMANAGER_HPP_

#include "TaskReducersManagerIf.hpp"
#include "SphinxTaskReducersStorageIf.hpp"
#include "SphinxTaskReducerFactoryIf.hpp"
#include "SphinxReduceJobBuilder.hpp"
#include "SphinxResultJSONConvertorIf.hpp"
#include "SphinxResultSorterIf.hpp"
#include "Exceptions.hpp"

namespace HCE {
namespace sphinx {
namespace reduce_task {

class SphinxTaskReducersManager: public HCE::reduce::additions::TaskReducersManagerIf{
public:
  SphinxTaskReducersManager(
   const Poco::SharedPtr<SphinxTaskReducersStorageIf>& sphinxTaskReducersStorage,
   const Poco::SharedPtr<SphinxTaskReducerFactoryIf>& sphinxTaskReducerFactory,
   const Poco::SharedPtr<SphinxResultJSONConvertorIf>& sphinxResultJSONConvertor);
  virtual ~SphinxTaskReducersManager();

  types::MessageType getType();
  bool isReducerExist(unsigned long long reduceTaskId);
  void createReducer(unsigned long long reduceTaskId);
  void addDataInReducer(unsigned long long reduceTaskId, const std::string& json);
  std::string runReduceTaskForTaskId(unsigned long long reduceTaskId);
  void deleteReducerBy(unsigned long long reduceTaskId);

private:
  Poco::SharedPtr<SphinxTaskReducersStorageIf>_sphinxTaskReducersStorage;
  Poco::SharedPtr<SphinxTaskReducerFactoryIf>_sphinxTaskReducerFactory;
  Poco::SharedPtr<SphinxResultJSONConvertorIf>_sphinxResultJSONConvertor;

  Poco::SharedPtr<SphinxResultData> transformSphinxResultData(
                                         Poco::SharedPtr<SphinxResultData>& reduceResult);
};

}
}
}

#endif
