/**
 * @file SphinxTaskReducersStorage.hpp
 *
 * The SphinxTaskReducersStorage storages data which is used in the reducing tasks.
 *
 * @author Igor Savonik <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE Reducing Task
 * @since 0.1
 */

#ifndef SPHINXTASKREDUCERSSTORAGE_HPP_
#define SPHINXTASKREDUCERSSTORAGE_HPP_

#include "SphinxTaskReducersStorageIf.hpp"
#include "SphinxReduceJobBuilder.hpp"
#include "Exceptions.hpp"
#include <map>
#include <utility>

namespace HCE{
namespace sphinx{
namespace reduce_task{

class SphinxTaskReducersStorage: public SphinxTaskReducersStorageIf{
public:
  typedef std::map<unsigned long long, Poco::SharedPtr<SphinxTaskReducerIf> >::
                                                      iterator StorageIterator;

  SphinxTaskReducersStorage();
  virtual ~SphinxTaskReducersStorage();

  Poco::SharedPtr<SphinxTaskReducerIf>findByKey (unsigned long long taskId);
  void deleteByKey(unsigned long long taskId);
  void addWithKey(unsigned long long taskId, Poco::SharedPtr<SphinxTaskReducerIf>&
                                                              sphinxTaskReducer);

private:
  std::map<unsigned long long, Poco::SharedPtr<SphinxTaskReducerIf> >storage;

  StorageIterator findIteratorBy(unsigned long long taskId);
};

}
}
}

#endif
