/*
 * SphinxDataStorageIteratorTest.cpp
 *
 *  Created on: May 1, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include "SphinxReduceDataStorage.hpp"
#include "SphinxResultData.hpp"
#include <vector>

using namespace HCE::sphinx;
using namespace HCE::sphinx::reduce_task;
using namespace std;

TEST(SphinxReduceDataStorage, IterableIfMustBeEmptyAfterInit)
{
  SphinxReduceDataStorage sphinxReduceDataStorage;

  sphinxReduceDataStorage.reset();
  ASSERT_TRUE(sphinxReduceDataStorage.hasNext() == false);
}



class SphinxReduceDataStorageTest: public ::testing::Test{
  protected:
    virtual void SetUp(){
      SphinxMatchInfo sphinxMatchInfo1;
      sphinxMatchInfo1.setDocId(14);
      sphinxMatchInfo1.setWeightFromDecString("12345");

      SphinxMatchInfo sphinxMatchInfo2;
      sphinxMatchInfo2.setDocId(7);
      sphinxMatchInfo2.setWeightFromDecString("256789");

      SphinxMatchInfo sphinxMatchInfo3;
      sphinxMatchInfo3.setDocId(21);
      sphinxMatchInfo3.setWeightFromDecString("1122334");

      resources.push_back(sphinxMatchInfo1);
      resources.push_back(sphinxMatchInfo2);
      resources.push_back(sphinxMatchInfo3);

      expectIdsWeights.push_back(make_pair(to_string(14LLU), toHexString(12345LLU)));
      expectIdsWeights.push_back(make_pair(to_string(7LLU), toHexString(256789LLU)));
      expectIdsWeights.push_back(make_pair(to_string(21LLU), toHexString(1122334LLU)));
    }

    bool isFindExpectData(string &expectDocId, string &expectWeight){
      for(size_t index = 0; index < expectIdsWeights.size(); ++index){
         if (expectDocId == expectIdsWeights[index].first &&
             expectWeight == expectIdsWeights[index].second){
             return true;
         }
      }
      return false;
    }

    vector<pair<string, string> > expectIdsWeights;
    vector<SphinxMatchInfo> resources;
    string expectDocId;
    string expectWeight;
};


TEST_F(SphinxReduceDataStorageTest, addOneResult)
{
  SphinxReduceDataStorage sphinxReduceDataStorage;
  int index = 0;

  sphinxReduceDataStorage.addData(resources[index]);

  sphinxReduceDataStorage.reset();
  ASSERT_TRUE(sphinxReduceDataStorage.hasNext());
  sphinxReduceDataStorage.getNext(expectDocId, expectWeight);
  EXPECT_EQ(expectDocId, expectIdsWeights[index].first);
  EXPECT_EQ(expectWeight, expectIdsWeights[index].second);
}


TEST_F(SphinxReduceDataStorageTest, creationComplexKeyForInputData)
{
  SphinxReduceDataStorage sphinxReduceDataStorage;
  int index = 0;
  SphinxReduceDataStorage::DataRangeIterator storageDataIter;

  sphinxReduceDataStorage.addData(resources[index]);

  string expectComplexKey = expectIdsWeights[index].first + expectIdsWeights[index].second;

  storageDataIter = sphinxReduceDataStorage.getData(expectComplexKey);

  ASSERT_TRUE(storageDataIter.first != storageDataIter.second);
  EXPECT_EQ(storageDataIter.first->second.getDocId(), resources[index].getDocId());
  EXPECT_EQ(storageDataIter.first->second.getWeight(), resources[index].getWeight());
}


TEST_F(SphinxReduceDataStorageTest, addManyResult)
{
  SphinxReduceDataStorage sphinxReduceDataStorage;

  for (size_t index = 0; index < resources.size(); ++index){
    sphinxReduceDataStorage.addData(resources[index]);
  }

  unsigned int docsNumber = 0;
  sphinxReduceDataStorage.reset();
  while(sphinxReduceDataStorage.hasNext()){
    sphinxReduceDataStorage.getNext(expectDocId, expectWeight);
    EXPECT_TRUE(isFindExpectData(expectDocId, expectWeight));
    ++docsNumber;
  }
  ASSERT_EQ(docsNumber, expectIdsWeights.size());
}


TEST_F(SphinxReduceDataStorageTest, orderDataWithEqualComplexKey)
{
  SphinxReduceDataStorage sphinxReduceDataStorage;
  SphinxReduceDataStorage::DataRangeIterator storageDataIter;

  int doubleIndex = 1;
  SphinxMatchInfo sphinxMatchInfo4;
  sphinxMatchInfo4.setDocId(7);
  sphinxMatchInfo4.setWeightFromDecString("256789");
  AttrInfo attrInfo1 = AttrInfo("first", "value");
  sphinxMatchInfo4.addAttrInfo(attrInfo1);

  resources.push_back(sphinxMatchInfo4);

  for (size_t index = 0; index < resources.size(); ++index){
    sphinxReduceDataStorage.addData(resources[index]);
  }

  string expectComplexKey = expectIdsWeights[doubleIndex].first + expectIdsWeights[doubleIndex].second;
  storageDataIter = sphinxReduceDataStorage.getData(expectComplexKey);

  ASSERT_EQ(storageDataIter.first->second.getAttributes().size(), 0U);
  storageDataIter.first++;
  ASSERT_EQ(storageDataIter.first->second.getAttributes().size(), 1U);
  storageDataIter.first++;
  ASSERT_TRUE(storageDataIter.first == storageDataIter.second);
}
