/*
 * SphinxReduceJobTest.cpp
 *
 *  Created on: May 1, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include "SphinxReduceJob.hpp"
#include "ScheduleStrategySingle.hpp"
#include "SphinxResultCompareUtils.hpp"
#include <vector>

using namespace HCE::reduce::core;
using namespace HCE::sphinx;
using namespace HCE::sphinx::reduce_task;
using namespace std;
using namespace Poco;

class SphinxReduceJobTest: public ::testing::Test{
  protected:
    virtual void SetUp(){
      sphinxDataStorage.assign(new SphinxReduceDataStorage());
      sphinxReduceTask.assign(new SphinxReduceTask());
      sphinxReduceResult.assign(new SphinxReduceResult());
      sphinxReduceJob.assign(
        new SphinxReduceJob(sphinxDataStorage, sphinxReduceTask, sphinxReduceResult));
    }

    virtual void TearDown(){
    }

    static SharedPtr<SphinxReduceDataStorage>sphinxDataStorage;
    static SharedPtr<SphinxReduceTask>sphinxReduceTask;
    static SharedPtr<SphinxReduceResult>sphinxReduceResult;
    static SharedPtr<SphinxReduceJob>sphinxReduceJob;
};
SharedPtr<SphinxReduceDataStorage>SphinxReduceJobTest::sphinxDataStorage;
SharedPtr<SphinxReduceTask>SphinxReduceJobTest::sphinxReduceTask;
SharedPtr<SphinxReduceResult>SphinxReduceJobTest::sphinxReduceResult;
SharedPtr<SphinxReduceJob>SphinxReduceJobTest::sphinxReduceJob;


TEST_F(SphinxReduceJobTest, emptyJob)
{
  ScheduleStrategySingle scheduleStrategySingle;

  scheduleStrategySingle.executeJob(sphinxReduceJob);

  sphinxReduceResult->reset();
  ASSERT_TRUE(sphinxReduceResult->hasNext() == false);
}

TEST_F(SphinxReduceJobTest, manyDocs)
{
  string id;
  string weight;
  vector<SphinxMatchInfo> resources;

  resources.push_back(buildSimpleMatchInfo(22, 1234LLU));
  resources.push_back(buildSimpleMatchInfo(25LLU, 4565646LLU));
  resources.push_back(buildSimpleMatchInfo(22LLU, 4565646LLU));
  resources.push_back(buildSimpleMatchInfo(13LLU, 456564645LLU));

  for (size_t resIndex = 0; resIndex < resources.size(); ++resIndex){
    sphinxDataStorage->addData(resources[resIndex]);
  }

  ScheduleStrategySingle scheduleStrategySingle;

  scheduleStrategySingle.executeJob(sphinxReduceJob);

  int resNumber = 0;
  sphinxReduceResult->reset();
  while (sphinxReduceResult->hasNext()){
    sphinxReduceResult->getNext(id, weight);

    bool isFind = (
         (id == to_string(resources[1].getDocId()) && weight == resources[1].getWeight()) ||
         (id == to_string(resources[2].getDocId()) && weight == resources[2].getWeight()) ||
         (id == to_string(resources[3].getDocId()) && weight == resources[3].getWeight()));
    ASSERT_TRUE(isFind);
    resNumber++;
  }
  EXPECT_EQ(resNumber, 3);
  ASSERT_TRUE(sphinxReduceResult->hasNext() == false);
}
