/*
 * SphinxReduceTaskTest.cpp
 *
 *  Created on: May 2, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include "SphinxReduceTask.hpp"
#include "SphinxReduceResult.hpp"
#include <vector>

using namespace std;
using namespace Poco;
using namespace HCE::sphinx;
using namespace HCE::sphinx::reduce_task;

TEST(SphinxReduceTask, emptyDataStorage)
{
  SphinxReduceTask sphinxReduceTask;
  SharedPtr<SphinxReduceResult>sphinxReduceResult(new SphinxReduceResult());


  sphinxReduceTask.fillReduceResult(sphinxReduceResult);

  ASSERT_TRUE(sphinxReduceResult->hasNext() == false);
}


class SphinxReduceTaskTest: public ::testing::Test{
  protected:
    virtual void SetUp(){
      idWeightResult.push_back(make_pair("13", "12345"));
      idWeightResult.push_back(make_pair("13", "2567891"));
      idWeightResult.push_back(make_pair("21", "1122334"));
      idWeightResult.push_back(make_pair("221", "99123453"));
      idWeightResult.push_back(make_pair("221", "5525"));
    }


    bool isFindExpectData(string &expectDocId, string &expectWeight, vector<int>& bestResIndex){
      for(size_t index = 0; index < bestResIndex.size(); ++index){
         if (expectDocId == idWeightResult[bestResIndex[index]].first &&
             expectWeight == idWeightResult[bestResIndex[index]].second){
             return true;
         }
      }
      return false;
    }

    vector<pair<string, string> > idWeightResult;
    string expectId;
    string expectWeight;
};


TEST_F(SphinxReduceTaskTest, addOneResult)
{
  SphinxReduceTask sphinxReduceTask;
  SharedPtr<SphinxReduceResult>sphinxReduceResult(new SphinxReduceResult());

  sphinxReduceTask.addKeyValue(idWeightResult[0].first, idWeightResult[0].second);
  sphinxReduceTask.fillReduceResult(sphinxReduceResult);

  sphinxReduceResult->reset();
  ASSERT_TRUE(sphinxReduceResult->hasNext());
  sphinxReduceResult->getNext(expectId, expectWeight);

  EXPECT_EQ(expectId, idWeightResult[0].first);
  EXPECT_EQ(expectWeight, idWeightResult[0].second);

  ASSERT_TRUE(sphinxReduceResult->hasNext() == false);
}


TEST_F(SphinxReduceTaskTest, equalDocIdsSecondGreat)
{
  SphinxReduceTask sphinxReduceTask;
  SharedPtr<SphinxReduceResult>sphinxReduceResult(new SphinxReduceResult());

  sphinxReduceTask.addKeyValue(idWeightResult[0].first, idWeightResult[0].second);
  sphinxReduceTask.addKeyValue(idWeightResult[1].first, idWeightResult[1].second);
  sphinxReduceTask.fillReduceResult(sphinxReduceResult);

  sphinxReduceResult->reset();
  ASSERT_TRUE(sphinxReduceResult->hasNext());
  sphinxReduceResult->getNext(expectId, expectWeight);

  EXPECT_EQ(expectId, idWeightResult[1].first);
  EXPECT_EQ(expectWeight, idWeightResult[1].second);

  ASSERT_TRUE(sphinxReduceResult->hasNext() == false);
}


TEST_F(SphinxReduceTaskTest, equalDocIdsSecondLess)
{
  SphinxReduceTask sphinxReduceTask;
  SharedPtr<SphinxReduceResult>sphinxReduceResult(new SphinxReduceResult());

  sphinxReduceTask.addKeyValue(idWeightResult[3].first, idWeightResult[3].second);
  sphinxReduceTask.addKeyValue(idWeightResult[4].first, idWeightResult[4].second);
  sphinxReduceTask.fillReduceResult(sphinxReduceResult);

  sphinxReduceResult->reset();
  ASSERT_TRUE(sphinxReduceResult->hasNext());
  sphinxReduceResult->getNext(expectId, expectWeight);

  EXPECT_EQ(expectId, idWeightResult[3].first);
  EXPECT_EQ(expectWeight, idWeightResult[3].second);

  ASSERT_TRUE(sphinxReduceResult->hasNext() == false);
}


TEST_F(SphinxReduceTaskTest, addManyDocs)
{
  SphinxReduceTask sphinxReduceTask;
  SharedPtr<SphinxReduceResult>sphinxReduceResult(new SphinxReduceResult());

  for (size_t resIndex = 0; resIndex < idWeightResult.size(); ++resIndex){
    sphinxReduceTask.addKeyValue(idWeightResult[resIndex].first,
                                 idWeightResult[resIndex].second);
  }
  vector<int>bestResIndex = {1, 2, 3};

  sphinxReduceTask.fillReduceResult(sphinxReduceResult);

  sphinxReduceResult->reset();
  int docNumber = 0;

  while (sphinxReduceResult->hasNext()){
    sphinxReduceResult->getNext(expectId, expectWeight);
    EXPECT_TRUE(isFindExpectData(expectId, expectWeight, bestResIndex));
    docNumber++;
  }

  ASSERT_EQ(docNumber, 3);
}
