/*
 * SphinxReducerTest.cpp
 *
 *  Created on: May 3, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include "SphinxTaskReducer.hpp"
#include "SphinxResultBuilder.hpp"
#include "SphinxResultCompareUtils.hpp"
#include "ScheduleStrategySingle.hpp"

using namespace HCE::sphinx;
using namespace HCE::sphinx::reduce_task;
using namespace HCE::reduce::core;
using namespace Poco;
using namespace std;

class SphinxReducerIntergationTest: public ::testing::Test{
  protected:
    virtual void SetUp(){
      sphinxResultBuilder.assign(new SphinxResultBuilder());
      reduceJobBuilder.assign(new SphinxReduceJobBuilder());
      sphinxResultDataAccumulatorFactory.assign(new
                               SphinxResultDataAccumulatorFactory());

      sphinxReducer.assign(new SphinxTaskReducer(*sphinxResultBuilder.get(),
                           *reduceJobBuilder.get(),
                           *sphinxResultDataAccumulatorFactory.get()));

      WordInfo word1 = WordInfo("word1", 10, 3);
      WordInfo word2 = WordInfo("word2", 1, 1);

      matchInfo1 = buildSimpleMatchInfo(7LLU, 111LLU);
      matchInfo2 = buildSimpleMatchInfo(9LLU, 222LLU);
      matchInfo3 = buildSimpleMatchInfo(15LLU, 674387LLU);
      matchInfo4 = buildSimpleMatchInfo(9LLU, 674387LLU);

      AttrInfo attrInfo1 = AttrInfo("first", "value");

      matchInfo1.addAttrInfo(attrInfo1);
      matchInfo2.addAttrInfo(attrInfo1);
      matchInfo3.addAttrInfo(attrInfo1);
      matchInfo4.addAttrInfo(attrInfo1);

      sphinxRequestInfo1.setNodeName("nodeName1");
      sphinxRequestInfo1.setQuery(query1);
      sphinxRequestInfo1.setTotal(totalRes1);
      sphinxRequestInfo1.setTotalFound(totalFound1);
      sphinxRequestInfo1.setTimeMsec(time1);
      sphinxRequestInfo1.addWordInfo(word1);
      sphinxRequestInfo1.addWordInfo(word2);

      sphinxRequestInfo2.setNodeName("nodeName2");
      sphinxRequestInfo2.addWordInfo(word1);
      sphinxRequestInfo2.addWordInfo(word2);

      sphinxResultData1.addMatchInfo(matchInfo1);
      sphinxResultData1.addMatchInfo(matchInfo2);
      sphinxResultData1.addRequestInfo(sphinxRequestInfo1);

      sphinxResultData2.addMatchInfo(matchInfo3);
      sphinxResultData2.addMatchInfo(matchInfo4);
      sphinxResultData2.addRequestInfo(sphinxRequestInfo2);
    }

    void buildExpectedSphinxResultData(SphinxResultData& expected){
      expected.clear();
      expected.addMatchInfo(matchInfo1);
      expected.addMatchInfo(matchInfo4);
      expected.addMatchInfo(matchInfo3);
      expected.addRequestInfo(sphinxRequestInfo1);
      expected.addRequestInfo(sphinxRequestInfo2);
    }

    SharedPtr<SphinxResultBuilder> sphinxResultBuilder;
    SharedPtr<SphinxReduceJobBuilder>reduceJobBuilder;
    SharedPtr<SphinxResultDataAccumulatorFactory>sphinxResultDataAccumulatorFactory;
    SharedPtr<SphinxTaskReducer>sphinxReducer;
    ScheduleStrategySingle scheduleStrategy;

    string query1;
    int totalRes1;
    int totalFound1;
    int time1;

    SphinxRequestInfo sphinxRequestInfo1;
    SphinxRequestInfo sphinxRequestInfo2;
    SphinxMatchInfo matchInfo1;
    SphinxMatchInfo matchInfo2;
    SphinxMatchInfo matchInfo3;
    SphinxMatchInfo matchInfo4;
    SphinxResultData sphinxResultData1;
    SphinxResultData sphinxResultData2;
};

TEST_F(SphinxReducerIntergationTest, getEmptyResult)
{
  SharedPtr<SphinxResultData> reducedSphinxResult = sphinxReducer->getResult();
  SphinxResultData emptySphinxResult;

  ASSERT_TRUE(isEqualSphinxResultData(*reducedSphinxResult.get(), emptySphinxResult));
}


TEST_F(SphinxReducerIntergationTest, reduceEmptyData)
{
  SphinxResultData emptyInputSphinxResult;
  SphinxResultData emptySphinxResult;

  sphinxReducer->addData(emptyInputSphinxResult);
  sphinxReducer->reduce(scheduleStrategy);
  SharedPtr<SphinxResultData> reducedSphinxResult = sphinxReducer->getResult();

  ASSERT_TRUE(isEqualSphinxResultData(*reducedSphinxResult.get(), emptySphinxResult));
}


TEST_F(SphinxReducerIntergationTest, reduceOneResult)
{
  sphinxReducer->addData(sphinxResultData1);
  sphinxReducer->reduce(scheduleStrategy);
  SharedPtr<SphinxResultData> reducedSphinxResult = sphinxReducer->getResult();

  ASSERT_TRUE(isEqualSphinxResultData(*reducedSphinxResult.get(), sphinxResultData1));
}


TEST_F(SphinxReducerIntergationTest, reduceTwoResults)
{
  ScheduleStrategySingle scheduleStrategy;
  SphinxResultData expectedSphinxResult;
  buildExpectedSphinxResultData(expectedSphinxResult);

  sphinxReducer->addData(sphinxResultData1);
  sphinxReducer->addData(sphinxResultData2);
  sphinxReducer->reduce(scheduleStrategy);
  SharedPtr<SphinxResultData> reducedSphinxResult = sphinxReducer->getResult();

  ASSERT_TRUE(isEqualRequestInfo(*reducedSphinxResult.get(), expectedSphinxResult));
  ASSERT_TRUE(isEqualUnSortMatchInfo(*reducedSphinxResult.get(), expectedSphinxResult));
}

