/*
 * SphinxResultCompareUtils.cpp
 *
 *  Created on: May 3, 2013
 *      Author: igor
 */

#include "SphinxResultCompareUtils.hpp"
#include "SphinxResultDataSerializator.hpp"
#include <algorithm>

bool isEqualWords(const WordInfo& word1, const WordInfo& word2)
{
    return word1.word == word2.word &&
    word1.hits == word2.hits &&
    word1.docs == word2.docs;
}

bool isEqualWordInfo(const vector<SphinxResultDataItem<WordInfo> > &checkWords,
                     const vector<SphinxResultDataItem<WordInfo> > &etalonWords){
  if (checkWords.size() != etalonWords.size()){
    return false;
  }
  for (size_t wordIndex = 0; wordIndex < checkWords.size(); ++wordIndex){
    if (!isEqualWords(checkWords[wordIndex].getItem(), etalonWords[wordIndex].getItem())){
      return false;
    }
  }
  return true;
}


bool isEqualRequestInfoStatParams(const SphinxRequestInfo &checkInfo, const SphinxRequestInfo &etalonInfo){
  if (!isEqualWordInfo(checkInfo.getWords(), etalonInfo.getWords())){
    return false;
  }
  if (!(checkInfo.getNodeName() == etalonInfo.getNodeName()) &&
         (checkInfo.getQuery() == etalonInfo.getQuery()) &&
         (checkInfo.getQueryId() == etalonInfo.getQueryId()) &&
         (checkInfo.getMaxResultsNumber() && etalonInfo.getMaxResultsNumber()) &&
         (checkInfo.getOrderBy() == etalonInfo.getOrderBy()) &&
         (checkInfo.getTotal() == etalonInfo.getTotal()) &&
         (checkInfo.getTotalFound() == etalonInfo.getTotalFound()) &&
         (checkInfo.getTimeMsec() == etalonInfo.getTimeMsec())){
    return false;
  }
  return true;
}

bool isEqualRequestInfo(SphinxResultData &checkResult, SphinxResultData& etalonResult){
  if (checkResult.getRequestInfoCount() != etalonResult.getRequestInfoCount()){
    return false;
  }
  for (size_t itemIndex = 0; itemIndex < checkResult.getRequestInfoCount(); ++itemIndex){
    if (!isEqualRequestInfoStatParams(checkResult.getRequestInfoItem(itemIndex),
                                      etalonResult.getRequestInfoItem(itemIndex))){
      return false;
    }
  }
  return true;
}

bool isEqAttrInfo(const vector<SphinxResultDataItem<AttrInfo> > &check,
                  const vector<SphinxResultDataItem<AttrInfo> > &etalon)
{
  if (check.size() != etalon.size()){
    return false;
  }
  for (size_t itemIndex = 0; itemIndex < check.size(); ++itemIndex){
    if (!(check[itemIndex].getItem().name == etalon[itemIndex].getItem().name &&
          check[itemIndex].getItem().value == etalon[itemIndex].getItem().value)){
      return false;
    }
  }
  return true;
}


bool isMatchInfoEq(const SphinxMatchInfo &checkInfo, const SphinxMatchInfo &etalonInfo){
  if (!isEqAttrInfo(checkInfo.getAttributes(), etalonInfo.getAttributes())){
     return false;
  }
  return checkInfo.getDocId()  == etalonInfo.getDocId() &&
         checkInfo.getWeight() == etalonInfo.getWeight();
}


bool isEqualMatchInfo(SphinxResultData &checkResult, SphinxResultData& etalonResult){
  if (checkResult.getMatchInfoCount() != etalonResult.getMatchInfoCount()){
    return false;
  }
  for (size_t itemIndex = 0; itemIndex < checkResult.getRequestInfoCount(); ++itemIndex){
    if (!isMatchInfoEq(checkResult.getMatchInfoItem(itemIndex),
                     etalonResult.getMatchInfoItem(itemIndex))){
      return false;
    }
  }
  return true;
}


bool isEqualUnSortMatchInfo(SphinxResultData &checkResult, SphinxResultData& etalonResult){
  if (checkResult.getMatchInfoCount() != etalonResult.getMatchInfoCount()){
    return false;
  }
  for (size_t itemIndex = 0; itemIndex < checkResult.getMatchInfoCount(); ++itemIndex){
    bool found = false;
    for(size_t etIndex = 0; etIndex < etalonResult.getMatchInfoCount() && !found; ++etIndex){
      if (isMatchInfoEq(checkResult.getMatchInfoItem(itemIndex),
                        etalonResult.getMatchInfoItem(etIndex))){
        found = true;
      }
    }
    if (!found){
      return false;
    }
  }
  return true;
}


bool isEqualSphinxResultData(SphinxResultData &checkResult, SphinxResultData& etalonResult){
  if (!isEqualRequestInfo(checkResult, etalonResult)){
    return false;
  }
  if (!isEqualMatchInfo(checkResult, etalonResult)){
    return false;
  }
  return true;
}

string buildInputJSONFromSphinxResultData(SphinxResultData& sphinxResultData)
{
  SphinxResultDataSerializator sphinxResultDataSerializator(sphinxResultData);
  string json = "";
  sphinxResultDataSerializator.setJsonType(15);
  sphinxResultDataSerializator.serialize(json);
  return json;
}

string buildOutputJSONFromSphinxResultData(SphinxResultData& sphinxResultData)
{
  SphinxResultDataSerializator sphinxResultDataSerializator(sphinxResultData);
  string json = "";
  sphinxResultDataSerializator.serialize(json);
  return json;
}

SphinxMatchInfo buildSimpleMatchInfo(unsigned long long id, unsigned long long weight){
  SphinxMatchInfo sphinxMatchInfo;
  sphinxMatchInfo.setDocId(id);
  sphinxMatchInfo.setWeightFromNumeric(weight);
  return sphinxMatchInfo;
}


