/*
 * SphinxResultAccumulatorTest.cpp
 *
 *  Created on: May 15, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include "SphinxResultDataAccumulator.hpp"
#include "SphinxReduceDataStorage.hpp"
#include "SphinxResultCompareUtils.hpp"

using namespace HCE::sphinx;
using namespace HCE::sphinx::reduce_task;
using namespace Poco;

TEST(SphinxResultDataAccumulator, accumulateEmptyResult)
{
  SphinxResultData emptySphinxResultData;
  SharedPtr<SphinxReduceDataStorage>sphinxDataStorage(new SphinxReduceDataStorage());

  SphinxResultDataAccumulator sphinxResultDataAccumulator(sphinxDataStorage);

  sphinxResultDataAccumulator.accumulate(emptySphinxResultData);

  ASSERT_EQ(sphinxResultDataAccumulator.getAllSphinxRequestInfo().size(), 0U);
  sphinxResultDataAccumulator.getSphinxDataStorage()->reset();
  ASSERT_TRUE(sphinxResultDataAccumulator.getSphinxDataStorage()->hasNext() == false);
}


class SphinxResultDataAccumulatorTest: public ::testing::Test{
  protected:
    virtual void SetUp(){
      query1 = "query";
      totalRes1 = 2;
      totalFound1 = 13;
      time1 = 55;

      WordInfo word1 = WordInfo("word1", 10, 3);
      WordInfo word2 = WordInfo("word2", 1, 1);

      matchInfo1 = buildSimpleMatchInfo(7LLU, 11LLU);
      matchInfo2 = buildSimpleMatchInfo(9LLU, 22LLU);
      matchInfo3 = buildSimpleMatchInfo(15LLU, 674387LLU);

      AttrInfo attrInfo1("first", "value");

      matchInfo1.addAttrInfo(attrInfo1);
      matchInfo2.addAttrInfo(attrInfo1);
      matchInfo3.addAttrInfo(attrInfo1);

      sphinxRequestInfo.setQuery(query1);
      sphinxRequestInfo.setTotal(totalRes1);
      sphinxRequestInfo.setTotalFound(totalFound1);
      sphinxRequestInfo.setTimeMsec(time1);
      sphinxRequestInfo.addWordInfo(word1);
      sphinxRequestInfo.addWordInfo(word2);

      sphinxResultData1.addMatchInfo(matchInfo1);
      sphinxResultData1.addMatchInfo(matchInfo2);
      sphinxResultData1.addRequestInfo(sphinxRequestInfo);

      sphinxResultData2.addMatchInfo(matchInfo3);
    }

    string query1;
    string key;
    string weight;
    int totalRes1;
    int totalFound1;
    int time1;

    SphinxRequestInfo sphinxRequestInfo;
    SphinxMatchInfo matchInfo1;
    SphinxMatchInfo matchInfo2;
    SphinxMatchInfo matchInfo3;
    SphinxResultData sphinxResultData1;
    SphinxResultData sphinxResultData2;
};


TEST_F(SphinxResultDataAccumulatorTest, accumulateSeveralResources)
{
  map<string, bool> expectStorage;
  expectStorage.insert(make_pair(to_string(matchInfo1.getDocId()) + matchInfo1.getWeight(),
                                 true));
  expectStorage.insert(make_pair(to_string(matchInfo2.getDocId()) + matchInfo2.getWeight(),
                                 true));
  expectStorage.insert(make_pair(to_string(matchInfo3.getDocId()) + matchInfo3.getWeight(),
                                 true));

  int storageElemNumber = 0;
  SharedPtr<SphinxReduceDataStorage>sphinxDataStorage(new SphinxReduceDataStorage());
  SphinxResultDataAccumulator sphinxResultDataAccumulator(sphinxDataStorage);

  sphinxResultDataAccumulator.accumulate(sphinxResultData1);
  sphinxResultDataAccumulator.accumulate(sphinxResultData2);

  //check info
  ASSERT_EQ(sphinxResultDataAccumulator.getAllSphinxRequestInfo().size(), 1U);

  EXPECT_TRUE(isEqualRequestInfoStatParams(sphinxResultDataAccumulator.getAllSphinxRequestInfo()[0],
                                           sphinxRequestInfo));

  sphinxResultDataAccumulator.getSphinxDataStorage()->reset();
  while(sphinxResultDataAccumulator.getSphinxDataStorage()->hasNext()){
    sphinxResultDataAccumulator.getSphinxDataStorage()->getNext(key, weight);
    ASSERT_TRUE(expectStorage.find(key + weight) != expectStorage.end());
    storageElemNumber++;
  }
  ASSERT_EQ(storageElemNumber, 3);
}


