/*
 * SphinxResultJSONConvertorTest.cpp
 *
 *  Created on: May 21, 2013
 *      Author: igor
 */
#include <gtest/gtest.h>
#include "SphinxResultJSONConvertor.hpp"
#include "SphinxOutputJsonMessage.hpp"
#include "SphinxResultCompareUtils.hpp"

using namespace HCE::sphinx;
using namespace HCE::sphinx::reduce_task;
using namespace HCE::reduce;
using namespace Poco;

class SphinxResultJSONConvertorTest: public ::testing::Test{
  protected:
    virtual void SetUp(){
      query = "query";
      totalRes = 4;
      totalFound = 10;
      time = 12;
      WordInfo word1 = WordInfo("word1", 10, 3);
      WordInfo word2 = WordInfo("word2", 1, 1);

      SphinxMatchInfo sphinxMatchInfo1 = buildSimpleMatchInfo(7LLU, 111LLU);
      SphinxMatchInfo sphinxMatchInfo2 = buildSimpleMatchInfo(9LLU, 222LLU);

      AttrInfo pAttrInfo1("first", "value");

      sphinxMatchInfo1.addAttrInfo(pAttrInfo1);
      sphinxMatchInfo2.addAttrInfo(pAttrInfo1);

      SphinxRequestInfo sphinxRequestInfo;
      sphinxRequestInfo.setQuery(query);
      sphinxRequestInfo.setTotal(totalRes);
      sphinxRequestInfo.setTotalFound(totalFound);
      sphinxRequestInfo.setTimeMsec(time);
      sphinxRequestInfo.addWordInfo(word1);
      sphinxRequestInfo.addWordInfo(word2);

      sphinxResultData.assign(new SphinxResultData());
      sphinxResultData->addMatchInfo(sphinxMatchInfo1);
      sphinxResultData->addMatchInfo(sphinxMatchInfo2);
      sphinxResultData->addRequestInfo(sphinxRequestInfo);
    }
    static string query;
    static int totalRes;
    static int totalFound;
    static int time;
    SharedPtr<SphinxResultData> sphinxResultData;
};
string SphinxResultJSONConvertorTest::query;
int SphinxResultJSONConvertorTest::totalRes;
int SphinxResultJSONConvertorTest::totalFound;
int SphinxResultJSONConvertorTest::time;

TEST_F(SphinxResultJSONConvertorTest, createSphinxResulDatatFromJSONString)
{
  SphinxResultJSONConvertor sphinxResultJSONConvertor;

  string json = buildInputJSONFromSphinxResultData(*sphinxResultData.get());


  SharedPtr<SphinxResultData> builtSphinxResult =
                          sphinxResultJSONConvertor.convertToSphinxResultDataFrom(json);

  ASSERT_TRUE(isEqualSphinxResultData(*sphinxResultData.get(), *builtSphinxResult.get()));
}


TEST_F(SphinxResultJSONConvertorTest, createJSONFromSphinxResultData)
{
  SphinxResultJSONConvertor sphinxResultJSONConvertor;

  string etalonJson = buildOutputJSONFromSphinxResultData(*sphinxResultData.get());

  string builtJSON = sphinxResultJSONConvertor.convertToJSONFrom(sphinxResultData);

  ASSERT_EQ(etalonJson, builtJSON);
}


TEST_F(SphinxResultJSONConvertorTest, throwWrongJSONStructureExceptionWhileParsingWrongJSONFormat)
{
  SphinxResultJSONConvertor sphinxResultJSONConvertor;
  string wrongStructuredJSON = "wrong structure";

  ASSERT_THROW(sphinxResultJSONConvertor.convertToSphinxResultDataFrom(wrongStructuredJSON),
               WrongJSONStructureException);
}

