/*
 * SphinxResultSortersTest.cpp
 *
 *  Created on: Jun 11, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include "SphinxResultNullSorter.hpp"
#include "SphinxResultDescSorter.hpp"
#include "SphinxResultAscSorter.hpp"

using namespace HCE::sphinx::reduce_task;
using namespace HCE::sphinx;
using namespace Poco;
using namespace std;

class SphinxResultSortersTest: public ::testing::Test{
  protected:
    virtual void SetUp(){
      expectedNullResult.push_back(make_pair( toHexString(111) + toHexString( 222 ), 7));
      expectedNullResult.push_back(make_pair( toHexString(222) + toHexString( 111 ), 9));
      expectedNullResult.push_back(make_pair( toHexString(222) + toHexString( 111 ), 11));
      expectedNullResult.push_back(make_pair( toHexString(99) + toHexString( 444 ), 12));
      expectedNullResult.push_back(make_pair( toHexString(444) + toHexString( 99 ), 13));

      expectedDescResult.push_back(make_pair( toHexString(444) + toHexString( 99 ), 13));
      expectedDescResult.push_back(make_pair( toHexString(222) + toHexString( 111 ), 9));
      expectedDescResult.push_back(make_pair( toHexString(222) + toHexString( 111 ), 11));
      expectedDescResult.push_back(make_pair( toHexString(111) + toHexString( 222 ), 7));
      expectedDescResult.push_back(make_pair( toHexString(99) + toHexString( 444 ), 12));

      expectedAscResult.push_back(make_pair(toHexString(99) + toHexString( 444 ), 12));
      expectedAscResult.push_back(make_pair(toHexString(111) + toHexString( 222 ), 7));
      expectedAscResult.push_back(make_pair(toHexString(222) + toHexString( 111 ), 9));
      expectedAscResult.push_back(make_pair(toHexString(222) + toHexString( 111 ), 11));
      expectedAscResult.push_back(make_pair(toHexString(444) + toHexString( 99 ), 13));

      initResult.insert(initResult.begin(), expectedNullResult.begin(),
                                            expectedNullResult.end());
    }

    virtual void TearDown(){
       expectedAscResult.clear();
       expectedDescResult.clear();
       expectedNullResult.clear();
    }

    vector<pair<string, unsigned long long> > initResult;
    static vector<pair<string, unsigned long long> >expectedDescResult;
    static vector<pair<string, unsigned long long> >expectedAscResult;
    static vector<pair<string, unsigned long long> >expectedNullResult;
};
vector<pair<string, unsigned long long> >SphinxResultSortersTest::expectedDescResult;
vector<pair<string, unsigned long long> >SphinxResultSortersTest::expectedAscResult;
vector<pair<string, unsigned long long> >SphinxResultSortersTest::expectedNullResult;


TEST_F(SphinxResultSortersTest, dontSortResourcesByWeight)
{
  SphinxResultNullSorter sphinxResultNullSorter;

  sphinxResultNullSorter.sortByWeight(initResult);

  ASSERT_EQ(initResult, expectedNullResult);
}


TEST_F(SphinxResultSortersTest, sortResourcesByWeightDesc)
{
  SphinxResultDescSorter sphinxResultDescSorter;

  sphinxResultDescSorter.sortByWeight(initResult);

  ASSERT_EQ(initResult, expectedDescResult);
}


TEST_F(SphinxResultSortersTest, sortResourcesByWeightAsc)
{
  SphinxResultAscSorter sphinxResultAscSorter;

  sphinxResultAscSorter.sortByWeight(initResult);

  ASSERT_EQ(initResult, expectedAscResult);
}

