/*
 * SphinxResultTrancaterTest.cpp
 *
 *  Created on: Jun 11, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include "SphinxResultTruncater.hpp"
#include "SphinxResultCompareUtils.hpp"

using namespace HCE::sphinx;
using namespace HCE::sphinx::reduce_task;
using namespace Poco;

class SphinxResultTrancaterTest: public ::testing::Test{
  protected:
    virtual void SetUp(){

      matchInfo1 = buildSimpleMatchInfo(7LLU, 111LLU);
      matchInfo2 = buildSimpleMatchInfo(9LLU, 222LLU);
      matchInfo3 = buildSimpleMatchInfo(11LLU, 222LLU);
      matchInfo4 = buildSimpleMatchInfo(12LLU, 99LLU);
      matchInfo5 = buildSimpleMatchInfo(13LLU, 444LLU);

      AttrInfo pAttrInfo("first", "value");
      matchInfo1.addAttrInfo(pAttrInfo);
      matchInfo2.addAttrInfo(pAttrInfo);
      matchInfo3.addAttrInfo(pAttrInfo);
      matchInfo4.addAttrInfo(pAttrInfo);
      matchInfo5.addAttrInfo(pAttrInfo);

      sphinxRequestInfo.setMaxResultsNumber(3);
      sphinxRequestInfo.setOrderBy(NULL_SORT);

      sphinxResultData.assign(new SphinxResultData());
      sphinxResultData->addMatchInfo(matchInfo1);
      sphinxResultData->addMatchInfo(matchInfo2);
      sphinxResultData->addMatchInfo(matchInfo5);
      sphinxResultData->addMatchInfo(matchInfo3);
      sphinxResultData->addMatchInfo(matchInfo4);

      expectSphinxResultData.addMatchInfo(matchInfo5);
      expectSphinxResultData.addMatchInfo(matchInfo2);
      expectSphinxResultData.addMatchInfo(matchInfo3);
    }

    SphinxRequestInfo sphinxRequestInfo;
    SphinxMatchInfo matchInfo1;
    SphinxMatchInfo matchInfo2;
    SphinxMatchInfo matchInfo3;
    SphinxMatchInfo matchInfo4;
    SphinxMatchInfo matchInfo5;
    SharedPtr<SphinxResultData> sphinxResultData;
    SphinxResultData expectSphinxResultData;
};

TEST_F(SphinxResultTrancaterTest, correctTrancareResult)
{
  SphinxResultTruncater sphinxResultTrancater;

  sphinxRequestInfo.setOrderBy(DESC_SORT);
  sphinxResultData->addRequestInfo(sphinxRequestInfo);
  expectSphinxResultData.addRequestInfo(sphinxRequestInfo);

  SharedPtr<SphinxResultData> trancatedResult = sphinxResultTrancater.trancate(sphinxResultData);

  ASSERT_TRUE(isEqualSphinxResultData(*trancatedResult.get(), expectSphinxResultData));
}


TEST_F(SphinxResultTrancaterTest, correctTruncateLessResultThanNeedToBeStored)
{
  SphinxResultTruncater sphinxResultTrancater;

  sphinxRequestInfo.setMaxResultsNumber(10);
  sphinxResultData->addRequestInfo(sphinxRequestInfo);

  SharedPtr<SphinxResultData> trancatedResult = sphinxResultTrancater.trancate(sphinxResultData);

  ASSERT_TRUE(isEqualSphinxResultData(*trancatedResult.get(),
                                      *sphinxResultData.get()));
}


TEST_F(SphinxResultTrancaterTest, correctTruncateEqualResultThanNeedToBeStored)
{
  SphinxResultTruncater sphinxResultTrancater;

  sphinxRequestInfo.setMaxResultsNumber(5);
  sphinxResultData->addRequestInfo(sphinxRequestInfo);

  SharedPtr<SphinxResultData> trancatedResult = sphinxResultTrancater.trancate(sphinxResultData);

  ASSERT_TRUE(isEqualSphinxResultData(*trancatedResult.get(),
                                        *sphinxResultData.get()));
}
