/*
 * SphinxResultTransformerTest.cpp
 *
 *  Created on: May 9, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include <gmock/gmock.h>
#include "SphinxResultTransformer.hpp"
#include "SphinxResultData.hpp"
#include "SphinxResultCompareUtils.hpp"

using namespace HCE::sphinx;
using namespace HCE::sphinx::reduce_task;
using namespace Poco;

using ::testing::ContainerEq;

class SphinxResultTransformerTest: public ::testing::Test{
  protected:
    virtual void SetUp(){
        nodeName = "node1";
        query = "My query";
        totalRes = 10;
        totalFound = 2;
        time = 1020;

        WordInfo word1 = WordInfo("word1", 10, 3);
        WordInfo word2 = WordInfo("word2", 1, 1);

        SphinxMatchInfo matchInfo1 = buildSimpleMatchInfo(7LLU, 111LLU);
        SphinxMatchInfo matchInfo2 = buildSimpleMatchInfo(9LLU, 222LLU);
        SphinxMatchInfo matchInfo3 = buildSimpleMatchInfo(17LLU, 99LLU);
        SphinxMatchInfo matchInfo4 = buildSimpleMatchInfo(9LLU, 444LLU);

        expectedWeightKeys.push_back(make_pair(toHexString(111LLU), 7LLU));
        expectedWeightKeys.push_back(make_pair(toHexString(222LLU), 9LLU));
        expectedWeightKeys.push_back(make_pair(toHexString(99LLU), 17LLU));
        expectedWeightKeys.push_back(make_pair(toHexString(444LLU), 9LLU));

        AttrInfo attrInfo("first", "value");

        matchInfo1.addAttrInfo(attrInfo);
        matchInfo2.addAttrInfo(attrInfo);
        matchInfo3.addAttrInfo(attrInfo);
        matchInfo4.addAttrInfo(attrInfo);

        SphinxRequestInfo sphinxRequestInfo;
        sphinxRequestInfo.setQuery(query);
        sphinxRequestInfo.setTotal(totalRes);
        sphinxRequestInfo.setTotalFound(totalFound);
        sphinxRequestInfo.setTimeMsec(time);
        sphinxRequestInfo.addWordInfo(word1);
        sphinxRequestInfo.addWordInfo(word2);

        sphinxResultData.assign(new SphinxResultData);
        sphinxResultData->addMatchInfo(matchInfo1);
        sphinxResultData->addMatchInfo(matchInfo2);
        sphinxResultData->addMatchInfo(matchInfo3);
        sphinxResultData->addMatchInfo(matchInfo4);
        sphinxResultData->addRequestInfo(sphinxRequestInfo);
      }

    SharedPtr<SphinxResultData> sphinxResultData;
    vector<pair<string, unsigned long long> > expectedWeightKeys;
    static string query;
    static string nodeName;
    static int totalRes;
    static int totalFound;
    static int time;
};
string SphinxResultTransformerTest::query;
string SphinxResultTransformerTest::nodeName;
int SphinxResultTransformerTest::totalRes;
int SphinxResultTransformerTest::totalFound;
int SphinxResultTransformerTest::time;


TEST_F(SphinxResultTransformerTest, emptyDataShouldExtractEmptyData)
{
  SphinxResultTransformer sphinxResultTransformer;

  sphinxResultData.assign(new SphinxResultData);
  vector<pair<string, unsigned long long> > weightKeys;

  sphinxResultTransformer.extractWeightKeys(sphinxResultData, weightKeys);

  ASSERT_EQ(weightKeys.size(), 0U);
}


TEST_F(SphinxResultTransformerTest, extractCorrectWeightKeysPairs)
{
  SphinxResultTransformer sphinxResultTransformer;

  vector<pair<string, unsigned long long> > weightKeys;

  sphinxResultTransformer.extractWeightKeys(sphinxResultData, weightKeys);

  ASSERT_EQ(expectedWeightKeys, weightKeys);
}
