/*
 * SphinxTaskReducersStorageTest.cpp
 *
 *  Created on: May 20, 2013
 *      Author: igor
 */

#include <gtest/gtest.h>
#include "SphinxTaskReducersStorage.hpp"
#include "Exceptions.hpp"

using namespace HCE::sphinx;
using namespace HCE::reduce;
using namespace HCE::sphinx::reduce_task;
using namespace Poco;

TEST(SphinxTaskReducersStorage, returnReducerByValidKey)
{
  SphinxTaskReducersStorage sphinxTaskReducersStorage;
  SharedPtr<SphinxTaskReducerIf>sphinxTaskReducer;
  int taskId = 1;

  sphinxTaskReducersStorage.addWithKey(taskId, sphinxTaskReducer);

  ASSERT_EQ(sphinxTaskReducer, sphinxTaskReducersStorage.findByKey(taskId));
}


TEST(SphinxTaskReducersStorage, throwNotFoundSphinxTaskReducerExceptionWhileSearchByInvalidKey)
{
  SphinxTaskReducersStorage sphinxTaskReducersStorage;
  int invalidTaskId = 8;

  ASSERT_THROW(sphinxTaskReducersStorage.findByKey(invalidTaskId),
               NotFoundByKeyException);
}


TEST(SphinxTaskReducersStorage, throwNotFoundSphinxTaskReducerExceptionWhileDeleteByInvalidKey)
{
  SphinxTaskReducersStorage sphinxTaskReducersStorage;
  int invalidTaskId = 8;

  ASSERT_THROW(sphinxTaskReducersStorage.deleteByKey(invalidTaskId),
               NotFoundByKeyException);
}


TEST(SphinxTaskReducersStorage, correctDeleteSphinxTaskReducerByValidKey)
{
  SphinxTaskReducersStorage sphinxTaskReducersStorage;
  SharedPtr<SphinxTaskReducerIf>sphinxTaskReducer;

  int taskId = 1;
  sphinxTaskReducersStorage.addWithKey(taskId, sphinxTaskReducer);
  sphinxTaskReducersStorage.deleteByKey(taskId);

  ASSERT_THROW(sphinxTaskReducersStorage.findByKey(taskId),
               NotFoundByKeyException);
}

