#ifndef CWORDS_HPP
#define CWORDS_HPP


#include <string>

#include "MorphChangeGradBitset.hpp"
#include "POSMaskBitset.hpp"
#include "Defs.hpp"


namespace HCE
{

	class CWords
	{
    private:
      unsigned long long _id;		/// 		crawler
      unsigned char _black;			/// refine
  	  MorphChangeGradBitset<MCG_NUM> _simClass; 		/// refine 	two  bytes morphology ( MorphChangeGrad )
      unsigned int _hCrc32;			/// 		crawler	CRC32 word ( CRC word for highlight on CDR )
      unsigned int _offset; 			/// refine offset word from beginning content in BYTES
      unsigned int _sentenceNumber; 	/// refine 	(deprecated) number word's sentence, start from begin
      unsigned char _lingIntegrity; 	/// refine	valuable of the word in the content ( val/unval content )
      unsigned int _initWordLen;		///	refine
      std::string _normWord;			/// refine
      std::string _originWord;			/// refine
      unsigned short _posMask;
      WORD_TYPE _wordType;                           //!< Token type: { WORD, NUMBER, DELIMITER, PUNCTUATION }
      //POSMaskBitset<POS_NUM> _posMask;
    public:
      //enum { UNDEFINED=0, DELIMITER, WORD, NUMBER, PUNCTUATION };
    public:
        CWords()
            : _id(0)
            , _black(0)
            , _simClass()
            , _hCrc32(0)
            , _offset(0)
            , _sentenceNumber(0)
            , _lingIntegrity(0)
            , _initWordLen(0)
            , _normWord()
            , _wordType(WORD_TYPE::UNDEFINED)
        {}
		/*virtual*/ ~CWords() throw() {}
	public:
      void setId ( const unsigned long long & id ) { _id = id; }
      void setBlack ( const unsigned char & black ) { _black = black; }
      void setSimClass ( const unsigned short & simClass ) { _simClass.setMask(simClass); }
      void setHCrc ( const unsigned int & hCrc32 ) { _hCrc32 = hCrc32; }
      void setOffset ( const unsigned int & offset ) { _offset = offset; } //!< set offset word from beginning content in SYMBOLS
      void setSentenceNumber( const unsigned int & sentenceNumber ) { _sentenceNumber = sentenceNumber; }
      void setLingIntegrity( const unsigned char & lingIntegrity ) { _lingIntegrity = lingIntegrity; }
      void setInitWordLen( const unsigned int & initWordLen ) { _initWordLen = initWordLen; }
      void setOriginWord( const std::string & originWord ) { _originWord = originWord; }
      void setNormWord( const std::string & normWord ) { _normWord = normWord; }
      void setPosMask( const unsigned short & posMask) {  _posMask = posMask; }
      //void setPosMask ( const unsigned short & posMask) {  _posMask.setMask( posMask ); }
      void setWordType( const WORD_TYPE& wordType) {  _wordType = wordType; }
  public:
      inline const unsigned long long & getId () const { return _id; }
      inline const unsigned char & getBlack () const { return _black; }
      inline const unsigned short getSimClass () const { return _simClass.getMask(); }
      inline const unsigned int & getHCrc () const { return _hCrc32; }
      inline const unsigned int & getOffset () const { return _offset; } //!< get offset word from beginning content in BYTES
      inline const unsigned int & getSentenceNumber () const { return _sentenceNumber; }
      inline const unsigned char & getLingIntegrity () const { return _lingIntegrity; }
      inline const unsigned int & getInitWordLen () const { return _initWordLen; }
      inline const std::string & getOriginWord () const { return _originWord; }
      inline const std::string & getNormWord () const { return _normWord; }
      const unsigned short getPosMask() const { return _posMask; }
      const WORD_TYPE getWordType() const { return _wordType; }
      //const unsigned short getPosMask() const { return _posMask.getMask(); }
	};
}

#endif
