/*
 * LCore.cpp
 *
 *  Created on: Dec 4, 2012
 *      Author: handler
 */
///
/// std headers
///
#include <algorithm>
#include <functional>
using namespace std::placeholders;

///
/// app headers
///
#include "LCore.hpp"
#include "ProcessInfo.hpp"
#include "IcuPlugin.hpp"
#include "BoostPlugin.hpp"
#include "CldPlugin.hpp"
#include "SnowballPlugin.hpp"
#include "TurglemPlugin.hpp"
#include "MecabPlugin.hpp"
#include "TagReducePlugin.hpp"
#include "FakeHCRCPlugin.hpp"


namespace HCE
{


namespace component
{


LCore::LCore()
{
	///
	/// fill vector of plugins
	///
	_vectorPlugins.push_back(TPlugin::Ptr(new IcuPlugin));
	_vectorPlugins.push_back(TPlugin::Ptr(new BoostPlugin));
	_vectorPlugins.push_back(TPlugin::Ptr(new CldPlugin));
	_vectorPlugins.push_back(TPlugin::Ptr(new SnowballPlugin));
	_vectorPlugins.push_back(TPlugin::Ptr(new TurglemPlugin));
    _vectorPlugins.push_back(TPlugin::Ptr(new MecabPlugin));
	_vectorPlugins.push_back(TPlugin::Ptr(new TagReducePlugin));
	_vectorPlugins.push_back(TPlugin::Ptr(new FakeHCRCPlugin));


	///
	/// fill map of the methods
	///
	std::vector<TPlugin::Ptr>::iterator plugin_iterator;
	for( plugin_iterator = _vectorPlugins.begin(); plugin_iterator != _vectorPlugins.end(); ++plugin_iterator )
	{
		std::vector<ProcessInfo>::iterator processInfo_iterator;
		for( processInfo_iterator = (*plugin_iterator)->_exported.begin(); processInfo_iterator != (*plugin_iterator)->_exported.end(); ++processInfo_iterator)
	   	{
			_mapExportedMethods[processInfo_iterator->_methodType].push_back(*processInfo_iterator);
	   	}
	}
}


LCore::~LCore() {}

void LCore::selectMethod(LCoreData& lCData) const
{
	typedef void(TPlugin::*FP)(LCoreData&);
	TPlugin* plugin = NULL;
	FP _fp;
    std::map<unsigned, std::vector<ProcessInfo> >::const_iterator it;
    std::vector<ProcessInfo>::const_iterator itr;

	it= _mapExportedMethods.find( lCData._processInfo._methodType );
	if (it!=_mapExportedMethods.end())
	{
		itr = std::find_if( it->second.begin(),
			it->second.end(),
			std::bind(&ProcessInfo::compare, _1, lCData._processInfo) );

		if (itr!=it->second.end())
		{
			_fp = (FP)( (*itr)._methodAddress );
			plugin = (TPlugin*)( (*itr)._object );
			if ( plugin && _fp )
			{
				(plugin->*_fp)(lCData);
				lCData._processInfo._status = METHOD_FOUND;
			}
		}
		else
		{
			lCData._processInfo._status = PLUGIN_NOT_FOUND;
#ifdef _DEBUG_
			std::cout << "Plugin " << lCData._processInfo._about << " not found" << std::endl;
#endif
		}
	}
	else
	{
		lCData._processInfo._status = METHOD_NOT_FOUND;
#ifdef _DEBUG_
		std::cout << "Method " << lCData._processInfo._about << " not found" << std::endl;
#endif
	}
}


} /* namespace component */


} /* namespace HCE */
