/*
 * Log.hpp
 *
 *  Created on: Dec 13, 2012
 *      Author: handler
 */

#ifndef LOG_HPP
#define LOG_HPP


/// Windows specific headers
#ifdef _WIN32
#include <Poco/EventLogChannel.h>
#endif


/// Linux specific headers
#ifdef __linux__
#include <Poco/SyslogChannel.h>
#endif


/// platform independent headers
#include <Poco/Logger.h>
#include <Poco/Message.h>
#include <Poco/Net/RemoteSyslogChannel.h>
#include <Poco/ConsoleChannel.h>
#include <Poco/NullChannel.h>
#include <Poco/PatternFormatter.h>
#include <Poco/FormattingChannel.h>
#include <Poco/FileChannel.h>
#include <Poco/AsyncChannel.h>
#include <Poco/SplitterChannel.h>
#include <Poco/AutoPtr.h>
#include "Poco/SharedPtr.h"
#include <Poco/NumberFormatter.h>
#include <Poco/LogStream.h>

///
/// std headers
///
#include <string>


namespace HCE
{


#define SET_MSG_FIELDS(_msg) { _msg.setSource(std::string(__FILE__) + ":" + std::string(__LINE__); \
		_msg.setText(std::string(__FUNCTION__)); }


namespace logger
{


enum
{
CONSOLE,
FILE
};


class Log
{
public:
	static void log(Poco::Message&);
	static void log(const std::string& msg)
	{
		Poco::Logger::root().information(msg);
	}
	static void setChannel(const std::string& name, const unsigned int type) {}
	static Poco::Logger& getChannel(const std::string& name, const unsigned int type) { return Poco::Logger::get("ch1"); }
public:
	static Poco::LogStream stream;
	Log();
	virtual ~Log();
private:
	std::vector<std::string> _v;
private:	/// channels
	/// null channel
	Poco::AutoPtr<Poco::NullChannel> _null;

	/// console channel
	Poco::AutoPtr<Poco::FormattingChannel> _console;

	/// file channel
	Poco::AutoPtr<Poco::FormattingChannel> _file;

	/// pool of channels
	Poco::AutoPtr<Poco::SplitterChannel> _pool;

	/// Linux specific channels
	#ifdef __linux__
	Poco::AutoPtr<Poco::SyslogChannel> _sysLog;
	#endif

	/// Windows specific channels
	#ifdef _WIN32
	Poco::AutoPtr<Poco::EventLogChannel> _eventLog;
	#endif

	/// Remote sys log over UDP
	Poco::AutoPtr<Poco::Net::RemoteSyslogChannel> _remoteSysLog;

	/// Async console channel
	Poco::AutoPtr<Poco::AsyncChannel> _async;
};


} /* namespace logger */


} /* namespace HCE */


#endif /* LOG_H_ */
