/**
 * header file MorphChangeGradBitset.hpp
 *
 * class representing morphology bit set structures
 */


#ifndef _MORPH_CHANGE_GRAD_BITSET_HPP
#define _MORPH_CHANGE_GRAD_BITSET_HPP

#include <string>

#include "BitsetBase.hpp"

 namespace HCE
 {


  enum MCG_Consistency
  {
    // 0-15 bit flags in MorphChangeGrad
    // 15 - topic bit
    MCG_TOPIC_BIT_NUMBER = 15,
    // 14,13 - case regulators:
    MCG_START_UPPERCASE = 14,
    MCG_OTHER_UPPERCASE = 13,
    // 8-12 - synonym id
    MCG_SYNONYM_BIT_SIZE = 5,
    MCG_SYNONYM_LOWEST_BIT_NUMBER = 8,
    // 7-0 - grammar category id
    MCG_GRAMMAR_BIT_SIZE = 8,
    MCG_GRAMMAR_LOWEST_BIT_NUMBER = 0, // MUST NOT be changed
    // MCG constants for the whole value
    MCG_OBSCENE_WORD_VALUE = 1 << MCG_TOPIC_BIT_NUMBER,
  };


  static const unsigned long MCG_NUM = 16;


	  template<size_t N>
	  class MorphChangeGradBitset : public BitsetBase<N>
	  {
	  private:
		  std::bitset<N> _synonymIdMask;
		  std::bitset<N> _grammarCategoryIdMask;
	  public:
		  MorphChangeGradBitset<N>() :
				/// Don't do that!!! 
				_synonymIdMask(std::string("0001111100000000")),
				_grammarCategoryIdMask(std::string("0000000011111111")) {}
	    ~MorphChangeGradBitset<N>() {}
	  public:
	    inline const bool isTopicBitSet() const { return BitsetBase<N>::_bitset.test( MCG_TOPIC_BIT_NUMBER ); }
	    inline const bool isStartUpperCaseBitSet() const { return BitsetBase<N>::_bitset.test( MCG_START_UPPERCASE ); }
	    inline const bool isOtherUpperCaseBitSet() const { return BitsetBase<N>::_bitset.test( MCG_OTHER_UPPERCASE ); }
	    inline const unsigned long getSynonimIdBitSet() const {	return ((BitsetBase<N>::_bitset&_synonymIdMask)>>MCG_SYNONYM_LOWEST_BIT_NUMBER).to_ulong(); }
	    inline const unsigned long getGrammarCategoryIdBitSet() const  { return ((BitsetBase<N>::_bitset&_grammarCategoryIdMask)>>MCG_GRAMMAR_LOWEST_BIT_NUMBER).to_ulong(); }
	  };


 } /// namespace HCE


#endif
