/**
 * header file POSMaskBitset.hpp
 *
 * class representing Part-Of-Speech bit set structures
 */


#ifndef _POS_MASK_BITSET_HPP
#define _POS_MASK_BITSET_HPP


#include "BitsetBase.hpp"


namespace HCE
{


///
/// Part-Of-Speech constants
///
enum POS_enum
{
	POS_NOUN  = 0,
	POS_ADJ_FULL = 1,
	POS_VERB = 2,
	POS_VBE = 3,
	POS_MOD = 4,
	POS_NUMERAL = 5,
	POS_CONJ = 6,
	POS_INTERJ = 7,
	POS_PREP = 8,
	POS_PARTICLE = 9,
	POS_ART = 10,
	POS_ADV = 11,
	POS_PN = 12,
	POS_ORDNUM = 13,
	POS_PRON = 14,
	POS_POSS = 15,
	//POS_PN_ADJ = 16
	/// ...
};


  template<size_t N>
  class POSMaskBitset : public BitsetBase<N>
  {
  public:
    POSMaskBitset<N>() {}
    ~POSMaskBitset<N>() {}
  public:
    inline const bool isNounBitSet() const { return BitsetBase<N>::_bitset.test( POS_NOUN ); }
    inline const bool isVerbBitSet() const { return BitsetBase<N>::_bitset.test( POS_VERB ); }
    inline const bool isParticleBitSet() const { return BitsetBase<N>::_bitset.test( POS_PARTICLE ); }
    inline const bool isInterjectionBitSet() const { return BitsetBase<N>::_bitset.test( POS_INTERJ ); }
    inline const bool isPronounBitSet() const { return BitsetBase<N>::_bitset.test( POS_PRON ); }
    inline const bool isPropositionBitSet() const { return BitsetBase<N>::_bitset.test( POS_PREP ); }
    inline const bool isAdverbBitSet() const { return BitsetBase<N>::_bitset.test( POS_ADV ); }
    inline const bool isConjunctionBitSet() const { return BitsetBase<N>::_bitset.test( POS_CONJ ); }
    inline const bool isAdjectivBitSet() const { return BitsetBase<N>::_bitset.test( POS_ADJ_FULL ); }
    inline const bool isNumeralBitSet() const { return BitsetBase<N>::_bitset.test( POS_NUMERAL ); }
    inline const bool isVbeBitSet() const { return BitsetBase<N>::_bitset.test( POS_VBE ); }
    inline const bool isModBitSet() const { return BitsetBase<N>::_bitset.test( POS_MOD ); }
    inline const bool isArtBitSet() const { return BitsetBase<N>::_bitset.test( POS_ART ); }
    inline const bool isPnBitSet() const { return BitsetBase<N>::_bitset.test( POS_PN ); }
    inline const bool isOrdnumBitSet() const { return BitsetBase<N>::_bitset.test( POS_ORDNUM ); }
    inline const bool isPossBitSet() const { return BitsetBase<N>::_bitset.test( POS_POSS ); }
  }; /// class POSMaskBitset


} /// namespace HCE


#endif
