/**
 *
 */


#ifndef PROCESS_INFO_HPP
#define PROCESS_INFO_HPP

#include <vector>
#include <cld/compact_lang_det.h>

#include "Defs.hpp"

namespace HCE
{

namespace component
{

class TPlugin;
class LCoreData;


///
/// method type
///
enum MethodType
{
	MT_NONE,
	SPLIT,
	LDET,
	NORM,
	POS,
	TAGS,
    HCRC,
    WTYPE
};
static SSTRING MTdescription[] =
{
	"Unknown method",
	"Split method",
	"Language detect method",
	"Normalize method",
	"pos method",
	"Tag reduce method",
	"HCRC method"
};

///
/// plugin type
///
enum PluginType
{
	PT_NONE,
	DEFAULT,
	SNOWBALL_NORM,
	ICU_BOUNDARIES,
	BOOST_SPLIT,
	BOOST_TOKENIZER,
	POCO_TOKENIZER,
	CHROMIUM_LDET,
	TAG_REDUCE,
	MECAB_NORM,
	TURGLEM_POS,
    FAKE_HCRC,
    MECAB_WTYPE
    , MECAB_TOKENIZE
};

static SSTRING PTdescription[] =
{
	"Unknown plugin",
	"Default plugin",
	"Snowball normalization plugin",
	"Icu boundaries plugin",
	"Boost split plugin",
	"Boost tokenizer plugin",
	"Poco tokinizer plugin",
	"Chromium language detector plugin",
	"Tag reduce plugin",
	"Mecab norm plugin",
	"Turglem pos plugin",
	"HCRC plugin"
    , "Mecab word type detector plugin"
    , "Mecab tokenizer plugin"
};

//enum MethodQuality { MQ_NONE, COMMON_QUALITY, LOW_QUALITY, BOOST_SPLIT, POCO_TOKENIZER, ICU_BOUNDARIES };


//enum LanguageMask { LM_NONE, LM_ALL, LM_JAPANESE };

typedef unsigned int LanguageMask;


///
/// Languages constants
///
enum Language_enum
{
  ENGLISH = 0,
  JAPANESE,
  RUSSIAN,
  UKRAINIAN,
  BYELORUSSIAN = UKRAINIAN,
  GERMAN,
  FRENCH,
  THAI
};


enum LanguageMask_enum
{
  LM_NONE = 0,

  LM_ENGLISH = 1 << ENGLISH,
  LM_JAPANESE = 1 << JAPANESE,
  LM_RUSSIAN = 1 << RUSSIAN,
  LM_UKRAINIAN = 1 << UKRAINIAN,
  LM_BYELORUSSIAN = 1 << BYELORUSSIAN,
  LM_GERMAN = 1 << GERMAN,
  LM_FRENCH = 1 << FRENCH,
  LM_THAI = 1 << THAI,

  LM_ALL = (LanguageMask) (-1)
};

/*
///
/// Part-Of-Speech constants
///
enum POS_enum
{
	POS_NOUN  = 0,
	POS_ADJ_FULL = 1,
	POS_VERB = 2,
	POS_VBE = 3,
	POS_MOD = 4,
	POS_NUMERAL = 5,
	POS_CONJ = 6,
	POS_INTERJ = 7,
	POS_PREP = 8,
	POS_PARTICLE = 9,
	POS_ART = 10,
	POS_ADV = 11,
	POS_PN = 12,
	POS_ORDNUM = 13,
	POS_PRON = 14,
	POS_POSS = 15,
	POS_PN_ADJ = 16
	/// ...
};


enum POS_Mask
{
	PM_NOUN = 1 << POS_NOUN,
	PM_ADJ_FULL = 1 << POS_ADJ_FULL,
	PM_VERB = 1 << POS_VERB,
	PM_VBE = 1 << POS_VBE,
	PM_MOD = 1 << POS_MOD,
	PM_NUMERAL = 1 << POS_NUMERAL,
	PM_CONJ = 1 << POS_CONJ,
	PM_INTERJ = 1 << POS_INTERJ,
	PM_PREP = 1 << POS_PREP,
	PM_PARTICLE = 1 << POS_PARTICLE,
	PM_ART = 1 << POS_ART,
	PM_ADV = 1 << POS_ADV,
	PM_PN = 1 << POS_PN,
	PM_ORDNUM = 1 << POS_ORDNUM,
	PM_PRON = 1 << POS_PRON,
	PM_POSS = 1 << POS_POSS,
	PM_PN_ADJ = 1 << POS_PN_ADJ
};
*/

static SSTRING LMdescription[] =
{
	"Unknown language",
	"All language"
};

enum Status { METHOD_FOUND, METHOD_NOT_FOUND, PLUGIN_NOT_FOUND };
static SSTRING Sdescription[] =
{
	"Method found",
	"Method not found",
	"Plugin not found"
};


#define SET_PROCESS_INFO_L(processInfo, _mType, _pType, _lMask) { processInfo._methodType = _mType; \
																		processInfo._pluginType = _pType; \
																		processInfo._languageMask = _lMask; \
                                                                        processInfo._about = HCE::component::PTdescription[_pType]; }


#define SET_PROCESS_INFO_P(processInfo, _mType, _pType, _lMask, /*_mQuality,*/ _mAddr, _mObj) { processInfo._methodType = _mType; \
																						processInfo._pluginType = _pType; \
																						processInfo._languageMask = _lMask; \
																						/*processInfo._methodQuality = _mQuality;*/ \
																						processInfo._methodAddress = _mAddr; \
																						processInfo._object = _mObj; }


struct ProcessInfo
{
	MethodType _methodType;
	PluginType _pluginType;
	LanguageMask _languageMask;
	//MethodQuality _methodQuality;
	typedef void(TPlugin::*FP)(LCoreData&);
	FP _methodAddress;
    TPlugin* _object;
    Status _status;
    SSTRING _about;
    //static const std::string PTdescription[];
    ProcessInfo()
    {
    	_methodType = MT_NONE;
    	_pluginType = PT_NONE;
    	_languageMask = LM_NONE; //UNKNOWN_LANGUAGE
    	_methodAddress = 0;
    	_object = 0;
    	_status = METHOD_NOT_FOUND;
    	_about = PTdescription[0];
    }
    bool compare( const ProcessInfo& processInfo ) const
    {
        return (_methodType == processInfo._methodType && \
                    _pluginType == processInfo._pluginType && \
                    /*_methodQuality == processInfo._methodQuality &&*/ \
                    _languageMask == processInfo._languageMask);
    }
};

} /* namespace component */

} /* namespace HCE */


#endif /* PROCESS_INFO_H_ */
