#include "MecabWordTypeTest.hpp"


namespace refine_test {

    void MecabWordTypeTest::setUp() {
        //!<
        //!< set process type: "word type detect"
        //!<
        _lcData._processInfo._pluginType = HCE::component::MECAB_WTYPE;
        //!<
        //!< fill data with one token
        //!<
        _lcData._mapOffsets = {{0,3}, {4,1}, {6,6}, {16,3}, {20,3}};
        _lcData._mapTokens = {{0,"123"}, {4,"?"}, {6,"impostors"}, {16,"。"}, {20,"~@^"}};
    }

    void MecabWordTypeTest::tearDown() {
    }

    void MecabWordTypeTest::testWordTypesCount() {
        //!<
        //!< run processing
        //!<
        _mecabPlugin.process(_lcData);
        //!<
        //!< check processing results
        //!<
        CPPUNIT_ASSERT(_lcData._mapWordType.size()==5);
    }

    void MecabWordTypeTest::testWordTypeNumber() {
        //!<
        //!< run processing
        //!<
        _mecabPlugin.process(_lcData);
        //!<
        //!< check processing results
        //!<
        CPPUNIT_ASSERT(_lcData._mapWordType[0]==HCE::WORD_TYPE::NUMBER);
    }


    void MecabWordTypeTest::testWordTypeDelimiter() {
        //!<
        //!< run processing
        //!<
        _mecabPlugin.process(_lcData);
        //!<
        //!< check processing results
        //!<
        CPPUNIT_ASSERT(_lcData._mapWordType[16]==HCE::WORD_TYPE::DELIMITER);
    }


    void MecabWordTypeTest::testWordTypePunctuation() {
        //!<
        //!< run processing
        //!<
        _mecabPlugin.process(_lcData);
        //!<
        //!< check processing results
        //!<
        CPPUNIT_ASSERT(_lcData._mapWordType[4]==HCE::WORD_TYPE::PUNCTUATION);
    }


    void MecabWordTypeTest::testWordTypeWord() {
        //!<
        //!< run processing
        //!<
        _mecabPlugin.process(_lcData);
        //!<
        //!< check processing results
        //!<
        CPPUNIT_ASSERT(_lcData._mapWordType[6]==HCE::WORD_TYPE::WORD);
    }

    void MecabWordTypeTest::testWordTypeUndefined() {
        //!<
        //!< run processing
        //!<
        _mecabPlugin.process(_lcData);
        //!<
        //!< check processing results
        //!<
        CPPUNIT_ASSERT(_lcData._mapWordType[15]==HCE::WORD_TYPE::UNDEFINED);

    }

    void MecabWordTypeTest::testTokensCount() {
        _lcData._content = "123one?";
        _lcData._mapOffsets.clear();
        _lcData._mapTokens.clear();
        _lcData._processInfo._pluginType = HCE::component::MECAB_TOKENIZE;
        //!<
        //!< run processing
        //!<
        _mecabPlugin.process(_lcData);
        //!<
        //!< check processing results
        //!<
        CPPUNIT_ASSERT(_lcData._mapTokens.size()==3);

    }

}
