//#include <ProcessInfo.hpp>
#include <Refine.hpp>
#include <CWords.hpp>
#include <OutDataRefine.hpp>

#include <string>
#include <vector>
#include <iostream>
#include <streambuf>
#include <Poco/String.h>
#include <Poco/StringTokenizer.h>

/**
 * incoming content: "happy New,Year"
 *
 * if no custom delimiters are defined
 * tokens: <happy> <New> <Year>
 *
 * if custom delimiters are defined
 * tokens: <happy Ne> <,Year>
 */

int main()
{
    std::string content(std::istreambuf_iterator<char>(std::cin), std::istreambuf_iterator<char>());
    HCE::component::Refine refine(HCE::CT_REFINE);
    Poco::SharedPtr<HCE::DataBase> inData(new HCE::InDataRefine(HCE::CT_REFINE, 0, 0, content));

    //!<
    //!< Definition custom delimiters
    //!<
    std::vector<std::string> custom_delimiters = {"w"};
    inData.cast<HCE::InDataRefine>()->setDelimiters(custom_delimiters);

    //!<
    //!< Prepare Process Info Table
    //!<
    std::vector<HCE::component::ProcessInfo> processInfoTable;
    HCE::component::ProcessInfo processInfo;
    SET_PROCESS_INFO_L(processInfo, HCE::component::SPLIT, HCE::component::BOOST_TOKENIZER, HCE::component::LM_ALL)
    processInfoTable.push_back(processInfo);
    inData.cast<HCE::InDataRefine>()->setProcessInfoTable(processInfoTable);

    Poco::SharedPtr<HCE::DataBase> outData = refine.process(inData);
    std::vector<HCE::CWords> cWords = outData.cast<HCE::OutDataRefine>()->getCWords();
    std::cout << "Content: \"" << content << "\" splitted by custom delimiters: \"" << custom_delimiters[0] << "\"" << std::endl;
    for_each( cWords.begin(), cWords.end(), [](HCE::CWords val) { std::cout << val.getOriginWord() << std::endl;} );

    //!<
    //!< Reset to default delimiters
    //!<
    std::vector<std::string> default_delimiters;
    inData.cast<HCE::InDataRefine>()->setDelimiters(default_delimiters);
    outData = refine.process(inData);
    cWords = outData.cast<HCE::OutDataRefine>()->getCWords();
    std::cout << "Content: \"" << content << "\" splitted by defaultdelimiters:" << std::endl;
    for_each( cWords.begin(), cWords.end(), [](HCE::CWords val) { std::cout << val.getOriginWord() << std::endl;} );
    return EXIT_SUCCESS;
}

