/**
 * @file sample.cpp
 *
 * Example Application main module. Example of how use JsonMessageCover class.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#include <sstream>
#include <iostream>

#include "JsonMessageCover.hpp"

void printTitle(const std::string& title)
{
	std::cout << std::string(title.length(), '=') << std::endl << title << std::endl << std::string(title.length(), '=') << std::endl;
} 
//-----------------------------------------------------------------------------
void sampleSerializeJsonMessageCover(void)
{
	printTitle("Sample use serialize of JsonMessageCover");

	const std::string data = "Hello world!";
	HCE::types::MessageType type = HCE::types::MessageType::mtSphinx;
	unsigned int ttl = 234;	

	HCE::JsonMessageCover jsonMessageCover;
	jsonMessageCover.setType(type);
	jsonMessageCover.setData(data);
	jsonMessageCover.setTTL(ttl);

	std::string json;
	std::cout << "serialize => " << std::boolalpha << jsonMessageCover.serialize(json) << std::endl;
	std::cout << "isError => " << std::boolalpha << jsonMessageCover.isError() << std::endl;
	if (jsonMessageCover.isError())
		std::cout << "ErrorCode: " << jsonMessageCover.getErrorCode() << "\tErrorMsg: " << jsonMessageCover.getErrorMsg() << std::endl;

	std::cout << "JSON: " << json << std::endl;
}
//-----------------------------------------------------------------------------
void sampleUnserializeJsonMessageCover(void)
{
	printTitle("Sample use unserialize of JsonMessageCover");

	std::string json;

#if JSON_USE_BASE64 == 1
	std::cout << "JSON_USE_BASE64 == 1" << std::endl;
	json = "{\"data\":\"SGVsbG8gd29ybGQh\",\"ttl\":234,\"type\":1}";
#else
	std::cout << "JSON_USE_BASE64 != 1" << std::endl;
	json = "{\"data\":\"Hello world!\",\"ttl\":234,\"type\":1}";
#endif

	HCE::JsonMessageCover jsonMessageCover;
	std::cout << "unserialize => " << std::boolalpha << jsonMessageCover.unserialize(json) << std::endl;
	std::cout << "isError => " << std::boolalpha << jsonMessageCover.isError() << std::endl;
	if (jsonMessageCover.isError())
		std::cout << "ErrorCode: " << jsonMessageCover.getErrorCode() << "\tErrorMsg: " << jsonMessageCover.getErrorMsg() << std::endl;

	std::cout << "Type: " << static_cast<unsigned int>(jsonMessageCover.getType()) << std::endl;
	std::cout << "Data: " << jsonMessageCover.getData() << std::endl;
	std::cout << "TTL: " << jsonMessageCover.getTTL() << std::endl;
}
//-----------------------------------------------------------------------------
void sampleUseOutputStreamForJsonMessageCover(void)
{
	printTitle("Sample use output stream for JsonMessageCover");

	const std::string data = "Hello world!";
	HCE::types::MessageType type = HCE::types::MessageType::mtSphinx;
	unsigned int ttl = 234;	

	HCE::JsonMessageCover jsonMessageCover;
	jsonMessageCover.setType(type);
	jsonMessageCover.setData(data);
	jsonMessageCover.setTTL(ttl);

	std::stringstream json;
	json << jsonMessageCover;

	std::cout << "isError => " << std::boolalpha << jsonMessageCover.isError() << std::endl;
	if (jsonMessageCover.isError())
		std::cout << "ErrorCode: " << jsonMessageCover.getErrorCode() << "\tErrorMsg: " << jsonMessageCover.getErrorMsg() << std::endl;

	std::cout << "JSON: " << json.str() << std::endl;
}
//-----------------------------------------------------------------------------
void sampleUseInputStreamForJsonMessageCover(void)
{
	printTitle("Sample use input stream for JsonMessageCover");

	std::stringstream json;

#if JSON_USE_BASE64 == 1
	std::cout << "JSON_USE_BASE64 == 1" << std::endl;
	json << "{\"data\":\"SGVsbG8gd29ybGQh\",\"ttl\":234,\"type\":1}";
#else
	std::cout << "JSON_USE_BASE64 != 1" << std::endl;
	json << "{\"data\":\"Hello world!\",\"ttl\":234,\"type\":1}";
#endif

	HCE::JsonMessageCover jsonMessageCover;
	json >> jsonMessageCover;

	std::cout << "isError => " << std::boolalpha << jsonMessageCover.isError() << std::endl;
	if (jsonMessageCover.isError())
		std::cout << "ErrorCode: " << jsonMessageCover.getErrorCode() << "\tErrorMsg: " << jsonMessageCover.getErrorMsg() << std::endl;

	std::cout << "Type: " << static_cast<unsigned int>(jsonMessageCover.getType()) << std::endl;
	std::cout << "Data: " << jsonMessageCover.getData() << std::endl;
	std::cout << "TTL: " << jsonMessageCover.getTTL() << std::endl;
}
//-----------------------------------------------------------------------------
void sampleErrorMessageJsonMessageCover(void)
{
	printTitle("Sample error message for JsonMessageCover");

	std::stringstream json;

#if JSON_USE_BASE64 == 1
	std::cout << "JSON_USE_BASE64 == 1" << std::endl;
	json << "{\"data\":\"SGVsbG8gd29ybGQh\",\"ttl\":234,\"typ";
#else
	std::cout << "JSON_USE_BASE64 != 1" << std::endl;
	json << "{\"data\":\"Hello world!\",\"ttl\":234,\"typ";
#endif

	HCE::JsonMessageCover jsonMessageCover;
	json >> jsonMessageCover;

	std::cout << "isError => " << std::boolalpha << jsonMessageCover.isError() << std::endl;
	if (jsonMessageCover.isError())
		std::cout << "ErrorCode: " << jsonMessageCover.getErrorCode() << "\tErrorMsg: " << jsonMessageCover.getErrorMsg() << std::endl;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
int main(void)
{
	sampleSerializeJsonMessageCover();
	sampleUnserializeJsonMessageCover();	
	sampleUseOutputStreamForJsonMessageCover();
	sampleUseInputStreamForJsonMessageCover();
	sampleErrorMessageJsonMessageCover();

	return 0;
}

