#include <Poco/JSON/Object.h>
#include <Poco/JSON/Array.h>
#include <Poco/JSON/JSON.h>
#include <Poco/JSON/Stringifier.h>
#include <Poco/JSON/Parser.h>
#include <Poco/JSON/JSONException.h>
#include <Poco/Dynamic/Var.h>

#include "EncodeDecodeBase64.hpp"
#include "JsonMessageCover.hpp"

namespace HCE
{
//-----------------------------------------------------------------------------
JsonMessageCover::JsonMessageCover(const std::string& json)
:inherited(), messageType(types::MessageType::mtSphinx), messageBody(""),
 messageTTL(jsonMessageCoverConst::defaultTTLValue)
{
  if (!json.empty())
    unserialize(json);
}
//-----------------------------------------------------------------------------
JsonMessageCover::JsonMessageCover(HCE::types::MessageType messageType_, const std::string& messageBody_, unsigned int ttl_)
:inherited(), messageType(messageType_), messageBody(messageBody_),
 messageTTL(ttl_)
{
}
//-----------------------------------------------------------------------------
JsonMessageCover::JsonMessageCover(unsigned int messageType_, const std::string& messageBody_, unsigned int ttl_)
:inherited(), messageType(static_cast<types::MessageType>(messageType_)), messageBody(messageBody_),
 messageTTL(ttl_)
{
}
//-----------------------------------------------------------------------------
bool JsonMessageCover::serialize(std::string& json)
{
  _isError = false;
  errorMsg.clear();
  errorCode = NO_ERROR;
  try
  {
    Poco::JSON::Object::Ptr obj = new Poco::JSON::Object();
    obj->set(jsonMessageCoverConst::messageType, static_cast<unsigned int>(messageType));
    obj->set(jsonMessageCoverConst::messageBody, HCE::encodeBase64(messageBody));
    obj->set(jsonMessageCoverConst::messageTTL, messageTTL);

    std::stringstream ostr;
    Poco::JSON::Stringifier::stringify(obj, ostr);
    json = ostr.str();
  }
  catch(std::exception& e)
  {
    errorMsg = e.what();
    errorCode = ERROR_SERIALIZE;
    _isError = true;
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
bool JsonMessageCover::unserialize(const std::string& json)
{
  _isError = false;
  errorMsg.clear();
  errorCode = NO_ERROR;
  messageType = types::MessageType::mtSphinx;
  messageTTL = jsonMessageCoverConst::defaultTTLValue;

  try
  {
    Poco::JSON::Parser parser;
    Poco::Dynamic::Var res = parser.parse(json);
    Poco::JSON::Object::Ptr obj = res.extract<Poco::JSON::Object::Ptr>();

    Poco::Dynamic::Var tmp = obj->get(jsonMessageCoverConst::messageType);
    messageType = static_cast<types::MessageType>(convertVarToNumeric<unsigned long>(tmp, static_cast<unsigned long>(messageType)));

    tmp = obj->get(jsonMessageCoverConst::messageBody);
    if (tmp.isString())
      messageBody = HCE::decodeBase64(tmp.convert<std::string>());

    if (messageBody.empty())
      throw Poco::Exception("Invalid body. Empty message.");

    tmp = obj->get(jsonMessageCoverConst::messageTTL);
    messageTTL = convertVarToNumeric<unsigned long>(tmp, messageTTL);
  }
  catch(Poco::JSON::JSONException& e)
  {
    errorMsg = e.message();
    errorCode = ERROR_UNSERIALIZE;
    _isError = true;
  }
  catch(Poco::Exception& e)
  {
    errorMsg = e.message();
    errorCode = ERROR_UNSERIALIZE;
    _isError = true;
  }
  catch(std::exception& e)
  {
    errorMsg = e.what();
    errorCode = ERROR_UNSERIALIZE;
    _isError = true;
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
std::istream& operator>>(std::istream& is, JsonMessageCover& jsonHandler)
{
  std::string json;
  is.seekg(0, std::ios::end);
  json.resize(is.tellg());
  is.seekg(0, std::ios::beg);
  is.read(const_cast<char*>(json.c_str()), json.size());
  jsonHandler.unserialize(json);
  return is;
}
//-----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const JsonMessageCover& jsonHandler)
{
  std::string json;
  const_cast<HCE::JsonMessageCover&>(jsonHandler).serialize(json);
  return os << json;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE
