/**
 * @file JsonMessageCover.hpp
 *
 * Implementation class JsonMessageCover describe object-cover json messages in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef JSON_MESSAGE_COVER_HPP
#define JSON_MESSAGE_COVER_HPP

#include "JsonSerializable.hpp"
#include "MessageType.hpp"

namespace HCE
{
//-----------------------------------------------------------------------------
namespace jsonMessageCoverConst
{
  const std::string messageType = "type";
  const std::string messageBody = "data";
  const std::string messageTTL = "ttl";
  const unsigned int defaultTTLValue = 1;
}
//-----------------------------------------------------------------------------
class JsonMessageCover : public IJsonSerializable
{
public:
  enum ErrorCode
  {
    NO_ERROR=0,
    ERROR_SERIALIZE=1,
    ERROR_UNSERIALIZE=2
  };
public:
  explicit JsonMessageCover(const std::string& json="");

  JsonMessageCover(types::MessageType messageType_,
      const std::string& messageBody_,
      unsigned int ttl_=HCE::jsonMessageCoverConst::defaultTTLValue);

  JsonMessageCover(unsigned int messageType_,
      const std::string& messageBody_,
      unsigned int ttl_=HCE::jsonMessageCoverConst::defaultTTLValue);

  virtual ~JsonMessageCover(void) {};

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void setType(unsigned int messageType_) {messageType=static_cast<types::MessageType>(messageType_);}
  void setType(types::MessageType messageType_) {messageType=messageType_;}
  types::MessageType getType(void) const {return messageType;}

  void setData(const std::string& messageBody_) {messageBody=messageBody_;}
  std::string getData(void) const {return messageBody;}

  void setTTL(unsigned int ttl) {messageTTL=ttl;}
  unsigned int getTTL(void) const {return messageTTL;}
private:
  types::MessageType messageType;
  std::string messageBody;
  unsigned int messageTTL;

private:
  friend std::istream& operator>>(std::istream& is, HCE::JsonMessageCover& jsonHandler);
  friend std::ostream& operator<<(std::ostream& os, const HCE::JsonMessageCover& jsonHandler);
  typedef HCE::IJsonSerializable inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE

#endif // JSON_MESSAGE_COVER_HPP
