/**
 * @file JsonSerializable.hpp
 *
 * Implementation basic interface for describe inheritor methods for all types object json messages.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef JSON_SERIALIZABLE_HPP
#define JSON_SERIALIZABLE_HPP

#include <Poco/Dynamic/Var.h>
#include <Poco/NumberFormatter.h>
#include <iostream>
#include <sstream>
#include <iomanip>

namespace HCE
{
//-----------------------------------------------------------------------------
class IJsonSerializable
{
public:
  enum
  {
    NO_ERROR=0
  };
protected:
  template <class T = unsigned long long>
  T convertVarToNumeric(Poco::Dynamic::Var& var, T defaultValue)
  {
    T result=defaultValue;
    try
    {
      if (!var.isEmpty())
      {
        if (var.isString())
          result = static_cast<T>(std::stoull(var.convert<std::string>()));
        else
        {
          if (var.isNumeric())
          {
            if (var.isInteger())
            {
              if (var.isSigned())
                result = static_cast<T>(var.convert<Poco::Int64>());
              else
                result = static_cast<T>(var.convert<Poco::UInt64>());
            }
            else
              result = static_cast<T>(var.convert<double>());
          }
        }
      }
    }
    catch(...)
    {
      result = defaultValue;
    }
    return result;
  }
public:
  IJsonSerializable(void) : errorMsg(""), errorCode(0), _isError(false) {}
  virtual ~IJsonSerializable(void) {}

  virtual bool serialize(std::string& json)=0;
  virtual bool unserialize(const std::string& json)=0;

  void setErrorMsg(const std::string& errorMsg_) {errorMsg=errorMsg_;}
  std::string getErrorMsg(void) const {return errorMsg;}

  void setErrorCode(unsigned int errorCode_) {errorCode=errorCode_;}
  unsigned int getErrorCode(void) const {return errorCode;}

  void setIsError(bool isError_) {_isError=isError_;}
  bool isError(void) const {return _isError;}
  void resetError(void)
  {
    errorMsg.clear();
    errorCode = NO_ERROR;
    _isError = false;
  }
protected:
  std::string errorMsg;
  unsigned int errorCode;
  bool _isError;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
template <typename T = std::string>
class TrimFormatter
{
  static const size_t PRECISION=2;
public:
  T operator()(double value, size_t precision=PRECISION)
  {
    T t;
    std::istringstream(Poco::NumberFormatter::format(value, precision)) >> t;
    return t;
  }
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace HCE

#endif // JSON_SERIALIZABLE_HPP
