#include "SphinxConfigCreator.hpp"

namespace HCE {
namespace sphinx {

SphinxConfigCreator::SphinxConfigCreator(void):Poco::Util::PropertyFileConfiguration(),data(){}

SphinxConfigCreator::SphinxConfigCreator (const std::string& path):
		Poco::Util::PropertyFileConfiguration(path),data() { this->parse(); }

SphinxConfigCreator::~SphinxConfigCreator(void) throw () {}

void SphinxConfigCreator::setPropertyFileConfiguration (const std::string& path) {
	this->load(path);
	this->parse();
}

void SphinxConfigCreator::parse (void) {
	this->data.clear();
	for (auto it = this->begin(); it != this->end(); ++it) {
		Poco::StringTokenizer record (it->first,".");
		std::string section ( *record.begin() );
		std::string key ( *( record.end()-1) );
		std::string value (it->second);
		data[section].push_back(std::pair<_key,_value>(key,value));
	}
}

std::string SphinxConfigCreator::getSphinxFileConfiguration (void) {
	std::string sphinxConfig("");
	for (std::map<_section,std::vector<std::pair<_key,_value>>>::const_iterator iteratorOnMap = this->data.begin(); iteratorOnMap != this->data.end(); ++iteratorOnMap) {
		sphinxConfig += (iteratorOnMap->first + "\n{\n");
		for (std::vector<std::pair<_key,_value> >::const_iterator iteratorOnVector = iteratorOnMap->second.begin(); iteratorOnVector != iteratorOnMap->second.end(); ++iteratorOnVector) {
			sphinxConfig += (iteratorOnVector->first + " = " + iteratorOnVector->second + "\n");
		}
		sphinxConfig += "}\n";
	}
	return sphinxConfig;
}

void SphinxConfigCreator::cloneSphinxConfigSection (const std::string& oldName, const std::string& newName) {
  auto pointerToTheDataWithOldName = this->data.find(oldName);
  if (pointerToTheDataWithOldName != data.end()) {
    auto dataWhereKeyIsOldName  (pointerToTheDataWithOldName->second);
    this->data[newName] = dataWhereKeyIsOldName;

    for (Poco::Util::MapConfiguration::StringMap::const_iterator it=this->begin();it!=this->end();++it) {
      Poco::StringTokenizer record (it->first,".");
      std::string section ( *record.begin() );
      std::string key ( *( record.end()-1) );
      std::string value (it->second);

      if (oldName==section) {
        setString(newName+"."+key, value);
      }
    }
  }
}

void SphinxConfigCreator::renameSphinxConfigSection (const std::string& oldName, const std::string& newName) {
	auto pointerToTheDataWithOldName = this->data.find(oldName);
	if (pointerToTheDataWithOldName != data.end()) {
		auto dataWhereKeyIsOldName  (pointerToTheDataWithOldName->second);
		this->data[newName] = dataWhereKeyIsOldName;
	}
	pointerToTheDataWithOldName = this->data.find(oldName);
	if (pointerToTheDataWithOldName != data.end()) {
		this->data.erase(pointerToTheDataWithOldName);
	}
}

void SphinxConfigCreator::deleteSphinxConfigSection(const std::string& sectionName) {
	auto pointerToTheSection = this->data.find(sectionName);
	if (pointerToTheSection != data.end()) {
		this->data.erase(pointerToTheSection);
	}
}

}
}

