/*
 * @file SphinxConfigCreator.hpp
 *
 * Creates Sphinx config file from Poco properties
 *
 * @author Eduard <eduard.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINXCONFIGCREATOR_HPP
#define SPHINXCONFIGCREATOR_HPP

#include <Poco/AutoPtr.h>
#include <Poco/Util/PropertyFileConfiguration.h>
#include <Poco/FileStream.h>
#include <iostream>
#include <Poco/StringTokenizer.h>
#include <sstream>
#include <fstream>
#include <map>
#include <string>

namespace HCE {
namespace sphinx {
    class SphinxConfigCreator : public Poco::Util::PropertyFileConfiguration {
        private:
    	       typedef std::string _section;
    	       typedef std::string _key;
    	       typedef std::string _value;
    	private:
    	       std::map<_section,std::vector<std::pair<_key,_value>>> data;
    	public:
    	       friend std::ostream& operator << (std::ostream& stream,SphinxConfigCreator& objSphinxConfig) {
    	    	   stream << objSphinxConfig.getSphinxFileConfiguration();
    	    	   return stream;
    	       }
    	public:
    	       SphinxConfigCreator (void);
    	       SphinxConfigCreator (const std::string& path);
    	       SphinxConfigCreator (const Poco::Util::PropertyFileConfiguration& object);
    	       ~SphinxConfigCreator (void) throw ();
    	public:
    	       //inserts data from propertyFileConfiguration to the map `data`
    	       void parse (void);
    	public:
    	       void setPropertyFileConfiguration (const std::string& path);
    	       std::string getSphinxFileConfiguration (void);
    	public:
    	       void cloneSphinxConfigSection (const std::string& oldName, const std::string& newName);
    	       void renameSphinxConfigSection (const std::string& oldName, const std::string& newName);
    	       void deleteSphinxConfigSection(const std::string& sectionName);
    };
}
}


#endif /* SPHINXCONFIGCREATOR_HPP */
