#include <sstream>

#include "SphinxConfigOptions.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
SphinxConfigOptions::SphinxConfigOptions(void)
: maxDocId(0), createdDate(time(nullptr)), mergedDate(time(nullptr)), branches(), ranker(0), rankerExpression("")
{
}
//-----------------------------------------------------------------------------
void SphinxConfigOptions::clear(void)
{
  maxDocId = 0;
  createdDate = time(nullptr);
  mergedDate = time(nullptr);
  branches.clear();
  ranker = 0;
  rankerExpression.clear();
}
//-----------------------------------------------------------------------------
// cppcheck-suppress unusedFunction
void SphinxConfigOptions::updateCreatedDate(void)
{
  createdDate = time(nullptr);
}
//-----------------------------------------------------------------------------
void SphinxConfigOptions::updateMargedDate(void)
{
  mergedDate = time(nullptr);
}
//-----------------------------------------------------------------------------
void SphinxConfigOptions::addBranch(const std::string& branchName)
{
  branches.push_back(branchName);
}
//-----------------------------------------------------------------------------
void SphinxConfigOptions::addBranch(std::string&& branchName)
{
  branches.push_back(std::forward<std::string>(branchName));
}
//-----------------------------------------------------------------------------
// cppcheck-suppress unusedFunction
void SphinxConfigOptions::addBranches(const std::vector<std::string>& branches_)
{
  branches.insert(branches.end(), branches_.begin(), branches_.end());
}
//-----------------------------------------------------------------------------
// cppcheck-suppress unusedFunction
size_t SphinxConfigOptions::getBranchesCount(void) const
{
  return branches.size();
}
//-----------------------------------------------------------------------------
std::string SphinxConfigOptions::getBranchItem(size_t index) const
{
  return branches[index];
}
//-----------------------------------------------------------------------------
std::string SphinxConfigOptions::getBranchesAsString(void) const
{
  std::stringstream outStr;
  for (const std::string& branchName: branches)
  {
    if (!outStr.str().empty())
      outStr << ',';
    outStr << branchName;
  }
  return outStr.str();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace sphinx 
} // end namespace HCE 
