/**
 * @file SphinxConfigOptions.hpp
 *
 * Implementation class SphinxConfigOptions of a Sphinx functional object in HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_CONFIG_OPTIONS_HPP
#define SPHINX_CONFIG_OPTIONS_HPP

#include <time.h>
#include <vector>

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
class SphinxConfigOptions
{
public:
  SphinxConfigOptions(void);
  ~SphinxConfigOptions(void) {}

  void clear(void);

  void setMaxDocId(unsigned long long maxDocId_) {maxDocId=maxDocId_;}
  unsigned long long getMaxDocId(void) const {return maxDocId;}

  void setCreatedDate(time_t createdDate_) {createdDate=createdDate_;}
  time_t getCreatedDate(void) const {return createdDate;}
  void updateCreatedDate(void);

  void setMergedDate(time_t mergedDate_) {mergedDate=mergedDate_;}
  time_t getMergedDate(void) const {return mergedDate;}
  void updateMargedDate(void);

  void addBranch(const std::string& branchName);
  void addBranch(std::string&& branchName);
  void addBranches(const std::vector<std::string>& branches);

  size_t getBranchesCount(void) const;
  std::string getBranchItem(size_t index) const;
  std::string getBranchesAsString(void) const;

  void setRanker(unsigned int ranker_) {ranker=ranker_;}
  unsigned int getRanker(void) const {return ranker;}

  void setRankerExpression(const std::string& rankerExpression_) {rankerExpression=rankerExpression_;}
  std::string getRankerExpression(void) const {return rankerExpression;}

private:
  unsigned long long maxDocId;
  time_t createdDate;
  time_t mergedDate;
  std::vector<std::string> branches;
  unsigned int ranker;
  std::string rankerExpression;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace sphinx 
} // end namespace HCE 

#endif // SPHINX_CONFIG_OPTIONS_HPP
