/**
 * @file SphinxDataFile.hpp
 *
 * Implementation class SphinxDataFile for merge two data files with documents content
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_DATA_FILE_H
#define SPHINX_DATA_FILE_H

#include <Poco/SAX/InputSource.h>
#include <Poco/DOM/Node.h>
#include <Poco/DOM/Document.h>
#include <Poco/AutoPtr.h>

#include<iostream>

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
class SphinxDataFile
{
public:
  SphinxDataFile(void);
  virtual ~SphinxDataFile(void) {};

  bool append(std::istream& is);
  void save(std::ostream& os);

  std::string getErrorMsg(void) const {return errorMsg;}
  bool isError(void) const {return _IsError;}

  size_t documentCount(std::istream& is);
  bool validateData(std::istream& is);
private:
  Poco::AutoPtr<Poco::XML::Document> pOutDoc;
  Poco::AutoPtr<Poco::XML::Element> pRoot;
  std::string errorMsg;
  bool _IsError;
private:
  friend std::ostream& operator << (std::ostream& os, const SphinxDataFile& dataFile);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace sphinx
} // namespace HCE

#endif // SPHINX_DATA_FILE_H
