/**
 * @file SphinxDataSource.h
 *
 * Implementation of class SphinxDataSource for make sphinx xmlpipe type file source.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_DATA_SOURCE_HPP
#define SPHINX_DATA_SOURCE_HPP

#include <Poco/SAX/InputSource.h>
#include <Poco/DOM/Node.h>

#include<iostream>

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
namespace sphinx_data_source_const
{
  const std::string namespaceURITag = "http://sphinxsearch.com";

  const std::string docsetTag = "sphinx:docset";
  const std::string schemaTag = "sphinx:schema";
  const std::string fieldTag = "sphinx:field";
  const std::string attrTag = "sphinx:attr";
  const std::string documentTag = "sphinx:document";
  const std::string nameAttrTag = "name";
  const std::string weightAttrTag = "weight";
  const std::string documentIdAttrTag = "id";

  const std::string hceParametersTag = "hce:parameters";
  const std::string hceDocsTag = "hce:docs";
  const std::string maxDocIdAttrTag = "max_doc_id";
  const std::string documentsAttrTag = "documents";

}
//-----------------------------------------------------------------------------
class SphinxDataSource
{
public:
  SphinxDataSource(std::istream& istrSchema, std::istream& istrDocument);
  virtual ~SphinxDataSource(void);

  bool Process(std::ostream& os);

  std::string getErrorMsg(void) const {return errorMsg;}
  bool isError(void) const {return _IsError;}

  unsigned long long getMaxDocId(void) const {return maxDocId;}
private:
  Poco::XML::InputSource sourceSchema;
  Poco::XML::InputSource sourceDocument;
  std::string errorMsg;
  bool _IsError;
  unsigned long long maxDocId;
private:
  static void save(Poco::XML::Node* pNode, std::ostream& os);
  friend std::ostream& operator << (std::ostream& os, const SphinxDataSource& dataSource) throw (Poco::Exception);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace sphinx
} // end namespace HCE

#endif // SPHINX_DATA_SOURCE_HPP
