#include <Poco/DOM/DOMParser.h>
#include <Poco/DOM/DOMWriter.h>

#include <Poco/DOM/NodeIterator.h>
#include <Poco/DOM/NodeFilter.h>
#include <Poco/DOM/AutoPtr.h>
#include <Poco/DOM/NamedNodeMap.h>
#include <Poco/XML/XMLWriter.h>
#include <Poco/SAX/InputSource.h>
#include <Poco/DOM/Node.h>
#include <Poco/DOM/Document.h>

#include <Poco/SharedPtr.h>
#include <Poco/AutoPtr.h>

#include <fstream>
#include <sstream>

#include "SphinxDataSource.hpp"
#include "SphinxDataSourceParameters.hpp"
#include "SphinxMessageConst.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
SphinxDataSourceParameters::SphinxDataSourceParameters(void)
:Map(), errorMsg(""), _IsError(false)
{
}
//-----------------------------------------------------------------------------
SphinxDataSourceParameters::SphinxDataSourceParameters(std::istream& is)
:Map(), errorMsg(""), _IsError(false)
{
  load(is);
}
//-----------------------------------------------------------------------------
bool SphinxDataSourceParameters::load(std::istream& is)
{
  errorMsg.clear();
  Map.clear();
  try
  {
    Poco::XML::InputSource sourceDoc(is);
    Poco::XML::DOMParser parser;
    Poco::AutoPtr<Poco::XML::Document> pDoc = parser.parse(&sourceDoc);
    Poco::XML::NodeIterator it_doc(pDoc, Poco::XML::NodeFilter::SHOW_ALL);
    Poco::XML::Node* pNode = it_doc.nextNode();

    while (pNode)
    {
      if (pNode->nodeName() == sphinx_data_source_const::hceDocsTag)
      {
        Poco::AutoPtr<Poco::XML::NamedNodeMap> pMap =  pNode->attributes();
        if (!pMap.isNull())
        {
          for (size_t i=0;i<pMap->length();++i)
          {
            try
            {
              Map.insert(PARAM_ATTR__MAP::value_type(pMap->item(i)->nodeName(), std::stoull(pMap->item(i)->nodeValue())));
            }
            catch(std::exception& exc)
            {
              throw Poco::Exception(message_const::attribute+message_const::space+pMap->item(i)->nodeName()+
                  message_const::space+message_const::haveInvalidArgument+message_const::delimiter+message_const::space+exc.what());
            }
          }
        }
      }
      pNode = it_doc.nextNode();
    }
    _IsError = false;
  }
  catch(Poco::Exception& e)
  {
    errorMsg = e.displayText();
    _IsError = true;
  }
  return !_IsError;
}
//-----------------------------------------------------------------------------
unsigned long long SphinxDataSourceParameters::getMaxDocId(void)
{
  return Map[sphinx_data_source_const::maxDocIdAttrTag];
}
//-----------------------------------------------------------------------------
// cppcheck-suppress unusedFunction
unsigned long long SphinxDataSourceParameters::getDocumentsCount(void)
{
  return Map[sphinx_data_source_const::documentsAttrTag];
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace sphinx 
} // namespace HCE 
