/**
 * @file SphinxFilters.hpp
 *
 * Implementation classes SphinxFilter and SphinxFiltersArray instated for search request of a functional object sphinx in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_FILTERS_HPP
#define SPHINX_FILTERS_HPP

#include <iostream>

#include "JsonSerializable.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
class SphinxFilter
{
public:
  enum class FilterType
  {
    ftSimple=0,
    ftRangeInteger=1,
    ftRangeFloat=2,
    ftSimpleMulti=3
  };
  enum class ExcludeType
  {
    etFalse=0,
    etTrue=1
  };
public:
  SphinxFilter(void);
  virtual ~SphinxFilter(void) {};

  void setFilterType(SphinxFilter::FilterType filterType_) {filterType=filterType_;}
  SphinxFilter::FilterType getFilterType(void) const {return filterType;}

  void setAttributeName(const std::string& attributeName_) {attributeName=attributeName_;}
  std::string getAttributeName(void) const {return attributeName;}

  void addAttributeValue(const std::string& value);
  void setAttributeValues(std::vector<std::string>& attributeValues_) {attributeValues=attributeValues_;}
  std::vector<std::string>& getAttributeValues(void) {return attributeValues;}

  void setExcludeValue(SphinxFilter::ExcludeType excludeValue_) {excludeValue=excludeValue_;}
  ExcludeType getExcludeValue(void) const {return excludeValue;}

  SphinxFilter& operator=(const SphinxFilter& sphinxFilter);
protected:
  FilterType filterType;
  std::string attributeName;
  std::vector<std::string> attributeValues;
  ExcludeType excludeValue;

  friend std::ostream& operator << (std::ostream& os, const SphinxFilter& filters);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class SphinxFiltersArray : public IJsonSerializable
{
public:
  SphinxFiltersArray(const std::string& json="");
  virtual ~SphinxFiltersArray(void) {};

  void clear(void);

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void addFilter(Poco::SharedPtr<SphinxFilter> pFilter);
  std::vector<Poco::SharedPtr<SphinxFilter> >& getFilters(void) {return vecFilters;}

  SphinxFiltersArray& operator=(const SphinxFiltersArray& filtersArray);
private:
  std::vector<Poco::SharedPtr<SphinxFilter> > vecFilters;

  friend std::ostream& operator << (std::ostream& os, const SphinxFiltersArray& filtersArray);
  typedef IJsonSerializable inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} /* namespace sphinx */
} /* namespace HCE */
#endif // SPHINX_FILTERS_HPP
