/**
 * @file SphinxInputJsonMessage.hpp
 *
 * Implementation class SphinxInputJsonMessage describe input object json messages for processing.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_INPUT_JSON_MESSAGE_HPP
#define SPHINX_INPUT_JSON_MESSAGE_HPP

#include <iostream>
#include <vector>

#include "JsonSerializable.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
namespace input_json_message_const
{
  const std::string messageType = "type";
  const std::string messageBody = "data";
  const std::string messageTTL = "ttl";
  const unsigned int defaultTTLValue = 1;

  const std::string queryString = "q";
  const std::string filtersJsonArray = "filters";
  const std::string queryParameters = "parameters";
  const std::string orderParameters = "order";

  const std::string filterType = "type";
  const std::string attributeName = "attribute";
  const std::string attributeValues = "values";
  const std::string excludeValue = "exclude";
  const std::string maxResultsNumber = "max_results";

  const std::string timeoutValueName = "timeout";
  const unsigned long long defaultTimeoutValue = 1000;

  const unsigned int defaultMaxResultsNumber = 1000;
  const unsigned int defaultMaxAttributesNumber = 50;
  const unsigned int defaultMaxWordsNumber = 10;
  const unsigned int extraAttributesNumber = 2; // need for ID docs and weight

  const std::string offsetResultsNumber = "offset";
  const std::string limitResultsNumber = "limit";
  const std::string cutoffResultNumber = "cutoff";

  const std::string indexName = "name";
  const std::string documentFile = "body";

  const std::string commandString = "command";
  const std::string commandOptionsString = "options";

  const std::string queryId = "queryId";
  const std::string JsonType = "JsonType";
  const std::string sortFieldName = "sort_by";
  const std::string sortOrderBy = "order_by";
  const std::string externalFieldsNames = "return_json_ext_fields";
  const std::string weightAlgorithm = "algorithm";
  const std::string orderFieldsNames = "fields";
  const std::string orderBy = "order_by";
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class SphinxInputJsonMessage : public IJsonSerializable
{
public:
  enum class MessageType
  {
    mtSearch=0,
    mtIndex=1,
    mtManage=2
  };
public:
  explicit SphinxInputJsonMessage(const std::string& json="");
  explicit SphinxInputJsonMessage(SphinxInputJsonMessage::MessageType messageType_);
  virtual ~SphinxInputJsonMessage(void) {};

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void setType(SphinxInputJsonMessage::MessageType messageType_) {messageType=messageType_;}
  SphinxInputJsonMessage::MessageType getType(void) const {return messageType;}

  void setData(const std::string& messageBody_) {messageBody=messageBody_;}
  std::string getData(void) {return messageBody;}

  void setTTL(unsigned int ttl) {messageTTL=ttl;}
  unsigned int getTTL(void) const {return messageTTL;}
private:
  MessageType messageType;
  std::string messageBody;
  unsigned int messageTTL;

private:
  friend std::istream& operator>>(std::istream& is, SphinxInputJsonMessage& jsonHandler);
  friend std::ostream& operator<<(std::ostream& os, const SphinxInputJsonMessage& jsonHandler);
  typedef IJsonSerializable inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace sphinx
} // end namespace HCE

#endif // SPHINX_INPUT_JSON_MESSAGE_HPP
