/**
 * @file SphinxInputJsonMessageIndex.hpp
 *
 * Implementation class SphinxInputJsonMessageIndex describe object json messages for sphinx indexation processing.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_INPUT_JSON_MESSAGE_INDEX_HPP
#define SPHINX_INPUT_JSON_MESSAGE_INDEX_HPP

#include "JsonSerializable.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
class SphinxInputJsonMessageIndex : public IJsonSerializable
{
public:
  SphinxInputJsonMessageIndex(const std::string& json="");
  virtual ~SphinxInputJsonMessageIndex(void) {}

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void setIndexName(const std::string& indexName_) {indexName=indexName_;}
  std::string getIndexName(void) const {return indexName;}

  void setDocumentFile(const std::string& documentFile_) {documentFile=documentFile_;}
  std::string getDocumentFile(void) const {return documentFile;}

  void addParameters(const std::string& paramName, const std::string& paramValue);
  std::vector<std::pair<std::string, std::string> >& getParameters(void) {return params;}
private:
  std::string indexName;
  std::string documentFile;
  std::vector<std::pair<std::string, std::string> > params;

  friend std::istream& operator>>(std::istream& is, SphinxInputJsonMessageIndex& messageIndex);
  friend std::ostream& operator<<(std::ostream& os, const SphinxInputJsonMessageIndex& messageIndex);
  typedef IJsonSerializable inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace sphinx
} // end namespace HCE

#endif // SPHINX_INPUT_JSON_MESSAGE_INDEX_HPP
