#include <Poco/JSON/Object.h>
#include <Poco/JSON/Array.h>
#include <Poco/JSON/JSON.h>
#include <Poco/JSON/Stringifier.h>
#include <Poco/JSON/Parser.h>
#include <Poco/JSON/JSONException.h>

#include "SphinxError.hpp"
#include "SphinxInputJsonMessageManage.hpp"
#include "SphinxInputJsonMessage.hpp"

namespace HCE
{
namespace sphinx
{
//----------------------------------------------------------------------------
SphinxInputJsonMessageManage::SphinxInputJsonMessageManage(const std::string& json)
:inherited(), commandString(""), commandOptionsString("")
{
  if (!json.empty())
    unserialize(json);
}
//-----------------------------------------------------------------------------
bool SphinxInputJsonMessageManage::serialize(std::string& json)
{
  _isError = false;
  errorMsg.clear();
  try
  {
    Poco::JSON::Object::Ptr obj = new Poco::JSON::Object();
    obj->set(input_json_message_const::commandString, commandString);
    obj->set(input_json_message_const::commandOptionsString, commandOptionsString);

    std::stringstream ostr;
    Poco::JSON::Stringifier::stringify(obj, ostr);
    json = ostr.str();

  }
  catch(std::exception& e)
  {
    errorMsg = e.what();
    errorCode = ERROR_SERIALIZE;
    _isError = true;
  }
  return !_isError;
}
//----------------------------------------------------------------------------
bool SphinxInputJsonMessageManage::unserialize(const std::string& json)
{
  commandString.clear();
  commandOptionsString.clear();
  errorMsg.clear();
  _isError = false;
  try
  {
    Poco::JSON::Parser parser;
    Poco::Dynamic::Var res = parser.parse(json);
    Poco::JSON::Object::Ptr obj = res.extract<Poco::JSON::Object::Ptr>();

    Poco::Dynamic::Var tmp = obj->get(input_json_message_const::commandString);
    if (tmp.isString())
      commandString = tmp.convert<std::string>();

    tmp = obj->get(input_json_message_const::commandOptionsString);
    if (tmp.isString())
      commandOptionsString = tmp.convert<std::string>();

  }
  catch(Poco::JSON::JSONException& e)
  {
    errorMsg = e.displayText();
    errorCode = ERROR_UNSERIALIZE;
    _isError = true;
  }
  return !_isError;
}
//----------------------------------------------------------------------------
std::istream& operator>>(std::istream& is, SphinxInputJsonMessageManage& messageManage)
{
  std::string json;
  std::getline(is, json);
  messageManage.unserialize(json);
  return is;
}
//-----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const SphinxInputJsonMessageManage& messageManage)
{
  std::string json;
  const_cast<SphinxInputJsonMessageManage&>(messageManage).serialize(json);
  return os << json;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace sphinx
} // end namespace HCE
