/**
 * @file SphinxInputJsonMessageSearch.hpp
 *
 * Implementation class SphinxInputJsonMessageSearch describe object json messages for sphinx search processing.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_INPUT_JSON_MESSAGE_SEARCH_HPP
#define SPHINX_INPUT_JSON_MESSAGE_SEARCH_HPP

#include "SphinxFilters.hpp"
#include "JsonSerializable.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
class SphinxInputJsonMessageSearch : public IJsonSerializable
{
  std::string convertVarToString(Poco::Dynamic::Var& var) throw (std::exception); // convert if input type Numeric or String
public:
  explicit SphinxInputJsonMessageSearch(const std::string& json="");
  virtual ~SphinxInputJsonMessageSearch(void) {}

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void setQueryString(const std::string& queryString_) {queryString=queryString_;}
  std::string getQueryString(void) const {return queryString;}

  void addQueryParameters(const std::string& paramName, const std::string& paramValue);
  std::vector<std::pair<std::string, std::string> >& getQueryParameters(void) {return queryParams;}

  void addOrderParameters(const std::string& paramName, const std::string& paramValue);
  std::vector<std::pair<std::string, std::string> >& getOrderParameters(void) {return orderParams;}

  void addFilter(Poco::SharedPtr<SphinxFilter> pFilter);
  void addFilter(SphinxFilter::FilterType filterType, const std::string& attributeName,
      std::vector<std::string> attributeValues, SphinxFilter::ExcludeType exclude);

  void setFilters(const SphinxFiltersArray& filtersArray_) {filtersArray=filtersArray_;}
  SphinxFiltersArray& getFilters(void) {return filtersArray;}

  void addExternalFields(const std::string& fieldName);
  std::vector<std::string>& getExternalFields(void) {return externalFields;}

  void addOrderFields(const std::string& fieldName);
  std::vector<std::string>& getOrderFields(void) {return orderFields;}
private:
  std::string queryString;
  std::vector<std::pair<std::string, std::string> > queryParams;
  std::vector<std::pair<std::string, std::string> > orderParams;
  SphinxFiltersArray filtersArray;

  std::vector<std::string> externalFields;
  std::vector<std::string> orderFields;

  friend std::istream& operator>>(std::istream& is, SphinxInputJsonMessageSearch& messageSearch);
  friend std::ostream& operator<<(std::ostream& os, const SphinxInputJsonMessageSearch& messageSearch);
  typedef IJsonSerializable inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace Sphinx
} // end namespace HCE

#endif // SPHINX_INPUT_JSON_MESSAGE_SEARCH_HPP
