/**
 * @file SphinxMatchInfo.hpp
 *
 * Implementation class SphinxMatchInfo of a sphinx functional object in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_MATCH_INFO_HPP
#define SPHINX_MATCH_INFO_HPP

#include <vector>

#include "SphinxResultDataItem.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
struct AttrInfo
{
  AttrInfo(void) : name(""), value("") {}
  AttrInfo(const std::string& name_, const std::string& value_) : name(name_), value(value_) {}
  AttrInfo(const AttrInfo& rhs);
  AttrInfo(AttrInfo&& rhs);

  AttrInfo& operator=(const AttrInfo& rhs);
  AttrInfo& operator=(AttrInfo&& rhs);

  std::string name;
  std::string value;
};
//-----------------------------------------------------------------------------
class SphinxMatchInfo
{
  enum {STRING_LENGTH=20};
public:
  SphinxMatchInfo(void);
  explicit SphinxMatchInfo(size_t attributesCount);
  SphinxMatchInfo(const SphinxMatchInfo& rhs);
  SphinxMatchInfo(SphinxMatchInfo&& rhs);
  ~SphinxMatchInfo(void) {}

  SphinxMatchInfo& operator=(const SphinxMatchInfo& rhs);
  SphinxMatchInfo& operator=(SphinxMatchInfo&& rhs);

  void setDocId(unsigned long long docId_) {docId = docId_;}
  unsigned long long getDocId(void) const {return docId;}
private:
  void setWeight(const std::string& weight_) {weight = weight_;}
public:
  std::string getWeight(void) const {return weight;}
  void setWeightFromNumeric(unsigned long long weight, unsigned long long length= STRING_LENGTH);
  void setWeightFromHexString(const std::string& weight);
  void setWeightFromDecString(const std::string& weight, unsigned long long length=STRING_LENGTH);

  void addAttrInfo(const AttrInfo& attrInfo);
  void addAttrInfo(AttrInfo&& attrInfo);
  void addAttrInfo(const std::string& name, const std::string& value);

  const std::vector<SphinxResultDataItem<AttrInfo> >& getAttributes(void) const {return attributes;}
  const AttrInfo getFieldByName(const std::string& attrName) const;

  size_t getAttributesCount(void) const;
  const AttrInfo& getAttrItem(size_t index) const;
  void setAttrItem(size_t index, const AttrInfo& attr);
  void setAttrItem(size_t index, AttrInfo&& attr);

  void clear(void);
  void reset(void);
  void resize(size_t sz);
private:
  unsigned long long docId;
  std::string weight;
  std::vector<SphinxResultDataItem<AttrInfo> > attributes;

  friend class SphinxResultDataMemoryManager;
  friend class CalculateStrategyDefault;
  friend class CalculateStrategyUseFieldNames;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace sphinx 
} // namespace HCE 

#endif // SPHINX_MATCH_INFO_HPP
