/**
 * @file SphinxNodeOptions.hpp
 *
 * Implementation class SphinxNodeOptions of a functional object is content data options need for search use sphinx in HCE project node
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_NODE_OPTIONS_HPP
#define SPHINX_NODE_OPTIONS_HPP

#include <iostream>

#include "SphinxSearchInterface.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
class SphinxNodeOptions
{
public:
  explicit SphinxNodeOptions(const std::string& nodeName_="",
      unsigned int nodeNumber_=0,
      const std::string& homeDir_=sphinx_search_const::defaultHomeDir,
      const std::string& indexName_="");

  virtual ~SphinxNodeOptions(void) {}

  void setNodeName(const std::string& nodeName_) {nodeName=nodeName_;}
  std::string getNodeName(void) const {return nodeName;}

  void setNodeNumber(unsigned int nodeNumber_) {nodeNumber=nodeNumber_;}
  unsigned int getNodeNumber(void) const {return nodeNumber;}

  void setHomeDir(const std::string& homeDir_);
  std::string getHomeDir(void) const {return homeDir;}

  void setIndexName(const std::string& indexName_) {indexName=indexName_;}
  std::string getIndexName(void) const {return indexName;}

  std::string getDataDir(void) const {return dataDir;}
  std::string getRunDir(void) const {return runDir;}
  std::string getLogDir(void) const {return logDir;}
  std::string getEtcDir(void) const {return etcDir;}

private:
  std::string nodeName;
  unsigned int nodeNumber;
  std::string homeDir;
  std::string indexName;

  std::string dataDir;
  std::string runDir;
  std::string logDir;
  std::string etcDir;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace sphinx 
} // namespace HCE 

#endif // SPHINX_NODE_OPTIONS_HPP
