#include <Poco/JSON/Object.h>
#include <Poco/JSON/Array.h>
#include <Poco/JSON/JSON.h>
#include <Poco/JSON/Stringifier.h>
#include <Poco/JSON/Parser.h>
#include <Poco/JSON/JSONException.h>
#include <Poco/Dynamic/Var.h>

#include "SphinxOutputJsonMessage.hpp"
#include "SphinxError.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
SphinxOutputJsonMessage::SphinxOutputJsonMessage(const std::string& json)
:inherited(), errorCode(0), errorMessage(""), data(""), time(0)
{
  if (!json.empty())
    unserialize(json);
}
//-----------------------------------------------------------------------------
SphinxOutputJsonMessage::SphinxOutputJsonMessage(const SphinxOutputJsonMessage& rhs)
:inherited(), errorCode(0), errorMessage(""), data(""), time(0)
{
  *this = rhs;
}
//-----------------------------------------------------------------------------
SphinxOutputJsonMessage& SphinxOutputJsonMessage::operator=(const SphinxOutputJsonMessage& rhs)
{
  if (&rhs!=this)
  {
    errorCode = rhs.getErrorCode();
    errorMessage = rhs.getErrorMessage();
    data = rhs.getData();
    time = rhs.getTime();
  }
  return *this;
}
//-----------------------------------------------------------------------------
void SphinxOutputJsonMessage::clear(void)
{
  errorCode = 0;
  errorMessage.clear();
  data.clear();
  time = 0;
  _isError = false;
}
//-----------------------------------------------------------------------------
bool SphinxOutputJsonMessage::serialize(std::string& json)
{
  _isError = true;
  errorMsg.clear();
  try
  {
    Poco::JSON::Object::Ptr obj = new Poco::JSON::Object();
    obj->set(output_json_message_const::errorCode, errorCode);
    obj->set(output_json_message_const::errorMessage, errorMessage);
    obj->set(output_json_message_const::data, data);
    obj->set(output_json_message_const::time, time);

    std::stringstream ostr;
    Poco::JSON::Stringifier::stringify(obj, ostr);
    json = ostr.str();

    _isError = false;
  }
  catch(std::exception& e)
  {
   errorMsg = e.what();
   errorCode = ERROR_SERIALIZE;
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
bool SphinxOutputJsonMessage::unserialize(const std::string& json)
{
  clear();
  _isError = true;
  errorMsg.clear();
  errorCode = NO_ERROR;
  try
  {
    Poco::JSON::Parser parser;
    Poco::Dynamic::Var res = parser.parse(json);
    Poco::JSON::Object::Ptr obj = res.extract<Poco::JSON::Object::Ptr>();

    Poco::Dynamic::Var tmp = obj->get(output_json_message_const::errorCode);
    errorCode = convertVarToNumeric<unsigned int>(tmp, 0);

    tmp = obj->get(output_json_message_const::errorMessage);
    if (tmp.isString())
      errorMessage = tmp.convert<std::string>();

    tmp = obj->get(output_json_message_const::data);
    if (tmp.isString())
      data = tmp.convert<std::string>();

    tmp = obj->get(output_json_message_const::time);
    time = convertVarToNumeric<unsigned int>(tmp, 0);

    _isError = false;
  }
  catch(Poco::JSON::JSONException& e)
  {
    errorMsg = e.displayText();
    errorCode = ERROR_UNSERIALIZE;
  }
  catch(Poco::Exception& e)
  {
    errorMsg = e.displayText();
    errorCode = ERROR_UNSERIALIZE;
  }
  return !_isError;
}
//-----------------------------------------------------------------------------
std::istream& operator>>(std::istream& is, SphinxOutputJsonMessage& jsonHandler)
{
  std::string json;
  is.seekg(0, std::ios::end);
  json.resize(is.tellg());
  is.seekg(0, std::ios::beg);
  is.read(const_cast<char*>(json.c_str()), json.size());
  jsonHandler.unserialize(json);
  return is;
}
//-----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const SphinxOutputJsonMessage& jsonHandler)
{
  std::string json;
  const_cast<SphinxOutputJsonMessage&>(jsonHandler).serialize(json);
  return os << json;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace sphinx
} // end namespace HCE
