/**
 * @file SphinxOutputJsonMessage.hpp
 *
 * Implementation class SphinxOutputJsonMessage describe output object json messages after processing.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_OUTPUT_JSON_MESSAGE_HPP
#define SPHINX_OUTPUT_JSON_MESSAGE_HPP

#include <iostream>

#include "JsonSerializable.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
namespace output_json_message_const
{
  const std::string errorCode = "error_code";
  const std::string errorMessage = "error_message";
  const std::string data = "data";
  const std::string time = "time";
}
//-----------------------------------------------------------------------------
class SphinxOutputJsonMessage : public IJsonSerializable
{
public:
  explicit SphinxOutputJsonMessage(const std::string& json="");
  SphinxOutputJsonMessage(const SphinxOutputJsonMessage&);
  virtual ~SphinxOutputJsonMessage(void) {};

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  void setErrorCode(unsigned int errorCode_) {errorCode=errorCode_;}
  unsigned int getErrorCode(void) const {return errorCode;}

  void setErrorMessage(const std::string& errorMessage_) {errorMessage=errorMessage_;}
  std::string getErrorMessage(void) const {return errorMessage;}

  void setData(const std::string& data_) {data=data_;}
  std::string getData(void) const {return data;}

  void setTime(unsigned int time_) {time=time_;}
  unsigned int getTime(void) const {return time;}

  void clear(void);
  SphinxOutputJsonMessage& operator=(const SphinxOutputJsonMessage&);
private:
  unsigned int errorCode;   // error code: 0 - successful, otherwise return really internal  code of errors
  std::string errorMessage; // message about error if need or empty
  std::string data;   // json object
  unsigned int time; // value in msec !!!

private:
  friend std::istream& operator>>(std::istream& is, SphinxOutputJsonMessage& jsonHandler);
  friend std::ostream& operator<<(std::ostream& os, const SphinxOutputJsonMessage& jsonHandler);
  typedef IJsonSerializable inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace sphinx
} // end namespace HCE

#endif // SPHINX_OUTPUT_JSON_MESSAGE_HPP
