#include "SphinxRequestInfo.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
WordInfo::WordInfo(const WordInfo& rhs)
: word(""), hits(0), docs(0)
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
WordInfo::WordInfo(WordInfo&& rhs)
: word(""), hits(0), docs(0)
{
  (*this) = std::forward<WordInfo>(rhs);
}
//-----------------------------------------------------------------------------
WordInfo& WordInfo::operator=(const WordInfo& rhs)
{
  if (this != &rhs)
  {
    word = rhs.word;
    hits = rhs.hits;
    docs = rhs.docs;
  }
  return *this;
}
//-----------------------------------------------------------------------------
WordInfo& WordInfo::operator=(WordInfo&& rhs)
{
  if (this != &rhs)
  {
    word = std::move(rhs.word);
    hits = std::move(rhs.hits);
    docs = std::move(rhs.docs);
  }
  return *this;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
SphinxRequestInfo::SphinxRequestInfo(void)
: nodeName(""), query(""), queryId(0), maxResultsNumber(0), orderBy(0), total(0), totalFound(0), timeMsec(0), words()
{
}
//-----------------------------------------------------------------------------
SphinxRequestInfo::SphinxRequestInfo(size_t wordsCount)
: nodeName(""), query(""), queryId(0), maxResultsNumber(0), orderBy(0), total(0), totalFound(0), timeMsec(0), words(wordsCount)
{
}
//-----------------------------------------------------------------------------
SphinxRequestInfo::SphinxRequestInfo(const SphinxRequestInfo& rhs)
: nodeName(""), query(""), queryId(0), maxResultsNumber(0), orderBy(0), total(0), totalFound(0), timeMsec(0), words(rhs.getWords().size())
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
SphinxRequestInfo::SphinxRequestInfo(SphinxRequestInfo&& rhs)
: nodeName(""), query(""), queryId(0), maxResultsNumber(0), orderBy(0), total(0), totalFound(0), timeMsec(0), words(rhs.getWords().size())
{
  (*this) = std::forward<SphinxRequestInfo>(rhs);
}
//-----------------------------------------------------------------------------
SphinxRequestInfo& SphinxRequestInfo::operator=(const SphinxRequestInfo& rhs)
{
  if (this != &rhs)
  {
    nodeName = rhs.getNodeName();
    query = rhs.getQuery();
    queryId = rhs.getQueryId();
    maxResultsNumber = rhs.getMaxResultsNumber();
    orderBy = rhs.getOrderBy();
    total = rhs.getTotal();
    totalFound = rhs.getTotalFound();
    timeMsec = rhs.getTimeMsec();
    words = rhs.getWords();
  }
  return *this;
}
//-----------------------------------------------------------------------------
SphinxRequestInfo& SphinxRequestInfo::operator=(SphinxRequestInfo&& rhs)
{
  if (this != &rhs)
  {
    nodeName = std::move(rhs.getNodeName());
    query = std::move(rhs.getQuery());
    queryId = std::move(rhs.getQueryId());
    maxResultsNumber = std::move(rhs.getMaxResultsNumber());
    orderBy = std::move(rhs.getOrderBy());
    total = std::move(rhs.getTotal());
    totalFound = std::move(rhs.getTotalFound());
    timeMsec = std::move(rhs.getTimeMsec());
    words = std::move(rhs.getWords());
  }
  return *this;
}
//-----------------------------------------------------------------------------
void SphinxRequestInfo::addWordInfo(const WordInfo& wordInfo)
{
  words.push_back(SphinxResultDataItem<WordInfo>(wordInfo));
}
//-----------------------------------------------------------------------------
void SphinxRequestInfo::addWordInfo(WordInfo&& wordInfo)
{
  words.push_back(SphinxResultDataItem<WordInfo>(std::forward<WordInfo>(wordInfo)));
}
//-----------------------------------------------------------------------------
void SphinxRequestInfo::addWordInfo(const std::string& word, unsigned int hits, unsigned int docs)
{
  addWordInfo(std::forward<WordInfo>(WordInfo(word, hits, docs)));
}
//-----------------------------------------------------------------------------
size_t SphinxRequestInfo::getWordsCount(void) const
{
  size_t count = words.size();
  for (size_t i=0;i<count;++i)
  {
    if (words[i].isEmpty())
    {
      count = i;
      break;
    }
  }
  return count;
}
//-----------------------------------------------------------------------------
const WordInfo& SphinxRequestInfo::getWordItem(size_t index) const
{
  return words[index].getItem();
}
//-----------------------------------------------------------------------------
// cppcheck-suppress unusedFunction
void SphinxRequestInfo::setWordItem(size_t index, const WordInfo& word)
{
  words[index].setItem(word);
}
//-----------------------------------------------------------------------------
void SphinxRequestInfo::setWordItem(size_t index, WordInfo&& word)
{
  words[index].setItem(std::move(word));
}
//-----------------------------------------------------------------------------
void SphinxRequestInfo::clear(void)
{
  nodeName.clear();
  query.clear();
  queryId = 0;
  maxResultsNumber = 0;
  orderBy = 0;
  total = 0;
  totalFound = 0;
  timeMsec = 0;
  words.clear();
}
//-----------------------------------------------------------------------------
void SphinxRequestInfo::reset(void)
{
  nodeName.clear();
  query.clear();
  queryId = 0;
  maxResultsNumber = 0;
  orderBy = 0;
  total = 0;
  totalFound = 0;
  timeMsec = 0;
  const size_t count = words.size();
  for (size_t i=0;i<count;++i)
    words[i].isEmpty(true);
}
//-----------------------------------------------------------------------------
void SphinxRequestInfo::resize(size_t sz)
{
  words.resize(sz);
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace sphinx 
} // namespace HCE 
