/**
 * @file SphinxRequestInfo.hpp
 *
 * Implementation class SphinxRequestInfo of a sphinx functional object in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_REQUEST_INFO_HPP
#define SPHINX_REQUEST_INFO_HPP

#include <vector>

#include "SphinxResultDataItem.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
struct WordInfo
{
  WordInfo(void) : word(""), hits(0), docs(0) {}
  WordInfo(const std::string& word_, unsigned int hits_, unsigned int docs_) : word(word_), hits(hits_), docs(docs_) {}
  WordInfo(const WordInfo& rhs);
  WordInfo(WordInfo&& rhs);

  WordInfo& operator=(const WordInfo& rhs);
  WordInfo& operator=(WordInfo&& rhs);

  std::string word;
  unsigned int hits;
  unsigned int docs;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class SphinxRequestInfo
{
public:
  SphinxRequestInfo(void);
  explicit SphinxRequestInfo(size_t wordsCount);
  SphinxRequestInfo(const SphinxRequestInfo& rhs);
  SphinxRequestInfo(SphinxRequestInfo&& rhs);
  ~SphinxRequestInfo(void) {}

  SphinxRequestInfo& operator=(const SphinxRequestInfo& rhs);
  SphinxRequestInfo& operator=(SphinxRequestInfo&& rhs);

  void setNodeName(const std::string& nodeName_) {nodeName=nodeName_;}
  std::string getNodeName(void) const {return nodeName;}

  void setQuery(const std::string& query_) {query=query_;}
  std::string getQuery(void) const {return query;}

  void setQueryId(unsigned int queryId_) {queryId=queryId_;}
  unsigned int getQueryId(void) const {return queryId;}

  void setMaxResultsNumber(unsigned int maxResultsNumber_) {maxResultsNumber=maxResultsNumber_;}
  unsigned int getMaxResultsNumber(void) const {return maxResultsNumber;}

  void setOrderBy(unsigned int orderBy_) {orderBy=orderBy_;}
  unsigned int getOrderBy(void) const {return orderBy;}

  void setTotal(unsigned int total_) {total=total_;}
  unsigned int getTotal(void) const {return total;}

  void setTotalFound(unsigned int totalFound_) {totalFound=totalFound_;}
  unsigned int getTotalFound(void) const {return totalFound;}

  void setTimeMsec(unsigned int timeMsec_) {timeMsec=timeMsec_;}
  unsigned int getTimeMsec(void) const {return timeMsec;}

  void addWordInfo(const WordInfo& wordInfo);
  void addWordInfo(WordInfo&& wordInfo);
  void addWordInfo(const std::string& word, unsigned int hits, unsigned int docs);

  const std::vector<SphinxResultDataItem<WordInfo> >& getWords(void) const {return words;}

  size_t getWordsCount(void) const;
  const WordInfo& getWordItem(size_t index) const;
  void setWordItem(size_t index, const WordInfo& word);
  void setWordItem(size_t index, WordInfo&& word);

  void clear(void);
  void reset(void);
  void resize(size_t sz);
private:
  std::string nodeName;     // node name for about words information
  std::string query;        // query line
  unsigned int queryId;     // hash or crc received outside and it must come back
  unsigned int maxResultsNumber;  // use for size elements of docs need for allocate, also result document count resend to reduce
  unsigned int orderBy;     // order by document resend to reduce
  unsigned int total;
  unsigned int totalFound;
  unsigned int timeMsec;    // time of request (in msec.)

  std::vector<SphinxResultDataItem<WordInfo> > words;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace sphinx 
} // namespace HCE 

#endif // SPHINX_REQUEST_INFO_HPP
