/**
 * @file SphinxResultData.hpp
 *
 * Implementation class SphinxResultData is a wrapper encapsulates a result of a search query in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_RESULT_DATA_HPP
#define SPHINX_RESULT_DATA_HPP

#include <iostream>
#include <vector>
#include <map>
#include <utility>
#include <stdint.h>

#include "SphinxMatchInfo.hpp"
#include "SphinxRequestInfo.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
enum {HEX_STRING_LENGTH=20};

struct HexString
{
  unsigned long long value;
  unsigned long long length;
public:
  HexString(unsigned long long value_, unsigned long long length_=HEX_STRING_LENGTH) : value(value_), length(length_) {}

  friend std::ostream& operator << (std::ostream& os, const HexString& rhs);
};
//-----------------------------------------------------------------------------
std::string toHexString(unsigned long long value, unsigned long long length=HEX_STRING_LENGTH);
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class SphinxResultData
{
public:
  SphinxResultData(void);
  explicit SphinxResultData(size_t matchesCount);
  SphinxResultData(size_t matchesCount, size_t attributesCount);
  ~SphinxResultData(void);

  void addMatchInfo(const SphinxMatchInfo& match);
  void addMatchInfo(SphinxMatchInfo&& match);

  size_t getMatchInfoCount(void) const;
  const SphinxMatchInfo& getMatchInfoItem(size_t index) const;
  void setMatchInfoItem(size_t index, const SphinxMatchInfo& match);
  void setMatchInfoItem(size_t index, SphinxMatchInfo&& match);

  void addRequestInfo(const SphinxRequestInfo& request);
  void addRequestInfo(SphinxRequestInfo&& request);

  size_t getRequestInfoCount(void) const;
  const SphinxRequestInfo& getRequestInfoItem(size_t index) const;
  void setRequestInfoItem(size_t index, const SphinxRequestInfo& request);
  void setRequestInfoItem(size_t index, SphinxRequestInfo&& request);

  void clear(void);
  void reset(void);
  void resize(size_t);
  void resize(size_t , size_t);

  void clearRequestInfo(void);

  void dump(std::ostream& os) const;
private:
  std::vector<SphinxResultDataItem<SphinxMatchInfo> > matchInfoList;
  std::vector<SphinxResultDataItem<SphinxRequestInfo> > requestInfoList;

  friend std::ostream& operator << (std::ostream& os, const SphinxResultData& resultData);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace sphinx 
} // namespace HCE 

#endif // SPHINX_RESULT_DATA_HPP
