/**
 * @file SphinxResultDataItem.hpp
 *
 * Implementation class SphinxResultDataItem is base template class of saved one result item a search query in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_RESULT_DATA_ITEM_HPP
#define SPHINX_RESULT_DATA_ITEM_HPP

#include <utility>
#include <iostream>

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
template <typename Elem>
class SphinxResultDataItem
{
public:
  SphinxResultDataItem(void) : elem(), _IsEmpty(false) {}
  explicit SphinxResultDataItem(const Elem& elem_);
  explicit SphinxResultDataItem(Elem&& elem_);
  SphinxResultDataItem(const SphinxResultDataItem& rhs);
  SphinxResultDataItem(SphinxResultDataItem&& rhs);
  ~SphinxResultDataItem(void) {}

  SphinxResultDataItem& operator=(const SphinxResultDataItem& rhs);
  SphinxResultDataItem& operator=(SphinxResultDataItem&& rhs);

  void setItem(const Elem& elem_);
  void setItem(Elem&& elem_);
  const Elem& getItem(void) const {return elem;}
  Elem& getItem(void) {return elem;}

  void isEmpty(bool isEmpty_) {_IsEmpty=isEmpty_;}
  bool isEmpty(void) const {return _IsEmpty;}
protected:
  Elem elem;
  bool _IsEmpty;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
template <typename Elem> SphinxResultDataItem<Elem>::SphinxResultDataItem(const Elem& elem_)
    : elem(elem_), _IsEmpty(false)
{
}
//-----------------------------------------------------------------------------
template <typename Elem> SphinxResultDataItem<Elem>::SphinxResultDataItem(Elem&& elem_)
    : elem(std::move(elem_)), _IsEmpty(false)
{
}
//-----------------------------------------------------------------------------
template <typename Elem> SphinxResultDataItem<Elem>::SphinxResultDataItem(const SphinxResultDataItem& rhs)
    : elem(), _IsEmpty(false)
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
template <typename Elem> SphinxResultDataItem<Elem>::SphinxResultDataItem(SphinxResultDataItem&& rhs)
    : elem(), _IsEmpty(false)
{
  (*this) = std::forward<SphinxResultDataItem<Elem> >(rhs);
}
//-----------------------------------------------------------------------------
template <typename Elem> SphinxResultDataItem<Elem>& SphinxResultDataItem<Elem>::operator=(const SphinxResultDataItem& rhs)
{
  if (this!=&rhs)
  {
    elem = rhs.getItem();
    _IsEmpty = rhs.isEmpty();
  }
  return *this;
}
//-----------------------------------------------------------------------------
template <typename Elem> SphinxResultDataItem<Elem>& SphinxResultDataItem<Elem>::operator=(SphinxResultDataItem&& rhs)
{
  if (this!=&rhs)
  {
    elem = std::move(rhs.getItem());
    _IsEmpty = std::move(rhs.isEmpty());
  }
  return *this;
}
//-----------------------------------------------------------------------------
template <typename Elem> void SphinxResultDataItem<Elem>::setItem(const Elem& elem_)
{
  elem=elem_;
  _IsEmpty = false;
}
//-----------------------------------------------------------------------------
template <typename Elem> void SphinxResultDataItem<Elem>::setItem(Elem&& elem_)
{
  elem=std::move(elem_);
  _IsEmpty = false;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace sphinx
} // namespace HCE

#endif // SPHINX_RESULT_DATA_ITEM_HPP
