#include "SphinxResultDataMemoryManager.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
SphinxResultDataMemoryManager::SphinxResultDataMemoryManager(SphinxResultData& resultData_)
: resultData(resultData_), matchesCount(0), attributesCount(0)
{
}
//-----------------------------------------------------------------------------
SphinxResultDataMemoryManager::~SphinxResultDataMemoryManager(void)
{
}
//-----------------------------------------------------------------------------
void SphinxResultDataMemoryManager::resetMatchInfo(void)
{
  matchInfo.reset();
}
//-----------------------------------------------------------------------------
void SphinxResultDataMemoryManager::setMatchInfoDocId(unsigned long long docId)
{
  matchInfo.setDocId(docId);
}
//-----------------------------------------------------------------------------
void SphinxResultDataMemoryManager::setMatchInfoWeight(const std::string& weight)
{
  matchInfo.setWeight(weight);
}
//-----------------------------------------------------------------------------
bool SphinxResultDataMemoryManager::addMatchInfoAttr(AttrInfo&& attrInfo)
{
  size_t index = matchInfo.getAttributesCount();
  if (index < attributesCount)
    matchInfo.setAttrItem(index, std::forward<AttrInfo>(attrInfo));
  return (index < attributesCount);
}
//-----------------------------------------------------------------------------
bool SphinxResultDataMemoryManager::applyMatchInfo(void)
{
  size_t index = resultData.getMatchInfoCount();
  if (index < matchesCount)
  {
    resultData.setMatchInfoItem(index, std::forward<SphinxMatchInfo>(matchInfo));
    matchInfo.reset();
  }
  return (index < matchesCount);
}
//-----------------------------------------------------------------------------
void SphinxResultDataMemoryManager::reset(void)
{
  matchInfo.reset();
  resultData.reset();
}
//-----------------------------------------------------------------------------
void SphinxResultDataMemoryManager::resize(size_t matchesCount_, size_t attributesCount_)
{
  matchInfo.resize(attributesCount_);
  resultData.resize(matchesCount_, attributesCount_);
  matchesCount = matchesCount_;
  attributesCount = attributesCount_;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace sphinx 
} // namespace HCE 
