#include <Poco/DOM/DOMParser.h>
#include <Poco/DOM/DOMWriter.h>

#include <Poco/DOM/NodeIterator.h>
#include <Poco/DOM/NodeFilter.h>
#include <Poco/DOM/AutoPtr.h>
#include <Poco/DOM/NamedNodeMap.h>
#include <Poco/XML/XMLWriter.h>

#include <Poco/SharedPtr.h>
#include <Poco/AutoPtr.h>

#include <fstream>
#include <sstream>

#include "SphinxDataSource.hpp"
#include "SphinxSchemaFile.hpp"
#include "SphinxMessageConst.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
SphinxSchemaFile::SphinxSchemaFile(std::istream& istrSchema)
:sourceSchema(istrSchema), errorMsg(""), _IsError(false), attributesCount(0)
{
}
//-----------------------------------------------------------------------------
bool SphinxSchemaFile::getFields(std::vector<std::pair<std::string, int> >& Vec)
{
  errorMsg.clear();
  Vec.clear();
  try
  {
    Poco::XML::DOMParser parser;
    Poco::AutoPtr<Poco::XML::Document> pDoc = parser.parse(&sourceSchema);
    Poco::XML::NodeIterator it_doc(pDoc, Poco::XML::NodeFilter::SHOW_ALL);
    Poco::XML::Node* pNode = it_doc.nextNode();

    while (pNode)
    {
      if (pNode->nodeName() == sphinx_data_source_const::fieldTag)
      {
        Poco::AutoPtr<Poco::XML::NamedNodeMap> pMap =  pNode->attributes();
        if (!pMap.isNull())
        {
          std::string name;
          int weight=0;
          for (size_t i=0;i<pMap->length();++i)
          {
            if (pMap->item(i)->nodeName()==sphinx_data_source_const::nameAttrTag)
              name = pMap->item(i)->nodeValue();
            if (pMap->item(i)->nodeName()==sphinx_data_source_const::weightAttrTag)
            {
              try
              {
                weight = std::stoul(pMap->item(i)->nodeValue());
              }
              catch(std::exception& exc)
              {
                throw Poco::Exception(message_const::attribute+message_const::space+name+message_const::space+message_const::haveInvalidArgument+
                    message_const::delimiter+message_const::space+exc.what());
              }
            }
          }
          Vec.push_back(std::make_pair(name, weight));
        }
      }
      pNode = it_doc.nextNode();
    }
    _IsError = false;
  }
  catch(Poco::Exception& e)
  {
    errorMsg = e.message();
    _IsError = true;
  }
  return !_IsError;
}
//-----------------------------------------------------------------------------
bool SphinxSchemaFile::getNames(std::vector<std::string>& names)
{
  errorMsg.clear();
  names.clear();
  attributesCount = 0;
  try
  {
    Poco::XML::DOMParser parser;
    Poco::AutoPtr<Poco::XML::Document> pDoc = parser.parse(&sourceSchema);
    Poco::XML::NodeIterator it_doc(pDoc, Poco::XML::NodeFilter::SHOW_ALL);
    Poco::XML::Node* pNode = it_doc.nextNode();

    while (pNode)
    {
      if ((pNode->nodeName()==sphinx_data_source_const::fieldTag) || (pNode->nodeName()==sphinx_data_source_const::attrTag))
      {
        Poco::AutoPtr<Poco::XML::NamedNodeMap> pMap =  pNode->attributes();
        if (!pMap.isNull())
        {
          for (size_t i=0;i<pMap->length();++i)
            if (pMap->item(i)->nodeName()==sphinx_data_source_const::nameAttrTag)
              names.push_back(pMap->item(i)->nodeValue());
        }
        if (pNode->nodeName()==sphinx_data_source_const::attrTag)
          ++attributesCount;
      }
      pNode = it_doc.nextNode();
    }
    _IsError = false;
  }
  catch(Poco::Exception& e)
  {
    errorMsg = e.message();
    _IsError = true;
  }
  return !_IsError;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace sphinx 
} // namespace HCE 
