#include <algorithm>
#include <ctype.h>

#include "SphinxSearchInterface.hpp"
#include "SphinxError.hpp"
#include "SphinxMessageConst.hpp"

namespace HCE
{
namespace sphinx
{
//-----------------------------------------------------------------------------
SphinxSearchInterface::SphinxSearchInterface(void)
:errorMsg(""), errorCode(NO_ERROR), _isError(false)
{
}
//-----------------------------------------------------------------------------
bool SphinxSearchInterface::insensitiveCompare(const std::string& lhs, const std::string& rhs)
{
  std::string lhsCpy(lhs);
  std::string rhsCpy(rhs);
  std::transform(lhsCpy.begin(), lhsCpy.end(), lhsCpy.begin(), tolower);
  std::transform(rhsCpy.begin(), rhsCpy.end(), rhsCpy.begin(), tolower);
  return (lhsCpy == rhsCpy);
}
//-------------------------------------------------------------------
size_t SphinxSearchInterface::getTimeInterval(timeval& start, timeval& stop)
{
  size_t result = 0;
  if (start.tv_sec == stop.tv_sec)
    result = abs(stop.tv_usec - start.tv_usec);
  else if (start.tv_sec > stop.tv_sec)
  {
    time_t sec = start.tv_sec - stop.tv_sec;
    result = sec*1000000+start.tv_usec - stop.tv_usec;
  }
  else
  {
    time_t sec = stop.tv_sec - start.tv_sec;
    result = sec*1000000+stop.tv_usec - start.tv_usec;
  }
  return result;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace sphinx
} // end namespace HCE




