/**
 * @file SphinxSearchInterface.hpp
 *
 * Basic interface for working with the HCE Sphinx cluster.
 * Contains a general description of the members of the heirs and their methods of common interface.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_SEARCH_INTERFACE_HPP
#define SPHINX_SEARCH_INTERFACE_HPP

#include <iostream>
#include <time.h>
#include <sys/time.h>

namespace HCE
{
namespace sphinx
{
class SphinxInputJsonMessage;
class SphinxOutputJsonMessage;
//-----------------------------------------------------------------------------
namespace sphinx_search_const
{
  const bool searchdStart = true;
  const bool searchdNotStart = false;
  const bool searchdStop = true;
  const bool searchdNotStop = false;

  const int startStopSearchdLimit = 10;
  const std::string defaultHost = "127.0.0.1";
  const unsigned int defaultPort = 9312;
  const unsigned int maxAllowedPort = 10312;
  const std::string searchdListen = "searchd.listen";
  const std::string searchdMaxMatches = "searchd.max_matches";
  const std::string searchdLogName = "searchd.log";
  const std::string searchdLogFile = "searchd.log";
  const std::string searchdPidName = "searchd.pid_file";
  const std::string searchdPidFileName = "searchd";
  const std::string searchdPidFileExt = "pid";
  const std::string searchdQueryLogName = "searchd.query_log";
  const std::string searchdQueryLogFile = "query.log";

  const std::string searchdGoodStatusFlag = "searchd status";

  const std::string indexNameField  = "index";
  const std::string sourceNameField = "source";
  const std::string pathNameField = "path";
  const std::string xmlpipeCommandNameField = "xmlpipe_command";

  const std::string branchPropertyFile = "branch.property";
  const std::string branchConfigFile = "branch.cfg";
  const std::string sphinxTemplateFile = "sphinx.template";
  const std::string sphinxPropertyFile = "sphinx.property";
  const std::string sphinxSchemaFile = "schema.xml";
  const std::string sphinxConfigName = "sphinx.conf";

//  const std::string defaultIndexName = "default_index";
  const std::string defaultBranchName = "default_branch";

  const std::string defaultHomeDir = ".";
  const std::string defaultDataDir = "data";
  const std::string defaultRunDir = "run";
  const std::string defaultLogDir = "log";
  const std::string defaultEtcDir = "etc";

  const std::string moduleName = "Sphinx";
}
//-----------------------------------------------------------------------------
class SphinxSearchInterface
{
public:
  SphinxSearchInterface(void);
  virtual ~SphinxSearchInterface(void) {}

  virtual std::string Process(const std::string& json)=0;
  virtual SphinxOutputJsonMessage Process(SphinxInputJsonMessage&)=0;

  void setErrorMsg(const std::string& errorMsg_) {errorMsg=errorMsg_;}
  std::string getErrorMsg(void) const {return errorMsg;}

  void setErrorCode(unsigned int errorCode_) {errorCode=errorCode_;}
  unsigned int getErrorCode(void) const {return errorCode;}

  void isError(bool isError_) {_isError=isError_;}
  bool isError(void) const {return _isError;}

protected:
  std::string errorMsg;
  unsigned int errorCode;
  bool _isError;
public:
  static bool insensitiveCompare(const std::string& lhs, const std::string& rhs);
  static size_t getTimeInterval(timeval& start, timeval& stop);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace sphinx
} // end namespace HCE

#endif // SPHINX_SEARCH_INTERFACE_HPP
