#include <gtest/gtest.h>
#include <Poco/SharedPtr.h>
#include <fstream>
#include <assert.h>
#include <unistd.h>
#include <iomanip>

#include "SphinxError.hpp"
#include "SphinxManageTest.hpp"
#include "SphinxSearchTest.hpp"
#include "SphinxIndexTest.hpp"
#include "SphinxMultiIndexTest.hpp"

#include "SphinxInputJsonMessageSearch.hpp"
#include "SphinxInputJsonMessage.hpp"
#include "SphinxOutputJsonMessage.hpp"
#include "SphinxFunctionalObject.hpp"
#include "SphinxAdminCommand.hpp"
#include "SphinxStressTest.hpp"

#include "SphinxFunctionalObjectTest.hpp"
#include "SphinxPrintStatus.hpp"

namespace HCE
{
namespace sphinx
{
namespace tests
{
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::fullAdminCommandsTest(SphinxFunctionalObject& fObj)
{
  const std::string indexName = "i003";

  fObj.setCheckIncomingDocuments(true);
  fObj.setPackIncomingDocuments(true);

  SphinxManageTest::testIndexCreate(fObj, indexName);

  SphinxManageTest::testIndexCheck(fObj, indexName, true);
  SphinxManageTest::testIndexCheck(fObj, "branch_0006", false);

  std::vector<std::string> branches;
  /*
  branches.push_back("_branch11");
  branches.push_back("_branch12");
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i]);
  SphinxManageTest::testIndexStoreSchemaFile(fObj, indexName);
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
  */

  branches.clear();
  branches.push_back("branch_0005");
  branches.push_back("branch_0006");
  branches.push_back("branch_0007");
  for (size_t i=0;i<branches.size();++i)
  {
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i], "");
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i]);
  }
  SphinxManageTest::testIndexStoreSchemaFile(fObj, indexName, "");
  SphinxManageTest::testIndexStoreSchemaFile(fObj, indexName);
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);

  SphinxManageTest::testIndexMaxDocId(fObj, indexName);
  SphinxManageTest::testIndexMaxDocIdExtended(fObj, indexName);

  for (size_t i=0;i<branches.size();++i)
  {
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i]);
  }
  branches.push_back("branch_0008");
  branches.push_back("branch_0009");
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);

  branches.clear();
  branches.push_back("*");
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
//  SphinxManageTest::testIndexSetDataDir(fObj);    // can use with or without it

  SphinxManageTest::testIndexMerge(fObj, indexName, branches);

  SphinxManageTest::testIndexStart(fObj, indexName);

  branches.clear();
  branches.push_back("branch_0005");
  SphinxManageTest::testIndexDeleteDataFile(fObj, indexName, branches);

  branches.clear();
  branches.push_back("branch_0005");
  branches.push_back("branch_0006");
  branches.push_back("branch_0007");
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
  SphinxManageTest::testIndexStart(fObj, indexName);

  branches.clear();
  branches.push_back("branch_0005");
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
  branches.push_back("branch_0006");
  SphinxManageTest::testIndexMergeTrunk(fObj, indexName, branches);
  branches.push_back("branch_0007");
  SphinxManageTest::testIndexMergeTrunk(fObj, indexName, branches);
  branches.push_back("branch_0008");  // not exists
  branches.push_back("branch_0009");  // not exists
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
  SphinxManageTest::testIndexStart(fObj, indexName);
  SphinxManageTest::testIndexStop(fObj, indexName);
  SphinxManageTest::testIndexStart(fObj, indexName);

  SphinxManageTest::testIndexDeleteDataFile(fObj, indexName, branches);
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i]);
  branches.clear();
  branches.push_back("*");
  SphinxManageTest::testIndexDeleteDataFile(fObj, indexName, branches);
  SphinxManageTest::testIndexDeleteSchemaFile(fObj, indexName);

  branches.clear();
  branches.push_back("branch_0005");
  branches.push_back("branch_0006");
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i]);
  SphinxManageTest::testIndexStoreSchemaFile(fObj, indexName);
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexAppendDataFile(fObj, indexName, branches[i]);

  std::vector<unsigned long long> ids = { 123456789012, 123456789023, 123456789034, 123456789045, 123456789056, 123456789067, 123456789078, 123456789089, 123456789090, 1234567890101};
  SphinxManageTest::testIndexDeleteDoc(fObj, indexName, ids);
  SphinxManageTest::testIndexDeleteDocNumber(fObj, indexName);

  branches.clear();
  branches.push_back("branch_0005");
  branches.push_back("branch_0006");
  branches.push_back("branch_0007");
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i]);

  SphinxManageTest::testIndexDeleteDocNumber(fObj, indexName);
  SphinxManageTest::testIndexPackDocData(fObj, indexName, branches);

  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches); ////
  branches.clear();
  branches.push_back("*");
  SphinxManageTest::testIndexPackDocData(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches); ////

  SphinxManageTest::testIndexRemove(fObj, indexName);
  SphinxManageTest::testIndexCheck(fObj, indexName, false);

  // prepare for next copy index
  SphinxManageTest::testIndexCreate(fObj, indexName);
  SphinxManageTest::testIndexCheck(fObj, indexName, true);
  branches.clear();
  branches.push_back("branch_0005");
  branches.push_back("branch_0006");
  branches.push_back("branch_0007");
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i]);
  SphinxManageTest::testIndexStoreSchemaFile(fObj, indexName);
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
  // make copy index
  const std::string copyIndexName = "i004";
  SphinxManageTest::testIndexStart(fObj, indexName);
  SphinxManageTest::testIndexStop(fObj, indexName);
  SphinxManageTest::testIndexStart(fObj, indexName);
  SphinxManageTest::testIndexCopy(fObj, indexName, copyIndexName);
  SphinxManageTest::testIndexCheck(fObj, copyIndexName, true);

  SphinxManageTest::testIndexStart(fObj, copyIndexName);

  // make rename index
  const std::string renameIndexName = "i005";
  SphinxManageTest::testIndexRename(fObj, copyIndexName, renameIndexName);
  SphinxManageTest::testIndexCheck(fObj, copyIndexName, false);
  fObj.resetError();
  SphinxManageTest::testIndexCheck(fObj, renameIndexName, true);

  SphinxManageTest::testIndexStart(fObj, renameIndexName);
  // make remove index
  SphinxManageTest::testIndexRemove(fObj, renameIndexName);
  SphinxManageTest::testIndexCheck(fObj, renameIndexName, false);
  fObj.resetError();

  // set config property value
  SphinxManageTest::testIndexSetConfigVar(fObj, indexName, "", "searchd", "test_param", "test_value1");
  SphinxManageTest::testIndexSetConfigVar(fObj, indexName, "", "index branch_0001", "test_param", "test_value2");
  SphinxManageTest::testIndexSetConfigVar(fObj, indexName, "branch_0005", "searchd", "test_param", "test_value1");
  SphinxManageTest::testIndexSetConfigVar(fObj, indexName, "branch_0005", "index branch_0001", "test_param", "test_value2");

  // get config property value
  std::string paramValue;
  SphinxManageTest::testIndexGetConfigVar(fObj, indexName, "", "searchd", "test_param", paramValue);
  SphinxManageTest::testIndexGetConfigVar(fObj, indexName, "", "index branch_0001", "test_param", paramValue);
  SphinxManageTest::testIndexGetConfigVar(fObj, indexName, "branch_0005", "searchd", "test_param", paramValue);
  SphinxManageTest::testIndexGetConfigVar(fObj, indexName, "branch_0005", "index branch_0001", "test_param", paramValue);

  // check validation of schema data and save in index if valid
  SphinxManageTest::testIndexDeleteSchemaFile(fObj, indexName);//////////////////////

  // recreate index for test
  SphinxManageTest::testIndexRemove(fObj, indexName);
  SphinxManageTest::testIndexCheck(fObj, indexName, false);
  fObj.resetError();
  SphinxManageTest::testIndexCreate(fObj, indexName);
  SphinxManageTest::testIndexCheck(fObj, indexName, true);
  // get data about status of Sphinx searchd
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i]);
  SphinxManageTest::testIndexStoreSchemaFile(fObj, indexName);

  SphinxManageTest::testIndexCheckSchema(fObj, indexName);

  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
  SphinxManageTest::testIndexStart(fObj, indexName);

  SphinxManageTest::testIndexStatusSearchd(fObj, indexName);
  SphinxManageTest::testIndexStatus(fObj, indexName);
  SphinxManageTest::testIndexDataList(fObj, indexName);
  SphinxManageTest::testIndexBranchesList(fObj, indexName);
  SphinxManageTest::testIndexBranchesInfo(fObj, indexName, branches);
  SphinxManageTest::testIndexBranchesStatus(fObj, indexName, branches);

  SphinxManageTest::testIndexConnect(fObj, indexName);
  SphinxManageTest::testIndexDisconnect(fObj);
  SphinxManageTest::testIndexDisconnect(fObj);
  SphinxManageTest::testIndexConnect(fObj, indexName);
  SphinxManageTest::testIndexConnect(fObj, indexName);

  SphinxManageTest::testIndexRemove(fObj, indexName);
}
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::changeSearchIndexTest(SphinxFunctionalObject& fObj)
{
  const std::string indexName = "i003";

  // create first index
  SphinxManageTest::testIndexCreate(fObj, indexName);
  SphinxManageTest::testIndexCheck(fObj, indexName, true);
  std::vector<std::string> branches = {"branch_0005"};
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i]);
  SphinxManageTest::testIndexStoreSchemaFile(fObj, indexName);
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
  SphinxManageTest::testIndexStart(fObj, indexName);
  std::cout << "IndexStart >>>>>> isActiveSearchd = " << std::boolalpha << fObj.isActiveSearchd() << std::endl;
  SphinxSearchTest::testSearcherProcess(fObj);
  // make second index
  const std::string newIndexName = "i004";
  SphinxManageTest::testIndexCopy(fObj, indexName, newIndexName);
  SphinxManageTest::testIndexStart(fObj, newIndexName);
  std::cout << "IndexStart >>>>>> isActiveSearchd = " << std::boolalpha << fObj.isActiveSearchd() << std::endl;
  SphinxSearchTest::testSearcherProcess(fObj);
  SphinxManageTest::testIndexRemove(fObj, newIndexName);
  SphinxManageTest::testIndexRemove(fObj, indexName);
}
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::autoGettingPortsSearchd(SphinxFunctionalObject& fObj)
{
  const std::string nodeName = "test_node";

  const unsigned int SZ = 3;
  const std::string indexName[SZ] = {"i003", "i004" , "i005"};
  Poco::SharedPtr<SphinxFunctionalObject> pObj[SZ];

  for (unsigned int i=0;i<SZ;++i)
  {
    pObj[i] = new SphinxFunctionalObject(nodeName, fObj.getHomeDir());

    SphinxManageTest::testIndexCreate(*(pObj[i]), indexName[i]);
    SphinxManageTest::testIndexCheck(*(pObj[i]), indexName[i], true);
    std::vector<std::string> branches = {"branch_0005"};
    for (size_t k=0;k<branches.size();++k)
      SphinxManageTest::testIndexStoreDataFile(*(pObj[i]), indexName[i], branches[k]);
    SphinxManageTest::testIndexStoreSchemaFile(*(pObj[i]), indexName[i]);
    SphinxManageTest::testIndexRebuild(*(pObj[i]), indexName[i], branches);
//    SphinxManageTest::testIndexStart(*(pObj[i]), indexName[i]);
    SphinxManageTest::testIndexMerge(*(pObj[i]), indexName[i], branches);  // TODO
    SphinxManageTest::testIndexStart(*(pObj[i]), indexName[i]);
//    SphinxManageTest::testIndexStatus(*(pObj[i]), indexName[i]);
    SphinxSearchTest::testSearcherProcess(*(pObj[i]));
  }
  for (unsigned int i=0;i<SZ;++i)
    SphinxManageTest::testIndexRemove(*(pObj[i]), indexName[i]);

  for (unsigned int i=0;i<SZ;++i)
    ASSERT_FALSE(pObj[i]->isError());
}
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::fullIndexerTest(SphinxFunctionalObject& fObj)
{
  SphinxIndexTest::testDataStorage();
  SphinxIndexTest::testInputJsonMessage();
  SphinxIndexTest::testIndexerProcess(fObj);
}
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::fullSearcherTest(SphinxFunctionalObject& fObj)
{
  const std::string indexName = "i003";

  // create first index
  SphinxManageTest::testIndexCreate(fObj, indexName);
  SphinxManageTest::testIndexCheck(fObj, indexName, true);
  std::vector<std::string> branches = {"branch_0005"};
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i]);
  SphinxManageTest::testIndexStoreSchemaFile(fObj, indexName);
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
  SphinxManageTest::testIndexStart(fObj, indexName);
  ASSERT_TRUE(fObj.isActiveSearchd());

  SphinxSearchTest::testWeightPacker();
  SphinxSearchTest::testWeightCalculator();

  SphinxSearchTest::testSerialization();
  SphinxSearchTest::testSerializationExt();
  SphinxSearchTest::testMemoryManager();
  SphinxSearchTest::testSearcherProcess(fObj);
  SphinxSearchTest::testFilterSearch(fObj);

  SphinxManageTest::testIndexRemove(fObj, indexName);
  SphinxFunctionalObjectTest::testSerializationSearchProcess();
}
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::testSerializationSearchProcess(void)
{
  std::string json =
      "{\"type\":\"0\",\"data\":\"{\\\"q\\\":\\\"post\\\",\\\"filters\\\":\\\"[{\\\\\\\"type\\\\\\\":0,\\\\\\\"attribute\\\\\\\":\\\\\\\"afflags7ex\\\\\\\",\\\\\\\"values\\\\\\\":[\\\\\\\"141665\\\\\\\"],\\\\\\\"exclude\\\\\\\":\\\\\\\"0\\\\\\\"}]\\\",\\\"max_results\\\":10,\\\"sort_mode\\\":1,\\\"parameters\\\":[{\\\"queryId\\\":\\\"2829799777\\\"},{\\\"JsonType\\\":7},{\\\"sort_by\\\":\\\"weight\\\"}]}\"}";
  SphinxSearchTest::testSerializationSearchProcess(json);

  json = "{\"type\":\"0\",\"data\":\"{\\\"q\\\":\\\"post\\\",\\\"filters\\\":\\\"[]\\\",\\\"max_results\\\":10,\\\"sort_mode\\\":1,\\\"parameters\\\":[{\\\"queryId\\\":\\\"2829799777\\\"},{\\\"JsonType\\\":7},{\\\"sort_by\\\":\\\"weight\\\"}]}\"}";
  SphinxSearchTest::testSerializationSearchProcess(json);

  json = "{\"type\":\"0\",\"data\":\"{\\\"q\\\":\\\"post\\\",\\\"filters\\\":\\\"[]\\\",\\\"max_results\\\":10,\\\"sort_mode\\\":1,\\\"parameters\\\":[]}\"}";
  SphinxSearchTest::testSerializationSearchProcess(json);

  json = "{\"type\":\"0\",\"data\":\"{\\\"query\\\":\\\"post\\\",\\\"filters\\\":\\\"[]\\\",\\\"sort_mode\\\":1,\\\"parameters\\\":[]}\"}";
  SphinxSearchTest::testSerializationSearchProcess(json);

  json = "{\"type\":\"0\",\"data\":\"{\\\"filters\\\":\\\"[]\\\",\\\"parameters\\\":[]}\"}";
  SphinxSearchTest::testSerializationSearchProcess(json);
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::initFilters(SphinxFilter::FilterType filterType, std::vector<Poco::SharedPtr<SphinxFilter> >& filters)
{
  filters.clear();
  Poco::SharedPtr<SphinxFilter> pFilter = new SphinxFilter;

  if (filterType==SphinxFilter::FilterType::ftRangeFloat)
  {
    pFilter->setFilterType(filterType);
    pFilter->setAttributeName("uid");
    pFilter->setExcludeValue(SphinxFilter::ExcludeType::etFalse);
    pFilter->addAttributeValue("1.9");
    pFilter->addAttributeValue("2.1");
    filters.push_back(pFilter);
  }
  else
  {
    pFilter->setFilterType(filterType);
    pFilter->setAttributeName("afflags4");
    pFilter->setExcludeValue(SphinxFilter::ExcludeType::etFalse);
    pFilter->addAttributeValue("4");
    pFilter->addAttributeValue("8");
    filters.push_back(pFilter);

    pFilter = new SphinxFilter;
    pFilter->setFilterType(filterType);
    pFilter->setAttributeName("afflags4ex");
    pFilter->setExcludeValue(SphinxFilter::ExcludeType::etFalse);
    pFilter->addAttributeValue("16");
    pFilter->addAttributeValue("64");
    filters.push_back(pFilter);
  }
}
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::makeSearchFilter(SphinxFunctionalObject& fObj, SphinxFilter::FilterType filterType, size_t repeatCount)
{
  const std::string query = "@bodys ceo";
  std::vector<Poco::SharedPtr<SphinxFilter> > filters;
  Poco::SharedPtr<SphinxResultData> resultData[repeatCount];
  Poco::SharedPtr<SphinxResultDataSerializator> resultSerializator[repeatCount];

  for (size_t i=0;i<repeatCount;++i)
  {
    resultData[i] = new SphinxResultData();
    resultSerializator[i] = new SphinxResultDataSerializator(*(resultData[i]));
  }
  // initialisation of filter early set values
  initFilters(filterType, filters);
  // testing search with filters
  for (size_t i=0;i<repeatCount;++i)
    ASSERT_TRUE(resultSerializator[i]->unserialize(SphinxSearchTest::searchWithFilter(fObj, query, filters).getData()));

  // check results of search
  for (size_t i=1;i<repeatCount;++i)
  {
    const size_t matchInfoCountPrev = resultData[i-1]->getMatchInfoCount();
    const size_t matchInfoCountNext = resultData[i]->getMatchInfoCount();

    ASSERT_EQ(matchInfoCountPrev, matchInfoCountNext);
    for (size_t k=0;k<matchInfoCountPrev;++k)
    {
      const SphinxMatchInfo& matchInfoPrev = resultData[i-1]->getMatchInfoItem(k);
      const SphinxMatchInfo& matchInfoNext = resultData[i]->getMatchInfoItem(k);

      ASSERT_EQ(matchInfoPrev.getDocId(), matchInfoNext.getDocId());
      ASSERT_EQ(matchInfoPrev.getWeight(), matchInfoNext.getWeight());

      const size_t attributesCountPrev = matchInfoPrev.getAttributesCount();
      const size_t attributesCountNext = matchInfoNext.getAttributesCount();

      ASSERT_EQ(attributesCountPrev, attributesCountNext);
      for (size_t j=0;j<attributesCountPrev;++j)
      {
        const AttrInfo& attrInfoPrev = matchInfoPrev.getAttrItem(j);
        const AttrInfo& attrInfoNext = matchInfoNext.getAttrItem(j);

        ASSERT_EQ(attrInfoPrev.name, attrInfoNext.name);
        ASSERT_EQ(attrInfoPrev.value, attrInfoNext.value);
      }
    }
  }
}
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::differentFiltersTest(SphinxFunctionalObject& fObj)
{
  std::stringstream  dataContent;
  dataContent << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<sphinx:docset xmlns:sphinx=\"http://sphinxsearch.com\">\n<sphinx:document id=\"9223372490088244475\">"
      << "<title><![CDATA[Mars One opens applications for one-way ticket to Mars]]></title>\n"
      << "<h><![CDATA[<a href='http://cdni.wired.co.uk/620x413/k_n/Mars_7.jpg' target=_blank><img src='http://cdni.wired.co.uk/620x413/k_n/Mars_7.jpg' title='size: 0 width: 0 height: 0'></a>one-way ticket\nMars\n∧one∧way∧ticket\n∧Mars http://wired.co.uk/news/archive/2013-04/25/mars-one-applications ]]></h>\n"
      << "<alt><![CDATA[2,8,32,128]]></alt>\n"
      << "<bodys><![CDATA[It's time, if you're so inclined, to plunk down a reservation\nfor a one-way ticket to Mars. The privately funded Mars One\n"
      << "foundation recently opened up applications for astronauts to take a\njourney to the Red Planet in 2022/23. Return trips, the organisers\n"
      << "said, just aren't feasible with the technology we have.\nFrom a wide field of applicants, organisers said two women and\ntwo men will be selected to make the journey. The technology that\n"
      << "takes them there has been tested before, and the potential for a\nworldwide audience will be there given four billion people will\nhave the internet by then, said Mars One CEO Bas Lansdorp.\n"
      << "By: Elizabeth Howell, SEN.com, Edited by: Kadhim ShubberContinue reading...]]></bodys>\n"
      << "<bodyn><![CDATA[10101010]]></bodyn>\n<meta><![CDATA[1191]]></meta>\n<ref><![CDATA[93]]></ref>\n"
      << "<url><![CDATA[http://api.snatz.com/index.php?app=hceapi&event=feeds&time=new&feed_id=11250311753507343583&item_id=14946343666313105260-13-04]]></url>\n"
      << "<cdate>1373619775</cdate>\n<mmedia>0</mmedia>\n<bseo>0</bseo>\n<uid>2</uid>\n<sid>1</sid>\n<lang>255</lang>\n"
      << "<afflags0>2,8,32,128</afflags0>\n<afflags1>24</afflags1>\n<afflags2>1</afflags2>\n<afflags3>2,8,32,128</afflags3>\n"
      << "<afflags4>2,8,32,128</afflags4>\n<afflags5>2,8,32,128</afflags5>\n<afflags6>2,8,32,128</afflags6>\n<afflags7>2,8,32,128</afflags7>\n\n"
      << "<afflags0ex>1,4,16,64</afflags0ex>\n<afflags1ex>1,4,16,64</afflags1ex>\n<afflags2ex>1,4,16,64</afflags2ex>\n<afflags3ex>1,4,16,64</afflags3ex>\n"
      << "<afflags4ex>1,4,16,64</afflags4ex>\n<afflags5ex>1,4,16,64</afflags5ex>\n<afflags6ex>1,4,16,64</afflags6ex>\n<afflags7ex>1,4,16,64</afflags7ex>\n"
      << "</sphinx:document>\n</sphinx:docset>\n";

  const std::string indexName = "i003";

  // create index
  SphinxManageTest::testIndexCreate(fObj, indexName);
  SphinxManageTest::testIndexCheck(fObj, indexName, true);
  std::vector<std::string> branches = {"branch_0005"};
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i], dataContent.str());
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
  SphinxManageTest::testIndexStart(fObj, indexName);

//  fObj.setLogStream(std::cout);

  const size_t testRepeat = 3;

  SphinxFunctionalObjectTest::makeSearchFilter(fObj, SphinxFilter::FilterType::ftSimple, testRepeat);
  ASSERT_FALSE(fObj.isError());
  printSuccess("Search Filter ftSimple");

  SphinxFunctionalObjectTest::makeSearchFilter(fObj, SphinxFilter::FilterType::ftRangeFloat, testRepeat);
  ASSERT_FALSE(fObj.isError());
  printSuccess("Search Filter ftRangeFloat");

  SphinxFunctionalObjectTest::makeSearchFilter(fObj, SphinxFilter::FilterType::ftRangeInteger, testRepeat);
  ASSERT_FALSE(fObj.isError());
  printSuccess("Search Filter ftRangeInteger");

  SphinxFunctionalObjectTest::makeSearchFilter(fObj, SphinxFilter::FilterType::ftSimpleMulti, testRepeat);
  ASSERT_FALSE(fObj.isError());
  printSuccess("Search Filter ftSimpleMulti");

  SphinxManageTest::testIndexRemove(fObj, indexName);
  ASSERT_FALSE(fObj.isError());
  printSuccess("Test search with different filters");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
bool SphinxFunctionalObjectTest::isUse64Bit(SphinxFunctionalObject& fObj)
{
  bool result = false;
  const unsigned long long docId = 9223372490088244475ULL;

  std::stringstream  dataContent;
  dataContent << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<sphinx:docset xmlns:sphinx=\"http://sphinxsearch.com\">\n<sphinx:document id=\""<<docId<<"\">"
      << "<title><![CDATA[Mars One opens applications for one-way ticket to Mars]]></title>\n"
      << "<h><![CDATA[<a href='http://cdni.wired.co.uk/620x413/k_n/Mars_7.jpg' target=_blank><img src='http://cdni.wired.co.uk/620x413/k_n/Mars_7.jpg' title='size: 0 width: 0 height: 0'></a>one-way ticket\nMars\n∧one∧way∧ticket\n∧Mars http://wired.co.uk/news/archive/2013-04/25/mars-one-applications ]]></h>\n"
      << "<alt><![CDATA[2,8,32,128]]></alt>\n"
      << "<bodys><![CDATA[It's time, if you're so inclined, to plunk down a reservation\nfor a one-way ticket to Mars. The privately funded Mars One\n"
      << "foundation recently opened up applications for astronauts to take a\njourney to the Red Planet in 2022/23. Return trips, the organisers\n"
      << "said, just aren't feasible with the technology we have.\nFrom a wide field of applicants, organisers said two women and\ntwo men will be selected to make the journey. The technology that\n"
      << "takes them there has been tested before, and the potential for a\nworldwide audience will be there given four billion people will\nhave the internet by then, said Mars One CEO Bas Lansdorp.\n"
      << "By: Elizabeth Howell, SEN.com, Edited by: Kadhim ShubberContinue reading...]]></bodys>\n"
      << "<bodyn><![CDATA[10101010]]></bodyn>\n<meta><![CDATA[1191]]></meta>\n<ref><![CDATA[93]]></ref>\n"
      << "<url><![CDATA[http://api.snatz.com/index.php?app=hceapi&event=feeds&time=new&feed_id=11250311753507343583&item_id=14946343666313105260-13-04]]></url>\n"
      << "<cdate>1373619775</cdate>\n<mmedia>0</mmedia>\n<bseo>0</bseo>\n<uid>2</uid>\n<sid>1</sid>\n<lang>255</lang>\n"
      << "<afflags0>2,8,32,128</afflags0>\n<afflags1>24</afflags1>\n<afflags2>1</afflags2>\n<afflags3>2,8,32,128</afflags3>\n"
      << "<afflags4>2,8,32,128</afflags4>\n<afflags5>2,8,32,128</afflags5>\n<afflags6>2,8,32,128</afflags6>\n<afflags7>2,8,32,128</afflags7>\n\n"
      << "<afflags0ex>1,4,16,64</afflags0ex>\n<afflags1ex>1,4,16,64</afflags1ex>\n<afflags2ex>1,4,16,64</afflags2ex>\n<afflags3ex>1,4,16,64</afflags3ex>\n"
      << "<afflags4ex>1,4,16,64</afflags4ex>\n<afflags5ex>1,4,16,64</afflags5ex>\n<afflags6ex>1,4,16,64</afflags6ex>\n<afflags7ex>1,4,16,64</afflags7ex>\n"
      << "</sphinx:document>\n</sphinx:docset>\n";

  const std::string query = "Mars";
  const std::string indexName = "i003";

  // create index
  SphinxManageTest::testIndexCreate(fObj, indexName);
  SphinxManageTest::testIndexCheck(fObj, indexName, true);

  std::vector<std::string> branches = {"branch_"+indexName};
  for (size_t i=0;i<branches.size();++i)
    SphinxManageTest::testIndexStoreDataFile(fObj, indexName, branches[i], dataContent.str());
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
  SphinxManageTest::testIndexStart(fObj, indexName);

  // test search document
  std::string json;
  SphinxInputJsonMessageSearch inputJsonMessageSearch;
  inputJsonMessageSearch.setQueryString(query);
  inputJsonMessageSearch.addQueryParameters("queryId", "1");
  inputJsonMessageSearch.addQueryParameters("jsonType", "1");
  inputJsonMessageSearch.addQueryParameters("max_results", "100");

  bool ret = inputJsonMessageSearch.serialize(json);
  fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "SphinxInputJsonMessageSearch serialize => " << std::boolalpha << ret << std::endl;
  assert(ret);
  printSuccess("SphinxInputJsonMessageSearch serialize");

  SphinxInputJsonMessage inputJsonMessage;
  inputJsonMessage.setType(SphinxInputJsonMessage::MessageType::mtSearch);
  inputJsonMessage.setData(json);

  SphinxOutputJsonMessage outputMessage = fObj.Process(inputJsonMessage);
  if (!fObj.getErrorMsg().empty())
    fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "fObj.getErrorMsg()  = " << fObj.getErrorMsg() << std::endl;
  if (fObj.getErrorCode()!=NO_ERROR)
    fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "fObj.getErrorCode() = " << fObj.getErrorCode() << std::endl;
  if (outputMessage.isError())
    fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "outputMessage.isError() = " << outputMessage.isError() << std::endl;

  assert(!fObj.isError());
  assert(fObj.getErrorCode()==NO_ERROR);
  assert(fObj.getErrorMsg().empty());

  SphinxResultData resultData;
  SphinxResultDataSerializator resultSerializator(resultData);

  ret = resultSerializator.unserialize(outputMessage.getData());
  fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "SphinxResultData unserialize => " << std::boolalpha << ret << std::endl;
  assert(ret);
  printSuccess("SphinxResultData unserialize");

  assert(resultData.getMatchInfoCount()==1);
  unsigned long long retDocId = resultData.getMatchInfoItem(0).getDocId();

  result = (retDocId==docId);
  std::cout << "retDocId = " << retDocId << std::endl;
  std::cout << "docId = " << docId << std::endl;

  fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "Use " << ((result)?"64":"32") << " Bit for document ID" << std::endl;

  SphinxManageTest::testIndexRemove(fObj, indexName);
  printSuccess("Test Use 32 or 64 Bit");

  return result;
}
//-----------------------------------------------------------------------------
float SphinxFunctionalObjectTest::testPerfomance(const std::string& homeDir, size_t indexCount, const std::string& dataFileName, std::vector<std::string>& queries, bool multiIndex)
{
  float fps = 0.0;
  std::vector<std::string> indexNames;
  // create indexes
  SphinxMultiIndexTest::testCreateIndexes(homeDir, indexCount, dataFileName, indexNames);
  // search use already created indexes
//   SphinxMultiIndexTest::makeIndexNames(indexCount, indexNames);

  if (multiIndex)
  {
    fps = SphinxMultiIndexTest::testSearchIndexes(indexNames, queries);
  }
  else
  {
    std::string indexName = indexNames.front();
    fps = SphinxMultiIndexTest::testSearchIndexes(homeDir, indexNames, queries, indexName);
  }
  // remove indexes
  SphinxMultiIndexTest::testRemoveIndexes(homeDir, indexNames);
  return fps;
}
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::testPerfomance(const std::string& homeDir, const std::string& dataFileName, const size_t indexCount)
{
  std::vector<std::string> queries=
  {"seo", "ISO", "with", "LibreOffice", "is", "free", "and", "open", "source", "office", "suite", "for", "Linux", "Mac", "Windows", "computers", "Forest", "Key", "Seattle", "MGM"};

  float multiIndexFps = SphinxFunctionalObjectTest::testPerfomance(homeDir, indexCount, dataFileName, queries, true);
  float singleIndexFps = SphinxFunctionalObjectTest::testPerfomance(homeDir, indexCount, dataFileName, queries, false);

  std::stringstream outMsg;
  outMsg << "Single index search perfomance: " << singleIndexFps << " fps." << std::endl;
  const size_t length = outMsg.str().length();
  std::cout << std::string(length, '=') << "\n";
  std::cout << "Multi index search perfomance: " << multiIndexFps << " fps." << std::endl;
  std::cout << outMsg.str() << std::string(length, '=') << std::endl;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::createBigIndex(const std::string& homeDir, const std::string& sourceTestDir, const std::string& indexName)
{
  SphinxFunctionalObject fObj("node", 2005, homeDir, "");
  std::string errorMsg;
  std::vector<std::string> dataFiles;
  bool ret = SphinxStressTest::makeDataFilesList(sourceTestDir, dataFiles, errorMsg);
  if (!ret)
    std::cout << "Error: " << errorMsg << std::endl;
  ASSERT_TRUE(ret);

  for (const auto& name: dataFiles)
    std::cout << name << std::endl;

  SphinxManageTest::testIndexCreate(fObj, indexName);
  SphinxManageTest::testIndexCheck(fObj, indexName, true);

  ret = SphinxStressTest::moveDataFiles(sourceTestDir, fObj.getDataDir()+"/"+indexName+"/"+sphinx_admin_command_const::dataDir, dataFiles, errorMsg);
  if (!ret)
    std::cout << "Error: " << errorMsg << std::endl;
  ASSERT_TRUE(ret);

  std::vector<std::string> branches = {"*"};
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  SphinxManageTest::testIndexMerge(fObj, indexName, branches);
}
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::testStressGetBigResult(SphinxFunctionalObject& fObj, const std::string& sourceTestDir,
    const std::string& indexName, bool needMakeIndex, bool needRemoveIndex)
{
  if (needMakeIndex)
    SphinxFunctionalObjectTest::createBigIndex(fObj.getHomeDir(), sourceTestDir, indexName);

  std::string paramValue;
  SphinxManageTest::testIndexGetConfigVar(fObj, indexName, "", "searchd", "max_matches", paramValue);
  SphinxManageTest::testIndexSetConfigVar(fObj, indexName, "", "searchd", "max_matches", "100000");

  const std::string query = "и";

  SphinxManageTest::testIndexStart(fObj, indexName);

  size_t maxReturnedDocs = 0;
  const size_t maxCount = 10000;
  for (size_t i=1000;i<maxCount;i+=1000)
  {
    printMessage("Try get "+std::to_string(i)+" documents");
    SphinxSearchTest::testSearcherProcess(fObj, query, i);
    if (fObj.isError())
    {
      maxReturnedDocs = i-1000;
      fObj.resetError();
      for (size_t j=maxReturnedDocs+100;j<maxCount;j+=100)
      {
        printMessage("Try get "+std::to_string(j)+" documents");
        SphinxSearchTest::testSearcherProcess(fObj, query, j);
        if (fObj.isError())
        {
          fObj.resetError();
          break;
        }
        else
          maxReturnedDocs = j;
      }
      break;
    }
  }

  SphinxManageTest::testIndexStop(fObj, indexName);
  SphinxManageTest::testIndexSetConfigVar(fObj, indexName, "", "searchd", "max_matches", paramValue);

  if (needRemoveIndex)
    SphinxManageTest::testIndexRemove(fObj, indexName);

  printMessage("MAX RETURNED DOCS: "+std::to_string(maxReturnedDocs));
  printSuccess("Stress test get max result");
}
//-----------------------------------------------------------------------------
void SphinxFunctionalObjectTest::testGetResultUseOffset(SphinxFunctionalObject& fObj, const std::string& sourceTestDir,
    const std::string& indexName, bool needMakeIndex, bool needRemoveIndex)
{
  if (needMakeIndex)
    SphinxFunctionalObjectTest::createBigIndex(fObj.getHomeDir(), sourceTestDir, indexName);

  std::string paramValue;
  SphinxManageTest::testIndexGetConfigVar(fObj, indexName, "", "searchd", "max_matches", paramValue);
  SphinxManageTest::testIndexSetConfigVar(fObj, indexName, "", "searchd", "max_matches", "500000"); // size must be over then documents count in index

  const std::string query = ""; // must be empty
  const unsigned int maxResults = 1000;
  const unsigned int offset = 999;
  SphinxManageTest::testIndexStart(fObj, indexName);
  SphinxSearchTest::testGetResultUseOffset(fObj, query, maxResults, offset);
  SphinxManageTest::testIndexStop(fObj, indexName);
  SphinxManageTest::testIndexSetConfigVar(fObj, indexName, "", "searchd", "max_matches", paramValue);

  if (needRemoveIndex)
    SphinxManageTest::testIndexRemove(fObj, indexName);

  printSuccess("Stress test get result use offset");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace tests
} // namespace sphinx 
} // namespace HCE 
