/**
 * @file SphinxFunctionalObjectTest.hpp
 *
 * Implementation tests of a functional object sphinx in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_FUNCTIONAL_OBJECT_TEST_HPP
#define SPHINX_FUNCTIONAL_OBJECT_TEST_HPP

namespace HCE
{
namespace sphinx
{
namespace tests
{
//-----------------------------------------------------------------------------
class SphinxFunctionalObjectTest
{
  static void initFilters(SphinxFilter::FilterType filterType, std::vector<Poco::SharedPtr<SphinxFilter> >& filters);
  static void makeSearchFilter(SphinxFunctionalObject& fObj, SphinxFilter::FilterType filterType, size_t repeatCount);
  static void createBigIndex(const std::string& homeDir, const std::string& sourceTestDir, const std::string& indexName);
public:
  static bool isUse64Bit(SphinxFunctionalObject& fObj);   // for check usable 64 bit document ID (return true if use 64 bit, but false if use 32 bit).

  static void fullAdminCommandsTest(SphinxFunctionalObject& fObj);
  static void changeSearchIndexTest(SphinxFunctionalObject& fObj);
  static void autoGettingPortsSearchd(SphinxFunctionalObject& fObj);

  static void fullIndexerTest(SphinxFunctionalObject& fObj);
  static void fullSearcherTest(SphinxFunctionalObject& fObj);
  static void differentFiltersTest(SphinxFunctionalObject& fObj);
  static void testSerializationSearchProcess(void);

  static float testPerfomance(const std::string& homeDir, size_t indexCount, const std::string& dataFileName, std::vector<std::string>& queries, bool multiIndex=true);
  static void testPerfomance(const std::string& homeDir, const std::string& dataFileName, const size_t indexCount=10);

  static void testStressGetBigResult(SphinxFunctionalObject& fObj, const std::string& sourceTestDir,
      const std::string& indexName, bool needMakeIndex = false, bool needRemoveIndex = false);
  static void testGetResultUseOffset(SphinxFunctionalObject& fObj, const std::string& sourceTestDir,
      const std::string& indexName, bool needMakeIndex = false, bool needRemoveIndex = false);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
struct TestPerfomanceData
{
  std::string dataFileName;
  size_t indexCount;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace tests
} // namespace sphinx 
} // namespace HCE 

#endif // SPHINX_FUNCTIONAL_OBJECT_TEST_HPP
