#include <gtest/gtest.h>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <set>

#include <Poco/SharedPtr.h>
#include "SphinxTestPrinter.hpp"
#include "SphinxError.hpp"
#include "SphinxFunctionalObject.hpp"
#include "SphinxFunctionalObjectTest.hpp"

namespace HCE
{
namespace sphinx
{
namespace tests
{
//-----------------------------------------------------------------------------
extern Poco::SharedPtr<SphinxFunctionalObject> pObj;
extern std::string homeDir;
extern TestPerfomanceData perfomanceData;

const std::string nodeName = "test_node";
const std::string defaultIndex = "i001";
const std::string indexName = "i003";
const std::string copyIndexName = "i004";
const std::string renameIndexName = "i005";
//-----------------------------------------------------------------------------
static std::string printError(SphinxFunctionalObject& fObj)
{
  std::ostringstream outMsg;
  outMsg << "ErrorCode: " << fObj.getErrorCode() << " ErrorMsg: " << fObj.getErrorMsg() << std::endl;
  return outMsg.str();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class SphinxFunctionalTest : public ::testing::Test
{
protected:
  static void SetUpTestCase(void)
  {
    pObj = Poco::SharedPtr<SphinxFunctionalObject>(new SphinxFunctionalObject(nodeName, 234, homeDir));
//    pObj->setLogStream(std::cout);
  }
  static void TearDownTestCase(void)
  {
    pObj = nullptr;
  }
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
TEST_F(SphinxFunctionalTest, isCreatedSphinxFunctionalObject)
{
  ASSERT_FALSE(pObj.isNull());
}
//-----------------------------------------------------------------------------
TEST_F(SphinxFunctionalTest, changeSearchIndex)
{
  SphinxFunctionalObjectTest::changeSearchIndexTest(*pObj);
  ASSERT_FALSE(pObj->isError()) << printError(*pObj);
}
//-----------------------------------------------------------------------------
TEST_F(SphinxFunctionalTest, autoGettingPortsSearchd)
{
  SphinxFunctionalObjectTest::autoGettingPortsSearchd(*pObj);
  ASSERT_FALSE(pObj->isError()) << printError(*pObj);
}
//-----------------------------------------------------------------------------
TEST_F(SphinxFunctionalTest, fullIndexerTest)
{
  SphinxFunctionalObjectTest::fullIndexerTest(*pObj);
  ASSERT_FALSE(pObj->isError()) << printError(*pObj);
}
//-----------------------------------------------------------------------------
TEST_F(SphinxFunctionalTest, fullSearcherTest)
{
  SphinxFunctionalObjectTest::fullSearcherTest(*pObj);
  ASSERT_FALSE(pObj->isError()) << printError(*pObj);
}
//-----------------------------------------------------------------------------
TEST_F(SphinxFunctionalTest, differentFiltersTest)
{
  SphinxFunctionalObjectTest::differentFiltersTest(*pObj);
  ASSERT_FALSE(pObj->isError()) << printError(*pObj);
}
//-----------------------------------------------------------------------------
TEST_F(SphinxFunctionalTest, isUse64Bit)
{
  bool ret = SphinxFunctionalObjectTest::isUse64Bit(*pObj);
  std::cout << "Use 64 Bit = " << std::boolalpha << ret << std::endl;
  ASSERT_FALSE(pObj->isError()) << printError(*pObj);
}
//-----------------------------------------------------------------------------
TEST(SphinxPerfomance, testPerfomance)
{
//  const size_t indexCount = 10;
//  const std::string dataFileName = "./test1000.dat";
  HCE::sphinx::tests::SphinxFunctionalObjectTest::testPerfomance(homeDir, perfomanceData.dataFileName, perfomanceData.indexCount);
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace tests
} // end namespace sphinx
} // end namespace HCE





