#include <gtest/gtest.h>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <set>

#include "EncodeDecodeBase64.hpp"
#include "SphinxError.hpp"
#include "SphinxManageTest.hpp"
#include "SphinxInputJsonMessage.hpp"
#include "SphinxOutputJsonMessage.hpp"
//#include "SphinxFunctionalObject.hpp"
#include "SphinxAdminCommand.hpp"
#include "SphinxInputJsonMessageManage.hpp"
#include "SphinxPrintStatus.hpp"

namespace HCE
{
namespace sphinx
{
namespace tests
{
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexCreate(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string json;
  IndexCreate indexCreate(fObj);
  indexCreate.setIndexName(indexName);
  ASSERT_TRUE(indexCreate.serialize(json));
  printSuccess("Serialize IndexCreate");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_CREATE");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";

  EXPECT_FALSE(ret.empty());
  EXPECT_TRUE(fObj.getErrorCode()==NO_ERROR || fObj.getErrorCode()==ERROR_CREATE_INDEX) << "ErrorCode: " << fObj.getErrorCode() << std::endl;

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_FALSE(outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR || outputJsonMessage.getErrorCode()==ERROR_CREATE_INDEX);
  printSuccess("Test command INDEX_CREATE");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexCheck(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, bool existed)
{
  std::string json;
  IndexCheck indexCheck(fObj);
  indexCheck.setIndexName(indexName);
  ASSERT_TRUE(indexCheck.serialize(json));
  printSuccess("Serialize IndexCheck");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_CHECK");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";

  EXPECT_FALSE(ret.empty());
  EXPECT_TRUE(fObj.getErrorCode()==NO_ERROR || fObj.getErrorCode()==ERROR_CHECK_INDEX) << "ErrorCode: " << fObj.getErrorCode() << std::endl;

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_FALSE(outputJsonMessage.isError());
  fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "outputJsonMessage.getData(): '" << outputJsonMessage.getData() << "'\n";
  ASSERT_TRUE(outputJsonMessage.getData()==((existed)?"1":"0"));
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR || outputJsonMessage.getErrorCode()==ERROR_CHECK_INDEX);
  printSuccess("Test command INDEX_CHECK");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexStoreDataFile(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, const std::string& branchName)
{
  std::stringstream dataContent;
  dataContent << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<sphinx:docset xmlns:sphinx=\"http://sphinxsearch.com\">\n<sphinx:document id=\"123\">\n"
      <<  "<title><![CDATA[[- Information interchange - Representation of dates and times. A discussion of ISO 8601 has been written by Markus Kuhn. ISO 8601 describes a large number of date/time formats. For example it defines Basic Format, without punctuation, and Extended Format, with punctuation, and it allows elements to be omitted. This profile defines a restricted range of formats, all of which are valid ISO 8601 dates and times. The aim is to simplify the use of ISO 8601 in World Wide Web-related standards, and to avoid the need for the developers and users of these standards to obtain copies of ISO 8601 itself.]]></title>\n"
      <<  "<h><![CDATA[[time, so this profile defines six levels. Standards that reference this profile should specify one or more of these granularities. If a given standard allows more than one granularity, it should specify the meaning of the dates and times with reduced precision, for example, the result of comparing two dates with different precisions. The formats are as follows. Exactly the components shown here must be present, with exactly this punctuation. Note that the \"T\" appears literally in the string, to indicate the beginning of the time element, as specified in ISO 8601. Year: YYYY (eg 1997) Year and month: YYYY-MM (eg 1997-07)]]></h>\n"
      <<  "<cdate>123</cdate>\n"
      << "</sphinx:document>\n"
      <<  "\n<sphinx:document id=\"53\">\n"
      <<  "<title><![CDATA[[- Information interchange -  ISO 8601 describes a large , and Extended Format, with punctuation, and it allows elements to be omitted. This profile defines a restricted range of formats, all of which are valid ISO 8601 dates and times. The aim is to simplify the use of ISO 8601 in World Wide Web-related standards, and to avoid the need for the developers and users of these standards to obtain copies of ISO 8601 itself.]]></title>\n"
      <<  "<h><![CDATA[[time, so this profile defines six levels. omparing two dates with different precisions. The formats are as follows. Exactly the components shown here must be present, with exactly this punctuation. Note that the \"T\" appears literally in the string, to indicate the beginning of the time element, as specified in ISO 8601. Year: YYYY (eg 1997) Year and month: YYYY-MM (eg 1997-07)]]></h>\n"
      <<  "<cdate>53</cdate>\n"
      << "</sphinx:document>\n"
      <<  "\n<sphinx:document id=\"133\">\n"
      <<  "<title><![CDATA[[ There are more differences between 'gcc' and 'g++' than only the standard libraries, so gcc -lstdc++ will still not get you the same behavior as g++. We put all of that language-specific behavior into its own driver .]]></title>\n"
      <<  "<h><![CDATA[[There are more differences between 'gcc' and 'g++' than only the standard libraries, so gcc -lstdc++ will still not get you the same behavior as g++. We put all of that language-specific behavior into its own driver ]]></h>\n"
      <<  "<cdate>133</cdate>\n"
      <<  "</sphinx:document>\n</sphinx:docset>";

  testIndexStoreDataFile(fObj, indexName, branchName, dataContent.str());
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexStoreDataFile(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, const std::string& branchName, const std::string& dataContent)
{
  std::string json;
  IndexStoreDataFile indexStoreDataFile(fObj);
  indexStoreDataFile.setIndexName(indexName);
  indexStoreDataFile.setBranchName(branchName);
  indexStoreDataFile.setDataContent(HCE::encodeBase64(dataContent));
  ASSERT_TRUE(indexStoreDataFile.serialize(json));
  printSuccess("Serialize IndexStoreDataFile");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_STORE_DATA_FILE");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";

//  if (!fObj.getErrorMsg().empty() || fObj.getErrorCode())
//    std::cout << "ErrorMsg: " << fObj.getErrorMsg() << " ErrorCode = " << fObj.getErrorCode() << std::endl;

  if (!dataContent.empty())
  {
    ASSERT_TRUE(fObj.getErrorMsg().empty());
    ASSERT_TRUE(fObj.getErrorCode()==NO_ERROR);

    const std::string file = fObj.getDataDir()+"/"+indexName+"/"+sphinx_admin_command_const::dataDir+"/"+branchName+".dat";
    std::ifstream ifs(file.c_str());
    ASSERT_TRUE(ifs.is_open());

    std::stringstream fileContent;
    fileContent << ifs.rdbuf();
    ifs.close();

    if (fObj.getPackIncomingDocuments()==false)
      ASSERT_TRUE(dataContent == fileContent.str());

    SphinxOutputJsonMessage outputJsonMessage(ret);
    ASSERT_FALSE(outputJsonMessage.isError());
    ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  }
  else
  {
    ASSERT_FALSE(fObj.getErrorMsg().empty());
    ASSERT_TRUE(fObj.getErrorCode()!=NO_ERROR);
    SphinxOutputJsonMessage outputJsonMessage(ret);
    ASSERT_TRUE(!outputJsonMessage.isError());
    ASSERT_TRUE(outputJsonMessage.getErrorCode()!=NO_ERROR);
  }
  printSuccess("Test command INDEX_STORE_DATA_FILE");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexStoreSchemaFile(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string dataContent = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<sphinx:docset xmlns:sphinx=\"http://sphinxsearch.com\">\n<sphinx:schema>\n<sphinx:field name=\"title\"/>\n<sphinx:field name=\"h\"/>\n<sphinx:attr name=\"cdate\" type=\"timestamp\"/>\n</sphinx:schema>\n</sphinx:docset>";
  testIndexStoreSchemaFile(fObj, indexName, dataContent);
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexStoreSchemaFile(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, const std::string& dataContent)
{
  std::string json;
  IndexStoreSchemaFile indexStoreSchemaFile(fObj);
  indexStoreSchemaFile.setIndexName(indexName);
  indexStoreSchemaFile.setSchemaContent(HCE::encodeBase64(dataContent));
  ASSERT_TRUE(indexStoreSchemaFile.serialize(json));
  printSuccess("Serialize IndexStoreSchemaFile");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_STORE_SCHEMA_FILE");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";

//  if (!fObj.getErrorMsg().empty() || fObj.getErrorCode())
//    std::cout << "ErrorMsg: " << fObj.getErrorMsg() << " ErrorCode = " << fObj.getErrorCode() << std::endl;

  if (!dataContent.empty())
  {
    const std::string file = fObj.getDataDir()+"/"+indexName+"/"+sphinx_search_const::sphinxSchemaFile;
    std::ifstream ifs(file.c_str());
    ASSERT_TRUE(ifs.is_open());

    std::stringstream fileContent;
    fileContent << ifs.rdbuf();
    ifs.close();

    ASSERT_TRUE(dataContent == fileContent.str());
  }
  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR || outputJsonMessage.getErrorCode()==ERROR_STORE_SCHEMA_FILE);
  printSuccess("Test command INDEX_STORE_SCHEMA_FILE");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexRebuild(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, std::vector<std::string>& branches)
{
  std::string json;
  IndexRebuild indexRebuild(fObj);
  indexRebuild.setIndexName(indexName);

  for (size_t i=0;i<branches.size();++i)
    indexRebuild.addBranch(branches[i]);

  ASSERT_TRUE(indexRebuild.serialize(json));

//  std::cout << "indexRebuild json: '" << json << "'\n";

  printSuccess("Serialize IndexRebuild");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_REBUILD");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
  if (!fObj.getErrorMsg().empty())
    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  if (!outputJsonMessage.getData().empty())
    std::cout << "outputJsonMessage.getData(): '" << outputJsonMessage.getData() << "'\n";
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_REBUILD");
}
//-----------------------------------------------------------------------------
// cppcheck-suppress unusedFunction
void SphinxManageTest::testIndexSetDataDir(HCE::sphinx::SphinxFunctionalObject& fObj)
{
  std::string json;
  IndexSetDataDir indexSetDataDir(fObj);
//  indexSetDataDir.setIndexName(indexName); //TODO  now it reserved

  ASSERT_TRUE(indexSetDataDir.serialize(json));
  printSuccess("Serialize IndexSetDataDir");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_SET_DATA_DIR");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_SET_DATA_DIR");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexStart(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string json;
  IndexStart indexStart(fObj);
  indexStart.setIndexName(indexName);
  ASSERT_TRUE(indexStart.serialize(json));
  printSuccess("Serialize IndexStart");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_START");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_START");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexStop(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string json;
  IndexStop indexStop(fObj);
  indexStop.setIndexName(indexName);
  ASSERT_TRUE(indexStop.serialize(json));
  printSuccess("Serialize IndexStop");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_STOP");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_STOP");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexMerge(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, std::vector<std::string>& branches)
{
  std::string json;
  IndexMerge indexMerge(fObj);
  indexMerge.setIndexName(indexName);

  for (size_t i=0;i<branches.size();++i)
    indexMerge.addBranch(branches[i]);

  ASSERT_TRUE(indexMerge.serialize(json));
  printSuccess("Serialize IndexMerge");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_MERGE");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  if (!outputJsonMessage.getData().empty())
    std::cout << "getData: '" << outputJsonMessage.getData() << "'\n";
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_MERGE");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexMergeTrunk(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, std::vector<std::string>& branches)
{
  std::string json;
  IndexMergeTrunk indexMergeTrunk(fObj);
  indexMergeTrunk.setIndexName(indexName);

  for (size_t i=0;i<branches.size();++i)
    indexMergeTrunk.addBranch(branches[i]);

  ASSERT_TRUE(indexMergeTrunk.serialize(json));
  printSuccess("Serialize IndexMergeTrunk");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_MERGE_TRUNK");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  if (!outputJsonMessage.getData().empty())
    std::cout << "getData: '" << outputJsonMessage.getData() << "'\n";
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_MERGE_TRUNK");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexDeleteDataFile(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, std::vector<std::string>& branches)
{
  std::string json;
  IndexDeleteDataFile indexDeleteDataFile(fObj);
  indexDeleteDataFile.setIndexName(indexName);

  for (size_t i=0;i<branches.size();++i)
    indexDeleteDataFile.addBranch(branches[i]);

  ASSERT_TRUE(indexDeleteDataFile.serialize(json));
  printSuccess("Serialize IndexDeleteDataFile");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_DELETE_DATA_FILE");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_DELETE_DATA_FILE");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexDeleteSchemaFile(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string json;
  IndexDeleteSchemaFile indexDeleteSchemaFile(fObj);
  indexDeleteSchemaFile.setIndexName(indexName);

  ASSERT_TRUE(indexDeleteSchemaFile.serialize(json));
  printSuccess("Serialize IndexDeleteSchemaFile");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_DELETE_SCHEMA_FILE");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_DELETE_SCHEMA_FILE");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexAppendDataFile(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, const std::string& branchName)
{
  std::string json;
  std::stringstream dataContent;
  dataContent << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<sphinx:docset xmlns:sphinx=\"http://sphinxsearch.com\">\n<sphinx:document id=\"123456\">\n"
      <<  "<title><![CDATA[[$ fObj --merge main delta --merge-dst-range deleted 0 0]]></title>\n"
      <<  "<h><![CDATA[[Any documents marked as deleted (value 1) would be removed from the newly-merged destination index. It can be added several times to the command line, to add successive filters to the merge, all of which must be met in order for a document to become part of the final index.]]></h>\n"
      <<  "</sphinx:document>\n<sphinx:document id=\"444\">\n"
      <<  "<title><![CDATA[[$ fObj --merge main delta ]]></title>\n"
      <<  "<h><![CDATA[[It can be added several times to the command line.]]></h>\n"
      <<  "</sphinx:document>\n</sphinx:docset>";

  IndexAppendDataFile indexAppendDataFile(fObj);
  indexAppendDataFile.setIndexName(indexName);
  indexAppendDataFile.setBranchName(branchName);
  indexAppendDataFile.setDataContent(HCE::encodeBase64(dataContent.str()));
  ASSERT_TRUE(indexAppendDataFile.serialize(json));
  printSuccess("Serialize IndexAppendDataFile");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_APPEND_DATA_FILE");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";

  ASSERT_TRUE(fObj.getErrorMsg().empty());
  ASSERT_TRUE(fObj.getErrorCode()==NO_ERROR);

  const std::string file = fObj.getDataDir()+"/"+indexName+"/"+sphinx_admin_command_const::dataDir+"/"+branchName+".dat";
  std::ifstream ifs(file.c_str());
  ASSERT_TRUE(ifs.is_open());

  bool isFound = false;
  std::string line;
  while(getline(ifs, line))
  {
    if (line.find("444")!=std::string::npos)
    {
      isFound = true;
      break;
    }
  }
  ifs.close();
  ASSERT_TRUE(isFound==true);

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_APPEND_DATA_FILE");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexDeleteDoc(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, std::vector<unsigned long long>& ids)
{
  std::string json;
  IndexDeleteDoc indexDeleteDoc(fObj);
  indexDeleteDoc.setIndexName(indexName);

  for (size_t i=0;i<ids.size();++i)
    indexDeleteDoc.addId(ids[i]);

  ASSERT_TRUE(indexDeleteDoc.serialize(json));
  printSuccess("Serialize IndexDeleteDoc");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_DELETE_DOC");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  const std::string file = fObj.getDataDir()+"/"+indexName+"/"+sphinx_admin_command_const::deletedFileName;
  std::ifstream ifs(file.c_str());
  ASSERT_TRUE(ifs.is_open());

  // check wrote data in file
  std::set<unsigned long long> readIds;
  std::string line;
  while(getline(ifs, line))
  {
    try
    {
      readIds.insert(std::stoull(line));
    }
    catch(...)
    {
      std::cout << "std::stoull(" << line << ") - BAD param." << std::endl;
    }
  }
  ifs.close();

  for (size_t i=0;i<ids.size();++i)
    ASSERT_TRUE(readIds.find(ids[i])!=readIds.end());

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_DELETE_DOC");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexDeleteDocNumber(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string json;
  IndexDeleteDocNumber indexDeleteDocNumber(fObj);
  indexDeleteDocNumber.setIndexName(indexName);

  ASSERT_TRUE(indexDeleteDocNumber.serialize(json));
  printSuccess("Serialize IndexDeleteDocNumber");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_DELETE_DOC_NUMBER");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(!outputJsonMessage.getData().empty());
  std::cout << "deleted list count = " << outputJsonMessage.getData() << std::endl;

  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_DELETE_DOC_NUMBER");
}
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexPackDocData(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, std::vector<std::string>& branches)
{
  std::string json;
  IndexPackDocData indexPackDocData(fObj);
  indexPackDocData.setIndexName(indexName);

  for (size_t i=0;i<branches.size();++i)
    indexPackDocData.addBranch(branches[i]);

  ASSERT_TRUE(indexPackDocData.serialize(json));
  printSuccess("Serialize IndexPackDocData");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_PACK_DOC_DATA");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(!outputJsonMessage.getData().empty());
  std::cout << "Document count = " << outputJsonMessage.getData() << std::endl;
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_PACK_DOC_DATA");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexRemove(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  fObj.setErrorMsg("");

  std::string json;
  IndexRemove indexCreate(fObj);
  indexCreate.setIndexName(indexName);
  ASSERT_TRUE(indexCreate.serialize(json));
  printSuccess("Serialize IndexRemove");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_REMOVE");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_REMOVE");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexCopy(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexFromName, const std::string& indexToName)
{
  std::string json;
  IndexCopy indexCopy(fObj);
  indexCopy.setIndexFromName(indexFromName);
  indexCopy.setIndexToName(indexToName);
  ASSERT_TRUE(indexCopy.serialize(json));
  printSuccess("Serialize IndexCopy");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_COPY");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_COPY");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexRename(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexFromName, const std::string& indexToName)
{
  std::string json;
  IndexRename indexRename(fObj);
  indexRename.setIndexFromName(indexFromName);
  indexRename.setIndexToName(indexToName);
  ASSERT_TRUE(indexRename.serialize(json));
  printSuccess("Serialize IndexRename");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_RENAME");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_RENAME");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexSetConfigVar(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, const std::string& branchName, const std::string& sectionName,
      const std::string& parameterName, const std::string& parameterValue)
{
  std::string json;
  IndexSetConfigVar indexSetConfigVar(fObj);
  indexSetConfigVar.setIndexName(indexName);
  indexSetConfigVar.setBranchName(branchName);
  indexSetConfigVar.setSectionName(sectionName);
  indexSetConfigVar.setParameterName(parameterName);
  indexSetConfigVar.setParameterValue(parameterValue);
  ASSERT_TRUE(indexSetConfigVar.serialize(json));
  printSuccess("Serialize IndexSetConfigVar");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_SET_CONFIG_VAR");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_SET_CONFIG_VAR");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexGetConfigVar(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, const std::string& branchName,
    const std::string& sectionName, const std::string& parameterName, std::string& parameterValue)
{
  std::string json;
  IndexGetConfigVar indexGetConfigVar(fObj);
  indexGetConfigVar.setIndexName(indexName);
  indexGetConfigVar.setBranchName(branchName);
  indexGetConfigVar.setSectionName(sectionName);
  indexGetConfigVar.setParameterName(parameterName);
  ASSERT_TRUE(indexGetConfigVar.serialize(json));
  printSuccess("Serialize IndexGetConfigVar");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_GET_CONFIG_VAR");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(!outputJsonMessage.getData().empty());
  std::cout << "Parameter value: '" << outputJsonMessage.getData() << "'" << std::endl;
  parameterValue = outputJsonMessage.getData();
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_GET_CONFIG_VAR");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexCheckSchema(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string json;
  std::string dataContent = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<sphinx:docset xmlns:sphinx=\"http://sphinxsearch.com\">\n<sphinx:schema>\n<sphinx:field name = \"title\" weight=\"2\"/>\n<sphinx:field name=\"h\" weight=\"5\"/>\n</sphinx:schema>\n</sphinx:docset>";

  IndexCheckSchema indexCheckSchema(fObj);
  indexCheckSchema.setIndexName(indexName);
  indexCheckSchema.setSchemaContent(HCE::encodeBase64(dataContent));
  ASSERT_TRUE(indexCheckSchema.serialize(json));
  printSuccess("Serialize IndexCheckSchema");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_CHECK_SCHEMA");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_CHECK_SCHEMA");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexStatusSearchd(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string json;
  IndexStatusSearchd indexStatusSearchd(fObj);
  indexStatusSearchd.setIndexName(indexName);
  ASSERT_TRUE(indexStatusSearchd.serialize(json));
  printSuccess("Serialize IndexStatusSearchd");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_STATUS_SEARCHD");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
//  std::cout << "Data:\n" << outputJsonMessage.getData() << std::endl;
  ASSERT_TRUE(!outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_STATUS_SEARCHD");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexStatus(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string json;
  IndexStatus indexStatus(fObj);
  indexStatus.setIndexName(indexName);
  ASSERT_TRUE(indexStatus.serialize(json));
  printSuccess("Serialize IndexStatus");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_STATUS");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
//  std::cout << "Data:\n" << outputJsonMessage.getData() << std::endl;
  ASSERT_TRUE(!outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_STATUS");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
unsigned long long SphinxManageTest::testIndexMaxDocId(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
//  assert(fObj.isLoadedConfig());

  std::string json;
  IndexMaxDocId indexMaxDocId(fObj);
  indexMaxDocId.setIndexName(indexName);
  assert(indexMaxDocId.serialize(json));
  printSuccess("Serialize IndexMaxDocId");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_MAX_DOC_ID");
  msgManage->setCommandOptionsString(json);
  assert(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  assert(!outputJsonMessage.isError());
//  std::cout << "MaxDocId = " << outputJsonMessage.getData() << std::endl;
  assert(!outputJsonMessage.getData().empty());
  assert(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_MAX_DOC_ID");
  return std::stoull(outputJsonMessage.getData());
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexMaxDocIdExtended(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::vector<std::string> branches = {"*"};
  SphinxManageTest::testIndexDeleteDataFile(fObj, indexName, branches);
  SphinxManageTest::testIndexDeleteSchemaFile(fObj, indexName);
  SphinxManageTest::testIndexStoreSchemaFile(fObj, indexName);

  std::stringstream dataContent;
  dataContent << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<sphinx:docset xmlns:sphinx=\"http://sphinxsearch.com\">\n<sphinx:document id=\"123\">\n"
      <<  "<title><![CDATA[[- Information interchange - Representation of dates and times. A discussion of ISO 8601 has been written by Markus Kuhn. ISO 8601 describes a large number of date/time formats. For example it defines Basic Format, without punctuation, and Extended Format, with punctuation, and it allows elements to be omitted. This profile defines a restricted range of formats, all of which are valid ISO 8601 dates and times. The aim is to simplify the use of ISO 8601 in World Wide Web-related standards, and to avoid the need for the developers and users of these standards to obtain copies of ISO 8601 itself.]]></title>\n"
      <<  "<h><![CDATA[[time, so this profile defines six levels. Standards that reference this profile should specify one or more of these granularities. If a given standard allows more than one granularity, it should specify the meaning of the dates and times with reduced precision, for example, the result of comparing two dates with different precisions. The formats are as follows. Exactly the components shown here must be present, with exactly this punctuation. Note that the \"T\" appears literally in the string, to indicate the beginning of the time element, as specified in ISO 8601. Year: YYYY (eg 1997) Year and month: YYYY-MM (eg 1997-07)]]></h>\n"
      << "</sphinx:document>\n</sphinx:docset>";

  SphinxManageTest::testIndexStoreDataFile(fObj, indexName, "b0001", dataContent.str());
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  unsigned long long firstMaxDocID = SphinxManageTest::testIndexMaxDocId(fObj, indexName);

  dataContent.str("");
  dataContent << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<sphinx:docset xmlns:sphinx=\"http://sphinxsearch.com\">\n<sphinx:document id=\"133\">\n"
      <<  "<title><![CDATA[[- Information interchange -  ISO 8601 describes a large , and Extended Format, with punctuation, and it allows elements to be omitted. This profile defines a restricted range of formats, all of which are valid ISO 8601 dates and times. The aim is to simplify the use of ISO 8601 in World Wide Web-related standards, and to avoid the need for the developers and users of these standards to obtain copies of ISO 8601 itself.]]></title>\n"
      <<  "<h><![CDATA[[time, so this profile defines six levels. omparing two dates with different precisions. The formats are as follows. Exactly the components shown here must be present, with exactly this punctuation. Note that the \"T\" appears literally in the string, to indicate the beginning of the time element, as specified in ISO 8601. Year: YYYY (eg 1997) Year and month: YYYY-MM (eg 1997-07)]]></h>\n"
      << "</sphinx:document>\n"
      <<  "\n<sphinx:document id=\"53\">\n"
      <<  "<title><![CDATA[[ There are more differences between 'gcc' and 'g++' than only the standard libraries, so gcc -lstdc++ will still not get you the same behavior as g++. We put all of that language-specific behavior into its own driver .]]></title>\n"
      <<  "<h><![CDATA[[There are more differences between 'gcc' and 'g++' than only the standard libraries, so gcc -lstdc++ will still not get you the same behavior as g++. We put all of that language-specific behavior into its own driver ]]></h>\n"
      <<  "</sphinx:document>\n</sphinx:docset>";

  SphinxManageTest::testIndexStoreDataFile(fObj, indexName, "b0002", dataContent.str());
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  unsigned long long secondMaxDocID = SphinxManageTest::testIndexMaxDocId(fObj, indexName);

  branches.clear();
  branches.push_back("b0002");
  SphinxManageTest::testIndexDeleteDataFile(fObj, indexName, branches);
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  unsigned long long threeMaxDocID = SphinxManageTest::testIndexMaxDocId(fObj, indexName);

  branches.clear();
  branches.push_back("*");
  SphinxManageTest::testIndexStoreDataFile(fObj, indexName, "b0002", dataContent.str());
  SphinxManageTest::testIndexAppendDataFile(fObj, indexName, "b0001");
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  unsigned long long fourMaxDocID = SphinxManageTest::testIndexMaxDocId(fObj, indexName);

  branches.clear();
  branches.push_back("b0001");
  SphinxManageTest::testIndexDeleteDataFile(fObj, indexName, branches);
  SphinxManageTest::testIndexRebuild(fObj, indexName, branches);
  unsigned long long fiveMaxDocID = SphinxManageTest::testIndexMaxDocId(fObj, indexName);

//  std::cout << "1) maxDocID = " << firstMaxDocID << std::endl;
//  std::cout << "2) maxDocID = " << secondMaxDocID << std::endl;
//  std::cout << "3) maxDocID = " << threeMaxDocID << std::endl;
//  std::cout << "4) maxDocID = " << fourMaxDocID << std::endl;
//  std::cout << "5) maxDocID = " << fiveMaxDocID << std::endl;

//  std::cout << "(" << firstMaxDocID << " == " << threeMaxDocID << ") -> " << ((firstMaxDocID==threeMaxDocID)?"GOOD":"BAD") << std::endl;
//  std::cout << "(" << secondMaxDocID << " == " << fiveMaxDocID << ") -> " << ((secondMaxDocID==fiveMaxDocID)?"GOOD":"BAD") << std::endl;
  ASSERT_TRUE(fourMaxDocID);
  ASSERT_TRUE(firstMaxDocID==threeMaxDocID);
  ASSERT_TRUE(secondMaxDocID==fiveMaxDocID);
  printSuccess("Extended test command INDEX_MAX_DOC_ID");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexDataList(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string json;
  IndexDataList indexDataList(fObj);
  indexDataList.setIndexName(indexName);
  ASSERT_TRUE(indexDataList.serialize(json));
  printSuccess("Serialize IndexDataList");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_DATA_LIST");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
//  std::cout << "Data: " << outputJsonMessage.getData() << std::endl;
  ASSERT_TRUE(!outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_DATA_LIST");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexBranchesList(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  std::string json;
  IndexBranchesList indexBranchesList(fObj);
  indexBranchesList.setIndexName(indexName);
  ASSERT_TRUE(indexBranchesList.serialize(json));
  printSuccess("Serialize IndexBranchesList");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_BRANCHES_LIST");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
//  std::cout << "Data: " << outputJsonMessage.getData() << std::endl;
  ASSERT_TRUE(!outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_BRANCHES_LIST");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexBranchesInfo(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, std::vector<std::string>& branches)
{
  std::string json;
  IndexBranchesInfo indexBranchesInfo(fObj);
  indexBranchesInfo.setIndexName(indexName);
  for (size_t i=0;i<branches.size();++i)
    indexBranchesInfo.addBranch(branches[i]);
  ASSERT_TRUE(indexBranchesInfo.serialize(json));
  printSuccess("Serialize IndexBranchesInfo");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_BRANCHES_INFO");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
//  std::cout << "Data: " << outputJsonMessage.getData() << std::endl;
  ASSERT_TRUE(!outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_BRANCHES_INFO");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexBranchesStatus(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName, std::vector<std::string>& branches)
{
  std::string json;
  IndexBranchesStatus indexBranchesStatus(fObj);
  indexBranchesStatus.setIndexName(indexName);
  for (size_t i=0;i<branches.size();++i)
    indexBranchesStatus.addBranch(branches[i]);
  ASSERT_TRUE(indexBranchesStatus.serialize(json));
  printSuccess("Serialize IndexBranchesStatus");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_BRANCHES_STATUS");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'\n";
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
//  std::cout << "Data: " << outputJsonMessage.getData() << std::endl;
  ASSERT_TRUE(!outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_BRANCHES_STATUS");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexConnect(HCE::sphinx::SphinxFunctionalObject& fObj, const std::string& indexName)
{
  const unsigned int SZ = 3;
  Poco::SharedPtr<HCE::sphinx::SphinxFunctionalObject> pObj[SZ];
  for (unsigned int i=0;i<SZ;++i)
  {
    pObj[i] = new HCE::sphinx::SphinxFunctionalObject(fObj.getNodeName(), fObj.getHomeDir());
//    pObj[i]->setLogStream(std::cout);

    std::string json;
    IndexConnect indexConnect(*(pObj[i]));
    indexConnect.setIndexName(indexName);
    ASSERT_TRUE(indexConnect.serialize(json));
    printSuccess("Serialize IndexConnect");

    Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
    msgManage->setCommandString("INDEX_CONNECT");
    msgManage->setCommandOptionsString(json);
    ASSERT_TRUE(msgManage->serialize(json));
    printSuccess("Serialize SphinxInputJsonMessageManage");

    SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
    handler.setData(json);

    std::stringstream str;
    str << handler;

    std::string ret = pObj[i]->Process(str.str());
    std::cout << "ret: '" << ret << "'" << std::endl;
//    if (!pObj[i]->getErrorMsg().empty())
//      std::cout << "Error: '" << pObj[i]->getErrorMsg() << "'\n";

    SphinxOutputJsonMessage outputJsonMessage(ret);
    ASSERT_TRUE(!outputJsonMessage.isError());
    ASSERT_TRUE(outputJsonMessage.getData().empty());
    ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  }
  printSuccess("Test command INDEX_CONNECT");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxManageTest::testIndexDisconnect(HCE::sphinx::SphinxFunctionalObject& fObj)
{
  std::string json;
  IndexDisconnect indexDisconnect(fObj);
  ASSERT_TRUE(indexDisconnect.serialize(json));
  printSuccess("Serialize IndexDisconnect");

  Poco::SharedPtr<SphinxInputJsonMessageManage> msgManage = new SphinxInputJsonMessageManage();
  msgManage->setCommandString("INDEX_DISCONNECT");
  msgManage->setCommandOptionsString(json);
  ASSERT_TRUE(msgManage->serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageManage");

  SphinxInputJsonMessage handler(SphinxInputJsonMessage::MessageType::mtManage);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());
  std::cout << "ret: '" << ret << "'" << std::endl;
//  if (!fObj.getErrorMsg().empty())
//    std::cout << "Error: '" << fObj.getErrorMsg() << "'\n";

  SphinxOutputJsonMessage outputJsonMessage(ret);
  ASSERT_TRUE(!outputJsonMessage.isError());
  ASSERT_TRUE(outputJsonMessage.getData().empty());
  ASSERT_TRUE(outputJsonMessage.getErrorCode()==NO_ERROR);
  printSuccess("Test command INDEX_DISCONNECT");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace tests
} // end namespace sphinx
} // end namespace HCE
