#ifndef SPHINX_MULTI_INDEX_TEST_HPP
#define SPHINX_MULTI_INDEX_TEST_HPP

#include <Poco/Task.h>
#include <Poco/TaskManager.h>
#include <Poco/TaskNotification.h>
#include <Poco/Observer.h>
#include <Poco/Mutex.h>

#include <iostream>
#include "SphinxFunctionalObject.hpp"
#include <string>

namespace HCE
{
namespace sphinx
{
namespace tests
{
//-----------------------------------------------------------------------------
class SearchTask : public Poco::Task
{
public:
  SearchTask(const std::string& indexName, std::vector<std::string>& queries, bool start=true, bool stop=true);
  void runTask(void);

  float getFps(void) const {return fps;}
  bool stopSearchd(void) { return fObj.stopSearchd();}
private:
  std::string indexName;
  std::vector<std::string> queries;
  SphinxFunctionalObject fObj;

  float fps;
  typedef Poco::Task inherited;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class SearchProgressHandler
{
public:
  SearchProgressHandler(void) : fps(0), mutex() {}
  void onProgress(Poco::TaskProgressNotification* pNf);
  void onFinished(Poco::TaskFinishedNotification* pNf);

  float getFps(void);
private:
  float fps;
  Poco::Mutex mutex;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class SphinxMultiIndexTest
{
  struct IndexProperty
  {
    std::string path;
    std::string listen;
  };
  friend class CreateTask;
  static void printSuccess(const std::string& msg, std::ostream& os=std::cout);
  static void printWarning(const std::string& msg, std::ostream& os=std::cout);

  static void setIndexProperty(const std::string& homeDir, const std::string& indexName, const IndexProperty& property) throw (Poco::Exception);
  static void getIndexProperty(const std::string& homeDir, const std::string& indexName, IndexProperty& property) throw (Poco::Exception);
public:
  static void testCreateIndex(SphinxFunctionalObject& fObj, const std::string& indexName, const std::string& dataContent);
  static unsigned int testSearchIndex(SphinxFunctionalObject& fObj, const std::string& indexName, const std::string& query);

  static void makeIndexNames(size_t count, std::vector<std::string>& names);

  static void testCreateIndexes(const std::string& homeDir, size_t count, const std::string& dataFileName, std::vector<std::string>& names);
  static float testSearchIndexes(std::vector<std::string>& names, std::vector<std::string> queries);  // return accumulate fps all indexes
  static float testSearchIndexes(const std::string& homeDir, std::vector<std::string>& names, std::vector<std::string> queries, const std::string& indexName);  // return accumulate fps all indexes
  static void testRemoveIndexes(const std::string& homeDir, std::vector<std::string>& names);
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace tests
} // namespace sphinx 
} // namespace HCE 

#endif // SPHINX_MULTI_INDEX_TEST_HPP
