#include <iomanip>
#include <sstream>

#include <Poco/ConsoleChannel.h>
#include <Poco/FileChannel.h>
#include <Poco/FormattingChannel.h>
#include <Poco/PatternFormatter.h>
#include <Poco/Logger.h>
#include <Poco/AutoPtr.h>

#include "SphinxPrintStatus.hpp"

namespace HCE
{
namespace sphinx
{
namespace tests
{
//-----------------------------------------------------------------------------
SphinxPrintStatus::SphinxPrintStatus(const std::string& debugLevel_, const std::string& postfix_)
: debugLevel(debugLevel_), postfix(postfix_)
{
  Poco::AutoPtr<Poco::ConsoleChannel> pCons(new Poco::ConsoleChannel());
  pCons->open();

  Poco::Logger::get(debugLevel).setChannel(pCons);
  Poco::Logger::get(debugLevel).setLevel(debugLevel);
}
//-----------------------------------------------------------------------------
void SphinxPrintStatus::operator ()(const std::string& msg, unsigned int width)
{
  std::stringstream outMsg;;
  outMsg << std::setw(width) << msg;

  if (!postfix.empty())
    outMsg << postfix;

  int level = Poco::Logger::get(debugLevel).getLevel();
  Poco::Logger::get(debugLevel).log(Poco::Message("", outMsg.str(), static_cast<Poco::Message::Priority>(level)));
}
//-----------------------------------------------------------------------------
std::string SphinxPrintStatus::operator()(HCE::sphinx::SphinxFunctionalObject& rhs)
{
  std::ostringstream outMsg;
  outMsg << "ErrorMsg: " << rhs.getErrorMsg() << "\tErrorCode: " << rhs.getErrorCode() << std::endl;
  return outMsg.str();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace tests 
} // end namespace sphinx 
} // end namespace HCE 

