#include <gtest/gtest.h>
#include <iomanip>
#include <set>

#include "SphinxResultDataSerializator.hpp"
#include "SphinxSearchTest.hpp"
#include "SphinxSearcher.hpp"
#include "SphinxInputJsonMessage.hpp"
#include "SphinxOutputJsonMessage.hpp"
#include "SphinxError.hpp"
#include "SphinxFunctionalObject.hpp"
#include "SphinxResultDataSerializator.hpp"
#include "SphinxWeightCalculator.hpp"
#include "SphinxInputJsonMessageSearch.hpp"
#include "SphinxPrintStatus.hpp"

namespace HCE
{
namespace sphinx
{
namespace tests
{
//---------------------------------------------------------------------------------------
SphinxPrintStatus printStartMessage("information", " - START");
//---------------------------------------------------------------------------------------
void SphinxSearchTest::testSerialization(void)
{
  const std::string json = "{\"MI\":[{\"At\":[{\"uid\":\"2\"},{\"sid\":\"1\"},{\"lang\":\"255\"}],\"Id\":\"9659189393397617664\",\"W\":\"00000000000000000002000000000000000000ff00000000000000000001\"}],\"RI\":[{\"WI\":[{\"d\":5231,\"h\":28086,\"w\":\"appl\"}],\"f\":5231,\"max\":1,\"node\":\"test_node\",\"order\":2,\"q\":\"apple\",\"qid\":123,\"r\":1,\"time\":1}]}";

  SphinxResultData resultData, outResultData;
  SphinxResultDataSerializator resultSerializator(resultData), outResultSerializator(outResultData);

  bool ret = resultSerializator.unserialize(json);
  if (!ret)
    std::cout << "ErrorCode: " << resultSerializator.getErrorCode() << "\tErrorMsg: " << resultSerializator.getErrorMsg() << std::endl;

  ASSERT_TRUE(ret);
  ASSERT_TRUE(resultSerializator.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(resultSerializator.getErrorMsg().empty());
  ASSERT_TRUE(resultSerializator.getJsonType() == (FILL_MATCH_INFO | FILL_REQUEST_INFO | FILL_ATTRIBUTES | FILL_WORD_INFO));

  printSuccess("Unserialize (FILL_MATCH_INFO | FILL_REQUEST_INFO | FILL_ATTRIBUTES | FILL_WORD_INFO)");

  std::string jsonString;
  ret = resultSerializator.serialize((FILL_MATCH_INFO | FILL_REQUEST_INFO | FILL_ATTRIBUTES | FILL_WORD_INFO), jsonString);
  if (!ret)
    std::cout << "ErrorCode: " << resultSerializator.getErrorCode() << "\tErrorMsg: " << resultSerializator.getErrorMsg() << std::endl;

  ASSERT_TRUE(ret);
  ASSERT_TRUE(resultSerializator.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(resultSerializator.getErrorMsg().empty());
  std::cout << "      json: " << json << std::endl;
  std::cout << "jsonString: " << jsonString << std::endl;

//  assert(jsonString=="{\"MI\":[{\"At\":[{\"uid\":\"2\"},{\"sid\":\"1\"},{\"lang\":\"255\"}],\"Id\":\"9659189393397617664\",\"W\":\":2:ff:1\"}],\"RI\":[{\"WI\":[{\"d\":5231,\"h\":28086,\"w\":\"appl\"}],\"f\":5231,\"max\":1,\"node\":\"test_node\",\"order\":2,\"q\":\"apple\",\"qid\":123,\"r\":1,\"time\":1}]}");
  printSuccess("Serialize (FILL_MATCH_INFO | FILL_REQUEST_INFO | FILL_ATTRIBUTES | FILL_WORD_INFO)");

  ret = resultSerializator.serialize((FILL_MATCH_INFO | FILL_REQUEST_INFO | FILL_ATTRIBUTES), jsonString);
  if (!ret)
    std::cout << "ErrorCode: " << resultSerializator.getErrorCode() << "\tErrorMsg: " << resultSerializator.getErrorMsg() << std::endl;

  ASSERT_TRUE(ret);
  ASSERT_TRUE(resultSerializator.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(resultSerializator.getErrorMsg().empty());
  printSuccess("Serialize (FILL_MATCH_INFO | FILL_REQUEST_INFO | FILL_ATTRIBUTES)");

  ret = outResultSerializator.unserialize(jsonString);
  if (!ret)
    std::cout << "ErrorCode: " << outResultSerializator.getErrorCode() << "\tErrorMsg: " << outResultSerializator.getErrorMsg() << std::endl;

  ASSERT_TRUE(ret);
  ASSERT_TRUE(outResultSerializator.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(outResultSerializator.getErrorMsg().empty());

  const size_t requestCount = outResultData.getRequestInfoCount();
  for (size_t i=0;i<requestCount;++i)
  {
    ASSERT_TRUE(outResultData.getRequestInfoItem(i).getWordsCount()==0);
  }
  printSuccess("Unserialize (FILL_MATCH_INFO | FILL_REQUEST_INFO | FILL_ATTRIBUTES)");

  ret = resultSerializator.serialize((FILL_MATCH_INFO | FILL_REQUEST_INFO), jsonString);
  if (!ret)
    std::cout << "ErrorCode: " << resultSerializator.getErrorCode() << "\tErrorMsg: " << resultSerializator.getErrorMsg() << std::endl;

  ASSERT_TRUE(ret);
  ASSERT_TRUE(resultSerializator.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(resultSerializator.getErrorMsg().empty());
  printSuccess("Serialize (FILL_MATCH_INFO | FILL_REQUEST_INFO)");

  ret = outResultSerializator.unserialize(jsonString);
  if (!ret)
    std::cout << "ErrorCode: " << outResultSerializator.getErrorCode() << "\tErrorMsg: " << outResultSerializator.getErrorMsg() << std::endl;

  ASSERT_TRUE(ret);
  ASSERT_TRUE(outResultSerializator.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(outResultSerializator.getErrorMsg().empty());

  const size_t matchesCount = outResultData.getMatchInfoCount();
  for (size_t i=0;i<matchesCount;++i)
  {
    ASSERT_TRUE(outResultData.getMatchInfoItem(i).getAttributesCount()==0);
  }
  printSuccess("Unserialize (FILL_MATCH_INFO | FILL_REQUEST_INFO)");

  ret = resultSerializator.serialize((FILL_MATCH_INFO), jsonString);
  if (!ret)
    std::cout << "ErrorCode: " << resultSerializator.getErrorCode() << "\tErrorMsg: " << resultSerializator.getErrorMsg() << std::endl;

  ASSERT_TRUE(ret);
  ASSERT_TRUE(resultSerializator.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(resultSerializator.getErrorMsg().empty());
  printSuccess("Serialize (FILL_MATCH_INFO)");

  ret = outResultSerializator.unserialize(jsonString);
  if (!ret)
    std::cout << "ErrorCode: " << outResultSerializator.getErrorCode() << "\tErrorMsg: " << outResultSerializator.getErrorMsg() << std::endl;

  ASSERT_TRUE(ret);
  ASSERT_TRUE(outResultSerializator.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(outResultSerializator.getErrorMsg().empty());
  ASSERT_TRUE(outResultData.getRequestInfoCount()==0);
  printSuccess("Unserialize (FILL_MATCH_INFO)");

  ret = resultSerializator.serialize((NO_FILL), jsonString);
  if (!ret)
    std::cout << "ErrorCode: " << resultSerializator.getErrorCode() << "\tErrorMsg: " << resultSerializator.getErrorMsg() << std::endl;

  assert(ret);
  assert(resultSerializator.getErrorCode() == NO_ERROR);
  assert(resultSerializator.getErrorMsg().empty());
  printSuccess("Serialize (NO_FILL)");

  ret = outResultSerializator.unserialize(jsonString);
  if (!ret)
    std::cout << "ErrorCode: " << outResultSerializator.getErrorCode() << "\tErrorMsg: " << outResultSerializator.getErrorMsg() << std::endl;

  ASSERT_TRUE(ret);
  ASSERT_TRUE(outResultSerializator.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(outResultSerializator.getErrorMsg().empty());
  ASSERT_TRUE(outResultData.getMatchInfoCount() == 0);
  printSuccess("Unserialize (NO_FILL)");

  SphinxDefaultJSON defaultJson(1, 1);
  if (defaultJson.isError())
    std::cout << "ErrorCode: " << defaultJson.getErrorCode() << "\tErrorMsg: " << defaultJson.getErrorMsg() << std::endl;

  ASSERT_FALSE(defaultJson.isError());
  ASSERT_TRUE(defaultJson.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(defaultJson.getErrorMsg().empty());
  ASSERT_FALSE(defaultJson.getJSON().empty());
  printSuccess("Generate default JSON");

  ret = defaultJson.makeJSON();
  if (!ret)
    std::cout << "ErrorCode: " << defaultJson.getErrorCode() << "\tErrorMsg: " << defaultJson.getErrorMsg() << std::endl;

  ASSERT_TRUE(ret);
  ASSERT_TRUE(defaultJson.getErrorCode() == NO_ERROR);
  ASSERT_TRUE(defaultJson.getErrorMsg().empty());
  ASSERT_FALSE(defaultJson.getJSON().empty());

	printSuccess("Test SphinxResultDataSerializator");
}
void SphinxSearchTest::testSerializationExt(void)
{
  const std::string query = "q";
  const unsigned int totalRes = 10;
  const unsigned int totalFound = 10;
  const unsigned int time = 1;


  Poco::SharedPtr<SphinxResultData> sphinxResultData;
  WordInfo word1 = WordInfo("word1", 10, 3);
  WordInfo word2 = WordInfo("word2", 1, 1);

  SphinxMatchInfo sphinxMatchInfo1;
  sphinxMatchInfo1.setDocId(7);
  sphinxMatchInfo1.setWeightFromDecString("111");

  SphinxMatchInfo sphinxMatchInfo2;
  sphinxMatchInfo2.setDocId(9);
  sphinxMatchInfo2.setWeightFromNumeric(222);

  AttrInfo pAttrInfo1("first", "value");

  sphinxMatchInfo1.addAttrInfo(pAttrInfo1);
  sphinxMatchInfo2.addAttrInfo(pAttrInfo1);

  SphinxRequestInfo sphinxRequestInfo;
  sphinxRequestInfo.setQuery(query);
  sphinxRequestInfo.setTotal(totalRes);
  sphinxRequestInfo.setTotalFound(totalFound);
  sphinxRequestInfo.setTimeMsec(time);
  sphinxRequestInfo.addWordInfo(word1);
  sphinxRequestInfo.addWordInfo(word2);

  sphinxResultData.assign(new SphinxResultData());
  sphinxResultData->addMatchInfo(sphinxMatchInfo1);
  sphinxResultData->addMatchInfo(sphinxMatchInfo2);
  sphinxResultData->addRequestInfo(sphinxRequestInfo);


  const size_t matchInfoCount = sphinxResultData->getMatchInfoCount();
  size_t attributesCount = 0;

  if (matchInfoCount)
     attributesCount = sphinxResultData->getMatchInfoItem(0).getAttributesCount();

  SphinxResultDataSerializator resultSerializator(*sphinxResultData);

  resultSerializator.setJsonType(FILL_MATCH_INFO | FILL_REQUEST_INFO | FILL_ATTRIBUTES | FILL_WORD_INFO);

  std::string resultJson;
  bool ret = resultSerializator.serialize(resultJson, attributesCount, attributesCount);
  if (resultSerializator.isError())
    std::cout << "Error: " << resultSerializator.getErrorMsg() << std::endl;
  ASSERT_TRUE(ret);

  std::cout << "JSON: " << resultJson << std::endl;

  sphinxResultData->reset();
  ret = resultSerializator.unserialize(resultJson);
  if (resultSerializator.isError())
    std::cout << "Error: " << resultSerializator.getErrorMsg() << std::endl;
  ASSERT_TRUE(ret);

  const size_t macthInfoCount = sphinxResultData->getMatchInfoCount();
  ASSERT_EQ(macthInfoCount, 2);
  ASSERT_EQ(sphinxResultData->getMatchInfoItem(0).getWeight(), toHexString(111));
  ASSERT_EQ(sphinxResultData->getMatchInfoItem(1).getWeight(), toHexString(222));
  printSuccess("Test SerializationExt");
}
//---------------------------------------------------------------------------------------
void SphinxSearchTest::testMemoryManager(void)
{
  SphinxResultData resultData;
  SphinxResultDataMemoryManager resultMemoryManager(resultData);

  size_t matchesCount = 100;
  size_t attributesCount = 10;

  resultMemoryManager.resize(matchesCount, attributesCount);
  for (size_t i=0;i<(matchesCount+attributesCount);++i) // max limit oversized
  {
    resultMemoryManager.setMatchInfoDocId(i+1);
    resultMemoryManager.setMatchInfoWeight(toHexString(i+1));
    for (size_t j=0;j<matchesCount;++j)                 // max limit oversized
      resultMemoryManager.addMatchInfoAttr(AttrInfo("at_"+std::to_string(i+1), "val_"+std::to_string(i+1)));
    resultMemoryManager.applyMatchInfo();
  }

  ASSERT_EQ(resultData.getMatchInfoCount(), matchesCount);
  for (size_t i=0;i<matchesCount;++i)
    assert(resultData.getMatchInfoItem(i).getAttributesCount() == attributesCount);
  printSuccess("Allocated memory for 100 MatchInfo each have 10 attributes");

  matchesCount = 10;
  attributesCount = 5;

  resultMemoryManager.resize(matchesCount, attributesCount);
  for (size_t i=0;i<(matchesCount+attributesCount);++i) // max limit oversized
  {
    resultMemoryManager.setMatchInfoDocId(i+1);
    resultMemoryManager.setMatchInfoWeight(toHexString(i+1));
    for (size_t j=0;j<matchesCount;++j)                 // max limit oversized
      resultMemoryManager.addMatchInfoAttr(AttrInfo("at_"+std::to_string(i+1), "val_"+std::to_string(i+1)));
    resultMemoryManager.applyMatchInfo();
  }

  ASSERT_EQ(resultData.getMatchInfoCount(), matchesCount);
  for (size_t i=0;i<matchesCount;++i)
    ASSERT_EQ(resultData.getMatchInfoItem(i).getAttributesCount(), attributesCount);
  printSuccess("Reallocated memory for 10 MatchInfo each have 5 attributes");

  resultMemoryManager.reset();
  ASSERT_EQ(resultData.getMatchInfoCount(), 0);
  for (size_t i=0;i<matchesCount;++i)
    ASSERT_EQ(resultData.getMatchInfoItem(i).getAttributesCount(), 0);
  printSuccess("Reset allocated memory");

  for (size_t i=0;i<(matchesCount/2);++i)                      // half max limit
  {
    resultMemoryManager.setMatchInfoDocId(i+1);
    resultMemoryManager.setMatchInfoWeight(toHexString(i+1));
    for (size_t j=0;j<(attributesCount/2);++j)                 // half max limit
      resultMemoryManager.addMatchInfoAttr(AttrInfo("at_"+std::to_string(i+1), "val_"+std::to_string(i+1)));
    resultMemoryManager.applyMatchInfo();
  }

  ASSERT_EQ(resultData.getMatchInfoCount(), matchesCount/2);
  for (size_t i=0;i<(matchesCount/2);++i)
    ASSERT_EQ(resultData.getMatchInfoItem(i).getAttributesCount(), attributesCount/2);
  printSuccess("Filling half allocated memory");

  resultMemoryManager.resize(1, 0);
  for (size_t i=0;i<matchesCount;++i)                      // max limit oversized
  {
    resultMemoryManager.setMatchInfoDocId(i+1);
    resultMemoryManager.setMatchInfoWeight(toHexString(i+1));
    for (size_t j=0;j<(attributesCount);++j)                 // max limit oversized
      resultMemoryManager.addMatchInfoAttr(AttrInfo("at_"+std::to_string(i+1), "val_"+std::to_string(i+1)));
    resultMemoryManager.applyMatchInfo();
  }

  ASSERT_EQ(resultData.getMatchInfoCount(), 1);
  ASSERT_EQ(resultData.getMatchInfoItem(0).getAttributesCount(), 0);
  printSuccess("Resize allocated memory for 1 MatchInfo and without attributes");

  resultMemoryManager.resize(0, 0);
  for (size_t i=0;i<matchesCount;++i)                      // max limit oversized
  {
    resultMemoryManager.setMatchInfoDocId(i+1);
    resultMemoryManager.setMatchInfoWeight(toHexString(i+1));
    for (size_t j=0;j<(attributesCount);++j)                 // max limit oversized
      resultMemoryManager.addMatchInfoAttr(AttrInfo("at_"+std::to_string(i+1), "val_"+std::to_string(i+1)));
    resultMemoryManager.applyMatchInfo();
  }
  ASSERT_EQ(resultData.getMatchInfoCount(), 0);
  printSuccess("Resize allocated memory for 0 MatchInfo and without attributes");

  const size_t MATCHES_SZ = 2;
  const size_t ATTRIBUTES_SZ = 3;
  resultMemoryManager.resize(MATCHES_SZ, ATTRIBUTES_SZ);
  unsigned long long docId[MATCHES_SZ] = {123, 456};
  std::string weight[MATCHES_SZ] = {"00000000000000000001", "00000000000000000002"};
  std::vector<std::pair<std::string, std::string> > attr[ATTRIBUTES_SZ] = {{
      {"uid", "67"},
      {"qid", "123"},
      {"lang", "255"}
  }, {
      {"uid", "87"},
      {"qid", "45"},
      {"lang", "1"}
  }};

  // add to SphinxResultData information about 2 matches
  for (size_t i=0;i<MATCHES_SZ;++i)
  {
    resultMemoryManager.setMatchInfoDocId(docId[i]);
    resultMemoryManager.setMatchInfoWeight(weight[i]);
    for (size_t j=0;j<ATTRIBUTES_SZ;++j)
      resultMemoryManager.addMatchInfoAttr(AttrInfo(attr[i][j].first, attr[i][j].second));
    resultMemoryManager.applyMatchInfo();
  }

  // check correct content of SphinxResultData
  matchesCount = resultData.getMatchInfoCount();
  for (size_t i=0;i<matchesCount;++i)
  {
    ASSERT_EQ(resultData.getMatchInfoItem(i).getDocId(), docId[i]);
    ASSERT_EQ(resultData.getMatchInfoItem(i).getWeight(), weight[i]);
    attributesCount = resultData.getMatchInfoItem(i).getAttributesCount();
    for (size_t j=0;j<attributesCount;++j)
    {
      ASSERT_EQ(resultData.getMatchInfoItem(i).getAttrItem(j).name, attr[i][j].first);
      ASSERT_EQ(resultData.getMatchInfoItem(i).getAttrItem(j).value, attr[i][j].second);
    }
  }
  printSuccess("Test SphinxResultDataMemoryMananger");
}
//---------------------------------------------------------------------------------------
void SphinxSearchTest::makeSearchRequest(SphinxFunctionalObject& fObj, const std::string& query,
    const std::vector<std::pair<std::string, std::string> >& queryParameters,
    const std::vector<std::pair<std::string, std::string> >& orderParameters,
    const std::vector<std::string>& externalFields,
    const std::vector<std::string>& orderFields,
    SphinxResultData& resultData)
{
  resultData.clear();
  SphinxResultDataSerializator resultSerializator(resultData);

  std::string json;
  SphinxInputJsonMessageSearch msgSearch;
  // prepare before request
  msgSearch.setQueryString(query);
  for (size_t i=0;i<queryParameters.size();++i)
    msgSearch.addQueryParameters(queryParameters[i].first, queryParameters[i].second);

  for (size_t i=0;i<orderParameters.size();++i)
    msgSearch.addOrderParameters(orderParameters[i].first, orderParameters[i].second);

  for (size_t i=0;i<externalFields.size();++i)
    msgSearch.addExternalFields(externalFields[i]);

  for (size_t i=0;i<orderFields.size();++i)
    msgSearch.addOrderFields(orderFields[i]);

  ASSERT_TRUE(msgSearch.serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageSearch");

  SphinxInputJsonMessage handler;
  handler.setType(SphinxInputJsonMessage::MessageType::mtSearch);
  handler.setData(json);

  std::stringstream str;
  str << handler;

  std::string ret = fObj.Process(str.str());

  std::cout << "ret: " << ret << std::endl;

  SphinxOutputJsonMessage outputMessage(ret);
  ASSERT_FALSE(outputMessage.isError());
  ASSERT_TRUE(resultSerializator.unserialize(outputMessage.getData()));

  ASSERT_FALSE(resultSerializator.isError());
  ASSERT_TRUE(resultSerializator.getErrorCode()==NO_ERROR);
  ASSERT_TRUE(resultSerializator.getErrorMsg().empty());
  printSuccess("Make Search Request");
}
//---------------------------------------------------------------------------------------
void SphinxSearchTest::testSearcherProcess(SphinxFunctionalObject& fObj, const std::string& query, unsigned int maxResults)
{
  const std::string title = "Test Searcher Process";
  printStartMessage(title);
  ASSERT_TRUE(fObj.isLoadedConfig());

  SphinxResultData resultData;
  const std::vector<std::pair<std::string, std::string> > queryParameters = {
      {"queryId", "7475839"},
      {"JsonType", "15"},
      {"timeout", "5"},
      {"max_results", std::to_string(maxResults)},
      {"offset", "0"},
      {"limit", "0"},
      {"cutoff", "0"},
      {"order_by", "0"},
      {"ttl", "3"}
  };
  const std::vector<std::pair<std::string, std::string> > orderParameters = {
      {"algorithm", "0"},
      {"order_by", "2"}
  };
  const std::vector<std::string> externalFields = {"uid", "sid", "lang"};
  const std::vector<std::string> orderFields = {"uid", "lang", "sid"};

  SphinxSearchTest::makeSearchRequest(fObj, query, queryParameters, orderParameters, externalFields, orderFields, resultData);

  if (!fObj.isError())
    printSuccess(title);
  else
    printFail(title);
}
//-----------------------------------------------------------------------------
void SphinxSearchTest::testGetResultUseOffset(SphinxFunctionalObject& fObj, const std::string& query, unsigned int maxResults, unsigned int offset)
{
  const std::string title = "Test Get Result Use Offset";
  printStartMessage(title);
  ASSERT_TRUE(fObj.isLoadedConfig());

  std::set<unsigned long long> ids;

  SphinxResultData resultData;
  std::vector<std::pair<std::string, std::string> > queryParameters = {
      {"queryId", "7475839"},
      {"JsonType", "15"},
      {"timeout", "5"},
      {"max_results", std::to_string(maxResults)},
      {"offset", std::to_string(offset)},
      {"limit", "0"},
      {"cutoff", "0"},
      {"order_by", "1"},
      {"ttl", "3"}
  };
  const std::vector<std::pair<std::string, std::string> > orderParameters = {
      {"algorithm", "0"},
      {"order_by", "2"}
  };
  const std::vector<std::string> externalFields = {"uid", "sid", "lang"};
  const std::vector<std::string> orderFields = {"uid", "lang", "sid"};

  SphinxSearchTest::makeSearchRequest(fObj, query, queryParameters, orderParameters, externalFields, orderFields, resultData);

  const size_t requestInfoCount = resultData.getRequestInfoCount();
  for (size_t i=0;i<requestInfoCount;++i)
  {
    const unsigned int totalFound = resultData.getRequestInfoItem(i).getTotalFound();
    std::cout << "TOTAL FOUND: " << totalFound << std::endl;

    std::cout << "offset = " << offset << std::endl;

    for (size_t k=0;k<totalFound;k+=maxResults)
    {
      for (size_t j=0;j<queryParameters.size();++j)
      {
        if (queryParameters[j].first=="offset")
        {
          queryParameters[j].second = std::to_string(offset);
          std::cout << "offset = " << queryParameters[j].second << std::endl;
        }

        if (queryParameters[j].first=="max_results")
        {
          queryParameters[j].second = std::to_string(offset+maxResults);
          std::cout << "max_results = " << queryParameters[j].second << std::endl;
        }
      }
      SphinxSearchTest::makeSearchRequest(fObj, query, queryParameters, orderParameters, externalFields, orderFields, resultData);
      const size_t matchInfoCount = resultData.getMatchInfoCount();
      std::cout << "MATCH INFO COUNT: " << matchInfoCount << std::endl;
      for (size_t q=0;q<matchInfoCount;++q)
        ids.insert(resultData.getMatchInfoItem(q).getDocId());

      const size_t requestInfoCount = resultData.getRequestInfoCount();
      for (size_t q=0;q<requestInfoCount;++q)
      {
        const unsigned int totalFound = resultData.getRequestInfoItem(q).getTotalFound();
        std::cout << "TOTAL FOUND: " << totalFound << std::endl;
      }
      offset += (maxResults-1);
    }
  }

  std::cout << "UNIQUE ID FOUND: " << ids.size() << std::endl;

  if (!fObj.isError())
    printSuccess(title);
  else
    printFail(title);
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// cppcheck-suppress unusedFunction
unsigned int SphinxSearchTest::simpleSearchQuery(SphinxFunctionalObject& fObj, const std::string& query, size_t repeatCount)
{
  unsigned int resultTime = 0;

  std::string json;
  SphinxInputJsonMessageSearch msgSearch;
  msgSearch.setQueryString(query);
  msgSearch.addQueryParameters("queryId", "777");
  msgSearch.addQueryParameters("JsonType", "7");
  msgSearch.addQueryParameters("max_results", "5");
  msgSearch.addQueryParameters("offset", "0");
  msgSearch.addQueryParameters("limit", "0");
  msgSearch.addQueryParameters("cutoff", "0");
  assert(msgSearch.serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageSearch");

  SphinxInputJsonMessage inputJsonMessage;
  inputJsonMessage.setType(SphinxInputJsonMessage::MessageType::mtSearch);
  inputJsonMessage.setData(json);

  for (size_t i=0;i<repeatCount;++i)
  {
    SphinxOutputJsonMessage outputMessage = fObj.Process(inputJsonMessage);
    if (!fObj.getErrorMsg().empty())
      std::cout << "fObj.getErrorMsg()  = " << fObj.getErrorMsg() << std::endl;
    assert(fObj.getErrorMsg().empty());
    assert(!outputMessage.isError());
    resultTime += outputMessage.getTime();;
  }
  return resultTime;
}
//-----------------------------------------------------------------------------
void SphinxSearchTest::testSerializationSearchProcess(const std::string& json)
{
  SphinxInputJsonMessage inputJsonMessage(json);

//  std::cout << "\njson: " << json << std::endl << std::endl;

//  std::cout << "inputJsonMessage.isError() = " << std::boolalpha << inputJsonMessage.isError() << std::endl;
//  std::cout << "inputJsonMessage.getErrorCode() = " << inputJsonMessage.getErrorCode() << std::endl;
//  std::cout << "inputJsonMessage.getErrorMsg(): '" << inputJsonMessage.getErrorMsg() << "'" << std::endl;
  ASSERT_FALSE(inputJsonMessage.isError());
  ASSERT_TRUE(inputJsonMessage.getErrorCode()==NO_ERROR);
  ASSERT_TRUE(inputJsonMessage.getErrorMsg().empty());
  printSuccess("Unserialize SphinxInputJsonMessage");

  SphinxInputJsonMessage::MessageType type = inputJsonMessage.getType();
  std::string dataString = inputJsonMessage.getData();

//  std::cout << "\n dataString = " << dataString << std::endl << std::endl;


  SphinxInputJsonMessageSearch msgSearch(dataString);
//  std::cout << "msgSearch.isError() = " << std::boolalpha << msgSearch.isError() << std::endl;
//  std::cout << "msgSearch.getErrorCode() = " << msgSearch.getErrorCode() << std::endl;
//  std::cout << "msgSearch.getErrorMsg(): '" << msgSearch.getErrorMsg() << "'" << std::endl;
  ASSERT_FALSE(msgSearch.isError());
  ASSERT_TRUE(msgSearch.getErrorCode()==NO_ERROR);
  ASSERT_TRUE(msgSearch.getErrorMsg().empty());
  printSuccess("Unserialize SphinxInputJsonMessageSearch");

  std::string restoreString;
  bool ret = msgSearch.serialize(restoreString);

//  std::cout << "\n restoreString = " << restoreString << std::endl << std::endl;

//  std::cout << "msgSearch.serialize() = " << std::boolalpha << ret << std::endl;
//  std::cout << "msgSearch.isError() = " << std::boolalpha << msgSearch.isError() << std::endl;
//  std::cout << "msgSearch.getErrorCode() = " << msgSearch.getErrorCode() << std::endl;
//  std::cout << "msgSearch.getErrorMsg(): '" << msgSearch.getErrorMsg() << "'" << std::endl;
  ASSERT_TRUE(ret);
  ASSERT_FALSE(msgSearch.isError());
  ASSERT_TRUE(msgSearch.getErrorCode()==NO_ERROR);
  ASSERT_TRUE(msgSearch.getErrorMsg().empty());
  printSuccess("Serialize SphinxInputJsonMessageSearch");

  std::string restoreJson;
  SphinxInputJsonMessage restoreInputJsonMessage;
  restoreInputJsonMessage.setType(type);
  restoreInputJsonMessage.setData(restoreString);
  ret = restoreInputJsonMessage.serialize(restoreJson);
//  std::cout << "restoreInputJsonMessage.serialize() = " << std::boolalpha << ret << std::endl;
//  std::cout << "restoreInputJsonMessage.isError() = " << std::boolalpha << restoreInputJsonMessage.isError() << std::endl;
//  std::cout << "restoreInputJsonMessage.getErrorCode() = " << restoreInputJsonMessage.getErrorCode() << std::endl;
//  std::cout << "restoreInputJsonMessage.getErrorMsg(): '" << restoreInputJsonMessage.getErrorMsg() << "'" << std::endl;
  ASSERT_TRUE(ret);
  ASSERT_FALSE(restoreInputJsonMessage.isError());
  ASSERT_TRUE(restoreInputJsonMessage.getErrorCode()==NO_ERROR);
  ASSERT_TRUE(restoreInputJsonMessage.getErrorMsg().empty());
  printSuccess("Serialize SphinxInputJsonMessage");

  SphinxInputJsonMessageSearch restoreSearch(restoreInputJsonMessage.getData());
//  std::cout << "restoreSearch.isError() = " << std::boolalpha << msgSearch.isError() << std::endl;
//  std::cout << "restoreSearch.getErrorCode() = " << msgSearch.getErrorCode() << std::endl;
//  std::cout << "restoreSearch.getErrorMsg(): '" << msgSearch.getErrorMsg() << "'" << std::endl;
  ASSERT_FALSE(restoreSearch.isError());
  ASSERT_TRUE(restoreSearch.getErrorCode()==NO_ERROR);
  ASSERT_TRUE(restoreSearch.getErrorMsg().empty());
  printSuccess("Unserialize SphinxInputJsonMessageSearch");

  // check validate
  ASSERT_EQ(msgSearch.getQueryString(), restoreSearch.getQueryString());

  std::vector<std::pair<std::string, std::string> >& sourceParams = msgSearch.getQueryParameters();
  std::vector<std::pair<std::string, std::string> >& restoreParams = restoreSearch.getQueryParameters();

//  std::cout << " sourceParams.size() = " << sourceParams.size() << std::endl;
//  for (size_t i=0;i<sourceParams.size();++i)
//    std::cout << i << ") " << sourceParams[i].first << " = " << sourceParams[i].second << std::endl;

//  std::cout << "restoreParams.size() = " << restoreParams.size() << std::endl;
//  for (size_t i=0;i<restoreParams.size();++i)
//    std::cout << i << ") " << restoreParams[i].first << " = " << restoreParams[i].second << std::endl;

  ASSERT_EQ(sourceParams.size(), restoreParams.size());


  for (size_t i=0;i<sourceParams.size();++i)
  {
    bool found = false;
    for (size_t k=0;k<restoreParams.size();++k)
      if (sourceParams[i].first==restoreParams[k].first)
      {
        found = true;
        break;
      }
    ASSERT_TRUE(found);
  }

  SphinxFiltersArray& sourceFilters = msgSearch.getFilters();
  SphinxFiltersArray& restoreFilters = restoreSearch.getFilters();
  ASSERT_EQ(sourceFilters.getFilters().size(), restoreFilters.getFilters().size());
  for (size_t i=0;i<sourceFilters.getFilters().size();++i)
  {
    ASSERT_EQ(sourceFilters.getFilters()[i]->getFilterType(), restoreFilters.getFilters()[i]->getFilterType());
    ASSERT_EQ(sourceFilters.getFilters()[i]->getAttributeName(), restoreFilters.getFilters()[i]->getAttributeName());
    ASSERT_EQ(sourceFilters.getFilters()[i]->getExcludeValue(), restoreFilters.getFilters()[i]->getExcludeValue());
    ASSERT_EQ(sourceFilters.getFilters()[i]->getAttributeValues().size(), restoreFilters.getFilters()[i]->getAttributeValues().size());
    for (size_t k=0;k<sourceFilters.getFilters()[i]->getAttributeValues().size();++k)
    {
      ASSERT_EQ(sourceFilters.getFilters()[i]->getAttributeValues()[k], restoreFilters.getFilters()[i]->getAttributeValues()[k]);
    }
  }
  printSuccess("Test Serialization Search Process");
}
//-----------------------------------------------------------------------------
SphinxOutputJsonMessage  SphinxSearchTest::searchWithFilter(SphinxFunctionalObject& fObj,
    const std::string& query, std::vector<Poco::SharedPtr<SphinxFilter> >& filters)
{
  assert(fObj.isLoadedConfig());

  std::string json;
  SphinxInputJsonMessageSearch inputJsonMessageSearch;
  inputJsonMessageSearch.setQueryString(query);
  inputJsonMessageSearch.addQueryParameters("queryId", "1234");
  inputJsonMessageSearch.addQueryParameters("jsonType", "15");
  inputJsonMessageSearch.addQueryParameters("max_results", "100");

  for (size_t i=0;i<filters.size();++i)
    inputJsonMessageSearch.addFilter(filters[i]);

  bool ret = inputJsonMessageSearch.serialize(json);
  fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "SphinxInputJsonMessageSearch serialize => " << std::boolalpha << ret << std::endl;
  assert(ret);
//  std::cout << "json: " << json << std::endl;

  SphinxInputJsonMessage inputJsonMessage;
  inputJsonMessage.setType(SphinxInputJsonMessage::MessageType::mtSearch);
  inputJsonMessage.setData(json);

  ret = inputJsonMessage.serialize(json);
  fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "\n\nSphinxInputJsonMessage serialize => " << std::boolalpha << ret << std::endl;
  assert(ret);
//  std::cout << "json: " << json << std::endl;

  SphinxOutputJsonMessage outputMessage = fObj.Process(inputJsonMessage);
  if (!fObj.getErrorMsg().empty())
    fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "fObj.getErrorMsg()  = " << fObj.getErrorMsg() << std::endl;
  if (fObj.getErrorCode()!=NO_ERROR)
    fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "fObj.getErrorCode() = " << fObj.getErrorCode() << std::endl;
  if (outputMessage.isError())
    fObj.log(HCE::LoggerStream::PRIO_INFORMATION) << "outputMessage.isError() = " << outputMessage.isError() << std::endl;

  printSuccess("Search with filters");

  return outputMessage;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void SphinxSearchTest::testFilterSearch(SphinxFunctionalObject& fObj)
{
  printStartMessage("Test use filter to Search Process");

  const std::string query = "some";
  std::string json;
  SphinxInputJsonMessageSearch msgSearch;
  std::vector<std::string> attrs = {"123"};
  msgSearch.addFilter((SphinxFilter::FilterType)0, "cdate", attrs, (SphinxFilter::ExcludeType)0);

  msgSearch.setQueryString(query);
  msgSearch.addQueryParameters("queryId", "123");
  msgSearch.addQueryParameters("JsonType", "15");
  ASSERT_TRUE(msgSearch.serialize(json));
  printSuccess("Serialize SphinxInputJsonMessageSearch");

  SphinxInputJsonMessage handler;
  handler.setType(SphinxInputJsonMessage::MessageType::mtSearch);
  handler.setData(json);

  std::stringstream str;
  str << handler;

//  std::cout << "str.str(): '" << str.str() << "'\n";

  std::string outputJson = fObj.Process(str.str());

//  std::cout << "JSON: '" << outputJson << "'\n";

  SphinxOutputJsonMessage outputMessage(outputJson);
  if (!fObj.getErrorMsg().empty())
    std::cout << "fObj.getErrorMsg()  = " << fObj.getErrorMsg() << std::endl;

  ASSERT_FALSE(fObj.isError());
  ASSERT_TRUE(fObj.getErrorCode()==NO_ERROR);
  ASSERT_TRUE(fObj.getErrorMsg().empty());
  ASSERT_FALSE(outputMessage.isError());

  SphinxResultData Result;
  SphinxResultDataSerializator ResultSerializator(Result);
  ASSERT_TRUE(ResultSerializator.unserialize(outputMessage.getData()));
  ASSERT_TRUE(ResultSerializator.getErrorCode()==NO_ERROR);
  ASSERT_TRUE(ResultSerializator.getErrorMsg().empty());

//  std::cout << "Result:\n" << Result << std::endl;
  printSuccess("Test use filter to Search Process");
}
//-----------------------------------------------------------------------------
void SphinxSearchTest::testWeightPacker(void)
{
  const size_t fieldWidth = HEX_STRING_LENGTH;
  size_t fieldsCount = 0;
  std::stringstream str;
  for (size_t i=45567;i<45570;++i)
  {
    str << HexString(i);
    ++fieldsCount;
  }

  std::string tmp = str.str();
  SphinxWeightPacker packer(fieldsCount, fieldWidth);
  for (size_t i=0;i<=fieldsCount+1;++i)
  {
    std::string packedData = packer.pack(tmp, i);
    std::string unpackedData = packer.unpack(packedData);
    ASSERT_EQ(tmp, unpackedData);
  }

  tmp.resize(fieldsCount*fieldWidth+5);
  for (size_t i=0;i<=fieldsCount+1;++i)
  {
    std::string packetData = packer.pack(tmp, i);
    std::string unpackedData = packer.unpack(packetData);
    ASSERT_TRUE(tmp.compare(0, fieldsCount*fieldWidth, unpackedData)==0 || tmp==unpackedData);
  }

  tmp.resize(fieldsCount*fieldWidth-5);
  for (size_t i=0;i<=fieldsCount+1;++i)
  {
    std::string packetData = packer.pack(tmp, i);
    std::string unpackedData = packer.unpack(packetData);
    ASSERT_TRUE(unpackedData.compare(0, fieldsCount*fieldWidth-5, tmp)==0);
  }
  printSuccess("Test Weight Packer");
}
//-----------------------------------------------------------------------------
void SphinxSearchTest::testWeightCalculator(void)
{
  SphinxResultData resultData;
  SphinxWeightCalculator calculator;

  SphinxMatchInfo matchInfo;
  matchInfo.setDocId(2345);
  matchInfo.setWeightFromDecString("2345");

  matchInfo.addAttrInfo("uid", "1");
  matchInfo.addAttrInfo("sid", "2");
  matchInfo.addAttrInfo("lang", "255");
  resultData.addMatchInfo(matchInfo);

  const std::vector<std::string> orderFields = {"sid", "lang", "uid"};

  calculator.setStrategy(new CalculateStrategyDefault());
  calculator.calculate(orderFields, resultData);

  size_t matchInfoCount = resultData.getMatchInfoCount();
  for (size_t i=0;i<matchInfoCount;++i)
    ASSERT_EQ(resultData.getMatchInfoItem(i).getWeight(), "00000000000000000929");

  resultData.clear();
  resultData.addMatchInfo(matchInfo);

  calculator.setStrategy(new CalculateStrategyUseFieldNames());
  calculator.calculate(orderFields, resultData);
  matchInfoCount = resultData.getMatchInfoCount();
  for (size_t i=0;i<matchInfoCount;++i)
    ASSERT_EQ(resultData.getMatchInfoItem(i).getWeight(), "00000000000000000002000000000000000000ff00000000000000000001");

  printSuccess("Test Weight Calculator");
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace tests
} // end namespace sphinx
} // end namespace HCE





