#include <gtest/gtest.h>
#include <Poco/SharedPtr.h>
#include <fstream>
#include <sstream>
#include <assert.h>
#include <unistd.h>
#include <iomanip>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>

#include "ProcExec.hpp"
#include "SphinxError.hpp"
#include "SphinxAdminCommand.hpp"
#include "SphinxManageTest.hpp"
#include "SphinxStressTest.hpp"
#include "SphinxFunctionalObjectTest.hpp"

namespace HCE
{
namespace sphinx
{
namespace tests
{
extern Poco::SharedPtr<SphinxFunctionalObject> pObj;
extern std::string homeDir;
extern StressTestData stressTestData;
//-----------------------------------------------------------------------------
bool SphinxStressTest::makeDataFilesList(const std::string& sourceDir, std::vector<std::string>& files, std::string& error)
{
  bool result = false;
  files.clear();

  struct dirent* d = nullptr;
  DIR* dir = opendir(sourceDir.c_str());
  if (dir != nullptr)
  {
    result = true;
    std::string fileName;
    while((d = readdir(dir)))
    {
      fileName = d->d_name;
      if (fileName != "." && fileName != "..")
        files.push_back(fileName);
    }
    if (closedir(dir)==-1)
      result = false;
  }
  else
    result = false;

  if (!result)
    error = strerror(errno);

  return result;
}
//-----------------------------------------------------------------------------
bool SphinxStressTest::moveDataFiles(const std::string& sourceDir, const std::string& destDir, std::vector<std::string>& files, std::string& error)
{
  bool result = true;
  std::string fileName, srcFile, dstFile;
  size_t i=0;
  for (;i<files.size();++i)
  {
    fileName = files[i];
    srcFile = sourceDir+"/"+fileName;
    size_t pos = fileName.find_last_of('.');
    if (pos!=std::string::npos)
      fileName.resize(pos+1);
    dstFile = destDir+"/"+fileName+sphinx_admin_command_const::dataFileExtension;

    std::stringstream outMsg, errMsg;
    ProcExec procExec("cp "+srcFile+" "+dstFile);
    if (!procExec.exec(outMsg, errMsg))
    {
      error = errMsg.str();
      result = false;
      break;
    }
    std::cout << "." << std::flush;
  }
  std::cout << std::endl << "Copied " << i << " files" << std::endl;
  return result;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
class StressTest : public ::testing::Test
{
protected:
  static void SetUpTestCase(void)
  {
    pObj = Poco::SharedPtr<SphinxFunctionalObject>(new SphinxFunctionalObject("node", 456, homeDir));
  }
  static void TearDownTestCase(void)
  {
    pObj = nullptr;
  }
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
TEST_F(StressTest, Is_Created_SphinxFunctionalObject)
{
  ASSERT_FALSE(pObj.isNull());
}
//-----------------------------------------------------------------------------
TEST_F(StressTest, testStressGetBigResult)
{
  SphinxFunctionalObjectTest::testStressGetBigResult(*pObj, stressTestData.sourceTestDir, stressTestData.indexName, stressTestData.needMakeIndex, stressTestData.needRemoveIndex);
}
//-----------------------------------------------------------------------------
TEST_F(StressTest, testGetResultUseOffset)
{
  SphinxFunctionalObjectTest::testGetResultUseOffset(*pObj, stressTestData.sourceTestDir, stressTestData.indexName, stressTestData.needMakeIndex, stressTestData.needRemoveIndex);
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace tests
} // namespace sphinx 
} // namespace HCE 
