/**
 * @file SphinxTestPrinter.hpp
 *
 * Implementation class SphinxTestPrinter need for test of a sphinx functional object in HCE project node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef SPHINX_TEST_PRINTER_HPP
#define SPHINX_TEST_PRINTER_HPP

#include <gtest/gtest.h>

namespace HCE
{
namespace sphinx
{
namespace tests
{
//-----------------------------------------------------------------------------
class SphinxTestPrinter : public ::testing::EmptyTestEventListener
{

  // Called before a test case starts.
  virtual void OnTestStart(const ::testing::TestInfo& test_info)
  {
    printf("******* Test %s.%s starting.\n",
           test_info.test_case_name(), test_info.name());
  }

  // Called after a failed assertion or a SUCCEED() invocation.
  virtual void OnTestPartResult(
      const ::testing::TestPartResult& test_part_result) {
    printf("%s in %s:%d\n%s\n",
           test_part_result.failed() ? "******* Failure" : "Success",
           test_part_result.file_name(),
           test_part_result.line_number(),
           test_part_result.summary());
  }

  // Called after a test case ends.
  virtual void OnTestEnd(const ::testing::TestInfo& test_info) {
    printf("******* Test %s.%s ending.\n",
           test_info.test_case_name(), test_info.name());
  }

};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // namespace tests
} // namespace sphinx 
} // namespace HCE 

#endif // SPHINX_TEST_PRINTER_HPP
