/**
 * @file XMLCleanerTest.cpp
 *
 * Implementation of XMLCleanerTest class methods.
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#include <Poco/SAX/SAXException.h>

#include "XMLCleanerTest.hpp"
#include "TestData.hpp"

std::string XMLpath = "sphinx:docset/sphinx:schema";
std::string XMLElement = "sphinx:document";
std::string XMLIdName = "id";

void  XMLCleanerTest::saveXMLTest()
{
  std::string ret;
  Poco::XML::DOMParser parser;
  Poco::AutoPtr<Poco::XML::Document> pDoc = parser.parseString(saveXMLTest_Data);
  Poco::XML::NodeIterator it(pDoc, Poco::XML::NodeFilter::SHOW_ALL);
  Poco::XML::Node* rootNode = it.root();
  ret = xmlCleanerA->saveXMLWrp(rootNode);
  
  CPPUNIT_ASSERT(!ret.empty());
  
  CPPUNIT_ASSERT(ret.find("<?xml version=\"1.0\" encoding=\"UTF-8\"?>") != ret.npos || ret.find("<?xml version=\"1.0\" encoding=\"utf-8\"?>") != ret.npos);
  CPPUNIT_ASSERT(ret.find("<sphinx:docset xmlns:sphinx=\"http://sphinxsearch.com\">") != ret.npos);
  CPPUNIT_ASSERT(ret.find("<sphinx:document id=\"24\">") != ret.npos);  
  CPPUNIT_ASSERT(ret.find("<h><![CDATA[[time element, as specified in ISO 8601. Year: YYYY (eg 1997) Year and month: YYYY-MM (eg 1997-07)]]></h>") != ret.npos);
  CPPUNIT_ASSERT(ret.find("</sphinx:docset>") != ret.npos);
  
  CPPUNIT_ASSERT(ret.find("<empty tag>") == ret.npos);
  
  ret = xmlCleanerA->saveXMLWrp(NULL);
  CPPUNIT_ASSERT(ret.empty());
}

void  XMLCleanerTest::processTest()
{
  std::string ret;
  std::set<unsigned long long> delList;
  
  ret = xmlCleanerA->process(processTest_Data, delList);
  
  CPPUNIT_ASSERT(!ret.empty());
  CPPUNIT_ASSERT(ret.find("<?xml version=\"1.0\" encoding=\"UTF-8\"?>") != ret.npos || ret.find("<?xml version=\"1.0\" encoding=\"utf-8\"?>") != ret.npos);
  CPPUNIT_ASSERT(ret.find("<sphinx:docset xmlns:sphinx=\"http://sphinxsearch.com\">") != ret.npos);
  CPPUNIT_ASSERT(ret.find("<ext_f4>0022_1</ext_f4>") != ret.npos);  
  CPPUNIT_ASSERT(ret.find("<ext_f4>0023_2</ext_f4>") != ret.npos);
  CPPUNIT_ASSERT(ret.find("<ext_f4>0024_1</ext_f4>") != ret.npos);
  CPPUNIT_ASSERT(ret.find("<ext_f4>0023_1</ext_f4>") == ret.npos);

  delList.insert(14);
  delList.insert(22);
  ret = xmlCleanerA->process(processTest_Data, delList);  
  
  CPPUNIT_ASSERT(!ret.empty());
  CPPUNIT_ASSERT(ret.find("<ext_f4>0022_1</ext_f4>") == ret.npos);  
  CPPUNIT_ASSERT(ret.find("<ext_f4>0023_2</ext_f4>") != ret.npos);
  CPPUNIT_ASSERT(ret.find("<ext_f4>0024_1</ext_f4>") != ret.npos);
  CPPUNIT_ASSERT(ret.find("<ext_f4>0023_1</ext_f4>") == ret.npos);  
  
  CPPUNIT_ASSERT_THROW(xmlCleanerA->process(processTestBD1_Data, delList), Poco::XML::SAXParseException);
  CPPUNIT_ASSERT_THROW(xmlCleanerA->process(processTestBD2_Data, delList), Poco::XML::SAXParseException);
  CPPUNIT_ASSERT_THROW(xmlCleanerA->process(processTestBD3_Data, delList), Poco::XML::SAXParseException);
}

XMLCleanerTest::XMLCleanerTest()
{
  xmlCleanerA = Poco::SharedPtr<XMLCleanerA>(new XMLCleanerA(XMLpath, XMLElement, XMLIdName));
}

void XMLCleanerTest::setUp()
{
}

void XMLCleanerTest::tearDown()
{
}

CppUnit::Test *XMLCleanerTest::suite()
{
  CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite("XMLCleanerTest");
  suiteOfTests->addTest(new CppUnit::TestCaller<XMLCleanerTest>("XMLCleanerTest_saveXMLTest", &XMLCleanerTest::saveXMLTest));
  suiteOfTests->addTest(new CppUnit::TestCaller<XMLCleanerTest>("XMLCleanerTest_processTest", &XMLCleanerTest::processTest));
  return suiteOfTests;     
}
