/**
 * @file XMLCleaner.cpp
 *
 * Implementation of XMLCleaner class's methods
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#include <sstream>

#include "XMLCleaner.hpp"

std::string XMLCleaner::saveXML(Poco::XML::Node* node)
{
  std::string ret;
  if(node)
  {
    std::stringstream strStream;
    Poco::XML::DOMWriter writer;
    writer.setNewLine("\n");
    writer.setOptions(Poco::XML::XMLWriter::PRETTY_PRINT | Poco::XML::XMLWriter::WRITE_XML_DECLARATION);
    writer.writeNode(strStream, node);
    ret = strStream.str();
  }
  return ret;
}

bool XMLCleaner::isOmitNode(const Poco::XML::XMLString &nodeValue) const
{
  bool ret = false;
  switch(resOmitType)
  {
    case ResOmitType::ROM_NOT_OMIT :
      break;
    case ResOmitType::ROM_EMPTY_OMIT :
      if(nodeValue.empty())
      {
        ret = true;
      }
      break;
    case ResOmitType::ROM_NOT_NUM_OMIT :
      {
        try
        {
          std::stoull(nodeValue, 0, 0);
        }
        catch(std::exception &exp)
        {
          ret = true;
        }
      }
      break;
    default :
      break;
  }
  return ret;
}

std::string XMLCleaner::process(const std::string &inXML, const std::set<std::string> &delList)
{
  std::string ret;
  Poco::XML::NamedNodeMap *nMap = NULL;
  Poco::XML::DOMParser parser;
  Poco::AutoPtr<Poco::XML::Document> pDoc = parser.parseString(inXML);
  Poco::XML::NodeIterator it(pDoc, Poco::XML::NodeFilter::SHOW_ALL);
  Poco::XML::Node* rootNode = it.root();
  Poco::XML::Node* mainNode = rootNode;
  Poco::XML::Node* dellNode = NULL;
  if(rootNode)
  { 
    dellNode = rootNode->getNodeByPath(XMLPath);
  }
  if(dellNode)
  {
    mainNode = dellNode;
    std::vector<Poco::XML::Node*> remNodes;
    std::map<std::string, unsigned int> alreadyNodes;
    if(dellNode)
    {
      dellNode = dellNode->lastChild();
      while(dellNode)
      {
        if(dellNode->nodeType() != Poco::XML::Node::TEXT_NODE && dellNode->nodeName() == elemName)
        {
          nMap = (dellNode->hasAttributes() ? dellNode->attributes() : NULL);
          if(nMap)
          {
            Poco::XML::Node *localNode = nMap->getNamedItem(idName);
            if(localNode)
            {
              std::map<std::string, unsigned int>::iterator alreadyNodesIt;

              if(isOmitNode(localNode->nodeValue()))
              {
                remNodes.push_back(dellNode);
                nMap->release();
                nMap = NULL;
                dellNode = dellNode->previousSibling();
                continue;
              }
              if((alreadyNodesIt = alreadyNodes.find(localNode->nodeValue())) == alreadyNodes.end())
              {
                if(delList.find(localNode->nodeValue()) != delList.end())
                {
                  remNodes.push_back(dellNode);
                }
                else
                {
                  alreadyNodes.insert(std::pair<std::string, unsigned int>(localNode->nodeValue(), 1));
                }
              }
              else
              {
                (*alreadyNodesIt).second++;
                remNodes.push_back(dellNode);
              }
            }
            nMap->release();
            nMap = NULL;
          }
        }
        dellNode = dellNode->previousSibling();
      }    
    }
    for(unsigned int i = 0; i < remNodes.size(); i++)
    {
      mainNode->removeChild(remNodes[i]);
    }
    ret = std::move<std::string &&>(saveXML(rootNode));
  }
  return ret;
}
