/**
 * @file XMLCleaner.hpp
 *
 * XMLCleaner - espesial xml processing. Class contents 3 fields : XMLPath - xml path to the processing elements
 * elemName - processing element's name, idName - name of element ident, representing as element's attribute.
 * process() method - gets xml string(inXML param), breaks duplicate elements, and elements included in the 
 * delList list (second param) and returns resulting xml string.
 *
 * @author Andrey Lapshov <developers.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */
#ifndef XML_CLEANER_HPP
#define XML_CLEANER_HPP

#include <Poco/DOM/DOMParser.h>
#include <Poco/DOM/DOMWriter.h>
#include <Poco/DOM/Document.h>
#include <Poco/DOM/NodeIterator.h>
#include <Poco/DOM/NodeFilter.h> 
#include <Poco/DOM/AutoPtr.h>
#include <Poco/DOM/NamedNodeMap.h>
#include <Poco/XML/XMLWriter.h>
#include <Poco/SAX/InputSource.h>
#include <Poco/SharedPtr.h>

#include <set>
#include <string>

enum class ResOmitType : std::uint8_t {ROM_NOT_OMIT = 0, ROM_EMPTY_OMIT = 1, ROM_NOT_NUM_OMIT = 2};

class XMLCleaner
{
  private:
    const std::string XMLPath;
    const std::string elemName;
    const std::string idName;
    ResOmitType resOmitType;
  protected:
    static std::string saveXML(Poco::XML::Node* node);
    inline bool isOmitNode(const Poco::XML::XMLString &nodeValue) const;
  public:
    XMLCleaner(): XMLPath(""), elemName(""), idName(""), resOmitType(ResOmitType::ROM_NOT_OMIT) {}
    XMLCleaner(const std::string &_XMLPath, const std::string &_elemName, const std::string &_idName, ResOmitType _resOmitType = ResOmitType::ROM_NOT_OMIT): 
               XMLPath(_XMLPath), elemName(_elemName), idName(_idName), resOmitType(_resOmitType) {}
    void setOmitType(ResOmitType _resOmitType) {resOmitType = _resOmitType;}
    ResOmitType getOmitType() const {return resOmitType;}
    std::string process(const std::string &inXML, const std::set<std::string> &delList);
    virtual ~XMLCleaner() {};
};

#endif
